#!/usr/bin/perl

###############################################################################
#                                                                             #
# Filename: domain_ftp.cgi                                                    #
#                                                                             #
# Description: CGI needed to display and modify ftp settings.                 #
#                                                                             #
###############################################################################
###############################################################################
#                                                                             #
#  Copyright  2000 - 2001 Intel Corporation.                                 #
#  Intel Corporation All Rights Reserved.                                     #
#                                                                             #
#  The source code contained or described herein and all documents related to #
#  the source code ("Material") are owned by Intel Corporation or its         #
#  suppliers or licensors.  Title to the Material remains with Intel          #
#  Corporation or its suppliers and licensors.  The Material contains trade   #
#  secrets and proprietary and confidential information of Intel or its       #
#  suppliers and licensors.  The Material is protected by worldwide copyright #
#  and trade secret laws and treaty provisions.  No part of the Material may  #
#  be used, copied, reproduced, modified, published, uploaded, posted,        #
#  transmitted, distributed, or disclosed in any way without Intel's prior    #
#  express written permission.                                                #
#                                                                             #
#  No license under any patent, copyright, trade secret or other intellectual #
#  property right is granted to or conferred upon you by disclosure or        #
#  delivery of the Materials, either expressly, by implication, inducement,   #
#  estoppel or otherwise. Any license under such intellectual property rights #
#  must be express and approved by Intel in writing.                          #
#                                                                             #
###############################################################################

# Put the PERL library in the path.
use lib '/usr/ins/intel/gui/perllib';

use strict;
use LanguageResource;
use VirtualDomain;
use VirtualServ;
use VirtualFtp;
use DisplayMain;
use MonSettings;
use Authorize;
use CGI;

###############################################################################
# GetFTPStatus
###############################################################################

sub GetFTPStatus
{
	my $Return = 0;
	
	# Try to open the configuration file...
	if (open(INETDCONF, '< /etc/inetd.conf'))
	{
		# Loop through the file...
        while(<INETDCONF>) 
		{
			# If a line matching ftp appears...
			if (/^ftp/) 
			{
				# FTP is configured to run.
				$Return = 1;
				last;
			}
		}
				
		# Close the file.	
		close(INETDCONF);
	}

	return $Return;
}

###############################################################################
# ValidatePostData
###############################################################################

sub ValidatePostData
{
	my ($cgi)	= @_;
	my $Return	= 0;

	# If data is being submitted...
	if ($cgi->param('apply'))
	{
		my $MaxLoginFailureAttempts	= $cgi->param('max_login_failure_attempts');
		my $MaxLoginUsers			= $cgi->param('max_login_users');
		my $MaxFileTransferSize		= $cgi->param('max_file_transfer_size');
		my $MaxFileTransferIdle		= $cgi->param('max_file_transfer_idle');

		if ($MaxLoginFailureAttempts !~ /^\d+$/)
		{
			# Set the return code.
			$Return = 670000;
		}
		elsif ($MaxLoginUsers !~ /^\d+$/)
		{
			# Set the return code.
			$Return = 670001;
		}
		elsif (($MaxFileTransferSize !~ /^\+*([0-9]*)\.?([0-9]*)$/) || 
			   (!($1 || ('0' eq $1) || $2 || ('0' eq $2))))
		{
			# Set the return code.
			$Return = 670002;
		}
		elsif ($MaxFileTransferIdle !~ /^\d+$/)
		{
			# Set the return code.
			$Return = 670003;
		}
	}

	return $Return;
}

###############################################################################
# Main
###############################################################################

sub Main
{
	my ($cgi)					= @_;
	my $PreferredLanguage		= (split(/,/, $ENV{'HTTP_ACCEPT_LANGUAGE'}))[0];
	my $StringResource			= new LanguageResource($PreferredLanguage, '/usr/ins/intel/gui/locale', [ 'domain_ftp.lrc', 'common.lrc' ]);
	my %_R						= ( 'IDT:6700'						=> '',
									'SERVICE'						=> '',
									'STATUS'						=> '',
									'CONTROLS'						=> '',
									'FTP'							=> '',
									'ENABLE'   						=> '',
									'DISABLE'						=> '',
									'RUNNING'						=> '',
									'DISABLED'						=> '',
									'ALLOW_ANONYMOUS_LOGIN'			=> '',
									'BANNER_MESSAGE_USERS'			=> '',
									'FAILED_LOGIN_ATTEMPTS'			=> '',
									'MAX_LOGGED_IN_USERS'			=> '',
									'MAX_SESSION_DATA_TRANSFER'		=> '',
									'IDLE_TIME_BEFORE_DISCONNECT'	=> '',
									'MB'							=> '',
									'SEC'							=> '',
									'APPLY'							=> '',
									'NOTE'							=> '',
									'VIRTUAL_FTP_NOTE'				=> '' );
	my @DomainList				= VirtualDomain::list_vws();
	my $DomainRef;
	my $Index;
	my $Item;
	my $EnableAnonymousFTP;
	my @BannerSystemUsers;
	my $MaxLoginFailureAttempts;
	my $MaxLoginUsers;
	my $MaxFileTransferSize;
	my $MaxFileTransferIdle;
	my $Domain;
	my $Level;
	my $HelpIndicator;
	my $ServiceStatus;
	my $DomainFTPStatus;
	my $Result;
	my $Feedback;
	my $html;

	# Get the string resources.
	$StringResource->GetStrings(\%_R);
	
	# Get the domain and level of the user.
	($Domain, $Level) = Authorize::getTargetDomain();

	# Check if user is authorized to run this script
	unless (Authorize::checkAuthorization()) { return 0; }

	# Run through the domain list...
	foreach $Item (@DomainList) 
	{
		# If the domain name matches...
		if ($Domain eq $Item->{'vdn'})
		{
			# Get the index.
			$Index = $Item->{'idx'};
			last;
		}
	}

	# Get the domain information.
	$DomainRef = $DomainList[$Index];
	
	# Get the domain FTP status.
	$DomainFTPStatus = VirtualFtp::domainFtpStatus($Domain, $DomainRef->{'dom_grp'}, $DomainRef->{'vdip'});

	# If toggling the configuration state...
	if ((defined($cgi->param('control_enable'))) ||
		(defined($cgi->param('control_disable'))))
	{
		my $Monitor = new MonSettings($Domain);
		
		# If the configuration can be opened...
		if (0 == $Monitor->open())
		{
			# Set the monitor state.
			$Monitor->setMonitorState('ftp', (($cgi->param('control_enable')) ? 1 : 0));

			# Close the configuration.
			$Monitor->close();
		}
	}
	
	SWITCH:
	{
		# Control the services.
		if (defined($cgi->param('control_enable')))		{ VirtualFtp::enableVftp($Domain, $DomainRef->{'dom_grp'}, $DomainRef->{'vdip'}) if(1 == $DomainFTPStatus); last SWITCH; }
		if (defined($cgi->param('control_disable')))	{ VirtualFtp::disableVftp($Domain, $DomainRef->{'dom_grp'}, $DomainRef->{'vdip'}); last SWITCH; }
	}
	
	# If the user is applying a change...
	if (defined($cgi->param('apply')))
	{
		# If the virtual FTP configuration is set up...
		if (('1' ne $DomainFTPStatus) &&
			('none' ne $cgi->param('virtual_support')))
		{
			# Validate the data...
			if (0 == ($Result = ValidatePostData($cgi)))
			{
				VirtualServ::set_virtual_ftp($DomainRef->{'dom_hom'}, 
											 $DomainRef->{'dom_grp'}, 
											 $DomainRef->{'vdn'}, 
											 $DomainRef->{'vdadm'}, 
											 $cgi->param('max_login_failure_attempts'),
											 $cgi->param('max_file_transfer_idle'),
											 $cgi->param('max_file_transfer_size'),
											 $cgi->param('max_login_users'),
											 $cgi->param('banner_system_users'),                                 
											 $cgi->param('enable_anonymous_ftp'));
			}
			else
			{
				$EnableAnonymousFTP			= $cgi->param('enable_anonymous_ftp') || '';
				@BannerSystemUsers			= $cgi->param('banner_system_users');
				$MaxLoginFailureAttempts	= $cgi->param('max_login_failure_attempts');
				$MaxLoginUsers				= $cgi->param('max_login_users');
				$MaxFileTransferSize		= $cgi->param('max_file_transfer_size');
				$MaxFileTransferIdle		= $cgi->param('max_file_transfer_idle');
			}
		}
		else
		{
			$Result = 670004;
		}
	}

	# If there is a virtual FTP host and the associated directory exists...
	if (('' ne VirtualFtp::getFtpStatus($DomainRef->{'vdip'}, $DomainRef->{'dom_grp'})) &&
		( -d "/usr/ins/intel/wu-ftpd/conf/$DomainRef->{'dom_grp'}" ))
	{
		my $Dummy;
		
		# If there aren't any negative results...
		unless ($Result)
		{
			# Get the configuration data.
			($MaxLoginFailureAttempts, $MaxFileTransferIdle, $MaxFileTransferSize, $MaxLoginUsers) = VirtualServ::ftp_conf($DomainRef->{'dom_grp'});
			($EnableAnonymousFTP, $Dummy) = VirtualServ::checkAnon($DomainRef->{'dom_grp'}); 
			@BannerSystemUsers = VirtualServ::ftp_msg($DomainRef->{'dom_grp'});
	
			# Adjust the file size.
			$MaxFileTransferSize /= 1048576;
		}
		
		# Adjust the checkbox value.
		$EnableAnonymousFTP = 'checked' if ($EnableAnonymousFTP eq 'yes');
		
		$html = <<HERE;
					<table border=0 width=100%>
					<tr>
						<td width=100% colspan=2>&nbsp;</td>
					</tr>
					<tr>
						<td width=95% valign=top><input type=checkbox name=enable_anonymous_ftp value=yes $EnableAnonymousFTP>&nbsp;<b>$_R{'ALLOW_ANONYMOUS_LOGIN'}</b></td>
						<td width=10%></td>
					</tr>
					<tr>
						<td width=100% colspan=2>&nbsp;</td>
					</tr>
					<tr>
						<td width=100% colspan=2><b>$_R{'BANNER_MESSAGE_USERS'}</b></td>
					</tr>
					<tr>
						<td width=100% colspan=2><textarea name=banner_system_users rows=4 cols=41 wrap=physical>@BannerSystemUsers</textarea></td>
					</tr>
					<tr>
						<td width=100% colspan=2>&nbsp;</td>
					</tr>
					<tr>
						<td nowrap width=95%><b>$_R{'FAILED_LOGIN_ATTEMPTS'}</b></td>
						<td width=10%><input type=text name=max_login_failure_attempts size=3 maxlength=3 value='$MaxLoginFailureAttempts'></td>
					</tr>
					<tr>
						<td nowrap width=95%><b>$_R{'MAX_LOGGED_IN_USERS'}</b>&nbsp;</td>
						<td width=10%><input type=text name=max_login_users size=3 maxlength=4 value='$MaxLoginUsers'></td>
					</tr>
					<tr>
						<td nowrap width=64%><b>$_R{'MAX_SESSION_DATA_TRANSFER'}</b></td>
						<td width=36%><input type=text name=max_file_transfer_size size=3 maxlength=4 value='$MaxFileTransferSize'>$_R{'MB'}</td>
					</tr>
					<tr>
						<td nowrap width=64%><b>$_R{'IDLE_TIME_BEFORE_DISCONNECT'}</b></td>
						<td width=36%><input type=text name=max_file_transfer_idle size=4 maxlength=5 value='$MaxFileTransferIdle'>$_R{'SEC'}</td>
					</tr>
					<tr>
						<td width=100% colspan=2>&nbsp;</td>
					</tr>
					<tr>
						<td width=100% align=right colspan=2>
							<input type=submit class=clsButton value='$_R{'APPLY'}' name=apply>
						</td>
					</tr>
					</table>
HERE
	}
	else
	{
		
		$html = <<HERE;
					<br>
					<table border=0 width=100%>
					<input type=hidden name=virtual_support value=none>
					<tr>
						<td width=100%><b>$_R{'NOTE'}</b>&nbsp;$_R{'VIRTUAL_FTP_NOTE'}</td>
					</tr>
					</table>
HERE
	}

	# Get the status information...
	$ServiceStatus	= (GetFTPStatus() && ('1' ne VirtualFtp::domainFtpStatus($Domain, $DomainRef->{'dom_grp'}, $DomainRef->{'vdip'}))) ? "<font color=green>$_R{'RUNNING'}<font>" : "<font color=red>$_R{'DISABLED'}<font>";
	
	# Branch based on the user level...
	if (10000 == (getpwnam($cgi->remote_user()))[3])
	{
		# Set the help indicator.
		$HelpIndicator = 'server';
		
		$html = <<HERE;
<form method=post action=domain_ftp.cgi enctype="application/x-www-form-urlencoded">
	<table border=0 width=375 bgcolor=#eeeeee>
	<tr>
		<td>
			&nbsp;
			<table border=0 cellpadding=0 cellspacing=0 width=100% bgcolor=#eeeeee>
			<tr> 
				<td width=100%> 
					<table border=1 cellpadding=2 cellspacing=0 width=100% bgcolor=#ffffff bordercolor=#eeeeee>
					<tr>
						<td width=40% bgcolor=#808080><b><font color=#ffffff>$_R{'SERVICE'}</font></b></td>
						<td width=15% bgcolor=#808080><b><font color=#ffffff>$_R{'STATUS'}</font></b></td>
						<td width=40% align=left bgcolor=#808080><b><font color=#ffffff>$_R{'CONTROLS'}</font></b></td>
					</tr>
					<tr>
						<td width=40%><b>$_R{'FTP'}</b></td>
						<td width=15%>$ServiceStatus</td>
						<td width=40% align=left nowrap>
							<input type=submit class=clsButton value='$_R{'ENABLE'}' name=control_enable>
							<input type=submit class=clsButton value='$_R{'DISABLE'}' name=control_disable>
						</td>
					</tr>
					</table>
$html
				</td>
			</tr>
			</table>
			<br>
		</td>
	</tr>
	</table>
</form>
HERE
	}
	else
	{
		# Set the help indicator.
		$HelpIndicator = 'domain';
		
		$html = <<HERE;
<form method=post action=domain_ftp.cgi enctype="application/x-www-form-urlencoded">
	<table border=0 width=375 bgcolor=#eeeeee>
	<tr>
		<td>
			&nbsp;
			<table border=0 cellpadding=0 cellspacing=0 width=100% bgcolor=#eeeeee>
			<tr> 
				<td width=100%> 
					<table border=1 cellpadding=2 cellspacing=0 width=100% bgcolor=#ffffff bordercolor=#eeeeee>
					<tr>
						<td width=40% bgcolor=#808080><b><font color=#ffffff>$_R{'SERVICE'}</font></b></td>
						<td width=55% bgcolor=#808080><b><font color=#ffffff>$_R{'STATUS'}</font></b></td>
					</tr>
					<tr>
						<td width=40%><b>$_R{'FTP'}</b></td>
						<td width=55%>$ServiceStatus</td>
					</tr>
					</table>
$html
				</td>
			</tr>
			</table>
			<br>
		</td>
	</tr>
	</table>
</form>
HERE
	}
	
	# If there was an error...
	$Feedback = $StringResource->_S($Result) if($Result);

	# Display the page.
	DisplayMain->printDocument({ 'q' => $cgi, 'html' => $html, 
								 'class' => 'domain_ftp', 'id' => 6700, 'title' => $_R{'IDT:6700'},
								 'root-link' => 'domain_ftp', 'helpIndicator' => $HelpIndicator, 'result' => $Result, 'feedback' => $Feedback });
}

###############################################################################
# The main body of the script.
###############################################################################

my $cgi = new CGI;

Main($cgi);
