#!/usr/bin/perl	-w

###############################################################################
#                                                                             #
# Filename: domain_http.cgi                                                   #
#                                                                             #
# Description: Contains CGI script needed for HTTP service information.       #
#                                                                             #
###############################################################################
###############################################################################
#                                                                             #
#  Copyright  2000 - 2001 Intel Corporation.                                 #
#  Intel Corporation All Rights Reserved.                                     #
#                                                                             #
#  The source code contained or described herein and all documents related to #
#  the source code ("Material") are owned by Intel Corporation or its         #
#  suppliers or licensors.  Title to the Material remains with Intel          #
#  Corporation or its suppliers and licensors.  The Material contains trade   #
#  secrets and proprietary and confidential information of Intel or its       #
#  suppliers and licensors.  The Material is protected by worldwide copyright #
#  and trade secret laws and treaty provisions.  No part of the Material may  #
#  be used, copied, reproduced, modified, published, uploaded, posted,        #
#  transmitted, distributed, or disclosed in any way without Intel's prior    #
#  express written permission.                                                #
#                                                                             #
#  No license under any patent, copyright, trade secret or other intellectual #
#  property right is granted to or conferred upon you by disclosure or        #
#  delivery of the Materials, either expressly, by implication, inducement,   #
#  estoppel or otherwise. Any license under such intellectual property rights #
#  must be express and approved by Intel in writing.                          #
#                                                                             #
###############################################################################

# Put the PERL library in the path.
use lib '/usr/ins/intel/gui/perllib';

use strict;
use LanguageResource;
use VirtualDomain;
use DisplayMain;
use Authorize;
use Daemon;
use CGI;

###############################################################################
# Main
###############################################################################

sub Main
{
	my ($cgi)					= @_;
	my $PreferredLanguage		= (split(/,/, $ENV{'HTTP_ACCEPT_LANGUAGE'}))[0];
	my $StringResource			= new LanguageResource($PreferredLanguage, '/usr/ins/intel/gui/locale', [ 'domain_http.lrc', 'common.lrc' ]);
	my %_R						= ( 'IDT:6000'				=> '',
									'SERVICE'				=> '',
									'STATUS'				=> '',
									'WEB'					=> '',
									'RUNNING'				=> '',
									'STOPPED'				=> '',
									'WEB_SSL'				=> '',
									'SSL_CONFIGURED_SELF'	=> '',
									'SSL_CONFIGURED_CA'		=> '',
									'SSL_NOT_CONFIGURED'	=> '' );
	my @DomainList				= VirtualDomain::list_vws();
	my $HTTPDaemon				= new Daemon('httpd');
	my $HTTPDaemonStatus;
	my $HelpIndicator;
	my $DomainRef;
	my $Domain;
	my $Level;
	my $Index;
	my $Item;
	my $SSLState				= 0;
	my $SSLDetails;
	my $CertificateType			= 'none';
	my $html;
	
	# Get the string resources.
	$StringResource->GetStrings(\%_R);

	# Get the daemon status...
	$HTTPDaemonStatus	= ($HTTPDaemon->GetStatus())  ? "<font color=green>$_R{'RUNNING'}<font>" : "<font color=red>$_R{'STOPPED'}<font>";

	# Get the domain and level of the user.
	($Domain, $Level) = Authorize::getTargetDomain();
	
	# Run through the domain list...
	foreach $Item (@DomainList) 
	{
		# If the domain name matches...
		if ($Domain eq $Item->{'vdn'})
		{
			# Get the index.
			$Index = $Item->{'idx'};
			last;
		}
	}

	# Get the domain information.
	$DomainRef = $DomainList[$Index];
	
	# If the domain is configured for SSL...
	if (('yes' eq $DomainRef->{'vdssl'}) &&
		( -e "$DomainRef->{'dom_hom'}/$DomainRef->{'dom_grp'}/cert/certificate" ))
	{
		# SSL is active.
		$SSLState			= 1;
		$CertificateType	= ( -e "$DomainRef->{'dom_hom'}/$DomainRef->{'dom_grp'}/cert/request" ) ? 'self' : 'ca';
	}

	# Set the SSL status text...
	if ($SSLState)
	{
		# Using SSL...
		$SSLDetails = '<font color=green>' . (('self' eq $CertificateType) ? $StringResource->_S('SSL_CONFIGURED_SELF') : $StringResource->_S('SSL_CONFIGURED_CA')) . '<font>';
	}
	else
	{
		# Not using SSL.
		$SSLDetails = '<font color=red>' . $StringResource->_S('SSL_NOT_CONFIGURED') . '<font>';
	}

	$html = <<HERE;
<table border=0 width=375 bgcolor=#eeeeee>
<tr>
	<td>
		<br>
		<table border=0 cellpadding=0 cellspacing=0 width=100% bgcolor=#eeeeee>
		<tr>
			<td width=100%>
				<table border=1 cellpadding=2 cellspacing=0 width=100% bgcolor=#ffffff bordercolor=#eeeeee>
				<tr>
					<td width=40% bgcolor=#808080 nowrap><font color=#ffffff><b>$_R{'SERVICE'}</b></font></td>
					<td width=55% bgcolor=#808080 nowrap><font color=#ffffff><b>$_R{'STATUS'}</b></font></td>
				</tr>
				<tr>
					<td width=40%><b>$_R{'WEB'}</b></td>
					<td width=55%><font color=green>$HTTPDaemonStatus</font></td>
				</tr>
				</table>
				<table border=0 width=100%>
				<tr>
					<td width=100% colspan=2>&nbsp;</td>
				</tr>
				<tr>
					<td width=100% colspan=2><b>$_R{'WEB_SSL'}</b></td>
				</tr>
				<tr>
					<td nowrap>$SSLDetails</td>
				</tr>
				</table>
			</td>
		</tr>
		</table>
		<br>
	</td>
</tr>
</table>
HERE

	# Get the appropriate help indicator.
	$HelpIndicator = (10000 == (getpwnam($cgi->remote_user()))[3]) ? 'server' : 'domain';
	
	# Display the page.
    DisplayMain->printDocument({ 'q' => $cgi, 'html' => $html,
								 'class' => 'domain_http', 'id' => 6000, 'title' => $_R{'IDT:6000'},
								 'root-link' => 'domain_http', 'helpIndicator' => $HelpIndicator });
}

###############################################################################
# The main body of the script.
###############################################################################

my $cgi = new CGI;

Main($cgi);
