#!/usr/bin/perl

###############################################################################
#                                                                             #
# Filename: domain_log.cgi                                                    #
#                                                                             #
# Description: CGI needed to display logged domain information.               #
#                                                                             #
###############################################################################
###############################################################################
#                                                                             #
#  Copyright  2000 - 2001 Intel Corporation.                                 #
#  Intel Corporation All Rights Reserved.                                     #
#                                                                             #
#  The source code contained or described herein and all documents related to #
#  the source code ("Material") are owned by Intel Corporation or its         #
#  suppliers or licensors.  Title to the Material remains with Intel          #
#  Corporation or its suppliers and licensors.  The Material contains trade   #
#  secrets and proprietary and confidential information of Intel or its       #
#  suppliers and licensors.  The Material is protected by worldwide copyright #
#  and trade secret laws and treaty provisions.  No part of the Material may  #
#  be used, copied, reproduced, modified, published, uploaded, posted,        #
#  transmitted, distributed, or disclosed in any way without Intel's prior    #
#  express written permission.                                                #
#                                                                             #
#  No license under any patent, copyright, trade secret or other intellectual #
#  property right is granted to or conferred upon you by disclosure or        #
#  delivery of the Materials, either expressly, by implication, inducement,   #
#  estoppel or otherwise. Any license under such intellectual property rights #
#  must be express and approved by Intel in writing.                          #
#                                                                             #
###############################################################################

# Put the PERL library in the path.
use lib '/usr/ins/intel/gui/perllib';

use strict;
use LanguageResource;
use LanguageLogCGI;
use LanguageTime;
use ConfigHandler;
use DisplayMain;
use Authorize;
use Safe;
use CGI;

###############################################################################
# Main
###############################################################################

sub Main
{
	my $cgi					= shift;
	my $PreferredLanguage	= (split(/,/, $ENV{'HTTP_ACCEPT_LANGUAGE'}))[0];
	my $StringResource		= new LanguageResource($PreferredLanguage, '/usr/ins/intel/gui/locale', [ 'domain_log.lrc' ]);
	my $TimeResource		= new LanguageTime($PreferredLanguage, '/usr/ins/intel/gui/locale');
	my $Log					= new LanguageLogCGI();
	my %_R					= ( 'IDT:6600'			=> '',
								'LAST_BOOT'			=> '',
								'REPORTED'			=> '',
								'DESCRIPTION'		=> '' );
	my %DisplayParameters;
	my @LogData;
	my $EntryDisplayLimit	= ConfigHandler::getConfig(ConfigHandler::getPath('CONSTANTS'), 'MAX_LOGS_TO_DISPLAY');
    my $LastBootTime;
	my $html3				= '';
	my $html4				= '';
	my $html5				= '';
	my $Enumerator;
	my $Domain;
	my $Level;
  
	# Get the domain and level of the user.
	($Domain, $Level) = Authorize::getTargetDomain();

	# Check if user is authorized to run this script
	unless (Authorize::checkAuthorization()) { return 0; }
	
	# Get the string resources.
	$StringResource->GetStrings(\%_R);
	
	# Get the last boot time.
	$LastBootTime = Safe::backtick('/usr/bin/procinfo');
	$LastBootTime = substr($LastBootTime, index($LastBootTime, 'Bootup:') + 8, 27); # TODO - deal with the internationalization

	# The middle of the page.
	$html3 = <<HERE;
		<b>$_R{'LAST_BOOT'}</b>&nbsp;$LastBootTime<br><br>
		<table border=1 cellpadding=4 cellspacing=0 bordercolor=#ffffff width=375>
		<tr>
			<td width=16 bgcolor=#808080>&nbsp;</td>
			<td nowrap width=105 bgcolor=#808080><b><font color=#ffffff>$_R{'REPORTED'}</font></b></td>
			<td width=254 bgcolor=#808080><b><font color="#ffffff">$_R{'DESCRIPTION'}</font></b></td>
		</tr>
HERE

	# Attach to the log file.
	$Log->Attach("/home/domain/$Domain/logs/error.log");

	# Set the preferred language.
	$Log->SetPreferredLanguage($PreferredLanguage);

	# Try and get the enumerator.
	if ($Enumerator = $Log->enumeratorElements(400))
	{
		my $Element;
		my $Message;
		my $i		= 0;

		# Run through the elements...
		while ($Enumerator->hasMoreElements())
		{
			# If there is an element and it's type matches...
			if (($Element = $Enumerator->nextElement()) &&
				($Message = $Element->GetMessage()))
			{
				# Merge the data into the common stream.
				$LogData[$i] = join('~', $Element->GetType(), $TimeResource->FormatTime($StringResource->_S('TIME_FORMAT'), localtime($Element->GetTimeStamp())), $Message);

				# Move on.
				++$i;
			}
		}
	}
	
	# If there is no data...
	unless (@LogData)
	{
		# Create a filler entry.
		$LogData[0] = join('~', 'information', $TimeResource->FormatTime($StringResource->_S('TIME_FORMAT'), localtime()), $StringResource->_S('NO_ENTRIES_FOUND'));
	}
	
	# If there is log data...
	if (@LogData)
	{
		my $Date;
		my $Message;
		my $Severity;
		my $bgcolor		= '#eeeeee';
		
		# Realign the entry display limit.
		$EntryDisplayLimit = (@LogData > $EntryDisplayLimit) ? $EntryDisplayLimit : @LogData;

		# Run though the data...
		for (my $i = 0; $i < $EntryDisplayLimit; ++$i)
		{
			# Get the date and message.
			($Severity, $Date, $Message) = split(/~/, $LogData[$i], 3);

			# If there was no message...
			if (!$Message)
			{
				# Re-split the data.
				($Date, $Message) = split(/~/, $LogData[$i], 2);
			}

			# Get the severity image.
			$Severity = ('critical' eq lc($Severity)) ? '/images/critical.gif' : (('warning' eq lc($Severity)) ? '/images/warning.gif' : '/images/blank.gif');

			# Build the table entry.
			$html4 .= <<HERE;
		<tr>
			<td width=16 bgcolor=$bgcolor><img align=center border=0 height=16 src=$Severity></td>
			<td nowrap width=105 bgcolor=$bgcolor>$Date</td>
			<td width=254 bgcolor=$bgcolor>$Message</td>
		</tr>
HERE

			# Toggle the background color.
			$bgcolor = ($bgcolor eq '#eeeeee') ? 'white' : '#eeeeee';
		}
	}

	# Finish off the table.
	$html5 = '</table>';

	# Set up the baseline display parameters.
	$DisplayParameters{'q'}			= $cgi;
	$DisplayParameters{'html'}		= $html3 . $html4 . $html5;
	$DisplayParameters{'bgcolor'}	= 'white';
	$DisplayParameters{'class'}		= 'domain_log';
	$DisplayParameters{'id'}		= 6600;
	$DisplayParameters{'title'}		= $StringResource->_S('IDT:6600');
	$DisplayParameters{'root-link'} = 'domain_log';

	# Display the page.
    DisplayMain->printDocument(\%DisplayParameters);
}

###############################################################################
# The main body of the script.
###############################################################################

my $cgi = new CGI;

Main($cgi);
