#!/usr/bin/perl

###############################################################################
#                                                                             #
# Filename: domain_navbar.cgi                                                 #
#                                                                             #
# Description: Contains CGI script needed to render the domain administrator  #
#              navigation bar.                                                #
#                                                                             #
###############################################################################
###############################################################################
#                                                                             #
#  Copyright  2000 - 2001 Intel Corporation.                                 #
#  Intel Corporation All Rights Reserved.                                     #
#                                                                             #
#  The source code contained or described herein and all documents related to #
#  the source code ("Material") are owned by Intel Corporation or its         #
#  suppliers or licensors.  Title to the Material remains with Intel          #
#  Corporation or its suppliers and licensors.  The Material contains trade   #
#  secrets and proprietary and confidential information of Intel or its       #
#  suppliers and licensors.  The Material is protected by worldwide copyright #
#  and trade secret laws and treaty provisions.  No part of the Material may  #
#  be used, copied, reproduced, modified, published, uploaded, posted,        #
#  transmitted, distributed, or disclosed in any way without Intel's prior    #
#  express written permission.                                                #
#                                                                             #
#  No license under any patent, copyright, trade secret or other intellectual #
#  property right is granted to or conferred upon you by disclosure or        #
#  delivery of the Materials, either expressly, by implication, inducement,   #
#  estoppel or otherwise. Any license under such intellectual property rights #
#  must be express and approved by Intel in writing.                          #
#                                                                             #
###############################################################################

# Put the PERL library in the path.
use lib '/usr/ins/intel/gui/perllib';

use strict;
use LanguageResource;
use Authorize;
use CGI;

###############################################################################
# Main
###############################################################################

sub Main
{
	my ($cgi) 				= @_;
	my $PreferredLanguage	= (split(/,/, $ENV{'HTTP_ACCEPT_LANGUAGE'}))[0];
	my $ImageResource		= new LanguageResource($PreferredLanguage, '/usr/ins/intel/gui/locale/images');
	my $ImgPathNavBar		= $ImageResource->ResolveResourceFile('navbar-domain.jpg');
	my $BodyTag;
	my $html;
	
	# Check if user is authorized to run this script
	unless (Authorize::checkAuthorization()) { return 0; }
	
	# Safe defaults.
	$ImgPathNavBar	= '' unless(defined($ImgPathNavBar));

	# Get just the hidden part of the file name.
	$ImgPathNavBar	=~ s/^\/usr\/ins\/intel\/gui(\/locale\/images\/.*)$/$1/;
	
	# Based on the user agent...
	if (-1 < (index($cgi->user_agent(), 'MSIE')))
	{ 
		# Use a MSIE compatible body tag.
		$BodyTag = "<body bgcolor=#ffffff background=$ImgPathNavBar onload=check_selected_item() leftmargin=0 topmargin=0>"; 
	}
	else
	{ 
		# Use a Netscape compatible body tag.
		$BodyTag = "<body bgcolor=#ffffff onResize=location.reload() onload=check_selected_item() background=$ImgPathNavBar leftmargin=0 topmargin=0>"; 
	}

	$html = <<HERE;
<html>
<head>
	<meta http-equiv="Content-Type" content="text/html; charset=iso-8859-1">
	<link rel=stylesheet href=/style/style.css type=text/css>
</head>
<script language=javascript>
var selected_item = "domain_status";

function check_selected_item()
{
	if(parent.main.selected_item)
	{
		if(parent.main.selected_item != selected_item)
			selectitem(parent.main.selected_item);
	}
}

function selectitem(itemname)
{
	eval("document.images."+ selected_item +".src = '/images/blank.gif';");
	eval("document.images."+ itemname +".src = '/images/domain_selected.gif';");
	selected_item = itemname;
}

function changeImages(itemname)
{
	if (itemname != selected_item)
	{
		var picvalue = eval("document.images."+ itemname +".src");
	
		if (picvalue.indexOf('blank.gif') != -1)
		{
			eval("document.images."+ itemname +".src = '/images/domain_selection.gif';");
		}
		else
		{
			eval("document.images."+ itemname +".src = '/images/blank.gif';");
		}
	}
}
</script>
$BodyTag
	<table border=0 cellpadding=0 cellspacing=0>
	<tr>
		<td colspan=3><IMG SRC=/images/blank.gif width=158 height=21></td>
	</tr>
	<tr>
		<td width=3></td>
		<td width=138>
			<a href=domain_status.cgi target=main onmouseover="changeImages('domain_status');" onmouseout="changeImages('domain_status');" onclick="selectitem('domain_status');">
			<img src=/images/domain_selected.gif width=131 height=15 border=0 id=domain_status name=domain_status></a></td>
	</tr>
	<tr>
		<td width=3></td>
		<td width=138>
			<a href=domain_users.cgi target=main onmouseover="changeImages('domain_users');" onmouseout="changeImages('domain_users');" onclick="selectitem('domain_users');">
			<img src=/images/blank.gif width=131 height=15 border=0 id=domain_users name=domain_users></a></td>
	</tr>
	<tr>
		<td width=3></td>
		<td width=138>
			<a href=domain_settings.cgi target=main onmouseover="changeImages('domain_settings');" onmouseout="changeImages('domain_settings');" onclick="selectitem('domain_settings');">
			<img src=/images/blank.gif width=131 height=15 border=0 id=domain_settings name=domain_settings></a></td>
	</tr>
	<tr>
		<td width=3></td>
		<td width=138>
			<a href=domain_backup.cgi target=main onmouseover="changeImages('domain_backup');" onmouseout="changeImages('domain_backup');" onclick="selectitem('domain_backup');">
			<img src=/images/blank.gif width=131 height=15 border=0 id=domain_backup name=domain_backup></a></td>
	</tr>
	<tr>
		<td width=3></td>
		<td width=138>
			<a href=domain_alerts.cgi target=main onmouseover="changeImages('domain_alerts');" onmouseout="changeImages('domain_alerts');" onclick="selectitem('domain_alerts');">
			<img src=/images/blank.gif width=131 height=15 border=0 id=domain_alerts name=domain_alerts></a></td>
	</tr>
	<tr>
		<td colspan=3><img src=/images/blank.gif width=131 height=22></td>
	</tr>
	<tr>
		<td width=3></td>
		<td width=138>
			<a href=domain_http.cgi target=main onmouseover="changeImages('domain_http');" onmouseout="changeImages('domain_http');" onclick="selectitem('domain_http');">
			<img src=/images/blank.gif width=131 height=15 border=0 id=domain_http name=domain_http></a></td>
	</tr>
	<tr>
		<td width=3></td>
		<td width=138>
			<a href=domain_mail.cgi target=main onmouseover="changeImages('domain_mail');" onmouseout="changeImages('domain_mail');" onclick="selectitem('domain_mail');">
			<img src=/images/blank.gif width=131 height=15 border=0 id=domain_mail name=domain_mail></a></td>
	</tr>
	<tr>
		<td width=3></td>
		<td width=138>
			<a href=domain_ftp.cgi target=main onmouseover="changeImages('domain_ftp');" onmouseout="changeImages('domain_ftp');" onclick="selectitem('domain_ftp');">
			<img src=/images/blank.gif width=131 height=15 border=0 id=domain_ftp name=domain_ftp></a></td>
	</tr>
	<tr>
		<td colspan=3><img src=/images/blank.gif width=131 height=22></td>
	</tr>
	<tr>
		<td width=3></td>
		<td width=138>
			<a href=domain_usage.cgi target=main onmouseover="changeImages('domain_usage');" onmouseout="changeImages('domain_usage');" onclick="selectitem('domain_usage');">
			<img src=/images/blank.gif width=131 height=15 border=0 id=domain_usage name=domain_usage></a></td>
	</tr>
	<tr>
		<td width=3></td>
		<td width=138>
			<a href=domain_log.cgi target=main onmouseover="changeImages('domain_log');" onmouseout="changeImages('domain_log');" onclick="selectitem('domain_log');">
			<img src=/images/blank.gif width=131 height=15 border=0 id=domain_log name=domain_log></a></td>
	</tr>
	<tr>
		<td colspan=3><img src=/images/blank.gif width=131 height=22></td>
	</tr>
	</table>
</body>
</html>
HERE

	# Send pack the page.
	print($cgi->header(-expires=>'Mon, 21-May-1971 00:00:00 GMT'));
	print($html);
}

###############################################################################
# The main body of the script.
###############################################################################

my $cgi = new CGI;

Main($cgi);
