#!/usr/bin/perl

###############################################################################
#                                                                             #
# Filename: domain_status.cgi                                                 #
#                                                                             #
# Description: CGI needed to display domain status.                           #
#                                                                             #
###############################################################################
###############################################################################
#                                                                             #
#  Copyright  2000 - 2001 Intel Corporation.                                 #
#  Intel Corporation All Rights Reserved.                                     #
#                                                                             #
#  The source code contained or described herein and all documents related to #
#  the source code ("Material") are owned by Intel Corporation or its         #
#  suppliers or licensors.  Title to the Material remains with Intel          #
#  Corporation or its suppliers and licensors.  The Material contains trade   #
#  secrets and proprietary and confidential information of Intel or its       #
#  suppliers and licensors.  The Material is protected by worldwide copyright #
#  and trade secret laws and treaty provisions.  No part of the Material may  #
#  be used, copied, reproduced, modified, published, uploaded, posted,        #
#  transmitted, distributed, or disclosed in any way without Intel's prior    #
#  express written permission.                                                #
#                                                                             #
#  No license under any patent, copyright, trade secret or other intellectual #
#  property right is granted to or conferred upon you by disclosure or        #
#  delivery of the Materials, either expressly, by implication, inducement,   #
#  estoppel or otherwise. Any license under such intellectual property rights #
#  must be express and approved by Intel in writing.                          #
#                                                                             #
###############################################################################

# Put the PERL library in the path.
use lib '/usr/ins/intel/gui/perllib';

use strict;
use LanguageResource;
use DisplayMain;
use VirtualDomain;
use VirtualFtp;
use Authorize;
use Quotaall;
use Daemon;
use Safe;
use Mail;
use CGI;

###############################################################################
# FormatFloat
###############################################################################

sub FormatFloat 
{
	my ($Float, $Precision) = @_;
	my $FormattedFloat;

	# Validate the input parameter(s)...
	if (defined($Float))
	{
		# Safe default.
		$Precision = 1 unless(defined($Precision));

		# Prime the format string.
		$Precision .= 'f';

		# Format the value.
		$FormattedFloat = sprintf("%5.$Precision", $Float);
		
		# See if the output is integer.
		$FormattedFloat = int($FormattedFloat) if($FormattedFloat =~ /\.0/);

		# Remove any leading whitespace.
		$FormattedFloat =~ s/^\s*(.*)$/$1/;
	}
	
	return $FormattedFloat;
}

###############################################################################
# GetFTPStatus
###############################################################################

sub GetFTPStatus
{
	my $Return = 0;
	
	# Try to open the configuration file...
	if (open(INETDCONF, '< /etc/inetd.conf'))
	{
		# Loop through the file...
        while(<INETDCONF>) 
		{
			# If a line matching ftp appears...
			if (/^ftp/) 
			{
				# FTP is configured to run.
				$Return = 1;
				last;
			}
		}
				
		# Close the file.	
		close(INETDCONF);
	}

	return $Return;
}

###############################################################################
# GetHomeUsage
###############################################################################

sub GetHomeUsage
{
	my $DFData = Safe::backtick('/bin/df -k /home');
    
	# Get the data.
	if ($DFData =~ /\/dev\/\S+\s*(\S+)\s*(\S+)\s*(\S+)\s*(\S+)\s*(\S+)/g)
	{
		return $1;
	}

	return 1;
}

###############################################################################
# Main
###############################################################################

sub Main
{
	my ($cgi) = @_;
	my $PreferredLanguage	= (split(/,/, $ENV{'HTTP_ACCEPT_LANGUAGE'}))[0];
	my $StringResource		= new LanguageResource($PreferredLanguage, '/usr/ins/intel/gui/locale', [ 'domain_status.lrc', 'common.lrc' ]);
	my %_R					= ( 'IDT:7000'						=> '',
								'UP_TIME'						=> '',
								'WEB_SERVER_STATISTICS'			=> '',
								'REPORT_GENERATED'				=> '',
								'TOTAL_HTML_PAGE_REQUESTS'		=> '',
								'TOTAL_VISITS'					=> '',
								'TOTAL_HITS'					=> '',
								'TOTAL_FILES'					=> '',
								'BYTES_TRANSFERRED'				=> '',
								'KB'							=> '',
								'NO_WEB_SERVER_ACTIVITY'		=> '',
								'DRIVE_SPACE_USAGE'				=> '',
								'MB_FREE'						=> '',
								'MB_TOTAL'						=> '',
								'SERVICE'						=> '',
								'STATUS'						=> '',
								'WEB'							=> '',
								'MAIL'							=> '',
								'FTP'							=> '',
								'RUNNING'						=> '',
								'DISABLED'						=> '' );
	my $HTTPDaemon			= new Daemon('httpd');
	my $MailDaemon			= new Daemon('sendmail', ': accepting connections');
	my @DomainList			= VirtualDomain::list_vws();
	my %ServiceStatus;
	my %Parameters;
	my $Uptime;
	my $WebalizerData		= '';
	my $WebStatistics;
	my $QuotaUsed;
	my $QuotaLimit;
	my $QuotaAvailable;
	my $QuotaPercentUsage;
	my $QuotaPercentRemain;
	my $GraphColor;
	my $DomainRef;
	my $Domain;
	my $Index;
	my $Item;
	my $Level;
	my $Days;
	my $Site;
	my $html;

	# Get the string resources.
	$StringResource->GetStrings(\%_R);
	
	# Get the domain and level of the user.
	($Domain, $Level) = Authorize::getTargetDomain();

	# Check if user is authorized to run this script
	unless (Authorize::checkAuthorization()) { return 0; }
	
	# Get the proc info and search for the uptime.
	$_		= Safe::backtick('/usr/bin/procinfo');
	$Uptime = $1 if /uptime:\s*(\S*\s*\S+)/g;
	$Uptime =~ s/context//;
	
	# Get the days (which do not exist for fresh systems).
	$Days = $Uptime;
	$Days =~ s/^(\d*)d\D+\d*:\d*:\d*\.*\d*$/$1/;
	$Days =  0 unless($Uptime ne $Days);
	
	# Pare down the uptime to remove the days and trailing space.
	$Uptime =~ s/^\d*d\D+(\d*:\d*:\d*\.*\d*)$/$1/;
	$Uptime =~ s/^(\d*:\d*:\d*\.*\d*)\s*$/$1/;
	
	# If the uptime is found...
	if ($Uptime =~ /^(\d*):(\d*):(\d*\.*\d*)$/)
	{
		my @Time	= ( $3, $2, $1, $Days );
		my $Match;
	
		$Uptime = $StringResource->_S('UP_TIME_FORMAT');
	
		$Match = '%d';
		$Uptime =~ s/$Match/$Time[3]/;
		$Match = '%H';
		$Uptime =~ s/$Match/$Time[2]/;
		$Match = '%M';
		$Uptime =~ s/$Match/$Time[1]/;
		$Match = '%S';
		$Uptime =~ s/$Match/$Time[0]/;
	}
	
	# If a usage report is required...
	if ($cgi->param('generate_usage_report')) 
	{
		# If the webalizer-domains directory does not exist...
		unless (-d "/usr/ins/intel/gui/htdocs/secure-data/webalizer-domains/$Domain")
		{
			# Create the webalizer-domains directory.
			mkdir("/usr/ins/intel/gui/htdocs/secure-data/webalizer-domains/$Domain", 0777);
        }
 
		# If the Apache access log exists...
        if (-e "/home/domain/$Domain/logs/access_log" ) 
		{
			# Run webalizer-domains on the log file...
			Safe::system("/usr/bin/webalizer -n $Domain /home/domain/$Domain/logs/access_log -o /usr/ins/intel/gui/htdocs/secure-data/webalizer-domains/$Domain");
        }
	}

	# Open the webalizer-domains statistics file...
	if (open(FILE, "< /usr/ins/intel/gui/htdocs/secure-data/webalizer-domains/$Domain/index.html"))
	{
		# Read in the web statistics.
		read(FILE, $WebalizerData, -s FILE);
		
		# Close the file.
		close(FILE);
	}

	# If there is any data...
	if ($WebalizerData)
	{
		# Parse the web statistics.
		$_							= $WebalizerData;
		my $WebReportTime			= $1 if(/Generated\s*(.+(?=<))/g);
		my $WebKBTransterred		= $1 if(/<TH BGCOLOR=\"\#C0C0C0\" ALIGN=right><FONT SIZE=\"-1\">(\w+)/g);
		my $WebTotalVisits 	     	= $1 if(/<TH BGCOLOR=\"\#C0C0C0\" ALIGN=right><FONT SIZE=\"-1\">(\w+)/g);
		my $WebTotalPageRequests	= $1 if(/<TH BGCOLOR=\"\#C0C0C0\" ALIGN=right><FONT SIZE=\"-1\">(\w+)/g);
		my $WebTotalFiles			= $1 if(/<TH BGCOLOR=\"\#C0C0C0\" ALIGN=right><FONT SIZE=\"-1\">(\w+)/g);
		my $WebTotalHits			= $1 if(/<TH BGCOLOR=\"\#C0C0C0\" ALIGN=right><FONT SIZE=\"-1\">(\w+)/g);
	
		$WebStatistics = <<HERE;
		<tr>
			<td width=281>&nbsp;<b>$_R{'REPORT_GENERATED'}</b></td>
			<td width=338>$WebReportTime</td>
		</tr>
		<tr>
			<td width=281>&nbsp;<b>$_R{'TOTAL_HTML_PAGE_REQUESTS'}</b></td>
			<td width=338>$WebTotalPageRequests</td>
		</tr>
		<tr>
			<td width=281>&nbsp;<b>$_R{'TOTAL_VISITS'}</b></td>
			<td width=338>$WebTotalVisits</td>
		</tr>
		<tr>
			<td width=281>&nbsp;<b>$_R{'TOTAL_HITS'}</b></td>
			<td width=338>$WebTotalHits</td>
		</tr>
		<tr>
			<td width=281>&nbsp;<b>$_R{'TOTAL_FILES'}</b></td>
			<td width=338>$WebTotalFiles</td>
		</tr>
		<tr>
			<td width=281>&nbsp;<b>$_R{'BYTES_TRANSFERRED'}</b></td>
			<td width=338>$WebKBTransterred $_R{'KB'}</td>
		</tr>                       
HERE
	}
	else 
	{
		$WebStatistics = <<HERE;
<tr>
	<td colspan=2>$_R{'NO_WEB_SERVER_ACTIVITY'}</td>
</tr>
HERE
	}
	
	# Get the quota information.
	$Parameters{'q'}			= $cgi;
	$Parameters{'domain'}		= $Domain;
	($QuotaUsed, $QuotaLimit)	= Quotaall::repquota(\%Parameters);
	$QuotaLimit					= GetHomeUsage() unless($QuotaLimit);
	$QuotaAvailable				= $QuotaLimit - $QuotaUsed;
    
	# Calculate the graph parameters.
	$QuotaPercentUsage	= (0 < $QuotaAvailable) ? FormatFloat($QuotaUsed / $QuotaLimit * 100) : 100;
	$QuotaPercentRemain	= FormatFloat(100 - $QuotaPercentUsage);
	$GraphColor			= ($QuotaPercentUsage) ? '#008000' : 'white';
	$QuotaPercentUsage	= $QuotaPercentUsage . '%';
	$QuotaPercentRemain	= $QuotaPercentRemain . '%';
	
	# Format the quota data.
	$QuotaLimit		= FormatFloat($QuotaLimit / 1024);
	$QuotaAvailable = FormatFloat($QuotaAvailable / 1024);

	# Run through the domain list...
	foreach $Item (@DomainList) 
	{
		# If the domain name matches...
		if ($Domain eq $Item->{'vdn'})
		{
			# Get the index.
			$Index = $Item->{'idx'};
			last;
		}
	}

	# Get the domain information.
	$DomainRef = $DomainList[$Index];
	
	# Get the associated site...
	$Site = Mail::get_site_from_domain($Domain);
	
	# Get the status information...
	$ServiceStatus{'http'}	= ($HTTPDaemon->GetStatus()) ? "<font color=green>$_R{'RUNNING'}<font>" : "<font color=red>$_R{'DISABLED'}<font>";
	$ServiceStatus{'mail'}	= ($MailDaemon->GetStatus() && ('checked' ne Mail::mail_check($Site))) ? "<font color=green>$_R{'RUNNING'}<font>" : "<font color=red>$_R{'DISABLED'}<font>";
	$ServiceStatus{'ftp'}	= (GetFTPStatus() && ('1' ne VirtualFtp::domainFtpStatus($Domain, $DomainRef->{'dom_grp'}, $DomainRef->{'vdip'}))) ? "<font color=green>$_R{'RUNNING'}<font>" : "<font color=red>$_R{'DISABLED'}<font>";
	
	$html = <<HERE;
<table border=0 cellpadding=0 cellspacing=1 width=375>
<tr>
	<td bgcolor=white height=25>
		<table border=0 width=100% cellspacing=0 cellpadding=2 bordercolor=#eeeeee bgcolor=#ffffff>
		<tr>
			<td align=left width=115><b>$_R{'UP_TIME'}</b></td>
			<td align=left width=213>$Uptime</td>
		</tr>
		</table>
	</td>
</tr>
</table>
<br>
<table border=0 cellpadding=2 cellspacing=0 width=375 bgcolor=#ffffff bordercolor=#eeeeee>
<tr>
	<td><b>$_R{'WEB_SERVER_STATISTICS'}</b></td>
</tr>
$WebStatistics
</table>
<br>
<table border=0 cellpadding=2 cellspacing=0 width=375 bgcolor=#ffffff bordercolor=#eeeeee>
<tr>
	<td><b>$_R{'DRIVE_SPACE_USAGE'}</b></td>
</tr>
</table>
<table border=1 cellpadding=2 cellspacing=0 width=375 bgcolor=#ffffff bordercolor=#eeeeee>
<tr>
	<td width=35% nowrap>
		<table border=0 cellpadding=0 cellspacing=0 width=100%>
		<tr align=right>
			<td width=25%>&nbsp;$QuotaPercentUsage&nbsp;</td>
			<td>
				<table border=1 cellpadding=0 cellspacing=0 width=70>
				<tr>
					<td width=95% bgcolor=#ffffff>
						<table border=0 width=100% cellspacing=0 cellpadding=0>
						<tr>
							<td width=$QuotaPercentUsage bgcolor=$GraphColor>&nbsp;</td>
							<td width=$QuotaPercentRemain></td>
						</tr>
						</table>
					</td>
				</tr>
				</table>
			</td>
		</tr>
		</table>
	</td>
	<td width=65%>&nbsp;<nobr>$QuotaAvailable $_R{'MB_FREE'} ($QuotaLimit $_R{'MB_TOTAL'})</nobr></td>
</tr>
</table>
<br>
<table border=0 cellpadding=0 cellspacing=0 width=375 bgcolor=#eeeeee>
<tr>
	<td width=100%>
		<table border=1 cellpadding=2 cellspacing=0 width=100% bgcolor=#ffffff bordercolor=#eeeeee>
		<tr>
			<td width=40% bgcolor=#808080><b><font color=#ffffff>$_R{'SERVICE'}</font></b></td>
			<td width=15% bgcolor=#808080><b><font color=#ffffff>$_R{'STATUS'}</font></b></td>
		</tr>
		<tr>
			<td width=40%><b><a href=domain_http.cgi>$_R{'WEB'}</a></b></td>
			<td width=15%>$ServiceStatus{'http'}</td>
		</tr>
		<tr>
			<td width=40%><b><a href=domain_mail.cgi>$_R{'MAIL'}</a></b></td>
			<td width=15%>$ServiceStatus{'mail'}</td>
		</tr>
		<tr>
			<td width=40%><b><a href=domain_ftp.cgi>$_R{'FTP'}</a></b></td>
			<td width=15%>$ServiceStatus{'ftp'}</td>
		</tr>
		</table>
	</td>
</tr>
</table>
HERE

	# Display the page.
	DisplayMain->printDocument({ 'q' => $cgi, 'html' => $html, 
								 'class' => 'domain_status', 'id' => 7000, 'title' => $_R{'IDT:7000'},
								 'root-link' => 'domain_status' });
}

###############################################################################
# The main body of the script.
###############################################################################

my $cgi = new CGI;

Main($cgi);
