#!/usr/bin/perl

###############################################################################
#                                                                             #
# Filename: domain_usage.cgi                                                  #
#                                                                             #
# Description: CGI needed to display domain usage information.                #
#                                                                             #
###############################################################################
###############################################################################
#                                                                             #
#  Copyright  2000 - 2001 Intel Corporation.                                 #
#  Intel Corporation All Rights Reserved.                                     #
#                                                                             #
#  The source code contained or described herein and all documents related to #
#  the source code ("Material") are owned by Intel Corporation or its         #
#  suppliers or licensors.  Title to the Material remains with Intel          #
#  Corporation or its suppliers and licensors.  The Material contains trade   #
#  secrets and proprietary and confidential information of Intel or its       #
#  suppliers and licensors.  The Material is protected by worldwide copyright #
#  and trade secret laws and treaty provisions.  No part of the Material may  #
#  be used, copied, reproduced, modified, published, uploaded, posted,        #
#  transmitted, distributed, or disclosed in any way without Intel's prior    #
#  express written permission.                                                #
#                                                                             #
#  No license under any patent, copyright, trade secret or other intellectual #
#  property right is granted to or conferred upon you by disclosure or        #
#  delivery of the Materials, either expressly, by implication, inducement,   #
#  estoppel or otherwise. Any license under such intellectual property rights #
#  must be express and approved by Intel in writing.                          #
#                                                                             #
###############################################################################

# Put the PERL library in the path.
use lib '/usr/ins/intel/gui/perllib';

use strict;
use LanguageResource;
use DisplayMain;
use Authorize;
use Safe;
use CGI;

###############################################################################
# Main
###############################################################################

sub Main
{
	my $cgi					= shift;
	my $PreferredLanguage	= (split(/,/, $ENV{'HTTP_ACCEPT_LANGUAGE'}))[0];
	my $StringResource		= new LanguageResource($PreferredLanguage, '/usr/ins/intel/gui/locale', [ 'domain_usage.lrc' ]);
	my %_R					= ( 'IDT:6500'					=> '',
								'NO_WEB_SERVER_ACTIVITY'	=> '',
								'SERVER_USAGE_REPORTS'		=> '' );
	my $WebalizerData		= '';
	my $WebStatistics;
	my $Domain;
	my $Level;
	my $html;
  
	# Get the domain and level of the user.
	($Domain, $Level) = Authorize::getTargetDomain();

	# Check if user is authorized to run this script
	unless (Authorize::checkAuthorization()) { return 0; }
	
	# Get the string resources.
	$StringResource->GetStrings(\%_R);

	# If a usage report is required...
	if ($cgi->param('generate_usage_report')) 
	{
		# If the webalizer-domains directory does not exist...
		unless (-d "/usr/ins/intel/gui/htdocs/secure-data/webalizer-domains/$Domain")
		{
			# Create the webalizer-domains directory.
			mkdir("/usr/ins/intel/gui/htdocs/secure-data/webalizer-domains/$Domain", 0777);
        }
 
		# If the Apache access log exists...
        if (-e "/home/domain/$Domain/logs/access_log" ) 
		{
			# Run webalizer-domains on the log file...
			Safe::system("/usr/bin/webalizer -n $Domain /home/domain/$Domain/logs/access_log -o /usr/ins/intel/gui/htdocs/secure-data/webalizer-domains/$Domain");
        }
	}

	# Open the webalizer-domains statistics file...
	if (open(FILE, "< /usr/ins/intel/gui/htdocs/secure-data/webalizer-domains/$Domain/index.html"))
	{
		# Read in the web statistics.
		read(FILE, $WebalizerData, -s FILE);
		
		# Close the file.
		close(FILE);
	}

	# If there is any data...
	if ($WebalizerData)
	{
		$WebStatistics = <<HERE;
		<tr>
			<td colspan=2>&nbsp;<b><a href=/secure-data/webalizer-domains/$Domain/index.html target=_blank>$_R{'SERVER_USAGE_REPORTS'}</a></b></td>
		</tr>
HERE
	}
	else 
	{
		$WebStatistics = <<HERE;
		<tr>
			<td colspan=2>$_R{'NO_WEB_SERVER_ACTIVITY'}</td>
		</tr>
HERE
	}

	$html = <<HERE;
<table border=0 cellpadding=0 cellspacing=1 width=375>
<tr>
	<td bgcolor=white height=25>
		<table border=0 width=100% cellspacing=0 cellpadding=2 bordercolor=#eeeeee bgcolor=#ffffff>
$WebStatistics	
		</table>
	</td>
</tr>
</table>
HERE
	
	# Display the page.
    DisplayMain->printDocument({ 'q' => $cgi, 'html' => $html, 
								 'class' => 'domain_usage', 'id' => 6500, 'title' => $_R{'IDT:6500'},
								 'root-link' => 'domain_usage' });
}

###############################################################################
# The main body of the script.
###############################################################################

my $cgi = new CGI;

Main($cgi);
