#!/usr/bin/perl	-w

###############################################################################
#                                                                             #
# Filename: go_date.cgi                                                       #
#                                                                             #
# Description: Contains CGI script needed to enable group operations for date #
#              configuration of element nodes.                                #
#                                                                             #
###############################################################################
###############################################################################
#                                                                             #
#  Copyright  2000 - 2001 Intel Corporation.                                 #
#  Intel Corporation All Rights Reserved.                                     #
#                                                                             #
#  The source code contained or described herein and all documents related to #
#  the source code ("Material") are owned by Intel Corporation or its         #
#  suppliers or licensors.  Title to the Material remains with Intel          #
#  Corporation or its suppliers and licensors.  The Material contains trade   #
#  secrets and proprietary and confidential information of Intel or its       #
#  suppliers and licensors.  The Material is protected by worldwide copyright #
#  and trade secret laws and treaty provisions.  No part of the Material may  #
#  be used, copied, reproduced, modified, published, uploaded, posted,        #
#  transmitted, distributed, or disclosed in any way without Intel's prior    #
#  express written permission.                                                #
#                                                                             #
#  No license under any patent, copyright, trade secret or other intellectual #
#  property right is granted to or conferred upon you by disclosure or        #
#  delivery of the Materials, either expressly, by implication, inducement,   #
#  estoppel or otherwise. Any license under such intellectual property rights #
#  must be express and approved by Intel in writing.                          #
#                                                                             #
###############################################################################

# Put the PERL library in the path.
use lib '/usr/ins/intel/gui/perllib';

use strict;
use LanguageResource;
use GroupOperations;
use Chronos;
use Zone;
use CGI;

###############################################################################
# GetTimeZone
###############################################################################

sub GetTimeZone
{
	my $TimeZone = '';

	# Try to open the timezone file...
	if (open(TIMEZONE, '/etc/sysconfig/clock'))
	{
		my $Label;

		# Loop through the file looking for the timezone.
		while (<TIMEZONE>) 
		{
			# If this line has to the right label...
			if (/^ZONE/) 
			{ 
				# Get the timezone information and clean it up.
				($Label, $TimeZone) = split(/=/, $_); 
				chomp($TimeZone);
			}
		}

		# Close the file.
		close(TIMEZONE);
	}

	return $TimeZone;
}

###############################################################################
# ValidatePostData
###############################################################################

sub ValidatePostData
{
	my ($cgi)	= @_;
	my $Return	= 0;

	# If data is being submitted...
	if ($cgi->param('applya'))
	{
		my $Time = $cgi->param('time');
		my ($Hour, $Minute, $Second);

		# Validate the time...
		if ($Time =~ /^\d{1,2}:\d{1,2}:\d{1,2}$/)
		{
			my ($Hour, $Minute, $Second) = split(/:/, $Time);

			# Validate the fields.
			if ((24 > $Hour) &&
				(60 > $Minute) &&
				(60 > $Second))
			{
				my %MonthDayHash	= ( '01' => 31,
									    '02' => 29, 
										'03' => 31,
										'04' => 30,
										'05' => 31,
										'06' => 30,
										'07' => 31,
										'08' => 31,
										'09' => 30,
										'10' => 31,
										'11' => 30,
										'12' => 31 );
				my $Month			= $cgi->param('month');
				my $Date			= $cgi->param('date');
				my $Year			= $cgi->param('year');
				
				# Check for a valid date other than February.
				if ($Date <= $MonthDayHash{$Month})
                {
					# Validate for February.
					if ((2 == $Month) &&
						($Date > (((0 == ($Year % 4)) && ((!(0 == ($Year % 100))) || ($Year % 400 == 0))) ? 29 : 28)))
					{
						# Set the return.
						$Return = 1901;
					}
				}
				else
				{
					# Set the return.
					$Return = 1901;
				}
			}
			else
			{
				# Set the return.
				$Return = 1900;
			}
		}
		else
		{
			# Set the return.
			$Return = 1900;
		}
	}

	return $Return;
}

###############################################################################
# Main
###############################################################################

sub Main
{
	my ($cgi)						= @_;
    my $html;
	my $Result;
	my $Status;
	my $PreferredLanguage			= (split(/,/, $ENV{'HTTP_ACCEPT_LANGUAGE'}))[0];
	my $TimeData					= new Chronos();
	my $StringResource				= new LanguageResource($PreferredLanguage, '/usr/ins/intel/gui/locale', [ 'go_date.lrc', 'common-em.lrc' ]);
	my $GroupOperations				= new GroupOperations($PreferredLanguage, 'go_date', 19, '/cgi-bin/emc/cp_date.cgi', 5555);
	my %_R							= ( 'IDT:19'				=> '',
										'TIME'					=> '',
										'DATE'					=> '',
										'TIME_ZONE'				=> '',
										'NETWORK_TIME_PROTOCOL'	=> '',
										'SYNCHRONIZE_WITH_NTP'	=> '',
										'NTP_SERVER_ADDRESS'	=> '',
										'JAN'					=> '',
										'FEB'					=> '',
										'MAR'					=> '',
										'APR'					=> '',
										'MAY'					=> '',
										'JUN'					=> '',
										'JUL'					=> '',
										'AUG'					=> '',
										'SEP'					=> '',
										'OCT'					=> '',
										'NOV'					=> '',
										'DEC'					=> '',
										'SERVICE'			=> '',
										'CONTROLS'			=> '',
										'START'				=> '',
										'STOP'				=> '',
										'RESTART'			=> '',
										'APPLY'					=> '' );
	my $TargetListContentHidden;
	my $PostActivity				= ($cgi->param('applya') || 
									   $cgi->param('applyb') ||
									   $cgi->param('control_start'  ) ||
									   $cgi->param('control_stop'   ) ||
									   $cgi->param('control_restart')) ? 1 : 0;
	my %NumberMonthHash;
	my %CurrentTime;
	my $Time						= $cgi->param('time');
	my $Month						= $cgi->param('month');
	my $Date						= $cgi->param('date');
	my $Year						= $cgi->param('year');
	my $TimeZone					= $cgi->param('zone');
	my $NTPServer					= $cgi->param('ntpserver');
	my $NTPTimeZone					= $cgi->param('ntptz');
	my $SelectionMonth				= '';
	my $SelectionDate				= '';
	my $SelectionYear				= '';
	my $SelectionTimeZone			= '';
	my $SelectionNTPTimeZone		= '';
	my $Selection;
	my @TimeZones					= Zone::tzone;
	my $Item;
	
	# Get the string resources.
	$StringResource->GetStrings(\%_R);
	
	# Fill in the hash.
	%NumberMonthHash				= ( '01' => $_R{'JAN'}, 
									    '02' => $_R{'FEB'},
									    '03' => $_R{'MAR'},
									    '04' => $_R{'APR'},
									    '05' => $_R{'MAY'},
									    '06' => $_R{'JUN'},
									    '07' => $_R{'JUL'},
									    '08' => $_R{'AUG'},
									    '09' => $_R{'SEP'},
									    '10' => $_R{'OCT'},
									    '11' => $_R{'NOV'},
									    '12' => $_R{'DEC'} );

	# Get the current time.
	$TimeData->GetTime(\%CurrentTime);

    # Default values.
	$Time				= sprintf("%02d:%02d:%02d", $CurrentTime{'hour'}, $CurrentTime{'minute'}, $CurrentTime{'second'}) unless(defined($Time));
	$Month				= sprintf("%02d", $CurrentTime{'month'}) unless(defined($Month));
	$Date				= $CurrentTime{'date'} unless(defined($Date));
	$Year				= $CurrentTime{'year'} unless(defined($Year));
	$TimeZone			= GetTimeZone unless(defined($TimeZone));
	$NTPServer			= '' unless(defined($NTPServer));
	$NTPTimeZone		= GetTimeZone unless(defined($NTPTimeZone));

	# Run through all of the months...
	foreach $Item (sort(keys(%NumberMonthHash)))
	{
		$Selection		 = ($Item eq $Month) ? 'selected' : '';
		$SelectionMonth	.= "<option value=$Item $Selection>$NumberMonthHash{$Item}\n";
	} 

	# Run through all of the dates...
	foreach $Item (1..31) 
	{
		$Selection		 = ($Item eq $Date) ? 'selected' : '';
		$SelectionDate	.= "<option value=$Item $Selection>$Item\n"; 
	}

	# Run through all of the years...
	foreach $Item (1970..2037) 
	{
		$Selection		 = ($Item eq $Year) ? 'selected' : '';
		$SelectionYear	.= "<option value=$Item $Selection>$Item\n"; 
	}

	# Run through all of the timezones...	
    foreach $Item (@TimeZones) 
	{
		$Selection			 = ($Item eq $TimeZone) ? 'selected' : '';
		$SelectionTimeZone	.= "<option value=$Item $Selection>$Item\n";
	}

	# Run through all of the timezones...	
    foreach $Item (@TimeZones) 
	{
		$Selection			 	= ($Item eq $NTPTimeZone) ? 'selected' : '';
		$SelectionNTPTimeZone	.= "<option value=$Item $Selection>$Item\n";
	}
	
	# If applying changes...
	if ($PostActivity)
	{
		# Validate the POST data...
		if (0 == ($Result = ValidatePostData($cgi)))
		{
			# Generate the distribution directive...
			unless (0 == ($Result = $GroupOperations->GenerateDirective($cgi)))
			{
				# Get the error string.
				$Status = $GroupOperations->GetErrorString($Result);
			}
		}
		else
		{
			# Get the error string.
			$Status = $StringResource->_S($Result);
		}
	}
	
	# Get the hidden target list content.
	$TargetListContentHidden = $GroupOperations->GetTargetListContentHidden;

    $html = <<HERE;
<form method="post" action="go_date.cgi" enctype="application/x-www-form-urlencoded">
  <table border="0" width="375" bgcolor="#EEEEEE">
    <tr>
      <td>
        <table border="0" width="100%">
          <tr>
            <td colspan="2" valign="top">&nbsp;</td>
          </tr>
          <tr>
            <td width="35%"><b>Time:</b></td>
            <td nowrap width="65%">
			<script>
		    if(mysel == 0)
		    {
	          document.write("<input type=text name=time DISABLED size=11 maxlength=8 value=$Time>");
		    }
		    else
		    {	   
		      document.write("<input type=text name=time size=11 maxlength=8 value=$Time>");
		    }
            </script>
			</td>
          </tr>
          <tr>
            <td width="35%"><b>Date:</b></td>
            <td nowrap width="65%">
		    <script>
		    if(mysel == 0)
		    {
		      document.write("<select size=1 name=month DISABLED>");
		    }
		    else
		    {	   
		      document.write("<select size=1 name=month>");
		    }
            </script>
			$SelectionMonth</select>
			<script>
		    if(mysel == 0)
		    {
			  document.write("<select size=1 name=date DISABLED>");
		    }
		    else
		    {	   
			  document.write("<select size=1 name=date>");
		    }
            </script>
			$SelectionDate</select>
			<script>
			if(mysel == 0)
		    {
			  document.write("<select size=1 name=year DISABLED>");
		    }
		    else
		    {	   
			  document.write("<select size=1 name=year>");
		    }
            </script>
			$SelectionYear</select>
			</td>
          </tr>
          <tr>
            <td width="35%"><b>Time zone:</b></td>
            <td nowrap width="65%">
		    <script>
		    if(mysel == 0)
		    {
	          document.write("<select size=1 name=zone DISABLED>");
		    }
		    else
		    {	   
		      document.write("<select size=1 name=zone>");
		    }
            </script>
			$SelectionTimeZone</select>
			</td>
          </tr>
          <tr>
            <td nowrap colspan="2">&nbsp;</td>
          </tr>
          <tr>
            <td nowrap colspan="2" align="right">
		    <script>
			if(mysel == 0)
		    {
	          document.write("<input type=submit class=clsButton DISABLED value='$_R{'APPLY'}' name=applya>");
		    }
		    else
		    {	   
		      document.write("<input type=submit class=clsButton value='$_R{'APPLY'}' name=applya>");
		    }
            </script>
            </td>
          </tr>
        </table>
      </td>
    </tr>
  </table>
  $TargetListContentHidden
</form>
<form method="post" action="go_date.cgi" enctype="application/x-www-form-urlencoded">
    <table border="0" width="375" bgcolor="#EEEEEE">
	<tr>
      <td colspan="2" width="100%">
		&nbsp;
		<table border="1" cellpadding="2" cellspacing="0" width="100%" bgcolor="#FFFFFF" bordercolor="#EEEEEE">
		<tr>
			<td width="58%" bgcolor="#808080"><b><font color="#FFFFFF">$_R{'SERVICE'}</font></b></td>
			<td width="42%" bgcolor="#808080"><b><font color="#FFFFFF">$_R{'CONTROLS'}</font></b></td>
		</tr>
		<tr>
		  <td width="58%"><b>$_R{'NETWORK_TIME_PROTOCOL'}</b></td>
		  <td width="42%" align="left" nowrap>
			<script>
			  if(mysel == 0)
			  {
	            document.write("<input type=submit class=clsButton DISABLED value='$_R{'START'}' name=control_start> ");
				document.write("<input type=submit class=clsButton DISABLED value='$_R{'STOP'}' name=control_stop> ");
				document.write("<input type=submit class=clsButton DISABLED value='$_R{'RESTART'}' name=control_restart>");
			  }
			  else
			  {
	            document.write("<input type=submit class=clsButton value='$_R{'START'}' name=control_start> ");
				document.write("<input type=submit class=clsButton value='$_R{'STOP'}' name=control_stop> ");
				document.write("<input type=submit class=clsButton value='$_R{'RESTART'}' name=control_restart>");
			  }
			</script>
		    </td>
    	  </tr>
		</table>
        <table border="0" width="100%">
          <tr>
            <td colspan="2" align="left">&nbsp;</td>
          </tr>
          <tr>
            <td width="35%" nowrap colspan=2>&nbsp;<b>$_R{'NTP_SERVER_ADDRESS'}</b>&nbsp;&nbsp;
		    <script>
		    if(mysel == 0)
		    {
	          document.write("<input type=text name=ntpserver DISABLED  size=19 maxlength=32 value=$NTPServer>");
		    }
		    else
		    {	   
		      document.write("<input type=text name=ntpserver size=19 maxlength=32 value=$NTPServer>");
		    }
            </script>
			</td>
          </tr>
          <tr>
			<td colspan="2">
			<table width='100%'>
			<tr>
              <td width="35%"><b>Time zone:</b></td>
              <td nowrap width="65%">
				<script>
				if(mysel == 0)
				{
				  document.write("<select size=1 name=ntptz DISABLED>");
				}
				else
				{	   
				  document.write("<select size=1 name=ntptz>");
				}
				</script>
				$SelectionNTPTimeZone</select>
			  </td>
			</tr>
			</table>
			</td>
          </tr>
          <tr>
            <td width="418" valign="top" align="left" colspan="2">&nbsp;&nbsp; &nbsp;</td>
          </tr>
          <tr>
            <td colspan="2" align="right">
		    <script>
		    if(mysel == 0)
		    {
	          document.write("<input type=submit class=clsButton DISABLED value='$_R{'APPLY'}' name=applyb>");
		    }
		    else
		    {	   
		      document.write("<input type=submit class=clsButton value='$_R{'APPLY'}' name=applyb>");
		    }
            </script>
          	</td>
          </tr>
        </table>
      </td>
    </tr>
  </table>
  <br>
  $TargetListContentHidden
</form>
HERE

	# Render the page.
	$GroupOperations->RenderPageContent({ 'cgi' => $cgi, 'title' => $_R{'IDT:19'}, 'html' => $html, 'result' => $Result, 'status' => $Status, 'root-link' => 'go_date' });
}

###############################################################################
# The main body of the script.
###############################################################################

my $cgi = new CGI;

Main($cgi);

