#!/usr/bin/perl	-w

###############################################################################
#                                                                             #
# Filename: go_ftp.cgi                                                        #
#                                                                             #
# Description: Contains CGI script needed to enable group operations for FTP  #
#              configuration of element nodes.                                #
#                                                                             #
###############################################################################
###############################################################################
#                                                                             #
#  Copyright  2000 - 2001 Intel Corporation.                                 #
#  Intel Corporation All Rights Reserved.                                     #
#                                                                             #
#  The source code contained or described herein and all documents related to #
#  the source code ("Material") are owned by Intel Corporation or its         #
#  suppliers or licensors.  Title to the Material remains with Intel          #
#  Corporation or its suppliers and licensors.  The Material contains trade   #
#  secrets and proprietary and confidential information of Intel or its       #
#  suppliers and licensors.  The Material is protected by worldwide copyright #
#  and trade secret laws and treaty provisions.  No part of the Material may  #
#  be used, copied, reproduced, modified, published, uploaded, posted,        #
#  transmitted, distributed, or disclosed in any way without Intel's prior    #
#  express written permission.                                                #
#                                                                             #
#  No license under any patent, copyright, trade secret or other intellectual #
#  property right is granted to or conferred upon you by disclosure or        #
#  delivery of the Materials, either expressly, by implication, inducement,   #
#  estoppel or otherwise. Any license under such intellectual property rights #
#  must be express and approved by Intel in writing.                          #
#                                                                             #
###############################################################################

# Put the PERL library in the path.
use lib '/usr/ins/intel/gui/perllib';

use strict;
use LanguageResource;
use GroupOperations;
use CGI;

###############################################################################
# ValidatePostData
###############################################################################

sub ValidatePostData
{
	my ($cgi)	= @_;
	my $Return	= 0;

	# If data is being submitted...
	if ($cgi->param('apply'))
	{
		my $MaxLoginFailureAttempts			= $cgi->param('max_login_failure_attempts');
		my $MaxLoginUsers					= $cgi->param('max_login_users');
		my $MaxFileTransferSize				= $cgi->param('max_file_transfer_size');
		my $MaxFileTransferIdle				= $cgi->param('max_file_transfer_idle');

		if ($MaxLoginFailureAttempts !~ /^\d+$/)
		{
			# Set the return code.
			$Return = 1700;
		}
		elsif ($MaxLoginUsers !~ /^\d+$/)
		{
			# Set the return code.
			$Return = 1701;
		}
		elsif (($MaxFileTransferSize !~ /^\+*([0-9]*)\.?([0-9]*)$/) || 
			   (!($1 || ('0' eq $1) || $2 || ('0' eq $2))))
		{
			# Set the return code.
			$Return = 1702;
		}
		elsif ($MaxFileTransferIdle !~ /^\d+$/)
		{
			# Set the return code.
			$Return = 1703;
		}
	}

	return $Return;
}

###############################################################################
# Main
###############################################################################

sub Main
{
	my ($cgi)						= @_;
    my $html;
	my $Result;
	my $Status;
	my $PreferredLanguage			= (split(/,/, $ENV{'HTTP_ACCEPT_LANGUAGE'}))[0];
	my $StringResource				= new LanguageResource($PreferredLanguage, '/usr/ins/intel/gui/locale', [ 'go_ftp.lrc', 'common-em.lrc' ]);
	my $GroupOperations				= new GroupOperations($PreferredLanguage, 'go_ftp', 17, '/cgi-bin/emc/cp_ftp.cgi', 5555);
	my %_R							= ( 'IDT:17'						=> '',
										'SERVICE'						=> '',
										'CONTROLS'						=> '',
										'FTP'							=> '',
										'START'							=> '',
										'STOP'							=> '',
										'ALLOW_ANONYMOUS_LOGIN'			=> '',
										'BANNER_MESSAGE_USERS'			=> '',
										'FAILED_LOGIN_ATTEMPTS'			=> '',
										'MAX_LOGGED_IN_USERS'			=> '',
										'MAX_SESSION_DATA_TRANSFER'		=> '',
										'IDLE_TIME_BEFORE_DISCONNECT'	=> '',
										'MB'							=> '',
										'SEC'							=> '',
										'APPLY'							=> '' );
	my $TargetListContentHidden;
	my $PostActivity				= ($cgi->param('apply') || $cgi->param('control_start') ||
															   $cgi->param('control_stop' )) ? 1 : 0;
	my $EnableAnonymousFTP			= $cgi->param('enable_anonymous_ftp') || '';
	my $BannerSystemUsers			= $cgi->param('banner_system_users');
	my $MaxLoginFailureAttempts		= $cgi->param('max_login_failure_attempts');
	my $MaxLoginUsers				= $cgi->param('max_login_users');
	my $MaxFileTransferSize			= $cgi->param('max_file_transfer_size');
	my $MaxFileTransferIdle			= $cgi->param('max_file_transfer_idle');

	# Readjust the value.
	$EnableAnonymousFTP = 'checked' if ($EnableAnonymousFTP eq 'yes');

	# Get the string resources.
	$StringResource->GetStrings(\%_R);
	
	# Safe defaults.
	$BannerSystemUsers			= ''  unless(defined($BannerSystemUsers));
	$MaxLoginFailureAttempts	= 5   unless(defined($MaxLoginFailureAttempts));
	$MaxLoginUsers				= 10  unless(defined($MaxLoginUsers));
	$MaxFileTransferSize		= 50  unless(defined($MaxFileTransferSize));
	$MaxFileTransferIdle		= 900 unless(defined($MaxFileTransferIdle));

	# If applying changes...
	if ($PostActivity)
	{
		# Validate the POST data...
		if (0 == ($Result = ValidatePostData($cgi)))
		{
			# Generate the distribution directive...
			unless (0 == ($Result = $GroupOperations->GenerateDirective($cgi)))
			{
				# Get the error string.
				$Status = $GroupOperations->GetErrorString($Result);
			}
		}
		else
		{
			# Get the error string.
			$Status = $StringResource->_S($Result);
		}
	}

	# Get the hidden target list content.
	$TargetListContentHidden = $GroupOperations->GetTargetListContentHidden();

    $html = <<HERE;
<form method="POST" action="go_ftp.cgi" ENCTYPE="application/x-www-form-urlencoded">
  <table border="0" width="375" bgcolor="#EEEEEE">
    <tr>
      <td>
        <br>
        <table border="0" cellpadding="0" cellspacing="0" width="100%" bgcolor="#EEEEEE">
        <tr> 
          <td width="100%"> 
          <table border="1" cellpadding="2" cellspacing="0" width="100%" bgcolor="#FFFFFF" bordercolor="#EEEEEE"> 
          <tr> 
            <td width="58%" bgcolor="#808080"><b><font color="#FFFFFF">$_R{'SERVICE'}</font></b></td> 
            <td width="42%" align="left" bgcolor="#808080"><font color="#FFFFFF"><b>$_R{'CONTROLS'}</b></font></td> 
          </tr> 
          <tr> 
            <td width="58%"><b>$_R{'FTP'}</b></td> 
            <td width="42%" align="left" nowrap>
			<script>
			  if(mysel == 0)
			  {
	            document.write("<input type=submit class=clsButton DISABLED value='$_R{'START'}' name=control_start> ");
				document.write("<input type=submit class=clsButton DISABLED value='$_R{'STOP'}' name=control_stop>");
			  }
			  else
			  {
	            document.write("<input type=submit class=clsButton value='$_R{'START'}' name=control_start> ");
				document.write("<input type=submit class=clsButton value='$_R{'STOP'}' name=control_stop>");
			  }
			</script>
            </td> 
          </tr> 
          </table> 
          </td> 
        </tr> 
        </table> 
        <table border="0" width="100%">
          <tr>
            <td width="100%" colspan="2">&nbsp;</td>
          </tr>
          <tr>
            <td width="81%" valign="top">
			<script>
			if(mysel == 0)
		    {
	          document.write("<input type=checkbox name=enable_anonymous_ftp DISABLED value=yes $EnableAnonymousFTP>");
		    }
		    else
		    {	   
		      document.write("<input type=checkbox name=enable_anonymous_ftp value=yes $EnableAnonymousFTP>");
		    }
            </script><b>$_R{'ALLOW_ANONYMOUS_LOGIN'}</b></td>
            <td width="19%"></td>
          </tr>
          <tr>
            <td width="100%" colspan="2">&nbsp;</td>
          </tr>
          <tr>
            <td width="100%" colspan="2"><b>$_R{'BANNER_MESSAGE_USERS'}</b></td>
          </tr>
          <tr>
            <td width="100%" colspan="2">
			<script>
			if(mysel == 0)
		    {
	          document.write("<textarea name=banner_system_users DISABLED rows=4 cols=41 wrap=physical>");
		    }
		    else
		    {	   
		      document.write("<textarea name=banner_system_users rows=4 cols=41 wrap=physical>");
		    }
            </script>$BannerSystemUsers</textarea>
			</td>
          </tr>
          <tr>
            <td width="100%" colspan="2">&nbsp;</td>
          </tr>
          <tr>
            <td width="81%"><b>$_R{'FAILED_LOGIN_ATTEMPTS'}</b></td>
            <td width="21%">
			<script>
		    if(mysel == 0)
		    {
	          document.write("<input type=text name=max_login_failure_attempts DISABLED size=3 maxlength=3 value=$MaxLoginFailureAttempts>");
		    }
		    else
		    {	   
		      document.write("<input type=text name=max_login_failure_attempts size=3 maxlength=3 value=$MaxLoginFailureAttempts>");
		    }
            </script>
			</td>
          </tr>
          <tr>
            <td width="81%"><b>$_R{'MAX_LOGGED_IN_USERS'}</b>&nbsp;</td>
            <td width="21%">
			<script>
		    if(mysel == 0)
		    {
	          document.write("<input type=text name=max_login_users DISABLED size=3 maxlength=4 value=$MaxLoginUsers>");
		    }
		    else
		    {	   
		      document.write("<input type=text name=max_login_users size=3 maxlength=4 value=$MaxLoginUsers>");
		    }
            </script>
			</td>
          </tr>
		  <tr>
		    <td width="81%"><b>$_R{'MAX_SESSION_DATA_TRANSFER'}</b></td>
			<td width="21%" nowrap>
			<script>
		    if(mysel == 0)
		    {
	          document.write("<input type=text name=max_file_transfer_size DISABLED size=3 maxlength=4 value=$MaxFileTransferSize>");
		    }
		    else
		    {	   
		      document.write("<input type=text name=max_file_transfer_size size=3 maxlength=4 value=$MaxFileTransferSize>");
		    }
            </script>$_R{'MB'}</td>
          </tr> 
		    <tr width="81%"><td><b>$_R{'IDLE_TIME_BEFORE_DISCONNECT'}</b></td>
			<td width="21%" nowrap>
	        <script>
		    if(mysel == 0)
		    {
	          document.write("<input type=text name=max_file_transfer_idle DISABLED size=4 maxlength=5 value=$MaxFileTransferIdle>");
		    }
		    else
		    {	   
		      document.write("<input type=text name=max_file_transfer_idle size=4 maxlength=5 value=$MaxFileTransferIdle>");
		    }
            </script>$_R{'SEC'}</td>
          <tr>
            <td width="100%" colspan="2">&nbsp;</td>
          </tr>
          <tr>
            <td width="100%" align="right" colspan="2">
	        <script>
		    if(mysel == 0)
		    {
	          document.write("<input type=submit class=clsButton DISABLED value='$_R{'APPLY'}' name=apply>");
		    }
		    else
		    {	   
		      document.write("<input type=submit class=clsButton value='$_R{'APPLY'}' name=apply>");
		    }
            </script>				  
			</td>
          </tr>
        </table>
      </td>
    </tr>
  </table>
  <br>
  $TargetListContentHidden
</form>
HERE

	# Render the page.
	$GroupOperations->RenderPageContent({ 'cgi' => $cgi, 'title' => $_R{'IDT:17'}, 'html' => $html, 'result' => $Result, 'status' => $Status, 'root-link' => 'go_ftp' });
}

###############################################################################
# The main body of the script.
###############################################################################

my $cgi = new CGI;

Main($cgi);
