#!/usr/bin/perl	-w

###############################################################################
#                                                                             #
# Filename: navbar.cgi                                                        #
#                                                                             #
# Description: Contains CGI script needed to render the group operations      #
#              navigation bar.                                                #
#                                                                             #
###############################################################################
###############################################################################
#                                                                             #
#  Copyright  2000 - 2001 Intel Corporation.                                 #
#  Intel Corporation All Rights Reserved.                                     #
#                                                                             #
#  The source code contained or described herein and all documents related to #
#  the source code ("Material") are owned by Intel Corporation or its         #
#  suppliers or licensors.  Title to the Material remains with Intel          #
#  Corporation or its suppliers and licensors.  The Material contains trade   #
#  secrets and proprietary and confidential information of Intel or its       #
#  suppliers and licensors.  The Material is protected by worldwide copyright #
#  and trade secret laws and treaty provisions.  No part of the Material may  #
#  be used, copied, reproduced, modified, published, uploaded, posted,        #
#  transmitted, distributed, or disclosed in any way without Intel's prior    #
#  express written permission.                                                #
#                                                                             #
#  No license under any patent, copyright, trade secret or other intellectual #
#  property right is granted to or conferred upon you by disclosure or        #
#  delivery of the Materials, either expressly, by implication, inducement,   #
#  estoppel or otherwise. Any license under such intellectual property rights #
#  must be express and approved by Intel in writing.                          #
#                                                                             #
###############################################################################

# Put the PERL library in the path.
use lib '/usr/ins/intel/gui/perllib';

use strict;
use LanguageResource;
use ProductInformation;
use Safe;
use CGI;

###############################################################################
# Main
###############################################################################

sub Main
{
	my ($cgi)			= @_;
	my $html;
	my $PreferredLanguage		= (split(/,/, $ENV{'HTTP_ACCEPT_LANGUAGE'}))[0];
	my $ImageResource		= new LanguageResource($PreferredLanguage, '/usr/ins/intel/gui/locale/images');
	my $ImgPathNavBar		= $ImageResource->ResolveResourceFile('navbar-em.jpg');
	my $Discover			= $cgi->param('discover');
	my $NewServerIP			= $cgi->param('newip');
	my $NewServerName		= $cgi->param('newname');
	my $DeleteServerGlob	= $cgi->param('delip');
	my $ProductInfo			= new ProductInformation();
	my $eCommerceHTML;


	# Safe defaults.
	$ImgPathNavBar	= '' unless(defined($ImgPathNavBar));

	# Get just the hidden part of the file name.
	$ImgPathNavBar	=~ s/^\/usr\/ins\/intel\/gui(\/locale\/images\/.*)$/$1/;

	# If performing a discovery...
	if ($Discover)
	{
		# Run subndd without the noscan option to read all machines on the subnet...
		Safe::system('/usr/ins/intel/bin/subndd');
	}
	
	# If adding a new server....
	if ((defined($NewServerName)) &&
		(''   ne $NewServerName))
	{
		my $DuplicateServer = 0;

		# Open the machine_list...
		if (open(MACHINELIST, '< /usr/ins/intel/bin/machine_list'))
		{
			# Read in the machine list.
			my @MachineList		= <MACHINELIST>;
			my @ParameterList;
			my $Line;

			# Ingnore the first line...
			shift(@MachineList);

			# Close the file.
			close(MACHINELIST);
			
			# Run through the machine list...
			foreach $Line (@MachineList) 
			{
				# Get the parameter list.
				@ParameterList = split(/,/, $Line);

				# Look for a duplicate address or server name...
				unless ((   $ParameterList[1]  eq    $NewServerIP) ||
						(lc($ParameterList[0]) eq lc($NewServerName)))
				{
					# Found that the new server already exists.
					$DuplicateServer = 1;
					last;
				}
			}
		}
		
		# Didn't find the new server so let's add it to the end...
		unless ($DuplicateServer)
		{
			# Open the machine list file for appending...
			if (open(MACHINELIST, '>> /usr/ins/intel/bin/machine_list'))
			{
				# Add the new server to the list.
				print MACHINELIST "$NewServerName,$NewServerIP,,,,-1,0,0,0,0\n";

				# Close the file.
				close(MACHINELIST);
			}
		}
	}

	# If removing servers...
	if ((defined($DeleteServerGlob)) &&
		(''   ne $DeleteServerGlob))
	{
        my @DeleteServerList = split(/,/, $DeleteServerGlob);

		# Open the machine list...
		if (open(MACHINELIST, '< /usr/ins/intel/bin/machine_list'))
		{
			# Read in the machine list.
			my @MachineList		= <MACHINELIST>;
			my @ParameterList;
			my $ServerIP;
			my $i;
			my $j;

			# Close the file.
			close(MACHINELIST);

			# Run through the machine list...
			for ($i = 1; $i < scalar(@MachineList); ++$i) 
			{
				# Get the parameter list.
				@ParameterList = split(/,/, $MachineList[$i]);

				# Run through the data...
				for ($j = 0; $j < scalar(@DeleteServerList); ++$j) 
				{
					# If the IP addresses match...
					if ($DeleteServerList[$j] eq $ParameterList[1])
					{
						# Delete the lines.
						splice(@MachineList, $i, 1);
						splice(@DeleteServerList, $j, 1);

						# Resynch the counter.
						--$i;

						# Break from this loop.
						last;
					}
				}
				
				# If there is anything in the list...
				unless (scalar(@DeleteServerList))
				{
					# Break from this loop.
					last;
				}
			}

			# Open the machine list again...
			if (open(MACHINELIST, '> /usr/ins/intel/bin/machine_list'))
			{
				# Write out the machine list.
				print MACHINELIST @MachineList;

				# Close the file.
				close(MACHINELIST);
			}
		}
	}

	$html = <<HERE;
<!doctype html public "-//IETF//DTD HTML//EN">
<html>
	<head>
        <meta name=copyright content="Copyright (c) 2000 Intel Corporation, et al.">
		<link rel=stylesheet href=/style/style.css type=text/css>
	</head>
	<form name=addmachine method=post enctype=application/x-www-form-urlencoded action=go_navbar.cgi>
		<input type=hidden name='newip' value=''>
		<input type=hidden name='newname' value=''>
		<input type=hidden name='discover' value=''>
	</form>
	<form name=delmachine method=post enctype=application/x-www-form-urlencoded action=go_navbar.cgi>
		<input type=hidden name='delip' value=''>
	</form>
	<script language=javascript>
	var machinelist	= new Array();
	var busy		= 0;

	function discoverMachineList()
	{
		busy = 1;
		document.addmachine.discover.value = '1';
		parent.main_frame.location = '/cgi-bin/ema/go_serverlistwait.cgi'
		document.addmachine.submit();	
	}
	
	function addtoMachineList(name, ip)
	{
		busy = 1;
		document.addmachine.newname.value = name;
		document.addmachine.newip.value = ip;	
		document.addmachine.submit();	
	}
	
	function deleteMachines()
	{
		var count = 0;
		
		busy = 1;
		
		for (var i = 0; i < machinelist.length; i++)
		{     
			if (1 == machinelist[i].selected)
			{	
				if (1 <= count)
				{
					document.delmachine.delip.value += ',';
				}
				
				document.delmachine.delip.value += machinelist[i].ip;
				count++;
			}
		}
		
		document.delmachine.submit();
	}
	
	function dorefresh()
	{
		document.location='/cgi-bin/ema/go_navbar.cgi';
	}
	
	function machine(machinename,ipaddress,status,webstatus,mailstatus,hardwarehealth,security)
	{
		this.name			= machinename;
		this.ip				= ipaddress;
		this.status			= status;
		this.webstatus		= webstatus;
		this.mailstatus		= mailstatus;
		this.hardwarehealth	= hardwarehealth;
		this.security		= security;
		this.selected		= 0;
	}
	
	function addMachineToList(machinename,ipaddress,dnsname,macaddress,version, status,webstatus,mailstatus,hardwarehealth,security)
	{
		machinelist[machinelist.length++] = new machine(machinename,ipaddress,status,webstatus,mailstatus,hardwarehealth,security);
	}
	
	function selectMachine(name)
	{
		for (var i = 0; i < machinelist.length; i++)
		{
			if (machinelist[i].name == name)
			{
				if (1 == machinelist[i].selected)
				{
					machinelist[i].selected= 0;
				}
				else
				{
					machinelist[i].selected= 1;
				}
			}
		}
	}
	
	function toggleMachine(name, active)
	{
		for (var i = 0; i < machinelist.length; i++)
		{
			if (machinelist[i].name == name)
			{
				machinelist[i].selected = active;
			}
		}
	}
	
HERE

	# If not performing a discovery...
	unless (defined($Discover))
	{
		# Refresh the server list status.
		Safe::system('/usr/ins/intel/bin/subndd -noscan');
	}

	# Open the machine list...
	if (open(MACHINELIST, '< /usr/ins/intel/bin/machine_list'))
	{
		# Read in the machine list.
		my ($Name, $IP, $DNSName, $MAC, $Version, $Status, $WebStatus, $MailStatus, $HardwareHealth, $Security);
		my @MachineList = <MACHINELIST>;
		my $Line;

		# Close the file.
		close(MACHINELIST);

		# Ingnore the first line...
		shift(@MachineList);

		# Run through the machine list...
		foreach $Line (@MachineList) 
		{
			# Clean off the end of the line.
			chomp($Line);

			# Get the parameter list.
			($Name, $IP, $DNSName, $MAC, $Version, $Status, $WebStatus, $MailStatus, $HardwareHealth, $Security) = split(/,/, $Line);

			# Add the server information to the machine list.
			$html .= "\taddMachineToList('$Name', '$IP', '$DNSName', '$MAC', '$Version', $Status, $WebStatus, $MailStatus, $HardwareHealth, $Security);\n";
		}
	}
	
	# Look for a storefront configuration...
	if (2 == $ProductInfo->getClassification())
	{
		$eCommerceHTML = <<HERE;
	<tr>
		<td>
			<a href=/cgi-bin/ema/go_ecomm.cgi target=main_frame onmouseover="changeImages('go_ecomm');" onmouseout="changeImages('go_ecomm');" onclick="selectitem('go_ecomm');">
			<img id=go_ecomm name=go_ecomm src=/images/blank.gif width=131 height=15 border=0></a></td>
	</tr>
HERE
	}
	else
	{
		$eCommerceHTML = '';
	}
	
	$html .= <<HERE;
	
	var selected_item = 'servers_list';
	
	selected_all = 0;

	function selectitem(itemname)
	{
		eval(\"document.images.\"+ selected_item +\".src = '/images/blank.gif';\");
		eval(\"document.images.\"+ itemname +\".src = '/images/selected_em.gif';\");
		selected_item = itemname;
	}
	
	function changeImages(itemname)
	{
		if (itemname != selected_item)
		{
			var picvalue = eval(\"document.images.\"+ itemname +\".src\");
			
			if (-1 != picvalue.indexOf('blank.gif'))
			{
				eval(\"document.images.\"+ itemname +\".src = '/images/selection_em.gif';\");
			}
			else
			{
				eval(\"document.images.\"+ itemname +\".src = '/images/blank.gif';\");
			}
		}
	}
	
	function do_main_refresh()
	{
		if (parent.main_frame.selected_item)
		{
			selectitem(parent.main_frame.selected_item);
		}
		
		if (parent.main_frame.dorefresh)
		{
			parent.main_frame.dorefresh();
		}
	}
	
	var str_appname = navigator.appName;
	
	if (-1 == str_appname.indexOf(\"Netscape\",0))
	{
		 document.write(\"<BODY BGCOLOR=#FFFFFF background=$ImgPathNavBar leftmargin=0 topmargin=0 onload='do_main_refresh();'>\");
	}
	else
	{
		 document.write(\"<BODY BGCOLOR=#FFFFFF background=$ImgPathNavBar leftmargin=0 topmargin=0 onload='do_main_refresh();' onresize='document.location.reload();'>\");
	}
	</script>
	<table border=0 cellpadding=0 cellspacing=0>
	<tr>
		<td colspan=3><img src=/images/blank.gif width=158 height=22></td>
	</tr>
	<tr>
		<td>
			<a href=/cgi-bin/ema/go_serverlist.cgi target=main_frame onmouseover="changeImages('servers_list');" onmouseout="changeImages('servers_list');" onclick="selectitem('servers_list');">
			<img id=servers_list name=servers_list src=/images/selected_em.gif width=131 height=15 border=0></a></td>
	</tr>
	<tr>
		<td colspan=3>
		<img src=/images/blank.gif width=131 height=45></td>
	</tr>
	<tr>
		<td>
			<a href=/cgi-bin/ema/go_network.cgi target=main_frame onmouseover="changeImages('go_network');" onmouseout="changeImages('go_network');" onclick="selectitem('go_network');">
			<img id=go_network name=go_network src=/images/blank.gif width=131 height=15 border=0></a></td>
	</tr>
	<tr>
		<td>
			<a href=/cgi-bin/ema/go_date.cgi target=main_frame onmouseover="changeImages('go_date');" onmouseout="changeImages('go_date');" onclick="selectitem('go_date');">
			<img id=go_date name=go_date src=/images/blank.gif width=131 height=15 border=0></a></td>
	</tr>
	<tr>
		<td>
			<a href=/cgi-bin/ema/go_password.cgi target=main_frame onmouseover="changeImages('go_password');" onmouseout="changeImages('go_password');" onclick="selectitem('go_password');">
			<img id=go_password name=go_password src=/images/blank.gif width=131 height=15 border=0></a></td>
	</tr>
	<tr>
		<td>
			<a href=/cgi-bin/ema/go_alerts.cgi target=main_frame onmouseover="changeImages('go_alerts');" onmouseout="changeImages('go_alerts');" onclick="selectitem('go_alerts');">
			<img id=go_alerts name=go_alerts src=/images/blank.gif width=131 height=15 border=0></a></td>
	</tr>
	<tr>
		<td colspan=3>
		<img src=/images/blank.gif width=131 height=30></td>
	</tr>
	<tr>
		<td>
			<a href=/cgi-bin/ema/go_http.cgi target=main_frame onmouseover="changeImages('go_http');" onmouseout="changeImages('go_http');" onclick="selectitem('go_http');">
			<img id=go_http name=go_http src=/images/blank.gif width=131 height=15 border=0></a></td>
	</tr>
	<tr>
		<td>
			<a href=/cgi-bin/ema/go_mail.cgi target=main_frame onmouseover="changeImages('go_mail');" onmouseout="changeImages('go_mail');" onclick="selectitem('go_mail');">
			<img id=go_mail name=go_mail src=/images/blank.gif width=131 height=15 border=0></a></td>
	</tr>
	<tr>
		<td>
			<a href=/cgi-bin/ema/go_ftp.cgi target=main_frame onmouseover="changeImages('go_ftp');" onmouseout="changeImages('go_ftp');" onclick="selectitem('go_ftp');">
			<img id=go_ftp name=go_ftp src=/images/blank.gif width=131 height=15 border=0></a></td>
	</tr>
	<tr>
		<td>
			<a href=/cgi-bin/ema/go_telnet.cgi target=main_frame onmouseover="changeImages('go_telnet');" onmouseout="changeImages('go_telnet');" onclick="selectitem('go_telnet');">
			<img id=go_telnet name=go_telnet src=/images/blank.gif width=131 height=15 border=0></a></td>
	</tr>
	<tr>
		<td>
			<a href=/cgi-bin/ema/go_snmp.cgi target=main_frame onmouseover="changeImages('go_snmp');" onmouseout="changeImages('go_snmp');" onclick="selectitem('go_snmp');">
			<img id=go_snmp name=go_snmp src=/images/blank.gif width=131 height=15 border=0></a></td>
	</tr>
$eCommerceHTML	
	<tr>
		<td colspan=3>
		<img src=/images/blank.gif width=131 height=30></td>
	</tr>
	<tr>
		<td>
			<a href=/cgi-bin/ema/go_swupdate.cgi target=main_frame onmouseover="changeImages('go_swupdate');" onmouseout="changeImages('go_swupdate');" onclick="selectitem('go_swupdate');">
			<img id=go_swupdate name=go_swupdate src=/images/blank.gif width=131 height=15 border=0></a></td>
	</tr>
	<tr>
		<td>
			<a href=/cgi-bin/ema/go_power.cgi target=main_frame onmouseover="changeImages('go_power');" onmouseout="changeImages('go_power');" onclick="selectitem('go_power');">
			<img id=go_power name=go_power src=/images/blank.gif width=131 height=15 border=0></a></td>
	</tr>
	</table>
</html>
HERE

	print($html);
}

###############################################################################
# The main body of the script.
###############################################################################

my $cgi = new CGI;

Main($cgi);
