#!/usr/bin/perl	-w

###############################################################################
#                                                                             #
# Filename: go_password.cgi                                                   #
#                                                                             #
# Description: Contains CGI script needed to enable password modification on  #
#              element nodes.                                                 #
#                                                                             #
###############################################################################
###############################################################################
#                                                                             #
#  Copyright  2000 - 2001 Intel Corporation.                                 #
#  Intel Corporation All Rights Reserved.                                     #
#                                                                             #
#  The source code contained or described herein and all documents related to #
#  the source code ("Material") are owned by Intel Corporation or its         #
#  suppliers or licensors.  Title to the Material remains with Intel          #
#  Corporation or its suppliers and licensors.  The Material contains trade   #
#  secrets and proprietary and confidential information of Intel or its       #
#  suppliers and licensors.  The Material is protected by worldwide copyright #
#  and trade secret laws and treaty provisions.  No part of the Material may  #
#  be used, copied, reproduced, modified, published, uploaded, posted,        #
#  transmitted, distributed, or disclosed in any way without Intel's prior    #
#  express written permission.                                                #
#                                                                             #
#  No license under any patent, copyright, trade secret or other intellectual #
#  property right is granted to or conferred upon you by disclosure or        #
#  delivery of the Materials, either expressly, by implication, inducement,   #
#  estoppel or otherwise. Any license under such intellectual property rights #
#  must be express and approved by Intel in writing.                          #
#                                                                             #
###############################################################################

# Put the PERL library in the path.
use lib '/usr/ins/intel/gui/perllib';

use strict;
use LanguageResource;
use GroupOperations;
use CGI;

###############################################################################
# ValidatePostData
###############################################################################

sub ValidatePostData
{
	my ($cgi)	= @_;
	my $Return	= 0;

	# If data is being submitted...
	if ($cgi->param('apply'))
	{
        # Cache the input data.
		my $UserName				= $cgi->param('user_name');
		my $UserPasswordNew			= $cgi->param('user_password_n');
		my $UserPasswordDup			= $cgi->param('user_password_d');
		my $UserNameLength			= (defined($UserName)) 			? length($UserName) 		: 0;
		my $UserPasswordNewLength	= (defined($UserPasswordNew))	? length($UserPasswordNew)	: 0;
		
		# Make sure that the data is valid.
		if ((0 == $UserNameLength) ||
			($UserName !~ /(\w|_|-){$UserNameLength}/))
		{
			# Set the return code.
			$Return = 1300;
		}
		elsif ($UserPasswordNew ne $UserPasswordDup)
		{
			# Set the return code.
			$Return = 1301;
		}
		elsif ($UserPasswordNew =~ /( ){$UserPasswordNewLength}/)
		{
			# Set the return code.
			$Return = 1302;
		}
	}

	return $Return;
}

###############################################################################
# Main
###############################################################################

sub Main
{
	my ($cgi)						= @_;
    my $html;
	my $Result;
	my $Status;
	my $PreferredLanguage			= (split(/,/, $ENV{'HTTP_ACCEPT_LANGUAGE'}))[0];
	my $StringResource				= new LanguageResource($PreferredLanguage, '/usr/ins/intel/gui/locale', [ 'go_password.lrc', 'common-em.lrc' ]);
	my $GroupOperations				= new GroupOperations($PreferredLanguage, 'go_password', 13, '/cgi-bin/emc/cp_password.cgi', 5555);
	my %_R							= ( 'IDT:13'					=> '',
										'USER_NAME'					=> '',
										'PASSWORD_NEW'				=> '',
										'PASSWORD_CONFIRM'			=> '',
										'APPLY'						=> '' );
	my $TargetListContentHidden;
	my $PostActivity				= ($cgi->param('apply')) ? 1 : 0;
	my $UserName					= $cgi->param('user_name');

	# Get the string resources.
	$StringResource->GetStrings(\%_R);
	
	# Safe defaults.
	$UserName = '' unless(defined($UserName));
	
	# If applying changes...
	if ($PostActivity)
	{
		# Validate the POST data...
		if (0 == ($Result = ValidatePostData($cgi)))
		{
			# Generate the distribution directive...
			unless (0 == ($Result = $GroupOperations->GenerateDirective($cgi)))
			{
				# Get the error string.
				$Status = $GroupOperations->GetErrorString($Result);
			}
		}
		else
		{
			# Get the error string.
			$Status = $StringResource->_S($Result);
		}
	}

	# Get the hidden target list content.
	$TargetListContentHidden = $GroupOperations->GetTargetListContentHidden();

	$html = <<HERE;
<form method="post" action="go_password.cgi" enctype="application/x-www-form-urlencoded">
  <table border="0" width="375" bgcolor="#EEEEEE">
    <tr>
      <td>
        <table border="0" width="100%">
          <tr>
            <td width="100%" align="left" colspan="2">&nbsp;</td>
          </tr>
          <tr>
            <td width="60%" align="left"><b>$_R{'USER_NAME'}</b></td>
            <td width="40%" align="left">
			<script>
			  if(mysel == 0)
			  {
	            document.write("<input name=user_name DISABLED size=20 maxlength=32 value='$UserName'>");
			  }
			  else
			  {
	            document.write("<input name=user_name size=20 maxlength=32 value='$UserName'>");
			  }
            </script>
			</td>
          </tr>
          <tr>
            <td width="60%" align="left"><b>$_R{'PASSWORD_NEW'}</b></td>
            <td width="40%" align="left">
			<script>
			  if(mysel == 0)
			  {
	            document.write("<input type=password DISABLED name=user_password_n size=20 maxlength=32>");
			  }
			  else
			  {
	            document.write("<input type=password name=user_password_n size=20 maxlength=32>");
			  }
            </script>
			</td>
          </tr>
          <tr>
            <td width="60%" align="left"><b>$_R{'PASSWORD_CONFIRM'}</b></td>
            <td width="40%" align="left">
			<script>
			  if(mysel == 0)
			  {
	            document.write("<input type=password DISABLED name=user_password_d size=20 maxlength=32>");
			  }
			  else
			  {
	            document.write("<input type=password name=user_password_d size=20 maxlength=32>");
			  }
            </script>
            </td>
          </tr>
          <tr>
            <td width="100%" colspan="2" align="left">&nbsp;</td>
          </tr>
          <tr>
            <td width="100%" colspan="2" align="right">
            <script>
			  if(mysel == 0)
			  {
	            document.write("<input type=submit class=clsButton DISABLED value='$_R{'APPLY'}' name=apply>");
			  }
			  else
			  {
	            document.write("<input type=submit class=clsButton value='$_R{'APPLY'}' name=apply>");
			  }
            </script>
            </td>
          </tr>
        </table>
      </td>
    </tr>
  </table>
  <br>
  $TargetListContentHidden
</form>
HERE

	# Render the page.
	$GroupOperations->RenderPageContent({ 'cgi' => $cgi, 'title' => $_R{'IDT:13'}, 'html' => $html, 'result' => $Result, 'status' => $Status, 'root-link' => 'go_password' });
}

###############################################################################
# The main body of the script.
###############################################################################

my $cgi = new CGI;

Main($cgi);
