#!/usr/bin/perl	-w

###############################################################################
#                                                                             #
# Filename: go_snmp.cgi                                                       #
#                                                                             #
# Description: Contains CGI script needed to enable group operations for SNMP #
#              configuration of element nodes.                                #
#                                                                             #
###############################################################################
###############################################################################
#                                                                             #
#  Copyright  2000 - 2001 Intel Corporation.                                 #
#  Intel Corporation All Rights Reserved.                                     #
#                                                                             #
#  The source code contained or described herein and all documents related to #
#  the source code ("Material") are owned by Intel Corporation or its         #
#  suppliers or licensors.  Title to the Material remains with Intel          #
#  Corporation or its suppliers and licensors.  The Material contains trade   #
#  secrets and proprietary and confidential information of Intel or its       #
#  suppliers and licensors.  The Material is protected by worldwide copyright #
#  and trade secret laws and treaty provisions.  No part of the Material may  #
#  be used, copied, reproduced, modified, published, uploaded, posted,        #
#  transmitted, distributed, or disclosed in any way without Intel's prior    #
#  express written permission.                                                #
#                                                                             #
#  No license under any patent, copyright, trade secret or other intellectual #
#  property right is granted to or conferred upon you by disclosure or        #
#  delivery of the Materials, either expressly, by implication, inducement,   #
#  estoppel or otherwise. Any license under such intellectual property rights #
#  must be express and approved by Intel in writing.                          #
#                                                                             #
###############################################################################

# Put the PERL library in the path.
use lib '/usr/ins/intel/gui/perllib';

use strict;
use LanguageResource;
use GroupOperations;
use CGI;

###############################################################################
# ValidatePostData
###############################################################################

sub ValidatePostData
{
	my ($cgi)	= @_;
	my $Return	= 0;

	# If the community data is being submitted...
	if ($cgi->param('apply'))
	{
        # Cache the input data.
		my $Community		= $cgi->param('community');
		my $CommunityLength	= length($Community);
	
		# Make sure that the data is valid.
		if ($Community !~ /\w{$CommunityLength}/)
		{
			# Set the status.
			$Return = 1000;
		}
	}

	return $Return;
}

###############################################################################
# Main
###############################################################################

sub Main
{
	my ($cgi)						= @_;
    my $html;
	my $Result;
	my $Status;
	my $PreferredLanguage			= (split(/,/, $ENV{'HTTP_ACCEPT_LANGUAGE'}))[0];
	my $StringResource				= new LanguageResource($PreferredLanguage, '/usr/ins/intel/gui/locale', [ 'go_snmp.lrc', 'common-em.lrc' ]);
	my $GroupOperations				= new GroupOperations($PreferredLanguage, 'go_snmp', 10, '/cgi-bin/emc/cp_snmp.cgi', 5555);
	my %_R							= ( 'IDT:10'			=> '',
										'SERVICE'			=> '',
										'CONTROLS'			=> '',
										'SNMP'				=> '',
										'START'				=> '',
										'STOP'				=> '',
										'RESTART'			=> '',
										'COMMUNITY_NAME'	=> '',
										'APPLY'				=> '' );
	my $TargetListContentHidden;
	my $PostActivity				= ($cgi->param('apply') || $cgi->param('control_start'  ) ||
															   $cgi->param('control_stop'   ) ||
															   $cgi->param('control_restart')) ? 1 : 0;
	my $Community					= $cgi->param('community'); 
	
	# Get the string resources.
	$StringResource->GetStrings(\%_R);
	
	# Safe defaults.
	$Community = '' unless(defined($Community));
	
	# If applying changes...
	if ($PostActivity)
	{
		# Validate the POST data...
		if (0 == ($Result = ValidatePostData($cgi)))
		{
			# Generate the distribution directive...
			unless (0 == ($Result = $GroupOperations->GenerateDirective($cgi)))
			{
				# Get the error string.
				$Status = $GroupOperations->GetErrorString($Result);
			}
		}
		else
		{
			# Get the error string.
			$Status = $StringResource->_S($Result);
		}
	}

	# Get the hidden target list content.
	$TargetListContentHidden = $GroupOperations->GetTargetListContentHidden();

	$html = <<HERE;
<form action="go_snmp.cgi" method="post" enctype="application/x-www-form-urlencoded">
  <table border="0" width="375" bgcolor="#EEEEEE">
    <tr>
      <td>
        &nbsp;
		  <table border="1" cellpadding="2" cellspacing="0" width="100%" bgcolor="#FFFFFF" bordercolor="#EEEEEE">
		  <tr>
			<td width="58%" bgcolor="#808080"><b><font color="#FFFFFF">$_R{'SERVICE'}</font></b></td>
			<td width="42%" bgcolor="#808080"><b><font color="#FFFFFF">$_R{'CONTROLS'}</font></b></td>
		  </tr>
		  <tr>
		    <td width="58%"><b>$_R{'SNMP'}</b></td>
		    <td width="42%" align="left" nowrap>
			<script>
			  if(mysel == 0)
			  {
	            document.write("<input type=submit class=clsButton DISABLED value='$_R{'START'}' name=control_start> ");
				document.write("<input type=submit class=clsButton DISABLED value='$_R{'STOP'}' name=control_stop> ");
				document.write("<input type=submit class=clsButton DISABLED value='$_R{'RESTART'}' name=control_restart>");
			  }
			  else
			  {
	            document.write("<input type=submit class=clsButton value='$_R{'START'}' name=control_start> ");
				document.write("<input type=submit class=clsButton value='$_R{'STOP'}' name=control_stop> ");
				document.write("<input type=submit class=clsButton value='$_R{'RESTART'}' name=control_restart>");
			  }
			</script>
		    </td>
    	  </tr>
		  </table>
		  <table border="0" width="100%">
			<tr>
			  <td width="100%" colspan="2">&nbsp;&nbsp;</td>
			</tr>
			<tr>
				<td width="50%"><font color="black"><b>$_R{'COMMUNITY_NAME'}</b></font></td>
				<td width="50%" align="right">
				<script>
			      if(mysel == 0)
			      {
	                document.write("<input type=text DISABLED name=community maxlength=16 value='$Community'>");
			      }
			      else
			      {	   
	                document.write("<input type=text name=community maxlength=16 value='$Community'>");
			      }
                  </script>
			    </td>
			</tr>
			<tr>
				<td width="100%" colspan="2">&nbsp;</td>
			</tr>
			<tr>
				<td width="100%" align="right" colspan="2">
			    <script>
			      if(mysel == 0)
			      {
	                document.write("<input type=submit class=clsButton DISABLED value='$_R{'APPLY'}' name=apply>");
			      }
			      else
			      {	   
				    document.write("<input type=submit class=clsButton value='$_R{'APPLY'}' name=apply>");
			      }
			    </script>				  
				</td>
			</tr>
		  </table>
	  </td>
	</tr>
  </table>
  <br>
  $TargetListContentHidden
</form>
HERE

	# Render the page.
	$GroupOperations->RenderPageContent({ 'cgi' => $cgi, 'title' => $_R{'IDT:10'}, 'html' => $html, 'result' => $Result, 'status' => $Status, 'root-link' => 'go_snmp' });
}

###############################################################################
# The main body of the script.
###############################################################################

my $cgi = new CGI;

Main($cgi);
