#!/usr/bin/perl	-w

###############################################################################
#                                                                             #
# Filename: go_swdownload.cgi                                                 #
#                                                                             #
# Description: Contains CGI script needed to enable group operations for      #
#              software updates on element nodes.                             #
#                                                                             #
###############################################################################
###############################################################################
#                                                                             #
#  Copyright  2000 - 2001 Intel Corporation.                                 #
#  Intel Corporation All Rights Reserved.                                     #
#                                                                             #
#  The source code contained or described herein and all documents related to #
#  the source code ("Material") are owned by Intel Corporation or its         #
#  suppliers or licensors.  Title to the Material remains with Intel          #
#  Corporation or its suppliers and licensors.  The Material contains trade   #
#  secrets and proprietary and confidential information of Intel or its       #
#  suppliers and licensors.  The Material is protected by worldwide copyright #
#  and trade secret laws and treaty provisions.  No part of the Material may  #
#  be used, copied, reproduced, modified, published, uploaded, posted,        #
#  transmitted, distributed, or disclosed in any way without Intel's prior    #
#  express written permission.                                                #
#                                                                             #
#  No license under any patent, copyright, trade secret or other intellectual #
#  property right is granted to or conferred upon you by disclosure or        #
#  delivery of the Materials, either expressly, by implication, inducement,   #
#  estoppel or otherwise. Any license under such intellectual property rights #
#  must be express and approved by Intel in writing.                          #
#                                                                             #
###############################################################################

# Put the PERL library in the path.
use lib '/usr/ins/intel/gui/perllib';

use strict;
use ProductInformation;
use LanguageResource;
use GroupOperations;
use SoftwareSupport;
use CGI;

###############################################################################
# Main
###############################################################################

sub Main
{
	my ($cgi)						= @_;
	my ($html0, $html1, $html2);
	my $Result;
	my $Status;
	my @Manifest;
	my $PreferredLanguage			= (split(/,/, $ENV{'HTTP_ACCEPT_LANGUAGE'}))[0];
	my $StringResource				= new LanguageResource($PreferredLanguage, '/usr/ins/intel/gui/locale', [ 'go_swdownload.lrc' ]);
	my $GroupOperations				= new GroupOperations($PreferredLanguage, 'go_swdownload', 21, '/cgi-bin/emc/cp_swupdate.cgi', 5555);
	my $ProductInfo	        		= new ProductInformation();
	my %_R							= ( 'IDT:21'						=> '',
										'TITLE_SOFTWARE_UPDATES'		=> '',
										'DOWNLOAD_MFG_PACKAGES'			=> '',
										'DOWNLOAD_LATEST_SOFTWARE'		=> '',
										'SOFTWARE_PACKAGES'				=> '',
										'VERSION_AVAILABLE'				=> '',
										'NO_UPDATES_AVAILABLE'			=> '',
										'DOWNLOAD_NON-MFG_PACKAGES'		=> '',
										'SPECIFY_PACKAGE_LOCATION'		=> '',
										'URL'							=> '',
										'DOWNLOAD'						=> '' );
	my $PostActivity				= ($cgi->param('download')    || 
									   $cgi->param('downloadrpm')) ? 1 : 0;
	my $RootLink					= $cgi->param('RL') || '';
	my $Location					= $cgi->param('location') if($cgi->param('downloadrpm'));
	my $SoftwareSupport;
	my $ManifestName;
	my $i;
    
	# Get the string resources.
	$StringResource->GetStrings(\%_R);
	
	# Get the product model and manifest.
	$ManifestName		= $ProductInfo->getManufacturerID() . "_" . $ProductInfo->getFamily() . "_" . $ProductInfo->getSeries();
	$ManifestName		= lc($ManifestName);
	
	# Get available update manifest from the manufacturer support site.
	$SoftwareSupport	= new SoftwareSupport($ManifestName, $ProductInfo->getVersion(), '/home/updates/');

	# Safe defaults.
	$Location = '' unless(defined($Location));
	
	# Make sure the hash arrays are empty.
	undef(@Manifest);
	
	# Get the manifest for the product.        
    $SoftwareSupport->GetOfficialProductManifest(\@Manifest);
	
	# Cache whatever manifest was available and reload it off disk.
	$SoftwareSupport->CacheProductManifest(1);
	
	# If there is a download request...
	if (($cgi->param('download')) ||
		($cgi->param('downloadrpm')))
	{
		# Overwrite the target list.
		$cgi->param(-name=>'targetnodes', -values=>["localhost:$ENV{'SERVER_ADDR'}:0"]);
	}

	# If applying changes...
	if ($PostActivity)
	{
		# Generate the distribution directive...
		unless (0 == ($Result = $GroupOperations->GenerateDirective($cgi)))
		{
			# Get the error string.
			$Status = $GroupOperations->GetErrorString($Result);
		}
	}
	
	$html0 = <<HERE;
  <table border="0" width="375" bgcolor="#EEEEEE">
    <tr>
      <td>
        <table border="0" width="367">
          <tr>
            <td width="359" valign="top" align="left" colspan="2"></td>
          </tr>
          <tr>
            <td width="359" valign="top" align="left" colspan="2"><b>$_R{'DOWNLOAD_MFG_PACKAGES'}</b></td>
          </tr>
          <tr>
            <td width="359" colspan="2">$_R{'DOWNLOAD_LATEST_SOFTWARE'}</td>
          </tr>
          <tr>
            <td width="359" colspan="2">
              <table border="0" cellpadding="0" cellspacing="0" width="355" bgcolor="#EEEEEE">
                <tr>
                  <td width="100%">
                    <table border="1" cellpadding="2" cellspacing="0" width="100%" bgcolor="#FFFFFF" bordercolor="#EEEEEE">
                      <tr>
                        <td width="40%" bgcolor="#808080"><b><font color="#FFFFFF">$_R{'SOFTWARE_PACKAGES'}</font></b></td>
                        <td width="20%" align="left" bgcolor="#808080"><b><font color="#FFFFFF">$_R{'VERSION_AVAILABLE'}</font></b></td>
                        <td width="20%" align="center" bgcolor="#808080">&nbsp;</td>
                      </tr>
HERE

	# Check to see if the manifest is available...
	if (defined(@Manifest))
	{
		# Loop through the packages...
		for $i (0 .. $#Manifest)
		{
			# Re-join the model numbers.
			my $Model = join(',', @{ $Manifest[$i]{'model'} });

			$html1 .= <<HERE;
					  <tr>
					  <form method=POST action=go_swdownload.cgi ENCTYPE=application/x-www-form-urlencoded>
						<td width=40%><b>$Manifest[$i]{'display-name'}</b></td>
						<td width=20% align=left nowrap>$Manifest[$i]{'version'}</td>
						<td width=20% align=center>
						  <input type=submit class=clsButton value='$_R{'DOWNLOAD'}' name=download>
						  <input type=hidden name=location value='$Manifest[$i]{'location'}'>
						  <input type=hidden name=filename value='$Manifest[$i]{'file-name'}'>
						  <input type=hidden name=model value='$Model'>
						  <input type=hidden name=RL value='$RootLink'>
						</td>
					  </form>
					  </tr>
HERE
		}
	}
	else
	{
		$html1 = "<tr><td width=40%><b>$_R{'NO_UPDATES_AVAILABLE'}</b></td><td width=20% align=left nowrap>&nbsp;</td><td width=20% align=center>&nbsp;</td></tr>";
	}

	$html2 = <<HERE;
                    </table>
                  </td>
                </tr>
              </table>
            </td>
          </tr>
          <tr>
            <td width="404" colspan="2">&nbsp;
              <table border="0" width="100%">
                <tr>
                  <td width="100%"><b>$_R{'DOWNLOAD_NON-MFG_PACKAGES'}</b></td>
                </tr>
                <tr>
                  <td width="100%">$_R{'SPECIFY_PACKAGE_LOCATION'}</td>
                </tr>
                <tr>
                <form method="POST" action="go_swdownload.cgi" ENCTYPE="application/x-www-form-urlencoded">
                  <td width="100%"><b>$_R{'URL'}&nbsp;</b>
				  <input class="text" type="text" name="location" size="31" maxlength="255" value="$Location">&nbsp; 
				  <input type="submit" class="clsButton" value="$_R{'DOWNLOAD'}" name="downloadrpm">
				  <input type=hidden name=model value='?'>
				  <input type="hidden" name="RL" value="$RootLink">
                  </td>
				</form>
                </tr>
              </table>
            </td>
          </tr>
	      <tr>
            <td width="404" colspan="2">&nbsp;</td>
          </tr>
        </table>
      </td>
    </tr>
  </table>
  <br>
HERE

	# Render the page.
	$GroupOperations->RenderPageContent({ 'cgi' => $cgi, 'title' => $_R{'IDT:21'}, 'html' => ($html0 . $html1 . $html2), 'result' => $Result, 'status' => $Status, 
										  'cookie-crumbs' => [ { 'name' => $_R{'TITLE_SOFTWARE_UPDATES'}, 'location' => 'go_swupdate.cgi'} ],
										  'root-link' => $RootLink, 'display-target-list' => 0 });
}

###############################################################################
# The main body of the script.
###############################################################################

my $cgi = new CGI;

Main($cgi);