#!/usr/bin/perl	-w

###############################################################################
#                                                                             #
# Filename: go_telnet.cgi                                                     #
#                                                                             #
# Description: Contains CGI script needed to enable group operations for      #
#              Telnet service control on element nodes.                       #
#                                                                             #
###############################################################################
###############################################################################
#                                                                             #
#  Copyright  2000 - 2001 Intel Corporation.                                 #
#  Intel Corporation All Rights Reserved.                                     #
#                                                                             #
#  The source code contained or described herein and all documents related to #
#  the source code ("Material") are owned by Intel Corporation or its         #
#  suppliers or licensors.  Title to the Material remains with Intel          #
#  Corporation or its suppliers and licensors.  The Material contains trade   #
#  secrets and proprietary and confidential information of Intel or its       #
#  suppliers and licensors.  The Material is protected by worldwide copyright #
#  and trade secret laws and treaty provisions.  No part of the Material may  #
#  be used, copied, reproduced, modified, published, uploaded, posted,        #
#  transmitted, distributed, or disclosed in any way without Intel's prior    #
#  express written permission.                                                #
#                                                                             #
#  No license under any patent, copyright, trade secret or other intellectual #
#  property right is granted to or conferred upon you by disclosure or        #
#  delivery of the Materials, either expressly, by implication, inducement,   #
#  estoppel or otherwise. Any license under such intellectual property rights #
#  must be express and approved by Intel in writing.                          #
#                                                                             #
###############################################################################

# Put the PERL library in the path.
use lib '/usr/ins/intel/gui/perllib';

use strict;
use LanguageResource;
use GroupOperations;
use CGI;

###############################################################################
# Main
###############################################################################

sub Main
{
	my ($cgi)						= @_;
    my $html;
	my $Result;
	my $Status;
	my $PreferredLanguage			= (split(/,/, $ENV{'HTTP_ACCEPT_LANGUAGE'}))[0];
	my $StringResource				= new LanguageResource($PreferredLanguage, '/usr/ins/intel/gui/locale', [ 'go_telnet.lrc', 'common-em.lrc' ]);
	my $GroupOperations				= new GroupOperations($PreferredLanguage, 'go_telnet', 12, '/cgi-bin/emc/cp_telnet.cgi', 5555);
	my %_R							= ( 'IDT:12'						=> '',
										'SERVICE'						=> '',
										'CONTROLS'						=> '',
										'TELNET'						=> '',
										'OPENSSH'						=> '',
										'START'							=> '',
										'STOP'							=> '' );
	my $TargetListContentHidden;
	my $PostActivity				= ($cgi->param('control_stop_telnet'   ) ||
									   $cgi->param('control_start_telnet'  ) ||
									   $cgi->param('control_stop_openssh'  ) ||
									   $cgi->param('control_start_openssh' )) ? 1 : 0;
	# Get the string resources.
	$StringResource->GetStrings(\%_R);
	
	# If applying changes...
	if ($PostActivity)
	{
		# Generate the distribution directive...
		unless (0 == ($Result = $GroupOperations->GenerateDirective($cgi)))
		{
			# Get the error string.
			$Status = $GroupOperations->GetErrorString($Result);
		}
	}

	# Get the hidden target list content.
	$TargetListContentHidden = $GroupOperations->GetTargetListContentHidden();

    $html = <<HERE;
<form METHOD="POST" action="go_telnet.cgi" ENCTYPE="application/x-www-form-urlencoded">
  <table border="0" width="375" bgcolor="#EEEEEE">
    <tr>
      <td>
        &nbsp;
<table border="0" cellpadding="0" cellspacing="0" width="100%" bgcolor="#EEEEEE">
  <tr> 
    <td width="100%"> 
      <table border="1" cellpadding="2" cellspacing="0" width="100%" bgcolor="#FFFFFF" bordercolor="#EEEEEE"> 
        <tr> 
          <td width="58%" bgcolor="#808080"><b><font color="#FFFFFF">$_R{'SERVICE'}</font></b></td> 
          <td width="42%" align="left" bgcolor="#808080"><font color="#FFFFFF"><b>$_R{'CONTROLS'}</b></font></td> 
        </tr> 
        <tr> 
          <td width="58%"><b>$_R{'TELNET'}</b></td> 
          <td width="42%" align="left" nowrap>
			<script>
			  if(mysel == 0)
			  {
	            document.write("<input type=submit class=clsButton DISABLED value='$_R{'START'}' name=control_start_telnet> ");
				document.write("<input type=submit class=clsButton DISABLED value='$_R{'STOP'}' name=control_stop_telnet>");
			  }
			  else
			  {
	            document.write("<input type=submit class=clsButton value='$_R{'START'}' name=control_start_telnet> ");
				document.write("<input type=submit class=clsButton value='$_R{'STOP'}' name=control_stop_telnet>");
			  }
			</script>
          </td> 
        </tr> 
      </table> 
	  <table border="0" width="100%"> 
	    <tr>
		  <td width="100%" colspan="2">&nbsp;</td>
		</tr>
	  </table> 
	  <table border="1" cellpadding="2" cellspacing="0" width="100%" bgcolor="#FFFFFF" bordercolor="#EEEEEE"> 
		<tr> 
          <td width="58%" bgcolor="#808080"><b><font color="#FFFFFF">$_R{'SERVICE'}</font></b></td> 
          <td width="42%" align="left" bgcolor="#808080"><font color="#FFFFFF"><b>$_R{'CONTROLS'}</b></font></td> 
        </tr> 
        <tr> 
          <td width="58%"><b>$_R{'OPENSSH'}</b></td> 
          <td width="42%" align="left" nowrap>
			<script>
			  if(mysel == 0)
			  {
	            document.write("<input type=submit class=clsButton DISABLED value='$_R{'START'}' name=control_start_openssh> ");
				document.write("<input type=submit class=clsButton DISABLED value='$_R{'STOP'}' name=control_stop_openssh>");
			  }
			  else
			  {
	            document.write("<input type=submit class=clsButton value='$_R{'START'}' name=control_start_openssh> ");
				document.write("<input type=submit class=clsButton value='$_R{'STOP'}' name=control_stop_openssh>");
			  }
			</script>
          </td> 
        </tr> 
      </table> 
    </td> 
  </tr> 
</table> 
        <table border="0" width="100%">
          <tr>
            <td width="100%">&nbsp;</td>
          </tr>
        </table>
      </td>
    </tr>
  </table>
  <br>
  $TargetListContentHidden
</form>
HERE

	# Render the page.
	$GroupOperations->RenderPageContent({ 'cgi' => $cgi, 'title' => $_R{'IDT:12'}, 'html' => $html, 'result' => $Result, 'status' => $Status, 'root-link' => 'go_telnet' });
}

###############################################################################
# The main body of the script.
###############################################################################

my $cgi = new CGI;

Main($cgi);
