#!/usr/bin/perl -w

###############################################################################
#                                                                             #
# Filename: cp_alerts.cgi                                                     #
#                                                                             #
# Description: Contains CGI script needed to enable group operations for      #
#              alert configuration of element nodes.                          #
#                                                                             #
###############################################################################
###############################################################################
#                                                                             #
#  Copyright  2000 - 2001 Intel Corporation.                                 #
#  Intel Corporation All Rights Reserved.                                     #
#                                                                             #
#  The source code contained or described herein and all documents related to #
#  the source code ("Material") are owned by Intel Corporation or its         #
#  suppliers or licensors.  Title to the Material remains with Intel          #
#  Corporation or its suppliers and licensors.  The Material contains trade   #
#  secrets and proprietary and confidential information of Intel or its       #
#  suppliers and licensors.  The Material is protected by worldwide copyright #
#  and trade secret laws and treaty provisions.  No part of the Material may  #
#  be used, copied, reproduced, modified, published, uploaded, posted,        #
#  transmitted, distributed, or disclosed in any way without Intel's prior    #
#  express written permission.                                                #
#                                                                             #
#  No license under any patent, copyright, trade secret or other intellectual #
#  property right is granted to or conferred upon you by disclosure or        #
#  delivery of the Materials, either expressly, by implication, inducement,   #
#  estoppel or otherwise. Any license under such intellectual property rights #
#  must be express and approved by Intel in writing.                          #
#                                                                             #
###############################################################################

# Put the PERL library in the path.
use lib '/usr/ins/intel/gui/perllib';

use strict;
use MonSettings;
use Usable;
use IO::File;
use CGI;

###############################################################################
# SetAlerts
###############################################################################

sub SetAlerts 
{
	my ($cgi) = @_;
	my ($Status, @ConfigFileContents, $Value, $MailList);
	my $Monitor								= new MonSettings();
	my ($DefaultDomain, $DefaultDomainIP)	= Usable::getDefaultDomain();
	my $CONFIGFILE							= '/etc/asm/asm.conf';

	# Open the configuration.
	if (0 == $Monitor->open())
	{
		my $Recipient;
        my $i = 0;

		# Get the mailing list.
		$MailList = $cgi->param('mailto');

		# Set the alert state.
		$Monitor->setAlertState('http', (($cgi->param('http')) ? 1 : 0));
		$Monitor->setAlertState('smtp', (($cgi->param('mail')) ? 1 : 0));
		$Monitor->setAlertState('ftp',  (($cgi->param('ftp' )) ? 1 : 0));

		# Clean out the e-mail addresses.
		$Monitor->deleteAllEmailAddresses();

		# Clean up the list.
		$MailList =~ s/\n/ /sg;
		$MailList =~ s/\,/ /g;
		$MailList =~ s/\;/ /g;
		$MailList =~ s/\cM//g;

		# Run through the recipients
		foreach $Recipient (split(/ /, $MailList))
		{
			# Add the e-mail address.
			$Monitor->addEmailAddress($Recipient) if('' ne $Recipient);
		}

		# Close the configuration.
		$Monitor->close();
		
		# Get the configuration informaiton.
		@ConfigFileContents = ConfigFileRead($CONFIGFILE);

		# Set Hardware Alerts #################################################
		$Value = ($cgi->param('cpu')) ? 'Email' : '';
		SetConfigDataMutli('PF_CPU_UTILIZATION_ACTIONS', 'Email', $Value, \@ConfigFileContents); 

		$Value = ($cgi->param('memory')) ? 'Email' : '';
		SetConfigDataMutli('PF_MEMORY_UTILIZATION_ACTIONS', 'Email', $Value, \@ConfigFileContents); 

		$Value = ($cgi->param('drive_warning')) ? 'Email' : '';
		SetConfigDataMutli('DISK_LOWER_ACTIONS', 'Email', $Value, \@ConfigFileContents); 

		$Value = ($cgi->param('drive_critical')) ? 'Email' : '';
		SetConfigDataMutli('DISK_LOWERCRITICAL_ACTIONS', 'Email', $Value, \@ConfigFileContents); 

		$Value = ($cgi->param('temperatures')) ? 'Email' : '';
		SetConfigDataMutli('TEMP_UPPER_ACTIONS', 'Email', $Value, \@ConfigFileContents); 
		SetConfigDataMutli('TEMP_LOWER_ACTIONS', 'Email', $Value, \@ConfigFileContents); 
		SetConfigDataMutli('PF_TEMP_UPPER_ACTIONS', 'Email', $Value, \@ConfigFileContents); 
		SetConfigDataMutli('PF_TEMP_LOWER_ACTIONS', 'Email', $Value, \@ConfigFileContents); 

		$Value = ($cgi->param('fans')) ? 'Email' : '';
		SetConfigDataMutli('FAN_LOWER_ACTIONS', 'Email', $Value, \@ConfigFileContents); 

		$Value = ($cgi->param('voltages')) ? 'Email' : '';
		SetConfigDataMutli('VOLT_UPPER_ACTIONS', 'Email', $Value, \@ConfigFileContents); 
		SetConfigDataMutli('VOLT_LOWER_ACTIONS', 'Email', $Value, \@ConfigFileContents); 
		SetConfigDataMutli('PF_VOLT_UPPER_ACTIONS', 'Email', $Value, \@ConfigFileContents); 
		SetConfigDataMutli('PF_VOLT_LOWER_ACTIONS', 'Email', $Value, \@ConfigFileContents);

		# Run through the recipients
		foreach $Recipient (split(/ /, $MailList))
		{
			# If there is a recipient...
			if ('' ne $Recipient)
			{
				# Add the e-mail address.
				SetConfigData('EMAIL_ALERTS_ADDRESS'.($i + 1), $Recipient, \@ConfigFileContents);
				last if(4 < ++$i);
			}
		}

		# Clean out the other slots...
		for ($i; 3 > $i; ++$i)
		{
			# Empty the slot.
			SetConfigData('EMAIL_ALERTS_ADDRESS'.($i + 1), '', \@ConfigFileContents);
		}

		# Write out the file.
		ConfigFileWrite($CONFIGFILE, @ConfigFileContents);

		# Set the status.
		$Status = 0;
	}
	else
	{
		$Status = 1;
	}

	return $Status;
}

###############################################################################
# ConfigFileRead
###############################################################################

sub ConfigFileRead
{
	my ($ConfigFileName) = @_;
	my $ConfigFile = new IO::File("<$ConfigFileName");
	my @ConfigFileContents;

	# If the file is not open...
	unless ($ConfigFile)
	{
		&error("The file $ConfigFileName can not be opened.");
		exit(0);
	}

	# Snag the contents.
	@ConfigFileContents = <$ConfigFile>;
	
	# Close the file.
	$ConfigFile->close;

	return @ConfigFileContents;
}

###############################################################################
# ConfigFileWrite
###############################################################################

sub ConfigFileWrite
{
	my ($ConfigFileName, @ConfigFileContents) = @_;
    my $ConfigFile = new IO::File(">$ConfigFileName");

	# If the file is not open...
	unless ($ConfigFile)
	{
		&error("The file $ConfigFileName can not be opened.");
		exit(0);
	}
	else
	{
		print $ConfigFile @ConfigFileContents;
	}
	
	# Close the file.
	$ConfigFile->close;

	return 1;
}

###############################################################################
# GetConfigData
###############################################################################

sub GetConfigData
{
	my ($Label, @ConfigFileContents) = @_;
    my $Value;
	my $Line;

	# Loop through all of the lines in the file.
	foreach $Line (@ConfigFileContents)
	{
		# Continue if there is a comment.
		next if $Line =~ /^#/;

		# Continue if the label is not found.
		next unless $Line =~ /$Label/;
		
		# Parse out the value.
		$Value = $Line;
		$Value =~ s/.*$Label=(.*)$/$1/;
		chomp $Value;
		
		return $Value;
	}

	return '';
}

###############################################################################
# SetConfigData
###############################################################################

sub SetConfigData
{
	my ($Label, $Value, $ConfigFileContents) = @_;
	my $Line;

	# Loop through the file and rewrite it to disk.
    foreach $Line (@$ConfigFileContents)
	{
		# Check to see if the line can be rerendered to file...
		unless ($Line !~ /$Label/ or $Line =~ /^#/)
		{
			$Line = "$Label=$Value\n";
		}
	}
}

###############################################################################
# SetConfigDataMutli
###############################################################################

sub SetConfigDataMutli
{
	my ($Label, $Tag, $Value, $ConfigFileContents) = @_;
	my $Line;
	
	# Loop through the file and rewrite it to disk.
    foreach $Line (@$ConfigFileContents)
	{
		# Check to see if the line can be rerendered to file...
		unless ($Line !~ /$Label/ or $Line =~ /^#/)
		{
			# Cache the current value.
			my $OldValue = $Line;
			$OldValue =~ s/.*$Label=(.*)$/$1/;
			$OldValue = ($OldValue =~ /Email/) ? 1 : 0;

			# Check to see if the line needs to be modified...
			unless ((( $OldValue) && ($Value eq $Tag)) ||
					((!$OldValue) && ($Value eq '')))
			{
				# If adding 'Email' to the line...
				if ($Value eq $Tag)
				{
					# Add the tag out of the line.
					chomp $Line;
					$Line .= ",$Tag\n";
				}
				else
				{
					# Pull the tag out of the line.
					$Line =~ s/(,)?$Tag(,)?//s;
				}
			}
		}
	}
}

###############################################################################
# Main
###############################################################################

sub Main
{
	my ($cgi)	= @_;
	my $Status;

	# Get the status.
	$Status = ($cgi->param('apply')) ? SetAlerts($cgi) : 1;
	
	print($cgi->header());
	print($Status);
}

###############################################################################
# The main body of the script.
###############################################################################

my $cgi = new CGI;

Main($cgi);
