#!/usr/bin/perl -w

###############################################################################
#                                                                             #
# Filename: cp_mail.cgi                                                       #
#                                                                             #
# Description: Contains CGI script needed to enable group operations for mail #
#              configuration of element nodes.                                #
#                                                                             #
###############################################################################
###############################################################################
#                                                                             #
#  Copyright  2000 - 2001 Intel Corporation.                                 #
#  Intel Corporation All Rights Reserved.                                     #
#                                                                             #
#  The source code contained or described herein and all documents related to #
#  the source code ("Material") are owned by Intel Corporation or its         #
#  suppliers or licensors.  Title to the Material remains with Intel          #
#  Corporation or its suppliers and licensors.  The Material contains trade   #
#  secrets and proprietary and confidential information of Intel or its       #
#  suppliers and licensors.  The Material is protected by worldwide copyright #
#  and trade secret laws and treaty provisions.  No part of the Material may  #
#  be used, copied, reproduced, modified, published, uploaded, posted,        #
#  transmitted, distributed, or disclosed in any way without Intel's prior    #
#  express written permission.                                                #
#                                                                             #
#  No license under any patent, copyright, trade secret or other intellectual #
#  property right is granted to or conferred upon you by disclosure or        #
#  delivery of the Materials, either expressly, by implication, inducement,   #
#  estoppel or otherwise. Any license under such intellectual property rights #
#  must be express and approved by Intel in writing.                          #
#                                                                             #
###############################################################################

# Put the PERL library in the path.
use lib '/usr/ins/intel/gui/perllib';

use strict;
use MonSettings;
use Safe;
use CGI;

###############################################################################
# GetMailConfiguration
###############################################################################

sub GetMailConfiguration 
{
	my $var_ref = $_[0];
	my ($smart_relay_host, $unresolvable_domains, @array);

	open (SENDMAIL_MC, "/etc/mail/sendmail.mc");
	# flock(SENDMAIL_MC, 2);
	while (<SENDMAIL_MC>) 
	{
		if (/`SMART_HOST', `(\S+)(?=')/ ) 
		{ 
			$smart_relay_host= $1; 
		}
		elsif (/^FEATURE.*`accept_unresolvable_domains'/ ) 
		{ 
			$unresolvable_domains = ''; 
		}
		elsif (/^dnl.*FEATURE.*`accept_unresolvable_domains'/ ) 
		{ 
			$unresolvable_domains = 'checked'; 
		}
		elsif (/^FEATURE.*`dnsbl'/ ) 
		{ 
			$var_ref->{rbl} = 'checked'; 
		}
		elsif (/^dnl.*FEATURE.*`dnsbl'/ ) 
		{ 
			$var_ref->{rbl} = ''; 
		}
		elsif (/^define.*`LOCAL_MAILER_MAX', `(\S+)(?=')/ ) 
		{ 
			$var_ref->{max_incoming_message_size} = $1/1048576; 
		}
		elsif (/^define.*`SMTP_MAILER_MAX', `(\S+)(?=')/ ) 
		{ 
			$var_ref->{max_outgoing_message_size} = $1/1048576; 
		}
		
		push(@array, $_);
	}
	close(SENDMAIL_MC);

	return ($smart_relay_host, $unresolvable_domains, \@array);
}

###############################################################################
# GetAccessConfiguration
###############################################################################
 
sub GetAccessConfiguration 
{
	my $vars_ref = $_[0];
	my ($rejects, $access_db);
	
	open (access_db, "/etc/mail/access");
	# flock(access_db, 2);

	while (<access_db>) 
	{
		unless ((/^#/) or
				(/^\s+$/)) 
		{
			if (/REJECT/) 
			{ 
				$_ =~ s/\s*REJECT\s*/\n/;
				$rejects .= $_;
			}
			else 
			{
				$access_db .= $_;
			}
		}
	}

	return ($rejects, $access_db);
	close(access_db);
}         

###############################################################################
# SetAccessConfiguration
###############################################################################

sub SetAccessConfiguration 
{
	my $vars_ref = shift;
	my $reject_list = shift;
	my $access_db = shift;

	# Replace the newlines and semicolons with a space.
	$reject_list =~ s/[\n;,]/ /g;
	
	# Add a space at the end of the string
	$reject_list = "$reject_list ";	
	
	# Shrink each occurance of multiple spaces into 1
	$reject_list =~ s/\s+/ /g;	
	
	# Add the word REJECT and a new line char after every word.
	$reject_list =~ s/(\S+ )/$1\t\tREJECT\n/g;
	
	# If reject list is empty then clean it up
	$reject_list = "" unless $reject_list =~ /\w/g;
	
	# Remove blank lines from $access_db
	$access_db =~ s/^\n//mg;	
	
	open (access_db, ">/etc/mail/access");
	flock(access_db, 2);
	print access_db 
"# Check the /usr/lib/sendmail-cf/README file for a description
# of the format of this file. (search for access_db in that file)
# The /usr/lib/sendmail-cf/README is part of the sendmail-doc
# package.
#
$reject_list#
#
$access_db";
	
	close(access_db);
	
	# Run makemap to create a hash table from the plain text access file.
	Safe::system("makemap hash /etc/mail/access.db < /etc/mail/access");
} 

###############################################################################
# SetHostNameConfiguration
###############################################################################

sub SetHostNameConfiguration 
{
	my $vars_ref = $_[0];
	my $host_aliases = $_[1];

	# Replace the newlines and semicolons with a space.
	$host_aliases =~ s/[\n;,]/ /g; 
	
	# Add a space at the end of the string
	$host_aliases = "$host_aliases ";
	
	# Shrink each occurance of multiple spaces into 1
	$host_aliases =~ s/\s+/ /g;
	
	# Add a new line char after every word.
	$host_aliases =~ s/(\S+) /$1\n/g;

	open (local_host_names, ">/etc/mail/local-host-names");
	flock(local_host_names, 2);
	print local_host_names 
"# local-host-names - include all aliases for your machine here.
# Entries from obsoleted /etc/sendmail.cw
#
";

	print local_host_names $host_aliases;
	close(local_host_names);
}

###############################################################################
# SetRelayConfiguration
###############################################################################

sub SetRelayConfiguration
{
	my $vars_ref = $_[0];
	my $relay_domains = $_[1];

	# Replace the newlines and semicolons with a space.
	$relay_domains =~ s/[\n;,]/ /g; 
	
	# Add a space at the end of the string
	$relay_domains = "$relay_domains ";
	
	# Shrink each occurance of multiple spaces into 1
	$relay_domains =~ s/\s+/ /g;
	
	# Add a new line char after every word.
	$relay_domains =~ s/(\S+) /$1\n/g;

	open (relay_domains, ">/etc/mail/relay-domains");
	flock(relay_domains, 2);
	print relay_domains $relay_domains;
	close(relay_domains);
}

###############################################################################
# SetMailConfiguration
#
# Description: Gets values for different Variables from the web form and puts 
#              them in the sendmail.mc file.
###############################################################################

sub SetMailConfiguration 
{
	my $vars_ref = $_[0];
	my $array_ref = $vars_ref->{arrayref};

	foreach (@$array_ref)  
	{
		if (/`SMART_HOST'/) 
		{ 
			$_ = "define(`SMART_HOST\', `$vars_ref->{smart_relay_host}\')\n" 
		}
        elsif (/`accept_unresolvable_domains'/) 
		{
			unless ($vars_ref->{unresolvable_domains}) 
			{
				$_ = "FEATURE(`accept_unresolvable_domains\')\n";
			} 
			else 
			{
				$_ = "dnl FEATURE(`accept_unresolvable_domains\')\n";
			}
        }
        elsif (/`dnsbl'/ ) 
		{
			if ($vars_ref->{q}->param('rbl')) 
			{
				$_ = "FEATURE(`dnsbl\')\n";
			}
			else 
			{
				$_ = "dnl FEATURE(`dnsbl\')\n";
			}
        }
        elsif (/`LOCAL_MAILER_MAX'/ ) 
		{
			$_ = "define(`LOCAL_MAILER_MAX\', `" . int($vars_ref->{max_incoming_message_size} * 1048576) . "\')\n"
        }
        elsif (/`SMTP_MAILER_MAX'/ ) 
		{
			$_ = "define(`SMTP_MAILER_MAX\', `" . int($vars_ref->{max_outgoing_message_size} * 1048576) . "\')\n"
        }
	}
 
	open (SENDMAIL_MC_NEW, ">/etc/mail/sendmail.mc.tmp");
	# flock(SENDMAIL_MC_NEW, 2);
	print SENDMAIL_MC_NEW @$array_ref;
	close(SENDMAIL_MC_NEW);
	
	rename("/etc/mail/sendmail.mc", "/etc/mail/sendmail.mc.orig");                      # Move sendmail.mc to sendmail.mc.orig
	rename("/etc/mail/sendmail.mc.tmp", "/etc/mail/sendmail.mc");                       # Move sendmail.mc.tmp to sendmail.mc 
} 

###############################################################################
# SetPostMasterAddress
###############################################################################

sub SetPostMasterAddress
{
	my $PostMasterAddress = shift;

	# Open the alias file in read/write mode.
	my $Alias = new Unix::AliasFile("/etc/aliases");
	
	# Set postmaster alias.
	$Alias->alias("postmaster", $PostMasterAddress);
	
	# Save changes.
	$Alias->commit();
	
	# Generate the /etc/aliases.db file from /etc/aliases
	Safe::system("/usr/bin/newaliases");
}

###############################################################################
# Main
###############################################################################

sub Main
{
	my ($cgi)		= @_;
	my $Status		= 0;
	my %Variables;

	$Variables{q} = $cgi;
	$Variables{q}->import_names('Mail');
	
	# If toggling the daemon state...
	if ((defined($cgi->param('control_start'))) ||
		(defined($cgi->param('control_stop'))))
	{
		my $Monitor = new MonSettings();
		
		# If the configuration can be opened...
		if (0 == $Monitor->open())
		{
			# Set the monitor state.
			$Monitor->setMonitorState('smtp', (($cgi->param('control_start')) ? 1 : 0));

			# Close the configuration.
			$Monitor->close();
		}
	}
	
	SWITCH :
	{
		# Control the services...
		if ($cgi->param('control_start'   )) { Safe::system("/etc/rc.d/init.d/sendmail start   > /dev/null 2>&1; /sbin/chkconfig --add sendmail"); last SWITCH; }
		if ($cgi->param('control_stop'    )) { Safe::system("/etc/rc.d/init.d/sendmail stop    > /dev/null 2>&1; /sbin/chkconfig --del sendmail"); last SWITCH; }
		if ($cgi->param('control_restart' )) { Safe::system("/etc/rc.d/init.d/sendmail restart > /dev/null 2>&1; /sbin/chkconfig --add sendmail"); last SWITCH; }
		if ($cgi->param('apply'          )) 
		{ 
			$| = 1;

			($Variables{smart_relay_host}, $Variables{unresolvable_domains}, $Variables{arrayref}) = GetMailConfiguration(\%Variables);
			$Variables{smart_relay_host}			= $Mail::smart_relay_host;
			$Variables{max_incoming_message_size}	= int($Mail::max_incoming_message_size);
			$Variables{max_outgoing_message_size}	= int($Mail::max_outgoing_message_size);
			$Variables{host_aliases} 				= $Mail::host_aliases;
			$Variables{relay_domains}				= $Mail::relay_domains;
			$Variables{rejects}						= $Mail::reject_list;
			$Variables{unresolvable_domains}		= ($Mail::unresolvable_domains) ? 'checked' : '';
			$Variables{rbl}							= ($Mail::rbl) ? 'checked' : '';
				
			# Get rid of control characters.
			$Variables{host_aliases}	=~ s/\n/;/sg;
			$Variables{host_aliases}	=~ s/\,/;/g;
			$Variables{host_aliases}	=~ s/\cM//g;
			$Variables{relay_domains}	=~ s/\n/;/sg;
			$Variables{relay_domains}	=~ s/\,/;/g;
			$Variables{relay_domains}	=~ s/\cM//g;
			$Variables{reject_list}		=~ s/\n/;/sg;
			$Variables{reject_list}		=~ s/\,/;/g;
			$Variables{reject_list}		=~ s/\cM//g;
			
			# Write the /etc/mail/sendmail.mc file    (Works)
			SetMailConfiguration(\%Variables);						
	
			($Variables{rejects_tmp}, $Variables{access_db}) = GetAccessConfiguration(\%Variables);
	
			# Write the /etc/mail/access file.
			SetAccessConfiguration(\%Variables, $Mail::reject_list, $Variables{access_db});
	
			# Write the /etc/mail/local-host-names file.
			SetHostNameConfiguration(\%Variables, $Mail::host_aliases);
	
			# Write the /etc/mail/relay-domains file
			SetRelayConfiguration(\%Variables, $Mail::relay_domains);
	
			#Set the postmaster alias.
			SetPostMasterAddress($Mail::post_master_address);

			# Get the current status.
			$Variables{sendmail_status} = (Safe::system("/etc/rc.d/init.d/sendmail status > /dev/null 2>&1")) ? 'Stopped' : 'Running';
	
			# Update sendmail if necessary...
			if ($Variables{sendmail_status} eq 'Running') 
			{
				my $return_value = Safe::system("/usr/bin/m4 /etc/mail/sendmail.mc > /etc/sendmail.cf; /etc/rc.d/init.d/sendmail restart > /dev/null 2>&1");
			} 
			elsif ($Variables{sendmail_status} eq 'Stopped') 
			{ 
				my $return_value = Safe::system("/usr/bin/m4 /etc/mail/sendmail.mc > /etc/sendmail.cf;");
			}

			last SWITCH;
		}
        
		$Status = 1;
	}
	
	print($cgi->header());
	print($Status);
}

###############################################################################
# The main body of the script.
###############################################################################

my $cgi = new CGI;

Main($cgi);
