#!/usr/bin/perl -w

###############################################################################
#                                                                             #
# Filename: cp_network.cgi                                                    #
#                                                                             #
# Description: Contains CGI script needed to enable group operations for      #
#              network configuration of element nodes.                        #
#                                                                             #
###############################################################################
###############################################################################
#                                                                             #
#  Copyright  2000 - 2001 Intel Corporation.                                 #
#  Intel Corporation All Rights Reserved.                                     #
#                                                                             #
#  The source code contained or described herein and all documents related to #
#  the source code ("Material") are owned by Intel Corporation or its         #
#  suppliers or licensors.  Title to the Material remains with Intel          #
#  Corporation or its suppliers and licensors.  The Material contains trade   #
#  secrets and proprietary and confidential information of Intel or its       #
#  suppliers and licensors.  The Material is protected by worldwide copyright #
#  and trade secret laws and treaty provisions.  No part of the Material may  #
#  be used, copied, reproduced, modified, published, uploaded, posted,        #
#  transmitted, distributed, or disclosed in any way without Intel's prior    #
#  express written permission.                                                #
#                                                                             #
#  No license under any patent, copyright, trade secret or other intellectual #
#  property right is granted to or conferred upon you by disclosure or        #
#  delivery of the Materials, either expressly, by implication, inducement,   #
#  estoppel or otherwise. Any license under such intellectual property rights #
#  must be express and approved by Intel in writing.                          #
#                                                                             #
###############################################################################

# Put the PERL library in the path.
use lib '/usr/ins/intel/gui/perllib';

use strict;
use VirtualServ;
use Safe;
use CGI;

###############################################################################
# ValidatePostData
###############################################################################

sub ValidatePostData
{
	my ($cgi)					= @_;
	my $Status;
	my $AddressDNSPrimary		= $cgi->param('dnsa_p');
	my $AddressDNSSecondary		= $cgi->param('dnsa_s');
	my $AddressDNSTertiary		= $cgi->param('dnsa_t');
	my $AddressDefaultGateway	= $cgi->param('dga');
	my @OctetsPrimary			= split(/\./, $AddressDNSPrimary);
	my @OctetsSecondary			= split(/\./, $AddressDNSSecondary);
	my @OctetsTertiary			= split(/\./, $AddressDNSTertiary);
	my @OctetsDefaultGateway	= split(/\./, $AddressDefaultGateway);

	# Default values.
	$AddressDNSPrimary		= '' unless(defined($AddressDNSPrimary));
	$AddressDNSSecondary	= '' unless(defined($AddressDNSSecondary));
	$AddressDNSTertiary		= '' unless(defined($AddressDNSTertiary));
	$AddressDefaultGateway	= '' unless(defined($AddressDefaultGateway));
	
	# Validate the data.
	if (($AddressDNSPrimary ne '') &&
		(($AddressDNSPrimary !~ /^\d{1,3}\.\d{1,3}\.\d{1,3}\.\d{1,3}$/) ||
		 (255 < $OctetsPrimary[0]) ||
		 (255 < $OctetsPrimary[1]) ||
		 (255 < $OctetsPrimary[2]) ||
		 (255 < $OctetsPrimary[3])))
	{
		# Set the status.
		$Status = 1;
	}
	elsif (($AddressDNSSecondary ne '') &&
		   (($AddressDNSSecondary !~ /^\d{1,3}\.\d{1,3}\.\d{1,3}\.\d{1,3}$/) ||
			(255 < $OctetsSecondary[0]) ||
			(255 < $OctetsSecondary[1]) ||
			(255 < $OctetsSecondary[2]) ||
			(255 < $OctetsSecondary[3])))
	{
		# Set the status.
		$Status = 1;
	}
	elsif (($AddressDNSTertiary ne '') &&
		   (($AddressDNSTertiary !~ /^\d{1,3}\.\d{1,3}\.\d{1,3}\.\d{1,3}$/) ||
			(255 < $OctetsTertiary[0]) ||
			(255 < $OctetsTertiary[1]) ||
			(255 < $OctetsTertiary[2]) ||
			(255 < $OctetsTertiary[3])))
	{
		# Set the status.
		$Status = 1;
	}
	elsif (($AddressDefaultGateway) &&
		   (($AddressDefaultGateway !~ /^\d{1,3}\.\d{1,3}\.\d{1,3}\.\d{1,3}$/) ||
			(255 < $OctetsDefaultGateway[0]) ||
			(255 < $OctetsDefaultGateway[1]) ||
			(255 < $OctetsDefaultGateway[2]) ||
			(255 < $OctetsDefaultGateway[3])))
	{
		# Set the status.
		$Status = 1;
	}
	elsif (((length($AddressDNSPrimary)) && 
			(length($AddressDNSSecondary)) && 
			($AddressDNSPrimary eq $AddressDNSSecondary)) ||
		   ((length($AddressDNSPrimary)) && 
			(length($AddressDNSTertiary)) && 
			($AddressDNSPrimary eq $AddressDNSTertiary)) ||
		   ((length($AddressDNSSecondary)) && 
			(length($AddressDNSTertiary)) && 
			($AddressDNSSecondary eq $AddressDNSTertiary)))
	{
		# Set the status.
		$Status = 1;
	}
	else
	{
		$Status = 0;
	}

	return $Status;
}

###############################################################################
# Main
###############################################################################

sub Main
{
	my ($cgi)	= @_;
	my $Status;

	if ($cgi->param('apply'))
	{
		# Validate the input data.
		if (0 == ($Status = ValidatePostData($cgi)))
		{
			my (%DNS, $DNS, $DNSOld);
			my $ConfigurationFile	= '/etc/sysconfig/network';
			my $DefaultGateway		= $cgi->param('dga'); 
			my @Lines;
			my $Item;
			my $GatewaySet			= 0;
			my $GatewayDeviceSet	= 0;
			
			# Add the DNS information to the hash.
			push(@{$DNS->{'name'}}, $cgi->param('dnsa_p'), $cgi->param('dnsa_s'), $cgi->param('dnsa_t'));
			
			# Cache the old DNS.
			$DNSOld = VirtualServ::list_dns();
			
			# Cache the domain...
			push(@{$DNS->{'domain'}}, $DNSOld->{'domain'}->[0]);

			# Save the configuration.
			VirtualServ::save_dns($DNS);

			# If there is a route to add...
			if ($DefaultGateway)
			{
                my @Configuration	= split(/\n/, Safe::backtick("/sbin/route -n"));
				my $Line;
				
				# Look for the default gateway...
				foreach $Line (@Configuration)
				{
					# If the default gateway already exists...
					if ($Line =~ /^0.0.0.0\s+?$DefaultGateway/)
					{
						$DefaultGateway = '1';
						last;
					}
				}
					   
				# If the gateway does not exist...
				if ('1' ne $DefaultGateway)
				{
					# Try to add the new route.
					unless (Safe::system('/sbin/route', 'add', 'default', 'gw', $cgi->param('dga')))
					{
						# Delete it if successfull.
						Safe::system('/sbin/route', 'del', 'default');
					}
					else
					{
						# Save the old configuration.
						VirtualServ::save_dns($DNSOld);

						$Status = 1;
					}
				}
			}

			# If things are still running fine...
			if (( 0  == $Status) &&
				('1' ne $DefaultGateway))
			{
				# Read in the file.
				open(NETWORK, $ConfigurationFile);
				while(<NETWORK>) 
				{
					push(@Lines, $_);
				}
				close(NETWORK);

				# Write out the new configuration.
				open(NETWORK, ">$ConfigurationFile");
				flock(NETWORK, 2);
				foreach $Item (@Lines) 
				{
					if ($Item =~ /^GATEWAY=/)
					{
						print NETWORK "GATEWAY=\"" . $cgi->param('dga') . "\"\n";
						$GatewaySet = 1;
					}
					elsif ($Item =~ /^GATEWAYDEV=/)
					{
						print NETWORK "GATEWAYDEV=\"eth0\"\n";
						$GatewayDeviceSet = 1;
					}
					else 
					{ 
						print NETWORK $Item; 
					}
				}

				# If the configuration file did not have the gateway in it...
				if (0 == $GatewaySet)
				{
					print NETWORK "GATEWAY=\"" . $cgi->param('dga') . "\"\n";
				}

				# If the configuration file did not have the gateway device in it...
				if (0 == $GatewayDeviceSet)
				{
					print NETWORK "GATEWAYDEV=\"eth0\"\n";
				}

				close(NETWORK);
				
				# Update the system.
				Safe::system('/sbin/route', 'del', 'default');
				
				# If there is a route to add...
				if ($cgi->param('dga'))
				{
					Safe::system('/sbin/route', 'add', 'default', 'gw', $cgi->param('dga'));
				}
			}
		}
	}
	else
	{
		$Status = 1;
	}

	print($cgi->header());
	print($Status);
}

###############################################################################
# The main body of the script.
###############################################################################

my $cgi = new CGI;

Main($cgi);
