#!/usr/bin/perl	-w

###############################################################################
#                                                                             #
# Filename: cp_swupdate.cgi                                                   #
#                                                                             #
# Description: Contains CGI script needed to enable group operations for      #
#              software updates on element nodes.                             #
#                                                                             #
###############################################################################
###############################################################################
#                                                                             #
#  Copyright  2000 - 2001 Intel Corporation.                                 #
#  Intel Corporation All Rights Reserved.                                     #
#                                                                             #
#  The source code contained or described herein and all documents related to #
#  the source code ("Material") are owned by Intel Corporation or its         #
#  suppliers or licensors.  Title to the Material remains with Intel          #
#  Corporation or its suppliers and licensors.  The Material contains trade   #
#  secrets and proprietary and confidential information of Intel or its       #
#  suppliers and licensors.  The Material is protected by worldwide copyright #
#  and trade secret laws and treaty provisions.  No part of the Material may  #
#  be used, copied, reproduced, modified, published, uploaded, posted,        #
#  transmitted, distributed, or disclosed in any way without Intel's prior    #
#  express written permission.                                                #
#                                                                             #
#  No license under any patent, copyright, trade secret or other intellectual #
#  property right is granted to or conferred upon you by disclosure or        #
#  delivery of the Materials, either expressly, by implication, inducement,   #
#  estoppel or otherwise. Any license under such intellectual property rights #
#  must be express and approved by Intel in writing.                          #
#                                                                             #
###############################################################################

# Put the PERL library in the path.
use lib '/usr/ins/intel/gui/perllib';

use strict;
use Safe;
use SoftwareSupport;
use URI::URL;
use CGI;
use ProductInformation;

###############################################################################
# InstallRPM
###############################################################################
 
sub InstallRPM
{
	my ($Location, $FileName, $RPMModel) = @_;
	my $RPMName			= $FileName ;
	my $ProductInfo		= new ProductInformation();
	my $ProductModel	= $ProductInfo->getModel();		
	my $ProductMatch	= 0;
	my $Model;
	my $Return			= 0;
   
   # No matches so far.
	$ProductMatch = 0;

	# Run through the acceptable models...
	foreach $Model (split(/,/, $RPMModel))
	{
		# Look for a matching model number...
		if (($Model eq $ProductModel) || 
			($Model eq '*')) 
		{
			# A match is found.
			$ProductMatch = 1;
			last;
		}			    
	}
	
	if ($ProductMatch)
	{
	   $RPMName	=~ s/^(.*)-.*$/$1/;
	   $RPMName	=~ s/^(.*)-\d.\d/$1/;
	   
	   my $AvailableVersion;
	   my $InstalledVersion	= Safe::backtick("/bin/rpm -q --queryformat %{VERSION}-%{RELEASE} $RPMName");
	   my @Arguments;
	   
	   # Get the file name.
	   $AvailableVersion = $FileName;
	   $AvailableVersion =~  s/^.*?-(.*?)(\.i386)?\.rpm/$1/;
	   
	
	   my $rpm_forkflag = 0;
	   my $r2 = $FileName;
	
	   if ($r2 =~ /Apache-admin|apache-admin/)
	   {
		# Matched with special package.  Set up a special case with different RPM upgrade
		# activity from other packages.
	
		$rpm_forkflag = 1;
	   }
	
	   if ($InstalledVersion eq $AvailableVersion)
	   {
		   @Arguments = ("/bin/rpm", "-i", "--force", "--replacepkgs", $Location . $FileName);
	   }
	   else
	   {
		   @Arguments = ("/bin/rpm", "-U", "--force", $Location . $FileName);
	   }
				
	   if ($rpm_forkflag)
	   {
			if (open(EXTSCRF, ">/var/tmp/em_extscript_rpm.sh"))
			{
				print EXTSCRF "#/bin/bash\n\n";
				print EXTSCRF "/bin/sleep 3\n";
				print EXTSCRF "exec " . @Arguments[0] . " " . @Arguments[1] . " " . @Arguments[2] . " " . @Arguments[3] . " \n";
				close(EXTSCRF);
			}
			else
			{
				$Return = 1;	# Unable to create scripts!
			}
			
			if (!(chmod (0700, "/var/tmp/em_extscript_rpm.sh")))
			{
				$Return = 1;	# Unable to change script modes!
			}
			
			Safe::spawn_daemon("/var/tmp/em_extscript_rpm.sh");
			sleep 1;
			$Return = 0;
			
			# Finished.
	   }
	   else
	   {
		  $Return = (Safe::system(@Arguments) == 0) ? 0 : 1;
	   }
	  }
	  else
	  {
		$Return = 240006;
	  }
	return $Return
}

###############################################################################
# Main
###############################################################################

sub Main
{
	my ($cgi)	= @_;
	my $Status;
	my $ManifestName;
	my $ProductInfo		= new ProductInformation();
	my $ProductModel;
	my $Model;
	my $RPMModel;
	my $ProductMatch;
	
	# If there is a download request...
	if ($cgi->param('download'))
	{
		$ProductModel = $ProductInfo->getModel();
		$RPMModel = $cgi->param('model');
		  
		# No matches so far.
		$ProductMatch = 0;

		# Run through the acceptable models...
		foreach $Model (split(/,/, $RPMModel))
		{
			# Look for a matching model number...
			if (($Model eq $ProductModel) || 
				($Model eq '*')) 
			{
				# A match is found.
				$ProductMatch = 1;
				last;
			}			    
		}
	
		if ($ProductMatch)
		{
			$ManifestName		= $ProductInfo->getManufacturerID() . "_" . $ProductInfo->getFamily() . "_" . $ProductInfo->getSeries();
			$ManifestName		= lc($ManifestName);
			my $SoftwareSupport	= new SoftwareSupport($ManifestName, $ProductInfo->getVersion(), '/home/updates/');
			
			# Get the update onto the local box.
			$Status = ($SoftwareSupport->GetProductUpdate($cgi->param('location') . $cgi->param('filename'), $cgi->param('filename'))) ? 0 : 1;
		}
		else
		{
			$Status = 240006;  
		}
	}
	elsif ($cgi->param('downloadrpm'))
	{
		$ManifestName		= $ProductInfo->getManufacturerID() . "_" . $ProductInfo->getFamily() . "_" . $ProductInfo->getSeries();
		$ManifestName		= lc($ManifestName);
		my $SoftwareSupport	= new SoftwareSupport($ManifestName, $ProductInfo->getVersion(), '/home/updates/');
	  
		my $url				= new URI::URL($cgi->param('location'));
		my $Source			= $url->scheme . "://" . $url->netloc . $url->epath;
		my $Target 			= $url->epath;

		# Get just the resource...
		$Target =~ s/.*\/(.*)$/$1/;

		# Get the update onto the local box.
		$Status = ($SoftwareSupport->GetProductUpdate($Source, $Target)) ? 0 : 1;
		
	}
	elsif ($cgi->param('install'))
	{
		# Install the RPM.
		$Status = InstallRPM($cgi->param('location'), $cgi->param('filename'), $cgi->param('model'));
	}
	else
	{
		$Status = 1;
	}

	print($cgi->header());
	print($Status);
}

###############################################################################
# The main body of the script.
###############################################################################

my $cgi = new CGI;

Main($cgi);

