#!/usr/bin/perl

###############################################################################
#                                                                             #
# Filename: firstboot_decline.cgi                                             #
#                                                                             #
# Description: CGI needed to display the first boot license decline page.     #
#                                                                             #
###############################################################################
###############################################################################
#                                                                             #
#  Copyright  2000 - 2001 Intel Corporation.                                 #
#  Intel Corporation All Rights Reserved.                                     #
#                                                                             #
#  The source code contained or described herein and all documents related to #
#  the source code ("Material") are owned by Intel Corporation or its         #
#  suppliers or licensors.  Title to the Material remains with Intel          #
#  Corporation or its suppliers and licensors.  The Material contains trade   #
#  secrets and proprietary and confidential information of Intel or its       #
#  suppliers and licensors.  The Material is protected by worldwide copyright #
#  and trade secret laws and treaty provisions.  No part of the Material may  #
#  be used, copied, reproduced, modified, published, uploaded, posted,        #
#  transmitted, distributed, or disclosed in any way without Intel's prior    #
#  express written permission.                                                #
#                                                                             #
#  No license under any patent, copyright, trade secret or other intellectual #
#  property right is granted to or conferred upon you by disclosure or        #
#  delivery of the Materials, either expressly, by implication, inducement,   #
#  estoppel or otherwise. Any license under such intellectual property rights #
#  must be express and approved by Intel in writing.                          #
#                                                                             #
###############################################################################

# Put the PERL library in the path.
use lib '/usr/ins/intel/gui/perllib';

use strict;
use LanguageResource;
use DisplayMain;
use CGI;

###############################################################################
# Main
###############################################################################

sub Main
{
	my ($cgi) = @_;
	
	# If a button is pressed...
	if (defined($cgi->param('back')))
	{
		# Redirect to the previous form.
		print $cgi->redirect( -url => 'firstboot_license.cgi' );
		return;
	}
	else
	{
		my $PreferredLanguage	= (split(/,/, $ENV{'HTTP_ACCEPT_LANGUAGE'}))[0];
		my $StringResource		= new LanguageResource($PreferredLanguage, '/usr/ins/intel/gui/locale', [ 'firstboot_decline.lrc', 'common.lrc' ]);
		my $FileResource		= new LanguageResource($PreferredLanguage, '/usr/ins/intel/gui/locale');
		my $DeclineFile			= $FileResource->ResolveResourceFile('license_decline');
		my %_R					= ( 'IDT:10200'			=> '',
									'BACK'				=> '' );
		my %Arrow;
		my $html;

		# Get the string resources.
		$StringResource->GetStrings(\%_R);
		
		# Based on the user agent...
		if (-1 < (index($cgi->user_agent(), 'MSIE')))
		{
			# Set the Microsoft arrows.
			$Arrow{'left'}	=  '&larr;';
		}
		else
		{
			# Set the other arrows.
			$Arrow{'left'}	=  '<-';
		}
		
		# Get the license agreement.
		if (open(LICENSE, $DeclineFile))
		{
			# Read in the file.
			read(LICENSE, $_R{'LICENSE_DECLINE'}, -s $DeclineFile);
	
			# Close the file.
			close (LICENSE);
		}
		else
		{
			# Bail out.
			return 0;
		}
	
		$html = <<HERE;
<table border=0 width=550 bgcolor=white>
<tr>
	<td bgcolor=#eeeeee>
		<table border=0 width=100% bgcolor=#eeeeee>
		<tr>
			<td>&nbsp;
				<table cellspacing=1 border=0 width=100% bgcolor=#eeeeee>
				<form method=post action=firstboot_decline.cgi enctype=application/x-www-form-urlencoded>
				<tr>
					<td valign=top width=626>$_R{'LICENSE_DECLINE'}</td>
				</tr>
				<tr>
					<td width=526 colspan=2>&nbsp;</td>
				</tr>
				<tr>
					<td nowrap colspan=2 align=right width=532>
						<input type=submit class=clsButton value='$Arrow{'left'}&nbsp;$_R{'BACK'}' name=back>
					</td>
				</tr>
				</form>
				</table>
			</td>
		</tr>
		</table>
	</td>
</tr>
</table>
HERE
		
		# Display the page.
		DisplayMain->printDocument({ 'q' => $cgi, 'html' => $html, 
									 'class' => 'firstboot1', 'id' => 10200, 'title' => $_R{'IDT:10200'}, 'margin-left' => 3 });
	}
}

###############################################################################
# The main body of the script.
###############################################################################

my $cgi = new CGI;

Main($cgi);
