#!/usr/bin/perl

###############################################################################
#                                                                             #
# Filename: firstboot_finalize.cgi                                            #
#                                                                             #
# Description: CGI needed to complete the first boot process.                 #
#                                                                             #
###############################################################################
###############################################################################
#                                                                             #
#  Copyright  2000 - 2001 Intel Corporation.                                 #
#  Intel Corporation All Rights Reserved.                                     #
#                                                                             #
#  The source code contained or described herein and all documents related to #
#  the source code ("Material") are owned by Intel Corporation or its         #
#  suppliers or licensors.  Title to the Material remains with Intel          #
#  Corporation or its suppliers and licensors.  The Material contains trade   #
#  secrets and proprietary and confidential information of Intel or its       #
#  suppliers and licensors.  The Material is protected by worldwide copyright #
#  and trade secret laws and treaty provisions.  No part of the Material may  #
#  be used, copied, reproduced, modified, published, uploaded, posted,        #
#  transmitted, distributed, or disclosed in any way without Intel's prior    #
#  express written permission.                                                #
#                                                                             #
#  No license under any patent, copyright, trade secret or other intellectual #
#  property right is granted to or conferred upon you by disclosure or        #
#  delivery of the Materials, either expressly, by implication, inducement,   #
#  estoppel or otherwise. Any license under such intellectual property rights #
#  must be express and approved by Intel in writing.                          #
#                                                                             #
###############################################################################

# Put the PERL library in the path.
use lib '/usr/ins/intel/gui/perllib';

use strict;
use ProductInformation;
use ConfigurationData;
use LanguageLogCGI;
use DisplayMain;
use VirtualDomain;
use VirtualServ;
use DomainMail;
use MonSettings;
use Mail;
use File::Copy;
use CookieData;
use NetConfig;
use Chronos;
use Daemon;
use Usable;
use Users;
use Safe;
use SSL;
use CGI;
use DNSConfig;
use DNSZone;

# Flush files.
$| = 1;

###############################################################################
# logMessage
###############################################################################

sub logMessage
{
	my ($Code, $Category, $Severity, $ParameterRef) = @_;
	my $Log	= new LanguageLogCGI('/usr/ins/intel/gui/logs/server.log', '/usr/ins/intel/gui/locale', [ 'firstboot5.lrc' ]);

	# Log the error.
	$Log->Append($Category, $Severity, $Code, $ParameterRef);
}

###############################################################################
# createDefaultDNS
###############################################################################

sub createDefaultDNS
{
	my ($fqdn,$zoneipaddr,$zoneemail,$useless) = @_;

	my ($ia,$ib,$ic,$ic1,$ic2,$ic3,$ic4);
	my (@a1,@a2,@a3);
	my ($t1,$t2,$t3,$unused);
	my @z_info;

	if (	($fqdn eq '') || (!(defined($fqdn))) ||
		($zoneipaddr eq '') || (!(defined($zoneipaddr))) )
	{
		return -1;
		# Do nothing.
	}
	
	# Split out extra dots at the end of the name.
	while ($fqdn =~ /(\.)$/)
	{
		$fqdn = $`;
	}
	
	@a1 = split('\.',$fqdn);
	
 
	my $finishedwork = 0;
	my $unabletocreate = 0;
	my $found = 0;
	my $dowork = 1;
	my $foundself = 0;
	my $createfirstparent = 1;
	my $failedprocess = 0;

	my @firstparentsoa;
	my $firstparentfile;

	while ( ($a1[0] eq '') && (scalar(@a1) > 1) )
	{
		$unused = shift(@a1);
	}

	# Next look for well-known host name ("www") at the start of the zone name.
	if (lc($a1[0]) eq "www")
	{
		# If so, make a zone based on the remainder of the virtual domain name.

		$unused = shift(@a1);
	}

	@a2 = @a1;

	$ia = DNSConfig->zone_getinfo(\@z_info, join('.',@a2));

	if ( ($ia eq 0) && ($z_info[1] ne '') )
	{
		# Zone already exists!
		$found = 1;
		$foundself = 1;
	}
	
	while ( ($found eq 0) && (scalar(@a2) > 2) )
	{
		# Begin a subzone check.
		# Pop off the first segment of the name and save to a new array.
		# Then check for the remainder of the array to see if it exists as a subzone.

		$t3 = '';
		$t3 = shift(@a2);
		push(@a3,$t3);

		@z_info = ['', '', '', '', ''];
		my $z_check = join('.', @a2);

		$ia = DNSConfig->zone_getinfo(\@z_info, $z_check);

		# Does the zone exist here, and is it a master zone?
		if ( ($ia eq 0) && ($z_info[0] ne '') && ($z_info[1] ne '') && ($z_info[2] ne '2') )
		{
			# If so, then check this subzone for incriminating records or delegations.

			$found = 1;

			my (@pz_soa,@pz_ns,@pz_mx,@pz_a,@pz_cname,@pz_ptr,@pz_nssub); 
			$ib = DNSZone->zone_getinfo($z_info[0],$z_info[1],\@pz_soa,\@pz_ns,\@pz_mx,\@pz_a,\@pz_cname,\@pz_ptr,\@pz_nssub);

			if ($ib eq 0)
			{
				# The zone exists and we can read its records.  Now let's check for any conflicts in the @pz_nssub arena.

				my $i;

				for ($i = 0; $i < scalar(@pz_nssub); $i++)
				{
					my @a4 = [];
					my ($t4,$t5);

					@a4 = @a3;

					my $foundsz = 0;

					while ( (scalar(@a4) > 0) && ($foundsz eq 0) )
					{
						$t4 = join('.',@a4);
						$t5 = $t4 . '.' . join('.',@a2);

						# Check for NS Record named $t4 or $t5.

						if ( ($t4 eq $pz_nssub[$i][0]) || ($t5 eq $pz_nssub[$i][0]) )
						{
							# Match!
							$foundsz = 1;
							$unabletocreate = 1;
							goto JUMPOUTDNSCHECK;
						}

						# If neither exists, remove first element of @a4 and go again.

						my $useless = shift(@a4);
					}
				}

				# Now check A records.

				for ($i = 0; $i < scalar(@pz_a); $i++)
				{
					my @a4 = [];
					my ($t4,$t5);

					@a4 = @a3;

					my $founda = 0;

					$t4 = join('.',@a4);
					$t5 = $t4 . '.' . join('.',@a2);
					# Check for A Record named $t4 or $t5.

					if ( ($t4 eq $pz_a[$i][0]) || ($t5 eq $pz_a[$i][0]) )
					{
						# Match!
						$founda = 1;
						$unabletocreate = 1;
						goto JUMPOUTDNSCHECK;
					}


				}

				# Now check CNAME records.

				for ($i = 0; $i < scalar(@pz_cname); $i++)
				{
					my @a4 = [];
					my ($t4,$t5);

					@a4 = @a3;

					my $foundcname = 0;

					$t4 = join('.',@a4);
					$t5 = $t4 . '.' . join('.',@a2);
					# Check for CNAME Record named $t4 or $t5.

					if ( ($t4 eq $pz_cname[$i][0]) || ($t5 eq $pz_cname[$i][0]) )
					{
						# Match!
						$foundcname = 1;
						$unabletocreate = 1;
						goto JUMPOUTDNSCHECK;
					}


				}

				# At this point, it is safe to assume that the zone does exist, but no conflicting records or
				# subzone delegations are present.  Therefore, we can make a subzone now.

				# Parent zone will be join('.',@a2)
				# Subzone name will be join('.',@a3)

				# Inherit fun properties from parent zone, and assemble new SOA array.

				$createfirstparent = 0;

				@firstparentsoa = @pz_soa;
				$firstparentsoa[0] = '';
				$firstparentsoa[1] = '';
				$firstparentsoa[2] = $zoneemail;
				$firstparentsoa[3] = '';

				$firstparentfile = $z_info[1];

				goto JUMPOUTDNSCHECK;

			}
		}
	}

	JUMPOUTDNSCHECK:

	
	if ( ($dowork eq 1) && ($finishedwork eq 0) && ($unabletocreate ne 1) && ($foundself ne 1) )
	{
		# Zone does not exist, so let's go make it!
		my (@new_soa,@new_ns,@new_mx,@new_a,@new_cname,@new_ptr,@new_nssub);

		my $new_sub_done;
		my $prev_file;

		my $new_zone_file;
		my $new_zone_full = join('.',@a2);
		my $new_zone_full_dot = $new_zone_full . '.';

		if ($createfirstparent eq 1)
		{
			# Begin with the assumption that @a2 is our final domain name,
			# and @a3 is our host portion.  Assume that size of @a2 is at least 1, hopefully 2 or greater.

			# Just make a domain with @a2 as the name.

			$ia = DNSConfig->zone_add($new_zone_full,\$new_zone_file, '1');

			if ($ia eq 0)
			{
				$ib = DNSZone->zone_construct_fwd($new_zone_full_dot,$new_zone_file,$zoneipaddr);

				if ($ib eq 0)
				{
					# Looks like we successfully created the zone.  Congratulations.

					my (@new_ns,@new_mx,@new_a,@new_cname,@new_ptr,@new_nssub);

					$ic1 = DNSZone->zone_getinfo($new_zone_full_dot,$new_zone_file,\@new_soa,\@new_ns,\@new_mx,\@new_a,\@new_cname,\@new_ptr,\@new_nssub);

					if ($ic1 eq 0)
					{
						$new_soa[2] = $zoneemail;
						$new_soa[4] = '8H';
						$new_soa[5] = '2H';
						$new_soa[6] = '1W';
						$new_soa[7] = '1D';

						$ic2 = DNSZone->zone_changeheader($new_zone_full_dot,$new_zone_file,\@new_soa);

						if ($ic2 eq 0)
						{
							$prev_file = $new_zone_file;	
						}
						
					}
				} 
			}
		
			if ( ($ia ne 0) || ($ib ne 0) || ($ic1 ne 0) || ($ic2 ne 0) )
			{
				$failedprocess = 1;
				goto FAILDNSCREATE;
			}
		
		}
		else
		{
			@new_soa = @firstparentsoa;
			$prev_file = $firstparentfile;
		}
		
		$new_sub_done = '';

		while (scalar(@a3) > 0)
		{
			my @asub;

			@asub = @a3;

			my $new_sub_file;
			my $new_sub_short = pop(@a3);
			my $new_sub_full = $new_sub_short . '.' . $new_sub_done . $new_zone_full;
			my $new_sub_full_dot = $new_sub_full . '.';

#			# About to begin (via loop) making subzone chain @a3 inside parent zone @a2

			# Make the new subzone first.
			$ia = DNSConfig->zone_add($new_sub_full,\$new_sub_file, '1');

			if ($ia eq 0)
			{
				# Next run the constructor for the subzone.

				$ib = DNSZone->zone_construct_fwd($new_sub_full_dot,$new_sub_file,$zoneipaddr);

				if ($ib eq 0)
				{
					# Looks like we successfully created the subzone.  Congratulations.

					my (@new_ns,@new_mx,@new_a,@new_cname,@new_ptr,@new_nssub);

					$ic1 = DNSZone->zone_getinfo($new_sub_full_dot,$new_sub_file,\@new_soa,\@new_ns,\@new_mx,\@new_a,\@new_cname,\@new_ptr,\@new_nssub);

					if ($ic1 eq 0)
					{
						$new_soa[2] = $zoneemail;
						$new_soa[4] = '8H';
						$new_soa[5] = '2H';
						$new_soa[6] = '1W';
						$new_soa[7] = '1D';
						
						$ic2 = DNSZone->zone_changeheader($new_sub_full_dot,$new_sub_file,\@new_soa);

						if ($ic2 eq 0)
						{
							# Okay, now we're really hoping that this will all work smoothly.
							# Let's add stuff to the parent zone, shall we?

							$ic3 = DNSZone->zone_addrecord($new_sub_done . $new_zone_full,$prev_file,"NS",$new_sub_short, 'dns.' . $new_sub_done . $new_zone_full_dot);
							
							if ($ic3 eq 0)
							{
								$ic4 = DNSZone->zone_increment($new_sub_done . $new_zone_full,$prev_file);
								
								if ($ic4 eq 0)
								{
									# If this is okay, push values and move on.

									$prev_file = $new_sub_file;
									my $t7 = $new_sub_short . '.' . $new_sub_done;
									$new_sub_done = $t7;
								}
							
							}
						
						}

					}

				}
			
			}

			if ( ($ia ne 0) || ($ib ne 0) || ($ic1 ne 0) || ($ic2 ne 0) || ($ic3 ne 0) || ($ic4 ne 0) )
			{
				# failure, so quitting now.
				$failedprocess = 1;
				goto FAILDNSCREATE;
			}

		}
	
	
	}
	elsif ($foundself eq 1)
	{
		return 2;
	}
	elsif ($unabletocreate eq 1)
	{
		return 0;
	}

	# Everything appears to be successful.

	return 1;

	FAILDNSCREATE:

	if ($failedprocess eq 1)
	{
		return -2;
	}
	
	return 0;

}

###############################################################################
# InvalidIP
###############################################################################

sub InvalidIP
{
	my ($ipaddress,$useless) = @_;
	
	my ($ipA,$ipB,$ipC,$ipD);
	my ($t1,$t2);
	my $tempIP = $ipaddress;

	my $chkval = ( $tempIP =~ /(\d{1,3})(\.)(\d{1,3})(\.)(\d{1,3})(\.)(\d{1,3})/ );

	if (!($chkval) )
	{
		# If the string does not match the approved format
		return 1;  # false
	}
	elsif ( ($` ne '') || ($' ne '') || ($& eq '') )
	{
		return 1;
		# Matched, but there's extra data before or after the string.
	}
	else
	{
		($ipA,$ipB,$ipC,$ipD) = ( $tempIP =~ m/(\d{1,3})\.(\d{1,3})\.(\d{1,3})\.(\d{1,3})/ );
		
		if ( ($ipA < 0) || ($ipA > 255) )
		{
			return 1;
		}
		elsif ( ($ipB < 0) || ($ipB > 255) )
		{
			return 1;
		}
		elsif ( ($ipC < 0) || ($ipC > 255) )
		{
			return 1;
		}
		elsif ( ($ipD < 0) || ($ipD > 255) )
		{
			return 1;
		}
		else
		{
			return 0;
		}

	}

}

###############################################################################
# createReverseDNS
###############################################################################

sub createReverseDNS
{
	my ($fqdn,$zoneipaddr,$zoneemail,$useless) = @_;

	my ($ia,$ib,$ic,$ic1,$ic2,$ic3,$ic4);
	my (@a1,@a2,@a3);
	my ($t1,$t2,$t3,$unused);
	my @z_info;
	my $new_fqdn;

	if (	($fqdn eq '') || (!(defined($fqdn))) ||
		($zoneipaddr eq '') || (!(defined($zoneipaddr))) )
	{
		return -1;
		# Do nothing.
	}
	
	# Check for invalid IP addresses
	if (InvalidIP($zoneipaddr))
	{
		return -1;
	}
	
 
	my $found = 0;
	my $dowork = 1;
	my $conflict = 0;
	my $success = 1;

	
	$new_fqdn = $fqdn;

	# Next look for well-known host name ("www") at the start of the zone name.
	if ($new_fqdn =~ /^www./)
	{
		# If so, use the remainder of the virtual domain name.

		$new_fqdn = $';
	}

	# Next, begin process of creating a reverse zone.

	@a1 = split('\.',$zoneipaddr);

	if (scalar(@a1) != 4)
	{
		return -1;
		# Error and quit.
	}

	@a2 = reverse(@a1);
	$t1 = shift(@a2);

	# Now, #t1 contains the name of the new PTR record, while
	# @a2 contains the reversed portion of the IP to be used
	# as the reverse zone name identifier.  Check to make sure that this
	# zone does not exist already.
	
	$t2 = join('.',@a2) . '.in-addr.arpa';
	
	# Start with level 1.
	
	$ia = DNSConfig->zone_getinfo(\@z_info,$t2); 

	if ($ia eq 0)
	{
		# Sure, its here, but is it a master?
		
		if ( ($z_info[0] ne '') && ($z_info[1] ne '') && ($z_info[2] eq '1') )
		{
		# If the zone exists, check it for PTR records to test for any conflicts.
		
			my (@pz_soa,@pz_ns,@pz_mx,@pz_a,@pz_cname,@pz_ptr,@pz_nssub); 
			$ib = DNSZone->zone_getinfo($z_info[0],$z_info[1],\@pz_soa,\@pz_ns,\@pz_mx,\@pz_a,\@pz_cname,\@pz_ptr,\@pz_nssub);

			if ($ib eq 0)
			{
				# It exists, and we can read the record information.  Now look for conflicting PTR or NSSUB records.
				# This is Firstboot and none should exist, but check anyway.

				my $i;

				for ($i = 0; $i < scalar(@pz_ptr); $i++)
				{
					# Loop through each PTR record.
					if (	($pz_ptr[$i][0] eq $t1) ||
						(($pz_ptr[$i][0] . '.' . $t2) eq $t1) )
					{
						# Match found!  Exit gracefully.
						$conflict = 1;
						goto JUMPDNSREVERSEFAILURE;
					}
				}

				for ($i = 0; $i < scalar(@pz_nssub); $i++)
				{
					# Loop through each NSSUB record.
					if (	($pz_nssub[$i][0] eq $t1) ||
						(($pz_nssub[$i][0] . '.' . $t2) eq $t1) )
					{
						# Match found!  Exit gracefully.
						$conflict = 1;
						goto JUMPDNSREVERSEFAILURE;
					}
				}

				# Assume by this point that the reverse zone does exist, and that a record can be created.

				$ic1 = DNSZone->zone_addrecord($z_info[0],$z_info[1],"PTR",$t1,$new_fqdn . '.');

				if ($ic1 eq 0)
				{
					$ic2 = DNSZone->zone_increment($z_info[0],$z_info[1]);

					if ($ic2 eq 0)
					{
						# Success!

						$success = 1;
						goto JUMPDNSREVERSEFINISH;
					}
				}
			}

			if ( ($ib ne 0) || ($ic1 ne 0) || ($ic2 ne 0) )
			{
				goto JUMPDNSREVERSEFAILURE;
			}
		}
	}

	elsif ( ($ia ne 0) && ($z_info[0] eq '') && ($z_info[1] eq '') )
	{
		# The zone does not exist.  Let's create it.

		my $newrevfile;
		
		$ib = DNSConfig->zone_add($t2,\$newrevfile, '1');

		if ($ib eq 0)
		{
			# If the add was successful, then construct the zone file.

			$ic1 = DNSZone->zone_construct_rev($t2 . '.',$newrevfile,'admin@localhost.');

			if ($ic1 eq 0)
			{
				# If the file was created successfully, update its SOA.

				my @new_soa;

				$new_soa[2] = $zoneemail;
				$new_soa[4] = '8H';
				$new_soa[5] = '2H';
				$new_soa[6] = '1W';
				$new_soa[7] = '1D';
				
				$ic2 = DNSZone->zone_changeheader($t2 . '.',$newrevfile,\@new_soa);

				if ($ic2 eq 0)
				{
					# If the SOA was updated successfully, then add the PTR record in question.

					$ic3 = DNSZone->zone_addrecord($t2 . '.',$newrevfile,"PTR",$t1,$new_fqdn . '.');

					if ($ic3 eq 0)
					{
						$ic4 = DNSZone->zone_increment($t2 . '.',$newrevfile);

						if ($ic4 eq 0)
						{
							# Success!

							$success = 1;
							goto JUMPDNSREVERSEFINISH;
						}
					}
				}
			}
		}
		
		if ( ($ib ne 0) || ($ic1 ne 0) || ($ic2 ne 0) || ($ic3 ne 0) || ($ic4 ne 0) )
		{
			goto JUMPDNSREVERSEFAILURE;
		}
	}

	
	JUMPDNSREVERSEFAILURE:
	if ( ($conflict eq 1) || ($success eq 0) )
	{
		return -1;
	}


	JUMPDNSREVERSEFINISH:
	return 0;

}

###############################################################################
# createDefaultDomain
###############################################################################

sub createDefaultDomain
{
	my ($host,$domain,$user,$passwd,$ipaddr,$sslon,$priv) = @_;
	my $Monitor		= new MonSettings($host . '.' . $domain);
	my $admin_ssl	= 'Yes' if ($sslon eq 'Yes');
	$sslon = 'no';
	my ($q) = @_;
	my $fqdn = "$host\."."$domain";
	my ($data,$domain_conf,$nwgrp,$nwgid,$lastgrp);
	my $grp = getgrnam("domain1");
	
	if (!$grp)
	{	
		($nwgrp, $nwgid, $lastgrp)  = Users::domainGrp;
		Users::grpAdd;
	}
	else
	{
		$nwgrp = "domain1";
		$nwgid = 11001;
		my ($vd_name,$vd_adm,$vd_ip);
		my @act1 = VirtualDomain::list_vws();
		
		foreach my $i (@act1)
		{
			if ($i->{dom_grp} eq "domain1")
			{
				$vd_name = $i->{vdn};
				$vd_adm = $i->{vdadm};
				$vd_ip = $i->{vdip};
				last;
			}
		}
		
		my $symlink = "/home/domain/$vd_name";
		VirtualDomain::del_vws($vd_name,$nwgrp,$vd_adm,$vd_ip);
		unlink("$symlink") if (-l "$symlink");
		DomainMail::mail_relay_remove($vd_name,$vd_adm);
		VirtualDomain::delFtp($vd_ip,$nwgrp);
	}
	my $type = '1';
	my $vdquota = '500';
	Quotaall::setquota($nwgid,$vdquota,$type);

	my $result = VirtualDomain::create_dom_home($fqdn,$nwgrp,$user,$passwd,'',$sslon,$priv,$admin_ssl);
	if ($result ne 'yes')
	{
		# Log the error.
		logMessage(1060006, 'error', 'critical', [ $fqdn, $user ]);

		VirtualDomain::cleanup($fqdn,$nwgrp,$user,'');
	}

	# Log the event.
	logMessage(1060007, 'security', 'information', [ $fqdn ]);
	
	# If IP is already in use then don't do this ....
	my ($var1, $var2) = VirtualDomain::ip_domain($ipaddr);
	if ($var1 eq '')
	{
		my $result1 = VirtualDomain::create_vip($ipaddr,'',$nwgrp, $user,$fqdn, $sslon);
		if ($result1 ne 'yes')
		{
			# Log the error.
			logMessage(1060008, 'error', 'critical', [ $fqdn ]);
			
			VirtualDomain::cleanup($fqdn,$nwgrp,$user,$ipaddr);
		}
		
		# Log the event.
		logMessage(1060009, 'security', 'information', [ $fqdn ]);
	}
	else
	{ 
		# Log the error.
		logMessage(1060010, 'error', 'warning', [ $fqdn, $var2 ]);
	}

	VirtualDomain::create_vd($ipaddr,$fqdn,$user,$domain,$nwgrp,'Yes','Yes',$sslon);
	VirtualServ::setHostFile();

	# create postmaster for the default domain and add entry in /etc/alias file for that
	my $post_master = "postmaster\@localhost";
	Usable::write_postmaster_alias($post_master);

	# create DNS configuration for new default domain;
	createDefaultDNS($fqdn,$ipaddr,"postmaster" . '@' . $domain);
	createReverseDNS($fqdn,$ipaddr,"postmaster" . '@' . $domain);

        # Restart apache apache server now ..........
	Safe::system("(/etc/rc.d/init.d/httpd stop; /bin/sleep 2; /etc/rc.d/init.d/httpd start)");
	Mail::set_mail_domain($nwgrp,$fqdn);
	Mail::set_mail_domain_aliases($nwgrp,$domain);
	my $alias = "postmaster,webmaster";
	DomainMail::modAlias($user,$fqdn,$alias);
	
	Safe::backtick('/usr/bin/makemap hash /etc/mail/virtusertable.db < /etc/mail/virtusertable');
	Safe::system("/etc/rc.d/init.d/sendmail", "restart");
	
	# Open the configuration.
	if (0 == $Monitor->open())
	{
		# Set the monitor states.
		$Monitor->setMonitorState('http', 1);
		$Monitor->setMonitorState('smtp', 1);
		$Monitor->setMonitorState('ftp',  1);

		# Close the configuration.
		$Monitor->close();
	}
}

###############################################################################
# configureAdministratorServer
###############################################################################

sub configureAdministratorServer
{
	my ($HostName, $Domain, $Administrator) = @_;
	my $fqdn;

	# If there is a domain...
	if ('' eq $Domain) 
	{
		# Use localhost.
		$fqdn	= $Domain = 'localhost';
	}
	else
	{
		# Build the FQDN.
		$fqdn	= $HostName . '.' . $Domain;
	}

	# Safe default.
	$Administrator = 'root' if('' eq $Administrator);

	# Build the administrator address.
	$Administrator = $Administrator . '@' . $Domain;
	
	# Try to read in the file...
	if (open(CONFIGURATION, '< /usr/ins/intel/apache/conf/httpd.conf'))
	{
		# Read in the configuration.
		my @Configuration  = <CONFIGURATION>;
		my $Line;
		
		# Close the file.
		close(CONFIGURATION);
        
		# Run through the file...
		foreach $Line (@Configuration)
		{
			# If the server name directive is found...
			if ($Line =~ /^ServerName/)
			{
				# Substitute in the FQDN.
				$Line =~ s/($1)\s+(\S+)/$1\t$fqdn/;
			}
            
			# If the server administrator is found...
			if ($Line =~ /^ServerAdmin/)
			{
				# Substitute in the server administrator.
				$Line =~ s/($1)\s+(\S+)/$1\t$Administrator/;
			}
        }

		# Open the file for writing and lock it...
        if ((open(CONFIGURATION,'> /usr/ins/intel/apache/conf/httpd.conf')) &&
			(flock(CONFIGURATION, 2)))
		{
			# Write out the data.
			print CONFIGURATION @Configuration;

			# Unlock the file.
			flock(CONFIGURATION, 8);

			# Close the file.
			close(CONFIGURATION);
		}
	}
}

###############################################################################
# initializeHostname
###############################################################################

sub initializeHostname
{
	my ($Hostname, $Domain) = @_;
	my $Return = 0;

	# Set the host information.
	VirtualServ::save_host($Hostname, $Domain);

	return $Return;
}

###############################################################################
# initializeDNS
###############################################################################

sub initializeDNS
{
	my ($DNSPrimary, $DNSSecondary, $Domain) = @_;
	my (%DNS, $DNS);
	my $Return = 0;

	# Create the appropriate hash.
	push(@{ $DNS->{'name'} }, $DNSPrimary, $DNSSecondary);
	push(@{ $DNS->{'domain'} }, $Domain);
	
	# Save the DNS configuration.
	VirtualServ::save_dns($DNS);
	
	return $Return;
}

###############################################################################
# initializeServiceMonitor
###############################################################################

sub initializeServiceMonitor
{
	my $Return = 0;
	
	# Try to open the service configuration file...
	if (open(SERVICES, '>> /etc/services'))
	{
		print SERVICES "mon             2583/tcp                        # MON\n";
		print SERVICES "mon             2583/udp                        # MON traps\n";
		
		# Close the file.
		close(SERVICES);
		
		# Copy the configuration file into place.
		copy('/usr/local/mon/etc/auth.cf', '/etc/auth.cf');
	}
	else
	{
		# Set the return code...
		$Return = 1060001;
	}
	
	return $Return;
}

###############################################################################
# initializeOpenSSH
###############################################################################

sub initializeOpenSSH
{
	my $DaemonSSH	= new Daemon('sshd');
	my $Return		= 0;
	
	# If the OpenSSH host key is not yet available...
	unless ( -e '/etc/ssh/ssh_host_key' )
	{
		# Generate the host key for OpenSSH...
		Safe::system("/usr/bin/ssh-keygen -b 1024 -f /etc/ssh/ssh_host_key -N ''");
	}
	
	# If the OpenSSH host DSA key is not yet available...
	unless ( -e '/etc/ssh/ssh_host_dsa_key' )
	{
		# Generate the host DSA key for OpenSSH...
		Safe::system("/usr/bin/ssh-keygen -d -f /etc/ssh/ssh_host_dsa_key -N ''");
	}
	
	# Try to start OpenSSH...
	unless (0 == $DaemonSSH->Control(1))
	{	
		# Log the error and set the return code.
		logMessage($Return = 1060000, 'error', 'warning');
	}

	return $Return;
}

################################################################################
#  createQuota
################################################################################

sub createQuota
{
	my $Log		= new LanguageLogCGI('/usr/ins/intel/gui/logs/server.log', '/usr/ins/intel/gui/locale', [ 'firstboot5.lrc' ]);
	my $Return	= 0;
	
	# If the file already exists...
	if ( -e '/etc/fstab' )
	{
		# Back up the old file.
		copy('/etc/fstab', '/etc/fstab.old');
	}

	# Try to open the file for reading...
	if (open(FSTAB, '< /etc/fstab'))
	{
		my @FSTable;
		my $Line;
		
		# Try to lock the file.
		flock(FSTAB, 2);
		
		# Read in the file.
		@FSTable = <FSTAB>;

		# Try to unlock the file.
		flock(FSTAB, 8);
		
		# Close the file.
		close(FSTAB);
		
		# Run through the table...
		foreach $Line (@FSTable)
		{
			# If the home partition is found...
			if ($Line =~ /home/)
			{
				my $Insert	= 'defaults,grpquota,usrquota';
				
				# Modify the line.
				$Line =~ s/(\S+)\s+(\S+)\s+(\S+)\s+(\S+)\s+(.*)/$1\t\t$2\t\t\t$3\t$Insert\t$5/;
				last;
			}
		}

		# Try to open the file for writing...
		if (open(FSTAB, '> /etc/fstab'))
		{
			# Try to lock the file.
			flock(FSTAB, 2);
			
			# Spew out the modified file.
			print FSTAB @FSTable;
			
			# Try to unlock the file.
			flock(FSTAB, 8);
			
			# Close the file.
			close(FSTAB);
	
			# Try to check the quota...
			unless (0 == Safe::system('/sbin/quotacheck', '-aug'))
			{
				# Log the error and set the return code.
				logMessage($Return = 1060003, 'error', 'warning');
			}
			
			# Try to set the quota...
			unless (0 == Safe::system('/sbin/quotaon', '-aug'))
			{
				# Log the error and set the return code.
				logMessage($Return = 1060002, 'error', 'critical');
			}
		}
		else
		{
			# Log the error and set the return code.
			logMessage($Return = 1060004, 'error', 'critical');
		}
	}
	else
	{
		# Log the error and set the return code.
		logMessage($Return = 1060005, 'error', 'critical');
	}
	
	return $Return;
}

################################################################################
# set13xxMailService - stop the mail service on the 1305 and 1320
################################################################################

sub set13xxMailService
{
	my $StoreFrontCount = Safe::backtick("/usr/ins/intel/bin/asm/sysSMBIOS dump system_information | /bin/grep PRODUCT_NAME | /bin/sed 's/\\(.*\\)\\(13\\)\\(..\\)\\(.*\\)/\\3/'");
	
	# Clean up the data...
	chop($StoreFrontCount);
	
	# If this is a 5 or 20 store SKU...
	if (('05' eq $StoreFrontCount) ||
		('20' eq $StoreFrontCount))
	{
		# Shutdown the mail service.
		Safe::system('/etc/rc.d/init.d/sendmail stop');
		Safe::system('/sbin/chkconfig --del sendmail');
	}
}

###############################################################################
# DisplayWaitMessage
###############################################################################

sub DisplayWaitMessage
{
	my ($cgi, $Phase, $RedirectURL, $RedirectDelay)	= @_;
	my $PreferredLanguage	= (split(/,/, $ENV{'HTTP_ACCEPT_LANGUAGE'}))[0];
	my $StringResource		= new LanguageResource($PreferredLanguage, '/usr/ins/intel/gui/locale', [ 'firstboot_finalize.lrc', 'common.lrc' ]);
	my $ImageResource		= new LanguageResource($PreferredLanguage, '/usr/ins/intel/gui/locale/images');
	my $ImgPathPleaseWait	= $ImageResource->ResolveResourceFile('please_wait.gif');
    my $ApplyingChanges;
	my $PhaseTitle;
	my $PhaseMessage;
	my $Address;

	# Flush all data.
	$| = 1;
	
	# Safe defaults.
	$ImgPathPleaseWait	= '' unless(defined($ImgPathPleaseWait));

	# Get just the hidden part of the file name.
	$ImgPathPleaseWait	=~ s/^\/usr\/ins\/intel\/gui(\/locale\/images\/.*)$/$1/;

	# Get the message.
	$ApplyingChanges	= $StringResource->_S('APPLYING_CHANGES');
	$PhaseTitle			= $StringResource->_S("PHASE_LABEL_$Phase");
	$PhaseMessage		= $StringResource->_S("PHASE_MESSAGE_$Phase");
	
	# Print the 'please wait' header and body.
	print $cgi->header( -Refresh => "$RedirectDelay; URL=$RedirectURL", -expires => 'now' );
	print <<HERE;
<!doctype html public "-//IETF//DTD HTML//EN">
<html>
<head>
	<meta name=copyright content="Copyright (c) 2000 - 2001 Intel Corporation, et al.">
	<link rel=stylesheet href=/style/style.css type=text/css>
	<base target=_self>
</head>
<body bgcolor=white>
<br>
<br>
<center>
	<b>$PhaseTitle</b>&nbsp;$PhaseMessage<br>
	$ApplyingChanges<br>
    <img src=$ImgPathPleaseWait>
</center>
<br>
<br>
</body>
</html>
HERE
	
	# Close the handle.
	close(STDOUT);
}

###############################################################################
# Main
###############################################################################

sub Main
{
	my ($cgi) = @_;

	# Get the current phase.
	if (1 == $cgi->param('phase'))
	{
		my $Cookie		= new CookieData(0);
		my $CookieFile	= '/usr/ins/intel/gui/cookies/firstboot';
		
		# Attach to the cookie file...
		if (( -e $CookieFile ) &&
			(0 == $Cookie->Attach($CookieFile)))
		{
			my $PreferredLanguage		= (split(/,/, $ENV{'HTTP_ACCEPT_LANGUAGE'}))[0];
			my $ProductInfo				= new ProductInformation();
			my $DaemonWCDD				= new Daemon('wcdd');
			my $TimeData				= new Chronos($PreferredLanguage, '/usr/ins/intel/gui/locale');
			my $SSLData					= new SSL('/usr/ins/intel/apache/cert');
			my $ServerConfiguration		= new ConfigurationData();
			my $Monitor					= new MonSettings();
			my @NetworkInterfaceList	= NetConfig::listNetInterfaces();
			my %FormData				= ( 'name_host'						=> '',
											'name_domain'					=> '',
											'dns_server_address_primary'	=> '',
											'dns_server_address_secondary'	=> '',
											'chronos_time'					=> '',
											'chronos_month'					=> '',
											'chronos_date'					=> '',
											'chronos_year'					=> '',
											'chronos_timezone'				=> '',
											'chronos_ntp_active'			=> '',
											'chronos_ntp_server'			=> '',
											'name_user'						=> '',
											'user_password'					=> '',
											'user_password_confirm'			=> '',
											'ssl_active'					=> '',
											'certificate_type'				=> '',
											'group_operations'				=> '' );
			my $ServerConfigurationFile	= '/usr/ins/intel/apache/conf/httpd.conf';
			my $GroupIfModSSL			= { 'directive'	=> 'IfModule', 
											'identity'	=> 'mod_ssl.c' };
			my $RedirectAddress;
			my $RedirectDelay;
			my $ApplyingChanges;
			my $Address;
			my $Item;
			my $Key;
			
			# Run through all of the keys...
			foreach $Key (keys(%FormData))
			{
				# Fetch the data.
				$FormData{$Key} = $Cookie->GetValueKeyed($Key);
			}
	
			# Set the hostname.
			initializeHostname($FormData{'name_host'}, $FormData{'name_domain'});
	
			# Set the DNS servers.
			initializeDNS($FormData{'dns_server_address_primary'}, $FormData{'dns_server_address_secondary'}, $FormData{'name_domain'});
			
			# Set up the service monitor.
			initializeServiceMonitor();
	
			# Create the initial quota.
			createQuota();
	
			# Try to set the timezone...
			if (0 == $TimeData->SetTimeZone($FormData{'chronos_timezone'}))
			{
				# Set the NTP server.
				$TimeData->SetNTPServers([ $FormData{'chronos_ntp_server'} ], 1);
				
				# If the NTP service is requested...
				if ($FormData{'chronos_ntp_active'})
				{
					# Start the NTP service.
					$TimeData->ControlNTP(1);
				}
				else
				{
					my %CurrentTime;
		
					# Prime the time hash.
					$CurrentTime{'year'}	= $FormData{'chronos_year'};
					$CurrentTime{'month'}	= $FormData{'chronos_month'};
					$CurrentTime{'date'}	= $FormData{'chronos_date'};
					($CurrentTime{'hour'}, $CurrentTime{'minute'}, $CurrentTime{'second'}) = split(/:/, $FormData{'chronos_time'});
		
					# Set the time.
					$TimeData->SetTime(\%CurrentTime);
				}
			}
	
			# Run through the network interface list...
			for $Item (@NetworkInterfaceList)
			{
				# Look for the IP address...
				if ('eth0' eq $Item->{'fullnic'})
				{ 
					# Use this address.
					$Address = $Item->{'ipaddress'};
				}
				elsif (('eth1' eq $Item->{'fullnic'}) && (!$Address))
				{
					# Use this address.
					$Address = $Item->{'ipaddress'};
				}
			}
			
			# If SSL is active...
			if ($FormData{'ssl_active'})
			{ 
				# Set the class.
				$SSLData->SetClass($FormData{'certificate_type'});
				
				# Clean out any other certificate.
				$SSLData->SetPrivateKey();
				$SSLData->SetCertificateSigningRequest();
				$SSLData->SetCertificate();
				
				# If using a self signed certificate...
				if ('self' eq $FormData{'certificate_type'})
				{
					my @KeysSelf	= ( 'country',
										'province',
										'locality',
										'organization',
										'organizational_unit',
										'common_name',
										'e_mail' );
					my %Subject;
		
					# Run through all of the keys...
					foreach $Key (@KeysSelf)
					{
						# Transfer the data.
						$Subject{$Key} = $Cookie->GetValueKeyed($Key);
					}
		
					# Generate the private key, CSR, and certificate...
					$SSLData->SetSubject(\%Subject);
					$SSLData->GetCertificate(1);
				}
				else
				{
					my $PrivateKey	= $Cookie->GetValueKeyed('private_key');
					my $Certificate	= $Cookie->GetValueKeyed('certificate');
	
					# Clean off goo.
					$PrivateKey		=~ s/\r//g;
					$Certificate	=~ s/\r//g;
	
					# Generate the private key, CSR, and certificate...
					$SSLData->SetPrivateKey($PrivateKey);
					$SSLData->SetCertificate($Certificate);
				}
			}
	
			# Detach from the cookie.
			$Cookie->Detach();
			
			# Open the configuration.
			if (0 == $Monitor->open())
			{
				# Set the monitor states.
				$Monitor->setMonitorState('http', 1);
				$Monitor->setMonitorState('smtp', 1);
				$Monitor->setMonitorState('ftp',  1);
	
				# Close the configuration.
				$Monitor->close();
			}
			
			# Configure the server.
			configureAdministratorServer($FormData{'name_host'},
										 $FormData{'name_domain'},
										 $FormData{'name_user'});
	
			# Add the default domain.
			createDefaultDomain($FormData{'name_host'},
								$FormData{'name_domain'},
								$FormData{'name_user'}, 
								$FormData{'user_password'},
								$Address,
								(($FormData{'ssl_active'}) ? 'Yes' : ''),
								'serveradmin');
	
			# Try to open the Apache configuration file...
			if (0 == $ServerConfiguration->Attach($ServerConfigurationFile, { 'auto-save' => 1, 'mode' => O_RDWR, 'permissions' => 0600 }))
			{
				# If SSL is active...
				if ($FormData{'ssl_active'})
				{ 
					my $PrivateKey;
					my $Certificate;
	
					# If the configuration paths are available...
					if (($PrivateKey = $SSLData->GetPrivateKeyPath()) &&
						($Certificate = $SSLData->GetCertificatePath()))
					{
						# Clean out any old data.
						$ServerConfiguration->DeleteDirectiveList('SSLengine', $GroupIfModSSL);
						$ServerConfiguration->DeleteDirectiveList('SSLCertificateFile', $GroupIfModSSL);
						$ServerConfiguration->DeleteDirectiveList('SSLCertificateKeyFile', $GroupIfModSSL);
						$ServerConfiguration->DeleteDirectiveList('SSLSessionCacheTimeout', $GroupIfModSSL);
						$ServerConfiguration->DeleteDirectiveList('SSLSessionCache', $GroupIfModSSL);
						$ServerConfiguration->DeleteDirectiveList('SSLRandomSeed', $GroupIfModSSL);
						$ServerConfiguration->DeleteDirectiveList('SetEnvIf', $GroupIfModSSL);
						$ServerConfiguration->DeleteDirectiveList('SSLMutex', $GroupIfModSSL);
	
						# Set the new data.
						$ServerConfiguration->SetDirectiveList('SSLengine', [ 'on' ], $GroupIfModSSL);
						$ServerConfiguration->SetDirectiveList('SSLCertificateFile', [ $Certificate ], $GroupIfModSSL);
						$ServerConfiguration->SetDirectiveList('SSLCertificateKeyFile', [ $PrivateKey ], $GroupIfModSSL);
						$ServerConfiguration->SetDirectiveList('SSLSessionCacheTimeout', [ 300 ], $GroupIfModSSL);
						$ServerConfiguration->SetDirectiveList('SSLSessionCache', [ 'dbm:/var/log/httpd/ssl_scache' ], $GroupIfModSSL);
						$ServerConfiguration->SetDirectiveList('SSLRandomSeed', [ 'startup   file:/dev/urandom 512', 'connect   builtin' ], $GroupIfModSSL);
						$ServerConfiguration->SetDirectiveList('SetEnvIf', [ "User-Agent \".*MSIE.*\" nokeepalive ssl-unclean-shutdownSSLEnable" ], $GroupIfModSSL);
						$ServerConfiguration->SetDirectiveList('SSLMutex', [ 'file:/var/log/httpd/ssl_mutex' ], $GroupIfModSSL);
	
						# Clean out the other certificate.
						$SSLData->SetClass(('self' eq $FormData{'certificate_type'}) ? 'ca' : 'self');
						$SSLData->SetPrivateKey();
						$SSLData->SetCertificateSigningRequest();
						$SSLData->SetCertificate();
					}
				}
				else
				{
					# Clean out any old data.
					$ServerConfiguration->DeleteDirectiveList('SSLengine', $GroupIfModSSL);
					$ServerConfiguration->DeleteDirectiveList('SSLCertificateFile', $GroupIfModSSL);
					$ServerConfiguration->DeleteDirectiveList('SSLCertificateKeyFile', $GroupIfModSSL);
					$ServerConfiguration->DeleteDirectiveList('SSLSessionCacheTimeout', $GroupIfModSSL);
					$ServerConfiguration->DeleteDirectiveList('SSLSessionCache', $GroupIfModSSL);
					$ServerConfiguration->DeleteDirectiveList('SSLRandomSeed', $GroupIfModSSL);
					$ServerConfiguration->DeleteDirectiveList('SetEnvIf', $GroupIfModSSL);
					$ServerConfiguration->DeleteDirectiveList('SSLMutex', $GroupIfModSSL);
		
					# Set the new data.
					$ServerConfiguration->SetDirectiveList('SSLengine', [ 'off' ], $GroupIfModSSL);
				}
	
				# Detach from the file.
				$ServerConfiguration->Detach(1);
			}
	
			# Set up OpenSSH.
			initializeOpenSSH();
	
			# Make sure that the root mailbox is available.
			Safe::system('/bin/touch /root/mbox');
	
			# Look for a storefront configuration...
			if (2 == $ProductInfo->getClassification())
			{
				# Run the configuration utility.
				Safe::system('/home/isroot4/bin/IS4-config');

				# Realign the mail service with the appliance.
				set13xxMailService();
			}
			
			# Set the group operations state...
			$DaemonWCDD->Control((($FormData{'group_operations'}) ? 1 : 0));

			# If SSL is active...
			if ($FormData{'ssl_active'})
			{ 
				# Run through the network interface list...
				for $Item (@NetworkInterfaceList)
				{
					# Look for the IP address...
					if ('eth0' eq $Item->{'fullnic'})
					{ 
						# Use this address.
						$Address = $Item->{'ipaddress'};
					}
					elsif (('eth1' eq $Item->{'fullnic'}) && (!$Address))
					{
						# Use this address.
						$Address = $Item->{'ipaddress'};
					}
				}
	
				# SSL redirect for the summary.
				$RedirectAddress	= "https://$Address\:5555/cgi-bin/firstboot/firstboot_summary.cgi";
				$RedirectDelay		= 18;
			}
			else
			{
				# Non-SSL redirect for the summary.
				$RedirectAddress	= 'firstboot_summary.cgi';
				$RedirectDelay		= 16;
			}
			
			# Restart Apache.
			Safe::spawn_daemon('/bin/sleep 6; /etc/rc.d/init.d/httpd-admin stop; /bin/sleep 3; /etc/rc.d/init.d/httpd-admin start');
			
			# If the mark file exists...
			if ( -e '/usr/ins/intel/gui/htdocs/firstboot' )
			{
				# Try to delete the marker file.
				unless(unlink('/usr/ins/intel/gui/htdocs/firstboot'))
				{
					my $Log	= new LanguageLogCGI('/usr/ins/intel/gui/logs/server.log', '/usr/ins/intel/gui/locale', [ 'firstboot_finalize.lrc' ]);

					# Log the error.
					$Log->Append('error', 'critical', 1070000);
				 }
			}

			# Lock down the first boot directory.
			Safe::spawn_daemon('/bin/sleep 15; /bin/chmod 600 /usr/ins/intel/gui/cgi-bin/firstboot/firstboot?.cgi; /bin/chmod 600 /usr/ins/intel/gui/cgi-bin/firstboot/firstboot_finalize.cgi');
			
			# Display the initial wait message...
			DisplayWaitMessage($cgi, 2, $RedirectAddress, $RedirectDelay);
			exit(0);
		}
		else
		{
			# Redirect to the first form.
			print $cgi->redirect( -url => 'firstboot1.cgi?restart=1' );
			return;
		}
	}
	else
	{
		# Display the initial wait message...
		DisplayWaitMessage($cgi, 1, 'firstboot_finalize.cgi?phase=1', 1);
	}
}

###############################################################################
# The main body of the script.
###############################################################################

my $cgi = new CGI;

Main($cgi);

