#!/usr/bin/perl

###############################################################################
#                                                                             #
# Filename: firstboot_license.cgi                                             #
#                                                                             #
# Description: CGI needed to display the first boot license page.             #
#                                                                             #
###############################################################################
###############################################################################
#                                                                             #
#  Copyright  2000 - 2001 Intel Corporation.                                 #
#  Intel Corporation All Rights Reserved.                                     #
#                                                                             #
#  The source code contained or described herein and all documents related to #
#  the source code ("Material") are owned by Intel Corporation or its         #
#  suppliers or licensors.  Title to the Material remains with Intel          #
#  Corporation or its suppliers and licensors.  The Material contains trade   #
#  secrets and proprietary and confidential information of Intel or its       #
#  suppliers and licensors.  The Material is protected by worldwide copyright #
#  and trade secret laws and treaty provisions.  No part of the Material may  #
#  be used, copied, reproduced, modified, published, uploaded, posted,        #
#  transmitted, distributed, or disclosed in any way without Intel's prior    #
#  express written permission.                                                #
#                                                                             #
#  No license under any patent, copyright, trade secret or other intellectual #
#  property right is granted to or conferred upon you by disclosure or        #
#  delivery of the Materials, either expressly, by implication, inducement,   #
#  estoppel or otherwise. Any license under such intellectual property rights #
#  must be express and approved by Intel in writing.                          #
#                                                                             #
###############################################################################

# Put the PERL library in the path.
use lib '/usr/ins/intel/gui/perllib';

use strict;
use LanguageResource;
use DisplayMain;
use CGI;

###############################################################################
# Main
###############################################################################

sub Main
{
	my ($cgi) = @_;
	my $PreferredLanguage	= (split(/,/, $ENV{'HTTP_ACCEPT_LANGUAGE'}))[0];
	my $StringResource		= new LanguageResource($PreferredLanguage, '/usr/ins/intel/gui/locale', [ 'firstboot_license.lrc', 'common.lrc' ]);
	my $FileResource		= new LanguageResource($PreferredLanguage, '/usr/ins/intel/gui/locale');
	my $LicenseFile			= $FileResource->ResolveResourceFile('license_agreement');
	my %_R					= ( 'IDT:10100'			=> '',
								'ACKNOWLEDGEMENT'	=> '',
								'BACK'				=> '',
								'ACCEPT'			=> '',
								'DECLINE'			=> '' );
	my %Arrow;
	my $Result;
	my $Feedback;
	my $html;
	
	# If a button is pressed...
	if (defined($cgi->param('back')))
	{
		# Redirect to the previous form.
		print $cgi->redirect( -url => 'firstboot1.cgi' );
		return;
	}
	elsif (defined($cgi->param('accept')))
	{
		# If the accept checkbox is lit...
		if (defined($cgi->param('accept_license')))
		{
			# Redirect to the next form.
			print $cgi->redirect( -url => 'firstboot2.cgi?fresh=1' );
			return;
		}
		else
		{
			# Get the error string.
			$Feedback = $StringResource->_S($Result = 10100);
		}
	}
	elsif (defined($cgi->param('decline')))
	{
		# Redirect to the decline form.
		print $cgi->redirect( -url => 'firstboot_decline.cgi' );
		return;
	}

	# Get the string resources.
	$StringResource->GetStrings(\%_R);

	# Based on the user agent...
	if (-1 < (index($cgi->user_agent(), 'MSIE')))
	{
		# Set the Microsoft arrows.
		$Arrow{'left'}	=  '&larr;';
	}
	else
	{
		# Set the other arrows.
		$Arrow{'left'}	=  '<-';
	}
	
	# Get the license agreement.
	if (open(LICENSE, $LicenseFile))
	{
		# Read in the file.
		read(LICENSE, $_R{'LICENSE_AGREEMENT'}, -s $LicenseFile);

		# Close the file.
		close (LICENSE);
	}
	else
	{
		# Bail out.
		return 0;
	}

	$html = <<HERE;
<table border=0 width=550 bgcolor=white>
<tr>
	<td bgcolor=#eeeeee>
		<table border=0 width=100% bgcolor=#eeeeee>
		<tr>
			<td>&nbsp;
				<table cellspacing=1 border=0 width=100% bgcolor=#eeeeee>
				<form method=post action=firstboot_license.cgi enctype=application/x-www-form-urlencoded>
				<tr>
					<td width=532 colspan=2 align=center>&nbsp;<textarea wrap=physical readonly rows=14 name=agreement cols=56>$_R{'LICENSE_AGREEMENT'}</textarea></td>
				</tr>
				<tr>
					<td width=526 colspan=2>&nbsp;</td>
				</tr>
				<tr>
					<td width=27>&nbsp;</td>
					<td width=498><input type=checkbox name=accept_license value=ON>&nbsp;<b>$_R{'ACKNOWLEDGEMENT'}</b></td>
				</tr>
				<tr>
					<td width=526 colspan=2>&nbsp;</td>
				</tr>
				<tr>
					<td nowrap colspan=2 align=right width=532>
						<input type=submit class=clsButton value='$Arrow{'left'}&nbsp;$_R{'BACK'}' name=back>
						<input type=submit class=clsButton value='$_R{'ACCEPT'}' name=accept>
						<input type=submit class=clsButton value='$_R{'DECLINE'}' name=decline>
					</td>
				</tr>
				</form>
				</table>
			</td>
		</tr>
		</table>
	</td>
</tr>
</table>
HERE

	# Display the page.
	DisplayMain->printDocument({ 'q' => $cgi, 'html' => $html, 
								 'class' => 'firstboot1', 'id' => 10100, 'title' => $_R{'IDT:10100'}, 'margin-left' => 3,
								 'result' => $Result, 'feedback' => $Feedback });
}

###############################################################################
# The main body of the script.
###############################################################################

my $cgi = new CGI;

Main($cgi);
