#!/usr/bin/perl

###############################################################################
#                                                                             #
# Filename: firstboot_summary.cgi                                             #
#                                                                             #
# Description: CGI needed to display the first boot summary page.             #
#                                                                             #
###############################################################################
###############################################################################
#                                                                             #
#  Copyright  2000 - 2001 Intel Corporation.                                 #
#  Intel Corporation All Rights Reserved.                                     #
#                                                                             #
#  The source code contained or described herein and all documents related to #
#  the source code ("Material") are owned by Intel Corporation or its         #
#  suppliers or licensors.  Title to the Material remains with Intel          #
#  Corporation or its suppliers and licensors.  The Material contains trade   #
#  secrets and proprietary and confidential information of Intel or its       #
#  suppliers and licensors.  The Material is protected by worldwide copyright #
#  and trade secret laws and treaty provisions.  No part of the Material may  #
#  be used, copied, reproduced, modified, published, uploaded, posted,        #
#  transmitted, distributed, or disclosed in any way without Intel's prior    #
#  express written permission.                                                #
#                                                                             #
#  No license under any patent, copyright, trade secret or other intellectual #
#  property right is granted to or conferred upon you by disclosure or        #
#  delivery of the Materials, either expressly, by implication, inducement,   #
#  estoppel or otherwise. Any license under such intellectual property rights #
#  must be express and approved by Intel in writing.                          #
#                                                                             #
###############################################################################

# Put the PERL library in the path.
use lib '/usr/ins/intel/gui/perllib';

use strict;
use ProductInformation;
use LanguageResource;
use DisplayMain;
use CookieData;
use CGI;

###############################################################################
# getStorefrontSummary
###############################################################################

sub getStorefrontSummary
{
	my $ProductInfo	= new ProductInformation();
	my $Return		= '';

	# Look for a storefront configuration...
	if (2 == $ProductInfo->getClassification())
	{
		# Safety first...
		eval
		{
			require IS4Control;
			
			my $PreferredLanguage	= (split(/,/, $ENV{'HTTP_ACCEPT_LANGUAGE'}))[0];
			my $StringResource		= new LanguageResource($PreferredLanguage, '/usr/ins/intel/gui/locale', [ 'firstboot_summary.lrc', 'common.lrc' ]);
			my %_R					= ( 'TITLE_STOREFRONT'		=> '',
										'MAC_ADDRESSES'			=> '',
										'LICENSE_KEY'			=> '',
										'NUMBER_OF_STOREFRONTS'	=> '',
										'NOTE'					=> '',
										'NOTE_STOREFRONT'		=> '' );
			my $MACAddress0 		= Safe::backtick('/sbin/ifconfig -a | grep -i "Link encap:Ethernet" | head -1 | awk \'{ print $1,": ",$5 }\'');
			my $MACAddress1 		= Safe::backtick('/sbin/ifconfig -a | grep -i "Link encap:Ethernet" | tail -1 | awk \'{ print $1,": ",$5 }\'');
			my $LicenseKey			= IS4Control->licenseKey();
			my $LicensesUsed		= IS4Control->licensesInUse();
			my $LicensesTotal		= IS4Control->totalLicenses();
			my $StoreFrontCount;

			# Get the string resources.
			$StringResource->GetStrings(\%_R);
			
			# Safe default...
			if ((0 == length($LicenseKey)) || 
				('0' eq $LicenseKey))
			{
				# Safe message.
				$LicenseKey	= $StringResource->_S('NOT_AVAILABLE');
			}

			# Get the license count information...
			$StoreFrontCount = $StringResource->_S('NUMBER_OF_STOREFRONTS_FORM', [ $LicensesUsed, $LicensesTotal ]);

			$Return = <<HERE;
				<tr>
					<td colspan=2>&nbsp;</td>
				</tr>
				<tr>
					<td width=40% nowrap><b>$_R{'TITLE_STOREFRONT'}</b></td>
					<td width=60%>&nbsp;</td>
				</tr>
				<tr>
					<td width=40% nowrap>&nbsp;<b>$_R{'MAC_ADDRESSES'}</b>&nbsp;&nbsp;</td>
					<td width=60%>$MACAddress0<br>$MACAddress1</td>
				</tr>
				<tr>
					<td width=40% nowrap>&nbsp;<b>$_R{'LICENSE_KEY'}</b>&nbsp;&nbsp;</td>
					<td width=60%>$LicenseKey</td>
				</tr>
				<tr>
					<td width=40% nowrap>&nbsp;<b>$_R{'NUMBER_OF_STOREFRONTS'}</b>&nbsp;&nbsp;</td>
					<td width=60%>$StoreFrontCount</td>
				</tr>
				<tr>
					<td colspan=2><b>$_R{'NOTE'}</b>&nbsp;$_R{'NOTE_STOREFRONT'}</td>
				</tr>
HERE
		};
	}

	return $Return;
}

###############################################################################
# Main
###############################################################################

sub Main
{
	my ($cgi) = @_;
	my $PreferredLanguage	= (split(/,/, $ENV{'HTTP_ACCEPT_LANGUAGE'}))[0];
	my $StringResource		= new LanguageResource($PreferredLanguage, '/usr/ins/intel/gui/locale', [ 'firstboot_summary.lrc', 'server_date.lrc', 'server_domains.lrc', 'firstboot2.lrc', 'firstboot3.lrc', 'firstboot4.lrc',  'common.lrc' ]);
	my $Cookie				= new CookieData(0);
	my %_R					= ( 'IDT:10600'						=> '',
								'INTRODUCTION'					=> '',
								'IDT:10300'						=> '',
								'HOSTNAME'						=> '',
								'DOMAINNAME'					=> '',
								'DNS_SERVER_ADDRESS_PRIMARY'	=> '',
								'DNS_SERVER_ADDRESS_SECONDARY'	=> '',
								'NTP_STATUS'					=> '',
								'NTP_SERVER_ADDRESS'			=> '',
								'IDT:10400'						=> '',
								'ADMIN_LOGIN_NAME'				=> '',
								'USE_SSL'						=> '',
								'IDT:10500'						=> '',
								'MANAGEMENT_SOFTWARE'			=> '',
								'INSTALLED'						=> '',
								'NOT_INSTALLED'					=> '',
								'ENABLED'						=> '',
								'DISABLED'						=> '',
								'NOT_AVAILABLE'					=> '',
								'PROCEED'						=> '' );
	my %FormData				= ( 'name_host'						=> '',
									'name_domain'					=> '',
									'dns_server_address_primary'	=> '',
									'dns_server_address_secondary'	=> '',
									'chronos_ntp_active'			=> '',
									'chronos_ntp_server'			=> '',
									'name_user'						=> '',
									'ssl_active'					=> '',
									'certificate_type'				=> '',
									'group_operations'				=> '' );
	my %Arrow;
	my $CookieFile			= '/usr/ins/intel/gui/cookies/firstboot';
	my $NTPEnabled;
	my $SSLDetails;
	my $ManagementEnabled;
	my $StorefrontSummary	= getStorefrontSummary();
	my $Item;
	my $Key;
	my $htmlTitle;
	my $html;
	
	# Get the string resources.
	$StringResource->GetStrings(\%_R);

	# If there is a cookie file...
	if ( -e $CookieFile )
	{
		# Attach to the cookie file.
		$Cookie->Attach($CookieFile);

		# Run through all of the keys...
		foreach $Key (keys(%FormData))
		{
			# Fetch the data.
			$FormData{$Key} = $Cookie->GetValueKeyed($Key);
		}

		# Detach from the cookie.
		$Cookie->Detach();
	}
	
	# Get the NTP configuration state.
	$NTPEnabled = ($FormData{'chronos_ntp_active'}) ? '<font color=green>' . $_R{'ENABLED'} . '<font>' : '<font color=red>' . $_R{'DISABLED'} . '<font>';

	# Set the SSL status text...
	if ($FormData{'ssl_active'})
	{
		# Using SSL...
		$SSLDetails = '<font color=green>' . (('self' eq $FormData{'certificate_type'}) ? $StringResource->_S('SSL_CONFIGURED_SELF') : $StringResource->_S('SSL_CONFIGURED_CA')) . '<font>';
	}
	else
	{
		# Not using SSL.
		$SSLDetails = '<font color=red>' . $StringResource->_S('SSL_NOT_CONFIGURED') . '<font>';
	}

	# Get the management support configuration state.
	$ManagementEnabled = ($FormData{'group_operations'}) ? $_R{'INSTALLED'} : $_R{'NOT_INSTALLED'};
	
	# Default entries.
	$FormData{'dns_server_address_primary'}		= $_R{'NOT_AVAILABLE'} if('' eq $FormData{'dns_server_address_primary'});
	$FormData{'dns_server_address_secondary'}	= $_R{'NOT_AVAILABLE'} if('' eq $FormData{'dns_server_address_secondary'});
	$FormData{'chronos_ntp_server'}				= $_R{'NOT_AVAILABLE'} if('' eq $FormData{'chronos_ntp_server'});

	$htmlTitle = <<HERE;
	<table width=548 border=0 cellspacing=0 cellpadding=0>
	<tr>
		<td align=left><p class=clstitlepage><b>$_R{'IDT:10600'}</b></p></td>
	</tr>
	</table>
HERE

	$html = <<HERE;
<form method=post action=firstboot_summary.cgi enctype=application/x-www-form-urlencoded>
<table border=0 width=550 bgcolor=white>
<tr>
	<td>$_R{'INTRODUCTION'}</td>
</tr>
<tr>
	<td bgcolor=white>
		<table border=0 width=100% bgcolor=white>
		<tr>
			<td>
				<table border=0 cellpadding=2 cellspacing=0 bgcolor=#ffffff>
				<tr>
					<td width=40% nowrap><b>$_R{'IDT:10300'}</b></td>
					<td width=60%>&nbsp;</td>
				</tr>
				<tr>
					<td width=40% nowrap>&nbsp;<b>$_R{'HOSTNAME'}</b>&nbsp;&nbsp;</td>
					<td width=60%>$FormData{'name_host'}</td>
				</tr>
				<tr>
					<td width=40% nowrap>&nbsp;<b>$_R{'DOMAINNAME'}</b>&nbsp;&nbsp;</td>
					<td width=60%>$FormData{'name_domain'}</td>
				</tr>
				<tr>
					<td width=40% nowrap>&nbsp;<b>$_R{'DNS_SERVER_ADDRESS_PRIMARY'}</b>&nbsp;&nbsp;</td>
					<td width=60%>$FormData{'dns_server_address_primary'}</td>
				</tr>
				<tr>
					<td width=40% nowrap>&nbsp;<b>$_R{'DNS_SERVER_ADDRESS_SECONDARY'}</b>&nbsp;&nbsp;</td>
					<td width=60%>$FormData{'dns_server_address_secondary'}</td>
				</tr>
				<tr>
					<td width=40% nowrap>&nbsp;<b>$_R{'NTP_STATUS'}</b>&nbsp;&nbsp;</td>
					<td width=60%>$NTPEnabled</td>
				</tr>
				<tr>
					<td width=40% nowrap>&nbsp;<b>$_R{'NTP_SERVER_ADDRESS'}</b>&nbsp;&nbsp;</td>
					<td width=60%>$FormData{'chronos_ntp_server'}</td>
				</tr>
				<tr>
					<td colspan=2>&nbsp;</td>
				</tr>
				<tr>
					<td width=40% nowrap><b>$_R{'IDT:10400'}</b></td>
					<td width=60%>&nbsp;</td>
				</tr>
				<tr>
					<td width=40% nowrap>&nbsp;<b>$_R{'ADMIN_LOGIN_NAME'}</b>&nbsp;&nbsp;</td>
					<td width=60%>$FormData{'name_user'}</td>
				</tr>
				<tr>
					<td width=40% nowrap>&nbsp;<b>$_R{'USE_SSL'}</b>&nbsp;&nbsp;</td>
					<td width=60%>$SSLDetails</td>
				</tr>
				<tr>
					<td colspan=2>&nbsp;</td>
				</tr>
				<tr>
					<td width=40% nowrap><b>$_R{'IDT:10500'}</b></td>
					<td width=60%>&nbsp;</td>
				</tr>
				<tr>
					<td width=40% nowrap>&nbsp;<b>$_R{'MANAGEMENT_SOFTWARE'}</b>&nbsp;&nbsp;</td>
					<td width=60%>$ManagementEnabled</td>
				</tr>
$StorefrontSummary
				</table>
			</td>
		</tr>
		</table>
	</td>
</tr>
<tr>
	<td nowrap colspan=3 align=right>
		<input type=submit class=clsButton value='$_R{'PROCEED'}' name=next onclick="javascript:parent.location.href='/login/index.cgi'">
	</td>
</tr>
</table>
<form>
HERE

	# Display the page.
	DisplayMain->printDocument({ 'q' => $cgi, 'html' => $html, 
								 'title-html' => $htmlTitle, 'margin-left' => 3 });
}

###############################################################################
# The main body of the script.
###############################################################################

my $cgi = new CGI;

Main($cgi);

