#!/usr/bin/perl -w

###############################################################################
#                                                                             #
# Filename: server_backup.cgi                                                 #
#                                                                             #
# Description: CGI needed to display and modify backup/restore configurations.#
#                                                                             #
###############################################################################
###############################################################################
#                                                                             #
#  Copyright  2000 - 2001 Intel Corporation.                                 #
#  Intel Corporation All Rights Reserved.                                     #
#                                                                             #
#  The source code contained or described herein and all documents related to #
#  the source code ("Material") are owned by Intel Corporation or its         #
#  suppliers or licensors.  Title to the Material remains with Intel          #
#  Corporation or its suppliers and licensors.  The Material contains trade   #
#  secrets and proprietary and confidential information of Intel or its       #
#  suppliers and licensors.  The Material is protected by worldwide copyright #
#  and trade secret laws and treaty provisions.  No part of the Material may  #
#  be used, copied, reproduced, modified, published, uploaded, posted,        #
#  transmitted, distributed, or disclosed in any way without Intel's prior    #
#  express written permission.                                                #
#                                                                             #
#  No license under any patent, copyright, trade secret or other intellectual #
#  property right is granted to or conferred upon you by disclosure or        #
#  delivery of the Materials, either expressly, by implication, inducement,   #
#  estoppel or otherwise. Any license under such intellectual property rights #
#  must be express and approved by Intel in writing.                          #
#                                                                             #
###############################################################################

# Put the PERL library in the path.
use lib '/usr/ins/intel/gui/perllib';

use strict;
use POSIX qw(strftime);
use LanguageTime;
use LanguageResource;
use DisplayMain;
use ProductInformation;
use Daemon;
use Backup;
use Safe;
use CGI;

my $ProductInfo	= new ProductInformation();
my $AreWeCommerce;

# Look for a storefront configuration...
if (2 == $ProductInfo->getClassification())
{
	$AreWeCommerce = 1;
}
else
{
	$AreWeCommerce = 0;
}

###############################################################################
# RenderDefault
###############################################################################

sub RenderDefault
{
	my ($cgi, $DisplayDataRef) = @_;
	my $Return;

	# Check for a button press...
	if (defined($cgi->param('configure_arkeia')))
	{
		# Redirect to configure arkeia.
		$Return = 'configure_arkeia';
	}
	else
	{
		my $PreferredLanguage	= (split(/,/, $ENV{'HTTP_ACCEPT_LANGUAGE'}))[0];
		my $StringResource		= new LanguageResource($PreferredLanguage, '/usr/ins/intel/gui/locale', [ 'server_backup.lrc', 'common.lrc' ]);
		my $TimeResource		= new LanguageTime($PreferredLanguage, '/usr/ins/intel/gui/locale');
		my $eCommerceHTML;
		my $eCommerceJScript1;
		my $eCommerceJScript2;


		my $ArkeiaDaemon		= new Daemon('nlservd');
		my %_R					= ( 'IDT:2900'					=> '',
									'BACKUP'			=> '',
									'CHOOSE_FILES_TO_BACKUP'	=> '',
									'ENTIRE_SYSTEM'			=> '',
									'PARTIAL_SYSTEM'		=> '',
									'ALL_MAIL'			=> '',
									'ALL_CONFIGURATION_FILES'	=> '',
									'ALL_DOMAIN_DIRECTORIES'	=> '',
									'VIEW_FILE_LISTING'		=> '',
									'CHOOSE_BACKUP_LOCATION'	=> '',
									'LOCAL_HARD_DRIVE'		=> '',
									'SERVER_HARD_DRIVE_PATH'	=> '',
									'NOTE'				=> '',
									'DOMAIN_BACKUP_NOTE'		=> '',
									'START_BACKUP'			=> '',  	
									'ARKEIA_BACKUP_AGENT'		=> '',
									'CONFIGURE'			=> '',
									'SERVICE'			=> '',
									'STATUS'			=> '',
									'CONTROLS'			=> '',
									'SPECIFY_RESTORE_LOCATION'	=> '',
									'LOCAL_FILE_PATH'		=> '',
									'RESTORE'			=> '',
									'ERROR_EMPTY_PATH'		=> '' );

		my %FormData			= ( 'entire_system'		=> '',
									'partial_system'   	=> 'checked',
									'users_mail'	   	=> '',
									'system_config'	   	=> '',
									'user_home_dirs'   	=> 'checked',
									'web_backup'		=> 'checked',
									'server_hd_backup'	=> '',
									'web_restore'		=> 'checked',
									'server_hd_restore'	=> '' );
		
		if ($AreWeCommerce)
		{
			# Add form field variable for eCommerce display/operation.

			$_R{'ALL_INTERSHOP_FILES'} = '';
			$FormData{'intershop4'} = '';
		}
			

		my @FilesToBackup		= $cgi->param('files_to_backup');
		my $SuggestedFileName;
		my %Parameters;
		my $ArkeiaStatus;
		my $Feedback;
		my $Result;
		my $html;
		
		# Get the string resources.
		$StringResource->GetStrings(\%_R);
		
		# Get the suggested file name...
		$SuggestedFileName = $TimeResource->FormatTime($StringResource->_S('BACKUP_HOME_TIME_FORMAT'), localtime());

		# Set up the configuration parameters.
		$Parameters{'caller'}			= 'BackupServer';
		$Parameters{'q'}				= $cgi;
		$Parameters{'backup_medium'}	= $cgi->param('backup_medium');
		$Parameters{'backup_filename'}	= ($cgi->param('backup_filename')) ? $cgi->param('backup_filename') : $SuggestedFileName; 
		
		
		
		# If applying a change...
		if (defined($cgi->param('start_backup')))
		{
			# If there is file selection data...
			if (('entire_system' eq $cgi->param('system_to_backup')) ||
				(('partial_system' eq $cgi->param('system_to_backup')) &&
				 (defined($cgi->param('files_to_backup')))))
			{
				# Trim the whitespace.
				$Parameters{'backup_filename'} =~ s/^\s*(.*?)\s*$/$1/;
				
				# Validate the file name...
				if ($Parameters{'backup_filename'} =~ /^[\w|_|-|\.]+$/)
				{
					# Start the backup sequence.
					Backup->initialize_backup_sequence(\%Parameters); 
				}
				else
				{
					$Result = 300008;
				}
			}
			else
			{
				$Result = 300004;
			}
		} 
		elsif (defined($cgi->param('restore')))
		{
			# Based on the restore source...
			if ('web' eq $cgi->param('restore_source')) 
			{
				# Start the restore process.
				Backup->restore_from_web(\%Parameters);
				undef($DisplayDataRef);
				return 0;
			}
			elsif ('server_hd' eq $cgi->param('restore_source'))
			{
				my $RestoreFile = $cgi->param('restore_file');

				# Trim the whitespace.
				$RestoreFile =~ s/^\s*(.*?)\s*$/$1/;

				# Validate the file name...
				if ($RestoreFile =~ /^[\w|_|-|\.]+$/)
				{
					Backup->restore_from_server_hd(\%Parameters);
					undef($DisplayDataRef);
					return 0;
				}
				else
				{
					$Result = 300007;
				}
			}
			else 
			{
				$Result = 300006;
			}
		}
		elsif (defined($cgi->param('view_file_list')))
		{
			# If there is file selection data...
			if (('entire_system' eq $cgi->param('system_to_backup')) ||
				(('partial_system' eq $cgi->param('system_to_backup')) &&
				 (defined($cgi->param('files_to_backup')))))
			{
				# Redirect to view the file list.
				return 'view_file_list';
			}
			else
			{
				$Result = 300004;
			}
		}
		
		# If there is an action...
		if ((defined($cgi->param('start_backup'))) ||
			(defined($cgi->param('view_file_list'))))
		{
			my $FileSet;
			
			# Set up the backup file name.
			$FormData{'backup_filename'}	= (defined($cgi->param('backup_filename'))) ? $cgi->param('backup_filename') : '';
			$FormData{'restore_file'}		= $SuggestedFileName;

			# Set up the radio buttons and checkboxes.
			$FormData{'entire_system'}		= ('entire_system' eq $cgi->param('system_to_backup')) ? 'checked' : '';
			$FormData{'partial_system'} 	= ('partial_system' eq $cgi->param('system_to_backup')) ? 'checked' : '';
			$FormData{'users_mail'}					= '';
			$FormData{'system_config'}				= '';
			$FormData{'user_home_dirs'}				= '';
			if ($AreWeCommerce) { $FormData{'intershop4'}		= ''; }

			$FormData{'web_backup'}			= ('web' eq $cgi->param('backup_medium')) ? 'checked' : '';
			$FormData{'server_hd_backup'}	= ('server_hd' eq $cgi->param('backup_medium')) ? 'checked' : '';

			# Run through each of the files...
			foreach $FileSet (@FilesToBackup)
			{
				SWITCH:
				{
					# Add the file path into the file source list...
					if ('users_mail' eq $FileSet) { $FormData{'users_mail'} = 'checked'; last SWITCH; }
					if ('system_config' eq $FileSet) { $FormData{'system_config'} = 'checked'; last SWITCH; }
					if ('user_home_dirs' eq $FileSet) { $FormData{'user_home_dirs'} = 'checked'; last SWITCH; }
					if (('intershop4' eq $FileSet) && ($AreWeCommerce) )
					{ $FormData{'intershop4'} = 'checked'; last SWITCH; }
				}
			}
		}
		elsif (defined($cgi->param('restore')))
		{
			# Set up the backup file name.
			$FormData{'restore_file'}		= (defined($cgi->param('restore_file'))) ? $cgi->param('restore_file') : '';
			$FormData{'backup_filename'}	= $SuggestedFileName;
			
			# Set up the radio buttons and checkboxes.
			$FormData{'web_restore'}		= ('web' eq $cgi->param('restore_source')) ? 'checked' : '';
			$FormData{'server_hd_restore'}	= ('server_hd' eq $cgi->param('restore_source')) ? 'checked' : '';
		}
		else
		{
			# Get the suggested names.
			$FormData{'backup_filename'}	= $SuggestedFileName;
			$FormData{'restore_file'}		= $SuggestedFileName;
		}
		
		# Get the data sizes.
		Backup->calculate_disk_usage(\%Parameters);

		# Check to see if we are an eCommerce appliance.
		if ($AreWeCommerce)
		{
			# Add the proper HTML segment to the main page, to enable
			# the InterShop check-box selector.

			$eCommerceHTML = <<ECOMHTMLMARKER;
			<tr>
				<td width=1></td>
				<td width=1><input type=checkbox name=files_to_backup value=intershop4 onClick='selectCorrespondingRadio()' $FormData{'intershop4'}></td>
				<td width=268><b>$_R{'ALL_INTERSHOP_FILES'}</b></td>
				<td width=96>($Parameters{'intershop4_usage'})</td>
				<input type=hidden name=na4 value='na4'>
			</tr>

ECOMHTMLMARKER

			# Next add some JavaScript glue to help with checkbox operations.

			$eCommerceJScript1 = "|| (document.backup.files_to_backup[3].checked)";
			$eCommerceJScript2 = "document.backup.files_to_backup[3].checked = ''" . "\;" ;

		}
		else
		{
			# Clear out these variables, as they will not be necessary for a non-eCommerce appliance.
			
			$eCommerceHTML = '';
			$eCommerceJScript1 = '';
			$eCommerceJScript2 = '';
		}

		

		
		# Get the Arkeia process status...
		if (1 == ($ArkeiaStatus = $ArkeiaDaemon->GetStatus()))
		{
			# Indicate running.
			$ArkeiaStatus = '<font color=green>' . $StringResource->_S('RUNNING') . '</font>';
		}
		else
		{
			# Indicate stopped.
			$ArkeiaStatus = '<font color=red>' . $StringResource->_S('STOPPED') . '</font>';
		}
		
		# If there was an error...
		if ($Result)
		{
			# Get the error string.
			$Feedback = $StringResource->_S($Result);
		}

		$html = <<HERE;
<script src=/utilities.js language=javascript></script>
<script language=javascript>
function selectCorrespondingRadio()
{
	if (	(document.backup.files_to_backup[0].checked)
		|| (document.backup.files_to_backup[1].checked)
		|| (document.backup.files_to_backup[2].checked)
		$eCommerceJScript1
	)
	{
		document.backup.system_to_backup[1].checked = "checked";
	}
}

function deselectCheckboxes()
{
	document.backup.files_to_backup[0].checked = '';
	document.backup.files_to_backup[1].checked = '';
	document.backup.files_to_backup[2].checked = '';
	$eCommerceJScript2
}

function validateRestorePaths()
{
	var cFieldRef;
	
	if (document.restore.restore_source[0].checked)
	{ 
		cFieldRef = document.restore.upload;
	}
	else if (document.restore.restore_source[1].checked)
	{ 
		cFieldRef = document.restore.restore_file;
	}
	
	if ((cFieldRef) &&
		((isEmpty(cFieldRef.value)) || 
		 (isWhitespace(cFieldRef.value))))
	{
		return warnEmpty(cFieldRef, '$_R{'ERROR_EMPTY_PATH'}');
	}
}
</script>
<form method=post name=backup action=server_backup.cgi enctype=multipart/form-data>
	<table border=0 width=375 bgcolor=#eeeeee>
	<tr>
		<td>
			<table border=0 width=100%>
			<tr>
				<td width=359 valign=top align=left colspan=4><b>$_R{'BACKUP'}</b></td>
			</tr>
			<tr>
				<td width=359 colspan=4>$_R{'CHOOSE_FILES_TO_BACKUP'}</td>
			</tr>
			<tr>
				<td width=1><input type=radio name=system_to_backup value=entire_system onClick='deselectCheckboxes()' $FormData{'entire_system'}></td>
				<td width=228 colspan=2><b>$_R{'ENTIRE_SYSTEM'}</b></td>
				<td width=96>($Parameters{'total_drive_usage'})</td>
				<input type=hidden name=na0 value='na0'>
			</tr>
			<tr>
				<td width=1><input type=radio name=system_to_backup value=partial_system $FormData{'partial_system'}></td>
				<td width=228 colspan=2><b>$_R{'PARTIAL_SYSTEM'}</b></td>
				<td width=96></td>
			</tr>
			<tr>
				<td width=1></td>
				<td width=1><input type=checkbox name=files_to_backup value=users_mail onClick='selectCorrespondingRadio()' $FormData{'users_mail'}></td>
				<td width=268><b>$_R{'ALL_MAIL'}</b></td>
				<td width=96>($Parameters{'mail_dir_usage'})</td>
				<input type=hidden name=na1 value='na1'>
			</tr>
			<tr>
				<td width=1></td>
				<td width=1><input type=checkbox name=files_to_backup value=system_config onClick='selectCorrespondingRadio()' $FormData{'system_config'}></td>
				<td width=268><b>$_R{'ALL_CONFIGURATION_FILES'}</b></td>
				<td width=96>($Parameters{'etc_dir_usage'})</td>
				<input type=hidden name=na2 value='na2'>
			</tr>
			<tr>
				<td width=1></td>
				<td width=1><input type=checkbox name=files_to_backup value=user_home_dirs onClick='selectCorrespondingRadio()' $FormData{'user_home_dirs'}></td>
				<td width=268><b>$_R{'ALL_DOMAIN_DIRECTORIES'}</b></td>
				<td width=96>($Parameters{'home_partition_usage'})</td>
				<input type=hidden name=na3 value='na3'>
			</tr>
$eCommerceHTML			
			<tr>
				<td width=1></td>
				<td width=269 colspan=3>&nbsp;<input type=submit class=clsButton value='$_R{'VIEW_FILE_LISTING'}' name=view_file_list></td>
			</tr>
			<tr>
				<td width=359 colspan=4>&nbsp;</td>
			</tr>
			<tr>
				<td width=359 colspan=4>$_R{'CHOOSE_BACKUP_LOCATION'}</td>
			</tr>
			<tr>
				<td width=1><input type=radio value=web name=backup_medium $FormData{'web_backup'}></td>
				<td width=330 colspan=3><b>$_R{'LOCAL_HARD_DRIVE'}</b></td>
			</tr>
			<tr>
				<td width=1 valign=top><input type=radio value=server_hd name=backup_medium $FormData{'server_hd_backup'}></td>
				<td width=330 colspan=3 nowrap><b>$_R{'SERVER_HARD_DRIVE_PATH'}</b>&nbsp;<br>
				&nbsp;/home/backup/<input type=text name=backup_filename size=27 maxlength=32 value='$FormData{'backup_filename'}'></td>
			</tr>
			<tr>
				<td width=359 colspan=4></td>
			</tr>
			<tr>
				<td width=359 colspan=4><b>$_R{'NOTE'}</b>&nbsp;$_R{'DOMAIN_BACKUP_NOTE'}</td>
			</tr>
			<tr>
				<td colspan=4 align=right width=100%>
					<input type=submit class=clsButton value='$_R{'START_BACKUP'}' name=start_backup>
				</td>
            </tr>
			</table>
		</td>
	</tr>
	</table>
</form>
<form method=post action=server_backup.cgi enctype=application/x-www-form-urlencoded>
	<table border=0 width=375 bgcolor=#eeeeee>
	<tr>
		<td>
			<table border=1 cellpadding=2 cellspacing=0 width=100% bgcolor=#ffffff bordercolor=#eeeeee>
			<tr>
				<td width=40% bgcolor=#808080><b><font color=#ffffff>$_R{'SERVICE'}</font></b></td>
				<td width=15% bgcolor=#808080><b><font color=#ffffff>$_R{'STATUS'}</font></b></td>
				<td width=40% bgcolor=#808080><b><font color=#ffffff>$_R{'CONTROLS'}</font></b></td>
			</tr>
			<tr>
				<td width=40%><b>$_R{'ARKEIA_BACKUP_AGENT'}</b></td>
				<td width=15%>$ArkeiaStatus</td>
				<td width=40% align=left nowrap>
					<input type=submit class=clsButton value="$_R{'CONFIGURE'}" name=configure_arkeia>
				</td>
			</tr>
			</table>
		</td>
	</tr>
	</table>
</form>
<form method=post name=restore action=server_backup.cgi enctype=multipart/form-data>
	<table border=0 width=375 bgcolor=#eeeeee>
	<tr>
		<td>
			<table border=0 width=100%>
			<tr>
				<td width=353 colspan=2><b>$_R{'RESTORE'}</b></td>
			</tr>
			<tr>
				<td width=353 colspan=2>$_R{'SPECIFY_RESTORE_LOCATION'}</td>
			</tr>
			<tr>
				<td width=21 valign=top align=left><input type=radio value=web name=restore_source $FormData{'web_restore'}></td>
				<td width=332><b>$_R{'LOCAL_FILE_PATH'}<br> <input type=file name=upload size=27 maxlength=80 class=clsButtonfile></td>
			</tr>
			<tr>
				<td width=21 valign=top align=left><input type=radio value=server_hd name=restore_source $FormData{'server_hd_restore'}></td>
				<td width=332 nowrap><b>$_R{'SERVER_HARD_DRIVE_PATH'}</b>&nbsp;<br>
				&nbsp;/home/backup/<input type=text name=restore_file size=27 maxlength=32 value='$FormData{'restore_file'}'></td>
			</tr>
			<tr>
				<td width=353 colspan=2 align=right>&nbsp;</td>
			</tr>
			<tr>
			<td width=100% colspan=2 align=right>
				<input type=submit class=clsButton value='$_R{'RESTORE'}' name=restore onClick='return validateRestorePaths()'></td>
			</tr>
			</table>
		</td>
	</tr>
	</table>
</form>
HERE

		# Overriding result...
		$Result		= $DisplayDataRef->{'result'} unless(defined($Result));
		$Feedback	= $DisplayDataRef->{'feedback'} unless(defined($Feedback));

		# Fill out the display information.
		$DisplayDataRef->{'q'}			= $cgi;
		$DisplayDataRef->{'html'}		= $html;
		$DisplayDataRef->{'class'}		= 'server_backup';
		$DisplayDataRef->{'id'}			= 2900;
		$DisplayDataRef->{'title'}		= $_R{'IDT:2900'};
		$DisplayDataRef->{'root-link'}	= 'server_backup';
		$DisplayDataRef->{'result'}		= $Result;
		$DisplayDataRef->{'feedback'}	= $Feedback;

		$Return = 0;
	}
	
	return $Return;
}

###############################################################################
# RenderConfigureArkeia
###############################################################################

sub RenderConfigureArkeia
{
	my ($cgi, $DisplayDataRef, $Redirect) = @_;
	my $PreferredLanguage	= (split(/,/, $ENV{'HTTP_ACCEPT_LANGUAGE'}))[0];
	my $StringResource		= new LanguageResource($PreferredLanguage, '/usr/ins/intel/gui/locale', [ 'server_backup.lrc', 'common.lrc' ]);
	my $ArkeiaDaemon		= new Daemon('arkeia');
	my %_R					= ( 'IDT:3000'				=> '',
								'ARKEIA'				=> '',
								'SERVICE'				=> '',
								'STATUS'				=> '',
								'CONTROLS'	   			=> '',
								'START'		   			=> '',
								'STOP'		   			=> '',
								'CONFIRM_STOP'			=> '',
								'ARKEIA_SERVER_NAME'	=> '',
								'APPLY'					=> '' );
	my $ArkeiaServer;
	my $ArkeiaStatus;
	my $Feedback;
	my $Result;
	my $html;
	my $Return;

	# Get the string resources.
	$StringResource->GetStrings(\%_R);
	
	# Get the server configuration.
	$ArkeiaServer = Safe::backtick('/bin/cat /usr/knox/nlp/admin.cfg');
	chomp($ArkeiaServer);

	SWITCH:
	{
		# Control the service.
		if (defined($cgi->param('control_start')))	{ if ('' ne $ArkeiaServer) { $ArkeiaDaemon->Control(1) } else { $Result = 300003 }; last SWITCH; }
		if (defined($cgi->param('control_stop')))	{ $ArkeiaDaemon->Control(0); last SWITCH; }
	}

	# If adding applying a change...
	if (defined($cgi->param('apply')))
	{
		# Get the data...
		$ArkeiaServer = (defined($cgi->param('arkeia_server'))) ? $cgi->param('arkeia_server') : '';

		# Trim the whitespace.
		$ArkeiaServer =~ s/^\s*(.*?)\s*$/$1/;
		
		# Validate the data...
		if ($ArkeiaServer !~ /^(\w|-|\.|_)*$/)
		{
			# Set the return code.
			$Result = 300001;
		}
		else
		{
			# Try to open the configuration file...
			if (open (ARKEIA, '> /usr/knox/nlp/admin.cfg'))
			{
				# Try to lock the file...
				if (flock(ARKEIA, 2))
				{
					# Attach to the real daemon name.
					$ArkeiaDaemon->Attach('nlservd');
					
					# If the server is running...
					if ($ArkeiaDaemon->GetStatus())
					{
						# Attach to the psuedo daemon name.
						$ArkeiaDaemon->Attach('arkeia');

						# If the name is empty...
						if ('' ne $ArkeiaServer)
						{
							# Stop the daemon.
							$ArkeiaDaemon->Control(2);
						}
						else
						{
							# Restart the daemon.
							$ArkeiaDaemon->Control(0);
						}
					}

					# Set the data.
					print ARKEIA $ArkeiaServer;

					$Result = 0;
				}
				else
				{
					# Set the return code.
					$Result = 300005;
				}

				# Close the file.
				close(ARKEIA);
			}
			else
			{
				# Set the return code.
				$Result = 300002;
			}
		}
	}
	
	# If there was an error...
	if ($Result)
	{
		# Get the error string.
		$Feedback = $StringResource->_S($Result);
	}

	# Attach to the real daemon name.
	$ArkeiaDaemon->Attach('nlservd');

	# Get the Arkeia process status...
	if (1 == ($ArkeiaStatus = $ArkeiaDaemon->GetStatus()))
	{
		# Indicate running.
		$ArkeiaStatus = '<font color=green>' . $StringResource->_S('RUNNING') . '</font>';
	}
	else
	{
		# Indicate stopped.
		$ArkeiaStatus = '<font color=red>' . $StringResource->_S('STOPPED') . '</font>';
	}

	$html = <<HERE;
<form action=server_backup.cgi method=post enctype="application/x-www-form-urlencoded">
	<input type=hidden name=type value=configure_arkeia>
	<table border=0 width=375 bgcolor=#eeeeee>
	<tr>
		<td>&nbsp;
			<table border=1 cellpadding=2 cellspacing=0 width=100% bgcolor=#ffffff bordercolor=#eeeeee>
			<tr>
				<td width=40% bgcolor=#808080><b><font color=#ffffff>$_R{'SERVICE'}</font></b></td>
				<td width=15% bgcolor=#808080><b><font color=#ffffff>$_R{'STATUS'}</font></b></td>
				<td width=40% bgcolor=#808080><b><font color=#ffffff>$_R{'CONTROLS'}</font></b></td>
			</tr>
			<tr>
				<td width=40%><b>$_R{'ARKEIA'}</b></td>
				<td width=15%>$ArkeiaStatus</td>
				<td width=40% align=left nowrap>
					<input type=submit class=clsButton value="$_R{'START'}" name=control_start>
					<input type=submit class=clsButton value="$_R{'STOP'}" name=control_stop onclick="javascript: return confirm('$_R{'CONFIRM_STOP'}');">
				</td>
			</tr>
			</table>
			<table border=0 width=100%>
			<tr>
				<td width=100% colspan=2>&nbsp;</td>
			</tr>
			<tr>
				<td width=50%><font color=black><b>$_R{'ARKEIA_SERVER_NAME'}</b></font></td>
				<td width=50% align=right><input type=text name=arkeia_server maxlength=16 value="$ArkeiaServer"></td>
			</tr>
			<tr>
				<td width=100% colspan=2>&nbsp;</td>
			</tr>
			<tr>
				<td width=100% align=right colspan=2><input type=submit class=clsButton value='$_R{'APPLY'}' name=apply></td>
			</tr>
			</table>
		</td>
	</tr>
	</table>
</form>
HERE
		
	# Fill out the display information.
	$DisplayDataRef->{'q'}				= $cgi;
	$DisplayDataRef->{'html'}			= $html;
	$DisplayDataRef->{'class'}			= 'server_backup';
	$DisplayDataRef->{'id'}				= 3000;
	$DisplayDataRef->{'title'}			= $StringResource->_S('IDT:3000');
	$DisplayDataRef->{'root-link'}		= 'server_backup';
	$DisplayDataRef->{'helpIndicator'}	= 'configure_arkeia';
	$DisplayDataRef->{'result'}			= $Result;
	$DisplayDataRef->{'feedback'}		= $Feedback;
	$DisplayDataRef->{'cookiecrumbs'}	= [ { 'name' => $StringResource->_S('IDT:2900'), 'location' => 'server_backup.cgi' } ];
		
	$Return = 0;
	
	return $Return;
}

###############################################################################
# RenderViewFileList
###############################################################################

sub RenderViewFileList
{
	my ($cgi, $DisplayDataRef, $Redirect) = @_;
	my $PreferredLanguage	= (split(/,/, $ENV{'HTTP_ACCEPT_LANGUAGE'}))[0];
	my $StringResource		= new LanguageResource($PreferredLanguage, '/usr/ins/intel/gui/locale', [ 'server_backup.lrc', 'common.lrc' ]);
	my %_R					= ( 'IDT:2900'				=> '',
								'IDT:3100'				=> '' );
	my $TargetSystem		= $cgi->param('system_to_backup');
	my @FileSources;
	my $UserMail			= 0;
	my $File;
	my $Return;

	# Get the string resources.
	$StringResource->GetStrings(\%_R);

	# Do not use the conventional display.
	undef($DisplayDataRef);

	# Check the backup group...
	if ('partial_system' eq $TargetSystem)
	{
		my @FilesToBackup = $cgi->param('files_to_backup');
		
		# Run through each of the files...
		foreach $File (@FilesToBackup)
		{
			SWITCH:
			{
				# Add the file path into the file source list...
				if ('users_mail' eq $File) { $UserMail = 1; last SWITCH; }
				if ('system_config' eq $File) { push(@FileSources, '/etc/'); last SWITCH; }
				if ('user_home_dirs' eq $File) { push(@FileSources, '/home/domain'); last SWITCH; }
				if (('intershop4' eq $File) && ($AreWeCommerce) )
				{ push(@FileSources, '/home/isroot4'); last SWITCH; }
			}
		}
	}
	else
	{
		# Show root.
		push(@FileSources, '/')
	}

	print <<HERE;
<!doctype html public "-//IETF//DTD HTML//EN">
<html>
	<head>
		<link rel=stylesheet href=/style/style.css>
	</head>
	<script>
		selected_item = "server_backup";
		parent.navbar.selectitem("server_backup");
	</script>
<body bgcolor=white marginheight=2 marginwidth=2 leftmargin=0 topmargin=2>
	<table align=left cellspacing=0 cellpadding=0 border=0 bgcolor=white width=100%>
	<tr align=left valign=top>
		<td align=left colspan=3>
			<b><font face=arial size=1><u><a href=server_backup.cgi>&lt;&nbsp;$_R{'IDT:2900'}</a></u>&nbsp;</font></b>
			<p class=clsTitlePage>$_R{'IDT:3100'}</p>
			<tr align=left valign=top>
				<td valign=top>
				<pre><hr noshade><br>
HERE
		
	# If there are any files...
	if (scalar(@FileSources)) 
	{
		# Get the file lists.
		my $File	= Safe::pipe_read("/usr/bin/find @FileSources -printf \"%s\\t%TD %TT\\t%p\\n\"");
		my $Buffer;

		# Read in the results...
		while (sysread $File, $Buffer, 1024) 
		{
			# Output the line.
			syswrite STDOUT, $Buffer;
		}
		
		# Close the pipe.
		Safe::pipe_close($File);
	}

	# If listing user mail too...
	if ($UserMail)
	{
		my $MailBoxPathsRef			= Backup::get_user_home_dirs('system');
		my $FileModificationTime;
		my $FileSize;

		# Run through the list of paths...
		foreach (@$MailBoxPathsRef) 
		{
			# Continue if the time and size of the file is not available...
			next unless (($FileSize, $FileModificationTime) = (stat("$_/mbox"))[7,9]);

			# Format the output data.
			$FileModificationTime	= strftime("%D %T", localtime($FileModificationTime));
			$FileSize				= "0" unless ($FileSize);
			
			# Output the line.
			print STDOUT "$FileSize\t$FileModificationTime\t$_/mbox\n";
		}
	}
	
	print <<HERE;
				<hr noshade>
				</td>
			</tr>
		</td>
	</tr>
	</table>
</body>
</html>
HERE

	return $Return;
}

###############################################################################
# RenderStatus
###############################################################################

sub RenderStatus
{
	my ($cgi, $DisplayDataRef, $Redirect) = @_;
	my $PreferredLanguage	= (split(/,/, $ENV{'HTTP_ACCEPT_LANGUAGE'}))[0];
	my $StringResource		= new LanguageResource($PreferredLanguage, '/usr/ins/intel/gui/locale', [ 'server_backup.lrc', 'common.lrc' ]);
	my %_R					= ( 'IDT:2900'				=> '',
								'IDT:3200'				=> '',
								'STATUS_LABEL'			=> '',
								'FILE_NAME'				=> '',
								'FILE_SIZE'				=> '' );
	my $ProcessID			= $cgi->param('pid');
	my $Status;
	my $ArchiveName			= $cgi->param('archive');
	my $ArchiveSize			= Backup::ConvertSize((stat("/home/backup/$ArchiveName"))[7]);
	my $Return;
	my $html;

	# Get the string resources.
	$StringResource->GetStrings(\%_R);

	# Safe default.
	$ProcessID = -1 unless(defined($ProcessID));

	# If the backup process is still running...
	if ( -e "/proc/$ProcessID" )
	{
		# Set the status and refresh rate.
		$Status							= $StringResource->_S('BACKUP_IN_PROGRESS'); 
		$DisplayDataRef->{'refresh'}	= "6; URL=server_backup.cgi?type=status&archive=$ArchiveName&pid=$ProcessID";
	}
	else
	{
		# Set the status.
		$Status	= $StringResource->_S('BACKUP_COMPLETED');
	}

	$html = <<HERE;
<table border=0 cellpadding=2 cellspacing=0 width=400 bgcolor=#ffffff>
<tr>
	<td width=167>&nbsp;<b>$_R{'STATUS_LABEL'}</b></td>
	<td width=221>$Status</td>
</tr>
<tr>
	<td width=167>&nbsp;<b>$_R{'FILE_NAME'}</b></td>
	<td width=221>$ArchiveName</td>
</tr>
<tr>
	<td width=167>&nbsp;<b>$_R{'FILE_SIZE'}</b></td>
	<td width=221>$ArchiveSize</td>
</tr>
</table>
HERE
				
	# Fill out the display information.
	$DisplayDataRef->{'q'}				= $cgi;
	$DisplayDataRef->{'html'}			= $html;
	$DisplayDataRef->{'class'}			= 'server_backup';
	$DisplayDataRef->{'id'}				= 3200;
	$DisplayDataRef->{'title'}			= $StringResource->_S('IDT:3200');
	$DisplayDataRef->{'root-link'}		= 'server_backup';
	$DisplayDataRef->{'cookiecrumbs'}	= [ { 'name' => $StringResource->_S('IDT:2900'), 'location' => 'server_backup.cgi' } ];

	$Return = 0;

	return $Return;
}

###############################################################################
# Main
###############################################################################

sub Main
{
#	my $cgi			= shift;
	my ($cgi)	= @_;
	my %DisplayData;
	my $FormType	= $cgi->param('type');
	my $Redirect	= 0;

	# Safe default.
	$FormType = 'default' unless(defined($FormType));
	
	SWITCH:
	{
		if ('default'          eq $FormType) { $FormType = RenderDefault($cgi, \%DisplayData);						last SWITCH; }
		if ('configure_arkeia' eq $FormType) { $FormType = RenderConfigureArkeia($cgi, \%DisplayData, $Redirect);	last SWITCH; }
		if ('view_file_list'   eq $FormType) { $FormType = RenderViewFileList($cgi, \%DisplayData, $Redirect);		last SWITCH; }
		if ('status'           eq $FormType) { $FormType = RenderStatus($cgi, \%DisplayData, $Redirect);			last SWITCH; }
		$FormType = 0;
	}

	# If there is still some form data...
	if ($FormType)
	{
		# Set the redirect.
		$Redirect = 1;
		goto SWITCH;
	}
	elsif (defined(%DisplayData))
	{
		# Display the page.
		DisplayMain->printDocument(\%DisplayData);
	}
}

###############################################################################
# The main body of the script.
###############################################################################

my $cgi = new CGI;

Main($cgi);
