#!/usr/bin/perl

###############################################################################
#                                                                             #
# Filename: server_cpu.cgi                                                    #
#                                                                             #
# Description: Provides status information and configuration support for      #
#              the appliance CPU monitoring.                                  #
#                                                                             #
###############################################################################
###############################################################################
#                                                                             #
#  Copyright  2000 - 2001 Intel Corporation.                                 #
#  Intel Corporation All Rights Reserved.                                     #
#                                                                             #
#  The source code contained or described herein and all documents related to #
#  the source code ("Material") are owned by Intel Corporation or its         #
#  suppliers or licensors.  Title to the Material remains with Intel          #
#  Corporation or its suppliers and licensors.  The Material contains trade   #
#  secrets and proprietary and confidential information of Intel or its       #
#  suppliers and licensors.  The Material is protected by worldwide copyright #
#  and trade secret laws and treaty provisions.  No part of the Material may  #
#  be used, copied, reproduced, modified, published, uploaded, posted,        #
#  transmitted, distributed, or disclosed in any way without Intel's prior    #
#  express written permission.                                                #
#                                                                             #
#  No license under any patent, copyright, trade secret or other intellectual #
#  property right is granted to or conferred upon you by disclosure or        #
#  delivery of the Materials, either expressly, by implication, inducement,   #
#  estoppel or otherwise. Any license under such intellectual property rights #
#  must be express and approved by Intel in writing.                          #
#                                                                             #
###############################################################################

# Put the PERL library in the path.
use lib '/usr/ins/intel/gui/perllib';

use strict;
use LanguageResource;
use ConfigurationData;
use DisplayMain;
use Safe;
use CGI;

###############################################################################
# FormatFloat
###############################################################################

sub FormatFloat 
{
	my ($Float, $Precision) = @_;
	my $FormattedFloat;

	# Validate the input parameter(s)...
	if (defined($Float))
	{
		# Safe default.
		$Precision = 1 unless(defined($Precision));

		# Prime the format string.
		$Precision .= 'f';

		# Format the value.
		$FormattedFloat = sprintf("%5.$Precision", $Float);
		
		# See if the output is integer.
		$FormattedFloat = int($FormattedFloat) if($FormattedFloat =~ /\.0/);

		# Remove any leading whitespace.
		$FormattedFloat =~ s/^\s*(.*)$/$1/;
	}
	
	return $FormattedFloat;
}

###############################################################################
# GetConfigData
###############################################################################

sub GetConfigData
{
	my ($Label, @ConfigFileContents) = @_;
    my $Value;
	my $Line;

	# Loop through all of the lines in the file.
	foreach $Line (@ConfigFileContents)
	{
		# Continue if there is a comment.
		next if $Line =~ /^#/;

		# Continue if the label is not found.
		next unless $Line =~ /$Label/;
		
		# Parse out the value.
		$Value = $Line;
		$Value =~ s/.*$Label=(.*)$/$1/;
		chomp $Value;
		
		return $Value;
	}

	return '';
}

###############################################################################
# Main
###############################################################################

sub Main
{
	my ($cgi) 					= @_;
	my $PreferredLanguage		= (split(/,/, $ENV{'HTTP_ACCEPT_LANGUAGE'}))[0];
	my $StringResource			= new LanguageResource($PreferredLanguage, '/usr/ins/intel/gui/locale', [ 'server_cpu.lrc', 'server_alerts.lrc', 'server_status.lrc', 'common.lrc' ]);
	my $Configuration			= new ConfigurationData;
	my %_R						= ( 'IDT:700'					=> '',
									'USER'						=> '',
									'SYSTEM'					=> '',
									'CPU_INFORMATION'			=> '',
									'SPEED'						=> '',
									'MHZ'						=> '',
									'TYPE'						=> '',
									'INTEL'						=> '',
									'PROCESSOR'					=> '',
									'TRENDING_THRESHOLD'		=> '',
									'TRENDING_THRESHOLD_NOTE'	=> '',
									'CPU_UTILIZATION_PERCENT'	=> '',
									'DURATION'					=> '',
									'HOUR'						=> '',
									'MIN'						=> '',
									'SEC'						=> '',
									'IDT:600'					=> '',
									'CONFIGURE_ALERTS'			=> '',
									'APPLY'						=> '' );
	my $ProcessInformation		= Safe::backtick('/usr/bin/procinfo -b');
	my $CPUUser					= 0;
	my $CPUSystem				= 0;
	my @CPUInformation			= Safe::backtick('/usr/ins/intel/bin/asm/bakAsm get_info_processor');
	my $CPUSpeed				= GetConfigData('PROCESSOR_CURRENT_SPEED', @CPUInformation);
	my $CPUType					= GetConfigData('PROCESSOR_VERSION', @CPUInformation);
	my $Occurance				= $cgi->param('occurance');
	my $Duration				= $cgi->param('duration');
	my $Units					= $cgi->param('units');
	my $TextAlignment			= '';
	my $Feedback;
	my $Result;
	my $html;
	my $i;

    # Get the string resources.
	$StringResource->GetStrings(\%_R);
	
	# Set the edit control text alignment.
	$TextAlignment = "style='text-align: right'" if($ENV{'HTTP_USER_AGENT'} =~ /MSIE/);
	
	# Get the CPU utilization.
	$CPUUser	= FormatFloat($1) if($ProcessInformation =~ /user  :..................\s*(\w+\.*\w*)/g);
	$CPUSystem	= FormatFloat($1) if($ProcessInformation =~ /system:..................\s*(\w+\.*\w*)/g);

	# Safe default(s).
	$Occurance	= '' unless(defined($Occurance));
	$Duration	= '' unless(defined($Duration));
	$Units		= 2  unless(defined($Units));

	# If applying data...
	if (defined($cgi->param('apply')))
	{
		my $SetOccurance	= $Occurance + 0;
		my $SetDuration		= $Duration;

		# Validate the input parameters...
		if ((($Occurance !~ /^\+*([0-9]*)\.?([0-9]*)$/) || 
			 (!($1 || ('0' eq $1) || $2 || ('0' eq $2)))) ||
			(100 < $Occurance))
		{
			# Send back an error.
			$Result = 70001;
		}
		elsif (($Duration !~ /^\+*([0-9]*)\.?([0-9]*)$/) || 
			   (!($1 || ('0' eq $1) || $2 || ('0' eq $2))))
		{
			# Send back an error.
			$Result	= 70002;
		}
		elsif ((2 < $Units) || (0 > $Units))
		{
			# Send back an error.
			$Result	= 70003;
		}
		else
		{
			$Result = 0;
		}

		# Adjust the time to seconds...
		SWITCH:
		{
			if (0 == $Units) { $SetDuration *= 3600; last SWITCH }
			if (1 == $Units) { $SetDuration *= 60; last SWITCH }
		}

		# If things are going well.
		if (0 == $Result)
		{
			# Round off the seconds.
			$SetDuration = int($SetDuration);

			# Try to open the configuration file...
			unless ((0 == $Configuration->Attach('/etc/asm/asm.conf', { 'auto-save' => 1, 'mode' => O_RDWR, 'permissions' => 0600 })) &&
					(0 == $Configuration->SetValueKeyed('PF_CPU_UTILIZATION_TRIGGER', "$SetOccurance,$SetDuration")))
			{
				# Send back an error.
				$Result		= 70000;
				$Feedback	= $StringResource->_S($Result);
			}
			else
			{
				# Tickle the ERA.
				Safe::backtick('/usr/ins/intel/bin/asm/bakAsm set_era');
			}
		}
		else
		{
			# Get the feedback string.
			$Feedback	= $StringResource->_S($Result);
		}
	}
	else
	{
		# Try to open the configuration file...
		if (0 == $Configuration->Attach('/etc/asm/asm.conf'))
		{
			# Get the current configuration.
			($Occurance, $Duration) = split(/,/, $Configuration->GetValueKeyed('PF_CPU_UTILIZATION_TRIGGER'));
			$Units					= 2;
		}
		else
		{
			# Send back an error.
			$Result		= 70000;
			$Feedback	= $StringResource->_S($Result);
		}
	}
	
	$html = <<HERE;
<form method=post enctype="application/x-www-form-urlencoded" action=server_cpu.cgi>
	<table border=1 cellpadding=2 cellspacing=0 width=375 bgcolor=#ffffff bordercolor=#eeeeee>
	<tr>
		<td width=20%><b>$_R{'USER'}&nbsp;</b></td>
		<td width=30%>
			<table border=0 cellpadding=0 cellspacing=0 width=100%>
			<tr align=right>
				<td align=right width=100%>&nbsp;$CPUUser%&nbsp;</td>
				<td align=right>
					<table border=1 cellpadding=0 cellspacing=0 width=70>
					<tr>
						<td width=95% bgcolor=#ffffff>
							<table border=0 width=100% cellspacing=0 cellpadding=0>
							<tr>
								<td width=$CPUUser% bgcolor=#008000>&nbsp;</td>
								<td width=96%></td>
							</tr>
							</table>
						</td>
					</tr>
					</table>
				</td>
			</tr>
			</table>
		</td>
		<td width=50%>&nbsp;</td>
	</tr>
	<tr>
		<td width=20%><b>$_R{'SYSTEM'}&nbsp;</b></td>
		<td width=30%>
			<table border=0 cellpadding=0 cellspacing=0 width=100%>
			<tr align=right>
				<td align=right width=100%>&nbsp;$CPUSystem%&nbsp;</td>
				<td align=right>
					<table border=1 cellpadding=0 cellspacing=0 width=70>
					<tr>
						<td width=95% bgcolor=#ffffff>
							<table border=0 width=100% cellspacing=0 cellpadding=0>
							<tr>
								<td width=$CPUSystem% bgcolor=#008000>&nbsp;</td>
								<td width=96%></td>
							</tr>
							</table>
						</td>
					</tr>
					</table>
				</td>
			</tr>
			</table>
		</td>
		<td width=50%>&nbsp;</td>
	</tr>
	</table>
	<br><b>$_R{'CPU_INFORMATION'}</b><br>
	<table border=0 cellpadding=2 cellspacing=0 width=375 bgcolor=#FFFFFF>
	<tr>
		<td width=167><b>$_R{'SPEED'}</b></td>
		<td width=221>$CPUSpeed $_R{'MHZ'}</td>
	</tr>
	<tr>
		<td width=167><b>$_R{'TYPE'}</b></td>
		<td width=221>$_R{'INTEL'} $CPUType $_R{'PROCESSOR'}</td>
	</tr>
	</table>
	<br>
	<table border=0 width=375 bgcolor=#EEEEEE>
	<tr>
		<td>
			<table border=0 width=100%>
			<tr>
				<td colSpan=2 width=100%><b>$_R{'TRENDING_THRESHOLD'}</b></td>
			</tr>
			<tr>
				<td colSpan=2 width=100%>&nbsp;</td>
			</tr>
			<tr>
				<td colSpan=2 width=100%>$_R{'TRENDING_THRESHOLD_NOTE'}</td>
			</tr>
			<tr>
				<td colspan=2 width=100%>&nbsp;</td>
			</tr>
			<tr>
				<td width=60%><b>$_R{'CPU_UTILIZATION_PERCENT'}</b></td>
				<td width=40%><input type=text name=occurance value="$Occurance" size=4 maxlength=4 $TextAlignment>%&nbsp</td>
			</tr>
			<tr>
				<td width=222><b>$_R{'DURATION'}&nbsp;</b></td>
				<td>
					<input type=text name=duration value="$Duration" size=5 maxlength=5 $TextAlignment>
					<select size=1 name=units>
HERE

	# Run through all of the units...
	for ($i = 0; $i < 3; ++$i)
	{
		# Tack on another unit option.
		$html .= "<option value=$i";
		$html .= ' selected' if($Units == $i);
		
		# Add the correct unit string...
		SWITCH:
		{
			if (0 == $i) { $html .= ">$_R{'HOUR'}</option>\n"; last SWITCH; }
			if (1 == $i) { $html .= ">$_R{'MIN'}</option>\n"; last SWITCH; }
			if (2 == $i) { $html .= ">$_R{'SEC'}</option>\n"; last SWITCH; }
		}
	}

	$html .= <<HERE;
					</select>
				</td>
			</tr>
			<tr>
				<td width=100% align=left colspan=2>&nbsp;</td>
			</tr>
			<tr>
				<td width=100% colspan=2 align=right><input class=clsButton name=apply type=submit value="$_R{'APPLY'}"></td>
			</tr>
			</table>
		</td>
	</tr>
	</table>
	<br>
	<a href=server_alerts.cgi>Configure alerts...</a>
</form>
HERE

	# Display the page.
    DisplayMain->printDocument({ 'q' => $cgi, 'html' => $html, 
								 'cookiecrumbs' => [ { 'name' => $_R{'IDT:600'}, 'location' => 'server_status.cgi' } ],
								 'class' => 'server_cpu', 'id' => 700, 'title' => $_R{'IDT:700'},
 								 'root-link' => 'server_status', 'result' => $Result, 'feedback' => $Feedback });
}

###############################################################################
# The main body of the script.
###############################################################################

my $cgi = new CGI;

Main($cgi);
