#!/usr/bin/perl
$VERSION = "0.5";

###############################################################################
#                                                                             #
# Filename: server_dns.cgi                                                    #
#                                                                             #
# Description: Main DNS management UI page, with links to most commonly-used  #
#              DNS UI functions.                                              #
#                                                                             #
###############################################################################
###############################################################################
#                                                                             #
#  Copyright  2000 - 2001 Intel Corporation.                                 #
#  Intel Corporation All Rights Reserved.                                     #
#                                                                             #
#  The source code contained or described herein and all documents related to #
#  the source code ("Material") are owned by Intel Corporation or its         #
#  suppliers or licensors.  Title to the Material remains with Intel          #
#  Corporation or its suppliers and licensors.  The Material contains trade   #
#  secrets and proprietary and confidential information of Intel or its       #
#  suppliers and licensors.  The Material is protected by worldwide copyright #
#  and trade secret laws and treaty provisions.  No part of the Material may  #
#  be used, copied, reproduced, modified, published, uploaded, posted,        #
#  transmitted, distributed, or disclosed in any way without Intel's prior    #
#  express written permission.                                                #
#                                                                             #
#  No license under any patent, copyright, trade secret or other intellectual #
#  property right is granted to or conferred upon you by disclosure or        #
#  delivery of the Materials, either expressly, by implication, inducement,   #
#  estoppel or otherwise. Any license under such intellectual property rights #
#  must be express and approved by Intel in writing.                          #
#                                                                             #
###############################################################################

use lib '/usr/ins/intel/gui/perllib';
#use strict;
use CGI;
use DisplayMain;
use LanguageResource;
use Daemon;
use IO::File;
use DNSZone;
use DNSConfig;


########################################
my $Result;
my $Feedback;

my $PreferredLanguage		= (split(/,/, $ENV{'HTTP_ACCEPT_LANGUAGE'}))[0];
my $StringResource		= new LanguageResource($PreferredLanguage, '/usr/ins/intel/gui/locale', [ 'server_dns.lrc', 'common.lrc' ]);

my %_R = (		'IDT:8000'			=> '',
			'IDQH:8000'			=> '',

			'DNS'				=> '',
			'ZONE'				=> '',
			'DNSZONES'			=> '',

			'SERVICE'			=> '',
			'STATUS'			=> '',
			'CONTROLS'			=> '',
			'SVC_START'			=> '',
			'SVC_STOP'			=> '',
			'SVC_RESTART'			=> '',
			'CONFIRM_STOP'			=> '',

			'ADDFWD'			=> '',
			'ADDREV'			=> '',
			'DELETE'			=> '',

			'ZONENAME'			=> '',
			'ZONEADDR'			=> '',
			'REVERSEFOR'			=> '',
			'SLAVEZONEDISP'			=> '',
			'MASTER'			=> '',
			'MASTERLC'			=> '',
			'MULTIPLEHOSTS'			=> '',
			'UNAVAILABLE'			=> '',

			'RUNNING'			=> '',
			'STOPPED'			=> '',

			'ALLSELECTEDZONESMSG'		=> '',
			'NOSELECTEDZONESMSG'		=> '',
			'DEFAULT_MESSAGE'		=> '');



$StringResource->GetStrings(\%_R);

########################################


sub filter_subzones
{
	my (@z_data) = @_;
	my ($i,$j,$k);
	my ($ia,$ib,$ic);
	
	my @listdat;
	@listdat = @z_data;
	
	@listdat = sort { length($$a[0]) <=> length($$b[0]) } @listdat;

	my %found_subs;
	
	for ($i = 0; $i < scalar(@listdat); $i++)
	{
		my (@rz_soa,@rz_ns,@rz_mx,@rz_a,@rz_cname,@rz_ptr,@rz_nssub);
	
		# Skip all slave zones in this process.
		next if ( ($listdat[$i][2] ne '1') || ($listdat[$i][3] ne '') );
		
		$ia = DNSZone->zone_getinfo($listdat[$i][0],$listdat[$i][1],\@rz_soa,\@rz_ns,\@rz_mx,\@rz_a,\@rz_cname,\@rz_ptr,\@rz_nssub);

#		if ($ia ne 0)
#		{
#			$listdat[$i][0] = '';
#			next;
#		}
		
		next if (scalar(@rz_nssub) < 1);
		
		for ($j = 0; $j < scalar(@rz_nssub); $j++)
		{
			my $addchild = $rz_nssub[$j][0] . '.' . $listdat[$i][0] ;
			$found_subs{"$addchild"} = 'found';
		}
		
	}

	$i = 0;
	
	for ($i = 0; $i < scalar(@listdat); $i++)
	{
		if ($found_subs{"$listdat[$i][0]"} eq 'found')
		{
			$listdat[$i][0] = '';
		}
	}

	my @final_array;
	$j = 0;
	for ($i = 0; $i < scalar(@listdat); $i++)
	{
		if ($listdat[$i][0] ne '')
		{
			$final_array[$j][0] = $listdat[$i][0];
			$final_array[$j][1] = $listdat[$i][1];
			$final_array[$j][2] = $listdat[$i][2];
			$final_array[$j][3] = $listdat[$i][3];
			$j++;
		}

	}

	return @final_array;

}	# END sub.


########################################
sub zone_recursivedelete
{
	my ($thiszone,$isfirstcall) = @_;
	my ($i,$j,$k);
	my $Result;
	my $Feedback;

	my ($i,$ia,$ib,$ic,$id,$ie);
	my @z_info;
	
	if  ( ($isfirstcall eq '') || (!(defined($isfirstcall))) )
	{
		$isfirstcall = 1;
	}

	$ic = DNSConfig->zone_getinfo(\@z_info,$thiszone);
	
	if ($ic ne 0)
	{
		if ($isfirstcall)
		{
			$Result = 800001;
			$Feedback .= "$thiszone " . $StringResource->_S($Result) . "<BR>";
		}
		else
		{
			$Feedback .= 'UNIQ8000BAD';
		}
		next;
	}

	my (@rz_soa,@rz_ns,@rz_mx,@rz_a,@rz_cname,@rz_ptr,@rz_nssub);
	
	$id = DNSZone->zone_getinfo($z_info[0],$z_info[1],\@rz_soa,\@rz_ns,\@rz_mx,\@rz_a,\@rz_cname,\@rz_ptr,\@rz_nssub);

	if ($id ne 0)
	{
		if ($isfirstcall)
		{
			$Result = 800001;
			$Feedback .= "$thiszone " . $StringResource->_S($Result) . "<BR>";
		}
		else
		{
			$Feedback .= 'UNIQ8000BAD';
		}
		next;
	}

	my $subcount;
	$subcount = scalar(@rz_nssub);
	my $RecFeedback;
	
	if ($subcount > 0)
	{
		for ($k = 0; $k < $subcount; $k++)
		{
			my $subzt = $rz_nssub[$k][0] . '.' . $thiszone;
			my @sub_info;
			
			if ((DNSConfig->zone_getinfo(\@sub_info,$subzt)) eq 0 )
			{
				$RecFeedback .= zone_recursivedelete($subzt,0);
			}

		}	# End for loop
	}
			
	$i = DNSZone->zone_erasedata($z_info[0],$z_info[1]);
	$ia = DNSConfig->zone_del($z_info[0]);

	if ($ia eq 0)
	{
		if ($isfirstcall)
		{
			if ($RecFeedback =~ /UNIQ8000BAD/)
			{
				# Didn't work... error somewhere in the recursive process.
				$Result = 800007;
				$Feedback = $StringResource->_S($Result) . "<BR>";
			}
			else
			{
				$Result = 800002;
				$Feedback = $StringResource->_S($Result) . " $thiszone <BR>";
			}
		}
		else
		{
			$Feedback .= '';
		}
	}
	else
	{
		if ($isfirstcall)
		{
			$Result = 800008;
			$Feedback = $StringResource->_S($Result) . "<BR>";
		}
		else
		{
			$Feedback .= 'UNIQ8000BAD';
		}
	}

	return $Feedback;
}


########################################


my $PAGE = <<HERE;

<script>
function checkall(count)
{
	if(document.DNS.delall.checked == true)
	{
		for(var i=0;i<count;i++)
		{
			eval(\"document.DNS.N\" + i + \".checked='checked';\");
		}
	}
	else
	{
		for(var i=0;i<count;i++)
		{
			eval(\"document.DNS.N\" + i + \".checked='';\");
		}
	}
}

function preparetodeletezones(count)
{
	var boxcount=0;
	for(var i=0;i<count;i++)	
	{
		if (eval(\"document.DNS.N\" + i + \".checked\"))
		{
			boxcount++;
		}
	}
	
	if (0 < boxcount)
	{
		return confirm("$_R{'ALLSELECTEDZONESMSG'}");
	}
	else
	{
		alert("$_R{'NOSELECTEDZONESMSG'}");
		return false;
	}
}
</script>
<form ACTION="server_dns.cgi" NAME="DNS" METHOD="POST" ENCTYPE="application/x-www-form-urlencoded">
<table border="0" width="375" bgcolor="#EEEEEE">
  <tr>
    <td>
      &nbsp;
      <table border="1" cellpadding="2" cellspacing="0" width="100%" bgcolor="#FFFFFF" bordercolor="#EEEEEE">
        <tr>
          <td width="40%" bgcolor="#808080"><b><font color="#FFFFFF">$_R{'SERVICE'}</font></b>
          </td>
          <td width="15%" bgcolor="#808080"><b><font color="#FFFFFF">$_R{'STATUS'}</font></b>
          </td>
          <td width="40%" bgcolor="#808080"><b><font color="#FFFFFF">$_R{'CONTROLS'}</font></b>
          </td>
        </tr>
        <tr>
          <td width="40%"><b>$_R{'DNS'}</b>
          </td>
          <td width="15%">SERVERSTATUS
          </td>
          <td width="40%" align="left" nowrap>
            <input type="submit" class="clsButton" value="$_R{'SVC_START'}" name="control_start">
            <input type="submit" class="clsButton" value="$_R{'SVC_STOP'}" name="control_stop" onclick="javascript: return confirm('$_R{'CONFIRM_STOP'}');">
            <input type="submit" class="clsButton" value="$_R{'SVC_RESTART'}" name="control_restart"> 
          </td>
        </tr>
      </table>
      <table border="0" width="100%">
        <tr>
          <td width="140%" align="left" valign="bottom" colspan="4">&nbsp;</td>
        </tr>
      </table>
    </td>
  </tr>
</table>
<br>

<table border="0" width="375" bgcolor="#EEEEEE">
  <tr>
    <td>
    &nbsp;
      <table border="0" bgcolor="#EEEEEE">
        <tr>
          <td width="140%" align="left" valign="bottom" colspan="4"><b>$_R{'DNSZONES'}</b>
          </td>
        </tr>
        <tr>
          <td width="140%" align="left" valign="bottom" colspan="4"><input type="button" class="clsButton" value="$_R{'ADDFWD'}" name="ADDFW16" onclick="javascript:location='server_dns_add.cgi'">
            <input type="button" class="clsButton" value="$_R{'ADDREV'}" name="ADDRV16" onclick="javascript:location='server_dns_addreverse.cgi'">
            <input type="submit" class="clsButton" value="$_R{'DELETE'}" name="Delete" onclick=" return preparetodeletezones(ZONENUM);">
          </td>
          </tr>
          <tr>
            <td width="140%" align="left" valign="bottom" colspan="4">&nbsp;
            </td>
          </tr>
          <tr>
            <td align="center" bgcolor="#808080" width="1%"><INPUT id=delall name=delall type=checkbox onclick="javascript:checkall(ZONENUM);">
            </td>
            <td width="20%" align="left" valign="middle" bgcolor="#808080" nowrap><b><font color="#FFFFFF">$_R{'ZONENAME'}</font></b>
            </td>
            <td width="20%" align="left" valign="middle" bgcolor="#808080" nowrap><b><font color="#FFFFFF">$_R{'ZONEADDR'}</font></b>
            </td>
          </tr>
ZONES
          <tr>
            <td width="140%" align="left" colspan="4">&nbsp;</td>
          </tr>
        </table>
      </td>
    </tr>
</table>

</form>
HERE


###############################################################################
# SetPage
###############################################################################

sub SetPage
{
	my ($html, $ServiceStatus, $Zones, $NumZones) = @_;
	
	# Set the server status.
	$html =~ s/SERVERSTATUS/$ServiceStatus/s;

	# Set the zone information.
	$html =~ s/ZONES/$Zones/s;

	# Set the zone information.
	$html =~ s/ZONENUM/$NumZones/g;

	return $html;
}

###############################################################################
# The main body of the script.
###############################################################################

my $cgi	= new CGI;
my $html;
my $Status,
my $Feedback;
my @ConfigFileContents;
my $Zones;
my (@pz_soa,@pz_ns,@pz_mx,@pz_a,@pz_cname,@pz_ptr,@pz_nssub);

#Update the list of Zones
my $DNSDaemon = new Daemon('named');
my @zone_dat;
my $Zones;
my $NumZones;
my ($ia,$ib,$ic,$ic7,$id,$ie);
my ($i,$j,$k,$l);

$ia = DNSConfig->zone_list(\@zone_dat);
$NumZones = scalar(@zone_dat);

SWITCH:
{
	# Control the web daemon.
	if (defined($cgi->param('control_start')))		{ $DNSDaemon->Control(1); last SWITCH; }
	if (defined($cgi->param('control_stop')))		{ $DNSDaemon->Control(0); last SWITCH; }
	if (defined($cgi->param('control_restart')))	{ $DNSDaemon->Control(2); last SWITCH; }
}

if ($cgi->param('Delete') && $cgi->param('recordcount') )
{
	my $checkbox_count;
	$checkbox_count = $cgi->param('recordcount');

	#Delete all the selected zones
	for($j=0;$j < $checkbox_count; $j++)
	{
		if($cgi->param("N$j"))
		{
			my $thiszone;
			$thiszone = $cgi->param("N$j");
			my @z_info;
			
			$ic = DNSConfig->zone_getinfo(\@z_info,$thiszone);
			
			if ($ic ne 0)
			{
				$Result = 800001;
				$Feedback .= "$thiszone " . $StringResource->_S($Result) . "<BR>";
				next;
			}
			
			my (@pz_soa,@pz_ns,@pz_mx,@pz_a,@pz_cname,@pz_ptr,@pz_nssub);
			
			$ic = DNSZone->zone_getinfo($z_info[0],$z_info[1],\@pz_soa,\@pz_ns,\@pz_mx,\@pz_a,\@pz_cname,\@pz_ptr,\@pz_nssub);

			my $subcount;
			$subcount = scalar(@pz_nssub);
			
			if ($subcount > 0)
			{
				for ($k = 0; $k < $subcount; $k++)
				{
					my $subzt = $pz_nssub[$k][0] . '.' . $thiszone;
					my @sub_info;
					
					$ic7 = DNSConfig->zone_getinfo(\@sub_info,$subzt);
					
					if ($ic7 eq 0)
					{
						$Feedback .= zone_recursivedelete($subzt);
					}

				}	# End for loop
			}
					
			$i = DNSZone->zone_erasedata($z_info[0],$z_info[1]);
			$ia = DNSConfig->zone_del($z_info[0]);

			if ($z_info[2] eq 2)
			{
				# If we're a slave zone, don't worry about the zone_erasedata file, as the
				# data file may not exist due to being a slave.  So just assume that this
				# function call will never produce a critical error.

				$i = 0;
			}

			
			if ($i != 0 || $ia != 0)
			{
				$Result = 800003;
				$Feedback .= "<b><font color=red>" . $StringResource->_S($Result) . " $z_info[0] . </font></b><br>";
			}
			else
			{
				$Result = 0;
				$Feedback .= $StringResource->_S('800002') . " $thiszone <BR>";
			}
		}
	}

	DNSConfig->named_update('named');
}

if ($cgi->param('newzonemessage') && $cgi->param('zname'))
{
	$Result = 0;
	$Feedback .= "$_R{'ZONE'} " . $cgi->param('zname') . " " . $StringResource->_S('800004') . "<BR>";
	DNSConfig->named_update('named');
}

my @unfilteredzones;
$ia = DNSConfig->zone_list(\@unfilteredzones);

my @temp_dat_list;

@temp_dat_list = filter_subzones(@unfilteredzones);

@zone_dat_list = sort { $$a[0] cmp $$b[0] } @temp_dat_list;

$NumZones = scalar(@zone_dat_list);

for ($i = 0; $i < $NumZones; $i++)
{
	# for each zone print out the following to $Zones
	my $zonename = $zone_dat_list[$i][0];
	my $zoneip;
	my $count;

	my $isslave = $zone_dat_list[$i][2];
	
	if ($zonename =~ /.in-addr.arpa$/)
	{
		my $RevSubnetIP;
		my ($t1,$t2);
		my (@a1,@a2);
		my $asize;
		my ($iii,$jjj);

		@a1 = split('\.',$zonename);
		$asize = scalar(@a1);

		if (@a1[$asize-1] eq "arpa")
		{
			pop(@a1);
		}
		$asize = scalar(@a1);
		if (@a1[$asize-1] eq "in-addr")
		{
			pop(@a1);
		}

		$asize = scalar(@a1);

		@a2 = reverse(@a1);

		# Now @a2 contains the reversed array of @a1, which should be the correct
		# IP subnet for this reverse zone.

		$t2 = join('.',@a2);

		if ($isslave eq 2)
		{
			$zoneip = $_R{'SLAVEZONEDISP'} . ' (' . $_R{'MASTERLC'} . ' ' . $zone_dat_list[$i][3] . ') ';
		}

		else
		{
			$RevSubnetIP = $t2;
			$zoneip .= $_R{'REVERSEFOR'} . ' ' . $t2;

			for ($jjj = 0; $jjj < (4 - scalar(@a2)) ; $jjj++ )
			{
				$zoneip .= '.0';
			}
		}
	}

	else
	{
		my $j;
		my $foundit = 0;
	
		if ($isslave eq 2)
		{
			$zoneip = $_R{'SLAVEZONEDISP'} . ' (' . $_R{'MASTER'} . ' ' . $zone_dat_list[$i][3] . ') ';
		}

		else
		{
			$ib = DNSZone->zone_getinfo($zonename,$zone_dat_list[$i][1],\@pz_soa,\@pz_ns,\@pz_mx,\@pz_a,\@pz_cname,\@pz_ptr,\@pz_nssub);

			if ($ib ne 0)
			{
				if ($ib eq 218)
				{
					# Cannot open file
					
					$Result = 0;
					$Feedback .= $StringResource->_S('800006') . ' ' . $zonename . '<BR>';
					$zoneip = $_R{'UNAVAILABLE'};

				}
				else
				{
					# Other error
					
					$Result = 0;
					$Feedback .= $StringResource->_S('800005') . ' ' . $zonename . '<BR>';
					$zoneip = $_R{'UNAVAILABLE'};
				}
			}
			
			else
			{
				# Iterate through the A data structure looking for the domain
				$count = scalar(@pz_a);
				for ($j = 0; $j < $count; $j++)
				{
					if($pz_a[$j][0] eq "$zonename.")
					{
						$zoneip = $pz_a[$j][1];
						$foundit++;
					}
				}
				if ($foundit eq 0 )
				{
					$zoneip = '&nbsp; ';
				}
				if ($foundit > 1)
				{
					$zoneip = $_R{'MULTIPLEHOSTS'};
				}
			}
		}
	}

	$Zones .= "<tr><td align=center bgcolor=#FFFFFF nowrap width=1%><input type=checkbox name=N$i value=$zonename></td>";
	$Zones .= "<td width=20% align=left bgcolor=#FFFFFF nowrap><a href=server_dns_edit.cgi?zname=$zonename>$zonename</a></td>";
	$Zones .= "<td width=20% align=left bgcolor=#FFFFFF nowrap>$zoneip</td></tr>\n";
}

$Zones .= "<input type=hidden name=recordcount value=$NumZones>\n";

# Get the daemon status...
my $DNSStatus	= ($DNSDaemon->GetStatus())  ? "<font color=green>$_R{'RUNNING'}<font>" : "<font color=red>$_R{'STOPPED'}<font>";

# Set the page and display it.
$html = SetPage($PAGE, $DNSStatus, $Zones, $NumZones);

my %var;
$var{'q'}			= $cgi;
$var{'html'}			= $html;
$var{'class'}			= 'server_dns';
$var{'id'}			= 8000;
$var{'title'}			= $_R{'IDT:8000'};
$var{'root-link'}		= 'server_dns';
$var{'result'}			= $Result;
$var{'feedback'}		= $Feedback;


DisplayMain->printDocument(\%var);

exit(0);

