#!/usr/bin/perl
$VERSION = "0.5";

###############################################################################
#                                                                             #
# Filename: server_dns_add.cgi                                                #
#                                                                             #
# Description: Can create a forward-lookup DNS zone on this appliance.        #
#                                                                             #
###############################################################################
###############################################################################
#                                                                             #
#  Copyright  2000 - 2001 Intel Corporation.                                 #
#  Intel Corporation All Rights Reserved.                                     #
#                                                                             #
#  The source code contained or described herein and all documents related to #
#  the source code ("Material") are owned by Intel Corporation or its         #
#  suppliers or licensors.  Title to the Material remains with Intel          #
#  Corporation or its suppliers and licensors.  The Material contains trade   #
#  secrets and proprietary and confidential information of Intel or its       #
#  suppliers and licensors.  The Material is protected by worldwide copyright #
#  and trade secret laws and treaty provisions.  No part of the Material may  #
#  be used, copied, reproduced, modified, published, uploaded, posted,        #
#  transmitted, distributed, or disclosed in any way without Intel's prior    #
#  express written permission.                                                #
#                                                                             #
#  No license under any patent, copyright, trade secret or other intellectual #
#  property right is granted to or conferred upon you by disclosure or        #
#  delivery of the Materials, either expressly, by implication, inducement,   #
#  estoppel or otherwise. Any license under such intellectual property rights #
#  must be express and approved by Intel in writing.                          #
#                                                                             #
###############################################################################

use lib '/usr/ins/intel/gui/perllib';
use strict;
use CGI;
use DisplayMain;
use LanguageResource;
use IO::File;
use DNSZone;
use DNSConfig;
my $DNSPATH = '/home/named/var/named/';

########################################
my $cgi					= new CGI;
my $Result;
my $Feedback;
my $FocusForm;
my $FocusField;
my $UseFocus = 0;

my $PreferredLanguage		= (split(/,/, $ENV{'HTTP_ACCEPT_LANGUAGE'}))[0];
my $StringResource		= new LanguageResource($PreferredLanguage, '/usr/ins/intel/gui/locale', [ 'server_dns.lrc', 'common.lrc' ]);

my %_R = (	'IDT:8100'			=> '',
		'IDQH:8100'			=> '',
		'DNS'				=> '',
		'ZONE'				=> '',

		'DNSZONENAME'			=> '',
		'IPADDRESS'			=> '',
		'ADMINEMAIL'			=> '',
		'REFRESHAFTER'			=> '',
		'RETRYAFTER'			=> '',
		'EXPIREAFTER'			=> '',
		'MINIMUMTTL'			=> '',
		'MASTER'			=> '',
		'SLAVE'				=> '',
		'MASTERZONE'			=> '',
		'SLAVEZONE'			=> '',
		'ADD'				=> '',
		'MASTERSERVERNAME'		=> '',

		'TIME_SECS'			=> '',
		'TIME_MINS'			=> '',
		'TIME_HOUR'			=> '',
		'TIME_DAYS'			=> '',
		'TIME_WEEK'			=> '',

		'DEFAULT_MESSAGE'		=> '');



$StringResource->GetStrings(\%_R);

########################################
# Form Data Hash
# (to contain default values at first)

my %FormData = (	'zonename'		=> '',
			'zoneipaddr'		=> '',
			'adminemail'		=> '',

			'refresh'		=> '8',
			'retry'			=> '2',
			'expire'		=> '1',
			'minttl'		=> '1',
			'D1'			=> 'H',
			'D2'			=> 'H',
			'D3'			=> 'W',
			'D4'			=> 'D',
			
			'R1'			=> '1',
			'slavenetname'		=> '',
		);

########################################
			
if ( $cgi->param('addzone') )
{
	my $Key;

	# Get the form data from the form.
	foreach $Key (keys(%FormData))
	{
		# Get the data...
		$FormData{$Key} = (defined($cgi->param($Key))) ? $cgi->param($Key) : '';
	}
}

# Set up the radio state.
$FormData{'R1_master'}		= ('1' eq $FormData{'R1'}) ? 'checked' : '';
$FormData{'R1_slave'}		= ('2' eq $FormData{'R1'}) ? 'checked' : '';

# Set up the option selectors for D1.
$FormData{'D1_S'}		= ('S' eq $FormData{'D1'}) ? 'selected' : '';
$FormData{'D1_M'}		= ('M' eq $FormData{'D1'}) ? 'selected' : '';
$FormData{'D1_H'}		= ('H' eq $FormData{'D1'}) ? 'selected' : '';
$FormData{'D1_D'}		= ('D' eq $FormData{'D1'}) ? 'selected' : '';
$FormData{'D1_W'}		= ('W' eq $FormData{'D1'}) ? 'selected' : '';

# Set up the option selectors for D2.
$FormData{'D2_S'}		= ('S' eq $FormData{'D2'}) ? 'selected' : '';
$FormData{'D2_M'}		= ('M' eq $FormData{'D2'}) ? 'selected' : '';
$FormData{'D2_H'}		= ('H' eq $FormData{'D2'}) ? 'selected' : '';
$FormData{'D2_D'}		= ('D' eq $FormData{'D2'}) ? 'selected' : '';
$FormData{'D2_W'}		= ('W' eq $FormData{'D2'}) ? 'selected' : '';

# Set up the option selectors for D3.
$FormData{'D3_S'}		= ('S' eq $FormData{'D3'}) ? 'selected' : '';
$FormData{'D3_M'}		= ('M' eq $FormData{'D3'}) ? 'selected' : '';
$FormData{'D3_H'}		= ('H' eq $FormData{'D3'}) ? 'selected' : '';
$FormData{'D3_D'}		= ('D' eq $FormData{'D3'}) ? 'selected' : '';
$FormData{'D3_W'}		= ('W' eq $FormData{'D3'}) ? 'selected' : '';

# Set up the option selectors for D4.
$FormData{'D4_S'}		= ('S' eq $FormData{'D4'}) ? 'selected' : '';
$FormData{'D4_M'}		= ('M' eq $FormData{'D4'}) ? 'selected' : '';
$FormData{'D4_H'}		= ('H' eq $FormData{'D4'}) ? 'selected' : '';
$FormData{'D4_D'}		= ('D' eq $FormData{'D4'}) ? 'selected' : '';
$FormData{'D4_W'}		= ('W' eq $FormData{'D4'}) ? 'selected' : '';

########################################

my $PAGE = <<HERE;
<SCRIPT>
function disabledforslave()
{
	document.addforward.zoneipaddr.disabled=true;
	document.addforward.adminemail.disabled=true;	
	document.addforward.refresh.disabled=true;	
	document.addforward.retry.disabled=true;	
	document.addforward.expire.disabled=true;	
	document.addforward.minttl.disabled=true;	
	document.addforward.D1.disabled=true;	
	document.addforward.D2.disabled=true;	
	document.addforward.D3.disabled=true;	
	document.addforward.D4.disabled=true;	
	
}
function enableformaster()
{
	document.addforward.zoneipaddr.disabled=false;
	document.addforward.adminemail.disabled=false;	
	document.addforward.refresh.disabled=false;	
	document.addforward.retry.disabled=false;	
	document.addforward.expire.disabled=false;	
	document.addforward.minttl.disabled=false;	
	document.addforward.D1.disabled=false;	
	document.addforward.D2.disabled=false;	
	document.addforward.D3.disabled=false;	
	document.addforward.D4.disabled=false;	
	
}
</SCRIPT>
<FORM METHOD="POST" NAME=addforward ACTION="server_dns_add.cgi" ENCTYPE="application/x-www-form-urlencoded">
<INPUT TYPE="hidden" NAME="addzone" value="on">
<TABLE BORDER="0" WIDTH="375" BGCOLOR="#eeeeee">
  <TR>
    <TD>
      <TABLE BORDER="0" WIDTH="100%">
        <TR>
          <TD WIDTH="100%" colspan="3">&nbsp;
          </TD>
        </TR>
        <tr>
          <TD WIDTH="53%" colspan="2"><b>$_R{'DNSZONENAME'}:</b>
          </TD>
          <TD WIDTH="47%"><INPUT TYPE="text" NAME="zonename" value="$FormData{'zonename'}" SIZE="23" MAXLENGTH="45">
          </TD>
        </tr>
        
        <tr>
          <td width="6%" align="left" valign="bottom"><input type="radio" name="R1" value="1" $FormData{'R1_master'} onclick='enableformaster();'>
          </td>
          <td width="94%" colspan="2" align="left"><b>$_R{'MASTERZONE'}</b>
          </td>
        </tr>
        
        <tr>
          <TD WIDTH="6">&nbsp; </TD>
          <TD WIDTH="47%">&nbsp;<b>$_R{'IPADDRESS'}:</b>
          </TD>
          <TD WIDTH="47%"><INPUT TYPE="text" NAME="zoneipaddr" value="$FormData{'zoneipaddr'}" SIZE=18 MAXLENGTH=15>
          </TD>
        </tr>
        <tr>
          <TD WIDTH="6">&nbsp; </TD>
          <TD WIDTH="47%">&nbsp;<b>$_R{'ADMINEMAIL'}:</b>
          </TD>
          <TD WIDTH="47%"><INPUT TYPE="text" NAME="adminemail" value="$FormData{'adminemail'}" SIZE=23 MAXLENGTH=45>
          </TD>
        </tr>
        <tr>
          <TD WIDTH="6">&nbsp; </TD>
          <TD WIDTH="47%">&nbsp;<b>$_R{'REFRESHAFTER'}:</b>
          </TD>
          <TD WIDTH="47%"><INPUT TYPE="text" NAME="refresh" SIZE=5 MAXLENGTH=15 value="$FormData{'refresh'}">
            <select size="1" name="D1">
              <option $FormData{'D1_S'} value=S>$_R{'TIME_SECS'}</option>
              <option $FormData{'D1_M'} value=M>$_R{'TIME_MINS'}</option>
              <option $FormData{'D1_H'} value=H>$_R{'TIME_HOUR'}</option>
              <option $FormData{'D1_D'} value=D>$_R{'TIME_DAYS'}</option>
              <option $FormData{'D1_W'} value=W>$_R{'TIME_WEEK'}</option>
            </select>
          </TD>
        </tr>
        <TR>
          <TD WIDTH="6">&nbsp; </TD>
          <TD WIDTH="47%">&nbsp;<b>$_R{'RETRYAFTER'}:</b>
          </TD>
          <TD WIDTH="47%"><INPUT TYPE="text" NAME="retry"  SIZE=5 MAXLENGTH=15 value="$FormData{'retry'}">
            <select size="1" name="D2">
              <option $FormData{'D2_S'} value=S>$_R{'TIME_SECS'}</option>
              <option $FormData{'D2_M'} value=M>$_R{'TIME_MINS'}</option>
              <option $FormData{'D2_H'} value=H>$_R{'TIME_HOUR'}</option>
              <option $FormData{'D2_D'} value=D>$_R{'TIME_DAYS'}</option>
              <option $FormData{'D2_W'} value=W>$_R{'TIME_WEEK'}</option>
            </select>
          </TD>
        </TR>
        <tr>
          <TD WIDTH="6">&nbsp; </TD>
          <TD WIDTH="47%">&nbsp;<b>$_R{'EXPIREAFTER'}:</b>
          </TD>
          <TD WIDTH="47%"><INPUT TYPE="text" NAME="expire"  SIZE=5 MAXLENGTH=15 value="$FormData{'expire'}">
            <select size="1" name="D3">
              <option $FormData{'D3_S'} value=S>$_R{'TIME_SECS'}</option>
              <option $FormData{'D3_M'} value=M>$_R{'TIME_MINS'}</option>
              <option $FormData{'D3_H'} value=H>$_R{'TIME_HOUR'}</option>
              <option $FormData{'D3_D'} value=D>$_R{'TIME_DAYS'}</option>
              <option $FormData{'D3_W'} value=W>$_R{'TIME_WEEK'}</option>
            </select>
          </TD>
        </tr>
        <tr>
          <TD WIDTH="6">&nbsp; </TD>
          <TD WIDTH="47%">&nbsp;<b>$_R{'MINIMUMTTL'}:</b>
          </TD>
          <TD WIDTH="47%"><INPUT TYPE="text" NAME="minttl"  SIZE=5 MAXLENGTH=15 value="$FormData{'minttl'}">
            <select size="1" name="D4">
              <option $FormData{'D4_S'} value=S>$_R{'TIME_SECS'}</option>
              <option $FormData{'D4_M'} value=M>$_R{'TIME_MINS'}</option>
              <option $FormData{'D4_H'} value=H>$_R{'TIME_HOUR'}</option>
              <option $FormData{'D4_D'} value=D>$_R{'TIME_DAYS'}</option>
              <option $FormData{'D4_W'} value=W>$_R{'TIME_WEEK'}</option>
            </select>
          </TD>
        </tr>
        
        <tr>
          <td width="6%" align="left" valign="bottom"><input type="radio" name="R1" value="2" $FormData{'R1_slave'} onclick='disabledforslave();'>
	  </td>
	  <td width="94%" colspan="2" align="left"><b>$_R{'SLAVEZONE'}</b>
          </td>
        </tr>
        <tr>
          <TD WIDTH="6">&nbsp; </TD>
          <TD WIDTH="47%">&nbsp;<b>$_R{'MASTERSERVERNAME'}:</b>
	  </TD>
	  <TD WIDTH="47%"><input type="text" name="slavenetname" size="23" value="$FormData{'slavenetname'}">
          </td>
        </tr>        
        <TR>
          <TD WIDTH="100%" colspan="3">&nbsp;
          </TD>
        </TR>
        </tr>
        <TR>
          <TD WIDTH="100%" colspan="3">&nbsp;
          </TD>
        </TR>
        <TR>
          <TD WIDTH="100%" colspan="3" align="right">&nbsp;<input type="submit" class="clsButton" value=" $_R{'ADD'} " name="submit">
          </TD>
        </TR>
      </TABLE>
    </TD>
  </TR>
</TABLE>
</FORM>
HERE

###############################################################################
# SetPage
###############################################################################

sub SetPage
{
	my ($html) = @_;
	
	# Set the server status.
	#$html =~ s/SERVERSTATUS/$ServiceStatus/s;

	return $html;
}

###############################################################################
# InvalidIP
###############################################################################

sub InvalidIP
{
	my ($ipaddress,$useless) = @_;
	
	my ($ipA,$ipB,$ipC,$ipD);
	my ($t1,$t2);
	my $tempIP = $ipaddress;

	my $chkval = ( $tempIP =~ /(\d{1,3})(\.)(\d{1,3})(\.)(\d{1,3})(\.)(\d{1,3})/ );

	if (!($chkval) )
	{
		# If the string does not match the approved format
		return 1;  # false
	}
	elsif ( ($` ne '') || ($' ne '') || ($& eq '') )
	{
		return 1;
		# Matched, but there's extra data before or after the string.
	}
	else
	{
		($ipA,$ipB,$ipC,$ipD) = ( $tempIP =~ m/(\d{1,3})\.(\d{1,3})\.(\d{1,3})\.(\d{1,3})/ );
		
		if ( ($ipA < 0) || ($ipA > 255) )
		{
			return 1;
		}
		elsif ( ($ipB < 0) || ($ipB > 255) )
		{
			return 1;
		}
		elsif ( ($ipC < 0) || ($ipC > 255) )
		{
			return 1;
		}
		elsif ( ($ipD < 0) || ($ipD > 255) )
		{
			return 1;
		}
		else
		{
			return 0;
		}

	}

}

###############################################################################
# InvalidTime
###############################################################################

sub InvalidTime
{
	my ($tval1,$useless) = @_;

	# Begin checking for the time values.
	# Valid include:
	# 1, 1S1, 1M1, 1H1, 1D1, 1W1, 1S1M1H1D1W1

	if ($tval1 =~ /^(((\d+)[SsMmHhDdWw])*)(\d+)$/ )
	{
		# Match value?
		return 0;
	}
	else
	{
		return 1;
	}

}


###############################################################################
# The main body of the script.
###############################################################################

my $html				= $PAGE;
my $fn_out1;
my $mymaster = $cgi->param('R1');
my $mymasterlen = length($cgi->param('slavenetname'));
my (@pz_soa,@pz_ns,@pz_mx,@pz_a,@pz_cname,@pz_ptr,@pz_nssub);
my ($ia,$ib,$ic,$ic1,$ic2);
my $filename;


# If applying a change...
if ($cgi->param('submit') && $cgi->param('addzone'))
{
	if ( $cgi->param('zonename') eq '' )
	{
		$Result = 810006;
		$Feedback = '<b><font color="red">' . $StringResource->_S($Result) . '</font></b>';
		
		$UseFocus = 1;
		$FocusForm = 'addforward';
		$FocusField = 'zonename';
		goto DISPLAYMAINFORM;
	}

	my $zonename = lc($cgi->param('zonename'));
	my $slavenetname = lc($cgi->param('slavenetname'));
	
	#Check to see is Master or Slave is checked and check to see if the Slaves Primary Master is set
	if ( ( ($mymaster eq '2') && ($mymasterlen) ) || ($mymaster eq '1') )
	{
		# Pass the form values into DNSConfig->addzone to create the zone information in the named.conf
		if ($mymaster eq '2')
		{
			$ia = DNSConfig->zone_add($zonename, \$fn_out1, 2, $slavenetname );
		}
		else
		{
			# Now go through and test for missing fields.
			
			if ($cgi->param('zoneipaddr') eq '')
			{
				$Result = 810005;
				$FocusField = 'zoneipaddr';
			}
			
			elsif ($cgi->param('adminemail') eq '')
			{
				$Result = 810005;
				$FocusField = 'adminemail';
			}
			
			elsif ($cgi->param('refresh') eq '')
			{
				$Result = 810005;
				$FocusField = 'refresh';
			}
			elsif (InvalidTime($cgi->param('refresh')))
			{
				$Result = 810019;
				$FocusField = 'refresh';
			}
			
			elsif ($cgi->param('retry') eq '')
			{
				$Result = 810005;
				$FocusField = 'retry';
			}
			elsif (InvalidTime($cgi->param('retry')))
			{
				$Result = 810019;
				$FocusField = 'retry';
			}
			
			elsif ($cgi->param('expire') eq '')
			{
				$Result = 810005;
				$FocusField = 'expire';
			}
			elsif (InvalidTime($cgi->param('expire')))
			{
				$Result = 810019;
				$FocusField = 'expire';
			}
			
			elsif ($cgi->param('minttl') eq '')
			{
				$Result = 810005;
				$FocusField = 'minttl';
			}
			elsif (InvalidTime($cgi->param('minttl')))
			{
				$Result = 810019;
				$FocusField = 'minttl';
			}
			
			if ( ($Result eq 810005) || ($Result eq 810019) )
			{
				$Feedback = '<b><font color="red">' . $StringResource->_S($Result) . '</font></b>';
				$UseFocus = 1;
				$FocusForm = 'addforward';
				goto DISPLAYMAINFORM;
			}

			if  (InvalidIP($cgi->param('zoneipaddr')))
			{
				$Result = 810009;
				$Feedback = '<b><font color="red">' . $StringResource->_S($Result) . '</font></b>';
				
				$UseFocus = 1;
				$FocusForm = 'addforward';
				$FocusField = 'zoneipaddr';
				goto DISPLAYMAINFORM;
			}

			if ($cgi->param('adminemail') !~ /^(\w|-|_|\+|%)+(\.(\w|-|_|\+|%)+)*@(\w|-|_)+(\.(\w|-|_)+)*$/)
			{
				# Invalid e-mail address!

				$Result = 810010;
				$Feedback = '<b><font color="red">' . $StringResource->_S($Result) . '</font></b>';
				
				$UseFocus = 1;
				$FocusForm = 'addforward';
				$FocusField = 'adminemail';
				goto DISPLAYMAINFORM;
			}

						
			$ia = DNSConfig->zone_add($zonename, \$fn_out1, 1);
			
			if ($ia eq 0)
			{
				$ib = DNSZone->zone_construct_fwd($zonename . ".",$fn_out1,$cgi->param('zoneipaddr'));
				
				if($ib eq 0)
				{
					$ic1 = DNSZone->zone_getinfo($zonename,$fn_out1,\@pz_soa,\@pz_ns,\@pz_mx,\@pz_a,\@pz_cname,\@pz_ptr,\@pz_nssub);
					$pz_soa[2] = $cgi->param('adminemail');
					$pz_soa[4] = $cgi->param('refresh') . $cgi->param('D1');
					$pz_soa[5] = $cgi->param('retry') . $cgi->param('D2');
					$pz_soa[6] = $cgi->param('expire') . $cgi->param('D3');
					$pz_soa[7] = $cgi->param('minttl') . $cgi->param('D4');

					$ic2 = DNSZone->zone_changeheader($zonename,$fn_out1,\@pz_soa);
				}
			}
		}

		if ($ia != 0 || $ib != 0)
		{
			# Unable to add or construct zone.
			$Result = 810002;
			$Feedback = '<b><font color="red">' . $StringResource->_S($Result) . '</font></b>';
		}
		elsif ($ic1 != 0)
		{
			# Unable to get info for newly created zone file.
			$Result = 810003;
			$Feedback = '<b><font color="red">' . $StringResource->_S($Result) . '</font></b>';
		}
		elsif ($ic2 != 0)
		{
			# Unable to set header info for newly created zone file.
			$Result = 810004;
			$Feedback = '<b><font color="red">' . $StringResource->_S($Result) . '</font></b>';
		}
		else
		{
			$cgi->redirect("server_dns.cgi?newzonemessage=1&zname=" . $zonename);
			exit(0);
		}	
	}
	
	else
	{
		$Result = 810001;
		$Feedback = '<b><font color="red">' . $StringResource->_S($Result) . '</font></b>';
		
		$UseFocus = 1;
		$FocusForm = 'addforward';
		$FocusField = 'slavenetname';
	}
}

DISPLAYMAINFORM:

my %var;
$var{'q'}			= $cgi;
$var{'html'}			= $html;
$var{'class'}			= 'server_dns';
$var{'id'}			= 8100;
$var{'title'}			= $_R{'IDT:8100'};
$var{'root-link'}		= 'server_dns';
$var{'result'}			= $Result;
$var{'feedback'}		= $Feedback;
$var{'cookiecrumbs'}		= [{name=>$StringResource->_S('IDT:8000'), location=>"server_dns.cgi"},];

if ($UseFocus)
{
	$var{'focus-form'}	= $FocusForm;
	$var{'focus-field'}	= $FocusField;
}

DisplayMain->printDocument(\%var);

exit(0);

