#!/usr/bin/perl
$VERSION = "0.5";

###############################################################################
#                                                                             #
# Filename: server_dns_addrecord_PTR.cgi                                      #
#                                                                             #
# Description: Creates PTR records (reverse-lookup DNS only) for a zone on    #
#              this appliance.                                                #
#                                                                             #
###############################################################################
###############################################################################
#                                                                             #
#  Copyright  2000 - 2001 Intel Corporation.                                 #
#  Intel Corporation All Rights Reserved.                                     #
#                                                                             #
#  The source code contained or described herein and all documents related to #
#  the source code ("Material") are owned by Intel Corporation or its         #
#  suppliers or licensors.  Title to the Material remains with Intel          #
#  Corporation or its suppliers and licensors.  The Material contains trade   #
#  secrets and proprietary and confidential information of Intel or its       #
#  suppliers and licensors.  The Material is protected by worldwide copyright #
#  and trade secret laws and treaty provisions.  No part of the Material may  #
#  be used, copied, reproduced, modified, published, uploaded, posted,        #
#  transmitted, distributed, or disclosed in any way without Intel's prior    #
#  express written permission.                                                #
#                                                                             #
#  No license under any patent, copyright, trade secret or other intellectual #
#  property right is granted to or conferred upon you by disclosure or        #
#  delivery of the Materials, either expressly, by implication, inducement,   #
#  estoppel or otherwise. Any license under such intellectual property rights #
#  must be express and approved by Intel in writing.                          #
#                                                                             #
###############################################################################

use lib '/usr/ins/intel/gui/perllib';
use strict;
use CGI;
use DisplayMain;
use LanguageResource;
use DNSZone;
use DNSConfig;

########################################
my $cgi					= new CGI;
my $Result;
my $Feedback;
my $FocusForm;
my $FocusField;
my $UseFocus = 0;

my $PreferredLanguage		= (split(/,/, $ENV{'HTTP_ACCEPT_LANGUAGE'}))[0];
my $StringResource		= new LanguageResource($PreferredLanguage, '/usr/ins/intel/gui/locale', [ 'server_dns.lrc', 'common.lrc' ]);

my %_R = (	'IDT:9300'			=> '',
		'IDQH:9300'			=> '',
		'DNS'				=> '',
		'ZONE'				=> '',

		'HOSTNAME'			=> '',
		'IPADDR'			=> '',
		'ADD'				=> '',

		'DEFAULT_MESSAGE'		=> '');


$StringResource->GetStrings(\%_R);

########################################
# Form Data Hash
# (to contain default values at first)

my %FormData = (	'realdname'		=> '',
			'ipaddrrev'		=> '',
		);

########################################
			
if ( $cgi->param('addrecord') )
{
	my $Key;

	# Get the form data from the form.
	foreach $Key (keys(%FormData))
	{
		# Get the data...
		$FormData{$Key} = (defined($cgi->param($Key))) ? $cgi->param($Key) : '';
	}
}


########################################

my $PAGE = <<HERE;
<FORM ACTION="server_dns_addrecord_PTR.cgi" name=addrec METHOD="POST" ENCTYPE="application/x-www-form-urlencoded">
<INPUT TYPE="hidden" NAME="zname" VALUE="DOMAINNAME">
<INPUT TYPE="hidden" NAME="addrecord" VALUE="1">

<TABLE BORDER="0" WIDTH="375" BGCOLOR="#eeeeee">
  <tr>
    <TD colspan="2">&nbsp;
    </TD>
  </tr>
  <tr>
    <TD WIDTH="50%"><b>$_R{'IPADDR'}:</b>
    </TD>
    <TD WIDTH="50%"><B>SUBNETIP.</B><INPUT TYPE="text" NAME="ipaddrrev" VALUE="$FormData{'ipaddrrev'}" SIZE=5 MAXLENGTH=15>
    </TD>
  </tr>
  <tr>
    <TD WIDTH="50%"><b>$_R{'HOSTNAME'}:</b>
    </TD>
    <TD WIDTH="50%"><INPUT TYPE="text" NAME="realdname" VALUE="$FormData{'realdname'}" SIZE=23 MAXLENGTH=48>
    </TD>
  </tr>
  <TR>
    <TD WIDTH="100%" colspan="2" align="right">&nbsp;
    </TD>
  </TR>
  <TR>
    <TD WIDTH="100%" colspan="2" align="right">&nbsp;<input type="submit" class="clsButton" value=" $_R{'ADD'} " name="Add">
    </TD>
  </TR>
</TABLE>

</FORM>
HERE

###############################################################################
# SetPage
###############################################################################

sub SetPage
{
	my ($html, $domainname, $subnetip) = @_;
	
	# Set the community information.
	$html =~ s/DOMAINNAME/$domainname/g;

	$html =~ s/SUBNETIP/$subnetip/g;

	return $html;
}


###############################################################################
# InvalidIP
###############################################################################

sub InvalidIP
{
	my ($ipaddress,$useless) = @_;
	
	my ($ipA,$ipB,$ipC,$ipD);
	my ($t1,$t2);
	my $tempIP = $ipaddress;

	my $chkval = ( $tempIP =~ /(\d{1,3})(\.)(\d{1,3})(\.)(\d{1,3})(\.)(\d{1,3})/ );

	if (!($chkval) )
	{
		# If the string does not match the approved format
		return 1;  # false
	}
	elsif ( ($` ne '') || ($' ne '') || ($& eq '') )
	{
		return 1;
		# Matched, but there's extra data before or after the string.
	}
	else
	{
		($ipA,$ipB,$ipC,$ipD) = ( $tempIP =~ m/(\d{1,3})\.(\d{1,3})\.(\d{1,3})\.(\d{1,3})/ );
		
		if ( ($ipA < 0) || ($ipA > 255) )
		{
			return 1;
		}
		elsif ( ($ipB < 0) || ($ipB > 255) )
		{
			return 1;
		}
		elsif ( ($ipC < 0) || ($ipC > 255) )
		{
			return 1;
		}
		elsif ( ($ipD < 0) || ($ipD > 255) )
		{
			return 1;
		}
		else
		{
			return 0;
		}

	}

}

###############################################################################
# InvalidFQDN
###############################################################################

sub InvalidFQDN
{
	my ($thisfqdn,$useless) = @_;
	
	# Look for valid fqdn host names.
	
	if ($thisfqdn =~ /_/)
	{
		return 1;
	}
	
	if ($thisfqdn =~ /^(\w|-)+(\.((\w|-)+))*$/ )
	{
		return 0;  
	}
	
	else
	{
		return 1;

	}

}

###############################################################################
# InvalidHost
###############################################################################

sub InvalidHost
{
	my ($thishost,$useless) = @_;
	
	# Look for valid host names only.
	
	if ($thishost =~ /_/)
	{
		return 1;
	}
	
	if ($thishost =~ /^(\w|-)+$/ )
	{
		return 0;  
	}
	
	else
	{
		return 1;

	}

}

###############################################################################
# The main body of the script.
###############################################################################

my $html;
my $ia;
my $ib;
my $i;
my $NumZones;

my $zonename = $cgi->param('zname');        

# If applying a change...
if ($cgi->param('Add'))
{
	my @zone_dat_list;

	$ia = DNSConfig->zone_list(\@zone_dat_list);
	if ($ia ne 0)
	{
		$Result = 930002;
		$Feedback = '<b><font color="red">' . $StringResource->_S($Result) . '</font></b>';
		goto DISPLAYMAINFORM;
	}

	$NumZones = scalar(@zone_dat_list);

	# Iterate through data structure and display meaningful information.
	for ($i = 0; $i < $NumZones; $i++)
	{
		# for each zone print out the records
		if($zone_dat_list[$i][0] eq $zonename)
		{
			my $realdname = lc($cgi->param('realdname'));
			my $realdomname = $realdname . '.';

			my $fwd_host = lc($cgi->param('ipaddrrev'));

			if ($fwd_host eq '')
			{
				$Result = 930001;
				$Feedback = '<b><font color="red">' . $StringResource->_S($Result) . '</font></b>';
					
				$UseFocus = 1;
				$FocusForm = 'addrec';
				$FocusField = 'ipaddrrev';
				goto DISPLAYMAINFORM;
			}
			
			if ($realdname eq '')
			{
				$Result = 930001;
				$Feedback = '<b><font color="red">' . $StringResource->_S($Result) . '</font></b>';
				
				$UseFocus = 1;
				$FocusForm = 'addrec';
				$FocusField = 'realdname';
				goto DISPLAYMAINFORM;
			}
			
			if (InvalidFQDN($realdname))
			{
				$Result = 930010;
				$Feedback = '<b><font color="red">' . $StringResource->_S($Result) . '</font></b>';
				
				$UseFocus = 1;
				$FocusForm = 'addrec';
				$FocusField = 'realdname';
				goto DISPLAYMAINFORM;
			}
			
			my (@a1,@a2);
			my $rev_host;
			
			@a1 = split('\.',$fwd_host);
			@a2 = reverse(@a1);
			$rev_host = join('.',@a2);

			# Now do a consistency check on the new record data, to check if it is a valid IP address.

			@a1 = split('\.',$zonename);

			my ($t2,$asize);

			$asize = scalar(@a1);

			if (@a1[$asize-1] eq "arpa")
			{
				pop(@a1);
			}

			$asize = scalar(@a1);

			if (@a1[$asize-1] eq "in-addr")
			{
				pop(@a1);
			}

			$asize = scalar(@a1);

			@a2 = reverse(@a1);

			$t2 = join('.',@a2);

			my $fwd_domain = $t2;

			# variable $fwd_domain now contains the corrected (forward format) zone IP subnet
			# variable $fwd_host contains the corrected zone IP host portion
			# Run a check on the unholy marriage of these two variables.

			if (InvalidIP($fwd_domain . '.' . $fwd_host))
			{
				$Result = 930009;
				$Feedback = '<b><font color="red">' . $StringResource->_S($Result) . " " . $fwd_domain . '.' . $fwd_host . '</font></b>';
					
				$UseFocus = 1;
				$FocusForm = 'addrec';
				$FocusField = 'ipaddrrev';
				goto DISPLAYMAINFORM;
			}
			
			
			$ib  = DNSZone->zone_addrecord($zonename,$zone_dat_list[$i][1],"PTR",$rev_host,$realdomname);

			if( $ib eq 0 )
			{
				DNSZone->zone_increment($zonename,$zone_dat_list[$i][1]);
				print $cgi->redirect("server_dns_edit.cgi?zname=$zonename&newrecordmessage=1");  
				exit(0);
			}
			else 
			{
				$Result = (($ib eq 215) ? 930004 : 930003);
				$Feedback = '<b><font color="red">' . $StringResource->_S($Result) . '</font></b>';
				goto DISPLAYMAINFORM;
			}
		}
	}
}

DISPLAYMAINFORM:

my $iszonereverse = 0;
if ($zonename =~ /.in-addr.arpa$/)
{
	# This should always execute.
	$iszonereverse = 1;
}

my $RevSubnetIP;

if ($iszonereverse eq 1)
{
	my ($t1,$t2);
	my (@a1,@a2);
	my $asize;
	my $iii;
	
	@a1 = split('\.',$zonename);
	$asize = scalar(@a1);

	if (@a1[$asize-1] eq "arpa")
	{
		pop(@a1);
	}
	$asize = scalar(@a1);
	if (@a1[$asize-1] eq "in-addr")
	{
		pop(@a1);
	}
	
	$asize = scalar(@a1);

	@a2 = reverse(@a1);

	# Now @a2 contains the reversed array of @a1, which should be the correct
	# IP subnet for this reverse zone.
	
	$t2 = join('.',@a2);
	
	$RevSubnetIP = $t2;
}


# Set the page and display it.
$html = SetPage($PAGE, $cgi->param('zname'),$RevSubnetIP);
my $domainname = $cgi->param('zname');

my %var;
$var{'q'}			= $cgi;
$var{'html'}			= $html;
$var{'class'}			= 'server_dns';
$var{'id'}			= 9300;
$var{'title'}			= $_R{'IDT:9300'};
$var{'root-link'}		= 'server_dns';
$var{'result'}			= $Result;
$var{'feedback'}		= $Feedback;
$var{'cookiecrumbs'} =	[	{name=>$StringResource->_S('IDT:8000'), location=>"server_dns.cgi"},
				{name=>$StringResource->_S('IDT:8300'), location=>"server_dns_edit.cgi?zname=$domainname"},
			];

if ($UseFocus)
{
	$var{'focus-form'}	= $FocusForm;
	$var{'focus-field'}	= $FocusField;
}

DisplayMain->printDocument(\%var);

exit(0);
