#!/usr/bin/perl
$VERSION = "0.5";

###############################################################################
#                                                                             #
# Filename: server_dns_assign.cgi                                             #
#                                                                             #
# Description: Can create delegations for subzones hosted on another DNS      #
#              server, based on address.  Called from server_dns_edit.cgi.    #
#                                                                             #
###############################################################################
###############################################################################
#                                                                             #
#  Copyright  2000 - 2001 Intel Corporation.                                 #
#  Intel Corporation All Rights Reserved.                                     #
#                                                                             #
#  The source code contained or described herein and all documents related to #
#  the source code ("Material") are owned by Intel Corporation or its         #
#  suppliers or licensors.  Title to the Material remains with Intel          #
#  Corporation or its suppliers and licensors.  The Material contains trade   #
#  secrets and proprietary and confidential information of Intel or its       #
#  suppliers and licensors.  The Material is protected by worldwide copyright #
#  and trade secret laws and treaty provisions.  No part of the Material may  #
#  be used, copied, reproduced, modified, published, uploaded, posted,        #
#  transmitted, distributed, or disclosed in any way without Intel's prior    #
#  express written permission.                                                #
#                                                                             #
#  No license under any patent, copyright, trade secret or other intellectual #
#  property right is granted to or conferred upon you by disclosure or        #
#  delivery of the Materials, either expressly, by implication, inducement,   #
#  estoppel or otherwise. Any license under such intellectual property rights #
#  must be express and approved by Intel in writing.                          #
#                                                                             #
###############################################################################

use lib '/usr/ins/intel/gui/perllib';
use strict;
use CGI;
use DisplayMain;
use LanguageResource;
use IO::File;
use DNSConfig;
use DNSZone;

my $DNSPATH = '/home/named/var/named/';

########################################
my $cgi					= new CGI;
my $Result;
my $Feedback;
my $FocusForm;
my $FocusField;
my $UseFocus = 0;

my $PreferredLanguage		= (split(/,/, $ENV{'HTTP_ACCEPT_LANGUAGE'}))[0];
my $StringResource		= new LanguageResource($PreferredLanguage, '/usr/ins/intel/gui/locale', [ 'server_dns.lrc', 'common.lrc' ]);

my %_R = (	'IDT:8800'			=> '',
		'IDQH:8800'			=> '',
		'DNS'				=> '',
		'ZONE'				=> '',
		'SUBZONE'			=> '',

		'DNSZONENAME'			=> '',
		'DNSSUBNAME'			=> '',
		'ASSIGN'			=> '',
		'REMOTESERVERNAME'		=> '',
		
		'DEFAULT_MESSAGE'		=> '');



$StringResource->GetStrings(\%_R);

########################################
# Form Data Hash
# (to contain default values at first)

my %FormData = (	'Asubzonename'		=> '',
			'Asubzoneipaddr'	=> '',
		);

########################################

if ( $cgi->param('AssignSubzoneAct') )
{
	my $Key;

	# Get the form data from the form.
	foreach $Key (keys(%FormData))
	{
		# Get the data...
		$FormData{$Key} = (defined($cgi->param($Key))) ? $cgi->param($Key) : '';
	}
}

########################################
my $zonename = $cgi->param('zname');        
my $html;

$html = <<EOS_ASSIGN1;

<FORM METHOD="POST" NAME=assignsubzone ACTION="server_dns_assign.cgi" ENCTYPE="application/x-www-form-urlencoded">
<INPUT TYPE="hidden" NAME="AssignSubzoneAct" value="1">
<INPUT TYPE="hidden" NAME="zname" value="$zonename">
<TABLE BORDER="0" WIDTH="375" BGCOLOR="#eeeeee">
  <TR>
    <TD>
      <TABLE BORDER="0" WIDTH="100%">
        <TR>
          <TD WIDTH="100%" colspan="2">&nbsp;
          </TD>
        </TR>
        <tr>
          <TD WIDTH="53%" NOWRAP><b>$_R{'DNSSUBNAME'}:</b>
          </TD>
          <TD WIDTH="47%" NOWRAP><INPUT TYPE="text" NAME="Asubzonename" value="$FormData{'Asubzonename'}" SIZE=10 MAXLENGTH=45> <B>.$zonename </B>
          </TD>
        </tr>
        <tr>
          <TD WIDTH="53%" NOWRAP><b>$_R{'REMOTESERVERNAME'}:</b>
          </TD>
          <TD WIDTH="47%" NOWRAP><INPUT TYPE="text" NAME="Asubzoneipaddr" value="$FormData{'Asubzoneipaddr'}" SIZE=18 MAXLENGTH=48>
          </TD>
        </tr>
        <TR>
          <TD WIDTH="100%" colspan="2">&nbsp;</TD>
        </TR>
        <TR>
          <TD WIDTH="100%" colspan="2" align="right">&nbsp;<input type="submit" class="clsButton" value="$_R{'ASSIGN'}" name="submit">
          </TD>
        </TR>
      </TABLE>
    </TD>
  </TR>
</TABLE>          
</FORM>

EOS_ASSIGN1
#######################################




###############################################################################
# SetPage
###############################################################################

sub SetPage
{
	my ($html, $FirstTableString, $Records, $Domainname, $recordcount, $SubzoneRecords, $subzonecount) = @_;
	
	# Set the DNS Records.
	$html =~ s/FIRSTTABLESTRING/$FirstTableString/s;

	# Set the DNS Records.
	$html =~ s/RECORDS/$Records/s;

	# Set the DNS Records.
	$html =~ s/DOMAINNAME/$Domainname/g;

	# Set the DNS Records.
	$html =~ s/RECNUM/$recordcount/g;

	# Set the DNS Records.
	$html =~ s/SUBZONERECORDS/$SubzoneRecords/s;

	# Set the DNS Records.
	$html =~ s/SUBZONENUM/$subzonecount/g;

	return $html;
}

###############################################################################
# InvalidIP
###############################################################################

sub InvalidIP
{
	my ($ipaddress,$useless) = @_;
	
	my ($ipA,$ipB,$ipC,$ipD);
	my ($t1,$t2);
	my $tempIP = $ipaddress;

	my $chkval = ( $tempIP =~ /(\d{1,3})\.(\d{1,3})\.(\d{1,3})\.(\d{1,3})/ );

	if (!($chkval) )
	{
		# If the string does not match the approved format
		return 1;  # false
	}
	elsif ( ($` ne '') || ($' ne '') || ($& eq '') )
	{
		return 1;
		# Matched, but there's extra data before or after the string.
	}
	else
	{
		($ipA,$ipB,$ipC,$ipD) = ( $tempIP =~ m/(\d{1,3})\.(\d{1,3})\.(\d{1,3})\.(\d{1,3})/ );
		
		if ( ($ipA < 0) || ($ipA > 255) )
		{
			return 1;
		}
		elsif ( ($ipB < 0) || ($ipB > 255) )
		{
			return 1;
		}
		elsif ( ($ipC < 0) || ($ipC > 255) )
		{
			return 1;
		}
		elsif ( ($ipD < 0) || ($ipD > 255) )
		{
			return 1;
		}
		else
		{
			return 0;
		}

	}

}

###############################################################################
# InvalidFQDN
###############################################################################

sub InvalidFQDN
{
	my ($thisfqdn,$useless) = @_;
	
	# Look for valid fqdn host names.
	
	if ($thisfqdn =~ /_/)
	{
		return 1;
	}
	
	if ($thisfqdn =~ /^(\w|-)+(\.((\w|-)+))*$/ )
	{
		return 0;  
	}
	
	else
	{
		return 1;

	}

}

###############################################################################
# InvalidHost
###############################################################################

sub InvalidHost
{
	my ($thishost,$useless) = @_;
	
	# Look for valid host names only.
	
	if ($thishost =~ /_/)
	{
		return 1;
	}
	
	if ($thishost =~ /^(\w|-)+$/ )
	{
		return 0;  
	}
	
	else
	{
		return 1;

	}

}###############################################################################
# The main body of the script.
###############################################################################

my $Records;
my $SubzoneRecords;
my $Domainname;
my $NumZones;
my ($ia,$ia1,$ib,$ic,$id);
my $i;
my $j;
my $count;
my $total;
my (@pz_soa,@pz_ns,@pz_mx,@pz_a,@pz_cname,@pz_ptr,@pz_nssub);
my $perc = '%';


if ( ($cgi->param('AssignSubzoneAct')) && ($cgi->param('AssignSubzoneAct')) )
{
	my $subzonename = lc($cgi->param('Asubzonename'));
	my $subzoneipaddr = lc($cgi->param('Asubzoneipaddr'));
	
	my ($ia,$ib,$ic);
	my @z_config_info;

	if ($subzonename eq '')
	{
		# Missing data.
		$Result = 880001;
		$Feedback = $StringResource->_S($Result) . "<BR>";
		
		$UseFocus = 1;
		$FocusForm = 'assignsubzone';
		$FocusField = 'Asubzonename';
		goto RETURNASSIGNSUBZONE;
	}
	elsif ($subzoneipaddr eq '')
	{
		# Missing data.
		$Result = 880001;
		$Feedback = $StringResource->_S($Result) . "<BR>";
		
		$UseFocus = 1;
		$FocusForm = 'assignsubzone';
		$FocusField = 'Asubzoneipaddr';
		goto RETURNASSIGNSUBZONE;
	}

	$ia = DNSConfig->zone_getinfo(\@z_config_info,$zonename);

	# Check to make sure that this is a valid FQDN style name.

	if (InvalidFQDN($subzoneipaddr))
	{
		$Result = 880010;
		$Feedback = $StringResource->_S($Result) . "<BR>";
		
		$UseFocus = 1;
		$FocusForm = 'assignsubzone';
		$FocusField = 'Asubzoneipaddr';
		goto RETURNASSIGNSUBZONE;
	}
	
	# If the remote server address is not an IP address, let's throw a trailing dot at the end,
	# and assume it's a hostname.

	if (InvalidIP($subzoneipaddr))
	{
		$subzoneipaddr .= '.';
	}
	else
	{
		# It's a valid IP, so that's bad.
		$Result = 880009;
		$Feedback = $StringResource->_S($Result) . "<BR>";
		
		$UseFocus = 1;
		$FocusForm = 'assignsubzone';
		$FocusField = 'Asubzoneipaddr';
		goto RETURNASSIGNSUBZONE;
	}

	
	if ( ($z_config_info[0] ne '') && ($z_config_info[1] ne '') )
	{
		# Add NS SUB record to parent zone.
		$ib = DNSZone->zone_addrecord($zonename,$z_config_info[1],"NS",$subzonename,$subzoneipaddr);

		if ($ib ne 0)
		{
			# Error adding record.
			$Result = 880002;
			$Feedback = $StringResource->_S($Result) . "<BR>";
			goto RETURNASSIGNSUBZONE;
		}
		else
		{
			$ic = DNSZone->zone_increment($zonename,$z_config_info[1]);
			if ($ic ne 0)
			{
				# Error incrementing parent zone.
				$Result = 880003;
				$Feedback = $StringResource->_S($Result) . "<BR>";
				goto RETURNASSIGNSUBZONE;
			}
			
		}
		
	}
	else
	{
		# Error.
		$Result = 880011;
		$Feedback = $StringResource->_S($Result) . "<BR>";
		goto RETURNASSIGNSUBZONE;
	}

	if ( ($ia eq 0) && ($ib eq 0) && ($ic eq 0) )
	{
		# Do redirect to zone edit.
		$cgi->redirect("server_dns_edit.cgi?zname=" . $zonename . "&newsubzonemessage=2&newsubzonename=" . $subzonename . '.' . $zonename );
		exit(0);
	}

}	# Fall through to re-display results on error.


RETURNASSIGNSUBZONE:



my %var;

$var{'q'}			= $cgi;
$var{'html'}			= $html;
$var{'class'}			= 'server_dns';
$var{'id'}			= 8800;
$var{'title'}			= $_R{'IDT:8800'};
$var{'root-link'}		= 'server_dns';
$var{'result'}			= $Result;
$var{'feedback'}		= $Feedback;
$var{'cookiecrumbs'}  =	[	{name=>$StringResource->_S('IDT:8000'), location=>"server_dns.cgi"},
				{name=>$StringResource->_S('IDT:8300'), location=>"server_dns_edit.cgi?zname=$zonename"},
			];

if ($UseFocus)
{
	$var{'focus-form'}	= $FocusForm;
	$var{'focus-field'}	= $FocusField;
}

DisplayMain->printDocument(\%var);

exit(0);


###########################################

