#!/usr/bin/perl
$VERSION = "0.5";

###############################################################################
#                                                                             #
# Filename: server_dns_edit.cgi                                               #
#                                                                             #
# Description: Manages A, CNAME, and PTR records as well as other             #
#              configuration for a zone on this appliance.                    #
#                                                                             #
###############################################################################
###############################################################################
#                                                                             #
#  Copyright  2000 - 2001 Intel Corporation.                                 #
#  Intel Corporation All Rights Reserved.                                     #
#                                                                             #
#  The source code contained or described herein and all documents related to #
#  the source code ("Material") are owned by Intel Corporation or its         #
#  suppliers or licensors.  Title to the Material remains with Intel          #
#  Corporation or its suppliers and licensors.  The Material contains trade   #
#  secrets and proprietary and confidential information of Intel or its       #
#  suppliers and licensors.  The Material is protected by worldwide copyright #
#  and trade secret laws and treaty provisions.  No part of the Material may  #
#  be used, copied, reproduced, modified, published, uploaded, posted,        #
#  transmitted, distributed, or disclosed in any way without Intel's prior    #
#  express written permission.                                                #
#                                                                             #
#  No license under any patent, copyright, trade secret or other intellectual #
#  property right is granted to or conferred upon you by disclosure or        #
#  delivery of the Materials, either expressly, by implication, inducement,   #
#  estoppel or otherwise. Any license under such intellectual property rights #
#  must be express and approved by Intel in writing.                          #
#                                                                             #
###############################################################################

use lib '/usr/ins/intel/gui/perllib';
use strict;
use CGI;
use DisplayMain;
use LanguageResource;
use IO::File;
use DNSConfig;
use DNSZone;

my $DNSPATH = '/home/named/var/named/';

########################################
my $Result;
my $Feedback;

my $PreferredLanguage		= (split(/,/, $ENV{'HTTP_ACCEPT_LANGUAGE'}))[0];
my $StringResource		= new LanguageResource($PreferredLanguage, '/usr/ins/intel/gui/locale', [ 'server_dns.lrc', 'common.lrc' ]);

my %_R = (	'IDT:8300'			=> '',
		'IDQH:8300'			=> '',
		'IDT:8800'			=> '',
		'IDQH:8800'			=> '',
		'DNS'				=> '',
		'ZONE'				=> '',
		'SUBZONE'			=> '',

		'DNSZONENAME'			=> '',
		'DNSSUBNAME'			=> '',
		'PARENTNAME'			=> '',
		'IPADDRESSSUBNET'		=> '',
		'ZONEDETAILS'			=> '',
		'ZONEMAILSERVERS'		=> '',
		'ZONESLAVESERVERS'		=> '',
		'ZONEREFRESHDATA'		=> '',

		'DNSRECORDS'			=> '',
		'HOST'				=> '',
		'ALIAS'				=> '',
		'POINTER'			=> '',
		'ADDHOST'			=> '',
		'ADDALIAS'			=> '',
		'ADDPOINTER'			=> '',
		'REC_COL1'			=> '',
		'REC_COL2'			=> '',
		'REC_COL3'			=> '',

		'DNSSUBZONESFORTHIS'		=> '',
		'ADD'				=> '',
		'ASSIGN'			=> '',
		'DELETE'			=> '',
		'SUBZONENAME'			=> '',
		'SUBZONEADDR'			=> '',
		'ASSIGNEDTO'			=> '',
		'REVERSEFOR'			=> '',
		'SLAVEZONEDISP'			=> '',
		'MULTIPLEHOSTS'			=> '',
		'REMOTESERVERADDR'		=> '',
		'SLAVESUBZONEDISP'		=> '',
		'MASTERLC'			=> '',
		
		'SLAVECANNOTBEEDITED'		=> '',
		'SLAVEUNAVAILABLE'		=> '',
		'MASTERUNAVAILABLE'		=> '',
		'ZONEFILECONTENTMSG'		=> '',

		'ALLSELECTEDSUBZONESMSG'	=> '',
		'NOSELECTEDSUBZONESMSG'		=> '',
		'ALLSELECTEDRECORDSMSG'		=> '',
		'NOSELECTEDRECORDSMSG'		=> '',
		'DEFAULT_MESSAGE'		=> '');



$StringResource->GetStrings(\%_R);

########################################


########################################

sub zone_recursivedelete
{
	my ($thiszone,$isfirstcall) = @_;
	my ($i,$j,$k);
	my $Result;
	my $Feedback;

	my ($i,$ia,$ib,$ic,$id,$ie);
	my @z_info;
	
	if  ( ($isfirstcall eq '') || (!(defined($isfirstcall))) )
	{
		$isfirstcall = 1;
	}

	$ic = DNSConfig->zone_getinfo(\@z_info,$thiszone);
	
	if ($ic ne 0)
	{
		if ($isfirstcall)
		{
			$Result = 830002;
			$Feedback .= "$thiszone " . $StringResource->_S($Result) . "<BR>";
		}
		else
		{
			$Feedback .= 'UNIQ8300BAD';
		}
		next;
	}

	my (@rz_soa,@rz_ns,@rz_mx,@rz_a,@rz_cname,@rz_ptr,@rz_nssub);
	
	$id = DNSZone->zone_getinfo($z_info[0],$z_info[1],\@rz_soa,\@rz_ns,\@rz_mx,\@rz_a,\@rz_cname,\@rz_ptr,\@rz_nssub);

	if ($id ne 0)
	{
		if ($isfirstcall)
		{
			$Result = 830002;
			$Feedback .= "$thiszone " . $StringResource->_S($Result) . "<BR>";
		}
		else
		{
			$Feedback .= 'UNIQ8300BAD';
		}
		next;
	}

	my $subcount;
	$subcount = scalar(@rz_nssub);
	my $RecFeedback;
	
	if ($subcount > 0)
	{
		for ($k = 0; $k < $subcount; $k++)
		{
			my $subzt = $rz_nssub[$k][0] . '.' . $thiszone;
			my @sub_info;
			
			if ((DNSConfig->zone_getinfo(\@sub_info,$subzt)) eq 0 )
			{
				$RecFeedback .= zone_recursivedelete($subzt,0);
			}

		}	# End for loop
	}
			
	$i = DNSZone->zone_erasedata($z_info[0],$z_info[1]);
	$ia = DNSConfig->zone_del($z_info[0]);

	if ($ia eq 0)
	{
		if ($isfirstcall)
		{
			if ($RecFeedback =~ /UNIQ8300BAD/)
			{
				# Didn't work... error somewhere in the recursive process.
				$Result = 830005;
				$Feedback = $StringResource->_S($Result) . "<BR>";
			}
			else
			{
				$Result = 830003;
				$Feedback = $StringResource->_S($Result) . " $thiszone <BR>";
			}
		}
		else
		{
			$Feedback .= '';
		}
	}
	else
	{
		if ($isfirstcall)
		{
			$Result = 830004;
			$Feedback = $StringResource->_S($Result) . "<BR>";
		}
		else
		{
			$Feedback .= 'UNIQ8300BAD';
		}
	}

	return $Feedback;
}

########################################

sub zone_amisubzone
{
	my ($thiszone,$useless) = @_;
	
	my ($i,$j,$k);
	my ($ia,$ib,$ic,$id);
	my @allzones;
	
	my $thisfqdn = $thiszone . '.';

	$ia = DNSConfig->zone_list(\@allzones);
	for ($i = 0; $i < scalar(@allzones); $i++)
	{
		next if ($allzones[$i][2] ne '1');

		my (@tz_soa,@tz_ns,@tz_mx,@tz_a,@tz_cname,@tz_ptr,@tz_nssub);
		
		$ia = DNSZone->zone_getinfo($allzones[$i][0],$allzones[$i][1],\@tz_soa,\@tz_ns,\@tz_mx,\@tz_a,\@tz_cname,\@tz_ptr,\@tz_nssub);
		
		my $subcount;
		$subcount = scalar(@tz_nssub);
		
		if ($subcount > 0)
		{
			for ($j = 0; $j < $subcount; $j++)
			{
				my $tempsubshort = $tz_nssub[$j][0];
				my $tempsublong = $tempsubshort . '.' . $allzones[$i][0];
				
				if ( ($tempsublong eq $thiszone) || ($tempsubshort eq $thisfqdn) )
				{
					return $allzones[$i][0];
				}

			}
		
		}
		
	}

	return '';

}
########################################


my $PAGEFWD = <<HEREFWD;
<script>
function checkall(count,type)
{
	if(eval(\"document.DNS.delall_\" + type + \".checked\") == true)
	{
		for(var i=0;i<count;i++)
			eval(\"document.DNS.\" + type + "" + i + \".checked='checked';\");
	}
	else
	{
		for(var i=0;i<count;i++)
			eval(\"document.DNS.\" + type + "" + i + \".checked='';\");
	}
}

function preparetodeleterecords(count,type)
{
	var boxcount=0;
	for(var i=0;i<count;i++)	
	{
		if (eval(\"document.DNS.\" + type + "" + i + \".checked\"))
		{
			boxcount++;
		}
	}
	
	if (0 < boxcount)
	{
		return confirm("$_R{'ALLSELECTEDRECORDSMSG'}");
	}
	else
	{
		alert("$_R{'NOSELECTEDRECORDSMSG'}");
		return false;
	}
}

function preparetodeletesubzones(count,type)
{
	var boxcount=0;
	for(var i=0;i<count;i++)	
	{
		if (eval(\"document.DNS.\" + type + "" + i + \".checked\"))
		{
			boxcount++;
		}
	}
	
	if (0 < boxcount)
	{
		return confirm("$_R{'ALLSELECTEDSUBZONESMSG'}");
	}
	else
	{
		alert("$_R{'NOSELECTEDSUBZONESMSG'}");
		return false;
	}
}

</script>
<FORM METHOD="POST" NAME=DNS ACTION="server_dns_edit.cgi" ENCTYPE="application/x-www-form-urlencoded">

<INPUT TYPE="hidden" NAME="zname" VALUE="DOMAINNAME">

<TABLE BORDER="0" WIDTH="375" BGCOLOR="#eeeeee">
  <TR>
    <TD>
      <TABLE BORDER="0" WIDTH="100%">  FIRSTTABLESTRING
        <TR>
          <TD Colspan=2 NOWRAP><input type="button" class="clsButton" value="$_R{'ZONEDETAILS'}" name="B18" onclick="javascript:location='server_dns_edit_SOA.cgi?zname=DOMAINNAME'">
            <input type="button" class="clsButton" value="$_R{'ZONEMAILSERVERS'}" name="MX18" onclick="javascript:location='server_dns_edit_mailserver.cgi?zname=DOMAINNAME'">
            <input type="button" class="clsButton" value="$_R{'ZONESLAVESERVERS'}" name="NS18" onclick="javascript:location='server_dns_edit_slaves.cgi?zname=DOMAINNAME'">
          </TD>
        </TR>
      </TABLE>
    </TD>
  </TR>
</TABLE>          
<BR>
<TABLE BORDER="0" WIDTH="375" BGCOLOR="#eeeeee">
  <TR>
    <TD>
      <TABLE BORDER="0" WIDTH="100%">
        <TR>
          <TD WIDTH="100%" colspan="2">&nbsp;</TD>
        </TR>
        <tr>
          <td width="140%" align="left" valign="bottom" colspan="2"><b>$_R{'DNSRECORDS'}</b>
          </td>
        </tr>
        <tr>
          <td width="140%" align="left" valign="bottom" colspan="2">
            <input type="button" class="clsButton" value="$_R{'ADDHOST'}" name="B16Host" onclick="javascript:location='server_dns_addrecord_A.cgi?zname=DOMAINNAME'">
            <input type="button" class="clsButton" value="$_R{'ADDALIAS'}" name="B16Alias" onclick="javascript:location='server_dns_addrecord_CNAME.cgi?zname=DOMAINNAME'">
            <input type="submit" class="clsButton" value="$_R{'DELETE'}" name="DeleteRec" onclick=" return preparetodeleterecords(RECNUM,'rec');">
          </td>
        </tr>
        <tr>
          <td width="140%" align="left" valign="bottom" colspan="2">&nbsp;
          </td>
        </tr>
        <TR>
          <TD WIDTH="100%" colspan="2">
            <table border="0" cellpadding="0" cellspacing="0" width="100%" bgcolor="#EEEEEE">
              <tr>
                <td width="100%">
                  <table border="1" cellpadding="2" cellspacing="0" width="100%" bgcolor="#FFFFFF" bordercolor="#EEEEEE">
                    <tr>
                      <td width="2%" bgcolor="#808080"><input type="checkbox" name="delall_rec" value="delall_rec" onclick="javascript:checkall(RECNUM,'rec');">&nbsp;
                      </td>
                      <td width="49%" bgcolor="#808080"><b><font color="#FFFFFF">$_R{'REC_COL1'}</font></b>
                      </td>
                      <td width="25%" nowrap bgcolor="#808080"><b><font color="#FFFFFF">$_R{'REC_COL2'}</font></b>
                      </td>
                      <td width="24%" bgcolor="#808080"><b><font color="#FFFFFF">$_R{'REC_COL3'}</font></b>
                      </td>
                    </tr>
                    RECORDS
                  </table>
                </td>
              </tr>
            </table>
          </TD>
        </TR>
      </TABLE>
    </TD>
  </TR>
</TABLE>          
<BR>
<TABLE BORDER="0" WIDTH="375" BGCOLOR="#eeeeee">
  <TR>
    <TD>
      <TABLE BORDER="0" WIDTH="100%">
        <TR>
          <TD WIDTH="100%" colspan="2">&nbsp;
          </TD>
        </TR>
        <tr>
          <td width="140%" align="left" valign="bottom" colspan="2"><b>$_R{'DNSSUBZONESFORTHIS'}</b>
          </td>
        </tr>
        <tr>
          <td width="140%" align="left" valign="bottom" colspan="2"><input type="button" class="clsButton" value="$_R{'ADD'}" name="SZ16" onclick="javascript:location='server_dns_subzone.cgi?zname=DOMAINNAME'">
            <input type="button" class="clsButton" value="$_R{'ASSIGN'}" name="AssignSubzone" onclick="javascript:location='server_dns_assign.cgi?zname=DOMAINNAME'">
            <input type="submit" class="clsButton" value="$_R{'DELETE'}" name="DeleteSubzone" onclick=" return preparetodeletesubzones(SUBZONENUM,'subzone');">
          </td>
        </tr>
        <tr>
          <td width="140%" align="left" valign="bottom" colspan="2">&nbsp;
          </td>
        </tr>
        <tr>
          <TD WIDTH="100%" colspan="2">
            <table border="0" cellpadding="0" cellspacing="0" width="100%" bgcolor="#EEEEEE">
              <tr>
                <td width="100%">
                  <table border="1" cellpadding="2" cellspacing="0" width="100%" bgcolor="#FFFFFF" bordercolor="#EEEEEE">
                    <tr>
                      <td width="2%" bgcolor="#808080"><input type="checkbox" name="delall_subzone" value="delall_subzone" onclick="javascript:checkall(SUBZONENUM,'subzone');">&nbsp;
                      </td>
                      <td width="49%" bgcolor="#808080"><b><font color="#FFFFFF">$_R{'SUBZONENAME'}</font></b>
                      </td>
                      <td width="49%" bgcolor="#808080"><b><font color="#FFFFFF">$_R{'SUBZONEADDR'}</font></b>
                      </td>
                    </tr>
                    SUBZONERECORDS
                  </table>
                </td>
              </tr>
            </table>
          </TD>
        </tr>
        <TR>
          <TD WIDTH="100%" colspan="2">&nbsp;
          </TD>
        </TR>
      </TABLE>
    </TD>
  </TR>
</TABLE>
</FORM>
HEREFWD

##############################################

my $PAGE_SLAVE_AVAIL = <<HERE_SLAVE_AVAIL;
<FORM METHOD="POST" NAME=DNS ACTION="server_dns_edit.cgi" ENCTYPE="application/x-www-form-urlencoded">

<INPUT TYPE="hidden" NAME="zname" VALUE="DOMAINNAME">

<TABLE BORDER="0" WIDTH="375" BGCOLOR="#eeeeee">
  <TR>
    <TD>
      <TABLE BORDER="0" WIDTH="100%">  FIRSTTABLESTRING
	<TR>
	  <TD COLSPAN=2>$_R{'SLAVECANNOTBEEDITED'}
	  </TD>
	</TR>
        <TR>
          <TD Colspan=2 NOWRAP><input type="button" class="clsButton" value="$_R{'ZONEDETAILS'}" name="B18" onclick="javascript:location='server_dns_edit_SOA.cgi?zname=DOMAINNAME'">
            <input type="button" class="clsButton" value="$_R{'ZONEREFRESHDATA'}" name="REF18" onclick="javascript:location='server_dns_edit.cgi?zname=DOMAINNAME&refreshdata=1'">
          </TD>
        </TR>
      </TABLE>
    </TD>
  </TR>
</TABLE>          

<BR>
<TABLE BORDER="0" WIDTH="375" BGCOLOR="#eeeeee">
  <TR>
    <TD><B>$_R{'ZONEFILECONTENTMSG'}</B>
    </TD>
  </TR>
  <TR>
    <TD>
      <TABLE BORDER="0" WIDTH="100%">
        <TR>
          <TD WIDTH="100%" colspan="2">
            <table border="0" cellpadding="0" cellspacing="0" width="100%" bgcolor="#EEEEEE">
              <tr>
                <td width="100%" BGCOLOR="#FFFFFF" NOWRAP>
                  <FONT SIZE=2>
<PRE>
ZONEFILECONTENTSTRING
</PRE>
                  <BR></FONT>
                </td>
              </tr>
            </table>
          </TD>
        </TR>
      </TABLE>
    </TD>
  </TR>
</TABLE>          
<BR>

</FORM>
HERE_SLAVE_AVAIL


##############################################

my $PAGE_SLAVE_UNAVAIL = <<HERE_SLAVE_UNAVAIL;
<FORM METHOD="POST" NAME=DNS ACTION="server_dns_edit.cgi" ENCTYPE="application/x-www-form-urlencoded">

<INPUT TYPE="hidden" NAME="zname" VALUE="DOMAINNAME">

<TABLE BORDER="0" WIDTH="375" BGCOLOR="#eeeeee">
  <TR>
    <TD>
      <TABLE BORDER="0" WIDTH="100%">  FIRSTTABLESTRING
	<TR>
	  <TD COLSPAN=2>$_R{'SLAVEUNAVAILABLE'}
	  </TD>
	</TR>
        <TR>
          <TD Colspan=2 NOWRAP><input type="button" class="clsButton" value="$_R{'ZONEDETAILS'}" name="B18" onclick="javascript:location='server_dns_edit_SOA.cgi?zname=DOMAINNAME'">
            <input type="button" class="clsButton" value="$_R{'ZONEREFRESHDATA'}" name="REF18" onclick="javascript:location='server_dns_edit.cgi?zname=DOMAINNAME&refreshdata=1'">
         </TD>
        </TR>
      </TABLE>
    </TD>
  </TR>
</TABLE>          


</FORM>
HERE_SLAVE_UNAVAIL


##############################################

my $PAGE_MASTER_UNAVAIL = <<HERE_MASTER_UNAVAIL;
<FORM METHOD="POST" NAME=DNS ACTION="server_dns_edit.cgi" ENCTYPE="application/x-www-form-urlencoded">

<INPUT TYPE="hidden" NAME="zname" VALUE="DOMAINNAME">

<TABLE BORDER="0" WIDTH="375" BGCOLOR="#eeeeee">
  <TR>
    <TD>
      <TABLE BORDER="0" WIDTH="100%">  FIRSTTABLESTRING
	<TR>
	  <TD COLSPAN=2>$_R{'MASTERUNAVAILABLE'}
	  </TD>
	</TR>
        <TR>
          <TD Colspan=2 NOWRAP><input type="button" class="clsButton" value="$_R{'ZONEDETAILS'}" name="B18" onclick="javascript:location='server_dns_edit_SOA.cgi?zname=DOMAINNAME'">
<!--
            <input type="button" class="clsButton" value="$_R{'ZONEREFRESHDATA'}" name="REF18" onclick="javascript:location='server_dns_edit.cgi?zname=DOMAINNAME&refreshdata=1'">
-->
         </TD>
        </TR>
      </TABLE>
    </TD>
  </TR>
</TABLE>          


</FORM>
HERE_MASTER_UNAVAIL




###############################################################################
my $PAGEREV = <<HEREREV;
<script>
function checkall(count,type)
{
	if(eval(\"document.DNS.delall_\" + type + \".checked\") == true)
	{
		for(var i=0;i<count;i++)
			eval(\"document.DNS.\" + type + "" + i + \".checked='checked';\");
	}
	else
	{
		for(var i=0;i<count;i++)
			eval(\"document.DNS.\" + type + "" + i + \".checked='';\");
	}
}

</script>
<FORM METHOD="POST" NAME=DNS ACTION="server_dns_edit.cgi" ENCTYPE="application/x-www-form-urlencoded">

<INPUT TYPE="hidden" NAME="zname" VALUE="DOMAINNAME">

<TABLE BORDER="0" WIDTH="375" BGCOLOR="#eeeeee">
  <TR>
    <TD>
      <TABLE BORDER="0" WIDTH="100%">  FIRSTTABLESTRING
        <TR>
          <TD Colspan=2 NOWRAP><input type="button" class="clsButton" value="$_R{'ZONEDETAILS'}" name="B18" onclick="javascript:location='server_dns_edit_SOA.cgi?zname=DOMAINNAME'">
            <input type="button" class="clsButton" value="$_R{'ZONESLAVESERVERS'}" name="NS18" onclick="javascript:location='server_dns_edit_slaves.cgi?zname=DOMAINNAME'">
          </TD>
        </TR>
      </TABLE>
    </TD>
  </TR>
</TABLE>          
<BR>
<TABLE BORDER="0" WIDTH="375" BGCOLOR="#eeeeee">
  <TR>
    <TD>
      <TABLE BORDER="0" WIDTH="100%">
        <TR>
          <TD WIDTH="100%" colspan="2">&nbsp;
          </TD>
        </TR>
        <tr>
          <td width="140%" align="left" valign="bottom" colspan="2"><b>$_R{'DNSRECORDS'}</b></td>
        </tr>
        <tr>
          <td width="140%" align="left" valign="bottom" colspan="2">
            <input type="button" class="clsButton" value="$_R{'ADDPOINTER'}" name="B16Pointer" onclick="javascript:location='server_dns_addrecord_PTR.cgi?zname=DOMAINNAME'">
            <input type="submit" class="clsButton" value="$_R{'DELETE'}" name="DeleteRec" onclick=" return confirm('$_R{q/ALLSELECTEDRECORDSMSG/}')">
          </td>
        </tr>
        <tr>
          <td width="140%" align="left" valign="bottom" colspan="2">&nbsp;</td>
        </tr>
        <TR>
          <TD WIDTH="100%" colspan="2">
            <table border="0" cellpadding="0" cellspacing="0" width="100%" bgcolor="#EEEEEE">
              <tr>
                <td width="100%">
                  <table border="1" cellpadding="2" cellspacing="0" width="100%" bgcolor="#FFFFFF" bordercolor="#EEEEEE">
                    <tr>
                      <td width="2%" bgcolor="#808080"><input type="checkbox" name="delall_rec" value="delall_rec" onclick="javascript:checkall(RECNUM,'rec');">&nbsp;
                      </td>
                      <td width="49%" bgcolor="#808080"><b><font color="#FFFFFF">$_R{'REC_COL1'}</font></b>
                      </td>
                      <td width="25%" nowrap bgcolor="#808080"><b><font color="#FFFFFF">$_R{'REC_COL2'}</font></b>
                      </td>
                      <td width="24%" bgcolor="#808080"><b><font color="#FFFFFF">$_R{'REC_COL3'}</font></b>
                      </td>
                    </tr>
                    RECORDS
                  </table>
                </td>
              </tr>
            </table>
          </TD>
        </TR>
      </TABLE>
    </TD>
  </TR>
</TABLE>          
<BR>
<TABLE BORDER="0" WIDTH="375" BGCOLOR="#eeeeee">
  <TR>
    <TD>
      <TABLE BORDER="0" WIDTH="100%">
        <TR>
          <TD WIDTH="100%" colspan="2">&nbsp;
          </TD>
        </TR>
        <tr>
          <td width="140%" align="left" valign="bottom" colspan="2"><b>$_R{'DNSSUBZONESFORTHIS'}</b>
          </td>
        </tr>
        <tr>
          <td width="140%" align="left" valign="bottom" colspan="2"><input type="button" class="clsButton" value="$_R{'ADD'}" name="SZ16" onclick="javascript:location='server_dns_subzone.cgi?zname=DOMAINNAME'">
            <input type="button" class="clsButton" value="$_R{'ASSIGN'}" name="AssignSubzone" onclick="javascript:location='server_dns_assign.cgi?zname=DOMAINNAME'">
            <input type="submit" class="clsButton" value="$_R{'DELETE'}" name="DeleteSubzone" onclick=" return confirm('$_R{'ALLSELECTEDSUBZONESMSG'}')">
          </td>
        </tr>
        <tr>
          <td width="140%" align="left" valign="bottom" colspan="2">&nbsp;
          </td>
        </tr>
        <tr>
          <TD WIDTH="100%" colspan="2">
            <table border="0" cellpadding="0" cellspacing="0" width="100%" bgcolor="#EEEEEE">
              <tr>
                <td width="100%">
                  <table border="1" cellpadding="2" cellspacing="0" width="100%" bgcolor="#FFFFFF" bordercolor="#EEEEEE">
                    <tr>
                      <td width="2%" bgcolor="#808080"><input type="checkbox" name="delall_subzone" value="delall_subzone" onclick="javascript:checkall(SUBZONENUM,'subzone');">&nbsp;
                      </td>
                      <td width="49%" bgcolor="#808080"><b><font color="#FFFFFF">$_R{'SUBZONENAME'}</font></b>
                      </td>
                      <td width="49%" bgcolor="#808080"><b><font color="#FFFFFF">$_R{'SUBZONEADDR'}</font></b>
                      </td>
                    </tr>
                    SUBZONERECORDS
                  </table>
                </td>
              </tr>
            </table>
          </TD>
        </tr>
        <TR>
          <TD WIDTH="100%" colspan="2">&nbsp;
          </TD>
        </TR>
      </TABLE>
    </TD>
  </TR>
</TABLE>
</FORM>
HEREREV


###############################################################################
# SetPage
###############################################################################

sub SetPage
{
	my ($html, $FirstTableString, $Records, $Domainname, $recordcount, $SubzoneRecords, $subzonecount) = @_;
	
	# Set the DNS Records.
	$html =~ s/FIRSTTABLESTRING/$FirstTableString/s;

	# Set the DNS Records.
	$html =~ s/RECORDS/$Records/s;

	# Set the DNS Records.
	$html =~ s/DOMAINNAME/$Domainname/g;

	# Set the DNS Records.
	$html =~ s/RECNUM/$recordcount/g;

	# Set the DNS Records.
	$html =~ s/SUBZONERECORDS/$SubzoneRecords/s;

	# Set the DNS Records.
	$html =~ s/SUBZONENUM/$subzonecount/g;

	return $html;
}

###############################################################################
# The main body of the script.
###############################################################################

my $cgi					= new CGI;
my $html;
my $Records;
my $SubzoneRecords;
my $Domainname;
my $NumZones;
my ($ia,$ia1,$ib,$ic,$id);
my $i;
my $j;
my $count;
my $total;
my (@pz_soa,@pz_ns,@pz_mx,@pz_a,@pz_cname,@pz_ptr,@pz_nssub);
my $perc = '%';



if ($cgi->param('DeleteRec'))
{
	my $zonename = $cgi->param('zname');
#	my $fname = $cgi->param('zonefilename');

	my @z_config_info;
	$ia = DNSConfig->zone_getinfo(\@z_config_info,$zonename);

	my $realdomname = $zonename.'.';

	$ia = 0;
	$ib = 0;

	my $delcount = 0;
	my $delsingle = 0;
	my %delhash;
	my $anyfailures = 0;
		my @selector;
		

	# Compute how many records to delete this time.

	for ($i = 0; $i < $cgi->param('recordcount'); $i++)
	{
		if ($cgi->param("rec$i"))
		{
			$delcount ++;
			$delsingle = $i;
			$delhash{"rec$i"} = 'yes';
		}
	}
	
	# Now if $delcount is not positive, then abort altogether.
	
	if ($delcount < 1)
	{
		$Result =  830019;
		$Feedback .= $StringResource->_S($Result) . "<BR>";
	}
	
	# Otherwise, if $delcount is equal to 1, then do a single delete call.

	elsif ($delcount eq 1)
	{
		my $retype = $cgi->param("rectype$delsingle");
		my $refrom = $cgi->param("recfrom$delsingle");
		my $reto = $cgi->param("recto$delsingle");
		$ib = DNSZone->zone_delrecord($zonename,$z_config_info[1],$retype,$refrom,$reto); 
	}

	# Final case, assume $delcount is greater than one, and do a multiple delete call.

	else
	{
		# Begin building the array of values to be deleted at once.
		my $delmarker = 0;

		for ($i = 0; $i < $cgi->param('recordcount'); $i++)
		{
			# if the item is checked
			if ($cgi->param("rec$i"))
			{	
				$selector[$delmarker][0] = $cgi->param("rectype$i");
				$selector[$delmarker][1] = $cgi->param("recfrom$i");
				$selector[$delmarker][2] = $cgi->param("recto$i");
				$delmarker ++;
			}
		}
		
		$ib = DNSZone->zone_delmultiple($zonename,$z_config_info[1],\@selector);
		
		$anyfailures = 0;
		
		for ($i = 0; ( ($i < scalar(@selector)) && ($anyfailures eq 0) ); $i++)
		{
			# Check for any items that failed.
			if ( (defined($selector[$i][3])) && ($selector[$i][3] ne '0') )
			{
				$anyfailures = 1;
			}
			
			# Upon failure, break out of loop.
		}
	
	}
		
	BREAKOUTDEL:
	
	if ($ib eq 0)
	{
		if ($anyfailures eq 1)
		{
			$Result = 830020;
			$Feedback .= $StringResource->_S($Result) . "<BR>";
			
			for ($i eq 0; $i < scalar(@selector); $i++)
			{
				$Feedback .= "MDel $selector[$i][0] $selector[$i][1] $selector[$i][2] $selector[$i][3] <BR>";
			} 
		}
		else
		{
			$Result = 0;
			$Feedback .= $StringResource->_S('830006') . "<BR>";
		}
		DNSZone->zone_increment($zonename,$z_config_info[1]);
	}
	
	else
	{
		$Result = 830007;
		$Feedback .= $StringResource->_S($Result) . "<BR>";
	}
	
}

if ($cgi->param('DeleteSubzone'))
{
	my $zonename = $cgi->param('zname');
	my $subzonecount = $cgi->param('subzonecount');
	
	$ia = 0;
	
	for ($i = 0; $i < $subzonecount; $i++)
	{
		if ($cgi->param("subzone$i"))
		{
			my $sz_short = $cgi->param("subzonename$i");
			my $sz_data = $cgi->param("subzoneval$i");
			my $sz_long = $sz_short . '.' . $zonename;

			my @z_info;
			my @sz_info;
			
			$ia = DNSConfig->zone_getinfo(\@sz_info,$sz_long);
			
			if ($ia eq 0)
			{
				# Appears to be a subzone that's hosted here. 
				# Let's weed it out by the roots, shall we?

				$Result = 0;
				
				if ($sz_info[2] eq '1')
				{
					# If this is a master zone...
					$Feedback .= zone_recursivedelete($sz_long);
				}
				else
				{
					# If this is a slave zone, just delete it.
					DNSZone->zone_erasedata($sz_info[0],$sz_info[1]);
					DNSConfig->zone_del($sz_info[0]);
				}
			}
			else
			{
				# Must be an assigned subzone, since it doesn't exist here.

				$Result = 0;
				$Feedback .= $StringResource->_S('830003') . " " . $sz_long . "<BR>";
			}
			
			# Either way, also remove the parent record pointer to the subzone.

			$ia1 = DNSConfig->zone_getinfo(\@z_info,$zonename);
			
			if ($ia1 ne 0)
			{
				$Result = 830011;
				$Feedback .= $StringResource->_S($Result) . "<BR>";
				next;
			}
			
			$ib  = DNSZone->zone_delrecord($z_info[0],$z_info[1],"NS",$sz_short,$sz_data);

			if ($ib ne 0)
			{
				$Result = 830009;
				$Feedback .= $StringResource->_S($Result) . "<BR>";
			}
			else
			{
				$ic  = DNSZone->zone_increment($z_info[0],$z_info[1]);

				if ($ic ne 0)
				{
					$Result = 830010;
					$Feedback .= $StringResource->_S($Result) . "<BR>";
				}

			}

		}

	}

}


######################################################
#
# my $subzonename = $cgi->param('Asubzonename');
# my $subzoneipaddr = $cgi->param('Asubzoneipaddr');
# 
# if ($cgi->param('AssignSubzoneAct'))
# {
# 	my $zonename = $cgi->param('zname');        
# 
# 	my ($ia,$ib,$ic);
# 	my @z_config_info;
# 
# 	if ( ($subzonename eq '') || ($subzoneipaddr eq '') )
# 	{
# 		# Missing data.
# 		$Result = 830012;
# 		$Feedback = $StringResource->_S($Result) . "<BR>";
# 		goto RETURNASSIGNSUBZONE;
# 	}
# 
# 	$ia = DNSConfig->zone_getinfo(\@z_config_info,$zonename);
# 
# 	if ( ($z_config_info[0] ne '') && ($z_config_info[1] ne '') )
# 	{
# 		# Add NS SUB record to parent zone.
# 		$ib = DNSZone->zone_addrecord($zonename,$z_config_info[1],"NS",$cgi->param('Asubzonename'),$cgi->param('Asubzoneipaddr'));
# 
# 		if ($ib ne 0)
# 		{
# 			# Error adding record.
# 			$Result = 830013;
# 			$Feedback = $StringResource->_S($Result) . "<BR>";
# 			goto RETURNASSIGNSUBZONE;
# 		}
# 		else
# 		{
# 			$ic = DNSZone->zone_increment($zonename,$z_config_info[1]);
# 			if ($ic ne 0)
# 			{
# 				# Error incrementing parent zone.
# 				$Result = 830014;
# 				$Feedback = $StringResource->_S($Result) . "<BR>";
# 				goto RETURNASSIGNSUBZONE;
# 			}
# 
# 		}
# 				
# 	}
# 	else
# 	{
# 		# Error.
# 		$Result = 830011;
# 		$Feedback = $StringResource->_S($Result) . "<BR>";
# 		goto RETURNASSIGNSUBZONE;
# 	}
# 
# 	if ( ($ia eq 0) && ($ib eq 0) && ($ic eq 0) )
# 	{
# 		$Result = 0;
# 		$Feedback .= "$_R{'SUBZONE'} " . $cgi->param('Asubzonename') . " " . $StringResource->_S('830016') . "<BR>";
# 		goto PROCESSNORMALVIEW;
# 	}
# 
# }	# Fall through to next block (AssignSubzone) to re-display results on error.
# 
# 
# if ($cgi->param('AssignSubzone'))
# {
# 	RETURNASSIGNSUBZONE:
# 
# 	my $zonename = $cgi->param('zname');        
# 
# 	########################################
# 	# Form Data Hash
# 	# (to contain default values at first)
# 
# 	my %FormData = (	'Asubzonename'		=> '',
# 				'Asubzoneipaddr'	=> '',
# 			);
# 
# 	########################################
# 
# 	if ( $cgi->param('AssignSubzoneAct') )
# 	{
# 		my $Key;
# 
# 		# Get the form data from the form.
# 		foreach $Key (keys(%FormData))
# 		{
# 			# Get the data...
# 			$FormData{$Key} = (defined($cgi->param($Key))) ? $cgi->param($Key) : '';
# 		}
# 	}
# 
# 	my $html;
# 	
# 	#######################################
# 	$html = <<EOS_ASSIGN1;
# 
# <FORM METHOD="POST" ACTION="server_dns_edit.cgi" ENCTYPE="application/x-www-form-urlencoded">
# <INPUT TYPE="hidden" NAME="AssignSubzone" value="2">
# <INPUT TYPE="hidden" NAME="AssignSubzoneAct" value="1">
# <INPUT TYPE="hidden" NAME="zname" value="$zonename">
# <TABLE BORDER="0" WIDTH="375" BGCOLOR="#eeeeee">
#   <TR>
#     <TD>
#       <TABLE BORDER="0" WIDTH="100%">
#         <TR>
#           <TD WIDTH="100%" colspan="2">&nbsp;
#           </TD>
#         </TR>
#         <tr>
#           <TD WIDTH="53%" NOWRAP><b>$_R{'DNSSUBNAME'}:</b>
#           </TD>
#           <TD WIDTH="47%" NOWRAP><INPUT TYPE="text" NAME="Asubzonename" value="$FormData{'Asubzonename'}" SIZE=10 MAXLENGTH=45> <B>.$zonename </B>
#           </TD>
#         </tr>
#         <tr>
#           <TD WIDTH="53%" NOWRAP><b>$_R{'REMOTESERVERADDR'}:</b>
#           </TD>
#           <TD WIDTH="47%" NOWRAP><INPUT TYPE="text" NAME="Asubzoneipaddr" value="$FormData{'Asubzoneipaddr'}" SIZE=18 MAXLENGTH=48>
#           </TD>
#         </tr>
#         <TR>
#           <TD WIDTH="100%" colspan="2">&nbsp;</TD>
#         </TR>
#         <TR>
#           <TD WIDTH="100%" colspan="2" align="right">&nbsp;<input type="submit" class="clsButton" value="$_R{'ASSIGN'}" name="AssignSubzoneAct">
#           </TD>
#         </TR>
#       </TABLE>
#     </TD>
#   </TR>
# </TABLE>          
# </FORM>
# 
# EOS_ASSIGN1
# 	#######################################
# 
# 	my %var;
# 
# 	$var{'q'}			= $cgi;
# 	$var{'html'}			= $html;
# 	$var{'class'}			= 'server_dns';
# 	$var{'id'}			= 8800;
# 	$var{'title'}			= $_R{'IDT:8800'};
# 	$var{'root-link'}		= 'server_dns';
# 	$var{'result'}			= $Result;
# 	$var{'feedback'}		= $Feedback;
# 	$var{'cookiecrumbs'}  =	[	{name=>$StringResource->_S('IDT:8000'), location=>"server_dns.cgi"},
# 					{name=>$StringResource->_S('IDT:8300'), location=>"server_dns_edit.cgi?zname=$zonename"},
# 				];
# 
# 	DisplayMain->printDocument(\%var);
# 
# 	exit(0);
# }
# 
#
###########################################

PROCESSNORMALVIEW:

my $zonename = $cgi->param('zname');        

if ($cgi->param('newsubzonemessage') && $cgi->param('newsubzonename'))
{
	$Result = 0;
	$Feedback .= "$_R{'SUBZONE'} " . $cgi->param('newsubzonename') . " " ;
	$Feedback .= ( ($cgi->param('newsubzonemessage') eq 2) ? $StringResource->_S('830016') : $StringResource->_S('830015') );
	$Feedback .= " <BR>";
	DNSConfig->named_update('named');
}

if ($cgi->param('newrecordmessage'))
{
	$Result = 0;
	$Feedback .= $StringResource->_S('830017') . " <BR>";
	DNSConfig->named_update('named');
}

if ($cgi->param('newSOAvalue'))
{
	$Result = 0;
	$Feedback .= $StringResource->_S('830018') . " <BR>";
}

if ($cgi->param('refreshdata'))
{
	DNSConfig->named_update('named');
	sleep(3);
	$Result = 0;
	$Feedback = $StringResource->_S('830001') . " <BR>";
}

my @zone_dat_list;

$ia = DNSConfig->zone_list(\@zone_dat_list);
$NumZones = scalar(@zone_dat_list);

my $FirstTableString;
my $sztestme1;

$sztestme1 = zone_amisubzone($zonename);

my $iszonereverse = 0;
if ($zonename =~ /.in-addr.arpa$/)
{
	$iszonereverse = 1;
}

my $RevSubnetString;
my $RevSubnetIP;

if ($iszonereverse eq 1)
{
	$RevSubnetString .= "<TR><TD WIDTH='53%'><B>$_R{'IPADDRESSSUBNET'}:</B></TD>";
	
	my ($t1,$t2);
	my (@a1,@a2);
	my $asize;
	my ($iii,$jjj);
	my $RevSubnetFull;
	
	@a1 = split('\.',$zonename);
	$asize = scalar(@a1);

	if (@a1[$asize-1] eq "arpa")
	{
		pop(@a1);
	}

	$asize = scalar(@a1);

	if (@a1[$asize-1] eq "in-addr")
	{
		pop(@a1);
	}
	
	$asize = scalar(@a1);

	@a2 = reverse(@a1);

	$t2 = join('.',@a2);
	
	$RevSubnetIP = $t2;
	$RevSubnetFull = $t2;

	for ($jjj = 0; $jjj < (4 - scalar(@a2)) ; $jjj++ )
	{
		$RevSubnetFull .= '.0';
	}

	$RevSubnetString .= "<TD WIDTH='47%'>$RevSubnetFull</TD></TR>\n";
}


if ($sztestme1 ne '')
{
	$FirstTableString = "\n<TR><TD WIDTH='53%'><b>$_R{'DNSSUBNAME'}:</b></TD><TD WIDTH='47%'>$zonename</TD></TR>\n";
	if ($iszonereverse eq 1)
	{
		$FirstTableString .= $RevSubnetString;
	}

	$FirstTableString .= "<TR><TD WIDTH='53%'><b>$_R{'PARENTNAME'}:</b></TD><TD WIDTH='47%'>";
	$FirstTableString .= "<A HREF=server_dns_edit.cgi?zname=$sztestme1>$sztestme1</A></TD></TR>\n";

}
else
{
	$FirstTableString = "\n<TR><TD WIDTH='53%'><b>$_R{'DNSZONENAME'}:</b></TD><TD WIDTH='47%'>$zonename</TD></TR>\n";
	if ($iszonereverse eq 1)
	{
		$FirstTableString .= $RevSubnetString;
	}
}


my $useform_master_unavail = 0;
my $useform_slave_unavail = 0;
my $useform_slave_avail = 0;
my $thismaster = '';
my $thisslave = 0;

my $RawZoneFileContents = '';

# Iterate through data structure and display meaningful information.
for ($i = 0; $i < $NumZones; $i++)
{
	# for each zone print out the records
	if($zone_dat_list[$i][0] eq $zonename)
	{
		# Begin checks for master or slave
		
		$ib = DNSZone->zone_getinfo($zonename,$zone_dat_list[$i][1],\@pz_soa,\@pz_ns,\@pz_mx,\@pz_a,\@pz_cname,\@pz_ptr,\@pz_nssub);

		$thisslave = $zone_dat_list[$i][2];
		
		if ($thisslave eq 2)
		{
			$thismaster = $zone_dat_list[$i][3];

			if ( ( -e $DNSPATH . $zone_dat_list[$i][1]) && ( -r $DNSPATH . $zone_dat_list[$i][1]) )
			{
				# Is slave, and file is ready.
				$useform_slave_avail = 1;
				# Get contents into temp variable
				
				$RawZoneFileContents = DNSZone->zone_rawdata($zonename,$zone_dat_list[$i][1]);
				
				# Now, format contents into something sane for HTML.  Use the <BR> tag
				# instead of carriage returns and line feeds.

				# $RawZoneFileContents =~ s[\n][ <BR>]g ;
				# $RawZoneFileContents =~ s[\r\f][ <BR>]g ;

			}
			else
			{
				# Is slave, and file is unavailable.
				$useform_slave_unavail = 1;
			}
			goto NORECORDJUMPSPOT;
		}
		
		# Assume master now
		
		if ($ib ne 0)
		{
			# Is master, and file is unavailable
			
			$useform_master_unavail = 1;
			goto NORECORDJUMPSPOT;
		}
	
		if ( !( ( -e $DNSPATH . $zone_dat_list[$i][1]) && ( -r $DNSPATH . $zone_dat_list[$i][1]) ) )
		{
			# Is master, and file is unavailable
			
			$useform_master_unavail = 1;
			goto NORECORDJUMPSPOT;
		}

		if ($iszonereverse eq 1)
		{	
			$count = scalar(@pz_ptr);
			for ($j = 0; $j < $count; $j++)
			{
				my $num = $total + $j;

				my (@a1,@a2);
				my ($rev_IP,$fwd_IP,$fwd_IPfull);
				
				$rev_IP = $pz_ptr[$j][0];
				@a1 = split('\.',$rev_IP);
				@a2 = reverse(@a1);
				$fwd_IP = join('.',@a2);
				$fwd_IPfull = $RevSubnetIP . '.' . $fwd_IP;
				
				$pz_ptr[$j][3] = $fwd_IPfull;
				my $tempval = 0;
				
				@a1 = split('\.',$fwd_IPfull);

				my $count = 4;
				while ($count > 0)
				{
					my $t2 = shift(@a1);
					
					$tempval *= 512;
					$tempval += $t2;
					$count--;
				}
				$pz_ptr[$j][4] = $tempval;
			}

			# Sort by [4] here

			@pz_ptr = sort { $$a[4] <=> $$b[4] } @pz_ptr;

			$total = 0;

			for ($j = 0; $j < $count; $j++)
			{
				my $num = $total + $j;
				$Records .= "\n<tr><td width=2$perc><input type=checkbox name=\"rec$num\" value=ON><input type=hidden name=\"rectype$num\" value=PTR><input type=hidden name=\"recfrom$num\" value=\"$pz_ptr[$j][0]\"><input type=hidden name=\"recto$num\" value=\"$pz_ptr[$j][1]\"></td>";
				$Records .= "<td width=49$perc>$pz_ptr[$j][3]</td>";
				$Records .= "<td width=25$perc nowrap>$_R{'POINTER'}&nbsp;</td>";
				$Records .= "<td width=24$perc nowrap>$pz_ptr[$j][1]</td></tr>";
			}
			$total += $count;
			$Records .= "\n";
		}
		
		else
		{	# If forward zone
			$count = scalar(@pz_a);

			$total = 0;

			for ($j = 0; $j < $count; $j++)
			{
				my $num = $total + $j;
				$Records .= "\n<tr><td width=2$perc><input type=checkbox name=\"rec$num\" value=ON><input type=hidden name=\"rectype$num\" value=A><input type=hidden name=\"recfrom$num\" value=\"$pz_a[$j][0]\"><input type=hidden name=\"recto$num\" value=\"$pz_a[$j][1]\"></td>";
				$Records .= "<td width=49$perc>$pz_a[$j][0]</td>";
				$Records .= "<td width=25$perc nowrap>$_R{'HOST'}&nbsp;</td>";
				$Records .= "<td width=24$perc nowrap>$pz_a[$j][1]</td></tr>";
			}
			$total += $count;

			$count = scalar(@pz_cname);
			for ($j = 0; $j < $count; $j++)
			{
				my $num = $total + $j;
				$Records .= "\n<tr><td width=2$perc><input type=checkbox name=\"rec$num\" value=ON><input type=hidden name=\"rectype$num\" value=CNAME><input type=hidden name=\"recfrom$num\" value=\"$pz_cname[$j][0]\"><input type=hidden name=\"recto$num\" value=\"$pz_cname[$j][1]\"></td>";
				$Records .= "<td width=49$perc>$pz_cname[$j][0]</td>";
				$Records .= "<td width=25$perc nowrap>$_R{'ALIAS'}&nbsp;</td>";
				$Records .= "<td width=24$perc nowrap>$pz_cname[$j][1]</td></tr>";
			}
			$total += $count;
			$Records .= "\n";
		}

		$Records .= "<input type=hidden name=zonefilename value=$zone_dat_list[$i][1]>";

		goto OUTLOOPDISPLAY1;  # Break out of the loop since we found the domain

	}
}

OUTLOOPDISPLAY1:

$Records .= "<input type=hidden name=recordcount value=$total>\n";


my @cur_zone;

$SubzoneRecords = '';
$count = scalar(@pz_nssub);

my $sztotal;
$sztotal = 0;
for ($i = 0; $i < $count; $i++)
{
	my $SubzoneLink = ' ';
	my $SubzoneName = $pz_nssub[$i][0] . '.' . $zonename;
	my $SubzoneAddress = '&nbsp; ';

	my $useless;
	$useless = DNSConfig->zone_getinfo(\@cur_zone,$SubzoneName);

	if ($useless eq 0)
	{
		$SubzoneLink = "<a href=server_dns_edit.cgi?zname=$SubzoneName>$SubzoneName</a>";

		if ($cur_zone[2] eq '2')
		{
			# If we have a slave zone, then display it as such.
			$SubzoneAddress = $_R{'SLAVESUBZONEDISP'} . ' (' . $_R{'MASTERLC'} . ' ' . $cur_zone[3] . ') ';
		
		}

		else
		{
			my $zoneip;
			my $count;
			my (@pz_soa,@pz_ns,@pz_mx,@pz_a,@pz_cname,@pz_ptr,@pz_nssub);

			$ib = DNSZone->zone_getinfo($cur_zone[0],$cur_zone[1],\@pz_soa,\@pz_ns,\@pz_mx,\@pz_a,\@pz_cname,\@pz_ptr,\@pz_nssub);

			# Add error case for $ib not zero

			my $j;
			my $foundit = 0;

			$count = scalar(@pz_a);
			for ($j = 0; $j < $count; $j++)
			{
				if($pz_a[$j][0] eq "$SubzoneName.")
				{
					$SubzoneAddress = $pz_a[$j][1];
					$foundit++;
				}
			}
			if ($foundit eq 0 )
			{
				$SubzoneAddress = '&nbsp; ';
			}
			if ($foundit > 1)
			{
				$SubzoneAddress = $_R{'MULTIPLEHOSTS'};
			}
		}
		
	}
	else
	{
		$SubzoneLink = $SubzoneName;
		$SubzoneAddress = "$_R{'ASSIGNEDTO'} $pz_nssub[$i][1] ";
	}

	$SubzoneRecords .= "\n<tr><td width=2$perc><input type=checkbox name=subzone$i value=ON><input type=hidden name=subzonename$i value=$pz_nssub[$i][0]><input type=hidden name=subzoneval$i value=$pz_nssub[$i][1]></td>";
	$SubzoneRecords .= "<td width=49$perc align=left bgcolor=#FFFFFF nowrap>$SubzoneLink</td>";
	$SubzoneRecords .= "<td width=49$perc align=left bgcolor=#FFFFFF nowrap>$SubzoneAddress</td></tr>";
	$sztotal++;
}

$SubzoneRecords .= "\n<input type=hidden name=subzonecount value=$sztotal>\n";

# Handle cases for slave zones (avail, unavail)

NORECORDJUMPSPOT:

if ($useform_slave_avail eq 1)
{
	# Slave, and available
	
	$html = $PAGE_SLAVE_AVAIL;
	
	$html =~ s/DOMAINNAME/$zonename/g;
	$html =~ s/FIRSTTABLESTRING/$FirstTableString/g;
	
	$html =~ s/ZONEFILECONTENTSTRING/$RawZoneFileContents/;
}

elsif ($useform_slave_unavail eq 1)
{
	# Slave, and unavailable
	
	$html = $PAGE_SLAVE_UNAVAIL;
	
	$html =~ s/DOMAINNAME/$zonename/g;
	$html =~ s/FIRSTTABLESTRING/$FirstTableString/g;
}

elsif ($useform_master_unavail eq 1)
{
	# Master, and unavailable
	
	$html = $PAGE_MASTER_UNAVAIL;
	
	$html =~ s/DOMAINNAME/$zonename/g;
	$html =~ s/FIRSTTABLESTRING/$FirstTableString/g;
}

else
{
	# Master, and available
	
	if ($iszonereverse eq 1)
	{
		$html = SetPage($PAGEREV, $FirstTableString, $Records, $zonename, $total, $SubzoneRecords, $sztotal);
	}
	else
	{
		$html = SetPage($PAGEFWD, $FirstTableString, $Records, $zonename, $total, $SubzoneRecords, $sztotal);
	}
}


my %var;

$var{'q'}			= $cgi;
$var{'html'}			= $html;
$var{'class'}			= 'server_dns';
$var{'id'}			= 8300;
$var{'title'}			= $_R{'IDT:8300'};
$var{'root-link'}		= 'server_dns';
$var{'result'}			= $Result;
$var{'feedback'}		= $Feedback;
$var{'cookiecrumbs'}  =	[	{name=>$StringResource->_S('IDT:8000'), location=>"server_dns.cgi"},
			];

DisplayMain->printDocument(\%var);

exit(0);

