#!/usr/bin/perl -w

###############################################################################
#                                                                             #
# Filename: server_domains.cgi                                                #
#                                                                             #
# Description: CGI needed to display and modify domain configurations.        #
#                                                                             #
###############################################################################
###############################################################################
#                                                                             #
#  Copyright  2000 - 2001 Intel Corporation.                                 #
#  Intel Corporation All Rights Reserved.                                     #
#                                                                             #
#  The source code contained or described herein and all documents related to #
#  the source code ("Material") are owned by Intel Corporation or its         #
#  suppliers or licensors.  Title to the Material remains with Intel          #
#  Corporation or its suppliers and licensors.  The Material contains trade   #
#  secrets and proprietary and confidential information of Intel or its       #
#  suppliers and licensors.  The Material is protected by worldwide copyright #
#  and trade secret laws and treaty provisions.  No part of the Material may  #
#  be used, copied, reproduced, modified, published, uploaded, posted,        #
#  transmitted, distributed, or disclosed in any way without Intel's prior    #
#  express written permission.                                                #
#                                                                             #
#  No license under any patent, copyright, trade secret or other intellectual #
#  property right is granted to or conferred upon you by disclosure or        #
#  delivery of the Materials, either expressly, by implication, inducement,   #
#  estoppel or otherwise. Any license under such intellectual property rights #
#  must be express and approved by Intel in writing.                          #
#                                                                             #
###############################################################################

# Put the PERL library in the path.
use lib '/usr/ins/intel/gui/perllib';

use strict;
use LanguageResource;
use LanguageLogCGI;
use VirtualDomain;
use VirtualServ;
use VirtualFtp;
use DisplayMain;
use MonSettings;
use CookieData;
use DomainMail;
use NetConfig;
use Quotaall;
use Mail;
use Safe;
use SSL;
use CGI;
use DNSConfig;
use DNSZone;

###############################################################################
# SortIP
###############################################################################

sub SortIP
{
	my @DomainList	= @_;
    
	@DomainList = sort
		{
			# Break up the number.
			my @a = split(/\./,$a->{'vdip'});
			my @b = split(/\./,$b->{'vdip'});
			my $i;

			# Run through the tuples...
			for $i (0..3) 
			{
				# If the tuples don't match...
				if ($a[$i] != $b[$i]) 
				{
					# Evaluate the current tuple.
					return ($a[$i] <=> $b[$i]);
				}
			}
			        
			# Evaluate the high tuple...
			$a[0] <=> $b[0];
		} @DomainList;

	return @DomainList;
}

###############################################################################
# ValidateFormData
###############################################################################

sub ValidateFormData
{
	my ($FormDataRef)	= @_;
	my $Return			= 0;
	my $Key;

	# Run through each of the keys...
	foreach $Key (keys(%{ $FormDataRef }))
	{
		# If the data is defined...
		if (defined($FormDataRef->{$Key}))
		{
			# Trim the whitespace.
			$FormDataRef->{$Key} =~ s/^\s*(.*?)\s*$/$1/;
		}
	}

#	my $baduser = '';
#	$baduser = getpwnam($FormDataRef->{'admin_name_login'});
		
	# Validate the data...
	if (($FormDataRef->{'name'} !~ /^(\w|-|\.|_)+$/) ||
		($FormDataRef->{'name'} =~ /^domain\d+$/))
	{
		# Set the return code.
		$Return = 260000;
	}
	elsif ($FormDataRef->{'alias'} !~ /^(\w|-|\.|_)*$/)
	{
		# Set the return code.
		$Return = 260001;
	}
	elsif (($FormDataRef->{'ip_address'} !~ /^(\d{1,3})\.(\d{1,3})\.(\d{1,3})\.(\d{1,3})$/) ||
		   (1 > $1) || (254 < $1) || (0 > $2) || (254 < $2) ||
		   (0 > $3) || (254 < $3) || (0 > $4) || (254 < $4))
	{
		# Set the return code.
		$Return = 260002;
	}
	elsif (($FormDataRef->{'network_mask'} !~ /^(\d{1,3})\.(\d{1,3})\.(\d{1,3})\.(\d{1,3})$/))
	{
		# Set the return code.
		$Return = 260003;
	}
	elsif ($FormDataRef->{'admin_name_full'} !~ /^\S+(\s*\S+)*$/)
	{
		# Set the return code.
		$Return = 260004;
	}
	elsif (	($FormDataRef->{'admin_name_login'} !~ /^(\w|-|_)*$/) ||
		($FormDataRef->{'admin_name_login'} eq 'root') )
	{
		# Set the return code.
		$Return = 260005;
	}
	elsif ('' ne scalar(getpwnam($FormDataRef->{'admin_name_login'})))
	{
		$Return = 260026;
	}
	elsif ($FormDataRef->{'password'} !~ /^\S+$/)
	{
		# Set the return code.
		$Return = 260006;
	}
	elsif ($FormDataRef->{'password'} ne $FormDataRef->{'password_confirm'})
	{
		# Set the return code.
		$Return = 260007;
	}
	elsif (($FormDataRef->{'disk_quota'} !~ /^\+*([0-9]*)\.?([0-9]*)$/) || 
	       (!($1 || ('0' eq $1) || $2 || ('0' eq $2))))
	{
		# Set the return code.
		$Return = 260008;
	}
	elsif ($FormDataRef->{'fpx_active'})
    {
		# Check the passwords...
		if ($FormDataRef->{'password_frontpage'} !~ /^\S+$/)
		{
			# Set the return code.
			$Return = 260009;
		}
		elsif ($FormDataRef->{'password_frontpage'} ne $FormDataRef->{'password_confirm_frontpage'})
		{
			# Set the return code.
			$Return = 260010;
		}
	}
	elsif ($FormDataRef->{'ssl_active'})
	{
		my @DomainList	= VirtualDomain::list_vws();
		my $Item;

		# Run through the domain list...
		foreach $Item (@DomainList) 
		{
			# If the addresses match but the domain does not...
			if ($FormDataRef->{'ip_address'} eq $Item->{'vdip'})
			{
				# Check for an existing virtual IP squatter...
				$Return = 260034 if(('yes' eq $Item->{'vdssl'}) && ( -e "$Item->{'dom_hom'}/$Item->{'dom_grp'}/cert/certificate" ));
			}

			# Bail out if needed...
			last if($Return);
		}
	}

	return $Return;
}

###############################################################################
# ConfigureSSL
###############################################################################

sub ConfigureSSL
{
	my ($FormDataRef)	= @_;
	my $SSLData			= new SSL('/etc/httpd/ssl');
	my $Return			= 0;

	# If the configuration is self-signed...
	if ('self' eq $FormDataRef->{'certificate_type'})
	{
		# Generate the private key, CSR, and certificate...
		unless ((0 == $SSLData->SetSubject($FormDataRef)) &&
				(defined($SSLData->GetCertificate(1))))
		{
			# Set the return code.
			$Return = 260017;
		}
	}
	else
	{
		# Clean off goo.
		$FormDataRef->{'private_key'}	=~ s/\r//g;
		$FormDataRef->{'certificate'}	=~ s/\r//g;

		# Generate the private key, CSR, and certificate...
		unless ((0 == $SSLData->SetPrivateKey($FormDataRef->{'private_key'})) &&
				(0 == $SSLData->SetCertificate($FormDataRef->{'certificate'})))
		{
			# Set the return code.
			$Return = 260018;
		}
	}

	# If things are running smoothly...
	unless ($Return)
	{
		my $PrivateKeyPath	= $SSLData->GetPrivateKeyPath();
		my $CertificatePath	= $SSLData->GetCertificatePath();
		my $CSRPath			= $SSLData->GetCertificateSigningRequestPath();

		# If the configuration paths are available...
		if (($PrivateKeyPath) &&
			($CertificatePath))
		{
			# Cache the files in a "well known" location.
			Safe::system("/bin/cp $CSRPath /etc/httpd/ssl/request");
			Safe::system("/bin/cp $PrivateKeyPath /etc/httpd/ssl/key");
			Safe::system("/bin/cp $CertificatePath /etc/httpd/ssl/certificate");
		}
		else
		{
			# Set the return code.
			$Return = 260019;
		}

		# Clean up the data.
		$SSLData->SetPrivateKey();
		$SSLData->SetCertificateSigningRequest();
		$SSLData->SetCertificate();
	}

	return $Return;
}

###############################################################################
# AddDomain
###############################################################################

sub AddDomain
{
	my ($cgi, $FormDataRef)	= @_;
	my $Return				= 0;

	# Convert to lower case.
	$FormDataRef->{'name'}	= "\L$FormDataRef->{'name'}";
	$FormDataRef->{'alias'} = "\L$FormDataRef->{'alias'}";

	# Try to open/create a lock file...
	if (open(LOCKFILE, ">> /var/lock/server.lock"))
	{
		# Try to lock the code chunk...
		if (flock(LOCKFILE, 2))
		{
			my @DomainList = VirtualDomain::list_vws();
			my $AddressIP;
			my $Name;
			my $Group;
			
			# Check for a squatter on the address.
			($AddressIP, $Name, $Group) = VirtualDomain::ip_domain($FormDataRef->{'ip_address'});

			# Make sure the IP address is not in use by another domain...
			if ('' ne $AddressIP) 
			{
				my $Item;
				
				# Run through the domain list...
				foreach $Item (@DomainList) 
				{
					# If the addresses match but the domain does not...
					if ($FormDataRef->{'ip_address'} eq $Item->{'vdip'})
					{
						# Check for an existing virtual IP squatter...
						$Return = 260033 if(VirtualFtp::getFtpStatus($Item->{'vdip'}, $Item->{'dom_grp'}));

						# Check for an existing virtual IP squatter...
						$Return = 260034 if(($FormDataRef->{'ssl_active'}) && ('yes' eq $Item->{'vdssl'}) && ( -e "$Item->{'dom_hom'}/$Item->{'dom_grp'}/cert/certificate" ));
					}

					# Bail out if needed...
					last if($Return);
				}

				# If things are still going good...
				unless ($Return)
				{
					# Make sure that Virtual FTP is not used on the shared IP...
					$Return = 260011 if($FormDataRef->{'ftp_active'});
				}
			}

			# If things are still going good...
			unless ($Return)
			{
				my $FreeSpace;
				my $Domain;

				# Run through the current domain list...
				foreach $Domain (@DomainList) 
				{
					# Check for an existing domain...
					if ($FormDataRef->{'name'} eq $Domain->{'vdn'}) 
					{
						# Set the return code.
						$Return = 260013;
						last;
					}
				}
				
				# If things are still going good...
				unless ($Return)
				{
					# Check for disk space against the current quota...
					$Return = 260014 if($FormDataRef->{'disk_quota'} >= Quotaall::comDomainQuota());
				}
				
				# Sanity checks for adding DNS configuration...

				unless ($Return)
				{
					# Perform a sample evaluation.

					my $dnsstat = EvaluateDNSCreation(	$FormDataRef->{'name'}, $FormDataRef->{'ip_address'},
										$FormDataRef->{'admin_name_login'}, '0');
					if ($dnsstat < 0)
					{
						# Could not evaluate with arguments given.  Fail!
						$Return = 260029;
					}
					elsif ($dnsstat eq 0)
					{
						# Can't create due to conflicts with other zones or records.
						$Return = 260030;
                                        }
					elsif ($dnsstat > 1)
					{
						# Already exists.
						$Return = 260032;
					}
					# Final case, $dnsstat eq 1, means success.
				
				}	

				# Configure SSL if necessary...
				if ((0 == $Return) &&
					($FormDataRef->{'ssl_active'}))
				{
					# Try to configure SSL.
					$Return = ConfigureSSL($FormDataRef);
				}
				
				# If things are still going good...
				unless ($Return)
				{
					# Get the next group information.
					my ($Group, $GroupID, $LastGroup)	= Users::domainGrp();
					my $SSLState 						= ($FormDataRef->{'ssl_active'}) ? 'Yes' : '';

					# Create a new group on the file system (very trusting).
					Users::grpAdd();

					# Try to create the domain home...
					if ('yes' eq VirtualDomain::create_dom_home($FormDataRef->{'name'}, $Group, 
																$FormDataRef->{'admin_name_login'}, $FormDataRef->{'password'}, 
																$FormDataRef->{'admin_name_full'}, $SSLState))
					{
						my @NetworkInterfaces	= NetConfig->listNetInterfaces();
						my $NetworkInterface;
						my $AddressIPExists		= 0;

						# Run through the list of network interfaces...
						foreach $NetworkInterface (@NetworkInterfaces) 
						{
							# Try to find the address in the interface list...
							if ($NetworkInterface->{'ipaddress'} eq $FormDataRef->{'ip_address'})
							{
								$AddressIPExists = 1;
								last;
							}
						}

						# Check to see if the address needs to be allocated...
						if (('' eq $AddressIP) && 
							(0 == $AddressIPExists))  
						{
							# Try to create the virtual IP address...
							$Return = ('yes' eq VirtualDomain::create_vip($FormDataRef->{'ip_address'}, $FormDataRef->{'network_mask'}, 
																		  $Group, $FormDataRef->{'admin_name_login'}, $FormDataRef->{'name'}, 
																		  $FormDataRef->{'ftp_active'})) ? 0 : 260016;
						} 
						
						# If things are still going good...
						unless ($Return)
						{
							my $Log			= new LanguageLogCGI('/usr/ins/intel/gui/logs/server.log', '/usr/ins/intel/gui/locale', [ 'server_domains.lrc' ]);
							my $Monitor		= new MonSettings($FormDataRef->{'name'});
							
							# Assume these will work (very trusting).
							VirtualDomain::create_vd($FormDataRef->{'ip_address'}, $FormDataRef->{'name'}, 
													 $FormDataRef->{'admin_name_login'}, $FormDataRef->{'alias'}, $Group, 
													 $FormDataRef->{'ssi_active'}, $FormDataRef->{'cgi_active'}, $SSLState, $FormDataRef->{'fpx_active'});
							VirtualServ::setHostFile($FormDataRef->{'name'}, '', $FormDataRef->{'ip_address'});
                            
							# Set the quota (very trusting).
							Quotaall::setquota($GroupID, $FormDataRef->{'disk_quota'}, 1);

							# If FrontPage is needed...
							if ($FormDataRef->{'fpx_active'}) 
							{
								# Activate the FrontPage extensions (very trusting).
								VirtualDomain::activateFpx($FormDataRef->{'name'}, (($FormDataRef->{'ssl_active'}) ? 443 : 80), 
														   $FormDataRef->{'admin_name_login'}, $FormDataRef->{'password_frontpage'}, $Group);
							}

							# Restart apache apache server.
							Safe::system("(/etc/rc.d/init.d/httpd stop; /bin/sleep 1; /etc/rc.d/init.d/httpd start)");
	
							# Configure mail (very trusting).
							Mail::set_mail_domain($Group, $FormDataRef->{'name'});
							Mail::set_mail_domain_aliases($Group, $FormDataRef->{'alias'});
							DomainMail::modAlias($FormDataRef->{'admin_name_login'}, $FormDataRef->{'name'}, 'postmaster,webmaster');
							Safe::system('/etc/rc.d/init.d/sendmail', 'restart');
						
							# Enable/disable mail support...
							if ($FormDataRef->{'mail_active'}) 
							{
								# Enable mail support (very trusting).
								Mail::enable_mail($Group, 3);	
							}
							else 
							{
								# Disable mail support (very trusting).
								Mail::disable_mail($Group, 3);
							}
							
							# Open the configuration.
							if (0 == $Monitor->open())
							{
								# Set the monitor states.
								$Monitor->setMonitorState('http', 1);
								$Monitor->setMonitorState('smtp', (($FormDataRef->{'mail_active'}) ? 1 : 0));
								$Monitor->setMonitorState('ftp', (('1' ne VirtualFtp::domainFtpStatus($FormDataRef->{'name'}, $Group, $FormDataRef->{'ip_address'})) ? 1 : 0));

								# Close the configuration.
								$Monitor->close();
							}

							# Begin evaluating the DNS checkbox and add zones/subzones as necessary.
							if ($FormDataRef->{'dns_active'})
							{
								# Begin parsing the domain name and see if it makes sense to add a zone.

								my $dnsstat = EvaluateDNSCreation(	$FormDataRef->{'name'}, $FormDataRef->{'ip_address'},
													$FormDataRef->{'admin_name_login'}, '1');

								# Do stuff with $dnsstat
								if ($dnsstat eq -1)
								{
									# Could not evaluate with arguments given.  Fail!
									$Return = 260029;
								}
								elsif ($dnsstat < -1)
								{
									# Error during creation!  Wither and die.
									$Return = 260028;
								}
								elsif ($dnsstat eq 0)
								{
									# Can't create due to conflicts with other zones or records.
									$Return = 260030;
								}
								elsif ($dnsstat > 1)
								{
									# Already exists.
									$Return = 260032;
								}
								# Final case, $dnsstat eq 1, means success.

							}
							
							# Log the event.
							$Log->Append('security', 'information', 260025, [ $FormDataRef->{'name'}, $cgi->remote_user() ]);
						}
					}
					else
					{
						# Set the return code.
						$Return = 260015;
					}
					
					# If there was an error...
					if ($Return)
					{
						# Try to clean up the mess...
						VirtualDomain::cleanup($FormDataRef->{'name'}, $Group, $FormDataRef->{'admin_name_login'}, $FormDataRef->{'ip_address'});
						
						# If SSL was configured...
						if ($FormDataRef->{'ssl_active'})
						{
							# Try to clean up the SSL files...
							unlink('/etc/httpd/ssl/key');
							unlink('/etc/httpd/ssl/request');
							unlink('/etc/httpd/ssl/certificate');
						}
					}
				}
			}
		}
		else
		{
			# Set the return code.
			$Return = 260012;
		}

		# Close the file.
		close(LOCKFILE);
	}
	else
	{
		# Set the return code.
		$Return = 260012;
	}

	return $Return;
}


###############################################################################
# EvaluateDNSCreation
###############################################################################

# Return codes:
#
# -2 - DNS zone/subzone creation process failed for any reason.
# -1 - missing arguments in function call.
# 0  - unable to create (or evaluate and find space for) DNS configuration, due
#      to conflicts in pre-existing zones, zone records, or subzone delegations.
# 1  - successfully created (or evaluated and found space for) DNS configuration
# 2  - DNS zone already existed as named.  Inform server administrator and ask them
#      to modify their DNS configuration manually for this new virtual domain.
# Possible outputs:
# $dowork eq 0 ->  -1,0,1,2
# $dowork eq 1 ->  -2,-1,0,1,2

sub EvaluateDNSCreation
{	
	my ($newtz1,$zoneipaddr,$zoneemail,$dowork) = @_;

	# Variable purposes:
	#
	# $newtz1     - (required) Name of new virtual domain, to use for DNS configuration
	# $zoneipaddr - (required) Virtual IP address of primary web/mail/FTP server for virtual domain
	# $zoneemail  - (optional) User name of new domain administrator, to use for DNS configuration
	# $dowork     - (optional) '0', 'no' or empty for "do no work, only check"
	#                          '1' or other text for "do all creation work"

	my ($ia,$ib,$ic,$ic1,$ic2,$ic3,$ic4);
	my (@a1,@a2,@a3);
	my ($t1,$t2,$t3,$unused);
	my @z_info;

	if (	($newtz1 eq '') || (!(defined($newtz1))) ||
		($zoneipaddr eq '') || (!(defined($zoneipaddr))) )
	{
		return -1;
		# Do nothing.
	}

	if (	($dowork eq '0') || ($dowork eq 'no') ||
		($dowork eq '') || (!(defined($dowork))) )
	{
		$dowork = 0;
	}
	else
	{
		$dowork = 1;
	}

	# Split out extra dots at the end of the name.
	while ($newtz1 =~ /(\.)$/)
	{
		$newtz1 = $`;
	}

	@a1 = split('\.',$newtz1);

	while ( ($a1[0] eq '') && (scalar(@a1) > 1) )
	{
		$unused = shift(@a1);
	}

	# Next look for well-known host name ("www") at the start of the zone name.
	if (lc($a1[0]) eq "www")
	{
		# If so, make a zone based on the remainder of the virtual domain name.

		$unused = shift(@a1);
	}

	if ( ($zoneemail eq '') || (!(defined($zoneemail))) )
	{
		$zoneemail = 'postmaster';
	}

	my $fullemail = $zoneemail . '@' . $newtz1;

	# Now start checking for the existence of the domain.  Process variables...

	@a2 = @a1;

	my $finishedwork = 0;
	my $unabletocreate = 0;
	my $found = 0;
	my $foundself = 0;
	my $createfirstparent = 1;
	my $failedprocess = 0;

	my @firstparentsoa;
	my $firstparentfile;
	
	@z_info = ['', '', '', '', ''];
	
	# Check for existence of ourself as a virtual domain on this server.
	my $z_check = join('.', @a2);
	$ia = DNSConfig->zone_getinfo(\@z_info, $z_check);

	# Does the zone exist here?
	if ( ($ia eq 0) && ($z_info[1] ne '') )
	{
		# Yes, it does exist.

		$found = 1;
		$foundself = 1;
		goto JUMPOUTDNSCHECK;

	}

	# If not, assume it doesn't, and check all subzones.

	
	while ( ($found eq 0) && (scalar(@a2) > 2) )
	{
		@z_info = ['', '', '', '', ''];

		# Begin a subzone check.
		# Pop off the first segment of the name and save to a new array.
		# Then check for the remainder of the array to see if it exists as a subzone.
		
		$t3 = '';
		$t3 = shift(@a2);
		push(@a3,$t3);

		@z_info = ['', '', '', '', ''];
		my $z_check = join('.', @a2);

		$ia = DNSConfig->zone_getinfo(\@z_info, $z_check);

		# Does the zone exist here, and is it a master zone?
		if ( ($ia eq 0) && ($z_info[0] ne '') && ($z_info[1] ne '') && ($z_info[2] ne '2') )
		{
			# If so, then check this subzone for incriminating records or delegations.
	
			$found = 1;

			my (@pz_soa,@pz_ns,@pz_mx,@pz_a,@pz_cname,@pz_ptr,@pz_nssub); 
			$ib = DNSZone->zone_getinfo($z_info[0],$z_info[1],\@pz_soa,\@pz_ns,\@pz_mx,\@pz_a,\@pz_cname,\@pz_ptr,\@pz_nssub);

			if ($ib eq 0)
			{
				# The zone exists and we can read its records.  Now let's check for any conflicts in the @pz_nssub arena.

				my $i;

				for ($i = 0; $i < scalar(@pz_nssub); $i++)
				{
					my @a4 = [];
					my ($t4,$t5);

					@a4 = @a3;

					my $foundsz = 0;

					while ( (scalar(@a4) > 0) && ($foundsz eq 0) )
					{
						$t4 = join('.',@a4);
						$t5 = $t4 . '.' . join('.',@a2);

						# Check for NS Record named $t4 or $t5.

						if ( ($t4 eq $pz_nssub[$i][0]) || ($t5 eq $pz_nssub[$i][0]) )
						{
							# Match!
							$foundsz = 1;
							$unabletocreate = 1;
							goto JUMPOUTDNSCHECK;
						}

						# If neither exists, remove first element of @a4 and go again.

						my $useless = shift(@a4);
					}
				}

				# Now check A records.

				for ($i = 0; $i < scalar(@pz_a); $i++)
				{
					my @a4 = [];
					my ($t4,$t5);

					@a4 = @a3;

					my $founda = 0;

					$t4 = join('.',@a4);
					$t5 = $t4 . '.' . join('.',@a2);
					# Check for A Record named $t4 or $t5.
						
					while ( (scalar(@a4) > 0) && ($founda eq 0) )
					{
						$t4 = join('.',@a4);
						$t5 = $t4 . '.' . join('.',@a2);

						# Check for A Record named $t4 or $t5.

						if ( ($t4 eq $pz_a[$i][0]) || ($t5 eq $pz_a[$i][0]) )
						{
							# Match!
							$founda = 1;
							$unabletocreate = 1;
							goto JUMPOUTDNSCHECK;
						}

						# If neither exists, remove first element of @a4 and go again.

						my $useless = shift(@a4);
					}
				}
					      
				# Now check CNAME records.

				for ($i = 0; $i < scalar(@pz_cname); $i++)
				{
					my @a4 = [];
					my ($t4,$t5);

					@a4 = @a3;

					my $foundcname = 0;

					$t4 = join('.',@a4);
					$t5 = $t4 . '.' . join('.',@a2);
					# Check for CNAME Record named $t4 or $t5.
					
					while ( (scalar(@a4) > 0) && ($foundcname eq 0) )
					{
						$t4 = join('.',@a4);
						$t5 = $t4 . '.' . join('.',@a2);

						# Check for CNAME Record named $t4 or $t5.

						if ( ($t4 eq $pz_cname[$i][0]) || ($t5 eq $pz_cname[$i][0]) )
						{
							# Match!
							$foundcname = 1;
							$unabletocreate = 1;
							goto JUMPOUTDNSCHECK;
						}

						# If neither exists, remove first element of @a4 and go again.

						my $useless = shift(@a4);
					}
				}

				# At this point, it is safe to assume that the zone does exist, but no conflicting records or
				# subzone delegations are present.  Therefore, we can make a subzone now.

				# Parent zone will be join('.',@a2)
				# Subzone name will be join('.',@a3)

				# Inherit fun properties from parent zone, and assemble new SOA array.

				$createfirstparent = 0;
				
				@firstparentsoa = @pz_soa;
				$firstparentsoa[0] = '';
				$firstparentsoa[1] = '';
				$firstparentsoa[2] = $fullemail;
				$firstparentsoa[3] = '';

				$firstparentfile = $z_info[1];

				goto JUMPOUTDNSCHECK;

			}
		}
	}
	
	JUMPOUTDNSCHECK:


	if ( ($dowork eq 1) && ($finishedwork eq 0) && ($unabletocreate ne 1) && ($foundself ne 1) )
	{
		# Do work here.

		my (@new_soa,@new_ns,@new_mx,@new_a,@new_cname,@new_ptr,@new_nssub);

		my $new_sub_done;
		my $prev_file;

		my $new_zone_file;
		my $new_zone_full = join('.',@a2);
		my $new_zone_full_dot = $new_zone_full . '.';

		if ($createfirstparent eq 1)
		{
			# Begin with the assumption that @a2 is our final domain name,
			# and @a3 is our host portion.  Assume that size of @a2 is at least 1, hopefully 2 or greater.

			# Just make a domain with @a2 as the name.

			$ia = DNSConfig->zone_add($new_zone_full,\$new_zone_file, '1');

			if ($ia eq 0)
			{
				$ib = DNSZone->zone_construct_fwd($new_zone_full_dot,$new_zone_file,$zoneipaddr);

				if ($ib eq 0)
				{
					# Looks like we successfully created the zone.  Congratulations.

					my (@new_ns,@new_mx,@new_a,@new_cname,@new_ptr,@new_nssub);

					$ic1 = DNSZone->zone_getinfo($new_zone_full_dot,$new_zone_file,\@new_soa,\@new_ns,\@new_mx,\@new_a,\@new_cname,\@new_ptr,\@new_nssub);

					if ($ic1 eq 0)
					{
						$new_soa[2] = $fullemail;
						$new_soa[4] = '8H';
						$new_soa[5] = '2H';
						$new_soa[6] = '1W';
						$new_soa[7] = '1D';

						$ic2 = DNSZone->zone_changeheader($new_zone_full_dot,$new_zone_file,\@new_soa);

						if ($ic2 eq 0)
						{
							$prev_file = $new_zone_file;	
						}
						
					}
				} 
			}
		
			if ( ($ia ne 0) || ($ib ne 0) || ($ic1 ne 0) || ($ic2 ne 0) )
			{
				$failedprocess = 1;
				goto JUMPFAILANDQUIT;
			}
		
		}
		
		else
		{
			@new_soa = @firstparentsoa;
			$prev_file = $firstparentfile;
		}
		
		$new_sub_done = '';

		while (scalar(@a3) > 0)
		{
			my @asub;

			@asub = @a3;

			my $new_sub_file;
			my $new_sub_short = pop(@a3);
			my $new_sub_full = $new_sub_short . '.' . $new_sub_done . $new_zone_full;
			my $new_sub_full_dot = $new_sub_full . '.';

#			# About to begin (via loop) making subzone chain @a3 inside parent zone @a2

			# Make the new subzone first.
			$ia = DNSConfig->zone_add($new_sub_full,\$new_sub_file, '1');

			if ($ia eq 0)
			{
				# Next run the constructor for the subzone.

				$ib = DNSZone->zone_construct_fwd($new_sub_full_dot,$new_sub_file,$zoneipaddr);

				if ($ib eq 0)
				{
					# Looks like we successfully created the subzone.  Congratulations.

					my (@new_ns,@new_mx,@new_a,@new_cname,@new_ptr,@new_nssub);

					$ic1 = DNSZone->zone_getinfo($new_sub_full_dot,$new_sub_file,\@new_soa,\@new_ns,\@new_mx,\@new_a,\@new_cname,\@new_ptr,\@new_nssub);

					if ($ic1 eq 0)
					{
						$new_soa[2] = $fullemail;
						$new_soa[4] = '8H';
						$new_soa[5] = '2H';
						$new_soa[6] = '1W';
						$new_soa[7] = '1D';
						
						$ic2 = DNSZone->zone_changeheader($new_sub_full_dot,$new_sub_file,\@new_soa);

						if ($ic2 eq 0)
						{
							# Okay, now we're really hoping that this will all work smoothly.
							# Let's add stuff to the parent zone, shall we?

							$ic3 = DNSZone->zone_addrecord($new_sub_done . $new_zone_full,$prev_file,"NS",$new_sub_short, 'dns.' . $new_sub_done . $new_zone_full_dot);
							
							if ($ic3 eq 0)
							{
								$ic4 = DNSZone->zone_increment($new_sub_done . $new_zone_full,$prev_file);
								
								if ($ic4 eq 0)
								{
									# If this is okay, push values and move on.

									$prev_file = $new_sub_file;
									my $t7 = $new_sub_short . '.' . $new_sub_done;
									$new_sub_done = $t7;
								}
							
							}
						
						}

					}

				}
			
			}

			if ( ($ia ne 0) || ($ib ne 0) || ($ic1 ne 0) || ($ic2 ne 0) || ($ic3 ne 0) || ($ic4 ne 0) )
			{
				# failure, so quitting now.
				$failedprocess = 1;
				goto JUMPFAILANDQUIT;
			}

		}
	
	
	}
	elsif ($foundself eq 1)
	{
		return 2;
	}
	elsif ($unabletocreate eq 1)
	{
		return 0;
	}

	# Everything appears to be successful.

	return 1;

	JUMPFAILANDQUIT:

	if ($failedprocess eq 1)
	{
		return -2;
	}
	
	return 0;
}


###############################################################################
# DeleteDomains
###############################################################################

sub DeleteDomains
{
	my ($cgi, $DomainNameListRef, $DomainListRef)	= @_;
	my @TargetList;
	my $DomainName;
	my $Domain;
	my $Return;

	# Run through all of the selected domains...
	foreach $DomainName (@{ $DomainNameListRef })
	{ 
		# Skip the controller checkbox.
		next if($DomainName eq 'domain1');

		# Run through the domain list...
		foreach $Domain (@{ $DomainListRef })
		{
			# If the name matches...
			if ($DomainName eq $Domain->{'vdn'})
			{
				# If the domain is not the default domain...
				if ('domain1' ne $Domain->{'dom_grp'})
				{
					# Add the domain to the target list.
					push(@TargetList, $Domain);
				}
				else
				{
					$Return = 260020;
					last;
				}
			}
		}

		last if($Return);
	}

	# If things are going smooth...
	unless ($Return)
	{
		my $Log		= new LanguageLogCGI('/usr/ins/intel/gui/logs/server.log', '/usr/ins/intel/gui/locale', [ 'server_domains.lrc' ]);
		my $Monitor;
		my @Users;
		my $User;

		# Run through the targeted domain list...
		foreach $Domain (@TargetList)
		{
			# Delete the domain.
			VirtualDomain::del_vws($Domain->{'vdn'}, $Domain->{'dom_grp'}, $Domain->{'vdadm'}, $Domain->{'vdip'});

			# Get the user list.
			@Users = Users::listUsrs();

			# Run through the user list...
			foreach $User (@Users) 
			{
				# Look for a matching user of the domain...
				if ($User->{'domain'} eq $Domain->{'dom_grp'}) 
				{
					# Delete the user.
					Users::usrDel($User->{'user'});
				}
			}

			# Delete the virtual ip address...
			VirtualDomain::delVip($Domain->{'vdn'}, $Domain->{'dom_grp'}, $Domain->{'vdadm'}, $Domain->{'vdip'});

			# Check the domain FTP status...
			if (1 == VirtualFtp::domainFtpStatus($Domain->{'vdn'}, $Domain->{'dom_grp'}, $Domain->{'vdip'}))
			{
				# If down to one in the group...
				VirtualFtp::enableVftp($Domain->{'vdn'}, $Domain->{'dom_grp'}, $Domain->{'vdip'});
				VirtualFtp::ftpserverDel($Domain->{'vdip'}, $Domain->{'dom_grp'});		
			}

			# Delete any FTP remnants.
			VirtualDomain::delFtp($Domain->{'vdip'}, $Domain->{'dom_grp'});

			# Set quota to null for the domain ...
			Quotaall::setquota(getgrnam($Domain->{'dom_grp'}), 0, 1);

			# Delete all the users of that domains.
			Mail::set_mail_domain($Domain->{'dom_grp'});
			Mail::set_mail_domain_aliases($Domain->{'dom_grp'});
			DomainMail::modAlias($Domain->{'vdadm'});

			# Delete entry from /etc/hosts.
			VirtualServ::setHostFile($Domain->{'vdn'}, '', $Domain->{'vdip'}, 'Delete');

			# Get a monitor.
			$Monitor = new MonSettings($Domain->{'vdn'});
			
			# Open the configuration.
			if (0 == $Monitor->open())
			{
				# Delete the domain reference.
				$Monitor->deleteDomainName();
		
				# Close the configuration.
				$Monitor->close();
			}

			# Log the event.
			$Log->Append('security', 'information', 260024, [ $Domain->{'vdn'}, $cgi->remote_user() ]);
			
			$Return = 0;
		}
	}
	
	return $Return;
}

###############################################################################
# RenderDefault
###############################################################################

sub RenderDefault
{
	my ($cgi, $DisplayDataRef) = @_;
	my $Return;

	# Check for a button press...
	if (defined($cgi->param('domain_add')))
	{
		# Redirect to the domain add.
		$Return = 'domain_add';
	}
	else
	{
		my $PreferredLanguage	= (split(/,/, $ENV{'HTTP_ACCEPT_LANGUAGE'}))[0];
		my $StringResource		= new LanguageResource($PreferredLanguage, '/usr/ins/intel/gui/locale', [ 'server_domains.lrc', 'common.lrc' ]);
		my %_R					= ( 'IDT:2500'				=> '',
									'FIND'					=> '',
									'ADD'					=> '',
									'DELETE'				=> '',
									'CONFIRM_DOMAIN_DELETE' => '',
									'NO_DOMAINS_SELECTED'	=> '',
									'SHOW_ALL'				=> '',
									'NAME'					=> '',
									'IP_ADDRESS'			=> '',
									'ADMINISTRATOR'			=> '' );
		my @DomainList			= VirtualDomain::list_vws();
		my $DomainHTML			= '';
		my $SortControl			= $cgi->param('sort_control');
		my $Search				= $cgi->param('search');
		my $Feedback;
		my $Result;
		my $html;
	
		# Get the string resources.
		$StringResource->GetStrings(\%_R);
		
		# Safe default(s).
		$SortControl	= 'none' unless(defined($SortControl));
		$Search			= '' unless(defined($Search));
	
		# If applying a change...
		if (defined($cgi->param('find')))
		{
			# Trim the whitespace.
			$Search =~ s/^\s*(.*?)\s*$/$1/;

			# Run through the list...
			for (my $i = 0; $i < scalar(@DomainList); ++$i)
			{
				# Try to match with the search...
				unless ($DomainList[$i]->{'vdn'} =~ /$Search/)
				{
					# Delete the item.
					splice(@DomainList, $i, 1);

					# Decrement the index.
					--$i;
				}
			}
		}
		elsif (defined($cgi->param('domain_delete')))
		{
			# Try to delete the specified domains.
			$Result = DeleteDomains($cgi, $cgi->param_fetch('check'), \@DomainList);
			
			# Get the new list.
			@DomainList = VirtualDomain::list_vws();

            # If there was an error...
			if ($Result)
			{
				# Get the error string.
				$Feedback = $StringResource->_S($Result);
			}
		}
		
		SWITCH:
		{
			# Apply any sorting...
			if ('domain'  eq $SortControl) { @DomainList = sort { $a->{'vdn'} cmp $b->{'vdn'} } @DomainList;     last SWITCH; }
			if ('address' eq $SortControl) { @DomainList = SortIP(@DomainList);                                  last SWITCH; }
			if ('user'    eq $SortControl) { @DomainList = sort { $a->{'vdadm'} cmp $b->{'vdadm'} } @DomainList; last SWITCH; }
		}
	
		# If there are any domains...
		if (scalar(@DomainList))
		{
			my $Domain;
	
			# Run through the domain list...
			foreach $Domain (@DomainList)
			{
				$DomainHTML .= <<HERE;		
		<tr align=left valign=center>
			<td><input type=checkbox name=check value='$Domain->{'vdn'}'></td>
			<td><a href=/login/index.cgi?domain=$Domain->{'vdn'} target='$Domain->{'vdn'}'>$Domain->{'vdn'}</a></td>
			<td>$Domain->{'vdip'}</td>
			<td><a href=/login/index.cgi?user=$Domain->{'vdadm'}&domain=$Domain->{'vdn'} target='$Domain->{'vdadm'}'>$Domain->{'vdadm'}</a></td>
		</tr>
HERE
			}
		}
		else
		{
			$DomainHTML .= '<tr align=left valign=center><td>&nbsp;</td><td colspan=3><b>' . $StringResource->_S('NO_DOMAINS_AVAILABLE') . '</b></td></tr>'
		}
	
		$html = <<HERE;
<script src=/utilities.js language=javascript></script>
<script language=javascript>
function confirmDeleteDomain()
{
	var bReturn = false;
	
	for(i = 1; i < document.server_domains.check.length; i++)
	{
		if (document.server_domains.check[i].checked)
		{
			return confirm('$_R{'CONFIRM_DOMAIN_DELETE'}');
		}
	}
	
	if (i == document.server_domains.check.length)
	{
		alert('$_R{'NO_DOMAINS_SELECTED'}');
	}
	
	return bReturn;
}
</script>
<form method=post name=server_domains action=server_domains.cgi enctype="application/x-www-form-urlencoded">
	<input type=hidden name=type value=default>
	<input type=hidden name=sort_control value=$SortControl>
	<table border=0 bgcolor=#eeeeee width=375>
	<tr>
		<td bgcolor=#eeeeee>
			<table width=100%>
			<tr>
				<td>
					<table border=0 width=100% bgcolor=#eeeeee>
					<tr align=left valign=center><b>$_R{'FIND'}</b>
						<td>
							<input type=text name=search value='$Search' size=20 maxlength=72>
							<input type=submit name=find value='$_R{'FIND'}' class=clsButton>
						</td>
						<td align=right>
							<input type=submit name=show_all value='$_R{'SHOW_ALL'}' class=clsButton>
						</td>
					</tr>
					</table>
				</td>
			<tr>
			</table>
		</td>
	</tr>
	<tr>
		<td>
			<table border=0>
			<tr>
				<td align=left nowrap colspan=2>
					<input type=submit name=domain_add value='$_R{'ADD'}' class=clsButton>
					<input type=submit name=domain_delete value='$_R{'DELETE'}' class=clsButton onclick="return confirmDeleteDomain();">
				</td>
			</tr>
			</table>
		</td>
	</tr>
	<tr>
		<td>&nbsp;</td>
	</tr>
	<tr>
		<td>
			<table bordercolor=#eeeeee border=1 cellpadding=2 width=375 bgcolor=#ffffff cellspacing=0>
			<tr align=left valign=center>
				<td bgcolor=#808080 width=1%><input type=checkbox name=check onclick=checkAll(document.server_domains.check) value=domain1></td>
				<td bgcolor=#808080><a href=server_domains.cgi?sort_control=domain class=clsGroup>$_R{'NAME'}</td>
				<td bgcolor=#808080><a href=server_domains.cgi?sort_control=address class=clsGroup>$_R{'IP_ADDRESS'}</td>
				<td bgcolor=#808080><a href=server_domains.cgi?sort_control=user class=clsGroup>$_R{'ADMINISTRATOR'}</td>
			</tr>
$DomainHTML
			</table>
		</td>
	</tr> 
	</table>
</form>
HERE
		
		# Overriding result...
		$Result = $DisplayDataRef->{'result'} unless(defined($Result));
        
		# Fill out the display information.
		$DisplayDataRef->{'q'}			= $cgi;
		$DisplayDataRef->{'html'}		= $html;
		$DisplayDataRef->{'class'}		= 'server_domains';
		$DisplayDataRef->{'id'}			= 2500;
		$DisplayDataRef->{'title'}		= $_R{'IDT:2500'};
		$DisplayDataRef->{'root-link'}	= 'server_domains';
		$DisplayDataRef->{'result'}		= $Result;
		$DisplayDataRef->{'feedback'}	= $Feedback;

		$Return = 0;
	}
	
	return $Return;
}

###############################################################################
# RenderDomainAdd
###############################################################################

sub RenderDomainAdd
{
	my ($cgi, $DisplayDataRef, $Redirect) = @_;
	my $PreferredLanguage	= (split(/,/, $ENV{'HTTP_ACCEPT_LANGUAGE'}))[0];
	my $StringResource		= new LanguageResource($PreferredLanguage, '/usr/ins/intel/gui/locale', [ 'server_domains.lrc', 'common.lrc' ]);
	my $Cookie				= new CookieData();
	my %FormData			= (	'sort_control'					=> '',
								'name'							=> '',
								'alias'							=> '',
								'ip_address'					=> '',
								'network_mask'					=> '255.255.255.0',
								'admin_name_full'				=> '',
								'admin_name_login'				=> '',
								'password'						=> '',
								'password_confirm'				=> '',
								'disk_quota'					=> 200,
								'mail_active'					=> 'checked',
								'ftp_active'					=> '',
								'ssi_active'					=> '',
								'cgi_active'					=> '',
								'fpx_active'					=> '',
								'dns_active'					=> '',
								'password_frontpage'			=> '', 
								'password_confirm_frontpage'	=> '' );
	my $CookieFile			= '/usr/ins/intel/gui/cookies/domain_add-' . $cgi->remote_user();
	my $Key;
	my $SSLDetails;
	my $Feedback;
	my $Result;
	my $html;
	my $Return;

	# If adding a domain...
	if ((defined($cgi->param('add'))) ||
		(defined($cgi->param('configure_ssl'))))
	{
		# Get the form data from the form...
		foreach $Key (keys(%FormData))
		{
			# Get the data...
			$FormData{$Key} = (defined($cgi->param($Key))) ? $cgi->param($Key) : '';
		}

		# If adding the domain...
		if (defined($cgi->param('add')))
		{
			# Attach to the cookie file.
			$Cookie->Attach($CookieFile);
			
			# Get the SSL data.
			$FormData{'ssl_active'}			= $Cookie->GetValueKeyed('ssl_active');
			$FormData{'certificate_type'}	= $Cookie->GetValueKeyed('certificate_type');

			# Detach from the cookie.
			$Cookie->Detach();
			
			# Validate the form data.
			if (0 == ($Result = ValidateFormData(\%FormData)))
			{
				my @KeysSSL	= ( 'country',
								'province',
								'locality',
								'organization',
								'organizational_unit',
								'common_name',
								'e_mail',
								'private_key',
								'certificate' );

				# Attach to the cookie file.
				$Cookie->Attach($CookieFile);
				
				# Run through the SSL keys.
				foreach $Key (@KeysSSL)
				{
					# Import the SSL entry.
					$FormData{$Key} = $Cookie->GetValueKeyed($Key);
					
					# Trim the whitespace.
					$FormData{$Key} =~ s/^\s*(.*?)\s*$/$1/ if($FormData{$Key});
				}
				
				# Detach from the cookie.
				$Cookie->Detach();

				# Try to add the domain...
				if (0 == ($Result = AddDomain($cgi, \%FormData)))
				{
					# Set the result.
					$DisplayDataRef->{'result'}	= $Result;
					
					# Move on to the main page.
					$Return = 'default';
				}
			}

			# If there was an error...
			if ($Result)
			{
				# Get the error string.
				$Feedback = $StringResource->_S($Result);
			}
		}
	}
	elsif (!$Redirect)
	{
		# Attach to the cookie file.
		$Cookie->Attach($CookieFile);
		
		# Get the form data from the cookie...
		foreach $Key (keys(%FormData))
		{
			# Get the data.
			$FormData{$Key} = (defined($Cookie->GetValueKeyed($Key))) ? $Cookie->GetValueKeyed($Key) : '';
		}

		# Detach from the cookie.
		$Cookie->Detach();
	}
	else
	{
		# Get the sort control.
		$FormData{'sort_control'} = $cgi->param('sort_control');

		# Attach to the cookie file.
		$Cookie->Attach($CookieFile);
		
		# Delete the cookie.
		$Cookie->Delete();
	}

	# Attach to the cookie file.
	$Cookie->Attach($CookieFile);
	
	# Run through all of the keys...
	foreach $Key (keys(%FormData))
	{
		# Save the data.
		$Cookie->SetValueKeyed($Key, $FormData{$Key});
	}
	
	# Get the SSL data.
	$FormData{'ssl_active'}			= (defined($Cookie->GetValueKeyed('ssl_active'))) ? $Cookie->GetValueKeyed('ssl_active') : '';
	$FormData{'certificate_type'}	= (defined($Cookie->GetValueKeyed('certificate_type'))) ? $Cookie->GetValueKeyed('certificate_type') : '';
    
	# Detach from the cookie.
	$Cookie->Detach();

	# Re-align the checkboxes...
	$FormData{'dns_active'}		= ($FormData{'dns_active'}) ? 'checked' : '';
	$FormData{'mail_active'}	= ($FormData{'mail_active'}) ? 'checked' : '';
	$FormData{'ftp_active'}		= ($FormData{'ftp_active'}) ? 'checked' : '';
	$FormData{'ssi_active'}		= ($FormData{'ssi_active'}) ? 'checked' : '';
	$FormData{'cgi_active'}		= ($FormData{'cgi_active'}) ? 'checked' : '';
	$FormData{'fpx_active'}		= ($FormData{'fpx_active'}) ? 'checked' : '';

	# If a destination is not set...
	unless($Return)
	{
		my %_R	= ( 'DOMAIN_NAME'		=> '',
					'DOMAIN_ALIAS'		=> '',
					'IP_ADDRESS_LABEL'	=> '',
					'NETMASK'		=> '',
					'ADMIN_FULL_NAME'	=> '',
					'ADMIN_LOGIN_NAME'	=> '',
					'PASSWORD'		=> '',
					'PASSWORD_CONFIRM'	=> '',
					'DOMAIN_DISK_QUOTA'	=> '',
					'MB'			=> '',
					'ADD_DNS_FOR_DOMAIN'	=> '',
					'MAIL_SUPPORT'		=> '',
					'FTP_SUPPORT'		=> '',
					'USE_SSL'		=> '',
					'CONFIGURE_SSL'		=> '',
					'SSI_SUPPORT'		=> '',
					'CGI_SUPPORT'		=> '',
					'FRONTPAGE_SUPPORT'	=> '',
					'ADD'			=> '' );

		# Get the string resources.
		$StringResource->GetStrings(\%_R);

		# If configuring SSL...
		if (defined($cgi->param('configure_ssl')))
		{
			# Validate the data...
			if (($FormData{'name'} !~ /^(\w|-|\.|_)+$/) ||
				($FormData{'name'} =~ /^domain\d+$/))
			{
				# Set the return code.
				$Result = 260021;
			}
			elsif (($FormData{'ip_address'} !~ /^(\d{1,3})\.(\d{1,3})\.(\d{1,3})\.(\d{1,3})$/) ||
				   (1 > $1) || (254 < $1) || (0 > $2) || (254 < $2) ||
				   (0 > $3) || (254 < $3) || (0 > $4) || (254 < $4))
			{
				# Set the return code.
				$Result = 260022;
			}

			# If there are no problems...
			unless (defined($Result))
			{
				my $URL	= "/cgi-bin/common/ssl_forms.cgi?type=default&cookie-file=domain_add-" . $cgi->remote_user();
				
				# Attach to the cookie file.
				$Cookie->Attach($CookieFile);

				# Set the POST URI.
				$Cookie->SetValueKeyed('post_uri', '/cgi-bin/server/server_domains.cgi?type=domain_add');

				# Set the root link.
				$Cookie->SetValueKeyed('root-link', 'server_domains');

				# Set up the cookie crumbs.
				$Cookie->SetValueKeyed('b-class0', 'server_domains');
				$Cookie->SetValueKeyed('b-label0', 'IDT:2500');
				$Cookie->SetValueKeyed('b-rpath0', '/usr/ins/intel/gui/locale');
				$Cookie->SetValueKeyed('b-uri0',   '/cgi-bin/server/server_domains.cgi?type=default');
				$Cookie->SetValueKeyed('b-class1', 'server_domains');
				$Cookie->SetValueKeyed('b-label1', 'IDT:2600');
				$Cookie->SetValueKeyed('b-rpath1', '/usr/ins/intel/gui/locale');
				$Cookie->SetValueKeyed('b-uri1',   '/cgi-bin/server/server_domains.cgi?type=domain_add');

				# Prime the common name.
				$Cookie->SetValueKeyed('common_name', $FormData{'name'});

				# Try to prime the e-mail address...
				if (($FormData{'name'}) && ($FormData{'admin_name_login'}))
				{
					# Prime the common name.
					$Cookie->SetValueKeyed('e_mail', $FormData{'admin_name_login'} . '@' . $FormData{'name'});
				}
				else
				{
					# Prime the common name.
					$Cookie->SetValueKeyed('e_mail');
				}

				# Detach from the cookie.
				$Cookie->Detach();

				# Redirect to the SSL forms.
				print $cgi->redirect( -url => $URL );
				return;
			}
			else
			{
				# Get the error string.
				$Feedback = $StringResource->_S($Result);
			}
		}

		# Set the SSL status text...
		if ($FormData{'ssl_active'})
		{
			# Using SSL...
			$SSLDetails = '<font color=green>' . (('self' eq $FormData{'certificate_type'}) ? $StringResource->_S('SSL_CONFIGURED_SELF') : $StringResource->_S('SSL_CONFIGURED_CA')) . '<font>';
		}
		else
		{
			# Not using SSL.
			$SSLDetails = '<font color=red>' . $StringResource->_S('SSL_NOT_CONFIGURED') . '<font>';
		}
		
		$FormData{'admin_name_full'} =~ s/'/&#039;/g;
		$html = <<HERE;
<form method=post action=server_domains.cgi enctype="application/x-www-form-urlencoded">
	<input type=hidden name=type value=domain_add>
	<input type=hidden name=sort_control value=$FormData{'sort_control'}>
	<table border=0 width=375 bgcolor=#eeeeee>
	<tr>
		<td>
			&nbsp;
			<table border=0 width=100%>
			<tr>
				<td><b>$_R{'DOMAIN_NAME'}</b></td>
				<td><input type=text size=25 maxlength=72 name=name value='$FormData{'name'}'></td>
			</tr>
			<tr>
				<td><b>$_R{'DOMAIN_ALIAS'}</b></td>
				<td><input type=text size=25 maxlength=72 name=alias value='$FormData{'alias'}'></td>
			</tr>
			<tr>
				<td><b>$_R{'IP_ADDRESS_LABEL'}</b></td>
				<td><input type=text size=15 maxlength=15 name=ip_address value='$FormData{'ip_address'}'></td>
			</tr>
			<tr>
				<td><b>$_R{'NETMASK'}</b></td>
				<td><input type=text size=15 maxlength=15 name=network_mask value='$FormData{'network_mask'}'></td>
			</tr>
			<tr>
				<td colspan=2>&nbsp</td>
			</tr>
			<tr>
				<td nowrap><b>$_R{'ADMIN_FULL_NAME'}</b></td>
				<td><input type=text size=20 maxlength=32 name=admin_name_full value='$FormData{'admin_name_full'}'></td>
			</tr>
			<tr>
				<td nowrap><b>$_R{'ADMIN_LOGIN_NAME'}</b></td>
				<td><input type=text size=20 maxlength=32 name=admin_name_login value='$FormData{'admin_name_login'}'></td>
			</tr>
			<tr>
				<td nowrap><b>$_R{'PASSWORD'}</b></td>
				<td><input type=password size=20 maxlength=32 name=password value='$FormData{'password'}'></td>
			</tr>
			<tr>
				<td nowrap><b>$_R{'PASSWORD_CONFIRM'}</b></td>
				<td><input type=password size=20 maxlength=32 name=password_confirm value='$FormData{'password_confirm'}'></td>
			</tr>
			<tr>
				<td colspan=2>&nbsp</td>
			</tr>
			<tr>
				<td nowrap><b>$_R{'DOMAIN_DISK_QUOTA'}</b></td>
				<td><input type=text size=4 maxlength=10 name=disk_quota value='$FormData{'disk_quota'}'>$_R{'MB'}</td>
			</tr>
			<tr>
				<td nowrap><b>$_R{'USE_SSL'}</b></td>
				<td><input type=submit name=configure_ssl value='$_R{'CONFIGURE_SSL'}' class=clsButton></td>
			</tr>
			<tr>
				<td colspan=2>&nbsp;$SSLDetails<br>&nbsp;</td>
			</tr>
			
			
			<tr>
				<td nowrap colspan=2><input type=checkbox value=Yes name=dns_active $FormData{'dns_active'}>&nbsp;<b>$_R{'ADD_DNS_FOR_DOMAIN'}</b></td>
			</tr>
			<tr>
				<td nowrap colspan=2><input type=checkbox value=Yes name=mail_active $FormData{'mail_active'}>&nbsp;<b>$_R{'MAIL_SUPPORT'}</b></td>
			</tr>
			<tr>
				<td nowrap colspan=2><input type=checkbox value=Yes name=ftp_active $FormData{'ftp_active'}>&nbsp;<b>$_R{'FTP_SUPPORT'}</b></td>
			</tr>
			<tr>
				<td nowrap colspan=2><input type=checkbox value=Yes name=ssi_active $FormData{'ssi_active'}>&nbsp;<b>$_R{'SSI_SUPPORT'}</b></td>
			</tr>
			<tr>
				<td nowrap colspan=2><input type=checkbox value=Yes name=cgi_active $FormData{'cgi_active'}>&nbsp;<b>$_R{'CGI_SUPPORT'}</b></td>
			</tr>
			<tr>
				<td nowrap colspan=2><input type=checkbox value=Yes name=fpx_active $FormData{'fpx_active'}>&nbsp;<b>$_R{'FRONTPAGE_SUPPORT'}</b></td>
			</tr>
			<tr>
				<td nowrap><b>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;$_R{'PASSWORD'}</b></td>
				<td><input type=password size=20 maxlength=25 name=password_frontpage value='$FormData{'password_frontpage'}'></td>
			</tr>
			<tr>
				<td nowrap><b>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;$_R{'PASSWORD_CONFIRM'}</b></td>
				<td><input type=password size=20 maxlength=25 name=password_confirm_frontpage value='$FormData{'password_confirm_frontpage'}'></td>
			</tr>
			<tr>
				<td colspan=2>&nbsp</td>
			</tr>
			<tr>
				<td colspan=2 align=right><input type=submit name=add value='$_R{'ADD'}' class=clsButton></td>
			</tr>
			</table>
		</td>
	</tr>
	</table>
</form>
HERE
		
		# Fill out the display information.
		$DisplayDataRef->{'q'}				= $cgi;
		$DisplayDataRef->{'html'}			= $html;
		$DisplayDataRef->{'class'}			= 'server_domains';
		$DisplayDataRef->{'id'}				= 2600;
		$DisplayDataRef->{'title'}			= $StringResource->_S('IDT:2600');
		$DisplayDataRef->{'root-link'}		= 'server_domains';
		$DisplayDataRef->{'helpIndicator'}	= 'domain_add';
		$DisplayDataRef->{'result'}			= $Result;
		$DisplayDataRef->{'feedback'}		= $Feedback;
		$DisplayDataRef->{'cookiecrumbs'}	= [ { 'name' => $StringResource->_S('IDT:2500'), 'location' => 'server_domains.cgi' } ];

		$Return = 0;
	}
	
	return $Return;
}

###############################################################################
# Main
###############################################################################

sub Main
{
	my $cgi			= shift;
	my %DisplayData;
	my $FormType	= $cgi->param('type');
	my $Redirect	= 0;

	# Safe default.
	$FormType = 'default' unless(defined($FormType));
	
	SWITCH:
	{
		if ('default'    eq $FormType) { $FormType = RenderDefault($cgi, \%DisplayData);   				last SWITCH; }
		if ('domain_add' eq $FormType) { $FormType = RenderDomainAdd($cgi, \%DisplayData, $Redirect);	last SWITCH; }
		$FormType = 0;
	}

	# If there is still some form data...
	if ($FormType)
	{
		# Set the redirect.
		$Redirect = 1;
		goto SWITCH;
	}
	else
	{
		# Display the page.
		DisplayMain->printDocument(\%DisplayData);
	}
}

###############################################################################
# The main body of the script.
###############################################################################

my $cgi = new CGI;

Main($cgi);
