#!/usr/bin/perl

###############################################################################
#                                                                             #
# Filename: server_frontpanel.cgi                                             #
#                                                                             #
# Description: CGI needed to display and modify the LCD front panel display.  #
#                                                                             #
###############################################################################
###############################################################################
#                                                                             #
#  Copyright  2000 - 2001 Intel Corporation.                                 #
#  Intel Corporation All Rights Reserved.                                     #
#                                                                             #
#  The source code contained or described herein and all documents related to #
#  the source code ("Material") are owned by Intel Corporation or its         #
#  suppliers or licensors.  Title to the Material remains with Intel          #
#  Corporation or its suppliers and licensors.  The Material contains trade   #
#  secrets and proprietary and confidential information of Intel or its       #
#  suppliers and licensors.  The Material is protected by worldwide copyright #
#  and trade secret laws and treaty provisions.  No part of the Material may  #
#  be used, copied, reproduced, modified, published, uploaded, posted,        #
#  transmitted, distributed, or disclosed in any way without Intel's prior    #
#  express written permission.                                                #
#                                                                             #
#  No license under any patent, copyright, trade secret or other intellectual #
#  property right is granted to or conferred upon you by disclosure or        #
#  delivery of the Materials, either expressly, by implication, inducement,   #
#  estoppel or otherwise. Any license under such intellectual property rights #
#  must be express and approved by Intel in writing.                          #
#                                                                             #
###############################################################################

# Put the PERL library in the path.
use lib '/usr/ins/intel/gui/perllib';

use strict;
use ConfigurationData;
use LanguageResource;
use DisplayMain;
use CGI;

###############################################################################
# Main
###############################################################################

sub Main
{
	my ($cgi) = @_;
	my $PreferredLanguage	= (split(/,/, $ENV{'HTTP_ACCEPT_LANGUAGE'}))[0];
	my $StringResource		= new LanguageResource($PreferredLanguage, '/usr/ins/intel/gui/locale', [ 'server_frontpanel.lrc', 'common.lrc' ]);
	my $Configuration		= new ConfigurationData();
	my %_R					= ( 'IDT:100'				=> '',
								'DEFAULT_MESSAGE'		=> '',
								'CUSTOM_MESSAGE'		=> '',
								'CUSTOM_MESSAGE_LIMIT'	=> '',
								'APPLY'					=> '' );
	my %RadioState			= ( 'default' 	=> '',
								'custom'	=> '' );
	my $CustomMessage;
	my $FieldFocus;
	my $Feedback;
	my $Result;
	my $html;

	# Get the string resources.
	$StringResource->GetStrings(\%_R);
	
	# If applying a change...
	if (defined($cgi->param('apply')))
	{
		# Assume success.
		$Result = 0;
		
		# Check to see what which radio button the user selected...
		if ('radio_custom' eq $cgi->param('message_type'))
		{
			# Cache the posted value.
			$CustomMessage = $cgi->param('message');
	
			# Trim the white space.
			$CustomMessage =~ s/^\s*(.*?)\s*$/$1/;
			
			# Convert all of the whitespace to spaces.
			while ($CustomMessage =~ /(\t|\n|\r|\f)+/m)
			{ $CustomMessage =~ s/(\t|\n|\r|\f)+/ /m; }
			
			# Validate the length.
			if (96 < length($CustomMessage))
			{
				# Set the status.
				$Result 	= 10000;
				$Feedback	= $StringResource->_S($Result);
				$FieldFocus	= 'message';
			}
		}
		else
		{
			# Write an empty value.
			$CustomMessage = '';
		}

		# If things are running smoothly...
		if (0 == $Result)
		{
			# Try to open the file and set the data...
			unless ((0 == $Configuration->Attach('/usr/ins/intel/bin/fpic.conf', { 'auto-save' => 1, 'mode' => O_RDWR, 'permissions' => 0600 })) &&
					(0 == $Configuration->SetValueKeyed('RID_DEFAULT_MESSAGE', $CustomMessage)))
			{
				# Set the status.
				$Result 	= 10001;
				$Feedback	= $StringResource->_S($Result);
			}
		}
	}
	elsif (0 == $Configuration->Attach('/usr/ins/intel/bin/fpic.conf'))
	{
		# Try to get the value.
		$CustomMessage = $Configuration->GetValueKeyed('RID_DEFAULT_MESSAGE');
	}

    # Safe default.
	$CustomMessage = '' unless(defined($CustomMessage));

	# Set the radio buttons.
	$RadioState{'default'}	= (0 == length($CustomMessage)) ? 'checked' : '';
	$RadioState{'custom'}	= (0 == length($CustomMessage)) ? '' : 'checked';
	$FieldFocus	= ($RadioState{'custom'}) ? 'message' : 'message_type[0]' unless(defined($FieldFocus));
	
	$html = <<HERE;
<form action=server_frontpanel.cgi method=post enctype=application/x-www-form-urlencoded>
	<table border=0 width=375 bgcolor=#EEEEEE>
	<tr>
		<td>
			&nbsp;
			<table border=0 width=100%>
			<tr>
				<td width=100%>&nbsp;<input type=radio value=radio_default name=message_type $RadioState{'default'}><b>$_R{'DEFAULT_MESSAGE'}</b></td>
			</tr>
			<tr>
				<td width=100%>&nbsp;<input type=radio value=radio_custom name=message_type $RadioState{'custom'}><b>$_R{'CUSTOM_MESSAGE'}</b> $_R{'CUSTOM_MESSAGE_LIMIT'}</td>
			</tr>
			<tr>
				<td width=100%>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<textarea name=message rows=4 cols=24 style=font-family: Courier;overflow:hidden>$CustomMessage</textarea>&nbsp;&nbsp;</td>
			</tr>
			<tr>
				<td width=100%>&nbsp;</td>
			</tr>
			<tr>
				<td width=100% align=right><input type=submit class=clsButton value="$_R{'APPLY'}" name=apply></td>
			</tr>
			</table>
		</td>
	</tr>
	</table>
</form>
HERE

	# Display the page.
    DisplayMain->printDocument({ 'q' => $cgi, 'html' => $html, 
								 'class' => 'server_frontpanel', 'id' => 100, 'title' => $_R{'IDT:100'},
								 'focus-field' => $FieldFocus,
								 'root-link' => 'server_frontpanel', 'result' => $Result, 'feedback' => $Feedback });
}

###############################################################################
# The main body of the script.
###############################################################################

my $cgi = new CGI;

Main($cgi);