#!/usr/bin/perl	-w

###############################################################################
#                                                                             #
# Filename: server_http.cgi                                                   #
#                                                                             #
# Description: Contains CGI script needed for HTTP service control.           #
#                                                                             #
###############################################################################
###############################################################################
#                                                                             #
#  Copyright  2000 - 2001 Intel Corporation.                                 #
#  Intel Corporation All Rights Reserved.                                     #
#                                                                             #
#  The source code contained or described herein and all documents related to #
#  the source code ("Material") are owned by Intel Corporation or its         #
#  suppliers or licensors.  Title to the Material remains with Intel          #
#  Corporation or its suppliers and licensors.  The Material contains trade   #
#  secrets and proprietary and confidential information of Intel or its       #
#  suppliers and licensors.  The Material is protected by worldwide copyright #
#  and trade secret laws and treaty provisions.  No part of the Material may  #
#  be used, copied, reproduced, modified, published, uploaded, posted,        #
#  transmitted, distributed, or disclosed in any way without Intel's prior    #
#  express written permission.                                                #
#                                                                             #
#  No license under any patent, copyright, trade secret or other intellectual #
#  property right is granted to or conferred upon you by disclosure or        #
#  delivery of the Materials, either expressly, by implication, inducement,   #
#  estoppel or otherwise. Any license under such intellectual property rights #
#  must be express and approved by Intel in writing.                          #
#                                                                             #
###############################################################################

# Put the PERL library in the path.
use lib '/usr/ins/intel/gui/perllib';

use strict;
use ProductInformation;
use ConfigurationData;
use LanguageResource;
use DisplayMain;
use MonSettings;
use CookieData;
use Daemon;
use Safe;
use SSL;
use CGI;

###############################################################################
# Main
###############################################################################

sub Main
{
	my ($cgi)					= @_;
	my $PreferredLanguage		= (split(/,/, $ENV{'HTTP_ACCEPT_LANGUAGE'}))[0];
	my $StringResource			= new LanguageResource($PreferredLanguage, '/usr/ins/intel/gui/locale', [ 'server_http.lrc', 'server_alerts.lrc', 'common.lrc' ]);
	my $ServerConfiguration		= new ConfigurationData();
	my $Cookie					= new CookieData();
	my %_R						= ( 'IDT:1300'				=> '',
									'SERVICE'				=> '',
									'STATUS'				=> '',
									'CONTROLS'				=> '',
									'ADMINISTRATOR_WEB'		=> '',
									'WEB'					=> '',
									'RUNNING'				=> '',
									'STOPPED'				=> '',
									'START'					=> '',
									'STOP'					=> '',
									'CONFIRM_STOP'			=> '',
									'RESTART'				=> '',
									'CONFIGURE'				=> '',
									'ADMINISTRATOR_WEB_SSL' => '',
									'CONFIGURE_ALERTS'		=> '',
									'WEB_SERVICE_LOG'		=> '' );
	my %CertificateSelf;
	my %CertificateCA;
	my @CertificateFile;
	my $ServerConfigurationFile	= '/usr/ins/intel/apache/conf/httpd.conf';
	my $GroupIfModSSL			= { 'directive'	=> 'IfModule', 
									'identity'	=> 'mod_ssl.c' };
	my $CertificateType			= '';
	my $HTTPDaemon				= new Daemon('httpd');
	my $HTTPADaemon				= new Daemon('httpd-admin');
	my $HTTPDaemonStatus;
	my $HTTPADaemonStatus;
	my $CookieFile				= '/usr/ins/intel/gui/cookies/ssl_form-sa-' . $cgi->remote_user();
	my @KeysSelf				= ( 'country',
									'province',
									'locality',
									'organization',
									'organizational_unit',
									'common_name',
									'e_mail' );
	my @KeysCA					= ( 'private_key',
									'certificate' );
	my $Key;
	my $Redirect;
	my $RedirectNotice;
	my $SSLStatus;
	my $Result;
	my $Feedback;
	my $html;
	
	# Get the string resources.
	$StringResource->GetStrings(\%_R);

	# Safe default(s).
	$Redirect = '';

	# If the deamon needs to be restarted...
	if (defined($cgi->param('controla_restart')))
	{
		# Flush data.
		$| = 1;
	}

	# If toggling the daemon state...
	if ((defined($cgi->param('control_start'))) ||
		(defined($cgi->param('control_stop'))))
	{
		my $Monitor = new MonSettings();
		
		# If the configuration can be opened...
		if (0 == $Monitor->open())
		{
			# Set the monitor state.
			$Monitor->setMonitorState('http', (($cgi->param('control_start')) ? 1 : 0));

			# Close the configuration.
			$Monitor->close();
		}
	}

	SWITCH:
	{
		# Control the web daemon.
		if (defined($cgi->param('control_start')))		{ $HTTPDaemon->Control(1); last SWITCH; }
		if (defined($cgi->param('control_stop')))		{ $HTTPDaemon->Control(0); last SWITCH; }
		if (defined($cgi->param('control_restart')))	{ $HTTPDaemon->Control(2); last SWITCH; }
	}
	
	# Get the daemon status...
	$HTTPDaemonStatus	= ($HTTPDaemon->GetStatus())  ? "<font color=green>$_R{'RUNNING'}<font>" : "<font color=red>$_R{'STOPPED'}<font>";
	$HTTPADaemonStatus	= ($HTTPADaemon->GetStatus()) ? "<font color=green>$_R{'RUNNING'}<font>" : "<font color=red>$_R{'STOPPED'}<font>";

	# Get the current server configuration...
	if (0 == $ServerConfiguration->Attach($ServerConfigurationFile))
	{
		my $SSLSelf = new SSL('/usr/ins/intel/apache/cert', 'self');
		my $SSLCA	= new SSL('/usr/ins/intel/apache/cert', 'ca');
        
		# If the self-signed data is available...
		if (defined($SSLSelf->GetCertificate()))
		{
			# Use the self-signed data.
			$CertificateType	= 'self';
		}
		elsif (defined($SSLCA->GetCertificate()))
		{
			# Use the CA data.
			$CertificateType	= 'ca';
		}

		# Get the configuration status.
		$SSLStatus = $ServerConfiguration->ValidateDirectiveValue('SSLengine', 'on', $GroupIfModSSL);
		
		# Get the certificate type and information...
		if ((defined($CertificateType)) &&
			((defined($cgi->param('cookie-file'))) ||
			 (defined($cgi->param('configure_ssl')))))
		{
			my $SSLData = new SSL('/usr/ins/intel/apache/cert');
			
			# Set the class.
			$SSLData->SetClass($CertificateType);

			# If the configuration is self-signed...
			if ('self' eq $CertificateType)
			{
				# Get the private key and certificate...
				$SSLData->GetSubject(\%CertificateSelf);
			}
			else
			{
				# Get the private key and certificate...
				$CertificateCA{'private_key'} = $SSLData->GetPrivateKey();
				$CertificateCA{'certificate'} = $SSLData->GetCertificate();
			}
		}

		# Detach from the file.
		$ServerConfiguration->Detach();
	}

	# If there is a new configuration...
	if ($cgi->param('cookie-file'))
	{
		# Try to open the configuration file...
		if (0 == $ServerConfiguration->Attach($ServerConfigurationFile, { 'auto-save' => 1, 'mode' => O_RDWR, 'permissions' => 0600 }))
		{
			# Get the cookie data.
			$Cookie->Attach($CookieFile);

            # If configuration is needed...
			if ($Cookie->GetValueKeyed('ssl_active'))
			{
				my $SSLData		= new SSL('/usr/ins/intel/apache/cert');
				my $CertType	= $Cookie->GetValueKeyed('certificate_type');
				my $Delta;
				
				# Set the class.
				$SSLData->SetClass($CertType);

				# Check for a delta.
				$Delta = 1 if((!$SSLStatus) ||
							  ($CertificateType ne $CertType));

				# If the configuration is self-signed...
				if ('self' eq $CertType)
				{
					# Run through all of the parameters...
					foreach $Key (keys(%CertificateSelf))
					{
						# Check for a delta...
						if ($CertificateSelf{$Key} ne $Cookie->GetValueKeyed($Key))
						{
							# Found one.
							$Delta = 1;
							last;
						}
					}
					
					# If there is a delta...
					if (defined($Delta))
					{
						my %Subject;

						# Run through all of the keys...
						foreach $Key (@KeysSelf)
						{
							# Transfer the data.
							$Subject{$Key} = $Cookie->GetValueKeyed($Key);
						}

						# Remove the old keys.
						$SSLData->SetPrivateKey();
						$SSLData->SetCertificateSigningRequest();
						$SSLData->SetCertificate();
						
						# Generate the private key, CSR, and certificate...
						unless ((0 == $SSLData->SetSubject(\%Subject)) &&
								(defined($SSLData->GetCertificate(1))))
						{
							# Set the error code.
							$Result = 130001;
						}
					}
				}
				else
				{
					my $PrivateKey	= $Cookie->GetValueKeyed('private_key');
					my $Certificate	= $Cookie->GetValueKeyed('certificate');

					# Clean off goo.
					$CertificateCA{'private_key'}	=~ s/\r//g;
					$CertificateCA{'certificate'}	=~ s/\r//g;
					$PrivateKey 					=~ s/\r//g;
					$Certificate					=~ s/\r//g;

					# Check for a delta.
					$Delta = 1 if(($CertificateCA{'private_key'} ne $PrivateKey) ||
								  ($CertificateCA{'certificate'} ne $Certificate));

					# If there is a delta...
					if (defined($Delta))
					{
						# Remove the old keys.
						$SSLData->SetPrivateKey();
						$SSLData->SetCertificateSigningRequest();
						$SSLData->SetCertificate();
						
						# Generate the private key, CSR, and certificate...
						unless ((0 == $SSLData->SetPrivateKey($PrivateKey)) &&
								(0 == $SSLData->SetCertificate($Certificate)))
						{
							# Set the error code.
							$Result = 130002;
						}
					}
				}

				# If things are running smoothly...
				unless (defined($Result))
				{
					# If there are changes...
					if (defined($Delta))
					{
						my $PrivateKey;
						my $Certificate;

						# If the configuration paths are available...
						if (($PrivateKey = $SSLData->GetPrivateKeyPath()) &&
							($Certificate = $SSLData->GetCertificatePath()))
						{
							# Clean out any old data.
							$ServerConfiguration->DeleteDirectiveList('SSLengine', $GroupIfModSSL);
							$ServerConfiguration->DeleteDirectiveList('SSLCertificateFile', $GroupIfModSSL);
							$ServerConfiguration->DeleteDirectiveList('SSLCertificateKeyFile', $GroupIfModSSL);
							$ServerConfiguration->DeleteDirectiveList('SSLSessionCacheTimeout', $GroupIfModSSL);
							$ServerConfiguration->DeleteDirectiveList('SSLSessionCache', $GroupIfModSSL);
							$ServerConfiguration->DeleteDirectiveList('SSLRandomSeed', $GroupIfModSSL);
							$ServerConfiguration->DeleteDirectiveList('SetEnvIf', $GroupIfModSSL);
							$ServerConfiguration->DeleteDirectiveList('SSLMutex', $GroupIfModSSL);

							# Set the new data.
							$ServerConfiguration->SetDirectiveList('SSLengine', [ 'on' ], $GroupIfModSSL);
							$ServerConfiguration->SetDirectiveList('SSLCertificateFile', [ $Certificate ], $GroupIfModSSL);
							$ServerConfiguration->SetDirectiveList('SSLCertificateKeyFile', [ $PrivateKey ], $GroupIfModSSL);
							$ServerConfiguration->SetDirectiveList('SSLSessionCacheTimeout', [ 300 ], $GroupIfModSSL);
							$ServerConfiguration->SetDirectiveList('SSLSessionCache', [ 'dbm:/var/log/httpd/ssl_scache' ], $GroupIfModSSL);
							$ServerConfiguration->SetDirectiveList('SSLRandomSeed', [ 'startup   file:/dev/urandom 512', 'connect   builtin' ], $GroupIfModSSL);
							$ServerConfiguration->SetDirectiveList('SetEnvIf', [ "User-Agent \".*MSIE.*\" nokeepalive ssl-unclean-shutdownSSLEnable" ], $GroupIfModSSL);
							$ServerConfiguration->SetDirectiveList('SSLMutex', [ 'file:/var/log/httpd/ssl_mutex' ], $GroupIfModSSL);

							# Clean out the other certificate.
							$SSLData->SetClass(('self' eq $CertType) ? 'ca' : 'self');
							$SSLData->SetPrivateKey();
							$SSLData->SetCertificateSigningRequest();
							$SSLData->SetCertificate();

							# Set the redirect.
							$Redirect = "10; URL=https://$ENV{'HTTP_HOST'}/logout.cgi";
							$RedirectNotice = 1;

							# Set success.
							$Result = 0;
						}
						else
						{
							# Set the error code.
							$Result = 130003;
						}
					}
					else
					{
						# Set success.
						$Result = 0;
					}
				}
			}
			else
			{
				# If the configuration is different...
				if ($SSLStatus)
				{
					# Clean out any old data.
					$ServerConfiguration->DeleteDirectiveList('SSLengine', $GroupIfModSSL);
					$ServerConfiguration->DeleteDirectiveList('SSLCertificateFile', $GroupIfModSSL);
					$ServerConfiguration->DeleteDirectiveList('SSLCertificateKeyFile', $GroupIfModSSL);
					$ServerConfiguration->DeleteDirectiveList('SSLSessionCacheTimeout', $GroupIfModSSL);
					$ServerConfiguration->DeleteDirectiveList('SSLSessionCache', $GroupIfModSSL);
					$ServerConfiguration->DeleteDirectiveList('SSLRandomSeed', $GroupIfModSSL);
					$ServerConfiguration->DeleteDirectiveList('SetEnvIf', $GroupIfModSSL);
					$ServerConfiguration->DeleteDirectiveList('SSLMutex', $GroupIfModSSL);

					# Set the new data.
					$ServerConfiguration->SetDirectiveList('SSLengine', [ 'off' ], $GroupIfModSSL);

					# Set the redirect.
					$Redirect = "10; URL=http://$ENV{'HTTP_HOST'}/logout.cgi";
					$RedirectNotice = 1;
				}
				
				# Set success.
				$Result = 0;
			}
			
			# Detach from the cookie.
			$Cookie->Detach();

			# Detach from the file...
			$ServerConfiguration->Detach(1);
		}
		else
		{
			# Set the error code.
			$Result = 130000;
		}
	}
	elsif (defined($cgi->param('configure_ssl')))
	{
		my $URL = "/cgi-bin/common/ssl_forms.cgi?type=default";

		# Get the cookie data.
		$Cookie->Attach($CookieFile);

		# Set the POST URI.
		$Cookie->SetValueKeyed('post_uri', '/cgi-bin/server/server_http.cgi');

		# Set the root link.
		$Cookie->SetValueKeyed('root-link', 'server_http');
		
		# Set up the cookie crumbs.
		$Cookie->SetValueKeyed('b-class0', 'server_http');
		$Cookie->SetValueKeyed('b-label0', 'IDT:1300');
		$Cookie->SetValueKeyed('b-rpath0', '/usr/ins/intel/gui/locale');
		$Cookie->SetValueKeyed('b-uri0',   '/cgi-bin/server/server_http.cgi');

		# Add in the certificate information.
		if ($SSLStatus) { $Cookie->SetValueKeyed('ssl_active', 'checked'); } else { $Cookie->SetValueKeyed('ssl_active'); }
		if ($CertificateType) { $Cookie->SetValueKeyed('certificate_type', $CertificateType); } else { $Cookie->SetValueKeyed('certificate_type'); }

		# Run through all of the self-signed data...
		foreach $Key (@KeysSelf)
		{
			# Save the value.
			$Cookie->SetValueKeyed($Key, $CertificateSelf{$Key});
		}
		
		# Run through all of the CA data...
		foreach $Key (@KeysCA)
		{
			# Save the value.
			$Cookie->SetValueKeyed($Key, $CertificateCA{$Key});
		}

		# Detach from the cookie.
		$Cookie->Detach();

		# Set the cookie file name.
		$URL .= "&cookie-file=ssl_form-sa-" . $cgi->remote_user();
		
		# Redirect to the SSL forms.
		print $cgi->redirect( -url => $URL );
		return;
	}

	# Set the SSL status text...
	if ($SSLStatus)
	{
		# Using SSL...
		$SSLStatus = '<font color=green>' . (('self' eq $CertificateType) ? $StringResource->_S('SSL_CONFIGURED_SELF') : $StringResource->_S('SSL_CONFIGURED_CA')) . '<font>';
	}
	else
	{
		# Not using SSL.
		$SSLStatus = '<font color=red>' . $StringResource->_S('SSL_NOT_CONFIGURED') . '<font>';
	}
	
	# If there was an error...
	$Feedback = $StringResource->_S($Result) if($Result);
	$Feedback = $StringResource->_S('CONFIGURATION_IN_PROGESS') if(defined($RedirectNotice));

	$html = <<HERE;
<form method=post name=server_http action=server_http.cgi enctype="application/x-www-form-urlencoded">
	<table border=0 width=375 bgcolor=#eeeeee>
	<tr>
		<td>
			<br>
			<table border=0 cellpadding=0 cellspacing=0 width=100% bgcolor=#eeeeee>
			<tr>
				<td width=100%>
					<table border=1 cellpadding=2 cellspacing=0 width=100% bgcolor=#ffffff bordercolor=#eeeeee>
					<tr>
						<td width=40% bgcolor=#808080 nowrap><font color=#ffffff><b>$_R{'SERVICE'}</b></font></td>
						<td width=15% bgcolor=#808080 nowrap><font color=#ffffff><b>$_R{'STATUS'}</b></font></td>
						<td width=40% bgcolor=#808080 nowrap><font color=#ffffff><b>$_R{'CONTROLS'}</b></font></td>
					</tr>
					<tr>
						<td width=40%><b>$_R{'ADMINISTRATOR_WEB'}</b></td>
						<td width=15%><font color=green>$HTTPADaemonStatus</font></td>
						<td width=40% align=right nowrap>
							<input type=submit class=clsButton value='$_R{'RESTART'}' name=controla_restart>
						</td>
					</tr>
					<tr>
						<td width=40%><b>$_R{'WEB'}</b></td>
						<td width=15%><font color=green>$HTTPDaemonStatus</font></td>
						<td width=40% nowrap>
							<input type=submit class=clsButton value='$_R{'START'}' name=control_start>
							<input type=submit class=clsButton value='$_R{'STOP'}' name=control_stop onclick="javascript: return confirm('$_R{'CONFIRM_STOP'}');">
							<input type=submit class=clsButton value='$_R{'RESTART'}' name=control_restart>
						</td>
					</tr>
					</table>
				</td>
			</tr>
			</table>
			<table border=0 width=100%>
			<tr>
				<td width=100% colspan=2>&nbsp;</td>
			</tr>
			<tr>
				<td width=100% colspan=2><b>$_R{'ADMINISTRATOR_WEB_SSL'}</b></td>
			</tr>
			<tr>
				<td nowrap>$SSLStatus</td>
				<td align=right><input type=submit name=configure_ssl value="$_R{'CONFIGURE'}" class=clsButton>&nbsp;</td>
			</tr>
			<tr>
				<td width=100% colspan=2>&nbsp;</td>
			</tr>
			</table>
		</td>
	</tr>
	</table>
	<br>
	<a href=server_alerts.cgi>$_R{'CONFIGURE_ALERTS'}</a>&nbsp;&nbsp;&nbsp;<a href=server_system_logs.cgi?logtype=web&backto=web>$_R{'WEB_SERVICE_LOG'}</a>
</form>
HERE

	# Display the page.
    DisplayMain->printDocument({ 'q' => $cgi, 'html' => $html,
								 'class' => 'server_http', 'id' => 1300, 'title' => $_R{'IDT:1300'},
								 'root-link' => 'server_http', 'refresh' => $Redirect, 'result' => $Result, 'feedback' => $Feedback });

	# If the deamon needs to be restarted...
	if (defined($cgi->param('controla_restart')))
	{
		# Close the pipe.
		close(STDOUT);

		# Restart the daemon.
		$HTTPADaemon->Control(2);
	}
	
	# Did something happen...
	if ($Redirect)
	{
		my $ProductInfo = new ProductInformation();
		
		# Look for a storefront configuration...
		if (2 == $ProductInfo->getClassification())
		{
			# Restart the intershop services.
			Safe::spawn_daemon('/bin/sleep 30; /etc/rc.d/init.d/intershop4 start');
		}

		# Restart the server.
		Safe::spawn_daemon('/bin/sleep 1; /etc/rc.d/init.d/httpd-admin stop; /bin/sleep 5; /etc/rc.d/init.d/httpd-admin start');
	}
}

###############################################################################
# The main body of the script.
###############################################################################

my $cgi = new CGI;

Main($cgi);
