#!/usr/bin/perl

###############################################################################
#                                                                             #
# Filename: server_hwinfo.cgi                                                 #
#                                                                             #
# Description: Provides status information support for the appliance hardware #
#              in general.                                                    #
#                                                                             #
###############################################################################
###############################################################################
#                                                                             #
#  Copyright  2000 - 2001 Intel Corporation.                                 #
#  Intel Corporation All Rights Reserved.                                     #
#                                                                             #
#  The source code contained or described herein and all documents related to #
#  the source code ("Material") are owned by Intel Corporation or its         #
#  suppliers or licensors.  Title to the Material remains with Intel          #
#  Corporation or its suppliers and licensors.  The Material contains trade   #
#  secrets and proprietary and confidential information of Intel or its       #
#  suppliers and licensors.  The Material is protected by worldwide copyright #
#  and trade secret laws and treaty provisions.  No part of the Material may  #
#  be used, copied, reproduced, modified, published, uploaded, posted,        #
#  transmitted, distributed, or disclosed in any way without Intel's prior    #
#  express written permission.                                                #
#                                                                             #
#  No license under any patent, copyright, trade secret or other intellectual #
#  property right is granted to or conferred upon you by disclosure or        #
#  delivery of the Materials, either expressly, by implication, inducement,   #
#  estoppel or otherwise. Any license under such intellectual property rights #
#  must be express and approved by Intel in writing.                          #
#                                                                             #
###############################################################################

# Put the PERL library in the path.
use lib '/usr/ins/intel/gui/perllib';

use strict;
use LanguageResource;
use DisplayMain;
use Safe;
use CGI;

###############################################################################
# GetConfigData
###############################################################################

sub GetConfigData
{
	my ($Label, @ConfigFileContents) = @_;
    my $Value;
	my $Line;

	# Loop through all of the lines in the file.
	foreach $Line (@ConfigFileContents)
	{
		# Continue if there is a comment.
		next if $Line =~ /^#/;

		# Continue if the label is not found.
		next unless $Line =~ /$Label/;
		
		# Parse out the value.
		$Value = $Line;
		$Value =~ s/.*$Label=(.*)$/$1/;
		chomp $Value;
		
		return $Value;
	}

	return '';
}

###############################################################################
# Main
###############################################################################

sub Main
{
	my ($cgi) 					= @_;
	my $PreferredLanguage		= (split(/,/, $ENV{'HTTP_ACCEPT_LANGUAGE'}))[0];
	my $StringResource			= new LanguageResource($PreferredLanguage, '/usr/ins/intel/gui/locale', [ 'server_hwinfo.lrc', 'server_temperature.lrc', 'server_fan.lrc', 'server_voltage.lrc', 'server_status.lrc' ]);
	my %_R						= ( 'IDT:500'				=> '',
									'NOT_AVAILABLE'			=> '',
									'PRODUCT_INFORMATION'	=> '',
									'SERIAL_NUMBER'			=> '',
									'BASEBOARD_INFORMATION'	=> '',
									'COMPONENT'				=> '',
									'REVISION'				=> '',
									'HARDWARE_ALERTING'		=> '',
									'IDT:400'				=> '',
									'IDT:300'				=> '',
									'IDT:200'				=> '',
									'IDT:600'				=> '' );
	my @BiosData				= Safe::backtick('/usr/ins/intel/bin/asm/sysSMBIOS dump baseboard');
	my $BaseboardComponent		= GetConfigData('BASEBOARD_PRODUCT_NAME', @BiosData);
	my $BaseboardRevision		= GetConfigData('BASEBOARD_VERSION', @BiosData);
	my $BaseboardSerialNumber	= GetConfigData('BASEBOARD_SERIAL_NUMBER', @BiosData);
	my @BiosData				= Safe::backtick('/usr/ins/intel/bin/asm/sysSMBIOS dump system_information');
	my $ProductSerialNumber		= GetConfigData('SYSTEM_INFORMATION_SERIAL_NUMBER', @BiosData);
	my $html;

	# Get the string resources.
	$StringResource->GetStrings(\%_R);
	
	# Safe defaults.
	if (($BaseboardComponent =~ /^\s*$/) ||
		(!defined($BaseboardComponent)))
	{ $BaseboardComponent		= $_R{'NOT_AVAILABLE'}; }
	
	if (($BaseboardRevision =~ /^\s*$/) ||
		(!defined($BaseboardRevision)))
	{ $BaseboardRevision		= $_R{'NOT_AVAILABLE'}; }
	
	if (($BaseboardSerialNumber =~ /^\s*$/) ||
		(!defined($BaseboardSerialNumber)))
	{ $BaseboardSerialNumber	= $_R{'NOT_AVAILABLE'}; }
	
	if (($ProductSerialNumber =~ /^\s*$/) ||
		(!defined($ProductSerialNumber)))
	{ $ProductSerialNumber		= $_R{'NOT_AVAILABLE'}; }

	$html = <<HERE;
<b>$_R{'PRODUCT_INFORMATION'}</b>
<table border=0 cellpadding=2 cellspacing=0 width=400 bgcolor=#ffffff>
<tr>
	<td width=167>&nbsp;<b>$_R{'SERIAL_NUMBER'}</b></td>
	<td width=221>$ProductSerialNumber</td>
</tr>
</table>
<br><b>$_R{'BASEBOARD_INFORMATION'}</b><br>
<table border=0 cellpadding=2 cellspacing=0 width=400 bgcolor=#ffffff>
<tr>
	<td width=167>&nbsp;<b>$_R{'COMPONENT'}</b></td>
	<td width=221>$BaseboardComponent</td>
</tr>
<tr>
	<td width=167>&nbsp;<b>$_R{'REVISION'}</b></td>
	<td width=221>$BaseboardRevision</td>
</tr>
<tr>
	<td width=167>&nbsp;<b>$_R{'SERIAL_NUMBER'}</b></td>
	<td width=221>$BaseboardSerialNumber</td>
</tr>
</table>
<p>
<b>$_R{'HARDWARE_ALERTING'}</b><br>
&nbsp;<a href=server_temperature.cgi>$_R{'IDT:400'}</a><br>
&nbsp;<a href=server_fan.cgi>$_R{'IDT:300'}</a><br>
&nbsp;<a href=server_voltage.cgi>$_R{'IDT:200'}</a></p>
HERE

	# Display the page.
    DisplayMain->printDocument({ 'q' => $cgi, 'html' => $html, 
								 'cookiecrumbs' => [ { 'name' => $_R{'IDT:600'}, 'location' => 'server_status.cgi' } ],
								 'root-link' => 'server_status', 'class' => 'server_hwinfo', 'id' => 500, 'title' => $_R{'IDT:500'} });
}

###############################################################################
# The main body of the script.
###############################################################################

my $cgi = new CGI;

Main($cgi);
