#!/usr/bin/perl
$VERSION="1.36";

###############################################################################
#                                                                             #
# Filename: server_network.cgi                                                #
#                                                                             #
# Description: CGI needed to display and modify the network settings.         #
#                                                                             #
###############################################################################
###############################################################################
#                                                                             #
#  Copyright  2000 - 2001 Intel Corporation.                                 #
#  Intel Corporation All Rights Reserved.                                     #
#                                                                             #
#  The source code contained or described herein and all documents related to #
#  the source code ("Material") are owned by Intel Corporation or its         #
#  suppliers or licensors.  Title to the Material remains with Intel          #
#  Corporation or its suppliers and licensors.  The Material contains trade   #
#  secrets and proprietary and confidential information of Intel or its       #
#  suppliers and licensors.  The Material is protected by worldwide copyright #
#  and trade secret laws and treaty provisions.  No part of the Material may  #
#  be used, copied, reproduced, modified, published, uploaded, posted,        #
#  transmitted, distributed, or disclosed in any way without Intel's prior    #
#  express written permission.                                                #
#                                                                             #
#  No license under any patent, copyright, trade secret or other intellectual #
#  property right is granted to or conferred upon you by disclosure or        #
#  delivery of the Materials, either expressly, by implication, inducement,   #
#  estoppel or otherwise. Any license under such intellectual property rights #
#  must be express and approved by Intel in writing.                          #
#                                                                             #
###############################################################################

use lib "/usr/ins/intel/gui/perllib";
use strict;
use ProductInformation;
use CGI;
use VirtualServ;
use SetNic;
use RouteConfig;
use NetConfig;
use LanguageLogCGI;
use Daemon;
use DisplayMain;
use Usable;
use Safe;

###############################################################################
# server_network::LogMessage
###############################################################################

sub LogMessage
{
	my ($Code, $Category, $Severity, $ParameterRef) = @_;
	my $Log	= new LanguageLogCGI('/usr/ins/intel/gui/logs/server.log', '/usr/ins/intel/gui/locale', [ 'server_network.lrc' ]);

	# Log the error.
	$Log->Append($Category, $Severity, $Code, $ParameterRef);
	return $Code;
}

##  Main
################################################################################

my $q = new CGI;
my $PreferredLanguage	= (split(/,/, $ENV{'HTTP_ACCEPT_LANGUAGE'}))[0];
my $StringResource		= new LanguageResource($PreferredLanguage, '/usr/ins/intel/gui/locale', [ 'server_network.lrc', 'common.lrc' ]);
my %_R					= ( 'IDT:4500'						=> '',
							'HOSTNAME_LABEL'				=> '',
							'DOMAIN_NAME_LABEL'				=> '',
							'PRIMARY_DNS_SERVER_ADDRESS_LABEL'	=> '',
							'SECONDARY_DNS_SERVER_ADDRESS_LABEL'	=> '',
							'TERTIARY_DNS_SERVER_ADDRESS_LABEL'	=> '',
							'IP_ADDRESSES'					=> '',
							'ADAPTER'						=> '',
							'DISABLE'						=> '',
							'INTERFACE'						=> '',
							'IP_ADDRESS'					=> '',
							'NETMASK'						=> '',
							'BROADCAST'						=> '',
							'USE_DEFAULT'					=> '',
							'DEFAULT_GATEWAY_LABEL'			=> '',
							'CONFIGURE_ROUTING'				=> '',
							'DIAGNOSTICS'					=> '',
							'VIRTUAL_IP_ADDRESSES'			=> '',
							'ADD'							=> '',
							'DELETE'						=> '',
							'ERR_iSpecial'					=> '',
							'ERR_iIP'						=> '',
							'ERR_iHost'						=> '',
							'ERR_iBasicIP'					=> '',
							'ERR_iAllNICDisable'			=> '',
							'ERR_mNotChecked'				=> '',
							'ERR_mConfirmVIPDelete'			=> '',
							'ERR_mEmptyString'				=> '',
							'APPLY'							=> '' );

# Get the string resources.
$StringResource->GetStrings(\%_R);

################################################################################
##  Start Subroutines 
################################################################################

sub updateHostDNS
{
	my ($q) = @_;
	$q->import_names('G');
	my $host_name = $G::host;
	my $feedback = 0;
	my (%dns, $dns,$dom);
	
	push(@{$dns->{'name'}}, $G::pdns1, $G::sdns1, $G::sdns2);

	$dom = $G::domain;
	push(@{$dns->{'domain'}}, $dom);

	VirtualServ::save_dns($dns);
	VirtualServ::save_host($host_name,$dom,$G::oldhost,$G::olddomain);
	return $feedback;
}

## Bring a Configured Interface 'Up' 
##############################################################################
##############################################################################

sub activateNic {

	my ($q, $nic) = @_;
	my $return;
	my @niclist = NetConfig->listNetInterfaces();

	for (@niclist) { 

		if ( ($_->{fullnic} eq $nic) && ($_->{up} ne '1') && ($q->param('disable_'."$nic") ne 'on' ) )
		{
			Safe::system("/sbin/ifconfig $nic up"); 
			
			$return = 1;
		}
		else
		{
			$return = 0;
		}
	}
	
	if ( ! $return ) { $return = 0; }

	return $return;

}

##  Route the (Add, Delete, Modify) Request to either:
##	1) modifyNic();          	(This script) 
##	2) SetNic->confirmNic();	(In perllib)
##############################################################################
##############################################################################

sub checkParams {

	my ($q, $purpose) = @_;
	my ($nicinfo, $feedback, $rejectvdom, $virtualip);
	my ($nic,$ipaddress,$netmask,$broadcast,$fakenic) = split /\,/, $q->param('selectnic') ;
	my ($nic1_purpose, $nic2_purpose);

	# Optimism works!
	$feedback = 0;

	## Reject Virtual Domain Changes (Except Default)
	################################################################################

	## Reject Domain Deletion (All Deletes)
	############################################################
	if ( $purpose eq $_R{'DELETE'} )
	{
		$rejectvdom = NetConfig->rejectVDomain($ipaddress,$nic); 
	}
	else
	{
		## Previously Rejected Domain Adds/Modifications
		############################################################
		if ($nic =~ /:/)
		{ 
			unless ( $purpose eq $_R{'ADD'} )
			{
#				$rejectvdom = NetConfig->rejectVDomain($ipaddress,$nic); 
				$virtualip = "1";
				$rejectvdom = 0;
			}
			else
			{
				$rejectvdom = '0';
			}
		}
		else
		{
			$rejectvdom = '0';
		}

	}

	if ( $rejectvdom ne '0' )
	{
		return (LogMessage(460011, 'error', 'critical') );
	}

	################################################################################

	if ( ($purpose eq $_R{'APPLY'}) || ($purpose eq $_R{'DELETE'}) )
	{ 
		my ($ap_nic,$ap_ipaddr,$ap_nmask,$ap_bcast) = split /\,/, $q->param('selectnic');
		my $nc_nic = "$ap_nic";
		my $nc_ipaddr = $q->param("$nc_nic"."_ip");
		my $nc_nmask = $q->param("$nc_nic"."_nmask");
		my $nc_bcast = $q->param("$nc_nic"."_bcast");

		# Begin working through the NIC settings and default gateway settings.

		if ( $q->param('trueip') )
		{
			# If we are configuring a real NIC device (eth0, eth1, etc...)

			my ($nic1_old_ip, $nic1_old_nmask, $nic1_old_bcast) = (split /\,/, $q->param('eth0'))[1,2,3] ;
			my ($nic2_old_ip, $nic2_old_nmask, $nic2_old_bcast) = (split /\,/, $q->param('eth1'))[1,2,3] ;

			my ($old_gw, $old_gw_nic);

			$nic1_purpose = ( ($q->param('disable_nic1') eq 'on') ? $_R{'DELETE'} : $_R{'APPLY'} );
			$nic2_purpose = ( ($q->param('disable_nic2') eq 'on') ? $_R{'DELETE'} : $_R{'APPLY'} );

			# If we are to make changes for eth0...
			if ($nic1_purpose eq $_R{'APPLY'})
			{
				if ( ($nic1_old_ip ne $q->param('eth0_ip')) ||
				     ($nic1_old_nmask ne $q->param('eth0_nmask')) ||
				     ($nic1_old_bcast ne $q->param('eth0_bcast')) ||
				     ($broadcast && ($nic1_old_bcast ne $q->param('eth0_bcast'))) )
				{
					$feedback = modifyNic($q,$nic1_purpose,'eth0',$q->param('eth0_ip'),$q->param('eth0_nmask'),$q->param('eth0_bcast'));
				}
				elsif ( ('' eq $q->param('eth0_ip')) ||
				        ('' eq $q->param('eth0_nmask')) ||
				        ('' eq $q->param('eth0_bcast')) )
				{
					$feedback = 4507;
				}
				elsif (0 == activateNic($q, 'eth0'))
				{
					LogMessage(460010, 'security', 'information');
				}
			}
			
			# If we are to delete an interface for eth0...
			elsif ( ($nic1_old_ip) && ($nic1_old_nmask) && ($nic1_old_bcast) )
			{ 
				$feedback = modifyNic($q,$nic1_purpose,'eth0',$nic1_old_ip,$nic1_old_nmask,$nic1_old_bcast); 
			}

			# If we are to make changes for eth1...
			if ( (!$feedback) && ($nic2_purpose eq $_R{'APPLY'}) )
			{
				if ( ($nic2_old_ip ne $q->param('eth1_ip')) ||
				     ($nic2_old_nmask ne $q->param('eth1_nmask')) ||
				     ($nic2_old_bcast ne $q->param('eth1_bcast')) ) 
				{
					$feedback = modifyNic($q,$nic2_purpose,'eth1',$q->param('eth1_ip'),$q->param('eth1_nmask'),$q->param('eth1_bcast'));
				}
				elsif ( ('' eq $q->param('eth1_ip')) ||
				        ('' eq $q->param('eth1_nmask')) ||
				        ('' eq $q->param('eth1_bcast')) )
				{
					$feedback = 4507;
				}
				elsif (0 == activateNic($q, 'eth1'))
				{
					LogMessage(460010, 'security', 'information');
				}
			}
			
			# If we are to delete an interface for eth1...
			elsif ( ($nic2_old_ip) && ($nic2_old_nmask) && ($nic2_old_bcast) )
			{ 
				$feedback = modifyNic($q,$nic2_purpose,'eth1',$nic2_old_ip,$nic2_old_nmask,$nic2_old_bcast); 
			}

		}
		else
		{
			# If we are configuring a virtual NIC device (eth0:1, eth0:2, etc)
			
			$feedback = modifyNic($q,$purpose,$nc_nic,$nc_ipaddr,$nc_nmask,$nc_bcast);

			my $new_realnic;

			if ($nc_nic =~ /:/)
			{
				$new_realnic = (split(':',$nc_nic))[0];
			}
			else
			{
				$new_realnic = 'eth0';
			}

			if  ( (!$feedback) && ( ($purpose eq $_R{'APPLY'}) || ($purpose eq $_R{'DELETE'}) ) )
			{
				# Remove the old host route for this IP address.
				RouteConfig->deleteRoute($ap_ipaddr, '0.0.0.0', '', $new_realnic);

			}
			
		}

		############################################################
	}
	else
	{
		if ( (!$q->param('addnic')) && (!$q->param('deletenic')) )
		{
			$feedback = SetNic->confirmNic($q,$nic,$ipaddress,$netmask,$broadcast,$purpose,$virtualip,$fakenic);
			exit();
		}
		else
		{
			$feedback = modifyNic($q,$purpose,$q->param('nic'));
		}
	}

	# Check to see if the default gateway has been changed.
	# Only execute this if no IP nic changes were made above.
	# Only execute this if no errors from earlier steps occurred.
	# Make sure that HTML form values for 'defaultgw' and 'gwnic' were given.
	
	if ( (!$feedback) && ($q->param('defaultgw')) && ($q->param('gwnic')) )
	{
		$feedback = checkGateway($q);
	}

	if ( (!$feedback) && ($q->param('trueip')) && (($purpose eq $_R{'APPLY'}) || ($purpose eq $_R{'DELETE'})) )
	{
		# Update NVRAM with new NIC Info 
		my $ap_ipaddr;
		my $ap_nmask;
		my $defaultgw;

		my $oldgw = getDefaultGateway();

		if ( ($nic1_purpose eq $_R{'APPLY'}) && ('eth0' eq $oldgw->{'nic'}) )
		{
			$ap_ipaddr = $q->param('eth0_ip');
			$ap_nmask = $q->param('eth0_nmask');
			$defaultgw =  $oldgw->{'gateway'};
		}
		elsif ( ($nic2_purpose eq $_R{'APPLY'}) && ('eth1' eq $oldgw->{'nic'}) )
		{
			$ap_ipaddr = $q->param('eth1_ip');
			$ap_nmask = $q->param('eth1_nmask');
			$defaultgw =  $oldgw->{'gateway'};
		}
		else
		{
			$defaultgw = $ap_nmask = $ap_ipaddr = '0.0.0.0';
		}
		
		NetConfig->updateBIOS($ap_ipaddr, $ap_nmask, $defaultgw);
	} 


	if ($feedback == 1800011)
	{
		$feedback = 0;
	}
	
	return ($feedback);

}

## Add, Delete, or Change the IP Address
##############################################################################
##############################################################################

sub modifyNic {

	############################################################
	##
	##  $ap_* are previous/current values
	##  $nc_* are new values
	##
	############################################################

	my ($q,$nic_purpose,$ap_nic,$ap_ipaddr,$ap_nmask,$ap_bcast) = @_;
	my ($result, $del_result, $result2);
	my $defaultgw	= $q->param('defaultgw');
	my $gwnic	= $q->param('gwnic');
	my $nc_nic	= $q->param("$ap_nic"."_nic");
	my $nc_ipaddr	= $q->param("$ap_nic"."_ip");
	my $nc_nmask	= $q->param("$ap_nic"."_nmask");
	my $nc_bcast	= $q->param("$ap_nic"."_bcast");

	## Make sure the IP Address doesn't belong to a Virtual Host (except Default Domain)
	############################################################
	if (!$q->param('trueip'))
	{	
		$nc_nic = $q->param('nic');
		$nc_ipaddr = $q->param('ipaddr');
		$nc_nmask = $q->param('netmask');
		$nc_bcast = $q->param('broadcast');
		
		$result = '0';

		if ($nc_ipaddr) { $result = NetConfig->rejectVDomain($nc_ipaddr, $nc_nic); }
		if ($ap_ipaddr and $result) { $result = NetConfig->rejectVDomain($ap_ipaddr, $nc_nic); }
		if ( $result ne '0' and $nic_purpose eq $_R{'DELETE'}) { return $result; }
	}
	else
	{
		$result = '0';
	}
	
	if ($result eq '0')
	{
		my $new_realnic;

		## Add Interface
		############################################################
		############################################################
		my $compare_add = $_R{'ADD'};
		if ($nic_purpose eq $_R{'ADD'} )
		{ 
			# Determine if we are being passed a NIC value like eth0:0, or just eth0
			if ( (defined($ap_nic)) && (!$q->param('trueip')) && ($ap_nic ne '') && ($ap_nic =~ /:/) )
			{
				# If so, then split out the colon and alias number.
				# Use this for all route calls.
				$new_realnic = (split(':',$ap_nic))[0];
			}
			else
			{
				$new_realnic = $nc_nic;
			}

			$result = NetConfig->changeNic($nc_nic,$nc_ipaddr,$nc_nmask,$nc_bcast,'','',$defaultgw);

			RouteConfig->createRoute($nc_ipaddr, '', '', $new_realnic, 0);
		}

		## Change (Apply) Interface
		############################################################
		############################################################

		if ($nic_purpose eq $_R{'APPLY'}) {

			my $confirm = "yes";
			my $eth0_up;

			## If Changing the Network Interface, Delete Old Interface
			############################################################

			if ( ! $q->param('trueip') eq 'yes') {                                                                                                        
				if ( "$ap_nic" ne "$nc_nic") { 
					$del_result = NetConfig->deleteNetInterface($ap_nic,$ap_ipaddr); 
				} 
			}
			else { $del_result = "0"; }	## Fake $del_result (variable is meaningless in this case)
		
			## Change IP Address (for 'Real' NICs)
			############################################################

			if ($del_result eq "0" ) {

				my @niclist = NetConfig->listNetInterfaces();

				for (@niclist) { 
					if ( $_->{fullnic} eq 'eth0' ) {
						$eth0_up = 'yes';
					}
				}

				## First Delete Host Route
				############################################################

				my ($del_ipaddr) = ( split /\,/, $q->param("$ap_nic"))[1];

				## Reconfigure the Nic
				############################################################
				$result = NetConfig->changeNic($ap_nic,$ap_ipaddr,$ap_nmask,$ap_bcast,'','',$defaultgw);

				# Determine if we are being passed a NIC value like eth0:0, or just eth0
				if ( (defined($ap_nic)) && (!$q->param('trueip')) && ($ap_nic ne '') && ($ap_nic =~ /:/) )
				{
					# If so, then split out the colon and alias number.
					# Use this for all route calls.
					$new_realnic = (split(':',$ap_nic))[0];
				}
				else
				{
					$new_realnic = $ap_nic;
				}
				
				##  Add the Host Route (Red Hat style)
				############################################################
				RouteConfig->createRoute($ap_ipaddr, '', '', $new_realnic, 0);

				## Find Old IP Address According to Apache Conf
				############################################################
				my $old_ipaddr = getOldIP($q, $ap_nic);

				if ($old_ipaddr ne $ap_ipaddr) {

					my $apache_file = '/etc/httpd/conf/httpd.conf';
					my $admin_file = '/usr/ins/intel/apache/conf/httpd.conf';
					my $ftp_file    = '/usr/ins/intel/wu-ftpd/conf/ftpservers';

					for ($apache_file, $ftp_file, $admin_file) { 
						updateDomain($old_ipaddr, $ap_ipaddr, $_); 
					}
				}

				############################################################

				VirtualServ::setHostFile();
			}
		}

		## Delete Interface
		############################################################
		############################################################

		if ($nic_purpose eq $_R{'DELETE'})
		{ 
			my $rejectvdom = NetConfig->rejectVDomain($ap_ipaddr,$ap_nic); 
			if ( $rejectvdom ne '0' ) { return (LogMessage(460011, 'error', 'warning') ); }


			## Check that IP isn't already disabled
			############################################################
			if ( $q->param("$ap_nic"."_ip") ) {

				$result = NetConfig->deleteNetInterface($ap_nic,$ap_ipaddr);
				VirtualServ::setHostFile();
				return ( $result);

			}

		}
	
	}
	if($result == 1800011)
	{ $result = 0; }

	return ($result);
}


## Update Files For Domains
################################################################################

sub getOldIP {

	my ($q, $ap_nic) = @_;
	my $old_ipaddr;

	if ( $q->param('eth0') and $ap_nic eq 'eth0' ) {
		$old_ipaddr = (split /,/, $q->param('eth0'))[1];
	}
	elsif ( $q->param('eth1') and $ap_nic eq 'eth1' ) {
		$old_ipaddr = (split /,/, $q->param('eth1'))[1];
	}
	elsif ( $q->param('selectnic') ) {
		$old_ipaddr = (split /,/, $q->param('selectnic'))[1];
	}	

	return ($old_ipaddr);

}



sub OLDgetOldIP {

        open (DOMCF, '/etc/httpd/conf/httpd.conf');
        my @raw_conf = <DOMCF>;
        close (DOMCF);

        my @raw_domains = grep(/^<VirtualHost/, @raw_conf);
        my $ip = $raw_domains[0];

	$ip =~ s/\n//;
	$ip =~ s/(<\w+)(\s+)(.*)(:)(.*)/$3/;

        return ($ip);
}

sub updateDomain {

        my ($old_ipaddr, $new_ipaddr, $conf) = @_;

        if ( ! $old_ipaddr or ! $new_ipaddr ) { return 0; }
	my $found = 'no';

        ## Check for Valid IP
        ############################################################
        for ($old_ipaddr, $new_ipaddr) {
                        if (Usable::check_ip($_) ne "1") { return ($StringResource->_S("INVALID_IP") . "$_"); }
        }
        ############################################################

        open (CONF, "$conf") || return  ( $StringResource->_S("FILE_OPEN_ERROR") . ": $!");
        flock (CONF, 2) || return ( $StringResource->_S("FILE_LOCK_ERROR") . ": $!");
        my @config_file = <CONF>;
        close (CONF);

        for (@config_file) {

                if ( $_ =~ /$old_ipaddr/ ) {
                        $_ =~ s/($old_ipaddr)/$new_ipaddr/;
			$found = 'yes';
                }
        }

        open (CONF, ">$conf") || return ( $StringResource->_S("FILE_OPEN_ERROR") . ": $!");
        flock (CONF, 2) || return ( $StringResource->_S("FILE_LOCK_ERROR") . ": $!");
        print CONF @config_file;
        close (CONF);

	## Restart Apache if SSL is enabled/disabled
	############################################################
	if ( ($found eq 'yes') and ($conf eq '/usr/ins/intel/apache/conf/httpd.conf'))
	{
		my $ProductInfo = new ProductInformation();
		
		# Look for a storefront configuration...
		if (2 == $ProductInfo->getClassification())
		{
			# Restart the intershop services.
			Safe::spawn_daemon('/bin/sleep 30; /etc/rc.d/init.d/intershop4 start');
		}
		
		# Restart the administrator web server.
		Safe::spawn_daemon("/bin/sleep 6; /etc/rc.d/init.d/httpd-admin stop; /bin/sleep 3; /etc/rc.d/init.d/httpd-admin start")
	}

	if ( ($found eq 'yes') and ($conf eq '/etc/httpd/conf/httpd.conf'))
	{
		my $HTTPDaemon = new Daemon('httpd');
		
		# Restart the public daemon.
		$HTTPDaemon->Control(2);
	}
	############################################################

}

## Gateway Operations
################################################################################

sub checkGateway
{
	my ($q) = @_;
	my $defaultgw = $q->param('defaultgw');
	my $gwnic = $q->param('gwnic');
	my $result;

	if ($gwnic)
	{
		my $oldgw = getDefaultGateway();

		if ($gwnic eq 'Adapter1')
		{
			$gwnic = 'eth0';
		}
		else
		{
			$gwnic = 'eth1';
		}

		if ( ($defaultgw ne $oldgw->{'gateway'}) ||
		     ($gwnic ne $oldgw->{'nic'}) )
		{
			RouteConfig->deleteDefaultGateway($oldgw->{'nic'});

			# If there's a problem setting the new default gateway...
			if ($result = RouteConfig->createDefaultGateway($defaultgw, $gwnic))
			{
				# Back out the changes.
				RouteConfig->deleteDefaultGateway($gwnic);
				
				# Restore the original default gateway settings.
				RouteConfig->createDefaultGateway($oldgw->{'gateway'}, $oldgw->{'nic'});
			}
		}
	}

	return ($result);
}

sub getDefaultGateway
{
	my ($switch) = @_;
	my @current_rt = RouteConfig->listRoutes();
	my ($gw_adapter, $current_gw);

	for (@current_rt)
	{
		my $name = "$_->{'nic'}:$_->{'destination'}";
		my $item_list = "$_->{'destination'},$_->{'gateway'},$_->{'netmask'},$_->{'nic'},$name";

		if ($_->{'nic'} eq "eth0")
		{
			$gw_adapter = $_R{"ADAPTER"} . '1';
		} 
		else
		{
			$gw_adapter = $_R{"ADAPTER"} . '2';
		}

		if ($_->{destination} eq "0.0.0.0")
		{
			$current_gw = $_;
		}
	}

	return (($switch eq 'gateway') ? ($current_gw->{gateway}) : ($current_gw))
}


##############################################################################
##############################################################################


## Create the HTML
##############################################################################
##############################################################################

sub makeBody {

	my ($q, $feedback) = @_;
	my ($nic1,$nic2);
	my $nic1_chk = '1';
	my $nic2_chk = '1';
	my $navheader= $_R{"NETWORK"};
	my ($self,$nic,$ipaddress,$netmask,$broadcast,$old_nic,$item,$html2,$item_list) = "";
	my @current_nic = NetConfig->listNetInterfaces();
	my $sp 	= "&nbsp;&nbsp;";
	my $spaces = "&nbsp;&nbsp;&nbsp;";

	##  Get Values, Generate HTML for NIC IP(s) (trueip)
	############################################################

	for (@current_nic) { 
		if    ( $_->{'fullnic'} eq 'eth0') { $nic1 = $_; $nic1_chk='0'; }
		elsif ( $_->{'fullnic'} eq 'eth1') { $nic2 = $_; $nic2_chk='0'; }
	}

	if (! $nic1) { $nic1->{fullnic} = 'eth0';}
	if (! $nic2) { $nic2->{fullnic} = 'eth1';}

	my $nic1_list = "$nic1->{fullnic},$nic1->{ipaddress},$nic1->{netmask},$nic1->{broadcast},$_R{'ADAPTER'} . '1'";
	my $nic2_list = "$nic2->{fullnic},$nic2->{ipaddress},$nic2->{netmask},$nic2->{broadcast},$_R{'ADAPTER'} . '2'";

	## HTML for Physical NICs
	############################################################	
		
	my $html3 = 
		## eth0 (nic1)
		############################################################	
		$q->Tr( {-bgcolor=>'white'}, 
		$q->td( {-align=>'center'}, $q->hidden( -name=>"$nic1->{fullnic}", -value=>"$nic1_list"). 
			$q->checkbox( -name=>'disable_nic1', -value=>'on', -checked=>"$nic1_chk", -label=>'')).
		$q->td( {-width=>'1'}, $_R{"ADAPTER"}. '1') . "\n" .
		$q->td($q->textfield(-name=>"$nic1->{'fullnic'}_ip",-value=>"$nic1->{'ipaddress'}", -size=>14,-maxlength=>15)) . "\n".
		$q->td($q->textfield(-name=>"$nic1->{'fullnic'}_nmask",-value=>"$nic1->{'netmask'}", -size=>14,-maxlength=>15)) . "\n".
		$q->td($q->textfield(-name=>"$nic1->{'fullnic'}_bcast",-value=>"$nic1->{'broadcast'}", -size=>14,-maxlength=>15))) . "\n".

		## eth1 (nic2)
		############################################################	
		$q->Tr( {-bgcolor=>'white'}, 
		$q->td( {-align=>'center'}, $q->hidden( -name=>"$nic2->{fullnic}", -value=>"$nic2_list"). 
			$q->checkbox( -name=>'disable_nic2', -value=>'on', -checked=>"$nic2_chk", -label=>'')).
		$q->td( {-width=>'1'}, $_R{"ADAPTER"}. '2') . "\n" .
		$q->td($q->textfield(-name=>"$nic2->{'fullnic'}_ip",-value=>"$nic2->{'ipaddress'}", -size=>14,-maxlength=>15)) . "\n".
		$q->td($q->textfield(-name=>"$nic2->{'fullnic'}_nmask",-value=>"$nic2->{'netmask'}", -size=>14,-maxlength=>15)) . "\n".
		$q->td($q->textfield(-name=>"$nic2->{'fullnic'}_bcast",-value=>"$nic2->{'broadcast'}", -size=>14,-maxlength=>15))) . "\n";


	##  Get Values, Generate HTML for Virtual IP(s)
	############################################################

	foreach $item (@current_nic) {
	  if ($item->{'fullnic'} =~ /:/) {
	     unless ( $item->{'fullnic'} eq 'lo') {

	          $item->{'fullnic'} =~ /(eth)(\d+)(:)(\d+)/;
	          my $fakevnum1 = $2 + 1;

		## Unsure of original intent -- but now gives incorrect value -- Delete!!
		############################################################		
		##  my $fakevnum2 = $4 + 1;

	          my $fakevnic = $_R{"ADAPTER"} . "$fakevnum1:$4";

		  my $item_list = "$item->{fullnic},$item->{ipaddress},$item->{netmask},$item->{broadcast},$fakevnic";
		  $html2 .=  $q->Tr( {-bgcolor=>'white'}, 
			$q->td( "<input type=radio name=selectnic onClick='nmask.checked=false; bcast.checked=false;' value=$item_list>". 
			$spaces."$fakevnic") . "\n" .
			$q->td($q->textfield(-name=>"$item->{'fullnic'}_ip",-value=>$item->{'ipaddress'}, -size=>14,-maxlength=>32)) . "\n".
			$q->td($q->textfield(-name=>"$item->{'fullnic'}_nmask",-value=>"$item->{'netmask'}", -size=>14,-maxlength=>32)) . "\n".
			$q->td($q->textfield(-name=>"$item->{'fullnic'}_bcast",-value=>"$item->{'broadcast'}", -size=>14,-maxlength=>32))) . "\n";
  	     } 
	  }
	}

	############################################################

	my $dns_table = VirtualServ::list_dns();
	my $hostname = Safe::backtick('/bin/hostname');
	my $defaultgw = getDefaultGateway('gateway');
	my $gw_opt1 = $_R{"ADAPTER"} . '1';
	my $gw_opt2 = $_R{"ADAPTER"} . '2';
	my $gw_opt  = getDefaultGateway();
	$gw_opt->{nic} =~ /(\w+)(\d+)/;
	my $gw_num_add = $2 + 1;	
	$gw_opt->{nic} = $_R{"ADAPTER"} . $gw_num_add;

	my $html = "<script>	
			var iSpecial			= '$_R{'ERR_iSpecial'}'; 
			var iIP					= '$_R{'ERR_iIP'}';  
			var iHost				= '$_R{'ERR_iHost'}';  
			var iBasicIP			= '$_R{'ERR_iBasicIP'}'; 
			var iAllNICDisable		= '$_R{'ERR_iAllNICDisable'}';  
			var mNotChecked			= '$_R{'ERR_mNotChecked'}'; 
			var mConfirmVIPDelete	= '$_R{'ERR_mConfirmVIPDelete'}';
			var mEmptyString		= '$_R{'ERR_mEmptyString'}';
		</script><script src=/style/net_val_lib.js language=javascript></script>" .
          $q->start_form({-name=>'action',-action=>'server_network.cgi'}) . "\n" .
		  $q->table({-border=>'0',-bgcolor=>'#eeeeee',-width=>'505'}, $q->Tr( $q->td ( 
		  $q->table({-border=>'0',-width=>'100%'},
			$q->Tr( 
				$q->td({-colspan=>'2',-width=>'100%'},"&nbsp")) . "\n".
            $q->Tr(
                                $q->td( $q->b( $spaces.$_R{"HOSTNAME_LABEL"})) .
                                $q->td( $q->textfield(-name=>'host',-value=>"$hostname", -size=>25,-maxlength=>30))) . "\n".
                                $q->hidden(-name=>'oldhost',-value=>"$hostname") . "\n".
                                $q->hidden(-name=>'olddomain',-value=>"$dns_table->{'domain'}->[0]") . "\n".

            $q->Tr(
				$q->td({-nowrap=>''},$q->b( $spaces.$_R{"DOMAIN_NAME_LABEL"})) . "\n" .
				$q->td($q->textfield(-name=>'domain',-value=>"$dns_table->{'domain'}->[0]", -size=>25,-maxlength=>32))) . "\n".
			$q->Tr(
				$q->td({-colspan=>'2',-width=>'100%'},"&nbsp")) . "\n",
			$q->Tr(
				$q->td({-nowrap=>''},$q->b( $spaces.$_R{"PRIMARY_DNS_SERVER_ADDRESS_LABEL"})) .
				$q->td($q->textfield(-name=>'pdns1',-value=>"$dns_table->{'name'}->[0]", -size=>14,-maxlength=>15))) . "\n".
			$q->Tr(
				$q->td({-nowrap=>''},$q->b( $spaces.$_R{"SECONDARY_DNS_SERVER_ADDRESS_LABEL"})) .
				$q->td($q->textfield(-name=>'sdns1',-value=>"$dns_table->{'name'}->[1]", -size=>14,-maxlength=>15))) . "\n".
			$q->Tr(
				$q->td({-nowrap=>''},$q->b( $spaces.$_R{"TERTIARY_DNS_SERVER_ADDRESS_LABEL"})) .
				$q->td($q->textfield(-name=>'sdns2',-value=>"$dns_table->{'name'}->[2]",-size=>14,-maxlength=>15))) . "\n".
			$q->Tr(
				$q->td({-colspan=>'2',-width=>'100%'},"&nbsp")) . "\n",
			$q->Tr(
				$q->td({-colspan=>'2',-align=>'right',-width=>'100%'},
					$q->submit(-class=>'clsButton',-name=>'action',-value=>$_R{"APPLY"},
                                -onClick=>'return validate_server_network(document.forms[0].host, false, document.forms[0].domain, true, document.forms[0].pdns1, true, document.forms[0].sdns1, true, document.forms[0].sdns2, true)'
			))))))) . "\n" .
		$q->endform . "\n" .
		
		$q->start_form({-name=>'TrueIP',-action=>'server_network.cgi'}) . "\n" .
		$q->table({-border=>'0',-bgcolor=>'#eeeeee',-width=>'505'}, $q->Tr( $q->td ( 
		$q->table({-border=>'0',-width=>'100%'},
			$q->Tr( 
				$q->td({-colspan=>'2',-width=>'100%'},"&nbsp" . $q->b($_R{'IP_ADDRESSES'})))) . "\n".
		$q->table({-border=>'0',-width=>'100%',-cellspacing=>'3',-cellpadding=>'2'},
			$q->Tr( {-bgcolor=>'#808080'}, 
				$q->td( {-align=>'center'}, $q->font( {-color=>'white',-height=>'35'}, $q->b($_R{'DISABLE'}))) . "\n".
				$q->td( $q->font( {-color=>'white',-height=>'35'}, $q->b($_R{'INTERFACE'}))) . "\n".
				$q->td( {-align=>'left',-height=>'35'}, $q->font( {-color=>'white'}, $q->b($_R{'IP_ADDRESS'}))) . "\n".
				$q->td( {-align=>'left',-height=>'35'}, $q->font( {-color=>'white'}, $q->b('&nbsp;'.$_R{'NETMASK'}), $q->br(), 
                                $q->checkbox(-name=>'nmask',-value=>'on',-label=>$_R{'USE_DEFAULT'},-onClick=>'setNetmask(document.forms[1],1)'))) . "\n".
				$q->td( {-align=>'left',-height=>'35'}, $q->font( {-color=>'white'}, $q->b('&nbsp;'.$_R{'BROADCAST'}), $q->br(), 
                                $q->checkbox(-name=>'bcast',-value=>'on',-label=>$_R{'USE_DEFAULT'},-onClick=>'setBcast(document.forms[1],1)')))) . "\n",
			$html3) . "\n".
		$q->table({-border=>'0',-width=>'100%'},
			$q->Tr( 
				$q->td({-colspan=>'2',-width=>'100%'},"&nbsp")) . "\n".

		## Default Gateway
		############################################################	
		
			$q->Tr( 
				$q->td($q->b( $sp . $_R{'DEFAULT_GATEWAY_LABEL'})) . "\n".
				$q->td($q->textfield( -name=>'defaultgw', -value=>"$defaultgw", -size=>'14', -maxlength=>'15') . '&nbsp;&nbsp;' . $q->popup_menu( -name=>'gwnic', -values=>["$gw_opt1", "$gw_opt2"], -default=>"$gw_opt->{nic}"))) . "\n".
            $q->Tr( 
				$q->td({-colspan=>'2',-width=>'100%'},"&nbsp")) . "\n".

		############################################################	

            $q->Tr( 
				$q->td( {-colspan=>'2',-width=>'100%'}, "&nbsp;" . "\n".
					$q->button(-class=>'clsButton', -name=>'server_route',-value=>$_R{'CONFIGURE_ROUTING'},-onclick=>"javascript:location='server_route.cgi'")."\n".
					$q->button(-class=>'clsButton', -name=>'netwk_diag',-value=>$_R{'DIAGNOSTICS'},-onclick=>"javascript:location='server_diag.cgi'")))."\n",
			$q->Tr( 
				$q->td( {-align=>'right',-colspan=>'2', -width=>'100%'}, 
				$q->submit(-class=>'clsButton', -name=>'purpose', -value=>$_R{'APPLY'}, -onClick=>'return validate_server_network_IP_address(document.forms[1], true, true, true, true)'))) ."\n".
			$q->hidden( {-name=>'trueip', -value=>'yes'}))))) . "\n" .
		$q->end_form . "\n" .
		
		$q->start_form( {-name=>'VirtualIP',-action=>'server_network.cgi'}) . "\n" .
		$q->table( {-border=>'0',-bgcolor=>'#eeeeee',-width=>'505'}, $q->Tr( $q->td ( 
		$q->table( {-border=>'0',-width=>'100%'},
			$q->Tr( 
				$q->td( {-colspan=>'4', -align=>'left'}, $q->b( "&nbsp;" . $_R{"VIRTUAL_IP_ADDRESSES"}))) . "\n".
			$q->Tr(
				$q->td( {-colspan=>'2'}, "\n".
					$q->submit(-class=>'clsButton', -name=>'purpose', -value=>$_R{"ADD"}). "\n".
					$q->submit(-class=>'clsButton', -name=>'purpose', -value=>$_R{"DELETE"},-onClick=>'return confirm_server_network_VIP_delete(document.forms[2])'))). "\n".
			$q->Tr( 
				$q->td({-colspan=>'2',-width=>'100%'},"&nbsp"))) . "\n".
		$q->table({-border=>'0',-width=>'100%',-cellspacing=>'3',-cellpadding=>'2'},
			$q->Tr( {-bgcolor=>'#808080'}, 
				$q->td( $q->font( {-color=>'white',-height=>'35'}, "&nbsp;".$q->b( $_R{"INTERFACE"}))) . "\n".
				$q->td( {-align=>'left',-height=>'35'}, $q->font( {-color=>'white'}, $q->b( $_R{"IP_ADDRESS"}))) . "\n".
                $q->td( {-align=>'left',-height=>'35'}, $q->font( {-color=>'white'}, $q->b('&nbsp;'.$_R{"NETMASK"}), $q->br(),
                        $q->checkbox(-name=>'nmask',-value=>'on',-label=>$_R{"USE_DEFAULT"},-onClick=>'setNetmask(document.forms[2],2)'))) . "\n".
                $q->td( {-align=>'left',-height=>'35'}, $q->font( {-color=>'white'}, $q->b('&nbsp;'.$_R{"BROADCAST"}), $q->br(),
                        $q->checkbox(-name=>'bcast',-value=>'on',-label=>$_R{"USE_DEFAULT"},-onClick=>'setBcast(document.forms[2],2)')))) . "\n".
			$html2) . "\n".
		$q->table( {-border=>'0',-width=>'100%'},
			$q->Tr( 
				$q->td( '&nbsp;', $q->hidden( {-name=>'virtualip', -value=>'yes'}))) . "\n".
			$q->td( {-align=>'right',-colspan=>'4'},
				$q->submit(-class=>'clsButton',-name=>'purpose',-value=>$_R{"APPLY"}, -onClick=>'return validate_server_network_VIP_address(document.forms[2], true, true, true)'))))))."\n".
		$q->endform;
	my $localfeedback;
	if ($feedback > 0)
	{
		$localfeedback = $StringResource->_S("$feedback");	
    }
	elsif ((defined($feedback)) && (0 == $feedback))
	{
		$html .= '<script>parent.title.location = "server_title.cgi";</script>';
	}

    DisplayMain->printDocument({ 'q' => $q, 'html' => $html,
								 'class' => 'server_network', 'id' => 4500, 'title' => $_R{'IDT:4500'},
								 'root-link' => 'server_network', 'result' => $feedback, 'feedback' => $localfeedback });
}

##############################################################################
##  Notify IS4 that network settings have changed.
##############################################################################

sub notifyIS4
{
	my $Status = Safe::system("/home/isroot4/bin/IS4-config");

	if (0 != $Status)
	{
		print STDERR "IS4-config returned $Status\n";
	}
	
	Safe::system("/etc/rc.d/init.d/httpd restart");
}

################################################################################
##  Main script
################################################################################

sub Main
{
	# define the vars
	my $q = new CGI;
	my $feedback;
	my $purpose = $q->param('purpose') || "";
	my $action = $q->param('action') || "";	
	
	if ($action)
	{
		$feedback = updateHostDNS($q);
	}
	elsif ($purpose)
	{ 
		my $ProductInfo = new ProductInformation();
		
		$feedback = checkParams($q, $purpose);
		
		# Look for a storefront configuration...
		if (2 == $ProductInfo->getClassification())
		{
			notifyIS4();
		}
	}
	
	makeBody($q, $feedback); 
}

Main();
