#!/usr/bin/perl

###############################################################################
#                                                                             #
# Filename: server_rpminfo.cgi                                                #
#                                                                             #
# Description: Provides status information for installed RPMs.                #
#                                                                             #
###############################################################################
###############################################################################
#                                                                             #
#  Copyright  2000 - 2001 Intel Corporation.                                 #
#  Intel Corporation All Rights Reserved.                                     #
#                                                                             #
#  The source code contained or described herein and all documents related to #
#  the source code ("Material") are owned by Intel Corporation or its         #
#  suppliers or licensors.  Title to the Material remains with Intel          #
#  Corporation or its suppliers and licensors.  The Material contains trade   #
#  secrets and proprietary and confidential information of Intel or its       #
#  suppliers and licensors.  The Material is protected by worldwide copyright #
#  and trade secret laws and treaty provisions.  No part of the Material may  #
#  be used, copied, reproduced, modified, published, uploaded, posted,        #
#  transmitted, distributed, or disclosed in any way without Intel's prior    #
#  express written permission.                                                #
#                                                                             #
#  No license under any patent, copyright, trade secret or other intellectual #
#  property right is granted to or conferred upon you by disclosure or        #
#  delivery of the Materials, either expressly, by implication, inducement,   #
#  estoppel or otherwise. Any license under such intellectual property rights #
#  must be express and approved by Intel in writing.                          #
#                                                                             #
###############################################################################

# Put the PERL library in the path.
use lib '/usr/ins/intel/gui/perllib';

use strict;
use LanguageResource;
use DisplayMain;
use Safe;
use CGI;

###############################################################################
# Main
###############################################################################

sub Main
{
	my ($cgi)						= @_;
	my $PreferredLanguage			= (split(/,/, $ENV{'HTTP_ACCEPT_LANGUAGE'}))[0];
	my $StringResource				= new LanguageResource($PreferredLanguage, '/usr/ins/intel/gui/locale', [ 'server_rpminfo.lrc', 'server_updates.lrc', 'server_updates_more.lrc' ]);
	my %_R							= ( 'IDT:2200'					=> '',
										'IDT:2300'					=> '',
										'IDT:2400'					=> '',
										'NAME'						=> '',
										'VERSION_LABEL'				=> '',
										'RELEASE'					=> '',
										'INSTALL_DATE_LABEL'		=> '',
										'SIZE'						=> '',
										'PACKAGER'					=> '',
										'VENDOR'					=> '',
										'BUILD_DATE'				=> '',
										'SOURCE_RPM'				=> '',
										'SUMMARY'					=> '',
										'DESCRIPTION'               => '' );
	my %RPMData;
	my $html;
	
	# Get the string resources.
	$StringResource->GetStrings(\%_R);

	# If there is an RPM name...
	if ($RPMData{'name'} = $cgi->param('rpmname'))
	{
		# Get the rest of the information...
		$RPMData{'query_info'}	= Safe::backtick("/bin/rpm -qi $RPMData{'name'}");
		$RPMData{'version'}		= Safe::backtick("/bin/rpm -q --queryformat '%{VERSION}' $RPMData{'name'}");
		$RPMData{'release'}		= Safe::backtick("/bin/rpm -q --queryformat '%{RELEASE}' $RPMData{'name'}");
		$RPMData{'installdate'}	= Safe::backtick("/bin/rpm -q --queryformat '%{INSTALLTIME:date}' $RPMData{'name'}");
		$RPMData{'size'}		= Safe::backtick("/bin/rpm -q --queryformat '%{SIZE}' $RPMData{'name'}");
		$RPMData{'packager'}	= Safe::backtick("/bin/rpm -q --queryformat '%{PACKAGER}' $RPMData{'name'}");
		$RPMData{'vendor'}		= Safe::backtick("/bin/rpm -q --queryformat '%{VENDOR}' $RPMData{'name'}");
		$RPMData{'builddate'}	= Safe::backtick("/bin/rpm -q --queryformat '%{BUILDTIME:date}' $RPMData{'name'}");
		$RPMData{'srcrpm'}		= Safe::backtick("/bin/rpm -q --queryformat '%{SOURCERPM}' $RPMData{'name'}");
		$RPMData{'summary'}		= Safe::backtick("/bin/rpm -q --queryformat '%{SUMMARY}' $RPMData{'name'}");
		$RPMData{'description'}	= Safe::backtick("/bin/rpm -q --queryformat '%{DESCRIPTION}' $RPMData{'name'}");
	}
    
	$html = <<HERE;
<table border=0 width=375>
<tr>
	<td width=30%><b>$_R{'NAME'}</b></td>
	<td width=70%>$RPMData{'name'}</td>
</tr>
<tr>
	<td width=30%><b>$_R{'VERSION_LABEL'}</b></td>
	<td width=70%>$RPMData{'version'}</td>
</tr>
<tr>
	<td width=30%><b>$_R{'RELEASE'}</b></td>
	<td width=70%>$RPMData{'release'}</td>
</tr>
<tr>
	<td width=30%><b>$_R{'INSTALL_DATE_LABEL'}</b></td>
	<td width=70%>$RPMData{'installdate'}</td>
</tr>
<tr>
	<td width=30%><b>$_R{'SIZE'}</b></td>
	<td width=70%>$RPMData{'size'}</td>
</tr>
<tr>
	<td width=30%><b>$_R{'PACKAGER'}</b></td>
	<td width=70%>$RPMData{'packager'}</td>
</tr>
<tr>
	<td width=30%><b>$_R{'VENDOR'}</b></td>
	<td width=70%>$RPMData{'vendor'}</td>
</tr>
<tr>
	<td width=30%><b>$_R{'BUILD_DATE'}</b></td>
	<td width=70%>$RPMData{'builddate'}</td>
</tr>
<tr>
	<td width=30%><b>$_R{'SOURCE_RPM'}</b></td>
	<td width=70%>$RPMData{'srcrpm'}</td>
</tr>
<tr>
	<td width=100% colspan=2>&nbsp;</td>
</tr>
<tr>
	<td width=100% colspan=2><b>$_R{'SUMMARY'}</b></td>
</tr>
<tr>
	<td width=100% colspan=2>$RPMData{'summary'}</td>
</tr>
<tr>
	<td width=100% colspan=2>&nbsp;</td>
</tr>
<tr>
	<td width=100% colspan=2><b>$_R{'DESCRIPTION'}</b></td>
</tr>
<tr>
	<td width=100% colspan=2>$RPMData{'description'}</td>
</tr>
</table>
HERE

	# Display the page.
    DisplayMain->printDocument({ 'q' => $cgi, 'html' => $html,
								 'class' => 'server_rpminfo', 'id' => 2200, 'title' => $_R{'IDT:2200'}, 'root-link' => 'server_updates', 
								 'cookiecrumbs' => [ { 'name' => $_R{'IDT:2400'}, 'location' => 'server_updates.cgi' }, 
													 { 'name' => $_R{'IDT:2300'}, 'location' => 'server_updates_more.cgi' } ] });
}

###############################################################################
# The main body of the script.
###############################################################################

my $cgi = new CGI;

Main($cgi);
