#!/usr/bin/perl

###############################################################################
#                                                                             #
# Filename: server_snmp.cgi                                                   #
#                                                                             #
# Description: CGI needed to display and modify the SNMP settings.            #
#                                                                             #
###############################################################################
###############################################################################
#                                                                             #
#  Copyright  2000 - 2001 Intel Corporation.                                 #
#  Intel Corporation All Rights Reserved.                                     #
#                                                                             #
#  The source code contained or described herein and all documents related to #
#  the source code ("Material") are owned by Intel Corporation or its         #
#  suppliers or licensors.  Title to the Material remains with Intel          #
#  Corporation or its suppliers and licensors.  The Material contains trade   #
#  secrets and proprietary and confidential information of Intel or its       #
#  suppliers and licensors.  The Material is protected by worldwide copyright #
#  and trade secret laws and treaty provisions.  No part of the Material may  #
#  be used, copied, reproduced, modified, published, uploaded, posted,        #
#  transmitted, distributed, or disclosed in any way without Intel's prior    #
#  express written permission.                                                #
#                                                                             #
#  No license under any patent, copyright, trade secret or other intellectual #
#  property right is granted to or conferred upon you by disclosure or        #
#  delivery of the Materials, either expressly, by implication, inducement,   #
#  estoppel or otherwise. Any license under such intellectual property rights #
#  must be express and approved by Intel in writing.                          #
#                                                                             #
###############################################################################

# Put the PERL library in the path.
use lib '/usr/ins/intel/gui/perllib';

use strict;
use LanguageResource;
use DisplayMain;
use IO::File;
use Daemon;
use Safe;
use CGI;

###############################################################################
# ConfigFileRead
###############################################################################

sub ConfigFileRead
{
	my $ConfigFile = new IO::File('< /etc/snmp/snmpd.conf');
	my @ConfigFileContents;

	# If the file is not open...
	unless ($ConfigFile)
	{
		exit(0);
	}

	# Snag the contents.
	@ConfigFileContents = <$ConfigFile>;
	
	# Close the file.
	$ConfigFile->close;

	return @ConfigFileContents;
}

###############################################################################
# GetConfigData
###############################################################################

sub GetConfigData
{
	my (@ConfigFileContents) = @_;
    my $Value;
	my $Line;

	# Loop through all of the lines in the file.
	foreach $Line (@ConfigFileContents)
	{
		# Continue if there is a comment.
		next if $Line =~ /^#/;

		# Continue if the label is not found.
		next unless $Line =~ /.*com2sec.*User.*default/;
		
		# Parse out the value.
		$Value = $Line;
		$Value =~ s/.*com2sec.*User.*default\s*(.*?)\s*$/$1/;
		chomp $Value;
		
		return $Value;
	}

	return '';
}

###############################################################################
# SetConfigData
###############################################################################

sub SetConfigData
{
	my ($Community) = @_;
	
	# Apply the new community settings.
	Safe::backtick("/usr/ins/intel/bin/asm/bakAsm set_snmp $Community");

	return 1;
}

###############################################################################
# Main
###############################################################################

sub Main
{
	my ($cgi) = @_;
	my $PreferredLanguage	= (split(/,/, $ENV{'HTTP_ACCEPT_LANGUAGE'}))[0];
	my $StringResource		= new LanguageResource($PreferredLanguage, '/usr/ins/intel/gui/locale', [ 'server_snmp.lrc', 'common.lrc' ]);
	my %_R					= ( 'IDT:1000'			=> '',
								'SERVICE'			=> '',
								'STATUS'			=> '',
								'CONTROLS'			=> '',
								'COMMUNITY_NAME'	=> '',
								'STOP'				=> '',
								'CONFIRM_STOP'		=> '',
								'START'				=> '',
								'RESTART'			=> '',
								'APPLY'				=> '' );
	my $SNMPDaemon			= new Daemon('snmpd');
	my $SNMPStatus;
	my $Community			= $cgi->param('community');
	my $Feedback;
	my $Result;
	my $html;
	
	# Get the string resources.
	$StringResource->GetStrings(\%_R);
	
	# Safe default.
	$Community = '' unless(defined($Community));

	SWITCH:
	{
		# Control the service.
		if (defined($cgi->param('control_start')))		{ $SNMPDaemon->Control(1); last SWITCH; }
		if (defined($cgi->param('control_stop')))		{ $SNMPDaemon->Control(0); last SWITCH; }
		if (defined($cgi->param('control_restart')))	{ $SNMPDaemon->Control(2); last SWITCH; }
	}
	
	# If applying a change...
	if (defined($cgi->param('apply')))
	{
		# Make sure that the data is valid.
		if ($Community !~ /^\w*$/)
		{
			# Set the status.
			$Result		= 100000;
			$Feedback	= $StringResource->_S($Result);
		}
		else
		{
			# Try to set the data.
			$Result = (SetConfigData($Community)) ? 0 : 100001;
		}
	}
	
	# If the community is not yet set...
	unless ($Community)
	{
		# Get the current community value.
		$Community = GetConfigData(ConfigFileRead());
	}
	
	# Get the SNMP process status...
	if (1 == ($SNMPStatus = $SNMPDaemon->GetStatus()))
	{
		# Indicate running.
		$SNMPStatus = '<font color=green>' . $StringResource->_S('RUNNING') . '</font>';
	}
	else
	{
		# Indicate stopped.
		$SNMPStatus = '<font color=red>' . $StringResource->_S('STOPPED') . '</font>';
	}
	
	$html = <<HERE;
<form action=server_snmp.cgi method=post enctype="application/x-www-form-urlencoded">
	<table border=0 width=375 bgcolor=#eeeeee>
	<tr>
		<td>&nbsp;
			<table border=1 cellpadding=2 cellspacing=0 width=100% bgcolor=#ffffff bordercolor=#eeeeee>
			<tr>
				<td width=40% bgcolor=#808080><b><font color=#ffffff>$_R{'SERVICE'}</font></b></td>
				<td width=15% bgcolor=#808080><b><font color=#ffffff>$_R{'STATUS'}</font></b></td>
				<td width=40% bgcolor=#808080><b><font color=#ffffff>$_R{'CONTROLS'}</font></b></td>
			</tr>
			<tr>
				<td width=40%><b>$_R{'IDT:1000'}</b></td>
				<td width=15%>$SNMPStatus</td>
				<td width=40% align=left nowrap>
					<input type=submit class=clsButton value="$_R{'START'}" name=control_start>
					<input type=submit class=clsButton value="$_R{'STOP'}" name=control_stop onclick="javascript: return confirm('$_R{'CONFIRM_STOP'}');">
					<input type=submit class=clsButton value="$_R{'RESTART'}" name=control_restart> 
				</td>
			</tr>
			</table>
			<table border=0 width=100%>
			<tr>
				<td width=100% colspan=2>&nbsp;</td>
			</tr>
			<tr>
				<td width=50%><font color=black><b>$_R{'COMMUNITY_NAME'}</b></font></td>
				<td width=50% align=right><input type=text name=community maxlength=16 value='$Community'></td>
			</tr>
			<tr>
				<td width=100% colspan=2>&nbsp;</td>
			</tr>
			<tr>
				<td width=100% align=right colspan=2><input type=submit class=clsButton value='$_R{'APPLY'}' name=apply></td>
			</tr>
			</table>
		</td>
	</tr>
	</table>
</form>
HERE
	
	# Display the page.
    DisplayMain->printDocument({ 'q' => $cgi, 'html' => $html, 
								 'class' => 'server_snmp', 'id' => 1000, 'title' => $_R{'IDT:1000'}, 
								 'root-link' => 'server_snmp', 'result' => $Result, 'feedback' => $Feedback });
}

###############################################################################
# The main body of the script.
###############################################################################

my $cgi = new CGI;

Main($cgi);
