#!/usr/bin/perl

###############################################################################
#                                                                             #
# Filename: server_status.cgi                                                 #
#                                                                             #
# Description: CGI needed to display the server status overview.              #
#                                                                             #
###############################################################################
###############################################################################
#                                                                             #
#  Copyright  2000 - 2001 Intel Corporation.                                 #
#  Intel Corporation All Rights Reserved.                                     #
#                                                                             #
#  The source code contained or described herein and all documents related to #
#  the source code ("Material") are owned by Intel Corporation or its         #
#  suppliers or licensors.  Title to the Material remains with Intel          #
#  Corporation or its suppliers and licensors.  The Material contains trade   #
#  secrets and proprietary and confidential information of Intel or its       #
#  suppliers and licensors.  The Material is protected by worldwide copyright #
#  and trade secret laws and treaty provisions.  No part of the Material may  #
#  be used, copied, reproduced, modified, published, uploaded, posted,        #
#  transmitted, distributed, or disclosed in any way without Intel's prior    #
#  express written permission.                                                #
#                                                                             #
#  No license under any patent, copyright, trade secret or other intellectual #
#  property right is granted to or conferred upon you by disclosure or        #
#  delivery of the Materials, either expressly, by implication, inducement,   #
#  estoppel or otherwise. Any license under such intellectual property rights #
#  must be express and approved by Intel in writing.                          #
#                                                                             #
###############################################################################

# Put the PERL library in the path.
use lib '/usr/ins/intel/gui/perllib';

use strict;
use ProductInformation;
use LanguageTime;
use LanguageResource;
use DisplayMain;
use Daemon;
use ConfigHandler;
my (%variables, %db, $defaultRefreshRate);
use CGI;
use MLDBM 'DB_File';
use Safe;

my $PreferredLanguage	= (split(/,/, $ENV{'HTTP_ACCEPT_LANGUAGE'}))[0];
my $ProductInfo			= new ProductInformation();
my $StringResource		= new LanguageResource($PreferredLanguage, 
											   '/usr/ins/intel/gui/locale', 
											   [ 'server_status.lrc',
											     'server_cpu.lrc',
												 'server_memory.lrc',
												 'server_disk.lrc',
												 'common.lrc' ]);
my %_R					= ( 'IDT:600'					=> '',
							'REFRESH_EVERY_X_SECONDS_A'	=> '',
							'REFRESH_EVERY_X_SECONDS_B'	=> '',
							'UP_TIME'					=> '',
							'LOAD_AVERAGE'				=> '',
							'SERVICE'					=> '',
							'STATUS'					=> '',
							'WEB'						=> '',
							'MAIL'						=> '',
							'FTP'						=> '',
							'TELNET'					=> '',
							'SNMP'						=> '',
							'DNS'						=> '',
							'MORE_ELLIPSIS'				=> '',
							'CPU_USAGE'					=> '',
							'USER'						=> '',
							'SYSTEM'					=> '',
							'MEMORY_USAGE'				=> '',
							'PHYSICAL'					=> '',
							'SWAP'						=> '',
							'DRIVE_SPACE_USAGE'			=> '',
							'MB'						=> '',
							'GB'						=> '',
							'FREE'						=> '',
							'TOTAL'						=> '',
							'RUNNING'					=> '',
							'STOPPED'					=> '',
							'MORE_HARDWARE_INFORMATION'	=> '',
							'OUT_OF_RANGE'				=> '',
							'AND'						=> '' );
my $Key;

# Get the string resources.
$StringResource->GetStrings(\%_R);

$variables{q} = new CGI;

$defaultRefreshRate = ConfigHandler::getConfig(ConfigHandler::getPath("CONSTANTS"), "DEFAULT_REFRESH_RATE");

$variables{procinfo}   = "/usr/bin/procinfo -b";
$variables{df_program} = "/bin/df -h";

$variables{username} = $variables{q}->remote_user;

($variables{name}, $variables{passwd}, $variables{uid}, $variables{gid},
 $variables{quota}, $variables{comment}, $variables{gcos}, $variables{homedir},
 $variables{shell}, $variables{expire}) = getpwnam("$variables{username}");

if ($variables{q}->param('refresh_rate')) {
    $variables{refresh_rate} = $variables{q}->param('refresh_rate');
}

  if (tie (%db, 'MLDBM', "$variables{homedir}/.user_config.db")) {
	my $hash_ref = $db{status_page};    
	if (!$hash_ref->{refresh_rate}) {
	  if ($variables{refresh_rate}) {
		$hash_ref->{refresh_rate} = $variables{refresh_rate};
	  }
	  else {
		$hash_ref->{refresh_rate} = $defaultRefreshRate;
	  	$variables{refresh_rate} = $defaultRefreshRate;
	  }
	} 
	elsif ($variables{refresh_rate}) {
	  $hash_ref->{refresh_rate} = $variables{refresh_rate};
	}
	elsif ($hash_ref->{refresh_rate}) {
	  $variables{refresh_rate} = $hash_ref->{refresh_rate};
	}

	$db{status_page} = $hash_ref;

     untie %db; # Close database file

  } else {

   warn 'FILE_OPEN_ERROR'	=> '', . ": $variables{homedir}/.user_config.db - $! \n";
   $variables{refresh_rate} = $defaultRefreshRate;

  }

# Set up the proc_status hash.
$variables{'proc_status'} = { 'httpd'		=> '',
							  'sendmail'	=> ': accepting connections',
							  'named'		=> '' };

# Run through the process list...
foreach $Key (keys(%{ $variables{'proc_status'} }))
{
	my $CurrentDaemon = new Daemon($Key, $variables{'proc_status'}->{$Key});

	# Get the daemon status text...
	$variables{'proc_status'}->{$Key}	= ($CurrentDaemon->GetStatus())  ? "<font color=green>$_R{'RUNNING'}<font>" : "<font color=red>$_R{'STOPPED'}<font>";
}

make_html_body_server(\%variables);

$variables{refresh_url} = "/cgi-bin/server/server_status.cgi";
$variables{'class'} = 'server_status';
$variables{'id'} = '600';
$variables{'root-link'} = 'server_status';

DisplayMain->printDocument(\%variables);

sub make_html_body_server {
 
my $var_ref = $_[0];
my $Inetd	= new Daemon('inetd', '');
my $telnet;
my $ftp;

# Get the status of "ftp" and "telnet" service...
if ($Inetd->GetStatus())
{
	open(FHH, "/etc/inetd.conf");
	$telnet = $var_ref->{q}->font({-color=>'red'}, $_R{'STOPPED'});	# Default telnet to disabled
	$ftp    = $var_ref->{q}->font({-color=>'red'}, $_R{'STOPPED'});	# Default ftp    to disabled
	
	while(<FHH>) {
		 if (/^telnet/) {
		 $telnet = $var_ref->{q}->font({-color=>'green'}, $_R{'RUNNING'});
		 }
		 elsif (/^ftp/) {
		 $ftp = $var_ref->{q}->font({-color=>'green'}, $_R{'RUNNING'});
		 }
	}
	close(FHH);              
}
else
{
	$telnet = $var_ref->{q}->font({-color=>'red'}, $_R{'STOPPED'});	# telnet to disabled
	$ftp    = $var_ref->{q}->font({-color=>'red'}, $_R{'STOPPED'});	# ftp    to disabled
}

$var_ref->{html} = Safe::backtick($var_ref->{procinfo});
$_ = $var_ref->{html};

my $kernel = $1 if /(\w+ \S+)/g;

my $actual_memory = $1 if /Mem:\s*(\w+)/g;
my $used_memory = $1 if /(\w+)/g;
my $free_memory = $1 if /(\w+)/g;

my $shared_memory = $1 if /(\w+)/g;
my $buffered_memory = $1 if /(\w+)/g;
my $cached_memory = $1 if /(\w+)/g;

   $used_memory = $used_memory - $buffered_memory - $cached_memory;
   $free_memory = $actual_memory - $used_memory;

my $memory_percent_used = formatFloat($used_memory/$actual_memory * 100);
my $memory_percent_remaining = 100 - $memory_percent_used;
   $free_memory = formatFloat($free_memory/1024);
   $actual_memory = formatFloat($actual_memory/1024);

my $swap_memory = $1 if /Swap:\s*(\w+)/g;
my $swap_used = $1 if /(\w+)/g;
my $swap_free = $1 if /(\w+)/g;

my $swap_percent_used = formatFloat($swap_used/$swap_memory * 100);
my $swap_percent_remaining = 100 - $swap_percent_used;
 
   $swap_free = formatFloat($swap_free/1024);
   $swap_memory = formatFloat($swap_memory/1024); 

my $boot_time = $1 if /Bootup:\s*(\w+\s*\w+\s*\w+\s*\S+\s*\w+)/g;

my $load_avg = $1 if /average:\s*(\S+ \S+ \S+ \S+ \S+)/g;

my $cpu_user   = $1 if /user  :..................\s*(\w+\.*\w*)/g;
my $cpu_user_graph = formatFloat($cpu_user);
my $page_in    = $1 if /page in :\s*(\w+)/g;
my $disk1_read = $1 if /disk 1:\s*(\d+)/g;
my $disk1_write= $1 if /(\d+)/g;
my $cpu_nice   = $1 if /nice  :..................\s*(\w+\.*\w*)/g;
my $cpu_nice_graph = formatFloat($cpu_nice);
my $page_out   = $1 if /page out:\s*(\w+)/g;

my $cpu_system = $1 if /system:..................\s*(\w+\.*\w*)/g;
my $cpu_system_graph = formatFloat($cpu_system);
my $swap_in    = $1 if /swap in :\s*(\w+)/g;

my $cpu_idle   = $1 if /idle  :..................\s*(\w+\.*\w*)/g;
my $cpu_idle_graph = formatFloat($cpu_idle);
my $swap_out   = $1 if /swap out:\s*(\w+)/g;

my $cpu_uptime = $1 if /uptime:\s*(\S*\s*\S+)/g;
$cpu_uptime =~ s/context//;

my $Days = $cpu_uptime;

# Get the days (which do not exist for fresh systems).
$Days =~ s/^(\d*)d\D+\d*:\d*:\d*\.*\d*$/$1/;
$Days =  0 unless($cpu_uptime ne $Days);

# Pare down the uptime to remove the days and trailing space.
$cpu_uptime =~ s/^\d*d\D+(\d*:\d*:\d*\.*\d*)$/$1/;
$cpu_uptime =~ s/^(\d*:\d*:\d*\.*\d*)\s*$/$1/;

# If there is a somewhat valid CPU uptime...
if ($cpu_uptime =~ /^(\d*):(\d*):(\d*\.*\d*)$/)
{
	my @Time	= ( $3, $2, $1, $Days );
	my $Match;

    $cpu_uptime = $StringResource->_S('UP_TIME_FORMAT');

	$Match = '%d';
	$cpu_uptime =~ s/$Match/$Time[3]/;
	$Match = '%H';
	$cpu_uptime =~ s/$Match/$Time[2]/;
	$Match = '%M';
	$cpu_uptime =~ s/$Match/$Time[1]/;
	$Match = '%S';
	$cpu_uptime =~ s/$Match/$Time[0]/;
}

my $context_switches = $1 if /context :\s*(\w+)/g;

$cpu_user_graph = 1   if ($cpu_user_graph == 0);
$cpu_system_graph = 1 if ($cpu_system_graph == 0);
$cpu_nice_graph = 1   if ($cpu_nice_graph == 0);
$cpu_idle_graph = 1   if ($cpu_idle_graph == 0);

my $df_info = Safe::backtick($var_ref->{df_program});

my $drive_info = <<HERE;
<table border=1 cellpadding=2 cellspacing=0 width=375 bgcolor=#ffffff bordercolor=#eeeeee>
HERE

my $DiskLabel			= '';
my $DiskPercentUsage	= '';
my $DiskPercentRemain	= '';
my $DiskSize			= '';
my $DiskUsed			= '';
my $DiskFree			= '';
my $graphColor;
my $Label;

# Run a loop to capture output from df
while ($df_info =~ /\/dev\/\S+\s*(\S+)\s*(\S+)\s*(\S+)\s*(\S+)\s*(\S+)/g) 
{
	$DiskLabel			= $5;
	$DiskPercentUsage	= $4;
	$DiskSize			= $1;
	$DiskUsed			= $2;
	$DiskFree			= $3;
	
	# Only show non-mounted partitions and exclude the emeregency partition...
	next if (($DiskLabel =~ /^\/mnt/) || ($DiskLabel eq '/emergency'));

	if ($DiskPercentUsage eq '0%') {
		$graphColor="white"; 
	} 
	else {
		$graphColor="#008000"; 
	}
	
	# Patch up the label.
	($DiskSize, $Label) = split(/(G|M)/, $DiskSize);
    
	# Add the label back on.
	$DiskSize .= ($Label eq 'G') ? " $_R{'GB'}" : " $_R{'MB'}";
	
	# Patch up the label.
	($DiskFree, $Label) = split(/(G|M)/, $DiskFree);
  
	# Add the label back on.
	$DiskFree .= ($Label eq 'G') ? " $_R{'GB'}" : " $_R{'MB'}";

	# Calculate the remaining percent.
	$DiskPercentRemain = 100 - substr($DiskPercentUsage, 0, -1);
  
	$drive_info .= <<HERE;
<tr>
  <td width="20%" nowrap><b>$DiskLabel&nbsp;</b></td>
  <td width="30%" nowrap>
	<table border="0" cellpadding="0" cellspacing="0" width="100%">
	  <tr align="right">
		<td align="right" width="100%">&nbsp;$DiskPercentUsage&nbsp;</td>
		<td align="right">
		  <table border="1" cellpadding="0" cellspacing="0" width="70">
			<tr>
			  <td width="95%" bgcolor="#FFFFFF">
				<table border="0" width="100%" cellspacing="0" cellpadding="0">
				  <tr>
					<td width="$DiskPercentUsage" bgcolor=$graphColor>&nbsp;</td>
					<td width="$DiskPercentRemain"></td>
				  </tr>
				</table>
			  </td>
			</tr>
		  </table>
		</td>
	  </tr>
	</table>
  </td>
  <td width="50%">&nbsp;<nobr>$DiskFree $_R{'FREE'} ($DiskSize $_R{'TOTAL'})</nobr></td>
</tr>
HERE
}
  
  $drive_info .= "</tr></table>";

$var_ref->{html} = $var_ref->{q}->start_form(-onSubmit=>"return checkBounds(document.forms[0].refresh_rate, 30, 9999, false)");

my $bgcolorCU	= (0 == $cpu_user_graph) ? "white" : "#008000";
my $bgcolorCS	= (0 == $cpu_system_graph) ? "white" : "#008000";
my $bgcolorMP	= (0 == $memory_percent_used) ? "white" : "#008000";
my $bgcolorMS	= (0 == $swap_percent_used) ? "white" : "#008000";

$var_ref->{title} = $_R{'IDT:600'};
$var_ref->{spillover} = <<HERE;
	<br>
	<table align=center border=0 cellspacing=0 cellpadding=2 bordercolor=#eeeeee bgcolor=#ffffff>
	<tr>
		<td align=right width=50% bgcolor=#eeeeee nowrap>$_R{'REFRESH_EVERY_X_SECONDS_A'}</td>
		<td align=center width=5% bgcolor=#eeeeee><input type=text name=refresh_rate value='$var_ref->{refresh_rate}' style='font-size: 10pt; width: 35; height: 23' size=4 tabindex=1></td>
		<td align=left width=50% bgcolor=#eeeeee nowrap>$_R{'REFRESH_EVERY_X_SECONDS_B'}</td>
	</tr>
	</table>
HERE

$var_ref->{html} .= <<HERE;
<script language=javascript>
var boolForIsEmpty = true; //this boolean must be the opposite of defaultBoolean
var whitespace = " ";

function isEmpty(s)
{
	return ((s == null) || (s.length == 0));
}

function isWhitespace(s)
{   
	if (!isEmpty(s))
	{
		var i;
		
		for (i = 0; i < s.length; i++)
		{
			if (-1 == whitespace.indexOf(s.charAt(i)))
			{
				return false;
			}
		}
	}
	
	return true;
}

function warnEmpty(theField, s)
{   
	theField.focus();
	
	alert(s);
	
	return false;
}

function isDigit (c)
{   return ((c >= "0") && (c <= "9")) }

function isInteger (s)
{   
    var i;

    if (isEmpty(s))
    { return (isItAnInteger.arguments[1] == boolForIsEmpty); }

    for (i = 0; i < s.length; i++)
    {
        var c = s.charAt(i);
        if (!isDigit(c)) return false;
    }

    return true;
}

function warnInvalid(theField, s)
{   
	theField.focus();
	theField.select();
	
	alert(s);
	
	return false;
}
 
function checkBounds(theField, minValue, maxValue, emptyOK)
{
    var iNum = '$_R{'OUT_OF_RANGE'}' + minValue + " " + '$_R{'AND'}' + " " + maxValue + '.';
    
    if (isEmpty(theField.value) && (checkBounds.arguments[1] == boolForIsEmpty))
    { return true; } 

    if (isWhitespace(theField.value) || isEmpty(theField.value))
       return warnEmpty (theField, iNum + minValue + " and " + maxValue) ;

    if(!isInteger(theField.value) || !((theField.value >= minValue) && (theField.value <= maxValue)))
       return warnInvalid (theField, iNum ) ;

    return true;
}
</script>
<table border="0" cellpadding="0" cellspacing="1" width="375">
    <TR>
      <TD BGCOLOR="white" HEIGHT=25>
        <table BORDER=0 width="100%" cellspacing="0" cellpadding="2" bordercolor="#EEEEEE" bgcolor="#FFFFFF">
		 <tr>
			<td align="left" width="115"><b>$_R{'UP_TIME'}</b></td>
			<td align="left" width="213">$cpu_uptime</td>
         </tr>
		 <tr>
			<td align="left" width="115"><b>$_R{'LOAD_AVERAGE'}</b></td>
			<td align="left" width="213">$load_avg</td>
         </tr>
		</table>
       </TD>
     </TR>
</table>
<br>
<table border="0" cellpadding="0" cellspacing="0" width="375" bgcolor="#EEEEEE">
  <tr>
    <td width="100%">
      <table border="1" cellpadding="2" cellspacing="0" width="100%" bgcolor="#FFFFFF" bordercolor="#EEEEEE">
		<tr>
		  <td width="40%" bgcolor="#808080"><b><font color="#FFFFFF">$_R{'SERVICE'}</font></b></td>
		  <td width="15%" valign="top" bgcolor="#808080"><b><font color="#FFFFFF">$_R{'STATUS'}</font></b></td>
		</tr>
          <tr>
          <td width="40%"><b><a href="server_http.cgi">$_R{'WEB'}</a></b></td>
          <td width="15%"><font color="#008000">$var_ref->{proc_status}->{'httpd'}</font></td>
        </tr>
        <tr>
          <td width="40%"><b><a href="server_mail.cgi">$_R{'MAIL'}</a></b></td>
          <td width="15%"><font color="#008000">$var_ref->{proc_status}->{'sendmail'}</font></td>
        </tr>
        <tr>
          <td width="40%"><b><a href="server_ftp.cgi">$_R{'FTP'}</a></b></td>
          <td width="15%">$ftp</td>
        </tr>
        <tr>
          <td width="40%"><b><a href="server_telnet.cgi">$_R{'TELNET'}</a></b></td>
          <td width="15%">$telnet</td>
        </tr>
        <tr>
          <td width="40%"><b><a href="server_dns.cgi">$_R{'DNS'}</a></b></td>
          <td width="15%"><font color="#008000">$var_ref->{proc_status}->{'named'}</font></td>
        </tr>
HERE

# Look for a storefront configuration...
if (2 == $ProductInfo->getClassification())
{
	eval
	{
		require IS4Control;
		require NetConfig;
		
		my @NetworkInterfaceList	= NetConfig->listNetInterfaces();
        my $IS4Status				= IS4Control::IS4Stopped();
		my $IS4StatusLabel;
		my $IS4Label				= $StringResource->_S('IS4_LABEL');
		my $IS4Login;
		my $IS4Unavailable;
		my $AddressIP;

		# Get the IS4Status...
		$IS4StatusLabel	= ($IS4Status) ? "<font color=red>$_R{'STOPPED'}<font>" : "<font color=green>$_R{'RUNNING'}<font>";
		$IS4Unavailable	= ($IS4Status) ? "onclick=\"javascript: alert('" . $StringResource->_S('IS4_DISABLED') . "'); return false;\" " : '';
		
		# Provide an alternate way of getting the IP address in case one fails.
		for (@NetworkInterfaceList)
		{
			# If the NIC matches...
			if ($_->{'fullnic'} eq 'eth0')
			{
				# Get the address.
				$AddressIP = $_->{'ipaddress'};
				last;
			}
		}

		# If the address is not yet available...
		if (0 == length($AddressIP))
		{
			# Try to get the address another way.
			$AddressIP = Safe::backtick("hostname --ip-address");
		}

		# Build the login string.
		$IS4Login = '<a ' . $IS4Unavailable . 'target=_blank href="' . ((IS4Control->adminSSLEnabled()) ? 'https:' : 'http:') . "//$AddressIP/cgi-bin/intershop.siteadmin\">" . $StringResource->_S('IS4_LOGIN') . '</a>';

$var_ref->{html} .= <<HERE;
        <tr>
          <td width="40%"><b><a href="server_ecomm.cgi">$IS4Label</a></b>&nbsp;[$IS4Login]</td>
          <td width="15%"><font color="#008000">$IS4StatusLabel</font></td>
        </tr>
HERE
	};
}

$var_ref->{html} .= <<HERE;
	  </table>
    </td>
  </tr>
</table>
<br>
<table border="0" cellpadding="2" cellspacing="0" width="375" bgcolor="#FFFFFF" bordercolor="#EEEEEE">
<tr>
  <td width="20%">
	<b>$_R{'CPU_USAGE'}</b>
  </td>
  <td width="20%" align="right">
	<a href="server_cpu.cgi">$_R{'MORE_ELLIPSIS'}</a>
  </td>
</tr>
</table>
<table border="1" cellpadding="2" cellspacing="0" width="375" bgcolor="#FFFFFF" bordercolor="#EEEEEE">
<tr>
  <td width="20%">
	<b>$_R{'USER'}&nbsp;</b>
  </td>
  <td width="30%" nowrap>
	<table border="0" cellpadding="0" cellspacing="0" width="100%">
	  <tr align="right">
		<td align="right" width="100%">$cpu_user%&nbsp;</td>
		<td align="right">
		  <table border="1" cellpadding="0" cellspacing="0" width="70">
			<tr>
			  <td width="95%" bgcolor="#FFFFFF">
				<table border="0" width="100%" cellspacing="0" cellpadding="0">
				  <tr>
					<td width="$cpu_user_graph%" bgcolor="$bgcolorCU">&nbsp;</td>
					<td width="96%"></td>
				  </tr>
				</table>
			  </td>
			</tr>
		  </table>
		</td>
	  </tr>
	</table>
  </td>
  <td width="50%">&nbsp;</td>
</tr>
<tr>
  <td width="20%">
	<b>$_R{'SYSTEM'}&nbsp;</b>
  </td>
  <td width="30%" nowrap>
	<table border="0" cellpadding="0" cellspacing="0" width="100%">
	  <tr align="right">
		<td align="right" width="100%">$cpu_system%&nbsp;</td>
		<td align="right">
		  <table border="1" cellpadding="0" cellspacing="0" width="70">
			<tr>
			  <td width="95%" bgcolor="#FFFFFF">
				<table border="0" width="100%" cellspacing="0" cellpadding="0">
				  <tr>
					<td width="$cpu_system_graph%" bgcolor="$bgcolorCS">&nbsp;</td>
					<td width="96%"></td>
				  </tr>
				</table>
			  </td>
			</tr>
		  </table>
		</td>
	  </tr>
	</table>
  </td>
  <td width="50%">&nbsp;</td>
</tr>
</table>
<br>
<table border="0" cellpadding="2" cellspacing="0" width="375" bgcolor="#FFFFFF" bordercolor="#EEEEEE">
<tr>
  <td width="20%">
	<b>$_R{'MEMORY_USAGE'}</b>
  </td>
  <td width="20%" align="right">
	<a href="server_memory.cgi">$_R{'MORE_ELLIPSIS'}</a>
  </td>
</tr>
</table>
<table border="1" cellpadding="2" cellspacing="0" width="375" bgcolor="#FFFFFF" bordercolor="#EEEEEE">
<tr>
  <td width="20%"><b>$_R{'PHYSICAL'}</b>&nbsp;</td>
  <td width="30%">
	<table border="0" cellpadding="0" cellspacing="0" width="100%">
	  <tr align="right">
		<td align="right" width="100%">&nbsp;$memory_percent_used%&nbsp;</td>
		<td align="right">
		  <table border="1" cellpadding="0" cellspacing="0" width="70">
			<tr>
			  <td width="95%" bgcolor="#FFFFFF">
				<table border="0" width="100%" cellspacing="0" cellpadding="0">
				  <tr>
					<td width="$memory_percent_used%" bgcolor="$bgcolorMP">&nbsp;</td>
					<td width="$memory_percent_remaining%"></td>
				  </tr>
				</table>
			  </td>
			</tr>
		  </table>
		</td>
	  </tr>
	</table>
  </td>
  <td width="50%" nowrap>&nbsp;<nobr>$free_memory $_R{'MB'} $_R{'FREE'} ($actual_memory $_R{'MB'} $_R{'TOTAL'})</nobr></td>
</tr>
<tr>
  <td width="20%"><b>$_R{'SWAP'}</b>&nbsp;</td>
  <td width="30%">
	<table border="0" cellpadding="0" cellspacing="0" width="100%">
	  <tr align="right">
		<td align="right" width="100%">&nbsp;$swap_percent_used%&nbsp;</td>
		<td align="right">
		  <table border="1" cellpadding="0" cellspacing="0" width="70">
			<tr>
			  <td width="95%" bgcolor="#FFFFFF">
				<table border="0" width="100%" cellspacing="0" cellpadding="0">
				  <tr>
					<td width="$swap_percent_used%" bgcolor="$bgcolorMS">&nbsp;</td>
					<td width="$swap_percent_remaining%"></td>
				  </tr>
				</table>
			  </td>
			</tr>
		  </table>
		</td>
	  </tr>
	</table>
  </td>
  <td width="50%" nowrap>&nbsp;<nobr>$swap_free $_R{'MB'} $_R{'FREE'} ($swap_memory $_R{'MB'} $_R{'TOTAL'})</nobr></td>
</tr>
</table>
<br>
<table border="0" cellpadding="2" cellspacing="0" width="375" bgcolor="#FFFFFF" bordercolor="#EEEEEE">
<tr>
  <td width="20%">
	<b>$_R{'DRIVE_SPACE_USAGE'}</b>
  </td>
  <td width="20%" align="right">
	<a href="server_disk.cgi">$_R{'MORE_ELLIPSIS'}</a>
  </td>
</tr>
</table>
$drive_info
<br>
<a href="server_hwinfo.cgi">$_R{'MORE_HARDWARE_INFORMATION'}</a>
HERE
}

sub formatFloat {
        my $origValue = shift;
        my $formattedValue = sprintf("%5.1f",$origValue);

        if($formattedValue =~ /\.0/) { $formattedValue = int($formattedValue); }

		# Remove any leading whitespace.
		$formattedValue =~ s/^\s*(.*)$/$1/;

        return $formattedValue;
}

