#!/usr/bin/perl

###############################################################################
#                                                                             #
# Filename: server_system_usage.cgi                                           #
#                                                                             #
# Description: CGI needed to display system usage information.                #
#                                                                             #
###############################################################################
###############################################################################
#                                                                             #
#  Copyright  2000 - 2001 Intel Corporation.                                 #
#  Intel Corporation All Rights Reserved.                                     #
#                                                                             #
#  The source code contained or described herein and all documents related to #
#  the source code ("Material") are owned by Intel Corporation or its         #
#  suppliers or licensors.  Title to the Material remains with Intel          #
#  Corporation or its suppliers and licensors.  The Material contains trade   #
#  secrets and proprietary and confidential information of Intel or its       #
#  suppliers and licensors.  The Material is protected by worldwide copyright #
#  and trade secret laws and treaty provisions.  No part of the Material may  #
#  be used, copied, reproduced, modified, published, uploaded, posted,        #
#  transmitted, distributed, or disclosed in any way without Intel's prior    #
#  express written permission.                                                #
#                                                                             #
#  No license under any patent, copyright, trade secret or other intellectual #
#  property right is granted to or conferred upon you by disclosure or        #
#  delivery of the Materials, either expressly, by implication, inducement,   #
#  estoppel or otherwise. Any license under such intellectual property rights #
#  must be express and approved by Intel in writing.                          #
#                                                                             #
###############################################################################

# Put the PERL library in the path.
use lib '/usr/ins/intel/gui/perllib';

use strict;
use LanguageResource;
use DisplayMain;
use Safe;
use CGI;

###############################################################################
# Main
###############################################################################

sub Main
{
	my $cgi					= shift;
	my $PreferredLanguage	= (split(/,/, $ENV{'HTTP_ACCEPT_LANGUAGE'}))[0];
	my $StringResource		= new LanguageResource($PreferredLanguage, '/usr/ins/intel/gui/locale', [ 'server_system_usage.lrc' ]);
	my %_R					= ( 'IDT:1100'						=> '',
								'ADMIN_WEB_SERVER_STATS'		=> '',
								'REPORT_GENERATED'				=> '',
								'TOTAL_HTML_PAGE_REQUESTS'		=> '',
								'TOTAL_VISITS'					=> '',
								'TOTAL_HITS'					=> '',
								'TOTAL_FILES'					=> '',
								'BYTES_TRANSFERRED'				=> '',
								'KB'							=> '',
								'NO_ADMIN_WEB_SERVER_ACTIVITY'	=> '',
								'MAIL_SERVER_STATS'				=> '',
								'FROM'							=> '',
								'MESSAGES_SENT'					=> '',
								'MESSAGES_RECD'					=> '',
								'MESSAGES_REJECTED'				=> '',
								'PRINTABLE_REPORTS'				=> '',
								'ADMIN_SERVER_USAGE_REPORTS'	=> '',
								'CPU_USAGE_REPORTS'				=> '',
								'NETWORK_USAGE_REPORTS'			=> '',
								'MEMORY_USAGE_REPORTS'			=> '' );
	my $WebalizerData		= '';
	my $WebStatistics		= '';
	my $WebStatisticsLink	= '';
	my $html;
  
	# Get the string resources.
	$StringResource->GetStrings(\%_R);

    # If a usage report is required...
	if ($cgi->param('generate_usage_report')) 
	{
		# If the webalizer directory does not exist...
		unless (-d '/usr/ins/intel/gui/htdocs/secure-data/webalizer')
		{
			# Create the webalizer directory.
			mkdir('/usr/ins/intel/gui/htdocs/secure-data/webalizer', 0777);
        }
 
		# If the Apache access log exists...
        if (-e '/usr/ins/intel/apache/logs/access_log' ) 
		{
			# Run webalizer on the log file...
			Safe::system('/usr/bin/webalizer /usr/ins/intel/apache/logs/access_log -o /usr/ins/intel/gui/htdocs/secure-data/webalizer >& /dev/null');
        }
	}

	# Parse the mail statistics.
	$_				 = Safe::backtick('/usr/sbin/mailstats');
	my $MailFrom	 = $1 if /Statistics from\s*(.+)/g; 
	my $MailSent	 = $1 if /===\n T\s*(\w+)/g; 
	my $MailReceived = $1 if /\s*\w+\s*(\w+)/g; 
	my $MailRejected = $1 if /\s*\w+\s*(\w+)/g; 
	
	# Open the webalizer statistics file...
	if (open(FILE, '< /usr/ins/intel/gui/htdocs/secure-data/webalizer/index.html'))
	{
		# Read in the web statistics.
		read(FILE, $WebalizerData, -s FILE);
		
		# Close the file.
		close(FILE);
	}

	# If there is any data...
	if ($WebalizerData)
	{
		# Parse the web statistics.
		$_							= $WebalizerData;
		my $WebReportTime			= $1 if(/Generated\s*(.+(?=<))/g);
		my $WebKBTransterred		= $1 if(/<TH BGCOLOR=\"\#C0C0C0\" ALIGN=right><FONT SIZE=\"-1\">(\w+)/g);
		my $WebTotalVisits 	     	= $1 if(/<TH BGCOLOR=\"\#C0C0C0\" ALIGN=right><FONT SIZE=\"-1\">(\w+)/g);
		my $WebTotalPageRequests	= $1 if(/<TH BGCOLOR=\"\#C0C0C0\" ALIGN=right><FONT SIZE=\"-1\">(\w+)/g);
		my $WebTotalFiles			= $1 if(/<TH BGCOLOR=\"\#C0C0C0\" ALIGN=right><FONT SIZE=\"-1\">(\w+)/g);
		my $WebTotalHits			= $1 if(/<TH BGCOLOR=\"\#C0C0C0\" ALIGN=right><FONT SIZE=\"-1\">(\w+)/g);
	
		$WebStatisticsLink = <<HERE;
		<tr>
			<td colspan=2>&nbsp;<b><a href=/secure-data/webalizer/index.html target=_blank>$_R{'ADMIN_SERVER_USAGE_REPORTS'}</a></b></td>
		</tr>
HERE

		$WebStatistics = <<HERE;
		<tr>
			<td width=281>&nbsp;<b>$_R{'REPORT_GENERATED'}</b></td>
			<td width=338>$WebReportTime</td>
		</tr>
		<tr>
			<td width=281>&nbsp;<b>$_R{'TOTAL_HTML_PAGE_REQUESTS'}</b></td>
			<td width=338>$WebTotalPageRequests</td>
		</tr>
		<tr>
			<td width=281>&nbsp;<b>$_R{'TOTAL_VISITS'}</b></td>
			<td width=338>$WebTotalVisits</td>
		</tr>
		<tr>
			<td width=281>&nbsp;<b>$_R{'TOTAL_HITS'}</b></td>
			<td width=338>$WebTotalHits</td>
		</tr>
		<tr>
			<td width=281>&nbsp;<b>$_R{'TOTAL_FILES'}</b></td>
			<td width=338>$WebTotalFiles</td>
		</tr>
		<tr>
			<td width=281>&nbsp;<b>$_R{'BYTES_TRANSFERRED'}</b></td>
			<td width=338>$WebKBTransterred $_R{'KB'}</td>
		</tr>                       
HERE
	}
	else 
	{
		$WebStatistics = <<HERE;
		<tr>
			<td colspan=2>$_R{'NO_ADMIN_WEB_SERVER_ACTIVITY'}</td>
		</tr>
HERE
	}

	$html = <<HERE;
<table border=0 cellpadding=0 cellspacing=1 width=375>
<tr>
	<td bgcolor=white height=25>
		<table border=0 width=100% cellspacing=0 cellpadding=2 bordercolor=#eeeeee bgcolor=#ffffff>
		<tr>
			<td colspan=2><b>$_R{'ADMIN_WEB_SERVER_STATS'}</b></td>
		</tr>
$WebStatistics	
		<tr>
			<td colspan=2><br><b>$_R{'MAIL_SERVER_STATS'}</b></td>
		</tr>
		<tr>
			<td width=281>&nbsp;<b>$_R{'FROM'}</b></td>
			<td width=338>$MailFrom</td>
		</tr>
		<tr>
			<td width=281>&nbsp;<b>$_R{'MESSAGES_SENT'}</b></td>
			<td width=338>$MailSent</td>
		</tr>
		<tr>
			<td width=281>&nbsp;<b>$_R{'MESSAGES_RECD'}</b></td>
			<td width=338>$MailReceived</td>
		</tr>
		<tr>
			<td width=281>&nbsp;<b>$_R{'MESSAGES_REJECTED'}</b></td>
			<td width=338>$MailRejected</td>
		</tr>
		<tr>
			<td colspan=2>&nbsp;</td>
		</tr>
		<tr>
			<td colspan=2><b>$_R{'PRINTABLE_REPORTS'}</b></td>
		</tr>
$WebStatisticsLink
		<tr>
			<td colspan=2>&nbsp;<b><a href=/secure-data/mrtg/cpu.html target=_blank>$_R{'CPU_USAGE_REPORTS'}</a></b></td>
		</tr>
		<tr>
			<td colspan=2>&nbsp;<b><a href=/secure-data/mrtg/network.html target=_blank>$_R{'NETWORK_USAGE_REPORTS'}</a></b></td>
		</tr>
		<tr>
			<td colspan=2>&nbsp;<b><a href=/secure-data/mrtg/memory.html target=_blank>$_R{'MEMORY_USAGE_REPORTS'}</a></b></td>
		</tr>
		</table>
	</td>
</tr>
</table>
HERE
	
	# Display the page.
    DisplayMain->printDocument({ 'q' => $cgi, 'html' => $html, 
								 'class' => 'server_system_usage', 'id' => 1100, 'title' => $_R{'IDT:1100'},
								 'root-link' => 'server_system_usage' });
}

###############################################################################
# The main body of the script.
###############################################################################

my $cgi = new CGI;

Main($cgi);
