#!/usr/bin/perl

###############################################################################
#                                                                             #
# Filename: server_temperature.cgi                                            #
#                                                                             #
# Description: CGI needed to display temperature information.                 #
#                                                                             #
###############################################################################
###############################################################################
#                                                                             #
#  Copyright  2000 - 2001 Intel Corporation.                                 #
#  Intel Corporation All Rights Reserved.                                     #
#                                                                             #
#  The source code contained or described herein and all documents related to #
#  the source code ("Material") are owned by Intel Corporation or its         #
#  suppliers or licensors.  Title to the Material remains with Intel          #
#  Corporation or its suppliers and licensors.  The Material contains trade   #
#  secrets and proprietary and confidential information of Intel or its       #
#  suppliers and licensors.  The Material is protected by worldwide copyright #
#  and trade secret laws and treaty provisions.  No part of the Material may  #
#  be used, copied, reproduced, modified, published, uploaded, posted,        #
#  transmitted, distributed, or disclosed in any way without Intel's prior    #
#  express written permission.                                                #
#                                                                             #
#  No license under any patent, copyright, trade secret or other intellectual #
#  property right is granted to or conferred upon you by disclosure or        #
#  delivery of the Materials, either expressly, by implication, inducement,   #
#  estoppel or otherwise. Any license under such intellectual property rights #
#  must be express and approved by Intel in writing.                          #
#                                                                             #
###############################################################################

# Put the PERL library in the path.
use lib '/usr/ins/intel/gui/perllib';

use strict;
use ConfigurationData;
use LanguageResource;
use DisplayMain;
use Safe;
use CGI;

###############################################################################
# GetCurrentTemperature
###############################################################################

sub GetCurrentTemperature
{
	my $CommandKey			= shift;
	my $CurrentTemperature	= 0;

	# Validate the input parameter(s)...
	if (defined($CommandKey))
	{
		# Get the requested data.
		$CurrentTemperature = (split(/ /, Safe::backtick("/usr/ins/intel/bin/asm/bakAsm $CommandKey")))[2];
	}

	return $CurrentTemperature;
}

###############################################################################
# GetIndicatorPosition
###############################################################################

sub GetIndicatorPosition
{
    my ($Lower, $Upper, $Current) = @_;
	my $Position = 50;

	if ((defined($Lower)) &&
		(defined($Upper)) &&
		(defined($Current)))
	{
		my $Base	= ($Lower - (($Upper - $Lower) / 2));
		my $End		= ($Upper + (($Upper - $Lower) / 2));

		# Calculate the position.
        $Position = ((($Current - $Base)/($End - $Base)) * 100); 
	}
	
	return $Position;
}

###############################################################################
# Main
###############################################################################

sub Main
{
	my ($cgi) 				= @_;
	my $PreferredLanguage	= (split(/,/, $ENV{'HTTP_ACCEPT_LANGUAGE'}))[0];
	my $StringResource		= new LanguageResource($PreferredLanguage, '/usr/ins/intel/gui/locale', [ 'server_temperature.lrc', 'server_hwinfo.lrc', 'server_alerts.lrc' ]);
	my $Configuration		= new ConfigurationData;
	my %_R					= ( 'IDT:400'				=> '',
								'CRITICAL_LOWER'		=> '',
								'CRITICAL_UPPER'		=> '',
								'WARNING_LOWER'			=> '',
								'WARNING_UPPER'			=> '',
								'CURRENT'				=> '',
								'C'						=> '',
								'WARNING'				=> '',
								'CRITICAL'				=> '',
								'IDT:500'				=> '',
								'CONFIGURE_ALERTS'		=> '' );
	my @TempData			= ( { 'c-label' => 'TEMPERATURE', 'label' => 'BASEBOARD_TEMPERATURE', 'command-key' => 'get_temp' } );
	my $Feedback;
	my $Result;
	my $html;
	my $T;

	# Get the string resources.
	$StringResource->GetStrings(\%_R);
	
	# Try to open the configuration file...
	if (0 == $Configuration->Attach('/etc/asm/asm.conf'))
	{
		# Run through each of the temperature(s)...
		foreach $T (@TempData)
		{
			# Get the label.
			$T->{'label'} = $StringResource->_S($T->{'label'});

			# Get the upper and lower thresholds.
			($T->{'warning-lower'},  $T->{'warning-upper'})		= split(/,/, $Configuration->GetValueKeyed("$T->{'c-label'}_THRESHOLDS"));
			($T->{'critical-lower'}, $T->{'critical-upper'})	= split(/,/, $Configuration->GetValueKeyed("$T->{'c-label'}_CRITICAL_THRESHOLDS"));

			# Get the current temperature.
			$T->{'current'} = GetCurrentTemperature($T->{'command-key'});

			# Get the indicator positions.
			$T->{'i-position-right'}	= 5 + GetIndicatorPosition($T->{'warning-lower'}, $T->{'warning-upper'}, $T->{'current'});
			$T->{'i-position-left'}		= 95 - $T->{'i-position-right'};

			# Get the current status...
			if (($T->{'critical-lower'} > $T->{'current'}) || 
				($T->{'critical-upper'} < $T->{'current'}))
			{
				# Critical.
				$T->{'status'} = " <img src=/images/critical.gif> $_R{'CRITICAL'}";
			}
			elsif (($T->{'warning-lower'} > $T->{'current'}) || 
				   ($T->{'warning-upper'} < $T->{'current'}))
			{
				# Warning.
				$T->{'status'} = " <img src=/images/warning.gif> $_R{'WARNING'}";
			}
			else 
			{
				# Normal.
				$T->{'status'} = '';
			}
		}
	}
	else
	{
		# Run through each of the temperature(s)...
		foreach $T (@TempData)
		{
			# Safe defaults.
			$T->{'label'}				= $StringResource->_S($T->{'label'});
			$T->{'warning-upper'}		= 0;
			$T->{'warning-lower'}		= 0;
			$T->{'critical-upper'}		= 0;
			$T->{'critical-lower'}		= 0;
			$T->{'current'} 			= 0;
			$T->{'i-position-left'}		= 50; 
			$T->{'i-position-right'}	= 50;
			$T->{'status'} 				= '';
		}
	}

	# The top of the page.
	$html = <<HERE;
<table border=0 width=375>
<tr>
	<td>
		<table border=0 cellPadding=0 cellSpacing=1 width=100%>
HERE

	# Run through each of the temperature(s)...
	foreach $T (@TempData)
	{
		# Normalize the data.
		$T->{'warning-upper'}	= sprintf("%.2f", $T->{'warning-upper'});
		$T->{'warning-lower'}	= sprintf("%.2f", $T->{'warning-lower'});
		$T->{'critical-upper'}	= sprintf("%.2f", $T->{'critical-upper'});
		$T->{'critical-lower'}	= sprintf("%0.2f", $T->{'critical-lower'});
		$T->{'current'} 		= sprintf("%.2f", $T->{'current'});
		
		# Add on the information for the current temperature.
		$html .= <<HERE;
		<tr>
			<td width=100% colspan=3><b>$T->{'label'} $T->{'status'}</b></td>
		</tr>
		<tr>
			<td width=382 colspan=3>&nbsp;</td>
		</tr>
		<tr>
			<td width=143 align=center rowspan=5>
				<table border=0 cellpadding=0 cellspacing=0 width=100>
				<tr>
					<td width=$T->{'i-position-right'} align=right>
						<img border=0 src=/images/meter.gif width=10 height=6>
					</td>
					<td width=$T->{'i-position-left'}>&nbsp;</td>
				</tr>
				</table>
				<table border=1 cellpadding=0 cellspacing=0 width=100>
				<tr>
					<td width=95% bgcolor=#ffffff>
						<table border=0 width=100% cellspacing=0 cellpadding=0>
						<tr>
							<td width=7% bgcolor=#ff0000>&nbsp;</td>
							<td width=18% bgcolor=#ffff00>&nbsp;</td>
							<td width=50% bgcolor=#00ff00>&nbsp;</td>
							<td width=18% bgcolor=#ffff00>&nbsp;</td>
							<td width=7% bgcolor=#ff0000>&nbsp;</td>
						</tr>
						</table>
					</td>
				</tr>
				</table>
			</td>
			<td width=96>$_R{'CRITICAL_LOWER'}&nbsp;</td>
			<td width=139>$T->{'critical-lower'} $_R{'C'}</td>
		</tr>
		<tr>
			<td width=96>$_R{'WARNING_LOWER'}&nbsp;</td>
			<td width=139>$T->{'warning-lower'} $_R{'C'}</td>
		</tr>
		<tr>
			<td width=96>$_R{'CURRENT'}&nbsp;</td>
			<td width=139><b>$T->{'current'} $_R{'C'}</b></td>
		</tr>
		<tr>
			<td width=96>$_R{'WARNING_UPPER'}&nbsp;</td>
			<td width=139>$T->{'warning-upper'} $_R{'C'}</td>
		</tr>
		<tr>
			<td width=96>$_R{'CRITICAL_UPPER'}&nbsp;</td>
			<td width=139>$T->{'critical-upper'} $_R{'C'}</td>
		</tr>
HERE
	}

	# Add on the tail of the page.
	$html .= <<HERE;
		<tr>
			<td colSpan=3 width=100%>&nbsp;</td>
		</tr>
		</table>
		<br>
		<a href=server_alerts.cgi>$_R{'CONFIGURE_ALERTS'}</a>
	</td>
</tr>
</table>
HERE

	# Display the page.
    DisplayMain->printDocument({ 'q' => $cgi, 'html' => $html, 
								 'cookiecrumbs' => [ { 'name' => $_R{'IDT:500'}, 'location' => 'server_hwinfo.cgi' } ],
								 'root-link' => 'server_status', 'class' => 'server_temperature', 'id' => 400, 'title' => $_R{'IDT:400'} });
}

###############################################################################
# The main body of the script.
###############################################################################

my $cgi = new CGI;

Main($cgi);
