#!/usr/bin/perl

###############################################################################
#                                                                             #
# Filename: server_updates_more.cgi                                           #
#                                                                             #
# Description: Provides status information for the installed appliance        #
#              software stack.                                                #
#                                                                             #
###############################################################################
###############################################################################
#                                                                             #
#  Copyright  2000 - 2001 Intel Corporation.                                 #
#  Intel Corporation All Rights Reserved.                                     #
#                                                                             #
#  The source code contained or described herein and all documents related to #
#  the source code ("Material") are owned by Intel Corporation or its         #
#  suppliers or licensors.  Title to the Material remains with Intel          #
#  Corporation or its suppliers and licensors.  The Material contains trade   #
#  secrets and proprietary and confidential information of Intel or its       #
#  suppliers and licensors.  The Material is protected by worldwide copyright #
#  and trade secret laws and treaty provisions.  No part of the Material may  #
#  be used, copied, reproduced, modified, published, uploaded, posted,        #
#  transmitted, distributed, or disclosed in any way without Intel's prior    #
#  express written permission.                                                #
#                                                                             #
#  No license under any patent, copyright, trade secret or other intellectual #
#  property right is granted to or conferred upon you by disclosure or        #
#  delivery of the Materials, either expressly, by implication, inducement,   #
#  estoppel or otherwise. Any license under such intellectual property rights #
#  must be express and approved by Intel in writing.                          #
#                                                                             #
###############################################################################

# Put the PERL library in the path.
use lib '/usr/ins/intel/gui/perllib';

use strict;
use LanguageResource;
use DisplayMain;
use Safe;
use CGI;

###############################################################################
# Main
###############################################################################

sub Main
{
	my ($cgi)				= @_;
	my $PreferredLanguage	= (split(/,/, $ENV{'HTTP_ACCEPT_LANGUAGE'}))[0];
	my $StringResource		= new LanguageResource($PreferredLanguage, '/usr/ins/intel/gui/locale', [ 'server_updates_more.lrc', 'server_updates.lrc' ]);
	my %_R					= ( 'IDT:2300'	   	=> '',
								'IDT:2400'	   	=> '',
								'INSTALLED_RPM'	=> '',
								'VERSION'	   	=> '',
								'INSTALL_DATE' 	=> '' );
	
	my @RPMList		= Safe::backtick("/bin/rpm -qa --queryformat '%{NAME}#:%{VERSION}#:%{RELEASE}#:%10{INSTALLTIME:date}\n' | sort ");		
	my $RPMHead = '<tr><td nowrap><a href=server_rpminfo.cgi?rpmname=';
	my $RPMQueryInfo = "";
	my $Line;
	my $EscapedText;
	my @RPMArray;
	
	foreach $Line (@RPMList)
	{
		@RPMArray=split(/#:/,$Line);		
		$EscapedText = $cgi->escape(join('-',@RPMArray[0],@RPMArray[1],@RPMArray[2]));		
		$RPMQueryInfo = $RPMQueryInfo . $RPMHead . $EscapedText . "> @RPMArray[0]</a></td> <td nowrap>@RPMArray[1]</td> <td nowrap>@RPMArray[3]</tr>\n";
	}
		
    my $html;
	
	# Get the string resources.
	$StringResource->GetStrings(\%_R);

	$html = <<HERE;
<table border=0 cellspacing=0 cellpadding=0 width=375 bgcolor=white>
<tr>
	<td>
		<table border=1 cellspacing=0 cellpadding=2 bgcolor=#ffffff bordercolor=#eeeeee width=100%>
		<tr>
			<td bgcolor=#808080><b><font color=#ffffff>$_R{'INSTALLED_RPM'}</font></b></td>
			<td bgcolor=#808080><b><font color=#ffffff>$_R{'VERSION'}</font></b></td>
			<td bgcolor=#808080><b><font color=#ffffff>$_R{'INSTALL_DATE'}</font></b></td>
		</tr>		
$RPMQueryInfo
		</table>
	</td>
</tr>
</table>
HERE

	# Display the page.
    DisplayMain->printDocument({ 'q' => $cgi, 'html' => $html,
								 'class' => 'server_updates_more', 'id' => 2300, 'title' => $_R{'IDT:2300'}, 'root-link' => 'server_updates',
								 'cookiecrumbs' => [ { 'name' => $_R{'IDT:2400'}, 'location' => 'server_updates.cgi' } ] });
}

###############################################################################
# The main body of the script.
###############################################################################

my $cgi = new CGI;

Main($cgi);
