#!/usr/bin/perl -w

###############################################################################
#                                                                             #
# Filename: server_users.cgi                                                  #
#                                                                             #
# Description: CGI needed to display and modify user configurations.          #
#                                                                             #
###############################################################################
###############################################################################
#                                                                             #
#  Copyright  2000 - 2001 Intel Corporation.                                 #
#  Intel Corporation All Rights Reserved.                                     #
#                                                                             #
#  The source code contained or described herein and all documents related to #
#  the source code ("Material") are owned by Intel Corporation or its         #
#  suppliers or licensors.  Title to the Material remains with Intel          #
#  Corporation or its suppliers and licensors.  The Material contains trade   #
#  secrets and proprietary and confidential information of Intel or its       #
#  suppliers and licensors.  The Material is protected by worldwide copyright #
#  and trade secret laws and treaty provisions.  No part of the Material may  #
#  be used, copied, reproduced, modified, published, uploaded, posted,        #
#  transmitted, distributed, or disclosed in any way without Intel's prior    #
#  express written permission.                                                #
#                                                                             #
#  No license under any patent, copyright, trade secret or other intellectual #
#  property right is granted to or conferred upon you by disclosure or        #
#  delivery of the Materials, either expressly, by implication, inducement,   #
#  estoppel or otherwise. Any license under such intellectual property rights #
#  must be express and approved by Intel in writing.                          #
#                                                                             #
###############################################################################

# Put the PERL library in the path.
use lib '/usr/ins/intel/gui/perllib';

use strict;
use LanguageResource;
use Virtusertable;
use VirtualFtp;
use DisplayMain;
use DomainMail;
use Quotaall;
use Users;
use CGI;
use ProductInformation;

## SName:  is4User
# Note:   determines whether the username is a reserved IS4 user.
sub is4User
{
    my ($user) = @_;

    if ($user eq 'is_appl' || $user eq 'is_dbusr' || $user eq 'is_web')
    {
		return 1;
    }
    return 0;
}

###############################################################################
# ValidateFormData
###############################################################################

sub ValidateFormData
{
	my ($FormDataRef)	= @_;
	my $Return			= 0;
	my $Key;

	# Run through each of the keys...
	foreach $Key (keys(%{ $FormDataRef }))
	{
		# If the data is defined...
		if (defined($FormDataRef->{$Key}))
		{
			# Trim the whitespace.
			$FormDataRef->{$Key} =~ s/^\s*(.*?)\s*$/$1/;
		}
	}
	
	# Validate the data...
	if (($FormDataRef->{'name_full'} ne '') &&
		($FormDataRef->{'name_full'} !~ /^\S+(\s*\S+)*$/))
	{
		# Set the return code.
		$Return = 280000;
	}
	elsif (('root' eq $FormDataRef->{'name_login'}) ||
		   ($FormDataRef->{'name_login'} !~ /^(\w|-|_)*$/))
	{
		# Set the return code.
		$Return = 280001;
	}
	elsif ($FormDataRef->{'password'} !~ /^\S+$/)
	{
		# Set the return code.
		$Return = 280002;
	}
	elsif ($FormDataRef->{'password'} ne $FormDataRef->{'password_confirm'})
	{
		# Set the return code.
		$Return = 280003;
	}
	elsif (($FormDataRef->{'disk_quota'} !~ /^\+*([0-9]*)\.?([0-9]*)$/) || 
	       (!($1 || ('0' eq $1) || $2 || ('0' eq $2))))
	{
		# Set the return code.
		$Return = 280004;
	}
	else
	{
		my @MailAliasListList;
		my $MailAliasList		= $FormDataRef->{'e_mail_alias'};
		my $BadAliasCount;

		# Clean up the list.
		$MailAliasList =~ s/\n/ /sg;
		$MailAliasList =~ s/\,/ /g;
		$MailAliasList =~ s/\;/ /g;
		$MailAliasList =~ s/\cM//g;

		# Get the individual addresses.
		@MailAliasListList = split(/\s+/, $MailAliasList);

		# Run though the mail list...
		for (@MailAliasListList)
		{
			# Look for bad addresses...
			if ($_ =~ /^(\w|-|_)+$/)
			{
				next;
			}
			else
			{
				# Increment the bad count.
				++$BadAliasCount;
			}
		}

		# If there are any bad addresses...
		if ($BadAliasCount)
		{
			# Set the return code.
			$Return = (1 < $BadAliasCount) ? 280006 : 280005;
		}
	}

	return $Return;
}

###############################################################################
# GetHomeUsage
###############################################################################

sub GetHomeUsage
{
	my $DFData = Safe::backtick('/bin/df -k /home');
    
	# Get the data.
	if ($DFData =~ /\/dev\/\S+\s*(\S+)\s*(\S+)\s*(\S+)\s*(\S+)\s*(\S+)/g)
	{
		return $1;
	}

	return 1;
}

###############################################################################
# AddUser
###############################################################################

sub AddUser
{
	my ($FormDataRef)	= @_;
	my $Return			= 0;

	my @DomainList = VirtualDomain::list_vws();
	my $Domain;
	my $IPAddress;
	my $DomainGroup;

	# Run through the user list...
	foreach $Domain (@DomainList) 
	{
		# If the user matches...
		if ($FormDataRef->{'domain_name'} eq $Domain->{'vdn'})
		{
			# Cache the group and IP address.
			$DomainGroup	= $Domain->{'dom_grp'};
			$IPAddress		= $Domain->{'vdip'};
			last;
		}
	}

	# If the assigned group and IP address exist...
	if (($DomainGroup) &&
		($IPAddress))
	{
		# Check to see if the user already exists...
		unless (getpwnam($FormDataRef->{'name_login'}))
		{
			my @Quota			= Quotaall::repquota({ 'domain' => $FormDataRef->{'domain_name'} });
			my $QuotaLimit		= $Quota[2] / 1024; # Sometimes the quota stuff returns 0.
			my $QuotaTotal		= Quotaall::quotaCheck($DomainGroup) / 1024;
			my $SpaceAvailable;
			
			# If the quota limit for the domain is zero.
			$QuotaLimit = GetHomeUsage() if(0 == $QuotaLimit);

			# Get the space available.
			$SpaceAvailable	= $QuotaLimit - $QuotaTotal;
			
			# Check for space and size limits on the quota...
			if ($SpaceAvailable <= $FormDataRef->{'disk_quota'}) 
			{
				# Set the return code.
				$Return = 280009;
			}
			elsif (1000 < $FormDataRef->{'disk_quota'})
			{
				# Set the return code.
				$Return = 280010;
			}
			elsif (0 >= $FormDataRef->{'disk_quota'})
			{
				# Set the return code.
				$Return = 280016;
			}
			else
			{
				my @AliasList	= split(/\s+/, $FormDataRef->{'e_mail_alias'});
				my @UserList	= Users::listUsrs();
				my $Alias;
				my $User;
				my $UserExists	= 0;

				# Run through the alias list...
				foreach $Alias (@AliasList)
				{
					# Run through the user list...
					foreach $User (@UserList)
					{
						# If there is a match...
						if ($User eq $Alias)
						{
							$UserExists = 1;
							last;
						}
					}

					# Bail out...
					last if($UserExists);
				}

				# If there is not a duplicate aliased user...
				unless ($UserExists)
				{
					my $Group		= getgrnam($DomainGroup);
					my $GroupAux	= '';
					my $Shell		= ($FormDataRef->{'telnet_access'}) ? '/bin/bash' : '/bin/false';
					my $HomePath	= "/home/domain/$DomainGroup/users/$FormDataRef->{'name_login'}";

					SWITCH:
					{
						# Determine group relationships...
						if ('server' eq $FormDataRef->{'rights'}) { $GroupAux = $Group; $Group = 10000; last SWITCH; }
						if ('domain' eq $FormDataRef->{'rights'}) { $GroupAux = $Group; $Group = 10001; last SWITCH; }
					}

					# If there is an e-mail alias list then try to apply it...
					if (($FormDataRef->{'e_mail_alias'}) &&
						(0 != Virtusertable::update_alias_list($FormDataRef->{'name_login'}, $FormDataRef->{'domain_name'}, @AliasList)))
					{
						# Set the return code.
						$Return = 280012;
					}
					elsif ('User' ne Users::usrAdd($FormDataRef->{'name_full'}, $HomePath, $Group, '/usr/ins/intel/gui/skel/user', $Shell, $GroupAux, 
												   $FormDataRef->{'name_login'}, $FormDataRef->{'password'}))
					{
						# Try to clean up the mess.
						Virtusertable::delete_by_username($FormDataRef->{'name_login'}, $FormDataRef->{'domain_name'});

						# Set the return code.
						$Return = 280013;
					}
					else
					{
						# Get the user ID.
						my $UserID = getpwnam($FormDataRef->{'name_login'});

						# Set up the quota.
						Quotaall::setquota($UserID, $FormDataRef->{'disk_quota'});
						
						# Add user in "ftpuser" if FTP disabled for the domain ...
						VirtualFtp::addDomainUser($FormDataRef->{'name_login'}, $FormDataRef->{'domain_name'}, $DomainGroup, $IPAddress);

						# Denied FTP access based on user...
						unless ($FormDataRef->{'ftp_access'})
						{
							# Add the user to the exclusion list...
							DomainMail::noftp($FormDataRef->{'name_login'});
						}

						# Denied mail access based on user...
						unless ($FormDataRef->{'mail_access'})
						{
							# Add the user to the exclusion list...
							DomainMail::nomail($FormDataRef->{'name_login'});
						}
					}
				}
				else
				{
					# Set the return code.
					$Return = 280011;
				}
			}
		}
		else
		{
			# Set the return code.
			$Return = 280008;
		}
	}
	else
	{
		# Set the return code.
		$Return = 280007;
	}

	return $Return;
}

###############################################################################
# DeleteUsers
###############################################################################

sub DeleteUsers
{
	my ($CurrentUser, $UserNameListRef, $UserListRef)	= @_;
	my @TargetList;
	my $UserName;
	my $User;
	my $Return;

	# Run through all of the selected users...
	foreach $UserName (@{ $UserNameListRef })
	{ 
		# Skip the controller checkbox.
		next if($UserName eq 'root');

		# Run through the user list...
		foreach $User (@{ $UserListRef })
		{
			# If the name matches...
			if ($UserName eq $User->{'user'})
			{
				# If the user is not the current user...
				if ($CurrentUser ne $UserName)
				{
					# Make sure the user is not a primary user administrator...
					if (1 == Users::primaryDA($UserName))
					{
						$Return = 280014;
						last;
					}
					else
					{
						# Add the user to the target list.
						push(@TargetList, $User);
					}
				}
				else
				{
					$Return = 280015;
					last;
				}
			}
		}

		last if($Return);
	}

	# If things are going smooth...
	unless ($Return)
	{
		my @Users;
		my $User;

		# Run through the targeted user list...
		foreach $User (@TargetList)
		{
			# Delete the user.
			Users::usrDel($User->{'user'});
			$Return = 0;
		}
	}

	return $Return;
}

###############################################################################
# RenderDefault
###############################################################################

sub RenderDefault
{
	my ($cgi, $DisplayDataRef) = @_;
	my $Return;

	# Check for a button press...
	if (defined($cgi->param('user_add')))
	{
		# Redirect to the user add.
		$Return = 'user_add';
	}
	else
	{
		my $PreferredLanguage	= (split(/,/, $ENV{'HTTP_ACCEPT_LANGUAGE'}))[0];
		my $ProductInfo			= new ProductInformation();
		my $StringResource		= new LanguageResource($PreferredLanguage, '/usr/ins/intel/gui/locale', [ 'server_users.lrc', 'common.lrc' ]);
		my %_R					= ( 'IDT:2700'				=> '',
									'FIND'					=> '',
									'ADD'					=> '',
									'DELETE'				=> '',
									'CONFIRM_USER_DELETE'	=> '',
									'NO_USERS_SELECTED'		=> '',
									'SHOW_ALL'				=> '',
									'NAME_LOGIN'			=> '',
									'NAME_FULL'				=> '',
									'DOMAIN'				=> '',
									'RIGHTS'				=> '' );
		my @UserList			= Users::listUsrs();
		my $UserHTML			= '';
		my $SortControl			= $cgi->param('sort_control');
		my $Search				= $cgi->param('search');
		my $Feedback;
		my $Result;
		my $html;
	
		# Get the string resources.
		$StringResource->GetStrings(\%_R);
		
		# Safe default(s).
		$SortControl	= 'none' unless(defined($SortControl));
		$Search			= '' unless(defined($Search));
	
		# If applying a change...
		if (defined($cgi->param('find')))
		{
			# Trim the whitespace.
			$Search =~ s/^\s*(.*?)\s*$/$1/;

			# Run through the list...
			for (my $i = 0; $i < scalar(@UserList); ++$i)
			{
				# Try to match with the search...
				unless ($UserList[$i]->{'user'} =~ /$Search/)
				{
					# Delete the item.
					splice(@UserList, $i, 1);

					# Decrement the index.
					--$i;
				}
			}
		}
		elsif (defined($cgi->param('user_delete')))
		{
			# Try to delete the specified users.
			$Result = DeleteUsers($cgi->remote_user(), $cgi->param_fetch('check'), \@UserList);
			
			# Get the new list.
			@UserList = Users::listUsrs();

            # If there was an error...
			if ($Result)
			{
				# Get the error string.
				$Feedback = $StringResource->_S($Result);
			}
		}
		
		SWITCH:
		{
			# Apply any sorting...
			if ('name_full'  eq $SortControl) { @UserList = sort { $a->{'real'} cmp $b->{'real'} } @UserList;				last SWITCH; }
			if ('name_login' eq $SortControl) { @UserList = sort { $a->{'user'} cmp $b->{'user'} } @UserList;				last SWITCH; }
			if ('domain'     eq $SortControl) { @UserList = sort { $a->{'domain_name'} cmp $b->{'domain_name'} } @UserList;	last SWITCH; }
			if ('rights'     eq $SortControl) { @UserList = sort { $a->{'right'} cmp $b->{'right'} } @UserList;				last SWITCH; }
		}
	
		# If there are any users...
		if (scalar(@UserList))
		{
			my $User;
			my %Rights	= ( 'SERVER_ADMINISTRATOR'	=> '',
							'DOMAIN_ADMINISTRATOR'	=> '',
							'REGULAR_USER'			=> '' );
			my $Type;
	
			# Get the string resources.
			$StringResource->GetStrings(\%Rights);
			
			# Run through the user list...
			foreach $User (@UserList)
			{
				# Look for our users...
				if ((499 < $User->{'uid'}) &&
					(!((2 == $ProductInfo->getClassification()) && (is4User($User->{'user'})))))
				{
					SWITCH:
					{
						# Get the localized user type...
						if (10000 == $User->{'type'}) { $Type = $Rights{'SERVER_ADMINISTRATOR'}; last SWITCH; }
						if (10001 == $User->{'type'}) { $Type = $Rights{'DOMAIN_ADMINISTRATOR'}; last SWITCH; }
						$Type = $Rights{'REGULAR_USER'}; 
					}

					$UserHTML .= <<HERE;		
		<tr align=left valign=center>
			<td><input type=checkbox name=check value='$User->{'user'}'></td>
			<td><a href=/login/index.cgi?user=$User->{'user'}&domain=$User->{'domain_name'} target='$User->{'user'}'>$User->{'user'}</a></td>
			<td>$User->{'real'}</td>
			<td><a href=/login/index.cgi?domain=$User->{'domain_name'} target='$User->{'domain_name'}'>$User->{'domain_name'}</a></td>
			<td>$Type</td>
		</tr>
HERE
				}
			}
		}
		else
		{
			$UserHTML .= '<tr align=left valign=center><td>&nbsp;</td><td colspan=4><b>' . $StringResource->_S('NO_USERS_AVAILABLE') . '</b></td></tr>'
		}
	
		$html = <<HERE;
<script src=/utilities.js language=javascript></script>
<script language=javascript>
function confirmDeleteUser()
{
	var bReturn = false;
	
	for(i = 1; i < document.server_users.check.length; i++)
	{
		if (document.server_users.check[i].checked)
		{
			return confirm('$_R{'CONFIRM_USER_DELETE'}');
		}
	}
	
	if (i == document.server_users.check.length)
	{
		alert('$_R{'NO_USERS_SELECTED'}');
	}
	
	return bReturn;
}
</script>
<form method=post name=server_users action=server_users.cgi enctype="application/x-www-form-urlencoded">
	<input type=hidden name=type value=default>
	<input type=hidden name=sort_control value=$SortControl>
	<table border=0 bgcolor=#eeeeee width=375>
	<tr>
		<td bgcolor=#eeeeee>
			<table width=100%>
			<tr>
				<td>
					<table border=0 width=100% bgcolor=#eeeeee>
					<tr align=left valign=center><b>$_R{'FIND'}</b>
						<td>
							<input type=text name=search value='$Search' size=20 maxlength=32>
							<input type=submit name=find value='$_R{'FIND'}' class=clsButton>
						</td>
						<td align=right>
							<input type=submit name=show_all value='$_R{'SHOW_ALL'}' class=clsButton>
						</td>
					</tr>
					</table>
				</td>
			<tr>
			</table>
		</td>
	</tr>
	<tr>
		<td>
			<table border=0>
			<tr>
				<td align=left nowrap colspan=2>
					<input type=submit name=user_add value='$_R{'ADD'}' class=clsButton>
					<input type=submit name=user_delete value='$_R{'DELETE'}' class=clsButton onclick="return confirmDeleteUser();">
				</td>
			</tr>
			</table>
		</td>
	</tr>
	<tr>
		<td>&nbsp;</td>
	</tr>
	<tr>
		<td>
			<table bordercolor=#eeeeee border=1 cellpadding=2 width=375 bgcolor=#ffffff cellspacing=0>
			<tr align=left valign=center>
				<td bgcolor=#808080 width=1%><input type=checkbox name=check onclick=checkAll(document.server_users.check) value=root></td>
				<td bgcolor=#808080><a href=server_users.cgi?sort_control=name_login class=clsGroup>$_R{'NAME_LOGIN'}</td>
				<td bgcolor=#808080><a href=server_users.cgi?sort_control=name_full class=clsGroup>$_R{'NAME_FULL'}</td>
				<td bgcolor=#808080><a href=server_users.cgi?sort_control=domain class=clsGroup>$_R{'DOMAIN'}</td>
				<td bgcolor=#808080><a href=server_users.cgi?sort_control=rights class=clsGroup>$_R{'RIGHTS'}</td>
			</tr>
$UserHTML
			</table>
		</td>
	</tr> 
	</table>
</form>
HERE

		# Overriding result...
		$Result = $DisplayDataRef->{'result'} unless(defined($Result));

		# Fill out the display information.
		$DisplayDataRef->{'q'}			= $cgi;
		$DisplayDataRef->{'html'}		= $html;
		$DisplayDataRef->{'class'}		= 'server_users';
		$DisplayDataRef->{'id'}			= 2700;
		$DisplayDataRef->{'title'}		= $_R{'IDT:2700'};
		$DisplayDataRef->{'root-link'}	= 'server_users';
		$DisplayDataRef->{'result'}		= $Result;
		$DisplayDataRef->{'feedback'}	= $Feedback;

		$Return = 0;
	}
	
	return $Return;
}

###############################################################################
# RenderUserAdd
###############################################################################

sub RenderUserAdd
{
	my ($cgi, $DisplayDataRef, $Redirect) = @_;
	my $PreferredLanguage	= (split(/,/, $ENV{'HTTP_ACCEPT_LANGUAGE'}))[0];
	my $StringResource		= new LanguageResource($PreferredLanguage, '/usr/ins/intel/gui/locale', [ 'server_users.lrc', 'common.lrc' ]);
	my %FormData			= (	'sort_control'					=> '',
								'domain_name'					=> '',
								'name_full'	   					=> '',
								'name_login'   					=> '',
								'password'						=> '',
								'password_confirm'				=> '',
								'e_mail_alias'					=> '',
								'telnet_access'					=> 'checked',
								'ftp_access'					=> 'checked',
								'mail_access'					=> 'checked',
								'rights'						=> 'none',
								'disk_quota'					=> 20 );
	my $Key;
	my $Feedback;
	my $Result;
	my $html;
	my $Return;

	# If adding a user...
	if (defined($cgi->param('add')))
	{
		# Get the form data from the form...
		foreach $Key (keys(%FormData))
		{
			# Get the data...
			$FormData{$Key} = (defined($cgi->param($Key))) ? $cgi->param($Key) : '';
		}

		# Validate the form data.
		if (0 == ($Result = ValidateFormData(\%FormData)))
		{
			# Clean up the list.
			$FormData{'e_mail_alias'} =~ s/\n/ /sg;
			$FormData{'e_mail_alias'} =~ s/\,/ /g;
			$FormData{'e_mail_alias'} =~ s/\;/ /g;
			$FormData{'e_mail_alias'} =~ s/\cM//g;
			
			# Try to add the user...
			if (0 == ($Result = AddUser(\%FormData)))
			{
				# Set the result.
				$DisplayDataRef->{'result'}	= $Result;
				
				# Move on to the main page.
				$Return = 'default';
			}
		}

		# If there was an error...
		if ($Result)
		{
			# Get the error string.
			$Feedback = $StringResource->_S($Result);
		}
	}

	# Re-align the checkboxes...
	$FormData{'telnet_access'}	= ($FormData{'telnet_access'}) ? 'checked' : '';
	$FormData{'ftp_access'}		= ($FormData{'ftp_access'}) ? 'checked' : '';
	$FormData{'mail_access'}	= ($FormData{'mail_access'}) ? 'checked' : '';

	# Escape the users full name so it displays properly (We're escaping the ' symbol)
	$FormData{'name_full'} 		=~ s/'/&#039;/g;
	
	# Set up the radio state...
	$FormData{'rights_server'}	= ('server' eq $FormData{'rights'}) ? 'checked' : '';
	$FormData{'rights_domain'}	= ('domain' eq $FormData{'rights'}) ? 'checked' : '';
	$FormData{'rights_none'}	= ('none'   eq $FormData{'rights'}) ? 'checked' : '';

	# If a destination is not set...
	unless($Return)
	{
		my %_R					= ( 'DOMAIN'				=> '',
									'NAME_FULL'				=> '',
									'NAME_LOGIN'			=> '',
									'PASSWORD'				=> '',
									'PASSWORD_CONFIRM'		=> '',
									'E_MAIL_ALIASES'		=> '',
									'TELNET_ACCESS'			=> '',
									'FTP_ACCESS'			=> '',
									'MAIL_ACCESS'			=> '',
									'ADMINISTRATOR_RIGHTS'	=> '',
									'SERVER_ADMINISTRATOR'	=> '',
									'DOMAIN_ADMINISTRATOR'	=> '',
									'NONE'					=> '',
									'USER_DISK_QUOTA'		=> '',
									'MB'					=> '',
									'ADD'					=> '' );
		my @DomainList			= VirtualDomain::list_vws();
		my $DomainOptions		= '';
		my $Domain;

		# Run through the domain list...
        foreach $Domain (@DomainList) 
		{
			# Check for the selected domain...
			unless ($Domain->{'vdn'} eq $FormData{'domain_name'})
			{
				# Add the domain to the option list.
				$DomainOptions .= "\t\t\t\t\t\t<option value='$Domain->{'vdn'}'>$Domain->{'vdn'}";
			}
			else
			{
				# Add the domain to the option list as selected.
				$DomainOptions .= "\t\t\t\t\t\t<option selected value='$Domain->{'vdn'}'>$Domain->{'vdn'}";
			}
		}

		# Get the string resources.
		$StringResource->GetStrings(\%_R);
		
		$html = <<HERE;
<form method=post action=server_users.cgi enctype="application/x-www-form-urlencoded">
	<input type=hidden name=type value=user_add>
	<input type=hidden name=sort_control value=$FormData{'sort_control'}>
	<table border=0 width=375 bgcolor=#eeeeee>
	<tr>
		<td>
			&nbsp;
			<table border=0 width=100%>
			<tr>
				<td><b>$_R{'DOMAIN'}</B></TD>
				<td colspan=2>
					<select name=domain_name>
$DomainOptions
					</select>
				</td>
			</tr>
			<tr>
				<td><b>$_R{'NAME_FULL'}</b></td>
				<td><input type=text size=20 maxlength=32 name=name_full value='$FormData{'name_full'}'></td>
			</tr>
			<tr>
				<td><b>$_R{'NAME_LOGIN'}</b></td>
				<td><input type=text size=20 maxlength=32 name=name_login value='$FormData{'name_login'}'></td>
			</tr>
			<tr>
				<td><b>$_R{'PASSWORD'}</b></td>
				<td><input type=password size=20 maxlength=32 name=password value='$FormData{'password'}'></td>
			</tr>
			<tr>
				<td><b>$_R{'PASSWORD_CONFIRM'}</b></td>
				<td><input type=password size=20 maxlength=32 name=password_confirm value='$FormData{'password_confirm'}'></td>
			</tr>
			<tr>
				<td colspan=2>&nbsp</td>
			</tr>
			<tr>
				<td valign=top><b>$_R{'E_MAIL_ALIASES'}</b></td>
				<td valign=top><textarea name=e_mail_alias rows=2 cols=20>$FormData{'e_mail_alias'}</textarea></td>
			</tr>			
			<tr>
				<td nowrap colspan=2><input type=checkbox value=Yes name=telnet_access $FormData{'telnet_access'}>&nbsp;<b>$_R{'TELNET_ACCESS'}</b></td>
			</tr>
			<tr>
				<td nowrap colspan=2><input type=checkbox value=Yes name=ftp_access $FormData{'ftp_access'}>&nbsp;<b>$_R{'FTP_ACCESS'}</b></td>
			</tr>
			<tr>
				<td nowrap colspan=2><input type=checkbox value=Yes name=mail_access $FormData{'mail_access'}>&nbsp;<b>$_R{'MAIL_ACCESS'}</b></td>
			</tr>
			<tr align=left valign=center>
				<td nowrap><b>$_R{'ADMINISTRATOR_RIGHTS'}</b></td>
			</tr>
			<tr align=left valign=center>
				<td colspan=2>
					<table cellspacing=0 border=0 cellpadding=0 width=100%>
					<tr align=left valign=center>
						<td width=25>&nbsp;<input type=radio name=rights value=server $FormData{'rights_server'}></td>
						<td nowrap width=315><b>$_R{'SERVER_ADMINISTRATOR'}</b></td>
					</tr>
					<tr align=left valign=center>
						<td width=25>&nbsp;<input type=radio name=rights value=domain $FormData{'rights_domain'}></td>
						<td nowrap width=315><b>$_R{'DOMAIN_ADMINISTRATOR'}</b></td>
					</tr>
					<tr align=left valign=center>
						<td width=25>&nbsp;<input type=radio name=rights value=none $FormData{'rights_none'}></td>
						<td nowrap width=315><b>$_R{'NONE'}</b></td>
					</tr>
					</table>
				</td>
			</tr>			
			<tr>
				<td nowrap><b>$_R{'USER_DISK_QUOTA'}</b></td>
				<td><input type=text size=4 maxlength=10 name=disk_quota value='$FormData{'disk_quota'}'>$_R{'MB'}</td>
			</tr>
			<tr>
				<td colspan=2 align=right><input type=submit name=add value='$_R{'ADD'}' class=clsButton></td>
			</tr>
			</table>
		</td>
	</tr>
	</table>
</form>
HERE
		
		# Fill out the display information.
		$DisplayDataRef->{'q'}				= $cgi;
		$DisplayDataRef->{'html'}			= $html;
		$DisplayDataRef->{'class'}			= 'server_users';
		$DisplayDataRef->{'id'}				= 2800;
		$DisplayDataRef->{'title'}			= $StringResource->_S('IDT:2800');
		$DisplayDataRef->{'root-link'}		= 'server_users';
		$DisplayDataRef->{'helpIndicator'}	= 'user_add';
		$DisplayDataRef->{'result'}			= $Result;
		$DisplayDataRef->{'feedback'}		= $Feedback;
		$DisplayDataRef->{'cookiecrumbs'}	= [ { 'name' => $StringResource->_S('IDT:2700'), 'location' => 'server_users.cgi' } ];

		$Return = 0;
	}
	
	return $Return;
}

###############################################################################
# Main
###############################################################################

sub Main
{
	my $cgi			= shift;
	my %DisplayData;
	my $FormType	= $cgi->param('type');
	my $Redirect	= 0;

	# Safe default.
	$FormType = 'default' unless(defined($FormType));
	
	SWITCH:
	{
		if ('default'  eq $FormType) { $FormType = RenderDefault($cgi, \%DisplayData);				last SWITCH; }
		if ('user_add' eq $FormType) { $FormType = RenderUserAdd($cgi, \%DisplayData, $Redirect);	last SWITCH; }
		$FormType = 0;
	}

	# If there is still some form data...
	if ($FormType)
	{
		# Set the redirect.
		$Redirect = 1;
		goto SWITCH;
	}
	else
	{
		# Display the page.
		DisplayMain->printDocument(\%DisplayData);
	}
}

###############################################################################
# The main body of the script.
###############################################################################

my $cgi = new CGI;

Main($cgi);

