#!/usr/bin/perl

###############################################################################
#                                                                             #
# Filename: server_voltage.cgi                                                #
#                                                                             #
# Description: CGI needed to display voltage information.                     #
#                                                                             #
###############################################################################
###############################################################################
#                                                                             #
#  Copyright  2000 - 2001 Intel Corporation.                                 #
#  Intel Corporation All Rights Reserved.                                     #
#                                                                             #
#  The source code contained or described herein and all documents related to #
#  the source code ("Material") are owned by Intel Corporation or its         #
#  suppliers or licensors.  Title to the Material remains with Intel          #
#  Corporation or its suppliers and licensors.  The Material contains trade   #
#  secrets and proprietary and confidential information of Intel or its       #
#  suppliers and licensors.  The Material is protected by worldwide copyright #
#  and trade secret laws and treaty provisions.  No part of the Material may  #
#  be used, copied, reproduced, modified, published, uploaded, posted,        #
#  transmitted, distributed, or disclosed in any way without Intel's prior    #
#  express written permission.                                                #
#                                                                             #
#  No license under any patent, copyright, trade secret or other intellectual #
#  property right is granted to or conferred upon you by disclosure or        #
#  delivery of the Materials, either expressly, by implication, inducement,   #
#  estoppel or otherwise. Any license under such intellectual property rights #
#  must be express and approved by Intel in writing.                          #
#                                                                             #
###############################################################################

# Put the PERL library in the path.
use lib '/usr/ins/intel/gui/perllib';

use strict;
use ConfigurationData;
use LanguageResource;
use DisplayMain;
use Safe;
use CGI;

###############################################################################
# GetSMBIOSData
###############################################################################

sub GetSMBIOSData
{
	my ($Key)			= @_;
	my $SMBIOSDataRaw	= Safe::backtick('/usr/ins/intel/bin/asm/sysSMBIOS dump');
	my @SMBIOSData		= split(/^/m, $SMBIOSDataRaw);
	my $KeyTemp;
	my $Value			= '';
	my $Line;

	# Convert to lower case.
	$Key = lc($Key);

	# Run through the data...
	foreach $Line (@SMBIOSData)
	{
		# If the line has an equal sign...
		if ($Line =~ /=/)
		{
			# Break out the key and value.
			($KeyTemp, $Value) = split(/=/, $Line);

			# Clear out the leading/trailing whitespace.
			$KeyTemp =~ s/^\s*(.*?)\s*$/$1/;

			# If the keys match...
			if ($Key eq lc($KeyTemp))
			{
				# Clear out the leading/trailing whitespace.
				$Value =~ s/^\s*(.*?)\s*$/$1/;
				last;
			}
			else
			{
				# Clean out the value.
				$Value = '';
			}
		}
	}

	return $Value;
}

###############################################################################
# GetCurrentVoltage
###############################################################################

sub GetCurrentVoltage
{
	my $CommandKey		= shift;
	my $CurrentVoltage	= 0;

	# Validate the input parameter(s)...
	if (defined($CommandKey))
	{
		# Get the requested data.
		$CurrentVoltage = (split(/ /, Safe::backtick("/usr/ins/intel/bin/asm/bakAsm $CommandKey")))[2];
	}

	return $CurrentVoltage;
}

###############################################################################
# GetIndicatorPosition
###############################################################################

sub GetIndicatorPosition
{
    my ($Lower, $Upper, $Current) = @_;
	my $Position = 50;

	if ((defined($Lower)) &&
		(defined($Upper)) &&
		(defined($Current)))
	{
		my $Base	= ($Lower - (($Upper - $Lower) / 2));
		my $End		= ($Upper + (($Upper - $Lower) / 2));

		# Calculate the position.
        $Position = ((($Current - $Base)/($End - $Base)) * 100); 
	}
	
	return $Position;
}

###############################################################################
# Main
###############################################################################

sub Main
{
	my ($cgi) 				= @_;
	my $PreferredLanguage	= (split(/,/, $ENV{'HTTP_ACCEPT_LANGUAGE'}))[0];
	my $StringResource		= new LanguageResource($PreferredLanguage, '/usr/ins/intel/gui/locale', [ 'server_voltage.lrc', 'server_hwinfo.lrc', 'server_alerts.lrc' ]);
	my $Configuration		= new ConfigurationData;
	my %_R					= ( 'IDT:200'				=> '',
								'VOLTAGE_THRESHOLDS'	=> '',
								'CRITICAL_LOWER'		=> '',
								'CRITICAL_UPPER'		=> '',
								'WARNING_LOWER'			=> '',
								'WARNING_UPPER'			=> '',
								'CURRENT'				=> '',
								'V'						=> '',
								'WARNING'				=> '',
								'CRITICAL'				=> '',
								'IDT:500'				=> '',
								'CONFIGURE_ALERTS'		=> '' );
	my @VoltData			= ( { 'c-label' => 'V0', 'label' => '12_VOLTS_POS', 'command-key' => 'get_volt_plus12' },
								{ 'c-label' => 'V1', 'label' => '12_VOLTS_NEG', 'command-key' => 'get_volt_minus12' },
								{ 'c-label' => 'V2', 'label' => '5_VOLTS_POS', 'command-key' => 'get_volt_plus5' },
								{ 'c-label' => 'V3', 'label' => '3.3_VOLTS_POS', 'command-key' => 'get_volt_plus3point3' },
								{ 'c-label' => 'V4', 'label' => 'CPU_I/O_VOLTAGE', 'command-key' => 'get_volt_cpuio' },
								{ 'c-label' => 'V5', 'label' => 'CPU_CORE_VOLTAGE', 'command-key' => 'get_volt_cpucore' } );
	my $Feedback;
	my $Result;
	my $html;
	my $V;

	# Get the string resources.
	$StringResource->GetStrings(\%_R);
	
	# Try to open the configuration file...
	if (0 == $Configuration->Attach('/etc/asm/asm.conf'))
	{
		my $KeyNormal;
		my $KeyCritical;
		my $ProcessorFamily;

		# Run through each of the voltages...
		foreach $V (@VoltData)
		{
			# Get the label.
			$V->{'label'} = $StringResource->_S($V->{'label'});

			# If this is the CPU core voltage...
			if (('V5' eq $V->{'c-label'}) &&
				($ProcessorFamily = uc(GetSMBIOSData('PROCESSOR_FAMILY'))))
			{
				# Clean up the data.
				$ProcessorFamily =~ s/ /_/g;
				$ProcessorFamily =~ s/^(.*)_{1}PROCESSOR(.*)/$1$2/;
				
				# Set the keys.
				$KeyNormal		= 'V5_THRESHOLDS_' . $ProcessorFamily;
				$KeyCritical	= 'V5_CRITICAL_THRESHOLDS_' . $ProcessorFamily;
			}
			else
			{
				# Set the keys.
				$KeyNormal		= $V->{'c-label'} . '_THRESHOLDS';
				$KeyCritical	= $V->{'c-label'} . '_CRITICAL_THRESHOLDS';
			}

			# Get the upper and lower thresholds.
			($V->{'warning-lower'},  $V->{'warning-upper'})		= split(/,/, $Configuration->GetValueKeyed($KeyNormal));
			($V->{'critical-lower'}, $V->{'critical-upper'})	= split(/,/, $Configuration->GetValueKeyed($KeyCritical));

			# Get the current voltage.
			$V->{'current'} = GetCurrentVoltage($V->{'command-key'});

			# Get the indicator positions.
			$V->{'i-position-right'}	= 5 + GetIndicatorPosition($V->{'warning-lower'}, $V->{'warning-upper'}, $V->{'current'});
			$V->{'i-position-left'}		= 95 - $V->{'i-position-right'};

			# Get the current status...
			if (($V->{'critical-lower'} > $V->{'current'}) || 
				($V->{'critical-upper'} < $V->{'current'}))
			{
				# Critical.
				$V->{'status'} = " <img src=/images/critical.gif> $_R{'CRITICAL'}";
			}
			elsif (($V->{'warning-lower'} > $V->{'current'}) || 
				   ($V->{'warning-upper'} < $V->{'current'}))
			{
				# Warning.
				$V->{'status'} = " <img src=/images/warning.gif> $_R{'WARNING'}";
			}
			else 
			{
				# Normal.
				$V->{'status'} = '';
			}
		}
	}
	else
	{
		# Run through each of the voltages...
		foreach $V (@VoltData)
		{
			# Safe defaults.
			$V->{'label'}				= $StringResource->_S($V->{'label'});
			$V->{'warning-upper'}		= 0;
			$V->{'warning-lower'}		= 0;
			$V->{'critical-upper'}		= 0;
			$V->{'critical-lower'}		= 0;
			$V->{'current'} 			= 0;
			$V->{'i-position-left'}		= 50; 
			$V->{'i-position-right'}	= 50;
			$V->{'status'} 				= '';
		}
	}

	# The top of the page.
	$html = <<HERE;
<table border=0 width=375>
<tr>
	<td>
		<table border=0 cellPadding=0 cellSpacing=1 width=100%>
        <tr>
			<td colSpan=3 width=100%><b>$_R{'VOLTAGE_THRESHOLDS'}</b></td>
		</tr>
HERE

	# Run through each of the voltages...
	foreach $V (@VoltData)
	{
		# Normalize the data.
		$V->{'warning-upper'}	= sprintf("%.2f", $V->{'warning-upper'});
		$V->{'warning-lower'}	= sprintf("%.2f", $V->{'warning-lower'});
		$V->{'critical-upper'}	= sprintf("%.2f", $V->{'critical-upper'});
		$V->{'critical-lower'}	= sprintf("%.2f", $V->{'critical-lower'});
		$V->{'current'} 		= sprintf("%.2f", $V->{'current'});
		
		# Add on the information the current voltage.
		$html .= <<HERE;
		<tr>
			<td width=382 colspan=3>&nbsp;</td>
		</tr>
		<tr>
			<td width=143><b>$V->{'label'} $V->{'status'}</b></td>
			<td width=239 colspan=2></td>
		</tr>
		<tr>
			<td width=143 align=center rowspan=5>
				<table border=0 cellpadding=0 cellspacing=0 width=100>
				<tr>
					<td width=$V->{'i-position-right'} align=right>
						<img border=0 src=/images/meter.gif width=10 height=6>
					</td>
					<td width=$V->{'i-position-left'}>&nbsp;</td>
				</tr>
				</table>
				<table border=1 cellpadding=0 cellspacing=0 width=100>
				<tr>
					<td width=95% bgcolor=#ffffff>
						<table border=0 width=100% cellspacing=0 cellpadding=0>
						<tr>
							<td width=23% bgcolor=#ff0000>&nbsp;</td>
							<td width=2% bgcolor=#ffff00>&nbsp;</td>
							<td width=50% bgcolor=#00ff00>&nbsp;</td>
							<td width=2% bgcolor=#ffff00>&nbsp;</td>
							<td width=23% bgcolor=#ff0000>&nbsp;</td>
						</tr>
						</table>
					</td>
				</tr>
				</table>
			</td>
			<td width=96>$_R{'CRITICAL_LOWER'}&nbsp;</td>
			<td width=139>$V->{'critical-lower'} $_R{'V'}</td>
		</tr>
		<tr>
			<td width=96>$_R{'WARNING_LOWER'}&nbsp;</td>
			<td width=139>$V->{'warning-lower'} $_R{'V'}</td>
		</tr>
		<tr>
			<td width=96>$_R{'CURRENT'}&nbsp;</td>
			<td width=139><b>$V->{'current'} $_R{'V'}</b></td>
		</tr>
		<tr>
			<td width=96>$_R{'WARNING_UPPER'}&nbsp;</td>
			<td width=139>$V->{'warning-upper'} $_R{'V'}</td>
		</tr>
		<tr>
			<td width=96>$_R{'CRITICAL_UPPER'}&nbsp;</td>
			<td width=139>$V->{'critical-upper'} $_R{'V'}</td>
		</tr>
HERE
	}

	# Add on the tail of the page.
	$html .= <<HERE;
		<tr>
			<td colSpan=3 width=100%>&nbsp;</td>
		</tr>
		</table>
		<br>
		<a href=server_alerts.cgi>$_R{'CONFIGURE_ALERTS'}</a>
	</td>
</tr>
</table>
HERE

	# Display the page.
    DisplayMain->printDocument({ 'q' => $cgi, 'html' => $html, 
								 'cookiecrumbs' => [ { 'name' => $_R{'IDT:500'}, 'location' => 'server_hwinfo.cgi' } ],
								 'root-link' => 'server_status', 'class' => 'server_voltage', 'id' => 200, 'title' => $_R{'IDT:200'} });
}

###############################################################################
# The main body of the script.
###############################################################################

my $cgi = new CGI;

Main($cgi);
