#!/usr/bin/perl

###############################################################################
#                                                                             #
# Filename: server_information.cgi                                            #
#                                                                             #
# Description: CGI needed to export basic system information.                 #
#                                                                             #
###############################################################################
###############################################################################
#                                                                             #
#  Copyright  2000 - 2001 Intel Corporation.                                 #
#  Intel Corporation All Rights Reserved.                                     #
#                                                                             #
#  The source code contained or described herein and all documents related to #
#  the source code ("Material") are owned by Intel Corporation or its         #
#  suppliers or licensors.  Title to the Material remains with Intel          #
#  Corporation or its suppliers and licensors.  The Material contains trade   #
#  secrets and proprietary and confidential information of Intel or its       #
#  suppliers and licensors.  The Material is protected by worldwide copyright #
#  and trade secret laws and treaty provisions.  No part of the Material may  #
#  be used, copied, reproduced, modified, published, uploaded, posted,        #
#  transmitted, distributed, or disclosed in any way without Intel's prior    #
#  express written permission.                                                #
#                                                                             #
#  No license under any patent, copyright, trade secret or other intellectual #
#  property right is granted to or conferred upon you by disclosure or        #
#  delivery of the Materials, either expressly, by implication, inducement,   #
#  estoppel or otherwise. Any license under such intellectual property rights #
#  must be express and approved by Intel in writing.                          #
#                                                                             #
###############################################################################

# Put the PERL library in the path.
use lib '/usr/ins/intel/gui/perllib';

use strict;
use Safe;
use ProductInformation;

###############################################################################
# GetSMBIOSData
###############################################################################

sub GetSMBIOSData
{
	my ($Key)			= @_;
	my $SMBIOSDataRaw	= Safe::backtick('/usr/ins/intel/bin/asm/sysSMBIOS dump');
	my @SMBIOSData		= split(/^/m, $SMBIOSDataRaw);
	my $KeyTemp;
	my $Value			= '';
	my $Line;

	# Convert to lower case.
	$Key = lc($Key);

	# Run through the data...
	foreach $Line (@SMBIOSData)
	{
		# If the line has an equal sign...
		if ($Line =~ /=/)
		{
			# Break out the key and value.
			($KeyTemp, $Value) = split(/=/, $Line);

			# Clear out the leading/trailing whitespace.
			$KeyTemp =~ s/^\s*(.*?)\s*$/$1/;

			# If the keys match...
			if ($Key eq lc($KeyTemp))
			{
				# Clear out the leading/trailing whitespace.
				$Value =~ s/^\s*(.*?)\s*$/$1/;
				last;
			}
			else
			{
				# Clean out the value.
				$Value = '';
			}
		}
	}

	return $Value;
}

###############################################################################
# GetDiskConfiguration
###############################################################################

sub GetDiskConfiguration
{
	my @DiskData;

	# Try to open the configuration file...
	if (open(ASMCONF, '< /etc/asm/asm.conf'))
	{
		my $ConfigurationData;
		my $Cache = $/;

		# Read in the data.
		read(ASMCONF, $ConfigurationData, (stat(ASMCONF))[7]);
		
		# Close the file handle.
		close(ASMCONF);

		# Undefine the input record separator.
		undef($/);
		
		# Get the configuration data.
		$ConfigurationData =~ s/.*BEGIN_DISKS.*?\n(.*?)END_DISKS.*/$1/s;
		
		# Reset the line separator.
		$/ = $Cache;
		
		# Fill out the array.
		@DiskData = split(/^/m, $ConfigurationData);
	}

	return @DiskData;
}

###############################################################################
# GetCurrentRPM
###############################################################################

sub GetCurrentRPM
{
	my $CommandKey	= shift;
	my $CurrentRPM	= 0;

	# Validate the input parameter(s)...
	if (defined($CommandKey))
	{
		# Get the requested data.
		$CurrentRPM = Safe::backtick("/usr/ins/intel/bin/asm/bakAsm $CommandKey");
	}

	return $CurrentRPM;
}

###############################################################################
# GetCurrentTemperature
###############################################################################

sub GetCurrentTemperature
{
	my $CommandKey			= shift;
	my $CurrentTemperature	= 0;

	# Validate the input parameter(s)...
	if (defined($CommandKey))
	{
		# Get the requested data.
		$CurrentTemperature = (split(/ /, Safe::backtick("/usr/ins/intel/bin/asm/bakAsm $CommandKey")))[2];
	}

	return $CurrentTemperature;
}

###############################################################################
# GetCurrentVoltage
###############################################################################

sub GetCurrentVoltage
{
	my $CommandKey		= shift;
	my $CurrentVoltage	= 0;

	# Validate the input parameter(s)...
	if (defined($CommandKey))
	{
		# Get the requested data.
		$CurrentVoltage = (split(/ /, Safe::backtick("/usr/ins/intel/bin/asm/bakAsm $CommandKey")))[2];
	}

	return $CurrentVoltage;
}

###############################################################################
# Main
###############################################################################

sub Main
{
	my ($cgi) 			= @_;
	my $Configuration	= new ConfigurationData;
	my $ProductInfo		= new ProductInformation();
	my $HostName		= Safe::backtick('/bin/hostname');
	my $Product			= Safe::backtick('/bin/cat /etc/redhat-release');
	my $StatusWeb		= ( -e '/var/run/httpd.pid' ) ? 1 : 2;
	my $StatusMail		= ( -e '/var/run/sendmail.pid' ) ? 1 : 2;
	my @FanData			= ( { 'c-label' => 'FAN0', 'command-key' => 'get_fan_0' },
							{ 'c-label' => 'FAN1', 'command-key' => 'get_fan_1' },
							{ 'c-label' => 'FAN2', 'command-key' => 'get_fan_2' },
							{ 'c-label' => 'FAN3', 'command-key' => 'get_fan_3' },
							{ 'c-label' => 'FAN4', 'command-key' => 'get_fan_4' } );
	my @VoltData		= ( { 'c-label' => 'V0', 'command-key' => 'get_volt_plus12' },
							{ 'c-label' => 'V1', 'command-key' => 'get_volt_minus12' },
							{ 'c-label' => 'V2', 'command-key' => 'get_volt_plus5' },
							{ 'c-label' => 'V3', 'command-key' => 'get_volt_plus3point3' },
							{ 'c-label' => 'V4', 'command-key' => 'get_volt_cpuio' },
							{ 'c-label' => 'V5', 'command-key' => 'get_volt_cpucore' } );
	my @TempData		= ( { 'c-label' => 'TEMPERATURE', 'command-key' => 'get_temp' } );
	my $HealthHardware	= 0;
	my $Version;

	# Build the version information.
	$Version = $ProductInfo->getManufacturer() . '_' . $ProductInfo->getFamily() . '_' . $ProductInfo->getSeries() . '_' . $ProductInfo->getVersion();

	# Try to open the configuration file...
	if (0 == $Configuration->Attach('/etc/asm/asm.conf'))
	{
		my ($Device, $MountPoint, $Type, $CriticalLower, $WarningLower);
		my @DiskData = GetDiskConfiguration();
		my $DiskFree;
		my $Disk;
		my $D;

		# Run through all of the disks...
		foreach $Disk (@DiskData)
		{
			# Cache the data.
			$DiskFree = $Disk;

			# Get just the data.
			$DiskFree =~ s/^DISKS=(.*)$/$1/;
			
			# Break out the disk data.
			($Device, $MountPoint, $Type, $WarningLower, $CriticalLower) = split(/:/, $DiskFree);

			# If there is disk information for the mount point...
			if ($DiskFree = Safe::backtick("/bin/df $MountPoint --block-size=1024"))
			{
				# Throw away the table line.
				$DiskFree = (split(/\s+/, (split(/^/m, $DiskFree))[1]))[3];

				# Get the current status...
				if ($DiskFree < $CriticalLower)
				{
					# Critical.
					$HealthHardware = 2;
					last;
				}
				elsif ($DiskFree < $WarningLower)
				{
					# Warning.
					$HealthHardware = 1;
				}
			}
		}

		# If critical has not been reached...
		if (2 != $HealthHardware)
		{
			# Run through each of the fan(s)...
			foreach $D (@FanData)
			{
				# Get the upper and lower thresholds.
				$D->{'warning-lower'}	= $Configuration->GetValueKeyed("$D->{'c-label'}_THRESHOLD");
				$D->{'critical-lower'}	= $Configuration->GetValueKeyed("$D->{'c-label'}_CRITICAL_THRESHOLD");
	
				# Get the current fan RPM.
				$D->{'current'} = GetCurrentRPM($D->{'command-key'});
	
				# Get the current status...
				if ($D->{'critical-lower'} > $D->{'current'})
				{
					# Critical.
					$HealthHardware = 2;
					last;
				}
				elsif ($D->{'warning-lower'} > $D->{'current'})
				{
					# Warning.
					$HealthHardware = 1;
				}
			}
		}

        # If critical has not been reached...
		if (2 != $HealthHardware)
		{
			# Run through each of the temperature(s)...
			foreach $D (@TempData)
			{
				# Get the upper and lower thresholds.
				($D->{'warning-lower'},  $D->{'warning-upper'})		= split(/,/, $Configuration->GetValueKeyed("$D->{'c-label'}_THRESHOLDS"));
				($D->{'critical-lower'}, $D->{'critical-upper'})	= split(/,/, $Configuration->GetValueKeyed("$D->{'c-label'}_CRITICAL_THRESHOLDS"));

				# Get the current temperature.
				$D->{'current'} = GetCurrentTemperature($D->{'command-key'});

				# Get the current status...
				if (($D->{'critical-lower'} > $D->{'current'}) || 
					($D->{'critical-upper'} < $D->{'current'}))
				{
					# Critical.
					$HealthHardware = 2;
					last;
				}
				elsif (($D->{'warning-lower'} > $D->{'current'}) || 
					   ($D->{'warning-upper'} < $D->{'current'}))
				{
					# Warning.
					$HealthHardware = 1;
				}
			}
		}
		
		# If critical has not been reached...
		if (2 != $HealthHardware)
		{
			my $ProcessorFamily;

			# Run through each of the voltages...
			foreach $D (@VoltData)
			{
				# Try to get the processor family...
				if (('V5' eq $D->{'c-label'}) &&
					($ProcessorFamily = uc(GetSMBIOSData('PROCESSOR_FAMILY') || '')))
				{
					# Clean up the data.
					$ProcessorFamily =~ s/ /_/g;
					$ProcessorFamily =~ s/^(.*)_{1}PROCESSOR(.*)/$1$2/;
					$ProcessorFamily = '_' . $ProcessorFamily;
				}
				else
				{
					# No delta.
					$ProcessorFamily = '';
				}

				# Get the upper and lower thresholds.
				($D->{'warning-lower'},  $D->{'warning-upper'})		= split(/,/, $Configuration->GetValueKeyed("$D->{'c-label'}_THRESHOLDS" . $ProcessorFamily));
				($D->{'critical-lower'}, $D->{'critical-upper'})	= split(/,/, $Configuration->GetValueKeyed("$D->{'c-label'}_CRITICAL_THRESHOLDS" . $ProcessorFamily));

				# Get the current voltage.
				$D->{'current'} = GetCurrentVoltage($D->{'command-key'});

				# Get the current status...
				if (($D->{'critical-lower'} > $D->{'current'}) || 
					($D->{'critical-upper'} < $D->{'current'}))
				{
					# Critical.
					$HealthHardware = 2;
					last;
				}
				elsif (($D->{'warning-lower'} > $D->{'current'}) || 
					   ($D->{'warning-upper'} < $D->{'current'}))
				{
					# Warning.
					$HealthHardware = 1;
				}
			}
		}
	}

	# Print the return data.
	print $cgi->header();
	print "hostname: $HostName";
	print "version: $Version\n"; 
	print "web: $StatusWeb\n";
	print "mail: $StatusMail\n";
	print "hardware: $HealthHardware\n";
	print "product: $Product";
}

###############################################################################
# The main body of the script.
###############################################################################

my $cgi = new CGI;

Main($cgi);
