#!/usr/bin/perl

###############################################################################
#                                                                             #
# Filename: user_diskuse.cgi                                                  #
#                                                                             #
# Description: CGI needed to display disk usage information.                  #
#                                                                             #
###############################################################################
###############################################################################
#                                                                             #
#  Copyright  2000 - 2001 Intel Corporation.                                 #
#  Intel Corporation All Rights Reserved.                                     #
#                                                                             #
#  The source code contained or described herein and all documents related to #
#  the source code ("Material") are owned by Intel Corporation or its         #
#  suppliers or licensors.  Title to the Material remains with Intel          #
#  Corporation or its suppliers and licensors.  The Material contains trade   #
#  secrets and proprietary and confidential information of Intel or its       #
#  suppliers and licensors.  The Material is protected by worldwide copyright #
#  and trade secret laws and treaty provisions.  No part of the Material may  #
#  be used, copied, reproduced, modified, published, uploaded, posted,        #
#  transmitted, distributed, or disclosed in any way without Intel's prior    #
#  express written permission.                                                #
#                                                                             #
#  No license under any patent, copyright, trade secret or other intellectual #
#  property right is granted to or conferred upon you by disclosure or        #
#  delivery of the Materials, either expressly, by implication, inducement,   #
#  estoppel or otherwise. Any license under such intellectual property rights #
#  must be express and approved by Intel in writing.                          #
#                                                                             #
###############################################################################

# Put the PERL library in the path.
use lib '/usr/ins/intel/gui/perllib';

use strict;
use LanguageResource;
use DisplayMain;
use Authorize;
use Usable;
use CGI;

###############################################################################
# GetQuotaInformation
###############################################################################

sub GetQuotaInformation
{
	my ($UserRef) = @_;
	my @Quota;
	
	# Cache the name.
	$UserRef->{'user'} = $UserRef->{'name'};
	
	# Get the user quota information.
	@Quota = Quotaall::repquota($UserRef);

	# If there is a quota...
	if (0 < $Quota[1])
	{
		$UserRef->{'quota'}			= 1;
		$UserRef->{'quotaused'}		= $Quota[0];
		$UserRef->{'quotatotal'}	= $Quota[1];
	}
	else
	{
		$UserRef->{'quota'} = 0;
	}
}

###############################################################################
# GetDiskUsageMail
###############################################################################

sub GetDiskUsageMail
{
	my ($UserRef) = @_; 
	my $DiskUsage;
	
	# If there is a mailbox directory...
	if ( -e "$UserRef->{'home'}/mbox" )
	{
		# Get the mail usage.
		$DiskUsage = -s "$UserRef->{'home'}/mbox";
		$DiskUsage = FormatFloat($DiskUsage / 1024);
	}
	else
	{
		# No disk usage.
		$DiskUsage = 0;
	}

	return $DiskUsage;
}

###############################################################################
# GetDiskUsageWeb
###############################################################################

sub GetDiskUsageWeb
{
	my ($UserRef) = @_; 
	my $DiskUsage;
	
	# If there is a web directory...
	if (-e "$UserRef->{'home'}/web" )
	{ 
		# Get the web usage.
		$DiskUsage = Usable::disk_usage($UserRef->{'name'}, "$UserRef->{'home'}/web");
	}
	else
	{
		# No disk usage.
		$DiskUsage = 0;
	}
	
	return $DiskUsage;
}

###############################################################################
# GetDiskUsageTotal
###############################################################################

sub GetDiskUsageTotal
{
	my ($UserRef)	= @_; 
	my $DiskUsage	= Usable::disk_usage($UserRef->{'name'}, $UserRef->{'home'});
	
	# Format the data.
	$DiskUsage = (split(/\s+/, $DiskUsage))[0];
	$DiskUsage = substr($DiskUsage, 0, (length($DiskUsage) - 1)); 

	return $DiskUsage;
}

###############################################################################
# FormatFloat
###############################################################################

sub FormatFloat 
{
	my ($Float, $Precision) = @_;
	my $FormattedFloat;

	# Validate the input parameter(s)...
	if (defined($Float))
	{
		# Safe default.
		$Precision = 1 unless(defined($Precision));

		# Prime the format string.
		$Precision .= 'f';

		# Format the value.
		$FormattedFloat = sprintf("%5.$Precision", $Float);
		
		# See if the output is integer.
		$FormattedFloat = int($FormattedFloat) if($FormattedFloat =~ /\.0/);

		# Remove any leading whitespace.
		$FormattedFloat =~ s/^\s*(.*)$/$1/;
	}
	
	return $FormattedFloat;
}

###############################################################################
# GetGraphColor
###############################################################################

sub GetGraphColor
{
	my $Percent = shift;
	
	# Set the color based
	# on the percent usage.
	if ($Percent > 80)
	{
		return 'red';
	}
	elsif (0 == $Percent)
	{
		return 'white';
	}
	
	return 'green';
}

###############################################################################
# Main
###############################################################################

sub Main
{
	my ($cgi) = @_;
	my $PreferredLanguage	= (split(/,/, $ENV{'HTTP_ACCEPT_LANGUAGE'}))[0];
	my $StringResource		= new LanguageResource($PreferredLanguage, '/usr/ins/intel/gui/locale', [ 'user_diskuse.lrc', 'common.lrc' ]);
	my %_R					= ( 'IDT:9200'			=> '',
								'WEB'				=> '',
								'MAIL'				=> '',
								'OTHER'				=> '',
								'TOTAL'				=> '',
								'KB'				=> '',
								'MB'				=> '',
								'USED'				=> '' );
	my $User;
	my $RemoteUser			= $cgi->remote_user();
	my $RemoteUserGID		= (getpwnam($RemoteUser))[3];
	my $Feedback;
	my $Result;
	my $html;
	
	# Get the user name.
	$User->{'name'} = $cgi->param('user');

	# If the user is logged in directly...
	$User->{'name'} = ((10000 != $RemoteUserGID) && (10001 != $RemoteUserGID)) ? $RemoteUser : '';

	# If there still is no user...
	if ('' eq $User->{'name'})
	{
		my $Level;
		
		# Try to get the target user.
		($User->{'name'}, $Level) = Authorize::getTargetUser();
			
		# Bail out if user is unauthorized...
		exit(0) if($Level == $Authorize::PRIV_NONE);
	}
	
	# Get the string resources.
	$StringResource->GetStrings(\%_R);
	
	# Collect the disk utilization data.
	GetQuotaInformation($User);
	$User->{'home'}			= (getpwnam($User->{'name'}))[7]; 
	$User->{'usage_web'}	= GetDiskUsageWeb($User);
	$User->{'usage_mail'}	= GetDiskUsageMail($User);
	$User->{'usage_total'}	= GetDiskUsageTotal($User);

	# If there is a quota...
	if ($User->{'quota'})
	{
		my $TotalSpace	= FormatFloat($User->{'quotatotal'} / 1024);
		my $DiskUsed	= FormatFloat($User->{'quotaused'});
		my $DiskUsedMB	= FormatFloat($User->{'quotaused'} / 1024);
		my $DiskFree	= FormatFloat($TotalSpace - $DiskUsedMB);
		my $OtherUsedMB	= 0;

		# Calculate the numbers.
		if ($DiskUsed > ($User->{'usage_mail'} + $User->{'usage_web'}))
		{
			$User->{'usage_other'} = FormatFloat($DiskUsed - ($User->{'usage_mail'} + $User->{'usage_web'}));
		}
		else
		{	
			$User->{'usage_other'} = 0;
		}

		if ($User->{'usage_other'} > 0)
		{
			$OtherUsedMB = $User->{'usage_other'} / 1024;
		}

		# Calculate the used percentages.
		$User->{'usage_percent_used_web'}	= FormatFloat((($User->{'usage_web'}  / 1024) / $TotalSpace) * 100);
		$User->{'usage_percent_used_mail'}	= FormatFloat((($User->{'usage_mail'} / 1024) / $TotalSpace) * 100);
		$User->{'usage_percent_used_other'} = ($TotalSpace > $OtherUsedMB) ? FormatFloat(($OtherUsedMB / $TotalSpace) * 100) : 100;
		$User->{'usage_percent_used_total'} = ($TotalSpace > $DiskUsedMB)  ? FormatFloat(($DiskUsedMB / $TotalSpace) * 100) : 100;
		
		# Calculate the free percentages.
		$User->{'usage_percent_free_web'}	= 100 - $User->{'usage_percent_used_web'};
		$User->{'usage_percent_free_mail'}	= 100 - $User->{'usage_percent_used_mail'};
		$User->{'usage_percent_free_other'}	= 100 - $User->{'usage_percent_used_other'};
		$User->{'usage_percent_free_total'}	= 100 - $User->{'usage_percent_used_total'};

		if (1024 <= $User->{'usage_web'})
		{ 
			# Format the data and set the units.
			$User->{'usage_web'}		= FormatFloat($User->{'usage_web'} / 1024);
			$User->{'usage_unit_web'}	= $_R{'MB'}; 
		}
		else
		{
			# Set the units.
			$User->{'usage_unit_web'}	= $_R{'KB'};
		} 

		if (1024 <= $User->{'usage_mail'})
		{ 
			# Format the data and set the units.
			$User->{'usage_mail'}		= FormatFloat($User->{'usage_mail'} / 1024);
			$User->{'usage_unit_mail'}	= $_R{'MB'}; 
		}
		else
		{
			# Set the units.
			$User->{'usage_unit_mail'} = $_R{'KB'};
		} 

		if (1024 <= $User->{'usage_other'})
		{ 
			# Format the data and set the units.
			$User->{'usage_other'}		= FormatFloat($User->{'usage_other'} / 1024);
			$User->{'usage_unit_other'} = $_R{'MB'}; 
		}
		else
		{
			# Set the units.
			$User->{'usage_unit_other'} = $_R{'KB'};
		} 

		# Get the graph colors.
		$User->{'color_mail'} 	= GetGraphColor($User->{'usage_percent_used_mail'});
		$User->{'color_web'}	= GetGraphColor($User->{'usage_percent_used_web'});
		$User->{'color_other'}	= GetGraphColor($User->{'usage_percent_used_other'});
		$User->{'color_total'}	= GetGraphColor($User->{'usage_percent_used_total'});

		# Get unique resources.
		$_R{'FREE'}				= $StringResource->_S('FREE');
		
		$html = <<HERE;
<table border=1 cellpadding=2 cellspacing=0 width=375 bgcolor=#ffffff bordercolor=#eeeeee>
<tr>  
	<td width=20% nowrap><b>$_R{'WEB'}</b></td>
	<td width=30% nowrap>
		<table border=0 cellpadding=0 cellspacing=0 width=100%>
		<tr align=right>
			<td align=right width=100%>&nbsp;$User->{'usage_percent_used_web'}%&nbsp;</td>
			<td align=right>
				<table border=1 cellpadding=0 cellspacing=0 width=70>
				<tr>
					<td width=95% bgcolor=#ffffff>
						<table border=0 width=100% cellspacing=0 cellpadding=0>
						<tr>
							<td width=$User->{'usage_percent_used_web'} bgcolor=$User->{'color_web'}>&nbsp;</td>
							<td width=$User->{'usage_percent_free_web'}></td>
						</tr>
						</table>
					</td>
				</tr>
				</table>
			</td>
		</tr>
		</table>
	</td>
	<td width=50%>&nbsp;<nobr>$User->{'usage_web'} $User->{'usage_unit_web'} $_R{'USED'}</nobr></td>
</tr>
<tr>  
	<td width=20% nowrap><b>$_R{'MAIL'}</b></td>
	<td width=30% nowrap>
		<table border=0 cellpadding=0 cellspacing=0 width=100%>
		<tr align=right>
			<td align=right width=100%>&nbsp;$User->{'usage_percent_used_mail'}%&nbsp;</td>
			<td align=right>
				<table border=1 cellpadding=0 cellspacing=0 width=70>
				<tr>
					<td width=95% bgcolor=#ffffff>
						<table border=0 width=100% cellspacing=0 cellpadding=0>
						<tr>
							<td width=$User->{'usage_percent_used_mail'} bgcolor=$User->{'color_mail'}>&nbsp;</td>
							<td width=$User->{'usage_percent_free_mail'}></td>
						</tr>
						</table>
					</td>
				</tr>
				</table>
			</td>
		</tr>
		</table>
	</td>
	<td width=50%>&nbsp;<nobr>$User->{'usage_mail'} $User->{'usage_unit_mail'} $_R{'USED'}</nobr></td>
</tr>
<tr>  
	<td width=20% nowrap><b>$_R{'OTHER'}</b></td>
	<td width=30% nowrap>
		<table border=0 cellpadding=0 cellspacing=0 width=100%>
		<tr align=right>
			<td align=right width=100%>&nbsp;$User->{'usage_percent_used_other'}%&nbsp;</td>
			<td align=right>
				<table border=1 cellpadding=0 cellspacing=0 width=70>
				<tr>
					<td width=95% bgcolor=#ffffff>
						<table border=0 width=100% cellspacing=0 cellpadding=0>
						<tr>
							<td width=$User->{'usage_percent_used_other'} bgcolor=$User->{'color_other'}>&nbsp;</td>
							<td width=$User->{'usage_percent_free_other'}></td>
						</tr>
						</table>
					</td>
				</tr>
				</table>
			</td>
		</tr>
		</table>
	</td>
	<td width=50%>&nbsp;<nobr>$User->{'usage_other'} $User->{'usage_unit_other'} $_R{'USED'}</nobr></td>
</tr>
<tr>  
	<td width=20% nowrap><b>$_R{'TOTAL'}</b></td>
	<td width=30% nowrap>
		<table border=0 cellpadding=0 cellspacing=0 width=100%>
		<tr align=right>
			<td align=right width=100%>&nbsp;$User->{'usage_percent_used_total'}%&nbsp;</td>
			<td align=right>
				<table border=1 cellpadding=0 cellspacing=0 width=70>
				<tr>
					<td width=95% bgcolor=#ffffff>
						<table border=0 width=100% cellspacing=0 cellpadding=0>
						<tr>
							<td width=$User->{'usage_percent_used_total'} bgcolor=$User->{'color_total'}>&nbsp;</td>
							<td width=$User->{'usage_percent_free_total'}></td>
						</tr>
						</table>
					</td>
				</tr>
				</table>
			</td>
		</tr>
		</table>
	</td>
	<td width=50%>&nbsp;<nobr>$DiskFree $_R{'MB'} $_R{'FREE'} ($TotalSpace $_R{'MB'} $_R{'TOTAL'})</nobr></td>
</tr>
</table>
HERE
	}
	else
	{
		my $UsageServices 	= $User->{'usage_web'} + $User->{'usage_mail'};
		
		# If there is any space left...
		if ($User->{'usage_total'} > $UsageServices)
		{
			# Get the non-service usage.
			$User->{'usage_other'} = $User->{'usage_total'} - $UsageServices;
		}
		else
		{
			# No disk usage.
			$User->{'usage_other'} = 0;
		}
		
		# Get unique resources.
		$_R{'NOTE'}				= $StringResource->_S('NOTE');
		$_R{'TOTAL_USED'} 		= $StringResource->_S('TOTAL_USED');
		$_R{'NO_QUOTA_NOTE'}	= $StringResource->_S('NO_QUOTA_NOTE');

		if (1024 <= $User->{'usage_web'})
		{ 
			# Format the data and set the units.
			$User->{'usage_web'}		= FormatFloat($User->{'usage_web'} / 1024);
			$User->{'usage_unit_web'}	= $_R{'MB'}; 
		}
		else
		{
			# Set the units.
			$User->{'usage_unit_web'}	= $_R{'KB'};
		} 

		if (1024 <= $User->{'usage_mail'})
		{ 
			# Format the data and set the units.
			$User->{'usage_mail'}		= FormatFloat($User->{'usage_mail'} / 1024);
			$User->{'usage_unit_mail'}	= $_R{'MB'}; 
		}
		else
		{
			# Set the units.
			$User->{'usage_unit_mail'} = $_R{'KB'};
		} 

		if (1024 <= $User->{'usage_other'})
		{ 
			# Format the data and set the units.
			$User->{'usage_other'}		= FormatFloat($User->{'usage_other'} / 1024);
			$User->{'usage_unit_other'} = $_R{'MB'}; 
		}
		else
		{
			# Set the units.
			$User->{'usage_unit_other'} = $_R{'KB'};
		} 
		
		if (1024 <= $User->{'usage_total'})
		{ 
			# Format the data and set the units.
			$User->{'usage_total'}		= FormatFloat($User->{'usage_other'} / 1024);
			$User->{'usage_unit_total'} = $_R{'MB'}; 
		}
		else
		{
			# Set the units.
			$User->{'usage_unit_total'} = $_R{'KB'};
		} 
		
		$html = <<HERE;
<table width=375>
<tr width=100%>
	<td colspan=2><b>$_R{'NOTE'}&nbsp;</b>$_R{'NO_QUOTA_NOTE'}</td>
</tr>
</table>
<table bordercolor=#eeeeee cellspacing=0 border=1 cellpadding=2 width=375 bgcolor=white>
<tr>
	<td><b>$_R{'WEB'}</b></td>
	<td>$User->{'usage_web'} $User->{'usage_unit_web'} $_R{'USED'}</td>
</tr>
<tr>
	<td><b>$_R{'MAIL'}</b></td>
	<td>$User->{'usage_mail'} $User->{'usage_unit_mail'} $_R{'USED'}</td>
</tr>
<tr>
	<td><b>$_R{'OTHER'}</b></td>
	<td>$User->{'usage_other'} $User->{'usage_unit_other'} $_R{'USED'}</td>
</tr>
<tr>
	<td><b>$_R{'TOTAL'}</b></td>
	<td>$User->{'usage_total'} $User->{'usage_unit_total'} $_R{'TOTAL_USED'}</td>
</tr>
</table>
HERE
	}
	
	# Display the page.
    DisplayMain->printDocument({ 'q' => $cgi, 'html' => $html, 
								 'class' => 'user_diskuse', 'id' => 9200, 'title' => $_R{'IDT:9200'}, 
								 'root-link' => 'user_diskuse' });
}

###############################################################################
# The main body of the script.
###############################################################################

my $cgi = new CGI;

Main($cgi);
