#!/usr/bin/perl

###############################################################################
#                                                                             #
# Filename: user_passwd.cgi                                                   #
#                                                                             #
# Description: CGI needed to modify user passwords.                           #
#                                                                             #
###############################################################################
###############################################################################
#                                                                             #
#  Copyright  2000 - 2001 Intel Corporation.                                 #
#  Intel Corporation All Rights Reserved.                                     #
#                                                                             #
#  The source code contained or described herein and all documents related to #
#  the source code ("Material") are owned by Intel Corporation or its         #
#  suppliers or licensors.  Title to the Material remains with Intel          #
#  Corporation or its suppliers and licensors.  The Material contains trade   #
#  secrets and proprietary and confidential information of Intel or its       #
#  suppliers and licensors.  The Material is protected by worldwide copyright #
#  and trade secret laws and treaty provisions.  No part of the Material may  #
#  be used, copied, reproduced, modified, published, uploaded, posted,        #
#  transmitted, distributed, or disclosed in any way without Intel's prior    #
#  express written permission.                                                #
#                                                                             #
#  No license under any patent, copyright, trade secret or other intellectual #
#  property right is granted to or conferred upon you by disclosure or        #
#  delivery of the Materials, either expressly, by implication, inducement,   #
#  estoppel or otherwise. Any license under such intellectual property rights #
#  must be express and approved by Intel in writing.                          #
#                                                                             #
###############################################################################

# Put the PERL library in the path.
use lib '/usr/ins/intel/gui/perllib';

use strict;
use LanguageResource;
use DisplayMain;
use Authorize;
use Password;
use CGI;

###############################################################################
# GetUserRights
#
# Description: Get Authorization Level of User (Server, Domain, or
#              Regular User)
#
###############################################################################

sub GetUserRights 
{
	my ($cgi)					= @_;
	my ($User, $Level)			= Authorize::getTargetUser();
	my $UserGID					= (getpwnam($User))[3];
	my $RemoteUser				= $cgi->remote_user();
	my $RemoteUserGID			= (getpwnam($RemoteUser))[3];
    my $AdministratorControl;

	# Start with the remote user level definitions...
	$AdministratorControl = ((10000 == $RemoteUserGID) || (10001 == $RemoteUserGID)) ? 1 : 0;
	
	# Checking for administrator overlap...
	if (($RemoteUser ne $User) && 
		($AdministratorControl))
	{
		# Domain admins can't change server admin's passwords...
		$AdministratorControl = ($RemoteUserGID <= $UserGID) ? 1 : 0;
	}
	else
	{
		# No override control.
		$AdministratorControl = 0; 
	}

	return ($User, $AdministratorControl);
}

###############################################################################
# SetUserPassword
###############################################################################

sub SetUserPassword
{
	my ($FormDataRef, $RemoteUser, $User, $AdministratorControl)	= @_;
	my $CheckPassword												= 1;
	my $Return														= 0;

	# If this is under administrator control
	# and a current password is not available...
	if (($AdministratorControl) &&
		(!$FormDataRef->{'password_current'}))
	{ 
		# No need to check the current password.
		$CheckPassword = 0; 
	}
	else
	{
		# Check the password.
		$CheckPassword = 0 if('VALID' eq Password->checkPassword($User, $FormDataRef->{'password_current'}));
	}

	# If the current password is good...
	if (0 == $CheckPassword) 
	{ 
		# Try to change the password...
		if (0 == Password->changePassword($User, $FormDataRef->{'password'}, $FormDataRef->{'password_current'}))
		{
			# If the remote user is the user...
			if ($RemoteUser eq $User)
			{
				# Adjust the cookie.
				print 	"HTTP/1.1 200 OK\n",
						"Set-Cookie: AuthCustom_Authenticate=$User:$FormDataRef->{'password'}; path=/\n";
			}
		}
		else
		{
			# Set the error code.
			$Return = 900000;
		}
	}
	else 
	{ 
		# Set the error code.
		$Return = 900001;
	}
	
	return $Return; 
}

###############################################################################
# ValidateFormData
###############################################################################

sub ValidateFormData
{
	my ($FormDataRef, $FocusFieldRef, $AdministratorControl)	= @_;
	my $Return	= 0;
	my $Key;

	# Run through each of the keys...
	foreach $Key (keys(%{ $FormDataRef }))
	{
		# If the data is defined...
		if (defined($FormDataRef->{$Key}))
		{
			# Trim the whitespace.
			$FormDataRef->{$Key} =~ s/^\s*(.*?)\s*$/$1/;
		}
	}
	
	# Validate the data...
	if ((!$AdministratorControl) &&
		($FormDataRef->{'password_current'} !~ /^\S+$/))
	{
		# Set the return code.
		$Return			= 900004;
		$$FocusFieldRef = 'password_current';
	}
	elsif ($FormDataRef->{'password'} !~ /^\S+$/)
	{
		# Set the return code.
		$Return			= 900002;
		$$FocusFieldRef = 'password';
	}
	elsif ($FormDataRef->{'password'} ne $FormDataRef->{'password_confirm'})
	{
		# Set the return code.
		$Return			= 900003;
		$$FocusFieldRef = 'password';
	}

	return $Return;
}

###############################################################################
# Main
###############################################################################

sub Main
{
	my ($cgi) = @_;
	my ($User, $AdministratorControl)	= GetUserRights($cgi);
	my $PreferredLanguage				= (split(/,/, $ENV{'HTTP_ACCEPT_LANGUAGE'}))[0];
	my $StringResource					= new LanguageResource($PreferredLanguage, '/usr/ins/intel/gui/locale', [ 'user_passwd.lrc', 'common.lrc' ]);
	my $Configuration					= new ConfigurationData;
	my %_R								= ( 'IDT:9000'					=> '',
											'PASSWORD_CURRENT'			=> '',
											'PASSWORD'					=> '',
											'PASSWORD_CONFIRM'			=> '',
											'APPLY'						=> '' );
	my %FormData						= ( 'password_current'	=> '',
											'password'			=> '',
											'password_confirm'	=> '' );
	my $RemoteUser						= $cgi->remote_user();
	my $RemoteUserGID					= (getpwnam($RemoteUser))[3];
	my $OriginalPassword;
	my $HelpIndicator;
	my $FocusField;
	my $Feedback;
	my $Result;
	my $html;

	# Get the string resources.
	$StringResource->GetStrings(\%_R);

	# If modifying the password...
	if (defined($cgi->param('apply')))
	{
		my $Key;

		# Get the form data from the form...
		foreach $Key (keys(%FormData))
		{
			# Get the data...
			$FormData{$Key} = (defined($cgi->param($Key))) ? $cgi->param($Key) : $FormData{$Key};
		}

		# Validate the form data.
		if (0 == ($Result = ValidateFormData(\%FormData, \$FocusField, $AdministratorControl)))
		{
			# Try to change the password.
			$Result = SetUserPassword(\%FormData, $RemoteUser, $User, $AdministratorControl);
		}

		# If there was an error...
		if ($Result)
		{
			# Get the error string.
			$Feedback	= $StringResource->_S($Result);
			$FocusField	= ((900001 == $Result) ? 'password_current' : undef) unless(defined($FocusField));
		}
	}
	
	# If there is administrator control...
	if ($AdministratorControl)
	{
		# Do not user a confirmation password.
		$OriginalPassword 	= '';
		$FocusField			= 'password' unless(defined($FocusField));
	}
	else
	{
		# Set up the password confirmation control.
		$OriginalPassword	= <<HERE;
			<tr>
				<td><b>$_R{'PASSWORD_CURRENT'}</b></td>
				<td><input type=password size=20 maxlength=32 name=password_current value='$FormData{'password_current'}'></td>
			</tr>
HERE
		$FocusField			= 'password_current' unless(defined($FocusField));
	}
	
	$html = <<HERE;
<form method=post action=user_passwd.cgi enctype=application/x-www-form-urlencoded> 
	<table border=0 width=375 bgcolor=#eeeeee>
	<tr>
		<td>
			<table border=0 width=100%>
			<tr>
				<td colspan=2>&nbsp</td>
			</tr>
$OriginalPassword
			<tr>
				<td><b>$_R{'PASSWORD'}</b></td>
				<td><input type=password size=20 maxlength=32 name=password value='$FormData{'password'}'></td>
			</tr>
			<tr>
				<td><b>$_R{'PASSWORD_CONFIRM'}</b></td>
				<td><input type=password size=20 maxlength=32 name=password_confirm value='$FormData{'password_confirm'}'></td>
			</tr>
			<tr>
				<td colspan=2>&nbsp;</td>
			</tr>
			<tr>
				<td align=right colspan=2><input type=submit class=clsButton value='$_R{'APPLY'}' name=apply></td>
			</tr>
			</table>
		</td>
	</tr>
	</table>
</form>
HERE

	# Get the appropriate help indicator.
	$HelpIndicator = ((10000 == $RemoteUserGID) || (10001 == $RemoteUserGID)) ? 'domain' : 'user';
	
	# Display the page.
	DisplayMain->printDocument({ 'q' => $cgi, 'html' => $html, 
								 'class' => 'user_passwd', 'id' => 9000, 'title' => $_R{'IDT:9000'},
								 'root-link' => 'user_passwd', 'helpIndicator' => $HelpIndicator,
								 'focus-field' => $FocusField, 'result' => $Result, 'feedback' => $Feedback });
}

###############################################################################
# The main body of the script.
###############################################################################

my $cgi = new CGI;

Main($cgi);
