#!/usr/bin/perl

###############################################################################
#                                                                             #
# Filename: user_setmail.cgi                                                  #
#                                                                             #
# Description: CGI needed to display and configure mail settings.             #
#                                                                             #
###############################################################################
###############################################################################
#                                                                             #
#  Copyright  2000 - 2001 Intel Corporation.                                 #
#  Intel Corporation All Rights Reserved.                                     #
#                                                                             #
#  The source code contained or described herein and all documents related to #
#  the source code ("Material") are owned by Intel Corporation or its         #
#  suppliers or licensors.  Title to the Material remains with Intel          #
#  Corporation or its suppliers and licensors.  The Material contains trade   #
#  secrets and proprietary and confidential information of Intel or its       #
#  suppliers and licensors.  The Material is protected by worldwide copyright #
#  and trade secret laws and treaty provisions.  No part of the Material may  #
#  be used, copied, reproduced, modified, published, uploaded, posted,        #
#  transmitted, distributed, or disclosed in any way without Intel's prior    #
#  express written permission.                                                #
#                                                                             #
#  No license under any patent, copyright, trade secret or other intellectual #
#  property right is granted to or conferred upon you by disclosure or        #
#  delivery of the Materials, either expressly, by implication, inducement,   #
#  estoppel or otherwise. Any license under such intellectual property rights #
#  must be express and approved by Intel in writing.                          #
#                                                                             #
###############################################################################

# Put the PERL library in the path.
use lib '/usr/ins/intel/gui/perllib';

use strict;
use LanguageResource;
use LanguageLogCGI;
use DisplayMain;
use Authorize;
use Forward;
use CGI;
$| = 1;

###############################################################################
# ValidateFormData
###############################################################################

sub ValidateFormData
{
	my ($FormDataRef)	= @_;
	my $Return			= 0;
	my $Key;

	# Run through each of the keys...
	foreach $Key (keys(%{ $FormDataRef }))
	{
		# If the data is defined...
		if (defined($FormDataRef->{$Key}))
		{
			# Trim the whitespace.
			$FormDataRef->{$Key} =~ s/^\s*(.*?)\s*$/$1/;
		}
	}
	
	# Look for bad addresses...
	if (($FormDataRef->{'forward_address'}) &&
		(!(($FormDataRef->{'forward_address'} =~ /^(\w|-|_)+$/) ||
		   ($FormDataRef->{'forward_address'} =~ /^(\w|-|_|\+|%)+(\.(\w|-|_|\+|%)+)*@(\w|-|_)+(\.(\w|-|_)+)*$/))))
	{
		# Set the return code.
		$Return = 930000;
	}

	return $Return;
}

###############################################################################
# GetForwardAddress
###############################################################################

sub GetForwardAddress
{
	my $FormDataRef = shift;
	my $ForwardData = Forward->new($FormDataRef->{'username'});
	
	# Clear the data.
	undef($FormDataRef->{'forward_address'});
	$FormDataRef->{'forwarding'} = 0;

	# If there is forward data...
	if (defined($ForwardData))
	{
		# Try to get the data.
		$FormDataRef->{'forward_address'}	= $ForwardData->get($Forward::ALIAS);
		$FormDataRef->{'forwarding'}		= 1 if($FormDataRef->{'forward_address'});
	}
	
	# If the data was not in the .forward file...
	unless (defined($FormDataRef->{'forward_address'}))
	{
		# Try to get the cached entry...
		if (open(ADDRESS, "< $FormDataRef->{'dir'}/.forward_address"))
		{
			# Get the address.
			{ local($/), $FormDataRef->{'forward_address'} = <ADDRESS> }
			
			# Close the file.
			close(ADDRESS);
		}
	}
}

###############################################################################
# SetForwardAddress
###############################################################################

sub SetForwardAddress
{
	my ($FormDataRef)	= @_;
	my $ForwardData		= Forward->new($FormDataRef->{'username'});
	my $Return			= 0;

	# If there is an object...
	if (!defined($ForwardData))
	{
		my $Log	= new LanguageLogCGI('/usr/ins/intel/gui/logs/server.log', '/usr/ins/intel/gui/locale', [ 'user_setmail.lrc' ]);

		# Log the error and set the return code.
		$Log->Append('error', 'critical', $Return = 930001);
	}
	else
	{
		# If there is forwarding data...
		if ($FormDataRef->{'forwarding'})
		{
			# Set the forwarding alias.
			$ForwardData->set($FormDataRef->{'forward_address'}, $Forward::ALIAS);

			# Update the checkbox.
			$FormDataRef->{'forwarding'} = ('' ne $FormDataRef->{'forward_address'}) ? 1 : 0;

			# Delete the cache file.
			unlink("$FormDataRef->{'dir'}/.forward_address");
		}
		else
		{
			# Try to open the cache file...
			if (open(ADDRESS, "> $FormDataRef->{'dir'}/.forward_address"))
			{
				# Cache the configuration.
				print ADDRESS $FormDataRef->{'forward_address'};
				
				# Close the file.
				close(ADDRESS);
			}
			else
			{
				# Set the return code.
				$Return = 930001;
			}

			# Remove any forwarding alias.
			$ForwardData->set(undef(), $Forward::ALIAS);
		}

		# Save the configuration.
		$ForwardData->save();
	}

	# Explicitly delete the object.
	undef($ForwardData);
	
	return $Return; 
}

###############################################################################
# GetReplyMessage
###############################################################################
 
sub GetReplyMessage
{
	my ($FormDataRef, $StringResource) = @_;
	my $ForwardData = Forward->new($FormDataRef->{'username'});

	# If the object is available...
	$FormDataRef->{'autoreply'} = ((defined($ForwardData)) && ($ForwardData->get($Forward::AUTOREPLY))) ? 1 : 0;
	
	# Try to get the cached entry...
	if (open(VACATION, "< $FormDataRef->{'dir'}/.vacation"))
	{
		# Get the message.
		{ local($/), $FormDataRef->{'vacation_message'} = <VACATION> }
		
		# Close the file.
		close(VACATION);
	}
	else
	{
		# Get the default.
		$FormDataRef->{'vacation_message'} = $StringResource->_S('OUT_OF_OFFICE');
	}
}

###############################################################################
# SetReplyMessage
###############################################################################
 
sub SetReplyMessage
{
	my ($FormDataRef)	= @_;
	my $ForwardData		= Forward->new($FormDataRef->{'username'});
	my $Return			= 0;
	
	# If there is an object...
	if (!defined($ForwardData))
	{
		my $Log	= new LanguageLogCGI('/usr/ins/intel/gui/logs/server.log', '/usr/ins/intel/gui/locale', [ 'user_setmail.lrc' ]);

		# Log the error and set the return code.
		$Log->Append('error', 'critical', $Return = 930001);
	}
	else
	{
		# If autoreply is checked...
		if ($FormDataRef->{'autoreply'})
		{  
			# which autoreply script should we use?
			my $ProcMail;
			
			# If there is an alias...
			if ($ForwardData->get($Forward::ALIAS))
			{
				# If an ALIAS exists in the .forward file, then use the
				# procmail script that does not deliver a copy to the
				# user's mailbox.
				$ProcMail = '"| exec /usr/bin/procmail /usr/ins/intel/gui/bin/autoreply-nocopy.proc # ' . $FormDataRef->{'username'} . '"';
				
			}
			else
			{
				# There is no ALIAS in the .forward file, so we presume
				# that the user would like a copy of mail in the mailbox.
				$ProcMail = '"| exec /usr/bin/procmail /usr/ins/intel/gui/bin/autoreply.proc # ' . $FormDataRef->{'username'} . '"';
			}
	
			# Set the autoreply.
			$ForwardData->set($ProcMail, $Forward::AUTOREPLY);
		}
		else
		{
			# Remove the autoreply
			$ForwardData->set(undef(), $Forward::AUTOREPLY);
		}
	
		# Prepair the .vacation file...
		if (open(VACATION, "> $FormDataRef->{'dir'}/.vacation"))
		{
			# Write out the configuration.
			print VACATION $FormDataRef->{'vacation_message'};

			# Close the file.
			close(VACATION);
		}
		else
		{
			my $Log	= new LanguageLogCGI('/usr/ins/intel/gui/logs/server.log', '/usr/ins/intel/gui/locale', [ 'user_setmail.lrc' ]);

			# Log the error and set the return code.
			$Log->Append('error', 'critical', $Return = 930002);
		}

		# Need root privs. to do this. 
		chown($FormDataRef->{'uid'}, $FormDataRef->{'gid'}, "$FormDataRef->{'dir'}/.vacation");
		
		# Save the configuration.
		$ForwardData->save();
	}

	# Explicitly delete the object.
	undef($ForwardData);

	return $Return;
}

###############################################################################
# Main
###############################################################################

sub Main
{
	my ($cgi) = @_;
	my $PreferredLanguage	= (split(/,/, $ENV{'HTTP_ACCEPT_LANGUAGE'}))[0];
	my $StringResource		= new LanguageResource($PreferredLanguage, '/usr/ins/intel/gui/locale', [ 'user_setmail.lrc', 'common.lrc' ]);
	my %_R					= ( 'IDT:9300'			=> '',
								'FORWARDING'		=> '',
								'E_MAIL_ADDRESS'	=> '',
								'AUTO_REPLY'		=> '',
								'REPLY_MESSAGE'		=> '' );
	my %FormData;
	my @UserList			= Users::listUsrs();
	my $HelpIndicator;
	my $UserRef;
	my $Item;
	my $Index;
	my $NoOverrideAttempt;
	my $RemoteUser			= $cgi->remote_user();
	my $RemoteUserGID		= (getpwnam($RemoteUser))[3];
	my $Feedback;
	my $Result;
	my $html;
	
	# Get the user name.
	$FormData{'username'} = $cgi->param('user');

	# If the user is logged in directly...
	$FormData{'username'} = ((10000 != $RemoteUserGID) && (10001 != $RemoteUserGID)) ? $RemoteUser : '';

	# If there still is no user...
	if ('' eq $FormData{'username'})
	{
		my $Level;
		
		# Try to get the target user.
		($FormData{'username'}, $Level) = Authorize::getTargetUser();
			
		# Bail out if user is unauthorized...
		exit(0) if($Level == $Authorize::PRIV_NONE);
	}
	
	# Run through the user list...
	foreach $Item (@UserList) 
	{
		# If the user name matches...
		if ($Item->{'user'} eq $FormData{'username'}) 
		{
			# Get the index.
			$Index = $Item->{'num'};
			last;
		}
	}

	# Get the user information.
	$UserRef = $UserList[$Index];
	
	# Bail out if user is unauthorized...
	exit(0) if((0 == $UserRef->{'gid'}) || ('root' eq $UserRef->{'user'}));

	# Get the string resources.
	$StringResource->GetStrings(\%_R);
	
	# Check for a domain administrator trying to modify a server administrator...
	$NoOverrideAttempt = ((10001 == $RemoteUserGID) && (10000 == $UserRef->{'gid'})) ? 0 : 1;
	
	# Get the user information.
	($FormData{'name'}, $FormData{'passwd'}, $FormData{'uid'}, $FormData{'gid'},
	 $FormData{'quota'}, $FormData{'comment'}, $FormData{'gcos'}, $FormData{'dir'},
	 $FormData{'shell'}, $FormData{'expire'}) = getpwnam($FormData{'username'});
	
	# Safe defaults.
	$FormData{'forwarding'}			= '';
	$FormData{'forward_address'}	= '';
	$FormData{'autoreply'}			= '';
	$FormData{'vacation_message'}	= '';

	# If the remote user has administrator rights...
	if ($NoOverrideAttempt)
	{
		# Set the help indicator.
		$HelpIndicator = 'domain';
		
		# If making changes...
		if (defined($cgi->param('apply')))
		{
			my $Key;

			# Get the form data from the form...
			foreach $Key (keys(%FormData))
			{
				# Get the data...
				$FormData{$Key} = (defined($cgi->param($Key))) ? $cgi->param($Key) : $FormData{$Key};
			}
	
			# Validate the form data...
			if ((0 == ($Result = ValidateFormData(\%FormData))) &&
				(0 == ($Result = SetForwardAddress(\%FormData))))
			{
				# Set the reply message.
				$Result = SetReplyMessage(\%FormData);
            }
			
			# If there was an error...
			if ($Result)
			{
				# Get the error string.
				$Feedback = $StringResource->_S($Result);
			}
		}
		else
		{
			# Get the form data.
			GetForwardAddress(\%FormData);
			GetReplyMessage(\%FormData, $StringResource);
		}

		# Get unique resources.
		$_R{'ENABLED'}	= $StringResource->_S('ENABLED');
		$_R{'APPLY'}	= $StringResource->_S('APPLY');
		
		# Re-align the checkboxes...
		$FormData{'forwarding'}	= ($FormData{'forwarding'}) ? 'checked' : '';
		$FormData{'autoreply'}	= ($FormData{'autoreply'}) ? 'checked' : '';
		
		$html = <<HERE;
<form method=post action=user_setmail.cgi enctype=application/x-www-form-urlencoded>
	<table border=0 width=375 bgcolor=#eeeeee>
	<tr>
		<td bgcolor=#eeeeee>
			<table border=0 width=100% bgcolor=#eeeeee>
			<tr>
				<td colspan=2>&nbsp;</td>
			</tr>
			<tr>
				<td colspan=2><b>$_R{'FORWARDING'}</b></td>
			</tr>
			<tr>
				<td colspan=2><input type=checkbox name=forwarding value=checked $FormData{'forwarding'}><b>$_R{'ENABLED'}</b></td>
			</tr>
			<tr>
				<td colspan=2>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>$_R{'E_MAIL_ADDRESS'}</b>&nbsp;&nbsp;&nbsp;<input type=text name=forward_address size=26 value='$FormData{'forward_address'}'></td>
			</tr>
			<tr>
				<td colspan=2>&nbsp;</td>
			</tr>
			<tr>
				<td colspan=2><b>$_R{'AUTO_REPLY'}</b></td>
			</tr>
			<tr>
				<td colspan=2><input type=checkbox name=autoreply value=checked $FormData{'autoreply'}><b>$_R{'ENABLED'}</b></td>
			</tr>
			<tr>
				<td colspan=2>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>$_R{'REPLY_MESSAGE'}</b></td>
			</tr>
			<tr>
				<td colspan=2>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<textarea rows=3 name=vacation_message cols=34>$FormData{'vacation_message'}</textarea></td>
			</tr>
			<tr>
				<td colspan=2>&nbsp;</td>
			</tr>
			<tr>
				<td align=right colspan=2 width=100%><input type=submit class=clsbutton value='$_R{'APPLY'}' name=apply></td>
			</tr>
			</table>
		</td>
	</tr>
	</table>
</form>
HERE
	}
	else
	{
		# Set the help indicator.
		$HelpIndicator = '';

		# Get the form data.
		GetForwardAddress(\%FormData);
		GetReplyMessage(\%FormData, $StringResource);

		# Get unique resources.
		$_R{'STATUS_LABEL'}	= $StringResource->_S('STATUS_LABEL');
		$_R{'ENABLED'}		= $StringResource->_S('ENABLED');
		$_R{'DISABLED'} 	= $StringResource->_S('DISABLED');
		
		# Get the display status.
		$FormData{'forwarding'}	= ($FormData{'forwarding'}) ? "<font color=green>$_R{'ENABLED'}</font>" : "<font color=red>$_R{'DISABLED'}</font>";
		$FormData{'autoreply'}	= ($FormData{'autoreply'}) ? "<font color=green>$_R{'ENABLED'}</font>" : "<font color=red>$_R{'DISABLED'}</font>";
		
		# Bring up the overrid note if needed...
		unless ($NoOverrideAttempt)
		{
			# Override note is needed.
			$NoOverrideAttempt = '<tr><td colspan=2><b>' . $StringResource->_S('NOTE') . '&nbsp;</b>' . $StringResource->_S('DOMAIN_OVERRIDE_NOTE') . '</td></tr><tr><td colspan=2>&nbsp;</td></tr>';
		}
		else
		{
			# No note needed.
			$NoOverrideAttempt = '';
		}
		
		$html = <<HERE;
<table border=0 width=375 bgcolor=white>
$NoOverrideAttempt
<tr>
	<td colspan=2><b>$_R{'FORWARDING'}</b></td>
</tr>
<tr>
	<td colspan=2>&nbsp;<b>$_R{'STATUS_LABEL'}</b>&nbsp;&nbsp;&nbsp;$FormData{'forwarding'}</td>
</tr>
<tr>
	<td colspan=2>&nbsp;<b>$_R{'E_MAIL_ADDRESS'}</b>&nbsp;&nbsp;&nbsp;$FormData{'forward_address'}</td>
</tr>
<tr>
	<td colspan=2>&nbsp;</td>
</tr>
<tr>
	<td colspan=2> <b>$_R{'AUTO_REPLY'}</b></td>
</tr>
<tr>
	<td colspan=2>&nbsp;<b>$_R{'STATUS_LABEL'}</b>&nbsp;&nbsp;&nbsp;$FormData{'autoreply'}</td>
</tr>
<tr>
	<td colspan=2>&nbsp;<b>$_R{'REPLY_MESSAGE'}</b></td>
</tr>
<tr>
	<td>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;</td>
	<td>$FormData{'vacation_message'}</td>
</tr>
</table>
HERE
	}
	
	# Display the page.
    DisplayMain->printDocument({ 'q' => $cgi, 'html' => $html, 
								 'class' => 'user_setmail', 'id' => 9300, 'title' => $_R{'IDT:9300'}, 'helpIndicator' => $HelpIndicator,
								 'root-link' => 'user_setmail', 'result' => $Result, 'feedback' => $Feedback });
}

###############################################################################
# The main body of the script.
###############################################################################

my $cgi = new CGI;

Main($cgi);
