package Daemon;

###############################################################################
#                                                                             #
# Filename: Daemon.pm                                                         #
#                                                                             #
# Description: Contains script needed to manage daemon processes.             #
#                                                                             #
###############################################################################
###############################################################################
#                                                                             #
#  Copyright  2000 - 2001 Intel Corporation.                                 #
#  Intel Corporation All Rights Reserved.                                     #
#                                                                             #
#  The source code contained or described herein and all documents related to #
#  the source code ("Material") are owned by Intel Corporation or its         #
#  suppliers or licensors.  Title to the Material remains with Intel          #
#  Corporation or its suppliers and licensors.  The Material contains trade   #
#  secrets and proprietary and confidential information of Intel or its       #
#  suppliers and licensors.  The Material is protected by worldwide copyright #
#  and trade secret laws and treaty provisions.  No part of the Material may  #
#  be used, copied, reproduced, modified, published, uploaded, posted,        #
#  transmitted, distributed, or disclosed in any way without Intel's prior    #
#  express written permission.                                                #
#                                                                             #
#  No license under any patent, copyright, trade secret or other intellectual #
#  property right is granted to or conferred upon you by disclosure or        #
#  delivery of the Materials, either expressly, by implication, inducement,   #
#  estoppel or otherwise. Any license under such intellectual property rights #
#  must be express and approved by Intel in writing.                          #
#                                                                             #
###############################################################################

# Put the PERL library in the path.
use lib '/usr/ins/intel/gui/perllib';

use strict;
use Errno qw(/.*/);
use Safe;

require Exporter;

###############################################################################
# Daemon::import
###############################################################################

sub import
{
	my $Caller	= caller;

	eval
	{
		require Errno;
	
		# Try to export the Errno symbols...
		Exporter::export('Errno', $Caller, qw(/.*/));
	};
}

###############################################################################
# Daemon::new
###############################################################################

sub new
{
	my ($class, $Daemon, $StatusExtension) = @_;

	# Initialize the object.
	my $this = 
	{
		'daemon'			=> $Daemon,
		'status-extension'	=> ((defined($StatusExtension)) && ('' ne $StatusExtension)) ? $StatusExtension : '\s+',
	};

	# Align the object with this package.
	bless($this, $class);
	
	# Return the object.
	return $this;
}

###############################################################################
# Daemon::DESTROY
###############################################################################

sub DESTROY
{
	my $this = shift;

	# Clean up any objects in use.
	$this->Detach();
}

###############################################################################
# Daemon::Attach
###############################################################################

sub Attach
{
	my $this	= shift;
	my $Daemon	= shift;

	# Detach from any previous bindings.
	$this->Detach();

	# Attach to the name.
	$this->{'daemon'} = $Daemon;

	# Return success.
	return 0;
}

###############################################################################
# Daemon::Detach
###############################################################################

sub Detach
{
	my $this = shift;
	
	# Undefine the list...
	undef($this->{'daemon'});
	
	# Return success.
	return 0;
}

###############################################################################
# Daemon::GetName
###############################################################################

sub GetName
{
	my $this = shift;

	# Return the requested data.
	return $this->{'daemon'};
}

###############################################################################
# Daemon::GetStatus
###############################################################################

sub GetStatus
{
	my $this				= shift;
	my @ProcessStatistics	= Safe::backtick('/bin/ps auwx');

	# Return the requested data.
	return (grep(/\b$this->{'daemon'}$this->{'status-extension'}/, @ProcessStatistics)) ? 1 : 0;
}

###############################################################################
# Daemon::Control
###############################################################################

sub Control
{
	my ($this, $Activity) = @_;
	my $Return;

	# Validate the input parameter(s)...
	if ((defined($Activity)) &&
		((2 >= $Activity) && (0 <= $Activity)))
	{
		my $Action;

		SWITCH:
		{
			# Set up the appropriate action...
			if (0 == $Activity) { $Action = '--del'; last SWITCH; }
			if (1 == $Activity) { $Action = '--add'; last SWITCH; }
			if (2 == $Activity) { $Action = '--add'; last SWITCH; }
		}                   

		# Try to modify the persisted state of the daemon...
		if (0 == ($Return = Safe::system("/sbin/chkconfig $Action $this->{'daemon'}")))
		{
			SWITCH:
			{
				# Set up the appropriate action...
				if (0 == $Activity) { $Action = 'stop'; last SWITCH; }
				if (1 == $Activity) { $Action = 'start'; last SWITCH; }
				if (2 == $Activity) { $Action = 'restart'; last SWITCH; }
			}                   
			
			# Try to modify the current state of the daemon...
			$Return = Safe::system("/etc/rc.d/init.d/$this->{'daemon'} $Action");
		}
	}
	else
	{
		$Return = EINVAL;
	}

	return $Return;
}

1;

__END__
