package DirectiveManager;

###############################################################################
#                                                                             #
# Filename: DirectiveManager.pm                                               #
#                                                                             #
# Description: Contains script needed to manage distribution directives.      #
#                                                                             #
###############################################################################
###############################################################################
#                                                                             #
#  Copyright  2000 - 2001 Intel Corporation.                                 #
#  Intel Corporation All Rights Reserved.                                     #
#                                                                             #
#  The source code contained or described herein and all documents related to #
#  the source code ("Material") are owned by Intel Corporation or its         #
#  suppliers or licensors.  Title to the Material remains with Intel          #
#  Corporation or its suppliers and licensors.  The Material contains trade   #
#  secrets and proprietary and confidential information of Intel or its       #
#  suppliers and licensors.  The Material is protected by worldwide copyright #
#  and trade secret laws and treaty provisions.  No part of the Material may  #
#  be used, copied, reproduced, modified, published, uploaded, posted,        #
#  transmitted, distributed, or disclosed in any way without Intel's prior    #
#  express written permission.                                                #
#                                                                             #
#  No license under any patent, copyright, trade secret or other intellectual #
#  property right is granted to or conferred upon you by disclosure or        #
#  delivery of the Materials, either expressly, by implication, inducement,   #
#  estoppel or otherwise. Any license under such intellectual property rights #
#  must be express and approved by Intel in writing.                          #
#                                                                             #
###############################################################################

# Put the PERL library in the path.
use lib '/usr/ins/intel/gui/perllib';

use strict;
use IO::File;

###############################################################################
# DirectiveManager::new
###############################################################################

sub new
{
	my ($class, $DirSource, $DirResult) = @_;

	# Initialize the object.
	my $this = 
	{
		"dir-source" => $DirSource,
		"dir-result" => $DirResult,
		"lock-file"	 => new IO::File(">/var/lock/subsys/ddmanager"),
	};

	# Align the object with this package.
	bless($this, $class);
	
	# Return the object.
	return $this;
}

###############################################################################
# DirectiveManager::DESTROY
###############################################################################

sub DESTROY
{
	my ($this) = @_;

	# Close the lock file.
	$this->{'lock-file'}->close();
    
	# Delete tke lock file.
	unlink("/var/lock/subsys/ddmanager");
}

###############################################################################
# DirectiveManager::GetSourceTarget
###############################################################################

sub GetSourceTarget
{
	my $this 			= shift;
	my $SourceTarget	= '';

	# Lock this block.
	$this->Lock;

	# Try to open the directory specified.
	if (opendir(SOURCEDIR, $this->{'dir-source'}))
	{
		# Get all of the files.
		my @FileList	= readdir(SOURCEDIR);
		my $FileRef;
		my $FileName;
		my $FileNameNew	= 0;
		
		# Close the directory handle.
		closedir(SOURCEDIR);

		# Run through the file list.
		foreach $FileRef (@FileList)
		{
			# If the file is a .dst file.
			if ($FileRef =~ /^.*\.dst/)
			{
				# Get the file name.
				$FileName = $FileRef;
				$FileName =~ s/^(.*)\.dst/$1/;

				# Check to see if this is NOT the smallest file.
				if ($FileNameNew <= $FileName)
				{
					# Set this as the next file.
					$FileNameNew = $FileName + 1;
				}
			}
		}

		# Set the next new file name.
		$SourceTarget = $this->{'dir-source'} . '/' . $FileNameNew . ".dst";
	}

	# Unlock this block.
	$this->Unlock;
	
	return $SourceTarget;
}

###############################################################################
# DirectiveManager::GetResultTarget
###############################################################################

sub GetResultTarget
{
	my ($this, $ID)		= @_;
	my $ResultTarget	= '';

	# Validate the input parameter(s)...
	if (defined($ID))
	{
		# Lock this block.
		$this->Lock;

		# Try to open the directory specified.
		if (opendir(RESULTDIR, $this->{'dir-result'}))
		{
			# Get all of the files.
			my $FileContents;
			my $FileTimeNew			= 0;
			my $FileNameNew			= '';
			my $FileTimeNewQueued	= 0;
			my $FileNameNewQueued	= '';
			my $FileName;
			my @FileList			= readdir(RESULTDIR);
			my $FileRef;
			my $File;

			# Close the directory handle.
			closedir(RESULTDIR);

			# Build the ID.
			$ID = "CID_ID=" . $ID;

			# Run through the file list.
			foreach $FileRef (@FileList)
			{
				# If the file is a .rst file.
				if ($FileRef =~ /^.*\.rst/)
				{
					# Form the complete file name and open the file.
					$FileName	= $this->{'dir-result'} . '/' . $FileRef;
					$File 		= new IO::File("<$FileName");

					# If the file was successfully opened.
					if ($File)
					{
						# If the file can be read and the ID is a match...
						if ((defined(read($File, $FileContents, 64))) &&
							($FileContents =~ /$ID/))
						{
							# If this file is not in process...
							unless ($FileContents =~ /CID_STATUS=-1/)
							{
								# Stat the file.
								my ($Device, $INode, $Mode, $NLink, $uid, $gid, $RDevice, $Size, $ATime, $MTime, $CTime, $BlockSize, $Blocks) = stat($FileName);

								# Check to see if this is NOT the oldest file.
								if ($FileTimeNew <= $MTime)
								{
									# If there is an old file name...
									if ($FileNameNew)
									{
										# Delete older results of the same ID.
										unlink($FileNameNew);
									}

									# Set this as the current file.
									$FileNameNew = $FileName;
									$FileTimeNew = $MTime;
								}
							}
							else
							{
								# Set this as the current file.
								$FileNameNew = $FileName;
								last;
							}
						}

						# Close the file.
						$File->close;
					}
				}
				elsif ($FileRef =~ /^.*\.qst/)
				{
					# Form the complete file name and open the file.
					$FileName	= $this->{'dir-result'} . '/' . $FileRef;
					$File 		= new IO::File("<$FileName");

					# If the file was successfully opened.
					if ($File)
					{
						# If the file can be read and the ID is a match...
						if ((defined(read($File, $FileContents, 64))) &&
							($FileContents =~ /$ID/))
						{
							# Stat the file.
							my ($Device, $INode, $Mode, $NLink, $uid, $gid, $RDevice, $Size, $ATime, $MTime, $CTime, $BlockSize, $Blocks) = stat($FileName);

							# Check to see if this is NOT the oldest file.
							if ($FileTimeNewQueued <= $MTime)
							{
								# Set this as the current file.
								$FileNameNewQueued = $FileName;
								$FileTimeNewQueued = $MTime;
							}
						}

						# Close the file.
						$File->close;
					}
				}
			}

			# If a file was found...
			if ($FileNameNewQueued)
			{
				# Set the next new file name.
				$ResultTarget = $FileNameNewQueued;
			}
			elsif ($FileNameNew)
			{
				# Set the next new file name.
				$ResultTarget = $FileNameNew;
			}
		}

		# Unlock this block.
		$this->Unlock;
	}
	
	return $ResultTarget;
}

###############################################################################
# DirectiveManager::GetResultTargetStatus
###############################################################################

sub GetResultTargetStatus
{
	my ($this, $ResultTarget)	= @_;
	my $Status					= '';
	my $File;
    
	# Lock this block.
	$this->Lock;

	# Open the result target.
	$File = new IO::File("<$ResultTarget");

	# If the file was successfully opened.
	if ($File)
	{
		my $FileContents;

		# If the file can be read and the ID is a match...
		if ((defined(read($File, $FileContents, 512))) &&
			($FileContents =~ /CID_STATUS/))
		{
			my @Lines;
			my $Line;

			# Seek to the beginning of the file.
			seek($File, 0, 0);
			@Lines = <$File>;

			# For each line in the file...
			foreach $Line (@Lines)
			{
				# If this is the line with the status...
				if ($Line =~ /CID_STATUS/)
				{
                    # Get the status;
					$Status = $Line;
					$Status =~ s/.*CID_STATUS=(.*)$/$1/;
					chomp($Status);

					last;
				}
			}
		}

		# Close the file.
		$File->close;
	}

	# Unlock this block.
	$this->Unlock;

    return ($Status ne '') ? $Status : undef;
}

###############################################################################
# DirectiveManager::DeleteAllResults
###############################################################################

sub DeleteAllResults
{
	my ($this, $ID) = @_;

	# Lock this block.
	$this->Lock;

	# Try to open the directory specified.
	if (opendir(RESULTDIR, $this->{'dir-result'}))
	{
		# Get all of the files.
        my $FileContents;
		my $FileNameDel;
		my $FileName;
		my @FileList		= readdir(RESULTDIR);
		my $FileRef;
		my $File;
		
		# Close the directory handle.
		closedir(RESULTDIR);

		# Build the ID.
		$ID = "CID_ID=" . $ID;

		# Run through the file list.
		foreach $FileRef (@FileList)
		{
			# If the file is a .rst file.
			if ($FileRef =~ /^.*\.rst/)
			{
				# Form the complete file name and open the file.
				$FileName	= $this->{'dir-result'} . '/' . $FileRef;
				$File 		= new IO::File("<$FileName");

				# If the file was successfully opened.
				if ($File)
				{
					# If the file can be read and the ID is a match...
                    $FileNameDel = ((defined(read($File, $FileContents, 64))) && ($FileContents =~ /$ID/)) ? $FileName : '';

					# Close the file.
					$File->close;

					# If this file is not in process...
					unless ($FileContents =~ /CID_STATUS=-1/)
					{
						# If there is a file to delete...
						if ($FileNameDel)
						{
							# Delete old result file.
							unlink($FileNameDel);
						}
					}
				}
			}
		}
	}

	# Unlock this block.
	$this->Unlock;
}

###############################################################################
# DirectiveManager::GenerateResultTarget
###############################################################################

sub GenerateResultTarget
{
	my ($this, $SourceTarget) = @_;
	my $ResultTarget = '';
	
	# Lock this block.
	$this->Lock;

	# If the object is initialized...
	if (($this->{'dir-source'}) &&
		($this->{'dir-result'}))
	{
		# If a valid source target was passed in...
		if ($SourceTarget =~ /$this->{'dir-source'}\/.*\.dst/)
		{
			# Cache the source target.
			$ResultTarget = $SourceTarget;

			# Build the result target.
			$ResultTarget =~ s/$this->{'dir-source'}\/(.*)\.dst/$this->{'dir-result'}\/$1\.qst/;
		}
	}

	# Unlock this block.
	$this->Unlock;

	return $ResultTarget;
}

###############################################################################
# DirectiveManager::GenerateQueuedResult
###############################################################################

sub GenerateQueuedResult($ID, $Class, $cgi)
{
	my ($this, $ID, $Class, $TargetNodesRaw) = @_;
	my @TargetNodes		= split(/,/, $TargetNodesRaw);
	my $QueuedResult	= '';
	my $Target;
	my $TargetSecurity;
	my $TargetIP;
	my $Item;

	# Build the header.
	$QueuedResult = "CID_ID=" . $ID . "\nCID_CLASS=" . $Class . "\nCID_STATUS=-1\n";

	# Run through the node list...
	foreach $Item (@TargetNodes)
    {
		# Break up the data.
		($Target, $TargetIP, $TargetSecurity) = split(/:/, $Item);

		# Set the target to 'processing'.
		$QueuedResult .= "$Target:$TargetIP=$TargetSecurity:-1\n";
	}

	return $QueuedResult;
}

###############################################################################
# DirectiveManager::GetDirSource
###############################################################################

sub GetDirSource
{
	my $this = shift;

	# Return the requested data.
	return $this->{'dir-source'};
}

###############################################################################
# DirectiveManager::SetDirSource
###############################################################################

sub SetDirSource
{
	my ($this, $DirSource) = @_;

	# Set the data.
	$this->{'dir-source'} = $DirSource;

	# Return the new value.
	return $this->{'dir-source'};
}

###############################################################################
# DirectiveManager::GetDirResult
###############################################################################

sub GetDirResult
{
	my $this = shift;

	# Return the requested data.
	return $this->{'dir-result'};
}

###############################################################################
# DirectiveManager::SetDirResult
###############################################################################

sub SetDirResult
{
	my ($this, $DirResult) = @_;

	# Set the data.
	$this->{'dir-result'} = $DirResult;

	# Return the new value.
	return $this->{'dir-result'};
}

###############################################################################
# DirectiveManager::Lock
###############################################################################

sub Lock
{
	my ($this) = @_;

	flock($this->{'lock-file'}, 2);
}

###############################################################################
# DirectiveManager::Unlock
###############################################################################

sub Unlock
{
	my ($this) = @_;

	flock($this->{'lock-file'}, 8);
}

1;

__END__
