package DomainMail;

###############################################################################
#                                                                             #
# Filename: DomainMail.pm                                                     #
#                                                                             #
# Description: Contains script needed to configure domain mail settings.      #
#                                                                             #
###############################################################################
###############################################################################
#                                                                             #
#  Copyright  2000 - 2001 Intel Corporation.                                 #
#  Intel Corporation All Rights Reserved.                                     #
#                                                                             #
#  The source code contained or described herein and all documents related to #
#  the source code ("Material") are owned by Intel Corporation or its         #
#  suppliers or licensors.  Title to the Material remains with Intel          #
#  Corporation or its suppliers and licensors.  The Material contains trade   #
#  secrets and proprietary and confidential information of Intel or its       #
#  suppliers and licensors.  The Material is protected by worldwide copyright #
#  and trade secret laws and treaty provisions.  No part of the Material may  #
#  be used, copied, reproduced, modified, published, uploaded, posted,        #
#  transmitted, distributed, or disclosed in any way without Intel's prior    #
#  express written permission.                                                #
#                                                                             #
#  No license under any patent, copyright, trade secret or other intellectual #
#  property right is granted to or conferred upon you by disclosure or        #
#  delivery of the Materials, either expressly, by implication, inducement,   #
#  estoppel or otherwise. Any license under such intellectual property rights #
#  must be express and approved by Intel in writing.                          #
#                                                                             #
###############################################################################

# Put the PERL library in the path.
use lib '/usr/ins/intel/gui/perllib';

use vars qw(@ISA @EXPORT @EXPORT_OK);
require Exporter;
@ISA = qw(Exporter);
@EXPORT = qw( ); 

use strict;
use LanguageResource;
use Virtusertable;
use Users;
use Mail;

############
# Globals
my $PreferredLanguage	= (defined($ENV{'HTTP_ACCEPT_LANGUAGE'})) ? (split(/,/, $ENV{'HTTP_ACCEPT_LANGUAGE'}))[0] : 'en-US';
my $StringResource		= new LanguageResource($PreferredLanguage, '/usr/ins/intel/gui/locale', [ 'DomainMail.lrc' ]);


sub mail_relay_add
{
	my ($fqdn,$admin) = @_;
	my ($host,$domain) = fqdn($fqdn);
	if ($domain eq '') { $domain = $host; }
	my $host_names = '/etc/mail/local-host-names';
	my $mail_relay = '/etc/mail/relay-domains';
	my $virt_table = '/etc/mail/virtusertable';
	my($flag,$flag1a,$flag1b,,$flag2);

        open(FHH1,"$mail_relay");
	flock(FHH1,2);
                while(<FHH1>) {
			 $flag = 'no' if (/^$domain$/);
		}
        close(FHH1);
	if(!$flag) {
		open(FHH, ">>$mail_relay") or return($StringResource->_S('FILE_OPEN_ERROR', [ " - $mail_relay" ] ));
		flock (FHH, 2) or return($StringResource->_S('FILE_LOCK_ERROR', [ " - $mail_relay" ] ));

		print FHH "$domain\n";
		close(FHH);
	}

	###########################################
        open(FHH2, "$host_names");
	flock(FHH2, 2);
                while(<FHH2>) {
                         $flag1a = 'no' if (/^$domain$/);
                         $flag1b = 'no' if (/^$fqdn$/);
                }
        close(FHH2);


		open(FHH, ">>$host_names") or return($StringResource->_S('FILE_OPEN_ERROR', [ " - $host_names" ] ));
		flock(FHH, 2) or return($StringResource->_S('FILE_LOCK_ERROR', [ "- $host_names" ] ));
	
		if(!$flag1b) {	print FHH "$fqdn\n"; }
		if ($domain eq $host) {
		}
		else {
			if (!$flag1a) {	print FHH "$domain\n"; }
		}
		close(FHH);

	######################################################
        my ($flag2a,$flag2b);
	open(MAIL, "$virt_table");
	flock(MAIL,2);
                while(<MAIL>) {
                       $flag2a = 'on' if (/postmaster\@$fqdn\s+$admin\@$fqdn$/);
                       $flag2b = 'on' if (/webmaster\@$fqdn\s+$admin\@$fqdn$/);
                }
        close(MAIL);

	if((!$flag2a) && (!$flag2b)) {  
		open(MAIL, ">>$virt_table") or return($StringResource->_S('FILE_OPEN_ERROR', [ " - $virt_table" ] ));
		flock(MAIL, 2) or return($StringResource->_S('FILE_LOCK_ERROR', [ "- $virt_table" ] ));
		print MAIL "postmaster\@$fqdn		$admin\@$fqdn\n";
		print MAIL "webmaster\@$fqdn		$admin\@$fqdn\n";
		close(MAIL);
	}
}

##################################
sub mail_relay_remove 
{

	my ($fqdn,$admin) =  @_;
        my(@line,@line_mail,@line_relay); 
	my($host,$domain) = fqdn($fqdn);
        if ($domain eq '') { $domain = $host; }
	my $host_names = '/etc/mail/local-host-names';
	my $mail_relay = '/etc/mail/relay-domains';
	my $virt_table = '/etc/mail/virtusertable';
	open(FHH, "$host_names");
	flock(FHH,2);
		while(<FHH>) {
			push(@line, $_);
		}
	close(FHH);

	open(FHH, ">$host_names") or return($StringResource->_S('FILE_OPEN_ERROR', [ " - $host_names" ] ));
	flock(FHH, 2) or return($StringResource->_S('FILE_LOCK_ERROR', [ " - $host_names" ] ));

	foreach my $item1 ( @line) {
		if (($item1 =~ /^$domain$/) or ($item1 =~ /^$fqdn$/)) {
		}
		else {
			print FHH "$item1";
		}

	}

	close(FHH);
##########
	open(FHH, "$mail_relay");
	flock(FHH,2);
		while(<FHH>) {
			push(@line_relay, $_);
		}
	close(FHH);
	open(FHH, ">$mail_relay") or return($StringResource->_S('FILE_OPEN_ERROR', [ " - $mail_relay: $!" ] ));
	flock(FHH, 2) or return($StringResource->_S('FILE_LOCK_ERROR', [ " - $mail_relay" ] ));
	foreach my $idx ( @line_relay) {
	 	if ($idx =~ /^$domain$/) { }
		else {	print FHH "$idx"; }

	}
	close(FHH);

##########

	open(MAIL, "$virt_table");
	flock(MAIL,2);
		while(<MAIL>) {
			push(@line_mail, $_);
		}
	close(MAIL);
	open(MAIL, ">$virt_table") or return($StringResource->_S('FILE_OPEN_ERROR', [ " - $virt_table: $!" ] ));
	flock(MAIL, 2) or return($StringResource->_S('FILE_LOCK_ERROR', [ " - $virt_table" ] ));
	foreach my $item2 (@line_mail) {
	 	if ($item2 =~ /$fqdn/ ) { }
		else { print MAIL "$item2"; }

	}
	close(MAIL);

}

#######################################################################

sub statMail
{
	my($fqdn,$adm) = @_;
        my $mail = '/etc/mail/relay-domains';;
	my $host_names = '/etc/mail/local-host-names';
	my ($host,$domain) = fqdn($fqdn);
        
	if ($domain eq '') { $domain = $host; }
	my ($result, $result1); 
	open(FHH, "$mail");
	flock(FHH,2);
        while(<FHH>) {
                if (/($domain)$/) { $result = $1; }
        }
        close(FHH);
       
	open(FHH1, "$host_names");
	flock(FHH1,2);
        while(<FHH1>) {
                if (/($domain)$/) { $result1 = $1; }
        }
        close(FHH1);

	return $result if (($result) && ($result1));
}

##########################################################
sub setalias  
{
	my ($login,$fqdn,$alias_value) = @_;

	my (@data,$new_alias,$uid);	
	my $alias_file = '/etc/mail/virtusertable';
	my @alias_name = split(/[\s+\n\r\,\;]/,$alias_value);	
	
	open(VIRT,"$alias_file");
	flock(VIRT,2);
		while(<VIRT>){
			push (@data, $_);
		}			
	close(VIRT);		
	
	foreach my $id (@data) {
		if ($id =~ /(\S+)\@(\S+)\s+(\S+)\@(\S+)$/) {
			foreach my $item (@alias_name) {
				return "Error $item is already in use" if ($1 eq $item);
			}	
		}
	}
	open(CONF,">>$alias_file");
	flock(CONF,2);
		foreach my $i (@alias_name) {
			
                	$uid = getpwnam($i);
                	if ($uid) {                             
                		close(CONF);
				return "Error $i is used as an user name, please try another name";                             
			}
			elsif ($i ne '') {
			print CONF "$i\@$fqdn\t\t\t$login\@$fqdn\n";
			}
		}
	close(CONF);
}		

	
#######################################################################
sub listAlias 
{
	my $alias_file = "/etc/mail/virtusertable";
        my (@var,$user,$alias,$fullalias,$domain1,$domain2,$user_mail);
	open(VIRT,"$alias_file");
	flock(VIRT,2);
        while(<VIRT>){
		s/\r|\n//g;
                if (!/^[#\+\-]/) {
			($fullalias,$user_mail) = split(/\s+/,$_);
			($alias,$domain1) = split(/\@/,$fullalias);
			($user,$domain2) = split(/\@/,$user_mail);
			push(@var, { 	'alias' => $alias,
					'domain' =>$domain2,
					'user' => $user,
					'num' => scalar(@var) });		

	        }

	}
	close(VIRT);
return @var;
}

###############################################################################

sub userAlias 
{
	my ($user,$domain) = @_;
	my (@useralias,$item);	
	my $alias_file = "/etc/mail/virtusertable";
        my (@var,$alias,$fullalias,$domain1,$domain2,$user_mail);
        open(VIRT,"$alias_file");
	flock(VIRT,2);
        	while(<VIRT>){
			if (/(\S+)\@($domain)\s+($user)\@($domain)$/) {
				if ($3 eq $user) {
					$item = $1;
					push (@useralias, $item);
				}
			$item = '';	 
			}
		}
	close(VIRT);
	$alias = join(',',@useralias);
	return($alias);
}




###############################################################
sub modAlias 

{
        my ($login,$domain,$alias_value) = @_;

	# is this a request to delete all aliases pointing to a user?
	if ( ($domain eq '') && ($alias_value eq '') ) {
		return Virtusertable::delete_by_username($login);
	}

	# split the value given by the user into an array of aliases
	my @aliases = split(/[\s+\n\r\,\;]+/,$alias_value);

	# update the virtusertable
	return Virtusertable::update_alias_list($login,$domain,@aliases);

#        my (@data,$new_alias,$user,$uid,@alias_name,$val1);
##		if ($domain eq '') {	$domain = Users::getUserDomain($login); }
#			$domain = Users::getUserDomain($login);
#	my $alias_file = '/etc/mail/virtusertable';
#	open(VIRT,"$alias_file");
#        while(<VIRT>){
#                push (@data, $_);
#        }
#        close(VIRT);
#	
#	if ($alias_value ne '') {
#		 	@alias_name = split(/[\s+\n\r\,\;]/,$alias_value);
#			foreach my $id (@data) {
#        			if ($id =~ /(\S+)\@(\S+)\s+(\S+)\@(\S+)$/) {
#					$val1 = $1;	
#            				foreach my $item (@alias_name) {
#                				return "Error $item is already in use" if ($val1 eq $item);
#            			}
#    			}  
#
#		}
#	}
#        open(CONF,">$alias_file");
#        flock(CONF,2);
#          foreach my $i (@data) {
#		if ($i =~ /(\S+)\@($domain)\s+($login)\@($domain)$/) {
#        	      if ($3 eq $login) {
#				if ($alias_value ne '') {
#
#					foreach my $j (@alias_name) {
#			              		$uid = getpwnam($j);
#                     				if ($uid) {
#                             				close(CONF);
#                               				return "Error $j is used as an user name, please try another name";
#                       				}
#                       				elsif ($j ne '') {
#                        				print CONF "$j\@$domain\t\t\t$login\@$domain\n";
#                       				}
#                				$j='';
#					}
#				}
#			}
#		}
#		else {
#                	print CONF "$i";
#		}
#	$i = '';
#      }
#     close(CONF);

}

##############################################################


sub noftp 
{
	my ($user,$some ) = @_;
	my $ftpuser = "/usr/ins/intel/wu-ftpd/conf/ftpusers";
	my $var = ftpaccess($user);
	my @line;	
	if ($var eq '') {
		open(FHH,"$ftpuser");
		while(<FHH>) {
			chomp;
			push (@line, $_);	
		}
		close(FHH);
		open(ADD,">$ftpuser");
		flock(ADD,2);
		print ADD "$user\n";	
		foreach my $item (@line) {
			print ADD "$item\n";
		} 
		close(ADD);
	}
}

########################################################
sub ftpaccess
{ 
	my ($user,$some ) = @_;
        my $ftpuser = "/usr/ins/intel/wu-ftpd/conf/ftpusers";
	my ($result, @allusr);
        open(FHH1, "$ftpuser");
        while(<FHH1>) {
                s/\r|\n//g;
                push (@allusr, $_);
                last if (/^\#/);
        }          
        close(FHH1);

        foreach my $item (@allusr) {
		if ($item eq $user) { 
		$result = $item; 
		}
	}
	close(FHH);
	return $result;
}


#########################################################
sub enftp 
{
        my ($user,$some ) = @_;
        my $ftpuser = "/usr/ins/intel/wu-ftpd/conf/ftpusers";
        my (@line,@data,$keep);
	open(FHH, "$ftpuser");

	while(<FHH>) {
		if (/^\##/ ... /^\##/) {
			chomp; 
			push(@data, $_); 
		}
		else { 
			chomp; 
			push(@line,$_); 
		} 
	}
	close(FHH);
	open(CONF,">$ftpuser");
	flock(CONF,2);
	foreach my $item (@line) {
		if($item eq $user) {
		}
		else { print CONF "$item\n"; }
	}
	foreach my $id (@data) {
		print CONF "$id\n";	
	}
	close(CONF);
}

#####################################################
sub nomail 
{
        my ($user,$some ) = @_;
        my $mail = "/etc/mail/access";
	my $var = mailaccess($user);
	if ($var eq '') {
		open(FHH, ">>$mail");
       	 	flock(FHH,2);
        	print FHH "$user\@\t550 Mailbox disable for this username\n";      
        	close(FHH);   
	} 
	Mail::restart_sendmail(); 
}


#######################################################
sub mailaccess
{ 
        my ($user,$some ) = @_;
        my $mail = "/etc/mail/access";
	my $result;
        open(FHH, "$mail");
	flock(FHH,2);
        while(<FHH>) {
                if (/($user)\S+/) { $result = $1; }
	}
        close(FHH);
        return $result;

}


#########################################################
sub enmail 

{
        my ($user,$some ) = @_;
        my $mail = "/etc/mail/access";
        my (@line);
        open(FHH, "$mail");
	flock(FHH,2);
        while(<FHH>) {
                push(@line,$_); 
        }
        close(FHH);
        open(CONF,">$mail");
        flock(CONF,2);
        foreach my $id (@line) {
                if($id =~ /$user/) {
                }
                else {
                        print CONF "$id";       
                }
                
                
        }
	close(CONF);
	Mail::restart_sendmail();
}


##########################
# argment is fqdn and then it breaks into hostname and domain name
#	  based on the input. If one dot do nothing and assume no nost name
#	  if more than one dot then break into parts for hostname and domainname.

sub fqdn
{
        my($fqdn,$some) = @_;
        my($host,$domain,@var);
        if ($fqdn =~ /^([A-z0-9\_\-]+)\.([A-z0-9\_\-]+)$/) {
                return $fqdn;
        }
        else {
                @var = split /\./,$fqdn;
                $host = shift @var;
		$domain = join '.',@var;
		return($host,$domain);
        }
}

1;
