package GroupOperations;

###############################################################################
#                                                                             #
# Filename: GroupOperations.pm                                                #
#                                                                             #
# Description: Contains script needed to enable group operations.             #
#                                                                             #
###############################################################################
###############################################################################
#                                                                             #
#  Copyright  2000 - 2001 Intel Corporation.                                 #
#  Intel Corporation All Rights Reserved.                                     #
#                                                                             #
#  The source code contained or described herein and all documents related to #
#  the source code ("Material") are owned by Intel Corporation or its         #
#  suppliers or licensors.  Title to the Material remains with Intel          #
#  Corporation or its suppliers and licensors.  The Material contains trade   #
#  secrets and proprietary and confidential information of Intel or its       #
#  suppliers and licensors.  The Material is protected by worldwide copyright #
#  and trade secret laws and treaty provisions.  No part of the Material may  #
#  be used, copied, reproduced, modified, published, uploaded, posted,        #
#  transmitted, distributed, or disclosed in any way without Intel's prior    #
#  express written permission.                                                #
#                                                                             #
#  No license under any patent, copyright, trade secret or other intellectual #
#  property right is granted to or conferred upon you by disclosure or        #
#  delivery of the Materials, either expressly, by implication, inducement,   #
#  estoppel or otherwise. Any license under such intellectual property rights #
#  must be express and approved by Intel in writing.                          #
#                                                                             #
###############################################################################

# Put the PERL library in the path.
use lib '/usr/ins/intel/gui/perllib';

use strict;
use LanguageTime;
use LanguageResource;
use DistributionDirective;
use DirectiveManager;
use QuickHelp;
use POSIX;
use Safe;
use CGI;
use CGI qw(escape);
use IO::File;

###############################################################################
# GroupOperations::new
###############################################################################

sub new
{
	my ($class, $PreferredLanguage, $Class, $ID, $TargetURI, $TargetPort)	= @_;
	
	# Initialize the object.
	my $this = 
	{
		'id'					=> $ID,
		'class'					=> $Class,
		'target-uri'			=> $TargetURI,
		'target-port'			=> $TargetPort,
		'preferred-language'	=> $PreferredLanguage,
		'lock-file'				=> new IO::File('> /var/lock/subsys/groupops'),
		'string-resource'		=> new LanguageResource($PreferredLanguage, '/usr/ins/intel/gui/locale', [ 'GroupOperations.lrc' ]),
		'directive-manager'		=> new DirectiveManager('/usr/ins/intel/bin/wcdd-source', '/usr/ins/intel/bin/wcdd-results'),
	};

	# Align the object with this package.
	bless($this, $class);
	
	# Return the object.
	return $this;
}

###############################################################################
# GroupOperations::DESTROY
###############################################################################

sub DESTROY
{
	my ($this) = @_;

	# Close the lock file.
	$this->{'lock-file'}->close();
    
	# Delete tke lock file.
	unlink('/var/lock/subsys/groupops');
}

###############################################################################
# GroupOperations::GenerateDirective
###############################################################################

sub GenerateDirective
{
	my ($this, $cgi)	= @_;
	my $SourceTarget;
	my $ResultTarget;
	my $Return			= 0;

	# Lock this block.
	$this->Lock;

	# Try to get the new target file path.
	if (($SourceTarget = $this->{'directive-manager'}->GetSourceTarget()) &&
		($ResultTarget = $this->{'directive-manager'}->GenerateResultTarget($SourceTarget)))
	{
		# Get the user information, prime the directive object and fetch the target file.
		my ($UserName, $UserPassword)	= split(/:/, $cgi->cookie(-name=>'AuthCustom_Authenticate'));
        my $DistributionDirective 		= new DistributionDirective($this->{'id'}, $this->{'class'}, $cgi->remote_user, $UserPassword, $this->{'target-uri'}, $this->{'target-port'}, $cgi->param('targetnodes'));
		my $Directive					= new IO::File("> $SourceTarget");
		my $QueuedResult				= new IO::File("> $ResultTarget");

		# If the files were successfully opened.
		if (($Directive) &&
			($QueuedResult))
		{
			# Clean up the all result files.
			$this->{'directive-manager'}->DeleteAllResults($this->{'id'});

			# Spew out the queued result to the target handle.
			print $QueuedResult $this->{'directive-manager'}->GenerateQueuedResult($this->{'id'}, $this->{'class'}, $cgi->param('targetnodes'));
			
			# Close the file.
			$QueuedResult->close;

			# Spew out the file to the target handle.
			print $Directive $DistributionDirective->GetDocument . $this->GeneratePostData($cgi); 

			# Close the file.
			$Directive->close;
			
			# Send a SIGHUP to the Web Content Distributor.
			Safe::backtick('/usr/bin/killall -HUP wcdd');
		}
		else
		{
			# Set the return.
			$Return = 50;
		}
	}
	else
	{
		# Set the return.
		$Return = 50;
	}

	# Unlock this block.
	$this->Unlock;

	return $Return;
}

###############################################################################
# GroupOperations::GeneratePostData
###############################################################################

sub GeneratePostData
{
	my ($this, $cgi)	= @_;
	my @NameList		= $cgi->param;
	my $i				= @NameList; 
	my $PostData;
	my $Name;

	# Create the POST data.
	foreach $Name (@NameList)
	{
		# Only add the pair if it is not 'targetnodes'...
		unless ($Name eq 'targetnodes') 
		{
			# Add on the name/value pair.
			$PostData .= (($i == @NameList) ? '' : '&') . $Name . '=' . escape($cgi->param($Name));
		}

		# One less pairing.
		--$i;
	}

	return $PostData;
}

###############################################################################
# GroupOperations::GetResultContent
###############################################################################

sub GetResultContent
{
	my ($this, $ResultTarget, $ResultStatus, $RootLink)	= @_;
	my $ReportStatus									= '';
	my $Time;

	# Lock this block.
	$this->Lock;

    # Try to get a result target...
	if ($ResultTarget)
	{
		# Stat the file and get the processing status.
		my ($Device, $INode, $Mode, $NLink, $uid, $gid, $RDevice, $Size, $ATime, $MTime, $CTime, $BlockSize, $Blocks) = stat($ResultTarget);
		 
		# If a file time and status could be found...
		if (($MTime) && 
			(defined($ResultStatus)))
		{
			my $_T	= new LanguageTime($this->{'preferred-language'}, '/usr/ins/intel/gui/locale'),
			my %_R	= ( 'PROCESSING'			=> '',
						'COMPLETED'				=> '',
						'COMPLETED_WITH_ERRORS'	=> '',
						'JOB_STATUS'			=> '',
						'DATE_TIME'				=> '',
						'TIME_FORMAT'			=> '' );

			# Load the necessary string resources.
			$this->{'string-resource'}->GetStrings(\%_R);
			
			# Normalize the status.
			SWITCH:
			{
				if (-1 == $ResultStatus) { $ResultStatus = "<font color=#000080>$_R{'PROCESSING'}</font>"; 			last SWITCH; }
				if ( 0 == $ResultStatus) { $ResultStatus = "<font color=green>$_R{'COMPLETED'}</font>"; 			last SWITCH; }
				if ( 1 == $ResultStatus) { $ResultStatus = "<font color=red>$_R{'COMPLETED_WITH_ERRORS'}</font>";	last SWITCH; }
				$ResultStatus = "<font color=#000080>$_R{'PROCESSING'}</font>";
			}

			# Set up the root link...
			$RootLink = "&RL=$RootLink" if(defined($RootLink));

			# See if the link needs to be added or not...
			$ResultStatus = "<a href=\"go_result.cgi?ID=$this->{'id'}$RootLink\">" . $ResultStatus . '</a>';
			
			# Build the status table...
			$Time			= $_T->FormatTime($_R{'TIME_FORMAT'}, localtime($MTime)); 
			$ReportStatus	= <<HERE;
	<table border=0 width=375>
	<tr>
		<td width=50% bgcolor=#808080><b><font color=#ffffff>$_R{'JOB_STATUS'}</font></b></td>
		<td width=50% bgcolor=#808080><b><font color=#ffffff>$_R{'DATE_TIME'}</font></b></td>
	</tr>
	<tr>
		<td width=50%>$ResultStatus</td>
		<td width=50%>$Time</td>
	</tr>
	</table>
HERE
		}
    }

	# Unlock this block.
	$this->Unlock;
	
	return $ReportStatus;
}
       
###############################################################################
# GroupOperations::GetTargetListContent
###############################################################################

sub GetTargetListContent
{
	my $this	= shift;
	my %_R		= ( 'SELECTED_SERVERS'		=> '',
					'SELECT_MORE'			=> '',
					'NO_SERVERS_SELECTED'	=> '' );
	
	
	# Load the necessary string resources.
	$this->{'string-resource'}->GetStrings(\%_R);

	return <<HERE;
	<script>
	function selectopen()
	{
		parent.menu_frame.selectitem("servers_list");
		location.href='/cgi-bin/ema/go_serverlist.cgi';
	}
	</script>
	<table align=left border=0 width=375 cellspacing=0 cellpadding=0>
	<tr>
		<td><b>$_R{'SELECTED_SERVERS'}</b></font></td>
		<td align=right><a href='javascript:selectopen()'>$_R{'SELECT_MORE'}</a></td>
	</tr>
	<tr>
		<td colspan=2>
			<table border=1 width=100% cellspacing=0 cellpadding=2 bordercolor=#eeeeee bgcolor=white>
			<tr>
				<td width=189 align=left nowrap colspan=2>
				<script>
				var mysel = 0;
		
				for (var j = 0; j < parent.menu_frame.machinelist.length; j++)
				{
					if (4 == mysel)
					{	
						document.write("&nbsp;<a href=javascript:selectopen()>...</a>");
						break;
					}
			
					if (1 == parent.menu_frame.machinelist[j].selected)
					{
						if (0 < mysel)
						{
							document.write(";&nbsp;");
						}
						else
						{
							document.write("&nbsp;");
						}
					
						mysel++;	
					
						if (1 == parent.menu_frame.machinelist[j].security)
						{
							document.write("<a href=https://" + parent.menu_frame.machinelist[j].ip + ":5555 target=_blank>" + parent.menu_frame.machinelist[j].name + "</a>");
						}
						else
						{
							document.write("<a href=http://" + parent.menu_frame.machinelist[j].ip + ":5555 target=_blank>" + parent.menu_frame.machinelist[j].name + "</a>");
						}
					}
				}										
		
				if (0 == mysel)
				{
					document.write("&nbsp;<a href=javascript:selectopen()><font color='#FF0000'><b>$_R{'NO_SERVERS_SELECTED'}</b></font></a>");
				}
				</script>                                              
				</td>
			</tr>
			</table>
			<br>
		</td>
	</tr>
	</table>
HERE
}

###############################################################################
# GroupOperations::GetTargetListContentHidden
###############################################################################

sub GetTargetListContentHidden
{
	return <<HERE;
	<script>
	var count = 0;
	
	document.write("<input type=hidden name=targetnodes value=");
	
	for(var j = 0; j < parent.menu_frame.machinelist.length; j++)
	{
		if (1 == parent.menu_frame.machinelist[j].selected)
		{
			if (0 < count)
			{
				document.write(",");
			}
		
			count++;
		
			document.write(parent.menu_frame.machinelist[j].name + ":" + parent.menu_frame.machinelist[j].ip + ":" + parent.menu_frame.machinelist[j].security);
		}
	}										
	
	document.write(">");
	</script>
HERE
}

###############################################################################
# GroupOperations::GetErrorString
###############################################################################

sub GetErrorString
{
	my ($this, $ErrorCode) = @_;
	my $ErrorString;

	# Validate the input parameter(s)...
	if (defined($ErrorCode))
	{
		# Return the error string.
        $ErrorString = $this->{'string-resource'}->_S($ErrorCode);
	}

	return $ErrorString;
}

###############################################################################
# GroupOperations::RenderPageContent
###############################################################################

sub RenderPageContent
{
    my ($this, $PageData)			= @_;
	my $QuickHelp					= new QuickHelp($this->{'preferred-language'}, $this->{'class'}, $this->{'id'});
	my $RootLink					= $PageData->{'root-link'};
	my $CookieCrumbs				= '';
	my $Status						= '';
	my $TargetListContent;
	my $ResultTarget;
	my $ResultStatus				= '';
	my $ResultContent;
	my $QuickHelpTitle;
	my $QuickHelpContent;
	my $html;

	# If there are crumbs...
	if (defined($PageData->{'cookie-crumbs'}))
	{
		my $CookieCrumb;

		# Run through any cookie crumbs...
		for $CookieCrumb (@{ $PageData->{'cookie-crumbs'} })
		{
			# Add on a space if necessary...
			if ($CookieCrumbs ne '')
			{
				$CookieCrumbs .= "&nbsp;";
			}

			# Add in the cookie crumb.
			$CookieCrumbs .= "<b><font face=Arial size=1><u><a href=$CookieCrumb->{'location'}>&lt;&nbsp;$CookieCrumb->{'name'}</a></u>&nbsp;</font></b>";
		}

		# Cap the line.
		$CookieCrumbs .= "\n";
	}
	
	# If there are any results to display...
	if (defined($PageData->{'result'}))
	{
		my $Heading;
		my $FontColor;
		
		# If there is any status information...
		if ((defined($Status = $PageData->{'status'})) &&
			(length($Status)))
		{
			# Append a line break.
			$Status = "<br>$Status";
		}
		else
		{
			$Status = '';
		}

		# If things are good...
		if (0 == $PageData->{'result'})
		{
			# Set the sucessful heading and font color.
			$Heading	= $this->{'string-resource'}->_S('CHANGES_APPLIED');
			$FontColor	= 'green';
		}
		else
		{
			# Set the unsucessfull heading and font color.
			$Heading	= $this->{'string-resource'}->_S('OPERATION_FAILED');
			$FontColor	= 'red';
		}

		# Render the status string.
		$Status = "<b><font color=$FontColor>$Heading $Status</font></b>";

		# Adjust the cookie crumbs...
		$CookieCrumbs = "<br>$CookieCrumbs" if ($CookieCrumbs ne '');
	}

	# Try to get a result target...
	if ($ResultTarget = $this->{'directive-manager'}->GetResultTarget($this->{'id'}))
	{
		# Get the result status.
		$ResultStatus = $this->{'directive-manager'}->GetResultTargetStatus($ResultTarget);
	}

	# Get the target list and the status report content.
	$TargetListContent	= ((defined($PageData->{'display-target-list'})) && (0 == $PageData->{'display-target-list'})) ? '' : $this->GetTargetListContent();
	$ResultContent 		= $this->GetResultContent($ResultTarget, $ResultStatus, $RootLink);

	# Build the root link if needed...
	if (defined($RootLink))
	{
		$RootLink = <<HERE;
	<script>
		selected_item = "$RootLink";
		parent.menu_frame.selectitem("$RootLink");
	</script>
HERE
	}
	else
	{
		# Safe default.
		$RootLink = '';
	}
	
	# Get the quick help information...
	if ((defined($QuickHelpTitle	= $QuickHelp->GetTitle())) &&
		(defined($QuickHelpContent	= $QuickHelp->GetContent())) &&
		(length($QuickHelpTitle)) && (length($QuickHelpContent)))
	{
		$QuickHelpContent = <<HERE;
		<td valign=top width=100%></td> 
		<td valign=top width=177>
			<table align=left border=1 width=60% cellspacing=0 cellpadding=3 bordercolor=#808080>
			<tr bgcolor=gray>
				<td bgcolor=#808080><b><font color=#ffffff>$QuickHelpTitle</font></b></td>
			</tr>
			<tr bgcolor=#ffffcc>
				<td width=160 valign=top align=left>
					$QuickHelpContent
					<p><img width=183 src='/images/blank.gif' height=1></p>
				</td>
			</tr>
			</table>
		</td> 
HERE
	}
	else
	{
		# Safe default.
		$QuickHelpContent = '';
	}
    
	$html = <<HERE;
<!doctype html public "-//IETF//DTD HTML//EN">
<html>
	<head>
		<link rel=stylesheet href=/style/style.css type=text/css>
		<meta name=Copyright content='Copyright (c) 2000 Intel Corporation, et al.'>
	</head>
	<body bgcolor=#ffffff topmargin=5 leftmargin=15>
$RootLink
	<table cellspacing=1 width=100%>
	<tr>
		<td align=left colspan=3>
			$Status
			$CookieCrumbs
			<p class=clsTitlePage>$PageData->{'title'}</p>
			<tr>
				<td align=left colspan=3>
					$TargetListContent
					<tr>
						<td valign=top width=80%>
							$PageData->{'html'}
							$ResultContent
						</td>
						$QuickHelpContent
					</tr>
				</td> 
			</tr>
		</td>
	</tr>		
	</table>
	</body>
</html>
HERE

	print($PageData->{'cgi'}->header());
	print($html);
}

###############################################################################
# GroupOperations::GetPreferredLanguage
###############################################################################

sub GetPreferredLanguage
{
	my $this = shift;

	# Return the requested data.
	return $this->{'preferred-langauge'};
}

###############################################################################
# GroupOperations::SetPreferredLanguage
###############################################################################

sub SetPreferredLanguage
{
	my ($this, $PreferredLanguage) = @_;
	
	# Set the data.
	$this->{'preferred-langauge'} = $PreferredLanguage;
	
	# Update the string resource object.
	$this->{'string-resource'}->SetPreferredLanguage($PreferredLanguage);
}

###############################################################################
# GroupOperations::GetClass
###############################################################################

sub GetClass
{
	my $this = shift;

	# Return the requested data.
	return $this->{'class'};
}

###############################################################################
# GroupOperations::SetClass
###############################################################################

sub SetClass
{
	my ($this, $Class) = @_;
	
	# Set the data.
	$this->{'class'} = $Class;

	# Return the new value.
	return $this->{'class'};
}

###############################################################################
# GroupOperations::GetID
###############################################################################

sub GetID
{
	my $this = shift;

	# Return the requested data.
	return $this->{'id'};
}

###############################################################################
# GroupOperations::SetID
###############################################################################

sub SetID
{
	my ($this, $ID) = @_;
	
	# Set the data.
	$this->{'id'} = $ID;

	# Return the new value.
	return $this->{'id'};
}

###############################################################################
# GroupOperations::GetTargetURI
###############################################################################

sub GetTargetURI
{
	my $this = shift;

	# Return the requested data.
	return $this->{'target-uri'};
}

###############################################################################
# GroupOperations::SetTargetURI
###############################################################################

sub SetTargetURI
{
	my ($this, $TargetURI) = @_;
	
	# Set the data.
	$this->{'target-uri'} = $TargetURI;

	# Return the new value.
	return $this->{'target-uri'};
}

###############################################################################
# GroupOperations::GetTargetPort
###############################################################################

sub GetTargetPort
{
	my $this = shift;

	# Return the requested data.
	return $this->{'target-port'};
}

###############################################################################
# GroupOperations::SetTargetPort
###############################################################################

sub SetTargetPort
{
	my ($this, $TargetPort) = @_;
	
	# Set the data.
	$this->{'target-port'} = $TargetPort;

	# Return the new value.
	return $this->{'target-port'};
}

###############################################################################
# GroupOperations::Lock
###############################################################################

sub Lock
{
	my $this = shift;

	flock($this->{'lock-file'}, 2);
}

###############################################################################
# GroupOperations::Unlock
###############################################################################

sub Unlock
{
	my $this = shift;

	flock($this->{'lock-file'}, 8);
}

1;

__END__
