package LanguageLogCGIElement;

###############################################################################
#                                                                             #
# Filename: LanguageLogCGIElement.pm                                          #
#                                                                             #
# Description: Contains script needed to articulate logged resource data.     #
#                                                                             #
###############################################################################
###############################################################################
#                                                                             #
#  Copyright  2000 - 2001 Intel Corporation.                                 #
#  Intel Corporation All Rights Reserved.                                     #
#                                                                             #
#  The source code contained or described herein and all documents related to #
#  the source code ("Material") are owned by Intel Corporation or its         #
#  suppliers or licensors.  Title to the Material remains with Intel          #
#  Corporation or its suppliers and licensors.  The Material contains trade   #
#  secrets and proprietary and confidential information of Intel or its       #
#  suppliers and licensors.  The Material is protected by worldwide copyright #
#  and trade secret laws and treaty provisions.  No part of the Material may  #
#  be used, copied, reproduced, modified, published, uploaded, posted,        #
#  transmitted, distributed, or disclosed in any way without Intel's prior    #
#  express written permission.                                                #
#                                                                             #
#  No license under any patent, copyright, trade secret or other intellectual #
#  property right is granted to or conferred upon you by disclosure or        #
#  delivery of the Materials, either expressly, by implication, inducement,   #
#  estoppel or otherwise. Any license under such intellectual property rights #
#  must be express and approved by Intel in writing.                          #
#                                                                             #
###############################################################################

# Put the PERL library in the path.
use lib '/usr/ins/intel/gui/perllib';

use strict;
use LanguageResource;

###############################################################################
# LanguageLogCGIElement::new
###############################################################################

sub new
{
	my ($class, $PreferredLanguage, $ElementRaw) = @_;
	
	# Initialize the object.
	my $this = 
	{
		'string-resource'	=> new LanguageResource($PreferredLanguage),
	};

	# If there is data...
	if ((defined($ElementRaw)) && ('' ne $ElementRaw))
	{
		# Fill out the object data.
		($this->{'time-stamp'},
		 $this->{'remote-host'},
		 $this->{'server-name'},
		 $this->{'user-name'},
		 $this->{'source-file'},
		 $this->{'resource-root-directory'},
		 $this->{'resource-suffix-list'},
		 $this->{'category'},
		 $this->{'type'},
		 $this->{'code'},
		 $this->{'parameter-list'}) = split(//, $ElementRaw);

		# Prime the resource suffix list.
		$this->{'resource-suffix-list'} =~ s/^\[(.*?)\]$/$1/;
		$this->{'resource-suffix-list'} = [ split(//, $this->{'resource-suffix-list'}) ];
		
		# If there is a parameter list...
		if (defined($this->{'parameter-list'}))
		{
			# Prime the parameter list.
			$this->{'parameter-list'} =~ s/^\[(.*?)\]$/$1/;
			$this->{'parameter-list'} = [ split(//, $this->{'parameter-list'}) ];
		}

		# Set the root directory.
		$this->{'string-resource'}->SetRootDirectory($this->{'resource-root-directory'});
		undef($this->{'resource-root-directory'});

		# Set the suffix list.
		$this->{'string-resource'}->SetSuffixList(@{ $this->{'resource-suffix-list'} });
		undef($this->{'resource-suffix-list'});
	}
	
	# Align the object with this package.
	bless($this, $class);
	
	# Return the object.
	return $this;
}

###############################################################################
# LanguageLogCGIElement::GetPreferredLanguage
###############################################################################

sub GetPreferredLanguage
{
	my $this = shift;

	# Return the requested data.
	return $this->{'string-resource'}->GetPreferredLanguage();
}

###############################################################################
# LanguageLogCGIElement::SetPreferredLanguage
###############################################################################

sub SetPreferredLanguage
{
	my ($this, $PreferredLanguage) = @_;

	# Apply the change.
	return $this->{'string-resource'}->SetPreferredLanguage($PreferredLanguage);
}

###############################################################################
# LanguageLogCGIElement::GetTimeStamp
###############################################################################

sub GetTimeStamp
{
	my $this = shift;

	# Return the requested data.
	return $this->{'time-stamp'};
}

###############################################################################
# LanguageLogCGIElement::GetRemoteHost
###############################################################################

sub GetRemoteHost
{
	my $this = shift;

	# Return the requested data.
	return $this->{'remote-host'};
}

###############################################################################
# LanguageLogCGIElement::GetServerName
###############################################################################

sub GetServerName
{
	my $this = shift;

	# Return the requested data.
	return $this->{'server-name'};
}

###############################################################################
# LanguageLogCGIElement::GetUserName
###############################################################################

sub GetUserName
{
	my $this = shift;

	# Return the requested data.
	return $this->{'user-name'};
}

###############################################################################
# LanguageLogCGIElement::GetSourceFile
###############################################################################

sub GetSourceFile
{
	my $this = shift;

	# Return the requested data.
	return $this->{'source-file'};
}

###############################################################################
# LanguageLogCGIElement::GetCategory
###############################################################################

sub GetCategory
{
	my $this = shift;

	# Return the requested data.
	return $this->{'category'};
}

###############################################################################
# LanguageLogCGIElement::GetType
###############################################################################

sub GetType
{
	my $this = shift;

	# Return the requested data.
	return $this->{'type'};
}

###############################################################################
# LanguageLogCGIElement::GetMessage
###############################################################################

sub GetMessage
{
	my $this = shift;
	
	# Get the message.
	return $this->{'string-resource'}->_S($this->{'code'}, $this->{'parameter-list'});
}

1;

__END__
