package MonAlertEmail;

###############################################################################
#                                                                             #
# Filename: MonAlertEmail.pm                                                  #
#                                                                             #
# Description: Container class for a monitor e-mail alert directive.          #
#                                                                             #
# Pattern(s): Iterator                                                        #
#                                                                             #
# Parent(s): MonAlert                                                         #
#                                                                             #
# References: IteratorArray                                                   #
#                                                                             #
###############################################################################
###############################################################################
#                                                                             #
#  Copyright  2000 - 2001 Intel Corporation.                                 #
#  Intel Corporation All Rights Reserved.                                     #
#                                                                             #
#  The source code contained or described herein and all documents related to #
#  the source code ("Material") are owned by Intel Corporation or its         #
#  suppliers or licensors.  Title to the Material remains with Intel          #
#  Corporation or its suppliers and licensors.  The Material contains trade   #
#  secrets and proprietary and confidential information of Intel or its       #
#  suppliers and licensors.  The Material is protected by worldwide copyright #
#  and trade secret laws and treaty provisions.  No part of the Material may  #
#  be used, copied, reproduced, modified, published, uploaded, posted,        #
#  transmitted, distributed, or disclosed in any way without Intel's prior    #
#  express written permission.                                                #
#                                                                             #
#  No license under any patent, copyright, trade secret or other intellectual #
#  property right is granted to or conferred upon you by disclosure or        #
#  delivery of the Materials, either expressly, by implication, inducement,   #
#  estoppel or otherwise. Any license under such intellectual property rights #
#  must be express and approved by Intel in writing.                          #
#                                                                             #
###############################################################################

# Put the PERL library in the path.
use lib '/usr/ins/intel/gui/perllib';

use strict;
use vars qw(@ISA);
use MonAlert;
use IteratorArray;

# Base class.
@ISA = qw(MonAlert);

###############################################################################
# MonAlertEmail::new
###############################################################################

sub new
{
	my ($class, $Type, $Arguments) = @_;

	# Initialize the object.
	my $this = new MonAlert($Type, 'mail.alert');
	
	$this->{'address-list'} = [ ];

	# Align the object with this package.
    bless($this, $class);

	# Set the arguments.
	$this->setArguments($Arguments);

	# Return the object.
	return $this;
}

###############################################################################
# MonAlert::getArguments
###############################################################################

sub getArguments
{
	my $this = shift;
	my $Arguments;
	
	# If there are arguments...
	if ((defined($this->{'address-list'})) &&
		(scalar(@{ $this->{'address-list'} })))
	{
		# Create the argument string.
		$Arguments = join(' ', @{ $this->{'address-list'} });
	}

	# Return the requested data.
	return $Arguments;
}

###############################################################################
# MonAlert::setArguments
###############################################################################

sub setArguments
{
	my ($this, $Arguments) = @_;
	
	# Set the data.
	@{ $this->{'address-list'} } = split(/\s+/, $Arguments);

	return 0;
}

###############################################################################
# MonAlertEmail::addEmailAddress
###############################################################################

sub addEmailAddress
{
	my ($this, $EmailAddress) = @_;
	my $Return = 0;
    my $i;
	
	# Validate the input parameter...
	if (defined($EmailAddress))
	{
		# Run through the data...
		for $i ( 0 .. $#{ $this->{'address-list'} } ) 
		{
			# If the element matches...
			if ($this->{'address-list'}[$i] eq $EmailAddress)
			{
				# Clear out the cache.
				undef($EmailAddress);
	
				# Bail out.
				last;
			}
		}
		
		# If the cache is available...
		if (defined($EmailAddress))
		{
			# Cache the object reference.
			push(@{ $this->{'address-list'} }, $EmailAddress);
		}
	}
	else
	{
		$Return = 20006001;
	}

	return $Return;
}

###############################################################################
# MonAlertEmail::deleteEmailAddress
###############################################################################

sub deleteEmailAddress
{
	my ($this, $EmailAddress) = @_;
	my $Return = 0;
    my $i;
	
	# Validate the input parameter...
	if (defined($EmailAddress))
	{
		# Run through the data...
		for $i ( 0 .. $#{ $this->{'address-list'} } ) 
		{
			# If the element matches...
			if ($this->{'address-list'}[$i] eq $EmailAddress)
			{
				# Delete the line.
				splice(@{ $this->{'address-list'} }, $i, 1);
	
				# Bail out.
				last;
			}
		}
	}
	else
	{
		$Return = 20006002;
	}

	return $Return;
}

###############################################################################
# MonAlertEmail::iteratorEmailAddresses
###############################################################################

sub iteratorEmailAddresses
{
	my $this = shift;
	my $Iterator;

	# If there is an alert list...
	if ((defined($this->{'address-list'})) &&
		(scalar(@{ $this->{'address-list'} })))
	{
		# Initialize the iterator object...
		$Iterator = new IteratorArray($this->{'address-list'});
	}

	return $Iterator;
}

1;

__END__

