package MonConfiguration;

###############################################################################
#                                                                             #
# Filename: MonConfiguration.pm                                               #
#                                                                             #
# Description: Container class for the monitor configuration.                 #
#                                                                             #
# Pattern(s): Strategy, Iterator                                              #
#                                                                             #
# Aggregates: MonWatch, MonHostGroup, MonSerializer                           #
#                                                                             #
# References: IteratorArray, IteratorHash                                     #
#                                                                             #
###############################################################################
###############################################################################
#                                                                             #
#  Copyright  2000 - 2001 Intel Corporation.                                 #
#  Intel Corporation All Rights Reserved.                                     #
#                                                                             #
#  The source code contained or described herein and all documents related to #
#  the source code ("Material") are owned by Intel Corporation or its         #
#  suppliers or licensors.  Title to the Material remains with Intel          #
#  Corporation or its suppliers and licensors.  The Material contains trade   #
#  secrets and proprietary and confidential information of Intel or its       #
#  suppliers and licensors.  The Material is protected by worldwide copyright #
#  and trade secret laws and treaty provisions.  No part of the Material may  #
#  be used, copied, reproduced, modified, published, uploaded, posted,        #
#  transmitted, distributed, or disclosed in any way without Intel's prior    #
#  express written permission.                                                #
#                                                                             #
#  No license under any patent, copyright, trade secret or other intellectual #
#  property right is granted to or conferred upon you by disclosure or        #
#  delivery of the Materials, either expressly, by implication, inducement,   #
#  estoppel or otherwise. Any license under such intellectual property rights #
#  must be express and approved by Intel in writing.                          #
#                                                                             #
###############################################################################

# Put the PERL library in the path.
use lib '/usr/ins/intel/gui/perllib';

use strict;
use MonWatch;
use MonHostGroup;
use MonSerializer;
use IteratorArray;
use IteratorHash;

###############################################################################
# MonConfiguration::new
###############################################################################

sub new
{
	my ($class, $Serializer) = @_;

	# Initialize the object.
	my $this = 
	{
		'status'						=> 0,
		'serializer'					=> undef,
		'directory-alert'				=> undef,
		'directory-monitor'				=> undef,
		'directory-state'				=> undef,
		'directory-log'					=> undef,
		'directory-base'				=> undef,
		'directory-configuration'		=> undef,
		'authorization-file'			=> undef,
		'authorization-type'			=> undef,
		'user-file'						=> undef,
		'snmp-port'						=> undef,
		'snmp-state'					=> 0,
		'bind-address-server'			=> undef,
		'bind-address-trap'				=> undef,
		'downtime-log'					=> undef,
		'downtime-state'				=> 0,
		'history-log'					=> undef,
		'history-limit'					=> undef,
		'history-length'				=> undef,
		'port-server'					=> undef,
		'port-trap'						=> undef,
		'pid-file'						=> undef,
		'process-limit'					=> undef,
		'client-inactivity-timeout'		=> undef,
		'random-start'					=> undef,
		'dependency-recursion-limit'	=> undef,
		'dependency-behavior'			=> undef,
		'syslog-facility'				=> undef,
		'enable-reset-startup-alerts'	=> 0,
		'host-group-hash'				=> { },
		'watch-list'					=> [ ],
	};

	# Align the object with this package.
    bless($this, $class);

	# Try to set the serializer.
	$this->setSerializer($Serializer);
	
	# Return the object.
	return $this;
}

###############################################################################
# MonConfiguration::open
###############################################################################

sub open
{
	my $this	= shift;
	my $Return	= 0;

	# If the object is open...
	unless ($this->{'status'})
	{
		# If there is a serializer...
		if ($this->{'serializer'})
		{
			# Load the data.
			if (0 == ($Return = $this->{'serializer'}->load()))
			{
				# The object is open.
				$this->{'status'} = 1;
			}
		}
	}

	return $Return;
}

###############################################################################
# MonConfiguration::close
###############################################################################

sub close
{
	my $this	= shift;
	my $Return	= 0;

	# If the object is open...
	if (($this->{'status'}) &&
		($this->{'serializer'}))
	{
		# Save the data.
		if (0 == ($Return = $this->{'serializer'}->save()))
		{
			# The object is closed.
			$this->{'status'} = 0;
		}
	}

	return $Return;
}

###############################################################################
# MonConfiguration::getSerializer
###############################################################################

sub getSerializer
{
	my $this = shift;
	
	# Return the requested data.
	return $this->{'serializer'};
}

###############################################################################
# MonConfiguration::setSerializer
###############################################################################

sub setSerializer
{
	my ($this, $Serializer) = @_;
	my $Return	= 0;
	
	# Validate the type...
	if ($Serializer->isa('MonSerializer'))
	{
		# Set the host.
		$Serializer->setHost($this);

		# Cache the object reference.
		$this->{'serializer'} = $Serializer;
	}
	else
	{
		$Return = 20005001;
	}

	return $Return;
}

###############################################################################
# MonConfiguration::getAlertLibraryDirectory
###############################################################################

sub getAlertLibraryDirectory
{
	my $this = shift;
	
	# Return the requested data.
	return $this->{'directory-alert'};
}

###############################################################################
# MonConfiguration::setAlertLibraryDirectory
###############################################################################

sub setAlertLibraryDirectory
{
	my ($this, $AlertLibraryDirectory) = @_;
	
	# Set the data.
	$this->{'directory-alert'} = $AlertLibraryDirectory;

	return 0;
}

###############################################################################
# MonConfiguration::getMonitorLibraryDirectory
###############################################################################

sub getMonitorLibraryDirectory
{
	my $this = shift;
	
	# Return the requested data.
	return $this->{'directory-monitor'};
}

###############################################################################
# MonConfiguration::setMonitorLibraryDirectory
###############################################################################

sub setMonitorLibraryDirectory
{
	my ($this, $MonitorLibraryDirectory) = @_;
	
	# Set the data.
	$this->{'directory-monitor'} = $MonitorLibraryDirectory;

	return 0;
}

###############################################################################
# MonConfiguration::getProcessStateDirectory
###############################################################################

sub getProcessStateDirectory
{
	my $this = shift;
	
	# Return the requested data.
	return $this->{'directory-state'};
}

###############################################################################
# MonConfiguration::setProcessStateDirectory
###############################################################################

sub setProcessStateDirectory
{
	my ($this, $ProcessStateDirectory) = @_;
	
	# Set the data.
	$this->{'directory-state'} = $ProcessStateDirectory;

	return 0;
}

###############################################################################
# MonConfiguration::getLogFileDirectory
###############################################################################

sub getLogFileDirectory
{
	my $this = shift;
	
	# Return the requested data.
	return $this->{'directory-log'};
}

###############################################################################
# MonConfiguration::setLogFileDirectory
###############################################################################

sub setLogFileDirectory
{
	my ($this, $LogFileDirectory) = @_;
	
	# Set the data.
	$this->{'directory-log'} = $LogFileDirectory;

	return 0;
}

###############################################################################
# MonConfiguration::getCommonBaseDirectory
###############################################################################

sub getCommonBaseDirectory
{
	my $this = shift;
	
	# Return the requested data.
	return $this->{'directory-base'};
}

###############################################################################
# MonConfiguration::setCommonBaseDirectory
###############################################################################

sub setCommonBaseDirectory
{
	my ($this, $CommonBaseDirectory) = @_;
	
	# Set the data.
	$this->{'directory-base'} = $CommonBaseDirectory;

	return 0;
}

###############################################################################
# MonConfiguration::getConfigurationDirectory
###############################################################################

sub getConfigurationDirectory
{
	my $this = shift;
	
	# Return the requested data.
	return $this->{'directory-configuration'};
}

###############################################################################
# MonConfiguration::setConfigurationDirectory
###############################################################################

sub setConfigurationDirectory
{
	my ($this, $ConfigurationDirectory) = @_;
	
	# Set the data.
	$this->{'directory-configuration'} = $ConfigurationDirectory;

	return 0;
}

###############################################################################
# MonConfiguration::getAuthorizationFile
###############################################################################

sub getAuthorizationFile
{
	my $this = shift;
	
	# Return the requested data.
	return $this->{'authorization-file'};
}

###############################################################################
# MonConfiguration::setAuthorizationFile
###############################################################################

sub setAuthorizationFile
{
	my ($this, $AuthorizationFile) = @_;
	
	# Set the data.
	$this->{'authorization-file'} = $AuthorizationFile;

	return 0;
}

###############################################################################
# MonConfiguration::getAuthorizationType
###############################################################################

sub getAuthorizationType
{
	my $this = shift;
	
	# Return the requested data.
	return $this->{'authorization-type'};
}

###############################################################################
# MonConfiguration::setAuthorizationType
###############################################################################

sub setAuthorizationType
{
	my ($this, $AuthorizationType) = @_;
	
	# Set the data.
	$this->{'authorization-type'} = $AuthorizationType;

	return 0;
}

###############################################################################
# MonConfiguration::getUserFile
###############################################################################

sub getUserFile
{
	my $this = shift;
	
	# Return the requested data.
	return $this->{'user-file'};
}

###############################################################################
# MonConfiguration::setUserFile
###############################################################################

sub setUserFile
{
	my ($this, $UserFile) = @_;
	
	# Set the data.
	$this->{'user-file'} = $UserFile;

	return 0;
}

###############################################################################
# MonConfiguration::getSNMPPort
###############################################################################

sub getSNMPPort
{
	my $this = shift;
	
	# Return the requested data.
	return $this->{'snmp-port'};
}

###############################################################################
# MonConfiguration::setSNMPPort
###############################################################################

sub setSNMPPort
{
	my ($this, $SNMPPort) = @_;
	
	# Set the data.
	$this->{'snmp-port'} = $SNMPPort;

	return 0;
}

###############################################################################
# MonConfiguration::isSNMPEnabled
###############################################################################

sub isSNMPEnabled
{
	my $this = shift;
	
	# Return the requested data.
	return $this->{'snmp-state'};
}

###############################################################################
# MonConfiguration::setSNMPState
###############################################################################

sub setSNMPState
{
	my ($this, $SNMPState) = @_;
	
	# Set the data.
	$this->{'snmp-state'} = $SNMPState;

	return 0;
}

###############################################################################
# MonConfiguration::getServerBindAddress
###############################################################################

sub getServerBindAddress
{
	my $this = shift;
	
	# Return the requested data.
	return $this->{'bind-address-server'};
}

###############################################################################
# MonConfiguration::setServerBindAddress
###############################################################################

sub setServerBindAddress
{
	my ($this, $ServerBindAddress) = @_;
	
	# Set the data.
	$this->{'bind-address-server'} = $ServerBindAddress;

	return 0;
}

###############################################################################
# MonConfiguration::getTrapBindAddress
###############################################################################

sub getTrapBindAddress
{
	my $this = shift;
	
	# Return the requested data.
	return $this->{'bind-address-trap'};
}

###############################################################################
# MonConfiguration::setTrapBindAddress
###############################################################################

sub setTrapBindAddress
{
	my ($this, $TrapBindAddress) = @_;
	
	# Set the data.
	$this->{'bind-address-trap'} = $TrapBindAddress;

	return 0;
}

###############################################################################
# MonConfiguration::getDowntimeLogFile
###############################################################################

sub getDowntimeLogFile
{
	my $this = shift;
	
	# Return the requested data.
	return $this->{'downtime-log'};
}

###############################################################################
# MonConfiguration::setDowntimeLogFile
###############################################################################

sub setDowntimeLogFile
{
	my ($this, $DowntimeLogFile) = @_;
	
	# Set the data.
	$this->{'downtime-log'} = $DowntimeLogFile;

	return 0;
}

###############################################################################
# MonConfiguration::isDowntimeLoggingEnabled
###############################################################################

sub isDowntimeLoggingEnabled
{
	my $this = shift;
	
	# Return the requested data.
	return $this->{'downtime-state'};
}

###############################################################################
# MonConfiguration::setDowntimeLoggingState
###############################################################################

sub setDowntimeLoggingState
{
	my ($this, $DowntimeLoggingState) = @_;
	
	# Set the data.
	$this->{'downtime-state'} = $DowntimeLoggingState;

	return 0;
}

###############################################################################
# MonConfiguration::getHistoryLogFile
###############################################################################

sub getHistoryLogFile
{
	my $this = shift;
	
	# Return the requested data.
	return $this->{'history-log'};
}

###############################################################################
# MonConfiguration::setHistoryLogFile
###############################################################################

sub setHistoryLogFile
{
	my ($this, $HistoryLogFile) = @_;
	
	# Set the data.
	$this->{'history-log'} = $HistoryLogFile;

	return 0;
}

###############################################################################
# MonConfiguration::getHistoryTimeLimit
###############################################################################

sub getHistoryTimeLimit
{
	my $this = shift;
	
	# Return the requested data.
	return $this->{'history-limit'};
}

###############################################################################
# MonConfiguration::setHistoryTimeLimit
###############################################################################

sub setHistoryTimeLimit
{
	my ($this, $HistoryTimeLimit) = @_;
	
	# Set the data.
	$this->{'history-limit'} = $HistoryTimeLimit;

	return 0;
}

###############################################################################
# MonConfiguration::getHistoryLength
###############################################################################

sub getHistoryLength
{
	my $this = shift;
	
	# Return the requested data.
	return $this->{'history-length'};
}

###############################################################################
# MonConfiguration::setHistoryLength
###############################################################################

sub setHistoryLength
{
	my ($this, $HistoryLength) = @_;
	
	# Set the data.
	$this->{'history-length'} = $HistoryLength;

	return 0;
}

###############################################################################
# MonConfiguration::getServerPort
###############################################################################

sub getServerPort
{
	my $this = shift;
	
	# Return the requested data.
	return $this->{'port-server'};
}

###############################################################################
# MonConfiguration::setServerPort
###############################################################################

sub setServerPort
{
	my ($this, $ServerPort) = @_;
	
	# Set the data.
	$this->{'port-server'} = $ServerPort;

	return 0;
}

###############################################################################
# MonConfiguration::getTrapPort
###############################################################################

sub getTrapPort
{
	my $this = shift;
	
	# Return the requested data.
	return $this->{'port-trap'};
}

###############################################################################
# MonConfiguration::setTrapPort
###############################################################################

sub setTrapPort
{
	my ($this, $TrapPort) = @_;
	
	# Set the data.
	$this->{'port-trap'} = $TrapPort;

	return 0;
}

###############################################################################
# MonConfiguration::getProcessIDFile
###############################################################################

sub getProcessIDFile
{
	my $this = shift;
	
	# Return the requested data.
	return $this->{'pid-file'};
}

###############################################################################
# MonConfiguration::setProcessIDFile
###############################################################################

sub setProcessIDFile
{
	my ($this, $ProcessIDFile) = @_;
	
	# Set the data.
	$this->{'pid-file'} = $ProcessIDFile;

	return 0;
}

###############################################################################
# MonConfiguration::getChildProcessLimit
###############################################################################

sub getChildProcessLimit
{
	my $this = shift;
	
	# Return the requested data.
	return $this->{'process-limit'};
}

###############################################################################
# MonConfiguration::setChildProcessLimit
###############################################################################

sub setChildProcessLimit
{
	my ($this, $ChildProcessLimit) = @_;
	
	# Set the data.
	$this->{'process-limit'} = $ChildProcessLimit;

	return 0;
}

###############################################################################
# MonConfiguration::getClientInactivityTimeout
###############################################################################

sub getClientInactivityTimeout
{
	my $this = shift;
	
	# Return the requested data.
	return $this->{'client-inactivity-timeout'};
}

###############################################################################
# MonConfiguration::setClientInactivityTimeout
###############################################################################

sub setClientInactivityTimeout
{
	my ($this, $ClientInactivityTimeout) = @_;
	
	# Set the data.
	$this->{'client-inactivity-timeout'} = $ClientInactivityTimeout;

	return 0;
}

###############################################################################
# MonConfiguration::getRandomStartSkew
###############################################################################

sub getRandomStartSkew
{
	my $this = shift;
	
	# Return the requested data.
	return $this->{'random-start'};
}

###############################################################################
# MonConfiguration::setRandomStartSkew
###############################################################################

sub setRandomStartSkew
{
	my ($this, $RandomStartSkew) = @_;
	
	# Set the data.
	$this->{'random-start'} = $RandomStartSkew;

	return 0;
}

###############################################################################
# MonConfiguration::getDependencyRecursionLimit
###############################################################################

sub getDependencyRecursionLimit
{
	my $this = shift;
	
	# Return the requested data.
	return $this->{'dependency-recursion-limit'};
}

###############################################################################
# MonConfiguration::setDependencyRecursionLimit
###############################################################################

sub setDependencyRecursionLimit
{
	my ($this, $DependencyRecursionLimit) = @_;
	
	# Set the data.
	$this->{'dependency-recursion-limit'} = $DependencyRecursionLimit;

	return 0;
}

###############################################################################
# MonConfiguration::getDependencyBehavior
###############################################################################

sub getDependencyBehavior
{
	my $this = shift;
	
	# Return the requested data.
	return $this->{'dependency-behavior'};
}

###############################################################################
# MonConfiguration::setDependencyBehavior
###############################################################################

sub setDependencyBehavior
{
	my ($this, $DependencyBehavior) = @_;
	
	# Set the data.
	$this->{'dependency-behavior'} = $DependencyBehavior;

	return 0;
}

###############################################################################
# MonConfiguration::getSyslogFacility
###############################################################################

sub getSyslogFacility
{
	my $this = shift;
	
	# Return the requested data.
	return $this->{'syslog-facility'};
}

###############################################################################
# MonConfiguration::setSyslogFacility
###############################################################################

sub setSyslogFacility
{
	my ($this, $SyslogFacility) = @_;
	
	# Set the data.
	$this->{'syslog-facility'} = $SyslogFacility;

	return 0;
}

###############################################################################
# MonConfiguration::areStartupAlertsAtResetEnabled
###############################################################################

sub areStartupAlertsAtResetEnabled
{
	my $this = shift;
	
	# Return the requested data.
	return $this->{'enable-reset-startup-alerts'};
}

###############################################################################
# MonConfiguration::setStartupAlertsAtResetState
###############################################################################

sub setStartupAlertsAtResetState
{
	my ($this, $StartupAlertsAtResetState) = @_;
	
	# Set the data.
	$this->{'enable-reset-startup-alerts'} = $StartupAlertsAtResetState;

	return 0;
}

###############################################################################
# MonConfiguration::getHostGroup
###############################################################################

sub addHostGroup
{
	my ($this, $HostGroup) = @_;
	my $Return	= 0;
	
	# Validate the type...
	if ($HostGroup->isa('MonHostGroup'))
	{
		my $HostGroupName = $HostGroup->getName();

		# If the object has a name...
		if ($HostGroupName)
		{
			# Cache the object reference.
			$this->{'host-group-hash'}->{$HostGroupName} = $HostGroup;
		}
	}
	else
	{
		$Return = 20005002;
	}

	return $Return;
}

###############################################################################
# MonConfiguration::getHostGroup
###############################################################################

sub deleteHostGroup
{
	my ($this, $HostGroup) = @_;
	my $Return	= 0;
	
	# Validate the type...
	if ($HostGroup->isa('MonHostGroup'))
	{
		my $HostGroupName = $HostGroup->getName();

		# If the object has a name...
		if ($HostGroupName)
		{
			# Cache the object reference.
			delete($this->{'host-group-hash'}->{$HostGroupName});
		}
	}
	else
	{
		$Return = 20005003;
	}

	return $Return;
}

###############################################################################
# MonConfiguration::iteratorHostGroups
###############################################################################

sub iteratorHostGroups
{
	my $this = shift;
	my $Iterator;

	# If there is an alert list...
	if ((defined($this->{'host-group-hash'})) &&
		(0 < keys(%{ $this->{'host-group-hash'} })))
	{
		# Initialize the iterator object...
		$Iterator = new IteratorHash($this->{'host-group-hash'});
	}

	return $Iterator;
}

###############################################################################
# MonConfiguration::addWatch
###############################################################################

sub addWatch
{
	my ($this, $Watch) = @_;
	my $Return	= 0;
	
	# Validate the type...
	if ($Watch->isa('MonWatch'))
	{
		# Cache the object reference.
		push(@{ $this->{'watch-list'} }, $Watch);
	}
	else
	{
		$Return = 20005004;
	}

	return $Return;
}

###############################################################################
# MonConfiguration::iteratorWatches
###############################################################################

sub iteratorWatches
{
	my $this = shift;
	my $Iterator;

	# If there is an alert list...
	if ((defined($this->{'watch-list'})) &&
		(scalar(@{ $this->{'watch-list'} })))
	{
		# Initialize the iterator object...
		$Iterator = new IteratorArray($this->{'watch-list'});
	}

	return $Iterator;
}

1;

__END__
