package MonHostGroup;

###############################################################################
#                                                                             #
# Filename: MonHostGroup.pm                                                   #
#                                                                             #
# Description: Container class for a monitor host group directive.            #
#                                                                             #
# Pattern(s): Iterator                                                        #
#                                                                             #
# References: IteratorArray                                                   #
#                                                                             #
###############################################################################
###############################################################################
#                                                                             #
#  Copyright  2000 - 2001 Intel Corporation.                                 #
#  Intel Corporation All Rights Reserved.                                     #
#                                                                             #
#  The source code contained or described herein and all documents related to #
#  the source code ("Material") are owned by Intel Corporation or its         #
#  suppliers or licensors.  Title to the Material remains with Intel          #
#  Corporation or its suppliers and licensors.  The Material contains trade   #
#  secrets and proprietary and confidential information of Intel or its       #
#  suppliers and licensors.  The Material is protected by worldwide copyright #
#  and trade secret laws and treaty provisions.  No part of the Material may  #
#  be used, copied, reproduced, modified, published, uploaded, posted,        #
#  transmitted, distributed, or disclosed in any way without Intel's prior    #
#  express written permission.                                                #
#                                                                             #
#  No license under any patent, copyright, trade secret or other intellectual #
#  property right is granted to or conferred upon you by disclosure or        #
#  delivery of the Materials, either expressly, by implication, inducement,   #
#  estoppel or otherwise. Any license under such intellectual property rights #
#  must be express and approved by Intel in writing.                          #
#                                                                             #
###############################################################################

# Put the PERL library in the path.
use lib '/usr/ins/intel/gui/perllib';

use strict;
use IteratorArray;

###############################################################################
# MonHostGroup::new
###############################################################################

sub new
{
	my ($class, $Name) = @_;

	# Initialize the object.
	my $this = 
	{
		'name'		=> $Name,
		'host-list'	=> [ ],
	};

	# Align the object with this package.
    bless($this, $class);

	# Return the object.
	return $this;
}

###############################################################################
# MonHostGroup::getName
###############################################################################

sub getName
{
	my $this = shift;
	
	# Return the requested data.
	return $this->{'name'};
}

###############################################################################
# MonHostGroup::setName
###############################################################################

sub setName
{
	my ($this, $Name) = @_;
	
	# Set the data.
	$this->{'name'} = $Name;

	return 0;
}

###############################################################################
# MonHostGroup::addHost
###############################################################################

sub addHost
{
	my ($this, $Host) = @_;
	
	# Cache the object reference.
	push(@{ $this->{'host-list'} }, $Host);

	return 0;
}

###############################################################################
# MonHostGroup::deleteHost
###############################################################################

sub deleteHost
{
	my ($this, $Host) = @_;
	my $i;
	
	# Run through the data...
	for $i ( 0 .. $#{ $this->{'host-list'} } ) 
	{
		# If the element matches...
		if ($this->{'host-list'}[$i] eq $Host)
		{
			# Delete the line.
			splice(@{ $this->{'host-list'} }, $i, 1);

			# Bail out.
			last;
		}
	}
	
	return 0;
}

###############################################################################
# MonHostGroup::iteratorHosts
###############################################################################

sub iteratorHosts
{
	my $this = shift;
	my $Iterator;

	# If there is an alert list...
	if ((defined($this->{'host-list'})) &&
		(scalar(@{ $this->{'host-list'} })))
	{
		# Initialize the iterator object...
		$Iterator = new IteratorArray($this->{'host-list'});
	}

	return $Iterator;
}

1;

__END__
