package MonMonitor;

###############################################################################
#                                                                             #
# Filename: MonMonitor.pm                                                     #
#                                                                             #
# Description: Container class for a monitor directive.                       #
#                                                                             #
###############################################################################
###############################################################################
#                                                                             #
#  Copyright  2000 - 2001 Intel Corporation.                                 #
#  Intel Corporation All Rights Reserved.                                     #
#                                                                             #
#  The source code contained or described herein and all documents related to #
#  the source code ("Material") are owned by Intel Corporation or its         #
#  suppliers or licensors.  Title to the Material remains with Intel          #
#  Corporation or its suppliers and licensors.  The Material contains trade   #
#  secrets and proprietary and confidential information of Intel or its       #
#  suppliers and licensors.  The Material is protected by worldwide copyright #
#  and trade secret laws and treaty provisions.  No part of the Material may  #
#  be used, copied, reproduced, modified, published, uploaded, posted,        #
#  transmitted, distributed, or disclosed in any way without Intel's prior    #
#  express written permission.                                                #
#                                                                             #
#  No license under any patent, copyright, trade secret or other intellectual #
#  property right is granted to or conferred upon you by disclosure or        #
#  delivery of the Materials, either expressly, by implication, inducement,   #
#  estoppel or otherwise. Any license under such intellectual property rights #
#  must be express and approved by Intel in writing.                          #
#                                                                             #
###############################################################################

use strict;

###############################################################################
# MonMonitor::new
###############################################################################

sub new
{
	my ($class, $Monitor, $Arguments) = @_;

	# Initialize the object.
	my $this = 
	{
		'monitor'	=> undef,
		'arguments'	=> undef,
	};

	# Align the object with this package.
    bless($this, $class);

	# Prime the object.
	$this->setMonitor($Monitor);
	$this->setArguments($Arguments);

	# Return the object.
	return $this;
}

###############################################################################
# MonMonitor::getMonitor
###############################################################################

sub getMonitor
{
	my $this = shift;
	
	# Return the requested data.
	return $this->{'monitor'};
}

###############################################################################
# MonMonitor::setMonitor
###############################################################################

sub setMonitor
{
	my ($this, $Monitor) = @_;
	
	$Monitor = '' unless(defined($Monitor));
	
	# Set the data.
	$this->{'monitor'} = $Monitor;

	return 0;
}

###############################################################################
# MonMonitor::getArguments
###############################################################################

sub getArguments
{
	my $this = shift;
	
	# Return the requested data.
	return $this->{'arguments'};
}

###############################################################################
# MonMonitor::setArguments
###############################################################################

sub setArguments
{
	my ($this, $Arguments) = @_;
	
	# Safe default.
	$Arguments = '' unless(defined($Arguments));
	
	# Set the data.
	$this->{'arguments'} = $Arguments;

	return 0;
}

###############################################################################
# MonMonitor::usesHostGroup
###############################################################################

sub usesHostGroup
{
	my $this = shift;
	
	# Return the requested data.
	return (($this->{'arguments'} =~ /^.*;;$/) ? 0 : 1);
}

###############################################################################
# MonMonitor::setHostGroupState
###############################################################################

sub setHostGroupState
{
	my ($this, $HostGroupState) = @_;
	
	# If using host groups...
	if ($HostGroupState)
	{
		# If not using host group state...
		unless ($this->usesHostGroup())
		{
			# Remove the restrictors.
			$this->{'arguments'} =~ s/^(.*);;$/$1/;
		}
	}
	else
	{
		# If not using host group state...
		if ($this->usesHostGroup())
		{
			# Add on the restrictors.
			$this->{'arguments'} .= ' ' if(length($this->{'arguments'}));
			$this->{'arguments'} .= ';;';
		}
	}

	return 0;
}

1;

__END__
