package MonService;

###############################################################################
#                                                                             #
# Filename: MonService.pm                                                     #
#                                                                             #
# Description: Container class for a monitor service directive.               #
#                                                                             #
# Pattern(s): Iterator                                                        #
#                                                                             #
# Aggregates: MonPeriod                                                       #
#                                                                             #
# References: IteratorArray                                                   #
#                                                                             #
###############################################################################
###############################################################################
#                                                                             #
#  Copyright  2000 - 2001 Intel Corporation.                                 #
#  Intel Corporation All Rights Reserved.                                     #
#                                                                             #
#  The source code contained or described herein and all documents related to #
#  the source code ("Material") are owned by Intel Corporation or its         #
#  suppliers or licensors.  Title to the Material remains with Intel          #
#  Corporation or its suppliers and licensors.  The Material contains trade   #
#  secrets and proprietary and confidential information of Intel or its       #
#  suppliers and licensors.  The Material is protected by worldwide copyright #
#  and trade secret laws and treaty provisions.  No part of the Material may  #
#  be used, copied, reproduced, modified, published, uploaded, posted,        #
#  transmitted, distributed, or disclosed in any way without Intel's prior    #
#  express written permission.                                                #
#                                                                             #
#  No license under any patent, copyright, trade secret or other intellectual #
#  property right is granted to or conferred upon you by disclosure or        #
#  delivery of the Materials, either expressly, by implication, inducement,   #
#  estoppel or otherwise. Any license under such intellectual property rights #
#  must be express and approved by Intel in writing.                          #
#                                                                             #
###############################################################################

# Put the PERL library in the path.
use lib '/usr/ins/intel/gui/perllib';

use strict;
use MonPeriod;
use MonMonitor;
use IteratorArray;

###############################################################################
# MonService::new
###############################################################################

sub new
{
	my ($class, $Name) = @_;

	# Initialize the object.
	my $this = 
	{
		'name'					=> $Name,
		'interval'				=> undef,
		'interval-skew'			=> undef,
		'trap-timeout'			=> undef,
		'trap-duration'			=> undef,
		'monitor'				=> undef,
		'empty-group-allowed'	=> 0,
		'description'			=> undef,
		'excluded-host-list'	=> [ ],
		'exclusion-period'		=> undef,
		'dependency'			=> undef,
		'dependency-behavior'	=> undef,
		'period-list'			=> [ ],
	};

	# Align the object with this package.
    bless($this, $class);

	# Return the object.
	return $this;
}

###############################################################################
# MonService::getName
###############################################################################

sub getName
{
	my $this = shift;
	
	# Return the requested data.
	return $this->{'name'};
}

###############################################################################
# MonService::setName
###############################################################################

sub setName
{
	my ($this, $Name) = @_;
	
	# Set the data.
	$this->{'name'} = $Name;

	return 0;
}

###############################################################################
# MonService::getMonitorInterval
###############################################################################

sub getMonitorInterval
{
	my $this = shift;
	
	# Return the requested data.
	return $this->{'interval'};
}

###############################################################################
# MonService::setMonitorInterval
###############################################################################

sub setMonitorInterval
{
	my ($this, $Interval) = @_;
	
	# Set the data.
	$this->{'interval'} = $Interval;

	return 0;
}

###############################################################################
# MonService::getMonitorIntervalSkew
###############################################################################

sub getMonitorIntervalSkew
{
	my $this = shift;
	
	# Return the requested data.
	return $this->{'interval-skew'};
}

###############################################################################
# MonService::setMonitorIntervalSkew
###############################################################################

sub setMonitorIntervalSkew
{
	my ($this, $MonitorIntervalSkew) = @_;
	
	# Set the data.
	$this->{'interval-skew'} = $MonitorIntervalSkew;

	return 0;
}

###############################################################################
# MonService::getMonitorTrapTimeout
###############################################################################

sub getMonitorTrapTimeout
{
	my $this = shift;
	
	# Return the requested data.
	return $this->{'trap-timeout'};
}

###############################################################################
# MonService::setMonitorTrapTimeout
###############################################################################

sub setMonitorTrapTimeout
{
	my ($this, $MonitorTrapTimeout) = @_;
	
	# Set the data.
	$this->{'trap-timeout'} = $MonitorTrapTimeout;

	return 0;
}

###############################################################################
# MonService::getMonitorTrapDuration
###############################################################################

sub getMonitorTrapDuration
{
	my $this = shift;
	
	# Return the requested data.
	return $this->{'trap-duration'};
}

###############################################################################
# MonService::setMonitorTrapDuration
###############################################################################

sub setMonitorTrapDuration
{
	my ($this, $MonitorTrapDuration) = @_;
	
	# Set the data.
	$this->{'trap-duration'} = $MonitorTrapDuration;

	return 0;
}

###############################################################################
# MonService::getMonitor
###############################################################################

sub getMonitor
{
	my $this = shift;
	
	# Return the requested data.
	return $this->{'monitor'};
}

###############################################################################
# MonService::setMonitor
###############################################################################

sub setMonitor
{
	my ($this, $Monitor) = @_;
	my $Return	= 0;
	
	# Validate the type...
	if ($Monitor->isa('MonMonitor'))
	{
		# Set the data.
		$this->{'monitor'} = $Monitor;
	}
	else
	{
		$Return = 20001001;
	}

	return $Return;
}

###############################################################################
# MonPeriod::isEmptyGroupAllowed
###############################################################################

sub isEmptyGroupAllowed
{
	my $this = shift;
	
	# Return the requested data.
	return $this->{'empty-group-allowed'};
}

###############################################################################
# MonPeriod::setEmptyGroupAllowed
###############################################################################

sub setEmptyGroupAllowed
{
	my ($this, $EmptyGroupAllowed) = @_;
	
	# Set the data.
	$this->{'empty-group-allowed'} = $EmptyGroupAllowed;

	return 0;
}

###############################################################################
# MonService::getDescription
###############################################################################

sub getDescription
{
	my $this = shift;
	
	# Return the requested data.
	return $this->{'description'};
}

###############################################################################
# MonService::setDescription
###############################################################################

sub setDescription
{
	my ($this, $Description) = @_;
	
	# Set the data.
	$this->{'description'} = $Description;

	return 0;
}

###############################################################################
# MonService::addExcludedHost
###############################################################################

sub addExcludedHost
{
	my ($this, $ExcludedHost) = @_;
	
	# Cache the object reference.
	push(@{ $this->{'excluded-host-list'} }, $ExcludedHost);

	return 0;
}

###############################################################################
# MonService::deleteExcludedHost
###############################################################################

sub deleteExcludedHost
{
	my ($this, $ExcludedHost) = @_;
	my $i;
	
	# Run through the data...
	for $i ( 0 .. $#{ $this->{'excluded-host-list'} } ) 
	{
		# If the element matches...
		if ($this->{'excluded-host-list'}[$i] eq $ExcludedHost)
		{
			# Delete the line.
			splice(@{ $this->{'excluded-host-list'} }, $i, 1);

			# Bail out.
			last;
		}
	}
	
	return 0;
}

###############################################################################
# MonService::iteratorExcludedHosts
###############################################################################

sub iteratorExcludedHosts
{
	my $this = shift;
	my $Iterator;

	# If there is an alert list...
	if ((defined($this->{'excluded-host-list'})) &&
		(scalar(@{ $this->{'excluded-host-list'} })))
	{
		# Initialize the iterator object...
		$Iterator = new IteratorArray($this->{'excluded-host-list'});
	}

	return $Iterator;
}

###############################################################################
# MonService::getExclusionPeriod
###############################################################################

sub getExclusionPeriod
{
	my $this = shift;
	
	# Return the requested data.
	return $this->{'exclusion-period'};
}

###############################################################################
# MonService::setExclusionPeriod
###############################################################################

sub setExclusionPeriod
{
	my ($this, $ExclusionPeriod) = @_;
	
	# Set the data.
	$this->{'exclusion-period'} = $ExclusionPeriod;

	return 0;
}

###############################################################################
# MonService::getDependency
###############################################################################

sub getDependency
{
	my $this = shift;
	
	# Return the requested data.
	return $this->{'dependency'};
}

###############################################################################
# MonService::setDependency
###############################################################################

sub setDependency
{
	my ($this, $Dependency) = @_;
	
	# Set the data.
	$this->{'dependency'} = $Dependency;

	return 0;
}

###############################################################################
# MonService::getDependencyBehavior
###############################################################################

sub getDependencyBehavior
{
	my $this = shift;
	
	# Return the requested data.
	return $this->{'dependency-behavior'};
}

###############################################################################
# MonService::setDependencyBehavior
###############################################################################

sub setDependencyBehavior
{
	my ($this, $DependencyBehavior) = @_;
	
	# Set the data.
	$this->{'dependency-behavior'} = $DependencyBehavior;

	return 0;
}

###############################################################################
# MonService::addPeriod
###############################################################################

sub addPeriod
{
	my ($this, $Period) = @_;
	my $Return	= 0;
	
	# Validate the type...
	if ($Period->isa('MonPeriod'))
	{
		# Cache the object reference.
		push(@{ $this->{'period-list'} }, $Period);
	}
	else
	{
		$Return = 20001002;
	}

	return $Return;
}

###############################################################################
# MonService::iteratorPeriods
###############################################################################

sub iteratorPeriods
{
	my $this = shift;
	my $Iterator;

	# If there is an alert list...
	if ((defined($this->{'period-list'})) &&
		(scalar(@{ $this->{'period-list'} })))
	{
		# Initialize the iterator object...
		$Iterator = new IteratorArray($this->{'period-list'});
	}

	return $Iterator;
}

1;

__END__
