package NetConfig;
$VERSION="1.06"; 

###############################################################################
#                                                                             #
# Filename: NetConfig.pm                                                      #
#                                                                             #
# Description: This module provides network configuration support.            #
#                                                                             #
###############################################################################
###############################################################################
#                                                                             #
#  Copyright  2000 - 2001 Intel Corporation.                                 #
#  Intel Corporation All Rights Reserved.                                     #
#                                                                             #
#  The source code contained or described herein and all documents related to #
#  the source code ("Material") are owned by Intel Corporation or its         #
#  suppliers or licensors.  Title to the Material remains with Intel          #
#  Corporation or its suppliers and licensors.  The Material contains trade   #
#  secrets and proprietary and confidential information of Intel or its       #
#  suppliers and licensors.  The Material is protected by worldwide copyright #
#  and trade secret laws and treaty provisions.  No part of the Material may  #
#  be used, copied, reproduced, modified, published, uploaded, posted,        #
#  transmitted, distributed, or disclosed in any way without Intel's prior    #
#  express written permission.                                                #
#                                                                             #
#  No license under any patent, copyright, trade secret or other intellectual #
#  property right is granted to or conferred upon you by disclosure or        #
#  delivery of the Materials, either expressly, by implication, inducement,   #
#  estoppel or otherwise. Any license under such intellectual property rights #
#  must be express and approved by Intel in writing.                          #
#                                                                             #
###############################################################################

# Put the PERL library in the path.
use lib "/usr/ins/intel/gui/perllib";

use strict;
use Usable;
use File::Copy;
use LanguageLogCGI;
use VirtualDomain;
use RouteConfig;
use Safe;

################################################################################
################################################################################
#  Public Methods
################################################################################
################################################################################


## Create a Network Interface Object (UNUSED, Public)
################################################################################
################################################################################

sub new {

	my ($self, $nic, $ipaddress, $netmask, $broadcast,$gateway,$gwdev,$bootproto,$onboot) = @_;
	my $network=$ipaddress || "";
	$network =~ s/^(\d+)\.(\d+)\.(\d+)\.(\d+)/$1\.$2\.$3\.0/;

	my $nic_setup = {
		"self"		=>	$self,
   		"nic"		=>	$nic,
   		"ipaddress"	=>	$ipaddress,
   		"netmask"	=>	$netmask,
   		"broadcast"	=>	$broadcast,
		"gateway"	=>	$gateway,
		"gwdev"		=>	$gwdev,
		"bootproto"	=>	$bootproto,
		"onboot"	=>	$onboot,
		"network"	=>	$network,
	};
	bless $nic_setup, 'NetConfig';
	return $nic_setup;
}
###############################################################################
# NetConfig::LogMessage
###############################################################################

sub LogMessage
{
	my ($Code, $Category, $Severity, $ParameterRef) = @_;
	my $Log	= new LanguageLogCGI('/usr/ins/intel/gui/logs/server.log', '/usr/ins/intel/gui/locale', [ 'Mail.lrc' ]);

	# Log the error.
	$Log->Append($Category, $Severity, $Code, $ParameterRef);
	return $Code;
}

## List Active Network Interfaces  (Public)
################################################################################
################################################################################


sub listNetInterfaces {

        my (@nicindex, @lines, $list);
        my @ifconfigdata = Safe::backtick("/sbin/ifconfig");

        for (@ifconfigdata)
	{
                s/\r|\n//g;
                if (/^\S+/)
		{
			push(@lines, $_);
		}
		else
		{
			$lines[$#lines] .= $_;
		}
        }
        
	foreach $list (@lines) {
                my %ifcfg;
                $list =~ /^([^:\s]+)/; $ifcfg{'nic'} = $1;
                $list =~ /^(\S+)/; $ifcfg{'fullnic'} = $1;
                if ($list =~ /^(\S+):(\d+)/) { $ifcfg{'virtual'} = $2; }
                if ($list =~ /inet addr:(\S+)/) { $ifcfg{'ipaddress'} = $1; }
                        else { next; }
                if ($list =~ /Mask:(\S+)/) { $ifcfg{'netmask'} = $1; }
                if ($list =~ /Bcast:(\S+)/) { $ifcfg{'broadcast'} = $1; }
		
		if ($ifcfg{'ipaddress'})
		{ 
	                my $bin_ip  	= str2bin($ifcfg{ipaddress});
			my $bin_nm  	= str2bin($ifcfg{netmask});
	                my $bin_net 	= $bin_ip & $bin_nm;
			$ifcfg{network}	= bin2str($bin_net);
		}
                
		if ($list =~ /HWaddr (\S+)/) { $ifcfg{'macaddr'} = $1; }
                if ($list =~ /MTU:(\d+)/) { $ifcfg{'mtu'} = $1; }
                $ifcfg{'up'}++ if ($list =~ /\sUP\s/);
                push(@nicindex, \%ifcfg);
        }
        return @nicindex;
}

## Refuse to Work on Virtual Domains (Public)
################################################################################
################################################################################

sub rejectVDomain {

	my ($self,$ipaddress,$nic) = @_;

# Check was previously only for Virtual Nics
	if ($nic) {
	   if ( $nic =~ /\:/ ) {

	        my ($chk_ip, $chk_dom) = VirtualDomain::ip_domain($ipaddress);

		if ($chk_ip or $chk_dom) {
			if ($0 =~ "server_network.cgi") {
				return LogMessage(1800000, 'error', 'critical'); 
			}
		}


	   }
	}

	return 0;

}

## Public Interface for Modifying a NIC (Public)
################################################################################
################################################################################

sub changeNic {

	my ($self,$nic,$ipaddress,$netmask,$broadcast,$confirm,$network,$gateway) = @_;
#	my ($self,$nic,$ipaddress,$netmask,$broadcast,$gateway,$bootproto,$onboot,$network) = @_;
	my ($nic_item, %broken);


	## Reject if IP Address is Invalid  
	############################################################

	for $nic_item ($ipaddress,$netmask,$broadcast) {
		if ($nic_item) {
			if (Usable::check_ip($nic_item) ne "1") { 
				return LogMessage(1800001, 'error', 'warning', [ "$nic_item" ]); 
			}
		}
	}

	## Reject if Netmask is Invalid
	############################################################

        if ($netmask) {

                my @valid_nmasks = ("0", "128", "192", "224", "240", "248", "252", "254", "255");
                $netmask =~ /(\d+)\.(\d+)\.(\d+)\.(\d+)/;

                for my $i ($1, $2, $3, $4) {
                        if ( grep( /\b$i\b/, @valid_nmasks)) { next; }
						else { 
							return LogMessage(1800002, 'error', 'warning', [ "$netmask" ]); 
						}
                }
        }


	## Reject if IP Address Belongs to a Virtual Domain
	############################################################

#	if ( $nic =~ ":" ) {
		my ($chk_ip, $chk_dom) = VirtualDomain::ip_domain($ipaddress);

#	}


	## Check for Sane IP Address. ( Redundant -- Check, then Delete! )
	############################################################

	if ( $ipaddress =~ /(\d+)\.(\d+)\.(\d+)\.0/ ) { 
		return LogMessage(1800003, 'error', 'warning', [ "$ipaddress" ]); 
	}
	if ( $ipaddress =~ /(\d+)\.(\d+)\.(\d+)\.255/ ) { 
		return LogMessage(1800004, 'error', 'warning', [ "$ipaddress" ]); 
	}
	if ( $ipaddress eq "127.0.0.1" ) { 
		return LogMessage(1800005, 'error', 'warning', [ "$ipaddress" ]); 
	}


	## Check that IP Address doesn't collide w/ default gateway(s).
	############################################################

	my @rt_list = RouteConfig->listRoutes();
	for my $rtcheck (@rt_list) {

		if ( $rtcheck->{destination} eq '0.0.0.0' and $rtcheck->{gateway} eq $ipaddress ) { 
			return LogMessage(1800006, 'error', 'warning', [ "$ipaddress" ]); 
		}
	}

	## Generate Network Address
	############################################################
	if ( ! $network ) { 

		my $bin_ip  = str2bin($ipaddress);
		my $bin_nm  = str2bin($netmask);
		my $bin_net = $bin_ip & $bin_nm;
		$network    = bin2str($bin_net);

	}
	############################################################

	my @nicindex = listNetInterfaces();

	if ( $nic eq 'eth0' or $nic eq 'eth1' ) {

		for (@nicindex) {

			## Make sure that the IP Address isn't already in use. 
			############################################################

			if ( $ipaddress eq "$_->{ipaddress}" ) {

				unless ( ($netmask ne $_->{netmask}) or ($broadcast ne $_->{broadcast}) or (!$netmask and !$broadcast) ) {
					return LogMessage(1800007, 'error', 'warning'); 
				}
				else { $broadcast = setBroadcast($ipaddress, $netmask); }
			}

			## Reject if More than 1 Address per Subnet (for 'true' IPs) 
			############################################################

			if ($nic eq 'eth0') { 
				for (@nicindex) {
					if ($_->{fullnic} eq 'eth1') {
						if ($_->{network} eq $network) {
							return LogMessage(1800008, 'error', 'warning'); 
						}

					}
				}
			}
			
			if ($nic eq 'eth1') { 
				for (@nicindex) {
					if ($_->{fullnic} eq 'eth0') {
						if ($_->{network} eq $network) {
							return LogMessage(1800008, 'error', 'warning'); 
						}

					}
				}
			}

		}
	}

	## Set a flag to clear unnecessary (and potentially harmful) routes.
	############################################################

	for my $element (@nicindex) {

		if ($element->{'ipaddress'} eq $ipaddress) { 
			unless ( $netmask ne $_->{netmask} ) {
				if ($confirm ne "yes") { 
					return LogMessage(1800009, 'security', 'information', [ "$ipaddress" ]); 
				}
			}
			else { 
				$broadcast = setBroadcast($ipaddress, $netmask); 
				$broken{'route'} = 'no'; 
			}
		}

		unless ( $broken{route} ne 'no' ) {
			if ($element->{'network'} eq $network and $element->{'netmask'} eq $netmask) { 
				$broken{'route'} ="yes"; 
			}
		}

	}

	## Make a guess at a broadcast address if there's a netmask given.
	############################################################

	if ( $netmask and (!$broadcast) ) { $broadcast = setBroadcast($ipaddress, $netmask); }


	## Everything should be okay -- Now configure the NIC.
	############################################################

	eval {
		NetConfig->writeEthConf($nic,$ipaddress,$netmask,$broadcast,$network);
		NetConfig->updateNetSettings($nic,$ipaddress,$netmask,$broadcast);
		
	 };

	## Write Out Startup Conf File (/etc/sysconfig/network)
	############################################################
	if ($nic eq 'eth0') {		
		for (@rt_list) { 
			if ( $_->{destination} eq '0.0.0.0' ) {
				$gateway = $_->{gateway};	
			}
		}

		writeNetConf($nic,$ipaddress,$gateway); 
	}
	elsif ($nic eq 'eth1') {

		my $config_eth1 = 'yes';

		for (@nicindex) { 
			if ($_->{fullnic} eq 'eth0') { $config_eth1 = 'no'; }
		}

		if ( $config_eth1 eq 'yes' ) { 
			writeNetConf($nic,$ipaddress,$gateway);
		}	
	} 
	############################################################

	if ($@) { 
		return LogMessage(1800010, 'error', 'warning', [ "$@" ]); 
	}

	if ($broken{'route'} eq "yes") { 

		$broken{'nic'} = $nic;
		$broken{'netmask'} = $netmask;
		$broken{'destination'} = $ipaddress;
		$broken{'destination'} =~ s/^(\d+)\.(\d+)\.(\d+)\.(\d+)/$1\.0\.0\.0/;
		$broken{'gateway'} = "0.0.0.0";
		RouteConfig->deleteRoute('','','','',\%broken); 

	}

	return LogMessage(1800011, 'security', 'information'); 
}


## Delete or Bring Down a Network Interface (Public)
################################################################################
################################################################################

sub deleteNetInterface {

	my ($self, $nic, $ipaddress) = @_;
	my $ifconfig="/sbin/ifconfig";
	my $down = "down";

	if ($ipaddress and !$nic) {
		my @niclist = listNetInterfaces();
		for (@niclist) {
			if ( $_->{ipaddress} eq $ipaddress) { 
				$nic = $_->{'fullnic'};
			}
		} 
	}

	if (!$ipaddress) {
                my @iplist = listNetInterfaces();
                for (@iplist) {
                        if ( $_->{nic} eq $nic) {
                                $ipaddress = $_->{ipaddress};
                        }
                }
	}

	## Reject Virtual Domains (other than Default)
	############################################################
	if ( $nic =~ /:/ ) { 
	        my ($chk_ip, $chk_dom) = VirtualDomain::ip_domain($ipaddress);
		if ($chk_ip or $chk_dom) {
		}
	}

	## Deal with an ifconfig bug
	############################################################
	my $result = reactivateAliases($nic);

	if ( $result eq '0' ) {

		my $deleted_conf_file = "/etc/sysconfig/network-scripts/ifcfg-".$nic;
		Safe::system("$ifconfig $nic $down") == 0  || return ( LogMessage(1800012, 'error', 'warning', [ "$nic" ]));
		unlink $deleted_conf_file || return ( LogMessage(1800013, 'error', 'warning', [ "$deleted_conf_file" ]));

		return ( LogMessage(1800011, 'security', 'information'));
	}
	else { return ($result); }

}


################################################################################
################################################################################
#  Private Methods
################################################################################
################################################################################



## Determine Broadcast Address (Private)
##	(for advanced subnetting -- ifconfig doesn't do properly) 
################################################################################
################################################################################

sub setBroadcast {

	my ($ipaddress, $netmask) = @_;

	my $bin_ipaddr = str2bin($ipaddress);
	my $bin_nmask  = str2bin($netmask);
	my $bin_bcast  = $bin_ipaddr | (~$bin_nmask);

	my $broadcast = bin2str($bin_bcast);

	return ($broadcast);
	
}

# Transform Between Binarys and Strings (Private -- used in setBroadcast)
################################################################################

sub str2bin {

	my $ipaddr = shift;
	my $bin;
	if ($ipaddr =~ /(\d+)\.(\d+)\.(\d+)\.(\d+)/) { $bin = pack('CCCC', $1, $2, $3, $4); }
	else {
		return "failed\n";
	}
	unpack 'N', $bin;

}


sub bin2str {			

	my $bin = shift;
	sprintf("%d.%d.%d.%d",
		$bin>>24,
		($bin>>16) & 255,
		($bin>>8) & 255,
		$bin & 255);

}

## Add/Change Network Interface (Private)
################################################################################
################################################################################

sub updateBIOS {

	my ($self, $ipaddress, $netmask, $gateway) = @_;
	my $biosconf = "/usr/ins/intel/bin/asm/sysBIOSConfNet";
	my $set_nvram = "$ipaddress $netmask $gateway";

	Safe::system("$biosconf $set_nvram") == 0 || LogMessage(1800014, 'error', 'warning');

	return 0;

}

sub updateNetSettings {

	my ($self,$nic,$ipaddress,$netmask,$broadcast) = @_;
	my $ifconfig = '/sbin/ifconfig';
	my $subnet   = 'netmask';
	my $bcast    = 'broadcast';

	my @sysargs = ("$nic", "$ipaddress");

	if ($netmask ne "") 	{ push @sysargs, $subnet, $netmask ; }
	if ($broadcast ne "") 	{ push @sysargs, $bcast, $broadcast; }

	Safe::system ("$ifconfig @sysargs") == 0 || return ( LogMessage(1800015, 'error', 'warning') ); 

	LogMessage(1800011, 'security', 'information');
	return 0;

}

## Write NIC Info to Conf File (/etc/sysconfig/network-scripts/ifcfg-ethX) (Private)
################################################################################
################################################################################

sub writeEthConf {

	my ($self, $nic, $ipaddress, $netmask, $broadcast, $bootproto, $onboot, $gateway) =@_;
	my $eth_conf_file = "/etc/sysconfig/network-scripts/ifcfg-" .$nic;

	$bootproto = "none" unless ($bootproto eq "bootp" or $bootproto eq "dhcp");
	$onboot = "yes" unless ($onboot eq "no");

	if ( -e $eth_conf_file ) {
		copy($eth_conf_file, $eth_conf_file.".old") || return ( LogMessage(1800016, 'error', 'warning', [ "$eth_conf_file" ]));
	}

	open (IFCONF, ">$eth_conf_file") || return ( LogMessage(1800017, 'error', 'warning', [ "$eth_conf_file" ]));
	flock (IFCONF, 2) || return ( LogMessage(1800018, 'error', 'warning', [ "$eth_conf_file" ]));
	print IFCONF "DEVICE=\"$nic\"\n",
		     "IPADDR=\"$ipaddress\"\n",
		     "NETMASK=\"$netmask\"\n",
	($gateway)?  "GATEWAY=\"$gateway\"\n" : "",
		     "ONBOOT=\"$onboot\"\n",
		     "BOOTPROTO=\"$bootproto\"\n";
	close IFCONF;

}

## Write NIC Info to /etc/sysconfic/network (Private)
################################################################################
################################################################################

sub writeNetConf {

	my ($nic, $ipaddress, $gateway) = @_;
	my $net_conf_file= "/etc/sysconfig/network";
	chomp (my $hostname = Safe::backtick('/bin/hostname') );

	## Get Gateway 
	############################################################		
	if ( ! $gateway ) {

		my @route_list = RouteConfig->listRoutes();

		for (@route_list) {
			if ( $_->{destination} eq '0.0.0.0' ) { $gateway = $_->{gateway}; }
		}
	}
	############################################################		

	if ( -e $net_conf_file ) {
		copy($net_conf_file, $net_conf_file.".old") || return ( LogMessage(1800016, 'error', 'warning', [ "$net_conf_file" ]));
	}

	open (NETCONF, ">$net_conf_file") || return ( LogMessage(1800017, 'error', 'warning', [ "$net_conf_file" ]));
	flock (NETCONF, 2) || return ( LogMessage(1800018, 'error', 'warning', [ "$net_conf_file" ]));
	print NETCONF	"NETWORKING=\"yes\"\n",
			"FORWARD_IPV4=\"no\"\n",
			"HOSTNAME=\"$hostname\"\n",
			"GATEWAY=\"$gateway\"\n",
			"GATEWAYDEV=\"$nic\"\n";
	close NETCONF;
}

## Write NIC Info for Static Routes (UNUSED, Private)
################################################################################
################################################################################

sub writeRTConf {

        my ($self,$nic,$ipaddress,$netmask,$gateway,$network) =@_;
        my $rt_conf_file= "/etc/sysconfig/static-routes";
	my $rtconf_append = "$nic net $network netmask $netmask gw $gateway\n";

        if ( -e $rt_conf_file ) {
                copy($rt_conf_file, $rt_conf_file.".old") || return ( LogMessage(1800016, 'error', 'warning', [ "$rt_conf_file" ]));
        }

        open (RTCONF, ">$rt_conf_file") || return ( LogMessage(1800017, 'error', 'warning', [ "$rt_conf_file" ]) );
	flock (RTCONF, 2) || return ( LogMessage(1800018, 'error', 'warning', [ "$rt_conf_file" ]));
	my @old_rtconf = <RTCONF>;
	push @old_rtconf, $rtconf_append;
	print RTCONF	@old_rtconf;
        close RTCONF;
}


## Reactivate Aliases  (Private)
##
## A workaround for an ifconfig issue (removing ip aliases that are on the same
## subnet, when the first in the "chain" is deleted)
################################################################################
################################################################################

sub reactivateAliases {

	my $nic = shift;
	my (@nics_to_config, @nics_update, $network);
	my @niclist = listNetInterfaces();

	## Get Info About the Interface's Network
	############################################################
	for (@niclist) {
		if ($_->{fullnic} eq $nic) {
			$network   = $_->{network};
		}
	}

	## Generate List of Aliases to Resurrect (Except Delete Target)
	############################################################
	for (@niclist) { if ( ($_->{network} eq $network) and ($_->{fullnic} ne $nic) ) { 

		push (@nics_to_config, $_); } 

	}

	## If No Matches, Return for Normal Delete
	############################################################
	if ( @nics_to_config < 1 ) { return 0; }

	## Delete the Target
	############################################################
	my $deleted_conf_file = "/etc/sysconfig/network-scripts/ifcfg-".$nic;
	Safe::system("/sbin/ifconfig $nic down") == 0  || return ( LogMessage(1800012, 'error', 'warning', [ "$nic" ]));
	unlink $deleted_conf_file || LogMessage(1800013, 'error', 'warning', [ "$deleted_conf_file" ]);


	################################################################################	
	## This section under construction -- Not required, but better implementation 
	################################################################################	
	##	Check to see if interfaces need to be re-added
	################################################################################	
	#	my @final_check = listNetInterfaces();
	#	my $check = $_->{fullnic};
	#	
	#	for (@nics_to_config) {
	#		if ( grep (/$check/, @final_check) == 1 ) { push (@nics_update, $_); }
	#	}
	#	
	#	## If No Matches, Return for Normal Delete
	#	############################################################
	#	if ( @nics_update < 1 ) { return 0; }  
	################################################################################	

	## Resurrect the Aliases
	############################################################
	for (@nics_to_config) {
	
		##  Don't Need to Perform Normal Checks (These were previously configured)
		############################################################	
		my $feedback = NetConfig->updateNetSettings($_->{fullnic},$_->{ipaddress},$_->{netmask},$_->{broadcast});

		## Log Errors, but Don't Return Them to the GUI
		##	(Primary Concern is Delete of Target Interface)
		############################################################	
		if ($feedback ne 0) {
			LogMessage(1800010, 'error', 'warning', [ "$_->{ipaddress}" ]);
		}
	}

	return ( LogMessage(1800011, 'security', 'information') );
}

1;
