package PrintIndex;

###############################################################################
#                                                                             #
# Filename: PrintIndex.pm                                                     #
#                                                                             #
# Description: Contains script needed to manage the master frameset.          #
#                                                                             #
###############################################################################
###############################################################################
#                                                                             #
#  Copyright  2000 - 2001 Intel Corporation.                                 #
#  Intel Corporation All Rights Reserved.                                     #
#                                                                             #
#  The source code contained or described herein and all documents related to #
#  the source code ("Material") are owned by Intel Corporation or its         #
#  suppliers or licensors.  Title to the Material remains with Intel          #
#  Corporation or its suppliers and licensors.  The Material contains trade   #
#  secrets and proprietary and confidential information of Intel or its       #
#  suppliers and licensors.  The Material is protected by worldwide copyright #
#  and trade secret laws and treaty provisions.  No part of the Material may  #
#  be used, copied, reproduced, modified, published, uploaded, posted,        #
#  transmitted, distributed, or disclosed in any way without Intel's prior    #
#  express written permission.                                                #
#                                                                             #
#  No license under any patent, copyright, trade secret or other intellectual #
#  property right is granted to or conferred upon you by disclosure or        #
#  delivery of the Materials, either expressly, by implication, inducement,   #
#  estoppel or otherwise. Any license under such intellectual property rights #
#  must be express and approved by Intel in writing.                          #
#                                                                             #
###############################################################################

# Put the PERL library in the path.
use lib '/usr/ins/intel/gui/perllib';

use strict;
use LanguageResource;
use Users;

###############################################################################
# PrintIndex::displayIndex
###############################################################################

sub displayIndex
{
	my ($cgi)				= @_;
	my $PreferredLanguage	= (split(/,/, $ENV{'HTTP_ACCEPT_LANGUAGE'}))[0];
	my $StringResource		= new LanguageResource($PreferredLanguage, '/usr/ins/intel/gui/locale', [ 'common.lrc' ]);
	my $ElementNavigation;
	my $ElementTitle;
	my $ElementMain;
	my $TargetUser	   		= $cgi->remote_user();
	my $TargetUserGID		= (getpwnam($TargetUser))[3];
	my $TargetDomain		= Users::getUserDomain($TargetUser);
	my $Level;
	my $Title;

	# Establish a user level...
	if (10000 == $TargetUserGID) 
	{ 
		# Server administrator.
		$Level = 'server';
	}
	elsif (10001 == $TargetUserGID) 
	{ 
		# Domain administrator.
		$Level = 'domain';
	}
	else
	{
		# Regular user.
		$Level = 'user';
	}

	# If there is a user override...
	if ($cgi->param('user'))
	{ 
		# Update the level and user information.
		$Level		= 'user'; 
		$TargetUser	= $cgi->param('user');
	}
	
	# If there is a domain override...
	if ($cgi->param('domain'))
	{ 
		# Update the level and possibly the domain information.
		$Level			= 'domain' unless('user' eq $Level); 
		$TargetDomain	= $cgi->param('domain');
	} 

	# Pad the target definitions.
	$TargetUser		= "user=$TargetUser/" if($TargetUser);
	$TargetDomain	= "domain=$TargetDomain/" if($TargetDomain);

	# Set up each of the frame element definitions.
	$ElementTitle		= "/cgi-bin/" . $TargetUser . $TargetDomain . "$Level/" . $Level . "_title.cgi";
	$ElementNavigation	= "/cgi-bin/" . $TargetUser . $TargetDomain . "$Level/" . $Level . "_navbar.cgi";
	$ElementMain		= "/cgi-bin/" . $TargetUser . $TargetDomain . "$Level/" . $Level . "_status.cgi";
	
	# Get the title bar.
	$Title = $StringResource->_S('TITLE_BAR_ADMINISTRATION_' . uc($Level));

	print $cgi->header(-expires=>'Mon, 21-May-1971 00:00:00 GMT');
	print <<HERE;
<title>$Title</title>
<frameset rows=101,* border=0 framespacing=0 frameborder=0>
	<frame noresize border=0 scrolling=no marginwidth=0 marginheight=0 src=$ElementTitle name=title>
	<frameset cols=139,*>
		<frame noresize border=0 scrolling=no marginwidth=0 marginheight=0 src=$ElementNavigation name=navbar>
		<frame noresize border=0 scrolling=auto marginwidth=30 marginheight=10 src=$ElementMain name=main leftmargin=30 topmargin=10>
	</frameset>
</frameset>
HERE
}

###############################################################################
# PrintIndex::displayFirstBoot
###############################################################################

sub displayFirstBoot
{
	my ($cgi) = @_;
	my $PreferredLanguage	= (split(/,/, $ENV{'HTTP_ACCEPT_LANGUAGE'}))[0];
	my $StringResource		= new LanguageResource($PreferredLanguage, '/usr/ins/intel/gui/locale', [ 'common.lrc' ]);
	my $Title;

	# Get the title bar.
	$Title = $StringResource->_S('TITLE_BAR_LOGIN');
	
	print $cgi->header(-expires=>'Mon, 21-May-1971 00:00:00 GMT');
	print <<HERE;
<title>$Title</title>
<frameset rows=101,* border=0 framespacing=0 frameborder=0>
	<frame noresize border=0 scrolling=no marginwidth=0 marginheight=0 bordercolor=black src=/cgi-bin/firstboot/firstboot_title.cgi name=title>
	<frame border=0 scrolling=auto marginwidth=20 marginheight=0 src=/cgi-bin/firstboot/firstboot1.cgi name=main>
</frameset>
HERE
}

###############################################################################
# PrintIndex::getLoginIndex
###############################################################################

sub getLoginIndex
{
	my ($cgi)				= @_;
	my $PreferredLanguage	= (split(/,/, $ENV{'HTTP_ACCEPT_LANGUAGE'}))[0];
	my $StringResource		= new LanguageResource($PreferredLanguage, '/usr/ins/intel/gui/locale', [ 'common.lrc' ]);
	my $ElementNavigation;
	my $ElementTitle;
	my $Message				= $cgi->param('message');
	my $Title;
	my $html;

	# Add the message to the URI if available...
	$Message = (defined($Message)) ? ('?message=' . $cgi->escape($Message)) : '';
	
	# Set up each of the frame element definitions.
	$ElementNavigation	= '/login_navbar.cgi' . $Message;
	$ElementTitle		= '/login_title.cgi' . $Message;

	# Get the title bar.
	$Title = $StringResource->_S('TITLE_BAR_LOGIN');
	
	$html = <<HERE;
<html>
	<head>
		<title>$Title</title>
	</head>
	<frameset rows=101,* border=0 framespacing=0 frameborder=0>
		<frame noresize border=0 scrolling=no marginwidth=0 marginheight=0 src=$ElementTitle bordercolor=black name=title>
		<frame border=0 scrolling=auto marginwidth=10 marginheight=0 src=$ElementNavigation name=navbar>
	</frameset>
</html>
HERE

	return $html;
}

1;
