package QuickHelp;

###############################################################################
#                                                                             #
# Filename: QuickHelp.pm                                                      #
#                                                                             #
# Description: Contains script needed to manage quickhelp.                    #
#                                                                             #
###############################################################################
###############################################################################
#                                                                             #
#  Copyright  2000 - 2001 Intel Corporation.                                 #
#  Intel Corporation All Rights Reserved.                                     #
#                                                                             #
#  The source code contained or described herein and all documents related to #
#  the source code ("Material") are owned by Intel Corporation or its         #
#  suppliers or licensors.  Title to the Material remains with Intel          #
#  Corporation or its suppliers and licensors.  The Material contains trade   #
#  secrets and proprietary and confidential information of Intel or its       #
#  suppliers and licensors.  The Material is protected by worldwide copyright #
#  and trade secret laws and treaty provisions.  No part of the Material may  #
#  be used, copied, reproduced, modified, published, uploaded, posted,        #
#  transmitted, distributed, or disclosed in any way without Intel's prior    #
#  express written permission.                                                #
#                                                                             #
#  No license under any patent, copyright, trade secret or other intellectual #
#  property right is granted to or conferred upon you by disclosure or        #
#  delivery of the Materials, either expressly, by implication, inducement,   #
#  estoppel or otherwise. Any license under such intellectual property rights #
#  must be express and approved by Intel in writing.                          #
#                                                                             #
###############################################################################

# Put the PERL library in the path.
use lib '/usr/ins/intel/gui/perllib';

use strict;
use LanguageResource;

###############################################################################
# new
###############################################################################

sub new
{
	my ($class, $PreferredLanguage, $Class, $ID) = @_;

	# Safe default(s).
	$ID		= '' unless(defined($ID));
	$Class	= '' unless(defined($Class));

	# Initialize the object.
	my $this = 
	{
		'id'				=> $ID,
		'class'				=> $Class,
		'string-resource'	=> new LanguageResource($PreferredLanguage, '/usr/ins/intel/gui/locale', [ "$Class.lrc" ]),
		'help-resource'		=> new LanguageResource($PreferredLanguage, '/usr/ins/intel/gui/locale/help'),
	};

	# Align the object with this package.
	bless($this, $class);
	
	# Return the object.
	return $this;
}

###############################################################################
# QuickHelp::GetTitle
###############################################################################

sub GetTitle 
{
	my $this = shift;
	
	# Get the help title from the resource object.
	return $this->{'string-resource'}->_S("IDQH:$this->{'id'}");
}

###############################################################################
# QuickHelp::GetContent
###############################################################################

sub GetContent
{
	my $this = shift;
	my $Content;
	my $QuickHelpPath;

	# Try to get the file name...
	if ('' eq ($QuickHelpPath = $this->{'string-resource'}->_S("IDQHCM:$this->{'id'}")))
	{
		# Otherwise, use the class name as a base.
		$QuickHelpPath = "$this->{'class'}.htm";
	}

	# Try to resolve the resource file...
	if ((defined($QuickHelpPath = $this->{'help-resource'}->ResolveResourceFile($QuickHelpPath))) &&
		(open(QUICKHELP, "< $QuickHelpPath")))
	{
		# Read in the contents of the document.
		read(QUICKHELP, $Content, -s QUICKHELP);

		# Close the file handle.
		close(QUICKHELP);
	}
	
	return $Content;
}

###############################################################################
# QuickHelp::GetPreferredLanguage
###############################################################################

sub GetPreferredLanguage
{
	my $this = shift;

	# Get the requested data.
	return $this->{'string-resource'}->GetPreferredLanguage();
}

###############################################################################
# QuickHelp::SetPreferredLanguage
###############################################################################

sub SetPreferredLanguage
{
	my ($this, $PreferredLanguage) = @_;

	# Set the data.
	$this->{'string-resource'}->SetPreferredLanguage($PreferredLanguage);
}

###############################################################################
# QuickHelp::GetClass
###############################################################################

sub GetClass
{
	my $this = shift;

	# Return the requested data.
	return $this->{'class'};
}

###############################################################################
# QuickHelp::SetClass
###############################################################################

sub SetClass
{
	my ($this, $Class) = @_;

	# Cache the class.
	$this->{'class'} = $Class;
	
	# Set the data.
	$this->{'string-resource'}->SetSuffixList("$Class.lrc");
}

###############################################################################
# QuickHelp::GetID
###############################################################################

sub GetID
{
	my $this = shift;

	# Return the requested data.
	return $this->{'id'};
}

###############################################################################
# QuickHelp::SetID
###############################################################################

sub SetID
{
	my ($this, $ID) = @_;
	
	# Set the data.
	$this->{'id'} = $ID;

	# Return the new value.
	return $this->{'id'};
}
1;

__END__
