package Quotaall;

###############################################################################
#                                                                             #
# Filename: Quotaall.pm                                                       #
#                                                                             #
# Description: This module provides disk quota configuration support.         #
#                                                                             #
###############################################################################
###############################################################################
#                                                                             #
#  Copyright  2000 - 2001 Intel Corporation.                                 #
#  Intel Corporation All Rights Reserved.                                     #
#                                                                             #
#  The source code contained or described herein and all documents related to #
#  the source code ("Material") are owned by Intel Corporation or its         #
#  suppliers or licensors.  Title to the Material remains with Intel          #
#  Corporation or its suppliers and licensors.  The Material contains trade   #
#  secrets and proprietary and confidential information of Intel or its       #
#  suppliers and licensors.  The Material is protected by worldwide copyright #
#  and trade secret laws and treaty provisions.  No part of the Material may  #
#  be used, copied, reproduced, modified, published, uploaded, posted,        #
#  transmitted, distributed, or disclosed in any way without Intel's prior    #
#  express written permission.                                                #
#                                                                             #
#  No license under any patent, copyright, trade secret or other intellectual #
#  property right is granted to or conferred upon you by disclosure or        #
#  delivery of the Materials, either expressly, by implication, inducement,   #
#  estoppel or otherwise. Any license under such intellectual property rights #
#  must be express and approved by Intel in writing.                          #
#                                                                             #
###############################################################################

# Put the PERL library in the path.
use lib '/usr/ins/intel/gui/perllib';

use vars qw(@ISA @EXPORT @EXPORT_OK);
require Exporter;
@ISA = qw(Exporter);
@EXPORT = qw( ); 
use strict;
use LanguageResource;
use Usable;
use VirtualDomain;
use Quota;
use Safe;

# Globals
my $PreferredLanguage	= (defined($ENV{'HTTP_ACCEPT_LANGUAGE'})) ? (split(/,/, $ENV{'HTTP_ACCEPT_LANGUAGE'}))[0] : 'en-US';
my $StringResource		= new LanguageResource($PreferredLanguage, '/usr/ins/intel/gui/locale', [ 'Quotaall.lrc' ]);

###############################
# 
# Quota type for group 1, following the Quota.pm.
sub setquota
{	
	my ($id,$soft,$type) = @_;
	my $quota_path = '/home/domain';
	my ( $uid,$dev,$hard,$soft1 );
	$uid = $id;	# $id will pass either gid or uid based on domain or user quota; 
	$soft = ($soft*1024);	## Input is in MB and needs to convert into Blcks of 1 Kb each
	if ($soft == 0) { $hard = 0; }
	else { $hard = ($soft+10); }
	if($type) { $type = '1'; }
	$dev = Quota::getqcarg($quota_path);
	return $StringResource->_S('CANT_SET_QUOTA', [ "$id" ]) if (Quota::sync($dev) and ($! != 1));
	my ($bc, $bs, $bh, $bt, $fc, $fs,$fh, $ft) = Quota::query($dev,$uid,$type);
	# Now set quota for the users and domains....
	Quota::setqlim($dev,$uid,$soft,$hard,0,0,1,$type) && return $StringResource->_S('CANT_SET_QUOTA', [ "$uid: $! \n" ] );
	
}

#########################################################################################
sub repquota

{
	my ($vref, $some ) = @_;
	my ($type, $id, $user, $home , $domain, $grp, $dev, @vws, @quota, $quse, $qlimit, $qhard, $qgrace, $fc, $fs,$fh, $ft);
	my $used;	
	@vws = VirtualDomain::list_vws();
	if ($vref->{'user'}) {
		$user = $vref->{'user'};
	        ($id, $home) = (getpwnam($user))[2,7];	

	}
	
	if ($vref->{'domain'}) { 
		$domain = $vref->{'domain'};
		$home = '/home/domain';
		$type = '1';
		foreach my $item ( @vws ) {
			$grp = $item->{'dom_grp'} if ($item->{'vdn'} eq $domain);
		}	
		$id = getgrnam($grp);
		$used = quotaCheck($grp,$type);
	}
	$dev = Quota::getqcarg($home);
    	Safe::backtick("/sbin/quotaon $dev 2>/dev/null");
	($quse, $qlimit, $qhard, $qgrace, $fc, $fs,$fh, $ft) = Quota::query($dev,$id,$type);
	if ($used) { $quse = $used; }
	push (@quota, $quse, $qlimit, $qhard, $qgrace, $fc, $fs,$fh, $ft);
	return (@quota);

}

#########################################################################################

# get the combined quota of users in a domain ###
# time taking process get the quota of all the users of a domain and then add...
sub quotaCheck {
        my $domain = shift;
        my $type = shift;
        my $quotalimit = 0;
        my $totalquota = 0;
#       my @users = Users::listUsrs();
        my (%vref,$total,$total_used);
        my $home = "/home";
        my $dev = Quota::getqcarg($home);
        setpwent;

        # Iterate over the passwd file
        my @pwent;
        while (@pwent = getpwent) {
                # see if this passwd entry has our domain group in its
                # home directory.  if not, move on.
                if ($pwent[7] !~ m|^/[^/]+/[^/]+/$domain/[^/]+|) {
                        next;
                }

                # get the UID of this passwd entry
                my $uid = $pwent[2];

                #print "uid $uid\n";

                # get the quota
		Safe::backtick("/sbin/quotaon $dev 2>/dev/null");
				my ($block_curr, $block_soft, $block_hard, $block_timelimit,
                        $inode_curr, $inode_soft, $inode_hard, $inode_timelimit) =
                        Quota::query($dev,$uid);

                #print "slimit is ",$block_hard,"\n";

                # add it to the total
                $total += $block_hard;
                $total_used += $block_curr;
        }
        endpwent;


#       if ($item->{domain} eq $domain) {
#               $vref{user} = $item->{user};
#               my @quota = repquota(\%vref);
#               $quotalimit = $quota[1];
#               $totalquota = $totalquota + $quotalimit;
#       }
#return $totalquota;
if ($type == 1 ) { return ($total_used); }
return ($total);

}
#####################################################################################

sub comDomainQuota {

	my @domains = VirtualDomain::list_vws();
	my $quotalimit = 0;
	my $totalvdquota = 0;
	my %vref;
	foreach my $item (@domains) {
		$vref{domain} = $item->{vdn};
		my @quota = repquota(\%vref);
		
		# If there is a limited quota...
		if (0 != ($quotalimit = $quota[1]))
		{
			$totalvdquota = $totalvdquota + $quotalimit;
		}
		else
		{
			return 0;
		}
	}
	my ($free, $used) = Quotaall::diskSpaceCheck();
	my $unused = $free - $totalvdquota/1000;		
	return ($unused);

}

sub diskSpaceCheck {
        my (@data);
        my $df = "/bin/df";
        my $home = "/home";
        my $cmd = "$df /home";

        return $StringResource->_S('HOME_MISSING', [ $home ] ) if (!( -d $home));
        open (DISK,"$cmd |");
        while(<DISK>) {
        push(@data,$_);
        }
        close(DISK);
        my $diskspace  = pop(@data);
        my ($usedspace,$freespace) = (split /\s+/,$diskspace)[2,3];
	return $StringResource->_S('NO_QUOTA_FOUND') if (($usedspace == 0) && ($freespace == 0));
        return ($freespace/1000,$usedspace/1000);

}
		
1;
