package Usable;

###############################################################################
#                                                                             #
# Filename: Usable.pm                                                         #
#                                                                             #
# Description: This module provides miscellaneous functional support.         #
#                                                                             #
###############################################################################
###############################################################################
#                                                                             #
#  Copyright  2000 - 2001 Intel Corporation.                                 #
#  Intel Corporation All Rights Reserved.                                     #
#                                                                             #
#  The source code contained or described herein and all documents related to #
#  the source code ("Material") are owned by Intel Corporation or its         #
#  suppliers or licensors.  Title to the Material remains with Intel          #
#  Corporation or its suppliers and licensors.  The Material contains trade   #
#  secrets and proprietary and confidential information of Intel or its       #
#  suppliers and licensors.  The Material is protected by worldwide copyright #
#  and trade secret laws and treaty provisions.  No part of the Material may  #
#  be used, copied, reproduced, modified, published, uploaded, posted,        #
#  transmitted, distributed, or disclosed in any way without Intel's prior    #
#  express written permission.                                                #
#                                                                             #
#  No license under any patent, copyright, trade secret or other intellectual #
#  property right is granted to or conferred upon you by disclosure or        #
#  delivery of the Materials, either expressly, by implication, inducement,   #
#  estoppel or otherwise. Any license under such intellectual property rights #
#  must be express and approved by Intel in writing.                          #
#                                                                             #
###############################################################################

# Put the PERL library in the path.
use lib '/usr/ins/intel/gui/perllib';

use strict;
use VirtualDomain;
use File::Find;
use Unix::AliasFile;
use vars qw(@ISA @EXPORT @EXPORT_OK);
use Safe;
require Exporter;
@ISA = qw(Exporter);
@EXPORT = qw(check_ip error getDefaultDomain changeIndex disk_usage);

######################################################################

#############################################################################
# SName: check_ip(ip) - Verify the IP Address and check for it's validity
# Arguments: IP Address
# Return Value: none don't handle ip-v6

sub check_ip
{
return $_[0] =~ /^(\d+)\.(\d+)\.(\d+)\.(\d+)$/ &&
	$1 >= 0 && $1 <= 255 && 
	$2 >= 0 && $2 <= 255 &&
	$3 >= 0 && $3 <= 255 &&
	$4 >= 0 && $4 <= 255;
}

##################################################################################
# SName: error(message) - Display an error message and exit. The variable "$mesgs" must be set
# to specify the type of error and service.
sub error
{
	my $msg;
	print "<p>\n";
	print "<b>",($msg ? "$msg : " : ""),@_,"</b>\n";
	print "<p>\n";
	print "<\html>";
	exit;
}

############################################
#
# write_postmaster_alias
# 	This subroutine expects username or email address and then
#	updates the postmaster entry in /etc/aliases
#	Example : write_postmaster_alias(postmaster@domain_xyz.com)
#
 
sub write_postmaster_alias {
 
my $postmasters = shift;
my @postmasters = split(',', $postmasters);
 
# Open the alias file in read/write mode.
my $al = new Unix::AliasFile ("/etc/aliases");
 
# Set postmaster alias
$al->alias("postmaster", @postmasters);
 
# Save changes
$al->commit();
 
# Generate the /etc/aliases.db file from /etc/aliases
Safe::system("/usr/bin/newaliases");
 
}          

#############################################
#
sub getDefaultDomain {

	my @domains = VirtualDomain::list_vws();

	my ($ip,$domain);

	foreach my $item (@domains) {

		if ($item->{dom_grp} eq 'domain1') {
			$domain = $item->{vdn};
			$ip = $item->{vdip};
			last;
		}
	
	}	

return ($domain,$ip);

}

###############################################
# Change the index.html of domain web to reflect the right url ..
sub changeIndex {
	my ($domainname,$path,$domaingrp,$ssl,$admin_ssl) = @_;
	my $page;
	$ssl = $ENV{'HTTPS'} if ($ssl eq '');
	$ssl = 'on' if ($admin_ssl eq 'Yes');
	my $index = "$path/index.html";
	open(HTM,$index); ## || return "File not found";
	while(<HTM>) { 
		if (/\"http\:\/\/\<your\.domain\>\:5555\"/) {
			if ($ssl eq 'on') {
				s/http/https/;	
			}	
			s/(\<your\.domain\>\:5555)/$domainname\:5555/;
		
		}
		if (/http\:\/\/\<your\.host\.domain\>\:5555/) {
                        if ($ssl eq 'on') {
                                s/http/https/;
                        }

			s/(\<your\.host\.domain\>\:5555)/$domainname\:5555/;
		}
		$page .= $_;	
	}
		
	close(HTM);
	open(HTM, ">$index") || return "File not found";
	flock(HTM,2);
		print HTM "$page";
	close(HTM);

}

################################################
# to get the disk usage for specific directory and users ...
#

sub disk_usage {

my ($user,$dir) = @_;
my ($userid,$uid);
my $sum = 0;

find sub {      $uid = (stat($File::Find::name))[4];
                $userid = getpwnam($user);
                $sum += -s if ($uid == $userid) },$dir;

$sum = sprintf("%0.2f", $sum/1024);
return $sum;

}

1;
