package VirtualFtp;

###############################################################################
#                                                                             #
# Filename: VirtualFtp.pm                                                     #
#                                                                             #
# Description: This module provides configuration support for managing        #
#              virtual FTP configuration.                                     #
#                                                                             #
###############################################################################
###############################################################################
#                                                                             #
#  Copyright  2000 - 2001 Intel Corporation.                                 #
#  Intel Corporation All Rights Reserved.                                     #
#                                                                             #
#  The source code contained or described herein and all documents related to #
#  the source code ("Material") are owned by Intel Corporation or its         #
#  suppliers or licensors.  Title to the Material remains with Intel          #
#  Corporation or its suppliers and licensors.  The Material contains trade   #
#  secrets and proprietary and confidential information of Intel or its       #
#  suppliers and licensors.  The Material is protected by worldwide copyright #
#  and trade secret laws and treaty provisions.  No part of the Material may  #
#  be used, copied, reproduced, modified, published, uploaded, posted,        #
#  transmitted, distributed, or disclosed in any way without Intel's prior    #
#  express written permission.                                                #
#                                                                             #
#  No license under any patent, copyright, trade secret or other intellectual #
#  property right is granted to or conferred upon you by disclosure or        #
#  delivery of the Materials, either expressly, by implication, inducement,   #
#  estoppel or otherwise. Any license under such intellectual property rights #
#  must be express and approved by Intel in writing.                          #
#                                                                             #
###############################################################################

# Put the PERL library in the path.
use lib '/usr/ins/intel/gui/perllib';

use strict;
use Users;
use VirtualDomain;

##########################################################################
# Disbale virtual ftp for domain a) remove ip from "ftpserver" and add users in 
# "ftpusers" file
#
sub disableVftp
{

	my ($domainname, $domaingrp,$ipadd) = @_;
	my @users = Users::listUsrs();
	my ($user_domain,$user_list,$set_flag);

	# Remove conf from "ftpservers "  
	my $ret_val1 = getFtpStatus($ipadd, $domaingrp);
	
	if ($ret_val1) {
		ftpserverDel($ipadd, $domaingrp);
	}
	
	my $ftpuser = "/usr/ins/intel/wu-ftpd/conf/ftpusers";
	foreach my $item (@users) {
		my $user_domain = Users::getUserDomain($item->{user});
       		if (($user_domain eq $domainname) && ($item->{domain} eq $domaingrp)) {
			$user_list .= "$item->{user}\n";
	
		}

	}
	open(USR,$ftpuser);
	while(<USR>) {
		if (/^##<$domainname:$domaingrp>##$/ ... /^##<\/$domainname:$domaingrp>##$/){
			$set_flag = 1;	
		}
	}
	close(USR);
	return 1 if ($user_list eq ''); #"User not found"
	if ($set_flag != 1) {
		open(USR,">>$ftpuser");
		flock(USR,2);
	
		print USR "##<$domainname:$domaingrp>##\n";
		print USR "$user_list";
		print USR "##</$domainname:$domaingrp>##\n";
		close(USR);	
	}
	else {
	return 2;#"Domain $domainname already disabled for virtual ftp service\n";
	}
}

##########################################################################
# Add ip in "ftpservers" and remove users from "ftpusers" 
sub enableVftp
{

        my ($domainname,$domaingrp,$ipadd) = @_;
	my @users = Users::listUsrs();
        my ($user_domain,$user_list,$set_flag,$allowuser,$other_user);	
	# Add conf in "ftpservers "
        my $ret_val1 = &getFtpStatus($ipadd, $domaingrp);
	# check if IP is also used by other domain 
	my ($ip_stat, $dom, $grp) = VirtualDomain::ip_domain($ipadd,$domainname);
	my $ftp_home = "/usr/ins/intel/wu-ftpd/conf/$domaingrp";
        if (!($ret_val1) && ($ip_stat eq '') && (-d $ftp_home)) {
                ftpserverAdd($ipadd,$domaingrp);
        }
        my $ftpuser = "/usr/ins/intel/wu-ftpd/conf/ftpusers";
       
        open(USR1,$ftpuser);
        while(<USR1>) {
                if (/^##<$domainname:$domaingrp>##$/ ... /^##<\/$domainname:$domaingrp>##$/)
				{
                        $allowuser .= $_;
                }
                else {  
                        $other_user .= $_;
                }
        }
        close(USR1);
        open(FTPUSR,">$ftpuser");
        flock(FTPUSR, 2);
        print FTPUSR "$other_user";
        close(FTPUSR);
}

##########################################################################
# Add ip and ftpaccess path for the domain to create ip_based virtual ftp server
sub ftpserverAdd {

	my ($ipadd,$domaingrp) = @_;
	
	my $ftpserver = "/usr/ins/intel/wu-ftpd/conf/ftpservers";
	my $conf = "/usr/ins/intel/wu-ftpd/conf/$domaingrp";
        my $ret_val = getFtpStatus($ipadd,$domaingrp);
        	if (!($ret_val)) {
                open(FFTP, ">>$ftpserver");
                print FFTP "$ipadd\t$conf\n";
                close(FFTP);

		}
}

##########################################################################
sub ftpserverDel 
{
        my ($ip,$domaingrp) = @_;
        my (@line,$command,$args);
        my $ftpserver = "/usr/ins/intel/wu-ftpd/conf/ftpservers";
	my $conf = "/usr/ins/intel/wu-ftpd/conf/$domaingrp";
        # Delete the ip and vdn entry from /ftpservers
        if ($ip) {
                open(FHH,"$ftpserver");
                        while(<FHH>) {
                                push(@line, $_);
        		}
                close(FHH);
        ## Now remove what we want and keep rest as it is
        ## or return("File $ftp_home can not be opened: $!");
                open(FHH, ">$ftpserver");
                flock(FHH, 2);
                        foreach my $item1 ( @line) {
                                if ($item1 =~ /$ip\t+$conf/) {
                                }
                                else {  
                                        print FHH "$item1";
                                }
                        }
                close(FHH);
        }
	else { return 1; } #"IP Address not found";

}

##########################################################################
sub getFtpStatus
{
        my ($ip,$domaingrp) = @_;
        my $ftpserver = "/usr/ins/intel/wu-ftpd/conf/ftpservers";
        my $conf = "/usr/ins/intel/wu-ftpd/conf/$domaingrp";
        my $var;
        open(FHH,$ftpserver);
        while(<FHH>) {
                $var = $_ if (/$ip\s+$conf$/);
		chomp($var);
        }
        return $var;
}

##########################################################################

sub domainFtpStatus 
{

	my ($domainname,$domaingrp,$ip) = @_;
	my $ftpuser = "/usr/ins/intel/wu-ftpd/conf/ftpusers";
	my $zero = 0;
	my $one = 1;
	my ($set_flag,$ret_val1);
	open(USR,$ftpuser);

        while(<USR>) {
                if (/^##<$domainname:$domaingrp>##$/ ... /^##<\/$domainname:$domaingrp>##$/)
				{
                        $set_flag = 1;
                }
        }
        close(USR);

	$ret_val1 = &getFtpStatus($ip,$domaingrp);	
	
	return $one if (($set_flag == 1) && ($ret_val1 eq ''));
	return $zero;
		
}


# Add any new user of the domain to the "ftpuser" if the FTP is disabled for that domain #

sub addDomainUser
{

my ($user,$domainname,$domaingrp,$ipadd) = @_;
my ($flag,$id,@data,@allusr,$lastdata);
	
## Add user only if the FTP is disabled for that domain
	my $ftp_access = domainFtpStatus($domainname,$domaingrp,$ipadd);
	if ($ftp_access == 1) {	
		my $ftpuser = "/usr/ins/intel/wu-ftpd/conf/ftpusers";
        	open(USR,$ftpuser);
        		while(<USR>) {
                		if (/^##<$domainname:$domaingrp>##$/ ... /^##<\/$domainname:$domaingrp>##$/){
                        		chomp;
                        		push (@data, $_);
                        		$flag = 1 if ($_ eq $user);
                		}
                		else {  
                        		chomp;
                        		push(@allusr, $_);
                		}
        		}
        	close(USR);
        	$lastdata = pop(@data) if (!$flag);
        	push (@data,$user) if (!$flag);
        	push (@data,$lastdata) if (!$flag);

        	open(USR,">$ftpuser");
        	flock(USR,2);
        	foreach $id (@allusr) {
                	print USR "$id\n";
        	}
        	foreach $id (@data) {
                	print USR "$id\n";
        	}
        	close(USR);
	}

}

1;

__END__
